#! /usr/bin/env python

"""
SYNOPSIS
    Asset Register Batch Error Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError
from tartanWork import artrtp

class ar3010:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None
        tabs = ["ctlbat", "asstrn"]
        self.sql = Sql(self.mf.dbm, tabs, prog=__name__)

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Asset Register Batch Error Listing (%s)" % __name__)
        data = ["All Types"]
        for x in range(1, len(artrtp)+1):
            data.append(artrtp[x-1][1])
        btt = {
            "stype": "C",
            "titl": "Select the Required Type",
            "data": data,
            "retn": "I"}
        btm = {
            "stype": "R",
            "tables": ("ctlbat",),
            "cols": (
                ("btm_batno", "", 0, "Bat-Num"),
                ("btm_rtyp", ("xx", artrtp), 20, "Type"),
                ("btm_curdt", "", 0, "Curr-Dt"),
                ("btm_multi", "", 0, "M")),
            "where": [
                ("btm_cono", "=", self.conum),
                ("btm_styp", "=", "ASS"),
                ("btm_ind", "=", "N")],
            "whera": []}
        fld = (
            (("T",0,0,0),"IUI",1,"Type","Transaction Type",
                "","Y",self.doBatTyp,btt,None,None),
            (("T",0,1,0),"INa",7,"Batch-Number","Batch Number",
                "","Y",self.doBatNum,btm,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"))

    def doBatTyp(self, frt, pag, r, c, p, i, w):
        if w > len(artrtp):
            return "Invalid Batch Type"
        self.rtyp = w
        if self.rtyp:
            self.df.topf[pag][1][8]["whera"] = [["T","btm_rtyp",0]]
        else:
            self.df.topf[pag][1][8]["whera"] = []

    def doBatNum(self, frt, pag, r, c, p, i, w):
        self.batno = w

    def doEnd(self):
        self.df.closeProcess()
        whr = [("btm_cono", "=", self.conum), ("btm_styp", "=", "ASS")]
        if self.rtyp:
            whr.append(("btm_rtyp", "=", self.rtyp))
        if self.batno:
            whr.append(("btm_batno", "=", self.batno))
        whr.append(("btm_ind", "=", "N"))
        ord = "btm_rtyp, btm_batno"
        all = self.sql.getRec(tables="ctlbat", where=whr, order=ord)
        if not all:
            showError(self.mf.body, "Batch Error",
            "No Batch Transactions Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %49s %10s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.btot = [0, 0]
        self.xtype = 0
        self.xbatno = ""
        self.pgnum = 0
        self.pglin = 99
        bc = self.sql.ctlbat_col
        tc = self.sql.asstrn_col
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            selt.rtyp = CCD(dat[bc.index("btm_rtyp")], "UI", 1)
            self.batch = CCD(dat[bc.index("btm_batno")], "Na", 7)
            self.trno = CCD(dat[bc.index("btm_trno")], "UI", 7)
            self.trval = CCD(dat[bc.index("btm_trval")], "SD", 13.2)
            self.batby = CCD(dat[bc.index("btm_batby")], "NA", 5)
            self.indby = CCD(dat[bc.index("btm_indby")], "NA", 5)
            whr = [("ast_cono", "=", self.conum), ("ast_type", "=",
                selt.rtyp.work), ("ast_batch", "=", self.batch.work),
                ("ast_batind", "=", "N")]
            ord = "ast_seq"
            trndat = self.sql.getRec(tables="asstrn", where=whr, order=ord)
            if not trndat:
                continue
            for d in trndat:
                grp = CCD(d[tc.index("ast_group")], "UA", 3)
                code = CCD(d[tc.index("ast_code")], "Na", 7)
                refno = CCD(d[tc.index("ast_refno")], "Na", 9)
                mtyp = CCD(d[tc.index("ast_mtyp")], "UI", 1)
                trdt = CCD(d[tc.index("ast_date")], "D1", 10)
                desc = CCD(d[tc.index("ast_desc")], "NA", 30)
                amt1 = CCD(d[tc.index("ast_amt1")], "SD", 13.2)
                vat = CCD(d[tc.index("ast_vat")], "SD", 13.2)
                taxind = CCD(d[tc.index("ast_taxind")], "NA", 1)
                if self.xtype != 0 and selt.rtyp.work != self.xtype:
                    self.batchTotal()
                    self.pglin = 99
                elif self.xbatno and self.batch.work != self.xbatno:
                    self.batchTotal()
                    self.batchHeading()
                if self.pglin > self.lpp:
                    if self.pgnum != 0:
                        self.c.drawText(self.t)
                        self.c.showPage()
                    self.pageHeading()
                    self.batchHeading()
                self.t.textLine("%s %s %s %s %s %s %s %s %s" % \
                    (grp.disp, code.disp, refno.disp, trdt.disp,
                    desc.disp, mtyp.disp, amt1.disp, vat.disp, taxind.disp))
                self.pglin = self.pglin + 1
                self.xtype = selt.rtyp.work
                self.xbatno = self.batch.work
                self.btot[0] = self.btot[0] + 1
                self.btot[1] = float(ASD(self.btot[1]) + ASD(amt1.work))
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.batchTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-34s %54s %5s" % \
            ("Asset Register Batch Error Listing", "Page", self.pgnum))
        self.t.textLine()
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 4

    def batchHeading(self):
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%-6s %-9s %-5s %-19s %-11s %-5s %-12s %-5s" % \
            ("Batch:", self.batch.disp, "Type:",
                artrtp[selt.rtyp.work - 1][1], "Batched by:",
                self.batby.disp, "Captured by:", self.indby.disp))
        self.t.textLine()
        self.t.textLine("%-3s %-7s %-9s %-10s %-30s %1s %-13s %-13s %1s" % \
            ("Grp", "Code", "Reference", "   Date", "Remarks", "M",
            "      Amount", "     VAT-Amt", "I"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = self.pglin + 4

    def batchTotal(self):
        self.t.textLine()
        j = CCD(self.trno.work, "SI", 7)
        k = CCD(self.trval.work, "SD", 13.2)
        self.t.textLine("%-20s %-10s %-23s %-10s %-13s" % \
                ("Prelisted", "Quantity", j.disp, "Value", k.disp))
        l = CCD(self.btot[0], "SI", 7)
        m = CCD(self.btot[1], "SD", 13.2)
        self.t.textLine("%-20s %-10s %-23s %-10s %-13s" % \
                ("Entered", "Quantity", l.disp, "Value", m.disp))
        n = CCD(j.work - l.work, "SI", 7)
        o = CCD(float(ASD(k.work) - ASD(m.work)), "SD", 13.2)
        self.t.textLine("%-20s %-10s %-23s %-10s %-13s" % \
                ("Difference", "Quantity", n.disp, "Value", o.disp))
        self.t.textLine()
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = self.pglin + 6
        self.btot[0] = 0
        self.btot[1] = 0

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()
