#! /usr/bin/env python

"""
SYNOPSIS
    Asset Register Transaction Audit Trail.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, GetCtl, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError
from tartanWork import artrtp

class ar3020:
    def __init__(self, mf, coy, nam, per):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = per
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        gc = GetCtl(self.mf)
        assctl = gc.getCtl("assctl", self.conum)
        if not assctl:
            return "no"
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.s_per = self.period[1][0] / 100
        self.e_per = self.period[2][0] / 100
        self.totind = "N"
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Asset Register Audit Trail (%s)" % __name__)
        data = ["All Types"]
        for x in range(1, len(artrtp)+1):
            data.append(artrtp[x-1][1])
        btt = {
            "stype": "C",
            "titl": "Select the Required Type",
            "data": data,
            "retn": "I"}
        btm = {
            "stype": "R",
            "tables": ("asstrn",),
            "cols": (
                ("ast_batch", "", 0, "Bat-Num"),
                ("ast_type", ("xx", artrtp), 20, "Type"),
                ("ast_curdt", "", 0, "Cur-Dat")),
            "where": [],
            "group": "ast_batch, ast_type, ast_curdt",
            "order": "ast_type, ast_curdt, ast_batch"}
        r1s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),"Id2",7,"Starting Period","Starting Period",
                "","Y",self.doStartPer,None,None,("efld",)),
            (("T",0,1,0),"Id2",7,"Ending Period","Ending Period",
                "","Y",self.doEndPer,None,None,("efld",)),
            (("T",0,2,0),"IUI",1,"Type","Transaction Type",
                "","Y",self.doBatTyp,btt,None,None),
            (("T",0,3,0),"INa",7,"Batch-Number","Batch Number",
                "","Y",self.doBatNum,btm,None,None),
            (("T",0,4,0),("IRB",r1s),0,"Totals Only","Totals Only (Y/N)",
                "Y","Y",self.doTots,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doStartPer(self, frt, pag, r, c, p, i, w):
        if w == 0:
            w = self.s_per
        elif w < self.s_per or w > self.e_per:
            return "Invalid Period"
        self.sdat = self.df.loadEntry(frt, pag, p, data=w)

    def doEndPer(self, frt, pag, r, c, p, i, w):
        if w == 0:
            w = self.e_per
        elif w < self.sdat.work or w > self.e_per:
            return "Invalid Period"
        self.edat = self.df.loadEntry(frt, pag, p, data=w)

    def doBatTyp(self, frt, pag, r, c, p, i, w):
        if w > len(artrtp):
            return "Invalid Batch Type"
        self.btype = w
        whr = [("ast_cono", "=", self.conum), ("ast_curdt", "between",
            self.sdat.work, self.edat.work)]
        if self.btype:
            whr.append(("ast_type", "=", self.btype))
        self.df.topf[pag][i+1][8]["where"] = whr

    def doBatNum(self, frt, pag, r, c, p, i, w):
        self.batch = w

    def doTots(self, frt, pag, r, c, p, i, w):
        self.totsonly = w

    def doEnd(self):
        self.df.closeProcess()
        if not self.btype:
            styp = 0
            etyp = 99
        else:
            styp = etyp = self.btype
        if not self.batch:
            sbat = ""
            ebat = "zzzzzzz"
        else:
            sbat = ebat = self.batch
        self.sql = Sql(self.mf.dbm, "asstrn", prog=__name__)
        all = self.sql.getRec(tables="asstrn", where=[("ast_cono", "=",
            self.conum), ("ast_curdt", "between", self.sdat.work,
            self.edat.work), ("ast_batch", "between", sbat, ebat), ("ast_type",
            "between", styp, etyp)], order="ast_type, ast_batch, ast_refno")
        if not all:
            showError(self.mf.body, "Processing Error",
            "No Asset Transactions Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %16s %33s %16s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.bcnt = 0
        self.bmt1 = 0
        self.bmt2 = 0
        self.bvat = 0
        self.tcnt = 0
        self.tmt1 = 0
        self.tmt2 = 0
        self.tvat = 0
        self.gcnt = [0,0,0,0,0]
        self.gmt1 = [0,0,0,0,0]
        self.gmt2 = [0,0,0,0,0]
        self.gvat = [0,0,0,0,0]
        self.trtp = 0
        self.pgnum = 0
        self.pglin = 99
        tc = self.sql.asstrn_col
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            trtp = CCD(dat[tc.index("ast_type")], "UI", 2)
            batch = CCD(dat[tc.index("ast_batch")], "Na", 7)
            grp = CCD(dat[tc.index("ast_group")], "UA", 3)
            code = CCD(dat[tc.index("ast_code")], "Na", 7)
            refno = CCD(dat[tc.index("ast_refno")], "Na", 9)
            mtyp = CCD(dat[tc.index("ast_mtyp")], "UI", 1)
            trdt = CCD(dat[tc.index("ast_date")], "D1", 10)
            desc = CCD(dat[tc.index("ast_desc")], "NA", 30)
            amt1 = CCD(dat[tc.index("ast_amt1")], "SD", 13.2)
            amt2 = CCD(dat[tc.index("ast_amt2")], "SD", 13.2)
            vat = CCD(dat[tc.index("ast_vat")], "SD", 13.2)
            taxind = CCD(dat[tc.index("ast_taxind")], "NA", 1)
            if not self.trtp:
                self.trtp = trtp.work
                self.batch = batch.work
            if trtp.work != self.trtp:
                self.batchTotal()
                self.typeTotal()
                self.trtp = trtp.work
                self.batch = batch.work
                self.pglin = 99
            if batch.work != self.batch:
                self.batchTotal()
                self.batch = batch.work
                if self.totsonly != "Y":
                    self.typeHeading()
            if self.pglin > self.lpp:
                if self.pgnum != 0:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            if self.totsonly != "Y":
                self.t.textLine("%s %s %s %s %s %s %s %s %s %s" % \
                    (grp.disp, code.disp, refno.disp, trdt.disp, desc.disp,
                    mtyp.disp, amt1.disp, amt2.disp, vat.disp, taxind.disp))
                self.pglin = self.pglin + 1
            self.bcnt = self.bcnt + 1
            self.bmt1 = float(ASD(self.bmt1) + ASD(amt1.work))
            self.bmt2 = float(ASD(self.bmt2) + ASD(amt2.work))
            self.bvat = float(ASD(self.bvat) + ASD(vat.work))
            self.tcnt = self.tcnt + 1
            self.tmt1 = float(ASD(self.tmt1) + ASD(amt1.work))
            self.tmt2 = float(ASD(self.tmt2) + ASD(amt2.work))
            self.tvat = float(ASD(self.tvat) + ASD(vat.work))
            self.gcnt[trtp.work -1] = self.gcnt[trtp.work -1] + 1
            self.gmt1[trtp.work -1] = float(ASD(self.gmt1[trtp.work -1]) + \
                ASD(amt1.work))
            self.gmt2[trtp.work -1] = float(ASD(self.gmt2[trtp.work -1]) + \
                ASD(amt2.work))
            self.gvat[trtp.work -1] = float(ASD(self.gvat[trtp.work -1]) + \
                ASD(vat.work))
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.batchTotal()
                self.typeTotal()
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-36s %-7s %-2s %-7s %47s %5s" % \
            ("Asset Register Audit Trail for Period",
            self.sdat.disp, "to", self.edat.disp, "Page", self.pgnum))
        self.t.textLine()
        self.pglin = 4
        if self.totind == "N":
            self.typeHeading()
        else:
            self.t.textLine("%-14s" % "Totals Summary")
            self.t.textLine()
            self.t.textLine("%-30s %-8s %-13s %-13s %-13s" %
                ("Document Type", " Number", "    Amount-1",
                "    Amount-2", "     VAT-Amt"))
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.t.setFont("Tartan", self.cpi, self.ldg)
            self.pglin = self.pglin + 4

    def typeHeading(self):
        if self.totsonly != "Y":
            batch = self.batch
        else:
            batch = "Various"
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%-7s %7s %-10s %3s" % \
            ("Batch", batch, "    Type", artrtp[self.trtp - 1][1]))
        self.t.textLine()
        self.t.textLine("%-3s %-7s %-9s %-10s %-30s %1s %-13s %-13s %-13s "\
            "%1s" % ("Grp", "Code", "Reference", "   Date", "Remarks", "M",
            "    Amount-1", "    Amount-2", "     VAT-Amt", "I"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = self.pglin + 4

    def batchTotal(self):
        j = CCD(self.bcnt, "SI", 7)
        k = CCD(self.bmt1, "SD", 13.2)
        l = CCD(self.bmt2, "SD", 13.2)
        m = CCD(self.bvat, "SD", 13.2)
        if self.totsonly != "Y":
            self.t.textLine()
            self.pglin = self.pglin + 1
        self.t.textLine("%-32s %-32s %13s %13s %13s" % \
            (" ", "Batch " + self.batch + " Totals", k.disp, l.disp, m.disp))
        self.pglin = self.pglin + 1
        if self.totsonly == "N":
            self.t.textLine()
            self.pglin = self.pglin + 1
        self.bcnt = 0
        self.bmt1 = 0
        self.bmt2 = 0
        self.bvat = 0

    def typeTotal(self):
        j = CCD(self.tcnt, "SI", 7)
        k = CCD(self.tmt1, "SD", 13.2)
        l = CCD(self.tmt2, "SD", 13.2)
        m = CCD(self.tvat, "SD", 13.2)
        if self.totsonly == "Y":
            self.t.textLine()
        self.t.textLine("%-32s %-32s %13s %13s %13s" % \
            (" ", "Type-Totals", k.disp, l.disp, m.disp))
        self.tcnt = 0
        self.tmt1 = 0
        self.tmt2 = 0
        self.tvat = 0
    
    def grandTotal(self):
        self.c.drawText(self.t)
        self.c.showPage()
        self.totind = "Y"
        self.pageHeading()
        tot = [0,0,0,0]
        for x in range(0, len(artrtp)):
            cnt = CCD(self.gcnt[x], "SI", 7)
            mt1 = CCD(self.gmt1[x], "SD", 13.2)
            mt2 = CCD(self.gmt2[x], "SD", 13.2)
            vat = CCD(self.gvat[x], "SD", 13.2)
            self.t.textLine("%-31s %s %s %s %s" % \
                (artrtp[x][1], cnt.disp, mt1.disp, mt2.disp, vat.disp))
            tot[0] = tot[0] + cnt.work
            tot[1] = float(ASD(tot[1]) + ASD(mt1.work))
            tot[2] = float(ASD(tot[2]) + ASD(mt2.work))
            tot[3] = float(ASD(tot[3]) + ASD(vat.work))
        self.t.textLine()
        cnt = CCD(tot[0], "SI", 8)
        mt1 = CCD(tot[1], "SD", 13.2)
        mt2 = CCD(tot[2], "SD", 13.2)
        vat = CCD(tot[3], "SD", 13.2)
        self.t.textLine("%-30s %s %s %s %s" % \
            ("Grand Totals", cnt.disp, mt1.disp, mt2.disp, vat.disp))
        self.t.textLine()

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()
