#! /usr/bin/env python

"""
SYNOPSIS
    Asset Register Interrogation.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, CCD, NotesCreate, SChoice, Sql
from TartanClasses import SRec, TartanDialog
from tartanFunctions import getModName, doPrinter
from tartanWork import artrtp, armvtp

class ar4010:
    def __init__(self, mf, coy, nam, per, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = per
        self.capnm = usr
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["assgrp", "assdep", "assmst", "asstrn"],
            prog=__name__)
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) +  (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.sper = self.period[1][0] / 100
        self.eper = self.period[2][0] / 100

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
                "Asset Register Interrogation (%s)" % __name__)
        grp = {
            "stype": "R",
            "tables": ("assgrp",),
            "cols": (
                ("asg_group", "", 0, "Grp"),
                ("asg_desc", "", 0, "Description", "Y")),
            "where": [("asg_cono", "=", self.conum)]}
        cod = {
            "stype": "R",
            "tables": ("assmst",),
            "cols": (
                ("asm_code", "", 0, "Code"),
                ("asm_desc", "", 0, "Description", "Y")),
            "where": [("asm_cono", "=", self.conum)],
            "whera": (("T", "asm_group", 0, 0),)}
        tag = (
            ("Depreciation", self.doTagSelect, ("T",0,0), ("T",0,1)),
            ("Balances", self.doTagSelect, ("T",0,0), ("T",0,1)),
            ("Transactions", self.doTrans1, ("T",0,0), ("T",0,1)))
        r1s = (
            ("Straight Line","S"),
            ("Balance","D"),
            ("None","N"))
        r2s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),"IUA",3,"Group","Asset Group",
                "","N",self.doGroup,grp,None,("notblank",)),
            (("T",0,0,0),"INa",7,"Code","Asset Code",
                "","N",self.doCode,cod,None,("notblank",)),
            (("T",0,0,0),"ONA",30,"Description"),
            (("T",1,0,0),"ONa",3,"Dep Code"),
            (("T",1,0,0),"ONA",40,""),
            (("T",1,1,0),("ORB",r1s[:-1]),0,"Company Type"),
            (("T",1,2,0),("ORB",r1s),0,"Receiver Type"),
            (("C",1,0,0),"OUD",17.2,"Company-Rate","",
                "","N",None,None,None,None,("Year",7)),
            (("C",1,0,1),"OUD",18.2,"Receiver-Rate"),
            (("T",2,0,0),"Od1",10,"Date of Purchase"),
            (("T",2,0,0),"OSD",15.2,"Purchase Price"),
            (("C",2,0,0),"OSD",15.2,"Company-Value","","","N",
                None,None,None,None,("Details",7)),
            (("C",2,0,1),"OSD",16.2,"Receiver-Value"),
            (("T",3,0,0),("IRB",r2s),0,"Show History","Show History",
                "N","N",self.doTrans2,None,None,None))
        but = (
            ("Clear",None,self.doClear,0,("T",0,0),("T",0,1)),
            ("Notes",None,self.doNotes,0,("T",0,0),("T",0,1)),
            ("Print",None,self.doPrint,0,("T",0,0),("T",0,1)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEndTop, "N"), None, None, None)
        txt = (self.doExit, None, None, self.doExit)
        self.df = TartanDialog(self.mf, title=self.tit, tags=tag, eflds=fld,
            butt=but, tend=tnd, txit=txt)
        for x in range(7):
            self.df.colLabel[1][x].set_text("Year %s" % (x + 1))
        self.df.colLabel[2][0].set_text("Opening Book Value")
        self.df.colLabel[2][1].set_text("Purchases         ")
        self.df.colLabel[2][2].set_text("Improvements      ")
        self.df.colLabel[2][3].set_text("Write Offs        ")
        self.df.colLabel[2][4].set_text("Depreciation      ")
        self.df.colLabel[2][5].set_text("Sales             ")
        self.df.colLabel[2][6].set_text("Closing Book Value")

    def doGroup(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="assgrp", where=[("asg_cono", "=",
            self.conum), ("asg_group", "=", w)], limit=1)
        if not acc:
            return "Invalid Asset Group"
        self.group = w

    def doCode(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="assmst", cols=["asm_desc", "asm_depcod"],
            where=[("asm_cono", "=", self.conum), ("asm_group", "=",
            self.group), ("asm_code", "=", w)], limit=1)
        if not acc:
            return "Asset Does Not Exist"
        self.code = w
        self.desc = acc[0]
        self.df.loadEntry(frt, pag, p+1, data=self.desc)
        dep = self.sql.getRec(tables="assdep", where=[("asd_cono", "=",
            self.conum), ("asd_code", "=", acc[1])], limit=1)
        for x in range(self.df.topq[1]):
            self.df.loadEntry("T", 1, x, data=dep[x+1])
        for y in range(14):
            self.df.loadEntry("C", 1, y, data=dep[x+y+2])
        self.loadBalances()
        self.mf.updateStatus("")

    def doEndTop(self):
        self.df.last[0] = [0, 0]
        self.df.setWidget(self.df.topEntry[0][0], state="disabled")

    def loadBalances(self):
        bals = Balances(self.mf, "ASS", self.conum, self.sper,
            keys=(self.group, self.code))
        asset = bals.doAssBals(start=self.sper, end=self.eper, trans="C")
        if not asset:
            return
        cap, cdp, rdp, cbl, rbl, mov, self.trn = asset
        dte = self.sql.getRec(tables="asstrn", cols=["min(ast_date)"],
            where=[("ast_cono", "=", self.conum), ("ast_group", "=",
            self.group), ("ast_code", "=", self.code), ("ast_mtyp", "=",
            1)], limit=1)
        if dte[0]:
            val = self.sql.getRec(tables="asstrn", cols=["ast_amt1"],
                where=[("ast_cono", "=", self.conum), ("ast_group", "=",
                self.group), ("ast_code", "=", self.code), ("ast_mtyp", "=",
                1), ("ast_date", "=", dte[0])], limit=1)
            self.df.loadEntry("T", 2, 0, data=dte[0])
            self.df.loadEntry("T", 2, 1, data=val[0])
        self.df.loadEntry("C", 2, 0, data=cbl)
        self.df.loadEntry("C", 2, 1, data=rbl)
        if mov:
            for n, c, r in mov:
                p = n * 2
                cbl = float(ASD(cbl) + ASD(c))
                rbl = float(ASD(rbl) + ASD(r))
                self.df.loadEntry("C", 2, p, data=c)
                self.df.loadEntry("C", 2, p+1, data=r)
        self.df.loadEntry("C", 2, 12, data=cbl)
        self.df.loadEntry("C", 2, 13, data=rbl)

    def doTagSelect(self):
        self.mf.updateStatus("")

    def doTrans1(self, widget=None):
        self.df.focusField("T", 3, 1)

    def doTrans2(self, frt, pag, r, c, p, i, w):
        tit = "Transactions for Item: %s %s - %s" % \
            (self.group, self.code, self.desc)
        tab = ["asstrn"]
        crt = (
            ("ast_date", "", 0, "   Date"),
            ("ast_curdt", "", 0, "Curr-Dt"),
            ("ast_batch", "", 0, "Batch"),
            ("ast_type", ("XX", artrtp), 3, "Typ"),
            ("ast_mtyp", ("XX", armvtp), 3, "Mov"),
            ("ast_refno", "", 0, "Reference", "Y"),
            ("ast_amt1", "", 0, "  Amount-1"),
            ("ast_amt2", "", 0, "  Amount-2"),
            ("ast_desc", "", 0, "Details"))
        whr = [
            ("ast_cono", "=", self.conum),
            ("ast_group", "=", self.group),
            ("ast_code", "=", self.code),
            ("ast_curdt", "<=", self.eper)]
        if w == "N":
            whr.append(("ast_curdt", ">=", self.sper))
        ord = "ast_date, ast_refno"
        state = self.df.disableButtons()
        rec = SRec(self.mf, screen=self.df.nb.Page2, title=tit, tables=tab,
            cols=crt, where=whr, order=ord, rows=12)
        self.df.enableButtons(state=state)
        self.doTrans1()

    def doNotes(self, widget=None):
        state = self.df.disableButtons()
        self.df.mstAlw.hide()
        nf = NotesCreate(self.mf, self.conum, self.conam, self.capnm, "ASS",
            "%3s%s" % (self.group, self.code))
        self.df.mstAlw.show()
        self.df.enableButtons(state=state)

    def doClear(self, widget=None):
        self.df.selPage("Depreciation")
        self.df.focusField("T", 0, 1)

    def doPrint(self, widget=None):
        titl = "Select the required print option!"
        data = (
            ("Balances Only", "B"),
            ("Transactions Only", "T"),
            ("Balances and Transactions", "A"))
        ss = SChoice(self.mf, titl=titl, data=data)
        self.mf.updateStatus("")
        if not ss.selection:
            self.pr_opts = " "
        else:
            self.pr_opts = ss.selection[1]
        if self.pr_opts != " ":
            self.printDetail()
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def printDetail(self):
        self.head = ("%03u %-30s %47s %10s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        from tartanFunctions import doReportlab
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        if self.pr_opts != "T":
            self.pageHeading()
            self.printInfo()
        if self.pr_opts == "B":
            self.c.drawText(self.t)
            self.c.showPage()
            self.c.save()
            doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                head=self.tit, view="V")
        elif self.pr_opts == "A":
            if not self.trn[0]:
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")
            else:
                self.c.drawText(self.t)
                self.c.showPage()
                self.pageHeading()
                self.pageHeadingTrans()
                self.printTrans()
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")
        elif self.pr_opts == "T":
            if self.trn[0]:
                self.pageHeading()
                self.pageHeadingTrans()
                self.printTrans()
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")

    def printInfo(self):
        for x in range(0, len(self.df.topf[0])):
            self.t.textLine("%-15s%-25s %s" % ("",
                self.df.topf[0][x][3], self.df.t_disp[0][0][x]))
        self.t.textLine()
        self.t.textLine("%-15s%-25s %s" % ("",
            self.df.topf[1][0][3], self.df.t_disp[1][0][0]))
        self.t.textLine("%-15s%-25s %s" % ("",
            "Dep Description", self.df.t_disp[1][0][1]))
        self.t.textLine()
        self.t.textLine("%-15s%-25s     %s" % ("",
            "Date of Purchase", self.df.t_disp[2][0][0]))
        self.t.textLine("%-15s%-25s %s" % ("",
            "Cost of Purchase", self.df.t_disp[2][0][1]))
        self.t.textLine()
        self.pglin = self.pglin + 3 + len(self.df.topf[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%15s%-35s %13s %13s" % \
            ("", "Details","Company ","Receiver "))
        self.t.textLine("%15s%s" % ("", self.suc * 62))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        desc = (
            "Opening Book Value",
            "Purchases         ",
            "Improvements      ",
            "Write Offs        ",
            "Depreciation      ",
            "Sales             ",
            "Closing Book Value")
        for n, d in enumerate(self.df.c_disp[2]):
            if n == 6:
                self.t.textLine("%15s%s" % ("", self.suc * 62))
            self.t.textLine("%15s%-30s %13s %13s" % ("", desc[n], d[0], d[1]))

    def printTrans(self):
        for ct in self.trn[0]:
            trdt = CCD(ct[self.trn[1].index("ast_date")], "D1", 10)
            ref1 = CCD(ct[self.trn[1].index("ast_refno")], "Na", 9)
            trtp = CCD(ct[self.trn[1].index("ast_type")], "UI", 1)
            batch = CCD(ct[self.trn[1].index("ast_batch")], "Na", 7)
            mtyp = CCD(ct[self.trn[1].index("ast_mtyp")], "UI", 1)
            amt1 = CCD(ct[self.trn[1].index("ast_amt1")], "SD", 13.2)
            amt2 = CCD(ct[self.trn[1].index("ast_amt2")], "SD", 13.2)
            detail = CCD(ct[self.trn[1].index("ast_desc")], "NA", 30)
            if self.pglin > self.lpp:
                if self.pgnum != 0:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
                self.pageHeadingTrans()
            self.t.textLine("%s %s %s %s %s %s %s %s" % (trdt.disp,
                ref1.disp, artrtp[(trtp.work - 1)][0], batch.disp,
                armvtp[(mtyp.work - 1)][0], amt1.disp, amt2.disp,
                detail.disp))
            self.pglin = self.pglin + 1

    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-35s %-10s %40s %5s" % \
            ("Assets Register Interrogation as at", self.sysdtd, "Page",
            self.pgnum))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 6

    def pageHeadingTrans(self):
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine()
        self.t.textLine("%-5s %-10s %s %s %-5s %s %s" %    ("", "",
                self.df.topf[0][0][3], self.df.t_disp[0][0][0], "",
                self.df.topf[1][0][3], self.df.t_disp[1][0][0]))
        self.t.textLine()
        self.t.textLine("%-10s %-9s %-3s %-7s %3s %-13s %-13s %-30s" % \
            ("   Date", "Reference", "Typ", "Batch", "Mov",
            "    Amount-1", "    Amount-2", "Remarks"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()
