#! /usr/bin/python

"""
SYNOPSIS
    Clubs's Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import TartanDialog, Sql
from tartanFunctions import askQuestion, getNextCode

class bc1010:
    def __init__(self, mf):
        self.mf = mf
        if self.setVariables():
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlmst", "bwlclb", "bwlflo", "bwlskp"],
            prog=__name__)
        acc = self.sql.getRec(tables="ctlmst", cols=["ctm_name"],
            where=[("ctm_cono", "=", 1)], limit=1)
        if not acc:
            showError(self.mf.body, "Missing Record",
                "The Club Record is Missing, Please Create it and then Retry")
            return
        conam = acc[0]
        acc = self.sql.getRec(tables="bwlclb")
        if not acc:
            self.sql.insRec("bwlclb", data=[1, conam])
            self.mf.dbm.commitDbase()
        return True

    def mainProcess(self):
        tit = ("Clubs's Maintenance",)
        clb = {
            "stype": "R",
            "tables": ("bwlclb",),
            "cols": (
                ("bcc_code", "", 0, "Cod"),
                ("bcc_name", "", 0, "Name", "Y")),
            "order": "bcc_name"}
        fld = (
            (("T",0,0,0),"I@bcc_code",0,"","",
                "","Y",self.doClbCod,clb,None,("efld",)),
            (("T",0,2,0),"I@bcc_name",0,"","",
                "","N",self.doName,None,self.doDelete,("notblank",)))
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            tend=tnd, txit=txt)

    def doClbCod(self, frt, pag, r, c, p, i, w):
        if not w:
            self.club = getNextCode(self.sql, "bwlclb", "bcc_code", last=999)
            self.df.loadEntry(frt, pag, p, data=self.club)
        else:
            self.club = w
        chk = self.sql.getRec(tables="bwlclb", where=[("bcc_code", "=",
            self.club)], limit=1)
        if not chk:
            self.newclb = True
        else:
            self.newclb = False
            for num, fld in enumerate(chk):
                self.df.loadEntry(frt, pag, num, data=fld)

    def doName(self, frt, pag, r, c, p, i, w):
        if self.newclb and self.sql.getRec(tables="bwlclb",
                        where=[("bcc_name", "ilike", w)]):
            return "A Club with this Name Already Exists"

    def doDelete(self, widget=None):
        if self.newclb:
            return
        error = False
        for tab in ( ("bwlflo", "bfo_club"), ("bwlskp", "bcs_club")):
            if self.sql.getRec(tables=tab[0], where=[(tab[1], "=", self.club)], limit=1):
                error = True
                break
        if error:
            return "There are Entries for this Club, Not Deleted"
        self.sql.delRec("bwlclb", where=[("bcc_code", "=", self.club)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self, widget=None):
        data = self.df.t_work[0][0]
        if self.newclb:
            self.sql.insRec("bwlclb", data=data)
        else:
            self.sql.updRec("bwlclb", data=data, where=[("bcc_code", "=",
                self.club)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()
