#! /usr/bin/python

"""
SYNOPSIS
    Competitions Player's Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import TartanDialog, Sql
from tartanFunctions import askQuestion, getNextCode

class bc1020:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["bwlclb", "bwlent", "bwlgme", "bwlskp"],
            prog=__name__)

    def mainProcess(self):
        tit = ("Player's Maintenance",)
        skp = {
            "stype": "R",
            "tables": ("bwlskp",),
            "cols": (
                ("bcs_code", "", 0, "Cod"),
                ("bcs_surname", "", 0, "Surame", "Y"),
                ("bcs_names", "", 0, "Names")),
            "order": "bcs_surname"}
        clb = {
            "stype": "R",
            "tables": ("bwlclb",),
            "cols": (
                ("bcc_code", "", 0, "Cod"),
                ("bcc_name", "", 0, "Name", "Y")),
            "butt": [("New Club", self.doNewClub)],
            "order": "bcc_name"}
        r1s = (("Male", "M"), ("Female", "F"))
        fld = (
            (("T",0,0,0),"I@bcs_code",0,"","",
                "","Y",self.doSkpCod,skp,None,("efld",)),
            (("T",0,1,0),"I@bcs_title",0,"","",
                "","N",None,None,self.doDelete,("notblank",)),
            (("T",0,2,0),"I@bcs_surname",0,"","",
                "","N",self.doSurname,None,None,("notblank",)),
            (("T",0,3,0),"I@bcs_names",0,"","",
                "","N",self.doNames,None,None,("notblank",)),
            (("T",0,4,0),("IRB",r1s),0,"Gender","Gender",
                "M","N",None,None,None,None),
            (("T",0,5,0),"I@bcs_club",0,"","",
                "","N",self.doClbCod,clb,None,("efld",)),
            (("T",0,5,0),"INA",30,"","Club Name",
                "","N",self.doClbNam,None,None,("notblank",)),
            (("T",0,6,0),"I@bcs_add1",0,"","",
                "","N",None,None,None,("notblank",)),
            (("T",0,7,0),"I@bcs_add2",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",0,8,0),"I@bcs_add3",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",0,9,0),"I@bcs_pcod",0,"","",
                "","N",None,None,None,("notblank",)),
            (("T",0,10,0),"I@bcs_email",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",0,11,0),"I@bcs_phone",0,"","",
                "","N",None,None,None,("efld",)))
        but = (("Accept",None,self.doEnd,0,("T",0,2),("T",0,0)),)
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            butt=but, tend=tnd, txit=txt, clicks=self.doClick)

    def doClick(self, *opts):
        if self.df.col == 1:
            return
        if self.click:
            self.df.loadEntry("T", self.click[0], self.click[1],
                data=self.df.t_work[self.click[0]][0][self.click[1]])
        self.click = opts[2]
        self.df.focusField("T", opts[2][0], opts[2][1] + 1)

    def doSkpCod(self, frt, pag, r, c, p, i, w):
        self.click = []
        if not w:
            self.skip = getNextCode(self.sql, "bwlskp", "bcs_code", last=999)
            self.df.loadEntry(frt, pag, p, data=self.skip)
        else:
            self.skip = w
        # Initialise Club Variables
        self.newclb = False
        self.club = None
        #
        chk = self.sql.getRec(tables="bwlskp", where=[("bcs_code", "=",
            self.skip)], limit=1)
        if not chk:
            self.newskp = True
        else:
            self.newskp = False
            for num, fld in enumerate(chk):
                if num > 5:
                    pos = num + 1
                else:
                    pos = num
                self.df.loadEntry(frt, pag, pos, data=fld)
                if num == 5:
                    self.club = fld
                    clb = self.sql.getRec(tables="bwlclb",
                        where=[("bcc_code", "=", fld)], limit=1)
                    self.df.loadEntry(frt, pag, pos+1, data=clb[1])

    def doSurname(self, frt, pag, r, c, p, i, w):
        self.sname = w

    def doNames(self, frt, pag, r, c, p, i, w):
        if self.newskp and self.sql.getRec(tables="bwlskp",
                where=[("bcs_surname", "ilike", self.sname, "and",
                "bcs_names", "ilike", w)]):
            return "A Skip with this Name Already Exists"

    def doDelete(self, widget=None):
        if self.newskp:
            return
        error = False
        for tab in (("bwlent", "bce_scod"), ("bwlgme", "bcg_scod")):
            if self.sql.getRec(tables=tab[0], where=[(tab[1], "=", self.skip)], limit=1):
                error = True
                break
        if error:
            return "There are Movements for this Skip, Not Deleted"
        self.sql.delRec("bwlskp", where=[("bcs_code", "=", self.skip)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doClbCod(self, frt, pag, r, c, p, i, w):
        if not w:
            self.club = getNextCode(self.sql, "bwlclb", "bcc_code", last=999)
            self.df.loadEntry(frt, pag, p, data=self.club)
        else:
            self.club = w
        chk = self.sql.getRec(tables="bwlclb", where=[("bcc_code", "=",
            self.club)], limit=1)
        if not chk:
            ok = askQuestion(self.mf.body, head="New Club",
                mess="This Club Code Does Not Exist, Do You Want To Create It?")
            if ok == "no":
                return "rf"
            self.doNewClub(self.club)
        else:
            self.newclb = False
            self.df.loadEntry(frt, pag, p+1, data=chk[1])
            return "sk1"

    def doNewClub(self, club=None):
        self.newclb = True
        if not club:
            self.club = getNextCode(self.sql, "bwlclb", "bcc_code", last=999)
            self.df.loadEntry("T", 0, 5, data=self.club)
            self.df.clearEntry("T", 0, 7)
            self.df.focusField("T", 0, 7)
        else:
            self.club = club

    def doClbNam(self, frt, pag, r, c, p, i, w):
        if not self.club:
            return "ff6|Invalid Club Code"

    def doEnd(self, widget=None):
        if widget:
            frt, pag, col, mes = self.df.doCheckFields()
            if mes:
                if col == 6:
                    self.df.focusField(frt, pag, (col), err=mes)
                else:
                    self.df.focusField(frt, pag, (col+1), err=mes)
                return
        data = self.df.t_work[0][0][:6]
        data.extend(self.df.t_work[0][0][7:])
        if self.newskp:
            self.sql.insRec("bwlskp", data=data)
        else:
            self.sql.updRec("bwlskp", data=data,
                where=[("bcs_code", "=", self.skip)])
        if self.newclb:
            self.sql.insRec("bwlclb", data=[self.club, self.df.t_work[0][0][6]])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()
