#! /usr/bin/python

"""
SYNOPSIS
    League Player's Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import RepPrt, TartanDialog, Sql
from tartanFunctions import askQuestion, getNextCode

class bc1050:
    def __init__(self, mf):
        self.mf = mf
        if self.setVariables():
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["bwlclb", "bwlflp"], prog=__name__)
        acc = self.sql.getRec(tables="ctlmst", cols=["ctm_name"],
            where=[("ctm_cono", "=", 1)], limit=1)
        if not acc:
            showError(self.mf.body, "Missing Record",
                "The Club Record is Missing, Please Create it and then Retry")
            return
        self.conam = acc[0]
        return True

    def mainProcess(self):
        tit = ("Player's Maintenance",)
        flp = {
            "stype": "R",
            "tables": ("bwlflp",),
            "cols": (
                ("bfp_code", "", 0, "Cod"),
                ("bfp_snam", "", 0, "Surame", "Y"),
                ("bfp_fnam", "", 0, "Names"),
                ("bfp_bsano", "", 0, "BSANO")),
            "order": "bfp_snam, bfp_fnam"}
        fld = (
            (("T",0,0,0),"I@bfp_code",0,"","",
                "","Y",self.doPlayer,flp,None,("efld",)),
            (("T",0,1,0),"I@bfp_snam",0,"","",
                "","N",None,None,self.doDelete,("notblank",)),
            (("T",0,2,0),"I@bfp_fnam",0,"","",
                "","N",None,None,None,("notblank",)),
            (("T",0,3,0),"I@bfp_bsano",0,"","",
                0,"N",self.doBSA,None,None,("notzero",)))
        but = (("Print",None,self.doPrint,1,None,None),)
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            butt=but, tend=tnd, txit=txt)

    def doPlayer(self, frt, pag, r, c, p, i, w):
        if not w:
            self.player = getNextCode(self.sql, "bwlflp", "bfp_code", last=999)
            self.df.loadEntry(frt, pag, p, data=self.player)
        else:
            self.player = w
        chk = self.sql.getRec(tables="bwlflp", where=[("bfp_code", "=",
            self.player)], limit=1)
        if not chk:
            self.newflp = True
        else:
            self.newflp = False
            for num, fld in enumerate(chk):
                self.df.loadEntry(frt, pag, num, data=fld)

    def doBSA(self, frt, pag, r, c, p, i, w):
        if self.sql.getRec(tables="bwlflp", where=[("bfp_code", "<>", self.player), ("bfp_bsano", "=", w)]):
            return "A Player with this BSA Number Already Exists"

    def doDelete(self, widget=None):
        if self.newflp:
            return
        error = False
        for tab in (("bwlflm", "bfm_captain"), ("bwlflt", "bft_skip"), ("bwlflt", "bft_player")):
            if self.sql.getRec(tables=tab[0], where=[(tab[1], "=", self.player)], limit=1):
                error = True
                break
        if error:
            return "There are Movements for this Player, Not Deleted"
        self.sql.delRec("bwlflp", where=[("bfp_code", "=", self.player)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doPrint(self, widget=None):
        self.df.mstAlw.hide()
        RepPrt(self.mf, conum=1, conam=self.conam, name=__name__,
            heads=["Player's Listing"], tables=["bwlflp"], order="bfp_snam",
            prtdia=("Y","V"))
        self.df.mstAlw.show()
        self.df.focusField("T", 0, 1)

    def doEnd(self, widget=None):
        data = self.df.t_work[0][0]
        if self.newflp:
            self.sql.insRec("bwlflp", data=data)
        else:
            self.sql.updRec("bwlflp", data=data, where=[("bfp_code", "=",
                self.player)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()
