#! /usr/bin/python

"""
SYNOPSIS
    Bowls League Side's Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os
from TartanClasses import TartanDialog, Sql
from tartanFunctions import getNextCode

class bc1060:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["bwlflf", "bwlflm", "bwlfls", "bwlflt"],
            prog=__name__)

    def mainProcess(self):
        tit = ("League Side's Maintenance",)
        fmt = {
            "stype": "R",
            "tables": ("bwlflf",),
            "cols": (
                ("bff_code", "", 0, "Cod"),
                ("bff_desc", "", 0, "Description", "Y"))}
        sid = {
            "stype": "R",
            "tables": ("bwlfls",),
            "cols": (
                ("bfs_code", "", 0, "Cod"),
                ("bfs_desc", "", 0, "Description", "Y"),
                ("bfs_league", "", 0, "L"),
                ("bfs_division", "", 0, "DV"),
                ("bfs_number", "", 0, "N")),
            "order": "bfs_code",
            "whera": [("T", "bfs_fmat", 0, 0)]}
        fle = {
            "stype":  "F",
            "types":  "fle",
            "initd": os.path.join(self.mf.prgdir, "img"),
            "ftype":  (("JPG Files", "*.jpg"),)}
        r1s = (("Main", "M"), ("Friendly", "F"))
        r2s = (("Yes", "Y"), ("No", "N"))
        fld = (
            (("T",0,0,0),"I@bff_code",0,"","",
                0,"Y",self.doFmat,fmt,None,("efld",)),
            (("T",0,0,0),"ONA",30,""),
            (("T",0,1,0),"I@bfs_code",0,"","",
                0,"Y",self.doSide,sid,None,("efld",)),
            (("T",0,2,0),"I@bfs_desc",0,"","",
                "","N",self.doDesc,None,self.doDelete,("notblank",)),
            (("T",0,3,0),("IRB",r1s),0,"League","",
                "M","N",None,None,None,None),
            (("T",0,4,0),"I@bfs_division",0,"","",
                "","N",None,None,None,("notblank",)),
            (("T",0,5,0),"I@bfs_number",0,"","",
                "","N",None,None,None,("in",(1,2,3))),
            (("T",0,6,0),("IRB",r2s),0,"Active Flag","",
                "Y","N",None,None,None,None))
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            tend=tnd, txit=txt)

    def doFmat(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="bwlflf", where=[("bff_code", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Format"
        self.fmat = w
        self.df.loadEntry(frt, pag, p+1, data=acc[1])

    def doSide(self, frt, pag, r, c, p, i, w):
        if not w:
            self.side = getNextCode(self.sql, "bwlfls", "bfs_code",
                where=[("bfs_fmat", "=", self.fmat)], last=999)
            self.df.loadEntry(frt, pag, p, data=self.side)
        else:
            self.side = w
        chk = self.sql.getRec(tables="bwlfls", where=[("bfs_fmat", "=",
            self.fmat), ("bfs_code", "=", self.side)], limit=1)
        if not chk:
            self.newfls = True
        else:
            self.newfls = False
            for num, fld in enumerate(chk[2:]):
                self.df.loadEntry(frt, pag, num+3, data=fld)

    def doDelete(self, widget=None):
        if self.newfls:
            return
        error = False
        for tab in (("bwlflm", "bfm_team"), ("bwlflt", "bft_team")):
            if self.sql.getRec(tables=tab[0], where=[(tab[1], "=", self.side)], limit=1):
                error = True
                break
        if error:
            return "There are Movements for this Side, Not Deleted"
        self.sql.delRec("bwlfls", where=[("bfs_fmat", "=", self.fmat),
            ("bfs_code", "=", self.side)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doDesc(self, frt, pag, r, c, p, i, w):
        chk = self.sql.getRec(tables="bwlflm", where=[("bfm_fmat", "=",
            self.fmat), ("bfm_team", "=", self.side)])
        if chk:
            return "sk3"

    def doEnd(self, widget=None):
        data = [self.fmat] + self.df.t_work[0][0][2:]
        if self.newfls:
            self.sql.insRec("bwlfls", data=data)
        else:
            self.sql.updRec("bwlfls", data=data, where=[("bfs_fmat", "=",
                self.fmat), ("bfs_code", "=", self.side)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()
