#! /usr/bin/python

"""
SYNOPSIS
    Bowls Competition Match Cards Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import TartanDialog, Sql
from tartanFunctions import askQuestion, showError

class bc2030:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        t = time.localtime()
        self.today = ((t[0] * 10000) + (t[1] * 100) + t[2])
        self.sql = Sql(self.mf.dbm, ["bwlcmp", "bwlskp", "bwlgme"],
            prog=__name__)

    def mainProcess(self):
        tit = ("Competition Match Cards",)
        com = {
            "stype": "R",
            "tables": ("bwlcmp",),
            "cols": (
                ("bcm_code", "", 0, "Cod"),
                ("bcm_name", "", 0, "Name", "Y"),
                ("bcm_date", "", 0, "Date"))}
        gme = {
            "stype": "R",
            "tables": ("bwlgme",),
            "cols": (
                ("bcg_game", "", 0, "GC"),
                ("bcg_date", "", 0, "Date")),
            "whera": [("T", "bcg_ccod", 0, 0)],
            "group": "bcg_game"}
        skp = {
            "stype": "R",
            "tables": ("bwlskp", "bwlgme"),
            "cols": (
                ("bcs_code", "", 0, "Cod"),
                ("bcs_surname", "", 0, "Surame", "Y"),
                ("bcs_names", "", 0, "Names"),
                ("bcg_rink", "", 0, "RK"),
                ("bcg_ocod", "", 0, "Opp")),
            "where": [("bcg_scod=bcs_code",)],
            "whera": [
                ("T", "bcg_ccod", 0, 0),
                ("T", "bcg_game", 2, 0)],
            "group": "bcs_code",
            "order": "bcs_code"}
        fld = (
            (("T",0,0,0),"I@bcm_code",0,"","",
                "","Y",self.doCmpCod,com,None,("notzero",)),
            (("T",0,0,0),"O@bcm_name",0,"Name"),
            (("T",0,1,0),"O@bcg_game",0,"Game Number"),
            (("T",0,1,0),"I@bcg_date",0,"Date","",
                self.today,"N",self.doDate,None,None,("efld",)),
            (("T",0,2,0),"IUA",30,"Greens","Greens (A,B,C)",
                "","N",self.doGreens,None,None,("efld",),None,"Available "\
                "Greens in the format A,B or A,B345 showing Green Code and "\
                "Rinks. If the Rinks are Not Entered they will Default to 6."),
            (("C",0,0,0),"I@bcg_scod",0,"","",
                "","N",self.doSkpCod,skp,None,("notzero",)),
            (("C",0,0,0),"ONA",30,"Skp-Name"),
            (("C",0,0,0),"I@bcg_ocod",0,"","",
                "","N",self.doOppCod,skp,None,("notzero",)),
            (("C",0,0,0),"ONA",30,"Opp-Name"),
            (("C",0,0,0),"I@bcg_rink",0,"","",
                "","N",self.doRink,None,None,("notblank",)))
        but = (("Quit",None,self.doQuit,1,None,None),)
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        cnd = ((self.doEnd,"y"),)
        cxt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            butt=but, tend=tnd, txit=txt, cend=cnd, cxit=cxt)

    def doCmpCod(self, frt, pag, r, c, p, i, w):
        com = self.sql.getRec(tables="bwlcmp", cols=["bcm_name", "bcm_date"],
            where=[("bcm_code", "=", w)], limit=1)
        if not com:
            return "Invalid Competition Code"
        name, self.sdat = com
        chk = self.sql.getRec(tables="bwlgme", cols=["bcg_game", "bcg_date",
            "sum(bcg_ocod)", "sum(bcg_points)"], where=[("bcg_ccod", "=", w)],
            group="bcg_game, bcg_date")
        self.ccod = w
        self.game = 0
        for c in chk:
            if c[2] and not c[3]:
                self.game = c[0]
                self.date = c[1]
                break
        if not self.game:
            ok = askQuestion(self.mf.body, "Manual Draw", "Is this Game "\
                "Number One and is it going to be a Manual Draw?", default="no")
            if ok == "no":
                return "No Valid Games in Competition"
            self.game = 1
            manual = True
        else:
            manual = False
        self.df.loadEntry(frt, pag, p+1, data=name)
        self.df.loadEntry(frt, pag, p+2, data=self.game)
        self.totskp = self.sql.getRec(tables="bwlent", cols=["count(*)"],
            where=[("bce_ccod", "=", self.ccod)], limit=1)[0]
        if manual:
            return "sk2"
        self.df.loadEntry(frt, pag, p+3, data=self.date)
        return "sk3"

    def doGamCod(self, frt, pag, r, c, p, i, w):
        chk = self.sql.getRec(tables="bwlgme", cols=["bcg_date",
            "sum(bcg_ocod)", "sum(bcg_points)"], where=[("bcg_ccod", "=",
            self.ccod), ("bcg_game", "=", w)], group="bcg_date", limit=1)
        if not chk[1]:
            return "Invalid Game Code, Draw Not Yet Done"
        if chk[2]:
            return "Invalid Game Code, Already Completed"
        self.game = w
        self.df.loadEntry(frt, pag, p+1, data=chk[0])

    def doDate(self, frt, pag, r, c, p, i, w):
        if w < self.today or w < self.sdat:
            return "Invalid Date, in the Past or Before the Starting Date"
        self.date = w

    def doGreens(self, frt, pag, r, c, p, i, w):
        if not w:
            return "Invalid Greens"
        self.rinks = []
        rinks = 0
        grns = w.split(",")
        for n, g in enumerate(grns):
            if len(g) == 1:
                for x in range(1, 7):
                    self.rinks.append("%s%s" % (g[0], x))
                rinks += 6
            else:
                for r in g[1:]:
                    self.rinks.append("%s%s" % (g[0], r))
                    rinks += 1
        if (self.totskp / 2) > rinks:
            return "Not Enough Rinks"

    def doSkpCod(self, frt, pag, r, c, p, i, w):
        skp = self.sql.getRec(tables=["bwlgme", "bwlskp"], cols=["bcs_surname",
            "bcs_names", "bcg_ocod", "bcg_rink"], where=[("bcg_ccod", "=",
            self.ccod), ("bcg_scod", "=", w), ("bcg_game", "=", self.game),
            ("bcs_code=bcg_scod",)], limit=1)
        if not skp:
            return "Invalid Skip Code"
        self.skip = w
        self.old_opp = skp[2]
        self.old_rink = skp[3]
        name = "%s, %s" % tuple(skp[:2])
        self.df.loadEntry(frt, pag, p+1, data=name)
        if self.old_opp:
            self.df.loadEntry(frt, pag, p+2, data=self.old_opp)
            opp = self.sql.getRec(tables="bwlskp", cols=["bcs_surname",
                "bcs_names"], where=[("bcs_code", "=", skp[2])], limit=1)
            name = "%s, %s" % tuple(opp[:2])
            self.df.loadEntry(frt, pag, p+3, data=name)
        if self.old_rink:
            self.df.loadEntry(frt, pag, p+4, data=self.old_rink)

    def doOppCod(self, frt, pag, r, c, p, i, w):
        opp = self.sql.getRec(tables=["bwlgme", "bwlskp"], cols=["bcs_surname",
            "bcs_names", "bcg_ocod", "bcg_rink"], where=[("bcg_ccod", "=",
            self.ccod), ("bcg_scod", "=", w), ("bcg_game", "=", self.game),
            ("bcs_code=bcg_scod",)], limit=1)
        if not opp:
            return "Invalid Opponents Code"
        self.new_opp = w
        self.chg_skp = opp[2]
        name = "%s, %s" % tuple(opp[:2])
        self.df.loadEntry(frt, pag, p+1, data=name)

    def doRink(self, frt, pag, r, c, p, i, w):
        if w not in self.rinks:
            return "Invalid Rink"
        self.new_rink = w

    def doEnd(self, widget=None):
        if self.df.frt == "T":
            self.df.focusField("C", 0, 1)
        else:
            self.df.advanceLine(0)
            self.sql.updRec("bwlgme", cols=["bcg_date", "bcg_ocod",
                "bcg_rink"], data=[self.date, self.new_opp, self.new_rink],
                where=[("bcg_ccod", "=", self.ccod), ("bcg_scod", "=",
                self.skip), ("bcg_game", "=", self.game)])
            self.sql.updRec("bwlgme", cols=["bcg_date", "bcg_ocod",
                "bcg_rink"], data=[self.date, self.skip, self.new_rink],
                where=[("bcg_ccod", "=", self.ccod), ("bcg_scod", "=",
                self.new_opp), ("bcg_game", "=", self.game)])

    def doExit(self, widget=None):
        if self.df.frt == "C":
            chk = self.sql.getRec(tables="bwlgme", cols=["bcg_ocod",
                "count(*)"], where=[("bcg_ccod", "=", self.ccod), ("bcg_game",
                "=", self.game)], group="bcg_ocod", order="bcg_ocod")
            for c in chk:
                if c[1] <> 1:
                    skp = self.sql.getRec(tables="bwlgme", cols=["bcg_scod"],
                        where=[("bcg_ccod", "=", self.ccod), ("bcg_game", "=",
                        self.game), ("bcg_ocod", "=", c[0])], order="bcg_scod")
                    err = "Opponent %s is Drawn Against %s Skips\n" % tuple(c)
                    for s in skp:
                        err = err + "\nSkip %s" % s[0]
                    skp = self.sql.sqlRec("Select bce_scod from bwlent where "\
                        "bce_ccod = %s and bce_scod not in (select bcg_ocod "\
                        "from bwlgme where bcg_ccod = %s and bcg_game = %s)" %
                        (self.ccod, self.ccod, self.game))
                    err = err + "\n\nSkips Without Opponents\n"
                    for s in skp:
                        err = err + "\nSkip %s" % s[0]
                    showError(self.mf.body, "Skip Error", err)
                    return
            chk = self.sql.getRec(tables="bwlgme", cols=["bcg_rink",
                "count(*)"], where=[("bcg_ccod", "=", self.ccod), ("bcg_game",
                "=", self.game)], group="bcg_rink", order="bcg_rink")
            for c in chk:
                if c[1] <> 2:
                    skp = self.sql.getRec(tables="bwlgme", cols=["bcg_scod",
                        "bcg_ocod"], where=[("bcg_ccod", "=", self.ccod),
                        ("bcg_game", "=", self.game), ("bcg_rink", "=", c[0])],
                        group="bcg_scod, bcg_rink", order="bcg_scod")
                    skps = []
                    for s in skp:
                        if [s[1], s[0]] not in skps:
                            skps.append(s)
                    err = "Rink %s Shows %s time(s)\n" % (c[0], c[1] / 2)
                    for s in skps:
                        err = err + "\nSkip %2s Opp %2s" % (s[0], s[1])
                    rnk = self.sql.getRec(tables="bwlgme", cols=["bcg_rink",
                        "count(*)"], where=[("bcg_ccod", "=", self.ccod),
                        ("bcg_game", "=", self.game)], group="bcg_rink",
                        order="bcg_rink")
                    rnks = []
                    for r in rnk:
                        rnks.append(r[0])
                    mis = ""
                    for r in self.rinks:
                        if r not in rnks:
                            if not mis:
                                mis = r
                            else:
                                mis = "%s, %s" % (mis, r)
                    err = err + "\n\nMissing Rink(s)\n\n%s" % mis
                    showError(self.mf.body, "Rink Error", err)
                    return
            self.mf.dbm.commitDbase(ask=True)
        self.df.closeProcess()
        self.mf.closeLoop()

    def doQuit(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.closeProcess()
        self.mf.closeLoop()
