#! /usr/bin/python

"""
SYNOPSIS
    Bowls League Selections.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, sys, time
from TartanClasses import TartanDialog, Sql
from tartanFunctions import askQuestion, dateDiff, doDrawTable, doReportlab
from tartanFunctions import doPrinter, getModName, getNextCode, showError
from tartanWork import mthnam

class bc2050:
    def __init__(self, mf):
        self.mf = mf
        if self.setVariables():
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlmst", "bwlclb", "bwlflf", "bwlflm",
            "bwlflo", "bwlflp", "bwlfls", "bwlflt"], prog=__name__)
        acc = self.sql.getRec(tables="ctlmst", cols=["ctm_name"],
            where=[("ctm_cono", "=", 1)], limit=1)
        if not acc:
            showError(self.mf.body, "Missing Record",
                "The Club Record is Missing, Please Create it and then Retry")
            return
        self.conam = acc[0]
        acc = self.sql.getRec(tables="bwlclb")
        if not acc:
            self.sql.insRec("bwlclb", data=[1, self.conam])
            self.mf.dbm.commitDbase()
        self.reprint = False
        self.fmat = 0
        self.date = 0
        self.skip = 0
        return True

    def mainProcess(self):
        tit = "League Selections"
        fmt = {
            "stype": "R",
            "tables": ("bwlflf",),
            "cols": (
                ("bff_code", "", 0, "Cod"),
                ("bff_desc", "", 0, "Description", "Y"))}
        tem = {
            "stype": "R",
            "tables": ("bwlfls",),
            "cols": (
                ("bfs_code", "", 0, "Cod"),
                ("bfs_desc", "", 0, "Description", "Y"),
                ("bfs_division", "", 0, "DV")),
            "where": [("bfs_active", "=", "Y")],
            "whera": [("T", "bfs_fmat", 0, 0)],
            "order": "bfs_desc"}
        opp = {
            "stype": "R",
            "tables": ("bwlflo",),
            "cols": (
                ("bfo_code", "", 0, "Cod"),
                ("bfo_desc", "", 0, "Description", "Y")),
            "whera": [("T", "bfo_fmat", 0, 0)],
            "order": "bfo_desc"}
        plr = {
            "stype": "R",
            "tables": ("bwlflp",),
            "cols": (
                ("bfp_code", "", 0, "Cod"),
                ("bfp_snam", "", 0, "Surname", "Y"),
                ("bfp_fnam", "", 0, "Names"))}
        r1s = (("Fixture", "F"), ("Practice", "P"))
        fld = (
            (("T",0,0,0),"I@bfm_fmat",0,"","",
                "","N",self.doFmat,fmt,None,("efld",)),
            (("T",0,0,0),"ONA",30,""),
            (("T",0,1,0),"I@bfm_date",0,"","",
                "","N",self.doDate,None,None,("efld",)),
            (("T",0,2,0),("IRB",r1s),0,"Type","",
                "F","N",self.doType,None,None,None),
            (("T",0,3,0),"I@bfm_round",0,"","",
                "","N",self.doRound,None,None,None),
            (("T",0,4,0),"I@bfm_team",0,"","",
                "","N",self.doSide,tem,None,None),
            (("T",0,4,0),"ONA",20,""),
            (("T",0,5,0),"I@bfm_opposition",0,"","",
                "","N",self.doOpp,opp,None,None),
            (("T",0,5,0),"ONA",30,""),
            (("T",0,6,0),"I@bfm_venue",0,"Venue (H/A/Name)","",
                "","N",self.doVenue,None,None,("efld",)),
            (("T",0,7,0),"IUD",5.2,"Meeting Time","",
                "","N",self.doMTime,None,None,("efld",)),
            (("T",0,7,0),"INA",20," At (H/A/Name)","",
                "H","N",self.doMPlace,None,None,("efld",)),
            (("T",0,8,0),"I@bfm_captain",0,"","",
                "","N",self.doCaptain,plr,None,None),
            (("T",0,8,0),"ONA",30,""),
            (("C",0,0,1),"I@bft_skip",0,"","",
                "","N",self.doSkip,plr,None,None),
            (("C",0,0,2),"ONA",20,""),
            (("C",0,0,3),"I@bft_player",0,"","",
                "","N",self.doPlayer,plr,None,None),
            (("C",0,0,4),"ONA",20,""),
            (("C",0,0,5),"I@bft_player",0,"","",
                "","N",self.doPlayer,plr,None,None),
            (("C",0,0,6),"ONA",20,""),
            (("C",0,0,7),"I@bft_player",0,"","",
                "","N",self.doPlayer,plr,None,None),
            (("C",0,0,8),"ONA",20,""))
        but = (("Quit",None,self.doQuit,1,None,None),)
        tnd = ((self.doEnd,"n"),)
        txt = (self.doExit,)
        cnd = ((self.doEnd,"n"),)
        cxt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            rows=[3], butt=but, tend=tnd, txit=txt, cend=cnd, cxit=cxt)

    def doFmat(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="bwlflf", where=[("bff_code", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Format"
        self.fmat = w
        self.fdsc = acc[1].upper()
        self.df.loadEntry(frt, pag, p+1, data=acc[1])

    def doDate(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="bwlflt", cols=["sum(bft_shotsf)"],
            where=[("bft_fmat", "=", self.fmat), ("bft_date", "=", w)],
            limit=1)
        if acc and acc[0]:
            self.reprint = True
        self.date = w
        self.disp = "%i %s %i" % (w % 100, mthnam[(w/100)%100][1], w/10000)
        if self.reprint:
            self.doExit()
        acc = self.sql.getRec(tables="bwlflm", cols=["bfm_round"],
            where=[("bfm_fmat", "=", self.fmat), ("bfm_date",
            "=", self.date)], limit=1)
        if acc:
            self.rnd = acc[0]
            if self.rnd == 0:
                self.df.loadEntry(frt, pag, p+1, data="P")
            else:
                self.df.loadEntry(frt, pag, p+1, data="F")
            self.df.loadEntry(frt, pag, p+2, data=self.rnd)
            return "sk2"
        acc = self.sql.getRec(tables="bwlflm", cols=["max(bfm_date)",
            "max(bfm_round)"], where=[("bfm_fmat", "=", self.fmat)],
            limit=1)
        if not acc[0] or dateDiff(acc[0], self.date, ptype="months") > 3:
            self.nxt = 1
        else:
            self.nxt = acc[1] + 1

    def doType(self, frt, pag, r, c, p, i, w):
        if w == "P":
            self.rnd = 0
            self.df.loadEntry(frt, pag, p+1, data=self.rnd)
            return "sk1"

    def doRound(self, frt, pag, r, c, p, i, w):
        if not w or w > self.nxt:
            return "Invalid Round"
        self.rnd = w

    def doSide(self, frt, pag, r, c, p, i, w):
        if not w:
            if askQuestion(self.mf.body, "New Side",
                    "Do You Want to Enter a New Side") == "yes":
                w = self.enterNewSide()
                self.df.loadEntry(frt, pag, p, data=w)
        acc = self.sql.getRec(tables="bwlfls", where=[("bfs_fmat", "=",
            self.fmat), ("bfs_code", "=", w), ("bfs_active", "=", "Y")],
            limit=1)
        if not acc:
            return "Invalid Side"
        self.team = w
        self.df.loadEntry(frt, pag, p+1, data=acc[2])
        self.qty = acc[5]
        #self.df.colPage0.show_all()
        #for x in range(self.qty, 9):
        #    for y in range(8):
        #        self.df.colEntry[0][(x * 8) + y].hide()
        acc = self.sql.getRec(tables=["bwlflm", "bwlfls", "bwlflo", "bwlflp"],
            cols=["bfo_code", "bfo_desc", "bfm_venue", "bfm_mtime",
            "bfm_mplace", "bfm_captain", "bfp_snam", "bfp_fnam"],
            where=[("bfm_fmat", "=", self.fmat), ("bfm_date", "=", self.date),
            ("bfm_team", "=", self.team), ("bfs_fmat=bfm_fmat",),
            ("bfs_code=bfm_team",), ("bfo_fmat=bfm_fmat",),
            ("bfo_code=bfm_opposition",), ("bfp_code=bfm_captain",)], limit=1)
        if acc:
            self.opp = acc[0]
            self.venue = acc[2]
            self.mtime = acc[3]
            self.mplace = acc[4]
            self.captain = acc[5]
            for x in range(6):
                self.df.loadEntry(frt, pag, p+x+2, data=acc[x])
            self.df.loadEntry(frt, pag, p+8, data=self.getName(acc[6], acc[7]))
            self.loadPlayers()
        else:
            for x in range(c+2, c+9):
                self.df.clearEntry("T", 0, x)
            self.df.clearFrame("C", 0)
            # Get previous captain
            acc = self.sql.getRec(tables=["bwlflm", "bwlflp"],
                cols=["bfm_captain", "bfp_snam", "bfp_fnam"],
                where=[("bfm_fmat", "=", self.fmat), ("bfm_team", "=",
                self.team), ("bfp_code=bfm_captain",)], order="bfm_date desc",
                limit=1)
            if acc:
                self.df.loadEntry("T", 0, 12, data=acc[0])
                self.df.loadEntry("T", 0, 13, data=self.getName(acc[1], acc[2]))
            # Get previous team members
            acc = self.sql.getRec(tables=["bwlflt", "bwlflp"], cols=["bft_date",
                "bft_player", "bfp_snam", "bfp_fnam"], where=[("bft_fmat",
                "=", self.fmat), ("bft_team", "=", self.team),
                ("bfp_code=bft_player",)],
                order="bft_date desc, bft_skip, bft_position")
            if acc:
                date = acc[0][0]
                for num, data in enumerate(acc):
                    if data[0] != date:
                        break
                    pos = num * 2
                    self.df.loadEntry("C", 0, pos, data=data[1])
                    self.df.loadEntry("C", 0, pos+1, data=self.getName(data[2],
                        data[3]))

    def doOpp(self, frt, pag, r, c, p, i, w):
        if not w:
            if askQuestion(self.mf.body, "New Opponent",
                    "Do You Want to Enter a New Opponent") == "yes":
                w = self.enterNewOpp()
                self.df.loadEntry(frt, pag, p, data=w)
        acc = self.sql.getRec(tables="bwlflm", where=[("bfm_fmat", "=",
            self.fmat), ("bfm_date", "=", self.date), ("bfm_opposition", "=",
            w), ("bfm_team", "<>", self.team)], limit=1)
        if acc:
            return "Invalid Opposition, Already Playing"
        acc = self.sql.getRec(tables="bwlflo", where=[("bfo_fmat", "=",
            self.fmat), ("bfo_code", "=", w)], limit=1)
        if not acc:
            return "Invalid Opposition"
        self.opp = w
        self.df.loadEntry(frt, pag, p+1, data=acc[3])

    def loadPlayers(self):
        acc = self.sql.getRec(tables=["bwlflt", "bwlflp"], cols=["bft_player",
            "bfp_snam", "bfp_fnam"], where=[("bft_fmat", "=", self.fmat),
            ("bft_date", "=", self.date), ("bft_team", "=", self.team),
            ("bfp_code=bft_player",)], order="bft_skip, bft_position")
        if acc:
            for num, name in enumerate(acc):
                pos = num * 2
                self.df.loadEntry("C", 0, pos, data=name[0])
                self.df.loadEntry("C", 0, pos+1, data=self.getName(name[1],
                    name[2]))

    def doVenue(self, frt, pag, r, c, p, i, w):
        w = w.strip()
        if len(w.strip()) == 1:
            w = w.upper()
            if w not in ("A", "H"):
                return "Invalid Venue"
        self.venue = w
        self.df.loadEntry(frt, pag, p, data=self.venue)
        if not self.df.t_work[0][0][p+1]:
            if self.venue == "H":
                self.df.loadEntry(frt, pag, p+1, data=1.30)
            else:
                self.df.loadEntry(frt, pag, p+1, data=1.15)

    def doMTime(self, frt, pag, r, c, p, i, w):
        self.mtime = w

    def doMPlace(self, frt, pag, r, c, p, i, w):
        w = w.strip()
        if len(w.strip()) == 1:
            w = w.upper()
            if w not in ("A", "H"):
                return "Invalid Place"
        self.mplace = w
        self.df.loadEntry(frt, pag, p, data=self.mplace)

    def doCaptain(self, frt, pag, r, c, p, i, w):
        if not w:
            if askQuestion(self.mf.body, "New Player",
                    "Do You Want to Enter a New Player") == "yes":
                w = self.enterNewPlayer()
                self.df.loadEntry(frt, pag, p, data=w)
        acc = self.sql.getRec(tables="bwlflp", cols=["bfp_snam", "bfp_fnam"],
            where=[("bfp_code", "=", w)], limit=1)
        if not acc:
            return "Invalid Captain"
        self.captain = w
        self.df.loadEntry(frt, pag, p+1, data=self.getName(acc[0], acc[1]))

    def doSkip(self, frt, pag, r, c, p, i, w):
        if not w:
            if askQuestion(self.mf.body, "New Player",
                    "Do You Want to Enter a New Player") == "yes":
                w = self.enterNewPlayer()
                self.df.loadEntry(frt, pag, p, data=w)
        acc = self.sql.getRec(tables="bwlflp", cols=["bfp_snam", "bfp_fnam"],
            where=[("bfp_code", "=", w)], limit=1)
        if not acc:
            return "Invalid Skip"
        self.skip = w
        self.df.loadEntry(frt, pag, p+1, data=self.getName(acc[0], acc[1]))

    def doPlayer(self, frt, pag, r, c, p, i, w):
        if not w:
            if askQuestion(self.mf.body, "New Player",
                    "Do You Want to Enter a New Player") == "yes":
                w = self.enterNewPlayer()
                self.df.loadEntry(frt, pag, p, data=w)
        if w == self.skip:
            return "Invalid Player, Is the Skip"
        acc = self.sql.getRec(tables="bwlflp", cols=["bfp_snam", "bfp_fnam"],
            where=[("bfp_code", "=", w)], limit=1)
        if not acc:
            return "Invalid Player"
        self.df.loadEntry(frt, pag, p+1, data=self.getName(acc[0], acc[1]))

    def enterNewSide(self):
        tit = ("New Side",)
        lge = (("Main", "M"), ("Friendly", "F"))
        fld = (
            (("T",0,0,0),"I@bfs_desc",0,"","",
                "","N",None,None,None,("notblank",)),
            (("T",0,1,0),("IRB",lge),0,"League","",
                "M","N",None,lge,None,None),
            (("T",0,2,0),"I@bfs_division",0,"","",
                "","N",None,None,None,("notblank",)),
            (("T",0,3,0),"I@bfs_number",0,"","",
                "","N",None,None,None,("notzero",)))
        state = self.df.disableButtons()
        self.tx = TartanDialog(self.mf, title=tit, newscn=(0,1), tops=True,
            eflds=fld, tend=((self.doSideEnd,"y"),), txit=(self.doSideExit,))
        self.mf.startLoop()
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)
        return self.team

    def doSideEnd(self):
        team = self.sql.getRec(tables="bwlfls", cols=["max(bfs_code)"],
            where=[("bfs_fmat", "=", self.fmat)], limit=1)
        if not team[0]:
            self.team = 1
        else:
            self.team = team[0] + 1
        self.sql.insRec("bwlfls", data=[self.fmat, self.team] +
            self.tx.t_work[0][0] + ["Y"])
        self.mf.dbm.commitDbase()
        self.tx.closeProcess()
        self.mf.closeLoop()

    def doSideExit(self):
        self.team = 0
        self.tx.closeProcess()
        self.mf.closeLoop()

    def enterNewOpp(self):
        tit = ("New Opponent",)
        clb = {
            "stype": "R",
            "tables": ("bwlclb",),
            "cols": (
                ("bcc_code", "", 0, "Cod"),
                ("bcc_name", "", 0, "Name", "Y"))}
        fld = (
            (("T",0,0,0),"I@bcc_code",0,"","",
                "","N",self.doClubCode,clb,None,("efld",)),
            (("T",0,0,0),"I@bcc_name",0,"","",
                "","N",self.doClubDesc,None,None,("notblank",)),
            (("T",0,1,0),"I@bfo_desc",0,"","",
                "","N",None,None,None,("notblank",)))
        state = self.df.disableButtons()
        self.tx = TartanDialog(self.mf, title=tit, newscn=(0,1), tops=True,
            eflds=fld, tend=((self.doOppEnd,"y"),), txit=(self.doOppExit,))
        self.mf.startLoop()
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)
        return self.opp

    def doClubCode(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="bwlclb", where=[("bcc_code", "=", w)],
            limit=1)
        if acc:
            self.tx.loadEntry(frt, pag, p+1, data=acc[1])
            return "sk1"

    def doClubDesc(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="bwlclb", where=[("bcc_name", "=", w)],
            limit=1)
        if acc:
            self.tx.loadEntry(frt, pag, p-1, data=acc[0])
        else:
            self.club = getNextCode(self.sql, "bwlclb", "bcc_code", last=999)
            self.tx.loadEntry(frt, pag, p-1, data=self.club)
            self.sql.insRec("bwlclb", data=[self.club, w])
            self.mf.dbm.commitDbase()

    def doOppEnd(self):
        opp = self.sql.getRec(tables="bwlflo", where=[("bfo_fmat", "=",
            self.fmat), ("bfo_desc", "=", self.tx.t_work[0][0][2])], limit=1)
        if opp:
            self.opp = opp[1]
        else:
            opp = self.sql.getRec(tables="bwlflo", cols=["max(bfo_code)"],
                where=[("bfo_fmat", "=", self.fmat)], limit=1)
            if not opp[0]:
                self.opp = 1
            else:
                self.opp = opp[0] + 1
            self.sql.insRec("bwlflo", data=[self.fmat, self.opp,
                self.tx.t_work[0][0][0], self.tx.t_work[0][0][2]])
            self.mf.dbm.commitDbase()
        self.tx.closeProcess()
        self.mf.closeLoop()

    def doOppExit(self):
        self.opp = 0
        self.tx.closeProcess()
        self.mf.closeLoop()

    def enterNewPlayer(self):
        tit = ("New Player",)
        fld = (
            (("T",0,0,0),"I@bfp_snam",0,"","",
                "","N",None,None,None,("notblank",)),
            (("T",0,1,0),"I@bfp_fnam",0,"","",
                "","N",None,None,None,("notblank",)),
            (("T",0,2,0),"I@bfp_bsano",0,"","",
                "","N",None,None,None,("notzero",)))
        state = self.df.disableButtons()
        self.tx = TartanDialog(self.mf, title=tit, newscn=(0,1), tops=True,
            eflds=fld, tend=((self.doPlayEnd,"y"),), txit=(self.doPlayExit,))
        self.mf.startLoop()
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)
        return self.play

    def doPlayEnd(self):
        play = self.sql.getRec(tables="bwlflp", cols=["max(bfp_code)"],
            limit=1)
        if not play[0]:
            self.play = 1
        else:
            self.play = play[0] + 1
        snam, fnam, bsano = self.tx.t_work[0][0]
        init = fnam.split()
        name = "%s, %s" % (snam, init[0][0])
        if self.sql.getRec(tables="bwlflp", where=[("bfp_bsano", "=", bsano)], limit=1):
            showError(self.mf.body, "Duplicate", "A Player with this BSA "\
                "Number (%s) Already Exists." % bsano)
            self.tx.focusField("T", 0, 3)
            return
        self.sql.insRec("bwlflp", data=[self.play, snam, fnam, bsano])
        self.mf.dbm.commitDbase()
        self.tx.closeProcess()
        self.mf.closeLoop()

    def doPlayExit(self):
        self.play = 0
        self.tx.closeProcess()
        self.mf.closeLoop()

    def doEnd(self):
        if self.df.frt == "T":
            self.df.focusField("C", 0, 1)
        elif self.df.row == (self.qty - 1):
            self.doUpdate()
        else:
            self.df.advanceLine(0)

    def doUpdate(self, widget=None):
        self.sql.delRec("bwlflm", where=[("bfm_fmat", "=", self.fmat),
            ("bfm_date", "=", self.date), ("bfm_team", "=", self.team)])
        self.sql.insRec("bwlflm", data=[self.fmat, self.date, self.rnd,
            self.team, self.opp, self.venue, self.mtime, self.mplace,
            self.captain, 0])
        self.sql.delRec("bwlflt", where=[("bft_fmat", "=", self.fmat),
            ("bft_date", "=", self.date), ("bft_team", "=", self.team)])
        for x in range(self.qty):
            for y in range(4):
                data = [self.fmat, self.date, self.team,
                    self.df.c_work[0][x][0], self.df.c_work[0][x][y*2],
                    y+1, 0, 0, 0, 0, 0, 0, ""]
                acc = self.sql.getRec(tables="bwlflt", where=[("bft_fmat", "=",
                    data[0]), ("bft_date", "=", data[1]), ("bft_team", "=",
                    data[2]), ("bft_player", "=", data[4])], limit=1)
                if acc:
                    showError(self.mf.body, "Duplicate", "Player Code %s is "\
                        "Entered More than Once" % data[4])
                    self.df.focusField("C", 0, 1)
                    self.mf.dbm.rollbackDbase()
                    return
                self.sql.insRec("bwlflt", data=data)
        for x in range(6, 15):
            self.df.clearEntry("T", 0, x)
        self.df.clearFrame("C", 0)
        self.df.skip = [[1, 2, 3, 4, 5]]
        self.df.focusField("T", 0, 6)

    def doExit(self):
        if self.df.frt == "C":
            self.df.focusField("C", 0, self.df.col)
            return
        if not self.fmat or not self.date:
            self.df.closeProcess()
            self.mf.closeLoop()
            return
        plrs = {}
        acc = self.sql.getRec(tables="bwlflt", where=[("bft_fmat", "=",
            self.fmat), ("bft_date", "=", self.date)], order="bft_player")
        for a in acc:
            if a[4] in plrs:
                showError(self.mf.body, "Duplicate", "Player Code %s is in "\
                    "Team Codes %s and %s" % (a[4], plrs[a[4]][0], a[2]))
                self.df.focusField("T", 0, 6)
                return
            else:
                plrs[a[4]] = [a[2], a[3], a[5]]
        self.df.closeProcess()
        if not self.reprint:
            self.mf.dbm.commitDbase()
        ok = askQuestion(self.mf.body, "Print",
            "Do You Want to View/Print the Selections")
        if ok == "yes":
            self.doPrintSelection()
        self.mf.closeLoop()

    def doPrintSelection(self):
        tit = "Print/View Selection"
        fld = (
            (("T",0,0,0),"INA",70,"Note-Line-1","",
                "","N",None,None,None,("efld",)),
            (("T",0,1,0),"INA",70,"Note-Line-2","",
                "","N",None,None,None,("efld",)))
        self.pr = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            tend=((self.doPrint,"y"),), txit=None, view=("N","V"),
            mail=("Y","N"))
        self.mf.startLoop()

    def doPrint(self):
        self.pr.closeProcess()
        pdfnam = getModName(self.mf.wrkdir, __name__, "select_%s" % self.date,
            ext="pdf")
        c, cpi, inch, ss, dd = doReportlab(self.mf.prgdir, pdfnam, orient="A4")
        cw = c.stringWidth(("X"), "Tartan", cpi)  # character width
        ld = inch / 6                             # line depth
        fm = {
            "margins": ((10, 80), (5, 12.5)),     # left, right, top, bottom
            "repeat": (1, 5),                     # repeat across and below
            "rows": (                             # x, y, w, h, fill, text, qty
                (10, 5, 80, 1.5, .8),
                (10, 6.5, (
                    (10, 1.5, .9, ("Skip", "Third", "Second", "Lead")),
                    (23, 1.5), (23, 1.5), (24, 1.5)), 4),
                (10, 12.5, 80, 1.5))}
        ff = {
            "margins": ((10, 80), ((5, 15))),     # left, right, top, bottom
            "repeat": (1, 3),                     # repeat across and below
            "rows": (                             # x, y, w, h, fill, text, qty
                (10, 5, 80, 2, .8),
                (10, 7, (
                    (10, 2, .9, ("Skip", "Third", "Second", "Lead")),
                    (70, 2)), 4),
                (10, 15, 80, 2))}
        for div in ("Main", "Friendly"):
            whr = [("bfm_fmat", "=", self.fmat), ("bfm_date", "=", self.date)]
            if div == "Main":
                ppad = 1
                rr = fm
                h1 = "TEAMS FOR %s FIXTURE - %s %s %s" % (self.fdsc,
                    self.date%100, mthnam[self.date/100%100][1],
                    self.date/10000)
                whr.append(("bfs_league", "=", "M"))
            else:
                ppad = 1.5
                rr = ff
                h1 = "TEAMS FOR %s FRIENDLY - %s %s %s" % (self.fdsc,
                    self.date%100, mthnam[self.date/100%100][1],
                    self.date/10000)
                whr.append(("bfs_league", "=", "F"))
            whr.extend([
                ("bfs_fmat=bfm_fmat",),
                ("bfs_code=bfm_team",),
                ("bfo_fmat=bfm_fmat",),
                ("bfo_code=bfm_opposition",),
                ("bcc_code=bfo_club",)])
            games = self.sql.getRec(tables=["bwlflm", "bwlfls", "bwlflo",
                "bwlclb"], cols=["bfm_round", "bfm_team", "bfs_desc",
                "bfo_desc", "bfm_venue", "bfm_mtime", "bfm_mplace",
                "bfm_captain", "bcc_name", "bfs_number"], where=whr,
                order="bfm_team")
            if not games:
                continue
            if games[0][0] == 0:
                h1 = h1.replace("FIXTURE", "PRACTICE")
            totp = 0
            for game in games:
                if totp < game[9]:
                    totp = game[9] * 4
            # Draw headings
            c.setFont("Helvetica-Bold", 18)
            c.drawCentredString(50*cw, 3*ld, h1)
            c.setFont("Helvetica-Bold", 12)
            # Draw table
            last, table = doDrawTable(c, cw, ld, rr, ppad, spad=2)
            # Fill Form
            tc = []
            pl = []
            c.setFont("Helvetica-Bold", 12)
            for x, game in enumerate(games):
                tc.append(game[7])
                if game[4].strip().upper() == "H":
                    text = "%s vs %s at %s" % (game[2], game[3], self.conam)
                elif game[4].strip().upper() == "A":
                    text = "%s vs %s at %s" % (game[2], game[3], game[8])
                else:
                    text = "%s vs %s at %s" % (game[2], game[3], game[4])
                pos = table[0][1][x] + ppad
                c.drawCentredString(50*cw, pos*ld,  text)
                pos = table[-1][1][x] + ppad
                if game[6].strip().upper() in ("H", "A"):
                    text = "Meet at the Club at %5.2f" % game[5]
                else:
                    text = "Meet at %s at %5.2f" % (game[6], game[5])
                c.drawCentredString(50*cw, pos*ld, text)
                plrs = self.sql.getRec(tables=["bwlflt", "bwlflp"],
                    cols=["bfp_code", "bfp_snam", "bfp_fnam"],
                    where=[("bft_fmat", "=", self.fmat), ("bft_date", "=",
                    self.date), ("bft_team", "=", game[1]),
                    ("bfp_code=bft_player",)],
                    order="bft_team, bft_skip, bft_position")
                while len(plrs) < totp:
                    plrs.append(["", "", ""])
                pl.extend(plrs)
            c.setFont("Helvetica", 12)
            col = len(rr["rows"][1][2])
            cnt = 0
            for p in pl:
                if p == ["", "", ""]:
                    continue
                pn = self.getName(p[1], p[2])
                if p[0] in tc:
                    pn = "%s (C)" % pn
                x = table[(((cnt%totp)/4)+2)][0][0] + 1
                y = table[((cnt-((cnt/4)*4))*col)+2][1][cnt/totp] + ppad
                c.drawString(x*cw, y*ld, pn)
                cnt += 1
            # Draw trailer
            c.setFont("Helvetica-Bold", 14)
            c.drawString(10.0*cw, (last+4)*ld, "Please tick your names "\
                "on the right hand side to indicate availability.")
            c.drawString(10.0*cw, (last+5.5)*ld, "If unavailable please "\
                "inform skips and skips to inform selectors.")
            c.drawString(10.0*cw, (last+7.5)*ld, "Captains are responsible "\
                "to distribute and return assessment forms, ")
            c.drawString(10.0*cw, (last+9)*ld, "completed and initialled, "\
                "to a selector. In this regard please note ")
            c.drawString(10.0*cw, (last+10.5)*ld, "assessment guide on the "\
                "board.")
            if self.pr.t_work[0][0][0]:
                c.drawString(10.0*cw, (last+13)*ld, self.pr.t_work[0][0][0])
            if self.pr.t_work[0][0][1]:
                c.drawString(10.0*cw, (last+15)*ld, self.pr.t_work[0][0][1])
            c.showPage()
        c.save()
        head = "Match Selections for %s" % self.disp
        doPrinter(mf=self.mf, head=head, rep=pdfnam, view=self.pr.repprt[0][1],
            prtnam=self.pr.repprt[0][2], mail=self.pr.repprt[1][2])
        self.mf.closeLoop()

    def getName(self, snam, fnam):
        i = fnam.split()
        return "%s, %s" % (snam, i[0].capitalize())

    def doQuit(self, widget=None):
        self.df.closeProcess()
        self.mf.dbm.rollbackDbase()
        self.mf.closeLoop()
