#! /usr/bin/python

"""
SYNOPSIS
    Bowls League Results and Assessments.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import TartanDialog, Sql

class bc2060:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["bwlflf", "bwlflm", "bwlflt"],
            prog=__name__)
        self.skip = 0

    def mainProcess(self):
        tit = "League Results and Assessments"
        fmt = {
            "stype": "R",
            "tables": ("bwlflf",),
            "cols": (
                ("bff_code", "", 0, "Cod"),
                ("bff_desc", "", 0, "Description", "Y"))}
        dte = {
            "stype": "R",
            "tables": ("bwlflt",),
            "cols": (("bft_date", "", 0, "Match-Date"),),
            "group": "bft_date",
            "whera": [("T", "bft_fmat", 0, 0)],
            "order": "bft_date"}
        skp = {
            "stype": "R",
            "tables": ("bwlflt", "bwlfls", "bwlflp"),
            "cols": (
                ("bft_skip", "", 0, "Skp"),
                ("bfp_snam", "", 0, "Surname", "Y"),
                ("bfp_fnam", "", 0, "Names"),
                ("bft_shotsf", "", 0, "For"),
                ("bft_shotsa", "", 0, "Agt")),
            "where": [
                ("bfs_league", "=", "M"),
                ("bfs_code=bft_team",),
                ("bfp_code=bft_skip",)],
            "whera": [
                ("T", "bft_fmat", 0, 0),
                ("T", "bft_date", 2, 0)],
            "group": "bft_skip",
            "order": "bfp_snam, bfp_fnam"}
        plr = {
            "stype": "R",
            "tables": ("bwlflt", "bwlflp"),
            "cols": (
                ("bft_player", "", 0, "Plr"),
                ("bfp_snam", "", 0, "Name", "Y"),
                ("bfp_fnam", "", 0, "Name")),
            "where": [("bfp_code=bft_player",)],
            "whera": [
                ["T", "bft_fmat", 0, 0],
                ["T", "bft_date", 2, 0],
                ["C", "bft_skip", 0, 0]],
            "group": "bft_player"}
        fld = (
            (("T",0,0,0),"I@bff_code",0,"","",
                0,"N",self.doFmat,fmt,None,("efld",)),
            (("T",0,0,0),"ONA",30,""),
            (("T",0,1,0),"I@bft_date",0,"","",
                "","N",self.doDate,dte,None,("efld",)),
            (("T",0,2,0),"IUI",1,"Number of Forms","",
                "","N",self.doForms,None,None,("notzero",)),
            (("C",0,0,0),"I@bft_skip",0,"","",
                "r","N",self.doSkip,skp,None,None),
            (("C",0,0,1),"ONA",20,""),
            (("C",0,0,2),"I@bft_player",0,"","",
                "","N",self.doPlayer,plr,None,None),
            (("C",0,0,3),"ONA",20,""),
            (("C",0,0,4),"I@bft_shotsf",0,"","",
                "","N",self.doShotsf,None,None,None),
            (("C",0,0,5),"I@bft_shotsa",0,"","",
                "","N",self.doShotsa,None,None,None),
            (("C",0,0,6),"I@bft_rating1",0,"","",
                "","N",self.doRate1,None,None,None),
            (("C",0,0,7),"I@bft_rating2",0,"","",
                "","N",self.doRate2,None,None,None),
            (("C",0,0,8),"I@bft_rating3",0,"","",
                "","N",self.doRate3,None,None,None),
            (("C",0,0,9),"I@bft_rating4",0,"","",
                "","N",self.doRate4,None,None,None),
            (("C",0,0,10),"I@bft_remarks",20,"","",
                "","N",self.doRemarks,None,None,None))
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        cnd = ((self.doEnd,"n"),)
        cxt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            tend=tnd, txit=txt, cend=cnd, cxit=cxt)

    def doFmat(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="bwlflf", where=[("bff_code", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Format"
        self.fmat = w
        self.df.loadEntry(frt, pag, p+1, data=acc[1])

    def doDate(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="bwlflm", where=[("bfm_fmat", "=",
            self.fmat), ("bfm_date", "=", w)], limit=1)
        if not acc:
            return "Invalid Date"
        self.date = w
        self.forms = acc[9]
        self.df.loadEntry(frt, pag, p+1, data=self.forms)

    def doForms(self, frt, pag, r, c, p, i, w):
        self.forms =w

    def doSkip(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables=["bwlflt", "bwlflp"], cols=["bfp_snam",
            "bfp_fnam", "bft_shotsf", "bft_shotsa", "bft_rating1",
            "bft_rating2", "bft_rating3", "bft_rating4", "bft_remarks"],
            where=[("bft_fmat", "=", self.fmat), ("bft_date", "=", self.date),
            ("bft_skip", "=", w), ("bfp_code=bft_skip",)], limit=1)
        if not acc:
            return "Invalid Skip"
        acc[0] = self.getName(acc[0], acc.pop(1))
        if w == self.skip:
            self.repeat = True
        else:
            self.repeat = False
        self.skip = w
        self.df.loadEntry(frt, pag, p+1, data=acc[0])
        if self.forms == 1:
            for x in range(1, 8):
                self.df.loadEntry(frt, pag, p+x+3, data=acc[x])
            return "sk3"

    def doPlayer(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables=["bwlflt", "bwlflp"], cols=["bfp_snam",
            "bfp_fnam", "bft_shotsf", "bft_shotsa", "bft_rating1",
            "bft_rating2", "bft_rating3", "bft_rating4", "bft_remarks"],
            where=[("bft_fmat", "=", self.fmat), ("bft_date", "=", self.date),
            ("bft_skip", "=", self.skip), ("bft_player", "=", w),
            ("bfp_code=bft_player",)], limit=1)
        if not acc:
            return "This Player Did Not Play for this Skip on this Date"
        acc[0] = self.getName(acc[0], acc.pop(1))
        self.player = w
        self.df.loadEntry(frt, pag, p+1, data=acc[0])
        if self.repeat:
            self.df.loadEntry(frt, pag, p+2, data=self.shotsf)
            self.df.loadEntry(frt, pag, p+3, data=self.shotsa)
            return "sk3"
        for x in range(1, 8):
            self.df.loadEntry(frt, pag, p+x+1, data=acc[x])

    def doShotsf(self, frt, pag, r, c, p, i, w):
        self.shotsf = w

    def doShotsa(self, frt, pag, r, c, p, i, w):
        self.shotsa = w

    def doRate1(self, frt, pag, r, c, p, i, w):
        self.rate1 = w

    def doRate2(self, frt, pag, r, c, p, i, w):
        self.rate2 = w

    def doRate3(self, frt, pag, r, c, p, i, w):
        self.rate3 = w

    def doRate4(self, frt, pag, r, c, p, i, w):
        self.rate4 = w

    def doRemarks(self, frt, pag, r, c, p, i, w):
        self.remarks = w

    def doEnd(self):
        if self.df.frt == "T":
            self.df.focusField("C", 0, 1)
            return
        col = ["bft_shotsf", "bft_shotsa"]
        dat = [self.shotsf, self.shotsa]
        self.sql.updRec("bwlflt", cols=col, data=dat, where=[("bft_fmat", "=",
            self.fmat), ("bft_date", "=", self.date), ("bft_skip", "=",
            self.skip)])
        col = ["bft_rating1", "bft_rating2", "bft_rating3", "bft_rating4",
            "bft_remarks"]
        dat = [self.rate1, self.rate2, self.rate3, self.rate4, self.remarks]
        if self.forms == 1:
            for x in range(1, 5):
                self.sql.updRec("bwlflt", cols=col, data=dat,
                    where=[("bft_fmat", "=", self.fmat), ("bft_date", "=",
                    self.date), ("bft_skip", "=", self.skip), ("bft_position",
                    "=", x)])
                dat[4] = ""
        else:
            self.sql.updRec("bwlflt", cols=col, data=dat, where=[("bft_fmat",
                "=", self.fmat), ("bft_date", "=", self.date), ("bft_skip",
                "=", self.skip), ("bft_player", "=", self.player)])
        self.df.advanceLine(0)

    def doExit(self):
        if self.df.frt == "C":
            self.mf.dbm.commitDbase(ask=True)
        self.df.closeProcess()
        self.mf.closeLoop()

    def getName(self, snam, fnam):
        for i in fnam.split():
            snam = "%s %s" % (snam, i[0].capitalize())
        return snam
