#! /usr/bin/python

"""
SYNOPSIS
    Bowls Draw Summary.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import RepPrt, TartanDialog, Sql

class bc3010:
    def __init__(self, mf, args=None):
        self.mf = mf
        self.args = args
        self.setVariables()
        if self.args:
            self.auto = True
            self.getCompDetails(self.args[0])
            self.doEnd()
        else:
            self.auto = False
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["bwlcmp", "bwlgme", "bwlskp", "bwltyp",
            "bwlent"], prog=__name__)

    def mainProcess(self):
        tit = ("Bowls Draw Summary",)
        cmp = {
            "stype": "R",
            "tables": ("bwlcmp",),
            "cols": (
                ("bcm_code", "", 0, "Cod"),
                ("bcm_name", "", 0, "Name", "Y"))}
        fld = (
            (("T",0,0,0),"I@bcm_code",0,"","",
                0,"N",self.doCmpCod,cmp,None,None),)
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            tend=tnd, txit=txt, view=("Y","V"), mail=("Y","N"))

    def doCmpCod(self, frt, pag, r, c, p, i, w):
        return self.getCompDetails(w)

    def getCompDetails(self, w):
        chk = self.sql.getRec(tables="bwlcmp", where=[("bcm_code", "=", w)],
            limit=1)
        if not chk:
            return "Invalid Competition Code"
        self.ccod = w
        self.cdes = chk[self.sql.bwlcmp_col.index("bcm_name")]
        typ = chk[self.sql.bwlcmp_col.index("bcm_type")]
        chk = self.sql.getRec(tables="bwltyp", where=[("bct_code", "=", typ)],
            limit=1)
        self.games = chk[self.sql.bwltyp_col.index("bct_games")]
        self.grgame = chk[self.sql.bwltyp_col.index("bct_grgame")]

    def doEnd(self):
        if not self.auto:
            self.df.closeProcess()
        groups = ["None", "A", "B", "C", "D", "E"]
        lst = self.sql.getRec(tables="bwlgme", cols=["bcg_game",
            "sum(bcg_points)"], where=[("bcg_ccod", "=", self.ccod)],
            group="bcg_game", order="bcg_game")
        lgame = 0
        for l in lst:
            if l[1]:
                lgame = l[0]
        col = ["bce_scod", "bcs_surname", "bcs_names"]
        if lgame <= self.grgame:
            col.extend(["sum(bcg_shots_for) as sfor",
                "sum(bcg_shots_agt) as sagt",
                "sum(bcg_shots_for - bcg_shots_agt) as agg",
                "sum(bcg_points) as pts"])
        else:
            col.extend(["sum(bcg_a_shots_for) as sfor",
                "sum(bcg_a_shots_agt) as sagt",
                "sum(bcg_a_shots_for - bcg_a_shots_agt) as agg",
                "sum(bcg_a_points) as pts"])
        skips = self.sql.getRec(tables=["bwlent", "bwlskp", "bwlgme"],
            cols=col, where=[("bce_ccod", "=", self.ccod),
            ("bcs_code=bce_scod",), ("bcg_ccod=bce_ccod",),
            ("bcg_scod=bce_scod",)], group="bcg_scod",
            order="bcg_group, pts desc, agg desc, sagt asc")
        data = []
        for skip in skips:
            nam = "%s, %s" % tuple(skip[1:3])
            dat = [skip[0], "%s" % nam.strip()]
            ops = []
            rks = []
            dup = ["-", "-"]
            games = self.sql.getRec(tables="bwlgme", where=[("bcg_ccod", "=",
                self.ccod), ("bcg_scod", "=", skip[0])],
                order="bcg_group, bcg_game")
            for game in games:
                gam = game[self.sql.bwlgme_col.index("bcg_game")]
                grp = game[self.sql.bwlgme_col.index("bcg_group")]
                opp = game[self.sql.bwlgme_col.index("bcg_ocod")]
                rnk = game[self.sql.bwlgme_col.index("bcg_rink")]
                dat.extend([opp, rnk])
                if opp:
                    if gam != self.games and opp in ops:
                        dup[0] = "X"
                    else:
                        dup[0] = "-"
                    ops.append(opp)
                if rnk:
                    if rnk in rks:
                        dup[1] = "X"
                    else:
                        dup[1] = "-"
                    rks.append(rnk)
            dat.insert(2, groups[grp])
            dat.append("%1s%1s" % tuple(dup))
            data.append(dat)
        head = ["", (self.cdes, 30), "Bowls Draw Summary Sheet"]
        cols = [
            ["a", "UI",  3, "Skp",  "y"],
            ["b", "NA", 29, "Name", "y"],
            ["c", "UA",  1, "G",    "y"]]
        for x in range(self.games):
            cols.extend([
                ["d%s" % x, "UI", 3, "Opp", "y"],
                ["e%s" % x, "UA", 2, "Rk",  "y"]])
        cols.append(["f", "UA",  2, "DR",  "y"])
        if self.auto:
            view = self.args[1]
            prtnam = self.args[2]
            mail = self.args[3]
        else:
            view = self.df.repprt[0][1]
            prtnam = self.df.repprt[0][2]
            mail = self.df.repprt[1][2]
        RepPrt(self.mf, name=__name__, tables=data, heads=head, cols=cols,
            ttype="D", pages=False, view=view, prtnam=prtnam, mail=mail)
        if not self.auto:
            self.mf.closeLoop()

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()
