#! /usr/bin/python

"""
SYNOPSIS
    Bowls League Assessment Forms Printing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, sys, time
from TartanClasses import Dbase, Sql, TartanDialog
from tartanFunctions import getModName, doDrawTable, doPrinter, doReportlab
from tartanFunctions import showError
from tartanWork import mthnam

class bc3040:
    def __init__(self, mf):
        self.mf = mf
        if self.setVariables():
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlmst", "bwlflf", "bwlflm", "bwlflo",
            "bwlflp", "bwlfls", "bwlflt"], prog=__name__)
        acc = self.sql.getRec(tables="ctlmst", cols=["ctm_name"],
            where=[("ctm_cono", "=", 1)], limit=1)
        if not acc:
            showError(self.mf.body, "Missing Record",
                "The Club Record is Missing, Please Cretae it and then Retry")
            return
        self.conam = acc[0]
        return True

    def mainProcess(self):
        tit = "League Assessment Forms"
        fmt = {
            "stype": "R",
            "tables": ("bwlflf",),
            "cols": (
                ("bff_code", "", 0, "Cod"),
                ("bff_desc", "", 0, "Description", "Y"))}
        dte = {
            "stype": "R",
            "tables": ("bwlflm",),
            "cols": (("bfm_date", "", 0, "Match-Date"),),
            "whera": [("T", "bfm_fmat", 0, 0)],
            "group": "bfm_date",
            "order": "bfm_date"}
        fld = (
            (("T",0,0,0),"I@bff_code",0,"","",
                "","N",self.doFmat,fmt,None,("efld",)),
            (("T",0,0,0),"ONA",30,""),
            (("T",0,1,0),"I@bfm_date",0,"","",
                "","N",self.doDate,dte,None,("efld",)),
            (("T",0,2,0),"IUI",1,"Number of Forms (1/4)","",
                1,"N",self.doForms,dte,None,("in",(1,4))))
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            tend=tnd, txit=txt, view=("N","V"), mail=("Y","N"))

    def doFmat(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="bwlflf", where=[("bff_code", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Format"
        self.fmat = w
        self.df.loadEntry(frt, pag, p+1, data=acc[1])

    def doDate(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="bwlflt", cols=["sum(bft_shotsf)"],
            where=[("bft_fmat", "=", self.fmat), ("bft_date", "=", w)],
            limit=1)
        if acc[0]:
            return "Match Completed"
        self.date = w
        self.disp = "%i %s %i" % (w % 100, mthnam[(w/100)%100][1], w/10000)

    def doForms(self, frt, pag, r, c, p, i, w):
        self.forms = w

    def doEnd(self):
        self.df.closeProcess()
        # Update match records with number of forms
        self.sql.updRec("bwlflm", cols=["bfm_forms"], data=[self.forms],
            where=[("bfm_fmat", "=", self.fmat), ("bfm_date", "=", self.date)])
        self.mf.dbm.commitDbase()
        # Get data
        cards = []
        whr = [("bft_fmat", "=", self.fmat), ("bft_date", "=", self.date)]
        if self.forms == 4:
            maxf = 8
        else:
            maxf = 3
            whr.append(("bft_position", "=", 1))
        whr.extend([
            ("bfs_league", "=", "M"),
            ("bfs_fmat=bft_fmat",),
            ("bfs_code=bft_team",)])
        all = self.sql.getRec(tables=["bwlfls", "bwlflt"], cols=["bft_team",
            "bft_skip", "bft_player"], where=whr,
            order="bft_team, bft_skip, bft_position asc")
        for a in all:
            match = self.sql.getRec(tables="bwlflm", where=[("bfm_fmat", "=",
                self.fmat), ("bfm_date", "=", self.date), ("bfm_team", "=",
                a[0])], limit=1)
            team = self.sql.getRec(tables="bwlfls", where=[("bfs_fmat", "=",
                self.fmat), ("bfs_code", "=", a[0])], limit=1)
            opp = self.sql.getRec(tables=["bwlflo", "bwlclb"],
                cols=["bfo_desc", "bcc_name"], where=[("bfo_fmat", "=",
                self.fmat), ("bfo_code", "=", match[4]),
                ("bcc_code=bfo_club",)], limit=1)
            skp = self.sql.getRec(tables="bwlflp", where=[("bfp_code", "=",
                a[1])], limit=1)
            if self.forms == 4:
                plr = self.sql.getRec(tables="bwlflp", cols=["bfp_snam",
                    "bfp_fnam"], where=[("bfp_code", "=", a[2])], limit=1)
            else:
                plr = self.sql.getRec(tables=["bwlflt", "bwlflp"],
                    cols=["bfp_snam", "bfp_fnam"], where=[("bft_fmat", "=",
                    self.fmat), ("bft_date", "=", self.date), ("bft_skip", "=",
                    a[1]), ("bfp_code=bft_player",)], order="bft_position")
            if match[5] == "H":
                ven = self.conam
            else:
                ven = "%s" % opp[1]
            if self.forms == 4:
                cards.append([team[2], opp[0], ven, skp[1],
                    self.getName(plr[0], plr[1])])
            else:
                card = [team[2], opp[0], ven, "%s, %s" % (skp[2], skp[3])]
                for p in plr:
                    card.append(self.getName(p[0], p[1]))
                cards.append(card)
        while len(cards) % maxf:
            if self.forms == 4:
                cards.append(["", "", "", "", ""])
            else:
                cards.append(["", "", "", "", "", "", "", ""])
        # Print
        pdfnam = getModName(self.mf.wrkdir, __name__, "assess_%s" % self.date,
            ext="pdf")
        c, cpi, inch, ss, dd = doReportlab(self.mf.prgdir, pdfnam, orient="A4")
        cw = c.stringWidth(("X"), "Tartan", cpi)    # character width
        ld = inch / 6                               # line depth
        if self.forms == 4:
            rr = {
                "margins": ((2.5, 47.5), (1.5, 15.5)),
                "repeat": (2, 4),
                "rows": (
                    (2.5, 1.5, 47.5, 14.5),
                    (2.5, 8.5, (
                        (8, 1.5, .8, "Title"),
                        (12, 1.5, .8, "Out of 10"),
                        (27.5, 1.5, .8, "Remarks"))),
                    (2.5, 10, (
                        (8, 1.5, .9, ("Skip", "Third", "Second", "Lead")),
                        (12, 1.5)), 4))}
        else:
            rr = {
                "margins": ((2.5, 47.5), (1.5, 20.5)),
                "repeat": (1, 3),
                "rows": (
                    (2.5, 1.5, 95.1, 19.5),
                    (2.5, 7.5, (
                        (15, 2, .8, "Shots For"), (8, 2),
                        (15, 2, .8, "Shots Against"), (8, 2),
                        (15, 2, .8, "Difference"), (8, 2),
                        (15, 2, .8, "Result W/L"), (11.1, 2))),
                    (2.5, 11, (
                        (10, 2, .8, "Position"),
                        (20, 2, .8, "Name"),
                        (12, 2, .8, "Rating 1-10"),
                        ( 7, 2, .8, "Initial"),
                        (46.1, 2, .8, "Remarks"))),
                    (2.5, 13, (
                        (10, 2, .9, ("Skip", "Third", "Second", "Lead")),
                        (20, 2), (12, 2), (7, 2)), 4))}
        if self.forms == 4:
            ppad = 1
            h1 = "WPCC Bowls - Assessment Form"
            hs = "Shots For:        Against:        Diff:        Result:"
        else:
            ppad = 1.5
            h1 = "WPCC Bowls - Assessment Form for %s" % self.disp
        for x in range(0, len(cards), maxf):
            last, table = doDrawTable(c, cw, ld, rr, ppad=ppad, spad=2)
            if self.forms == 4:
                c.setFont("Helvetica-Bold", 11)
                c.drawCentredString(26*cw,  3*ld, h1)
                c.drawCentredString(75*cw,  3*ld, h1)
                c.drawCentredString(26*cw, 19*ld, h1)
                c.drawCentredString(75*cw, 19*ld, h1)
                c.drawCentredString(26*cw, 35*ld, h1)
                c.drawCentredString(75*cw, 35*ld, h1)
                c.drawCentredString(26*cw, 51*ld, h1)
                c.drawCentredString(75*cw, 51*ld, h1)
                c.setFont("Helvetica-Bold", 10)
                c.drawString(03*cw,    7.5*ld, hs)
                c.drawString(51.5*cw,  7.5*ld, hs)
                c.drawString(03*cw,   23.5*ld, hs)
                c.drawString(51.5*cw, 23.5*ld, hs)
                c.drawString(03*cw,   39.5*ld, hs)
                c.drawString(51.5*cw, 39.5*ld, hs)
                c.drawString(03*cw,   55.5*ld, hs)
                c.drawString(51.5*cw, 55.5*ld, hs)
            else:
                c.setFont("Helvetica-Bold", 18)
                c.drawCentredString(50*cw,  3*ld, h1)
                c.drawCentredString(50*cw, 24*ld, h1)
                c.drawCentredString(50*cw, 45*ld, h1)
            for y in range(maxf):
                card = cards[x+y]
                if self.forms == 4:
                    p = y%2 * 48.5
                    r = y/2 * 16
                    c.setFont("Helvetica-Bold", 10)
                    h2 = "%s vs %s at %s on %s" % (card[0], card[1], card[2],
                        self.disp)
                    c.drawCentredString((26+p)*cw,  (4.5+r)*ld, h2)
                    h3 = "Skip: %s    Player: %s" % (card[3], card[4])
                    c.drawString((3+p)*cw, (6+r)*ld, h3)
                else:
                    r = y * 21
                    c.setFont("Helvetica-Bold", 16)
                    h2 = "%s vs %s at %s" % (card[0], card[1], card[2])
                    c.drawCentredString(50*cw,  (6+r)*ld, h2)
                    c.setFont("Helvetica", 12)
                    c.drawString(13.5*cw, (14.4+r)*ld, card[4])
                    c.drawString(13.5*cw, (16.4+r)*ld, card[5])
                    c.drawString(13.5*cw, (18.4+r)*ld, card[6])
                    c.drawString(13.5*cw, (20.4+r)*ld, card[7])
            c.showPage()
        c.save()
        head = "Match Assessment Forms for %s" % self.disp
        doPrinter(mf=self.mf, head=head, rep=pdfnam, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
        self.mf.closeLoop()

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

    def getName(self, snam, fnam):
        for i in fnam.split():
            snam = "%s %s" % (snam, i[0].capitalize())
        return snam
