#! /usr/bin/python

"""
SYNOPSIS
    Bowls League Match Assessment Report.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import TartanDialog, RepPrt, Sql
from tartanFunctions import dateDiff, textFormat

class bc3060:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["bwlflf", "bwlflm", "bwlflt", "bwlflp"],
            prog=__name__)
        self.pos = ["", "Skip", " 3rd", " 2nd", "Lead"]

    def mainProcess(self):
        tit = "Assessment Summary Report"
        fmt = {
            "stype": "R",
            "tables": ("bwlflf",),
            "cols": (
                ("bff_code", "", 0, "Cod"),
                ("bff_desc", "", 0, "Description", "Y"))}
        dte = {
            "stype": "R",
            "tables": ("bwlflm",),
            "cols": (("bfm_date", "", 0, "Match-Date"),),
            "whera": [("T", "bfm_fmat", 0, 0)],
            "group": "bfm_date"}
        fld = (
            (("T",0,0,0),"I@bff_code",0,"","",
                "","N",self.doFmat,fmt,None,("efld",)),
            (("T",0,0,0),"ONA",30,""),
            (("T",0,1,0),"I@bfm_date",0,"","",
                "","N",self.doDate,dte,None,("efld",)))
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            tend=tnd, txit=txt, view=("Y","V"), mail=("Y","N"))

    def doFmat(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="bwlflf", where=[("bff_code", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Format"
        self.fmat = w
        self.fnam = acc[1]
        self.df.loadEntry(frt, pag, p+1, data=acc[1])

    def doDate(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="bwlflm", cols=["bfm_forms"],
            where=[("bfm_fmat", "=", self.fmat), ("bfm_date", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Match Date"
        self.forms = acc[0]
        self.date = w
        self.disp = self.df.t_disp[0][0][2]

    def doEnd(self):
        self.df.closeProcess()
        data = []
        all = self.sql.getRec(tables=["bwlfls", "bwlflt"], cols=["bft_skip",
            "bft_player", "bft_position", "bft_shotsf", "bft_shotsa",
            "bft_rating1", "bft_rating2", "bft_rating3", "bft_rating4",
            "bft_remarks"], where=[("bft_fmat", "=", self.fmat), ("bft_date",
            "=", self.date), ("bfs_fmat=bft_fmat",), ("bfs_league", "=", "M"),
            ("bfs_code=bft_team",)], order="bfs_code, bft_skip, bft_position")
        skip = 0
        for player in all:
            pos = player[2]
            his = player[4 + pos]
            if pos == 1:
                names = self.sql.getRec(tables=["bwlflt", "bwlflp"],
                    cols=["bfp_snam", "bfp_fnam"], where=[("bft_fmat", "=",
                    self.fmat), ("bft_date", "=", self.date), ("bft_skip", "=",
                    player[0]), ("bfp_code=bft_player",)], order="bft_position")
                for x in range(len(names)):
                    names[x][0] = self.getName(names[x][0], names[x].pop(1))
            sc = self.sql.getRec(tables="bwlflt",
                cols=["bft_rating%s" % pos], where=[("bft_fmat", "=",
                self.fmat), ('bft_date', '=', self.date), ("bft_skip", "=",
                player[0])])
            tot = sc[0][0] + sc[1][0] + sc[2][0] + sc[3][0]
            tav = self.getHistory(player[1])
            if not tav:
                tav = tot
            if self.forms == 1:
                if player[1] == player[0]:
                    desc = textFormat(player[9], width=70)
                if desc:
                    text = desc.pop(0)
                else:
                    text = ""
                data.append([names[player[2]-1][0], self.pos[pos], player[3],
                    player[4], sc[0][0], tav, text])
            else:
                data.append([names[player[2]-1][0], self.pos[pos], player[3],
                    player[4], sc[0][0], sc[1][0], sc[2][0], sc[3][0],
                    round(tot/4.0, 1), his, round((tot-his)/3.0, 1), tav,
                    player[9]])
            if pos == 4:
                data.append(["BLANK"])
        head = ("Bowls Match Assessment for %s Match Played on %s" % \
            (self.fnam, self.disp),)
        cols = [
            ["a", "NA",  20, "Player",  "y"],
            ["b", "NA",   4, "Posn",    "y"],
            ["c", "UI",   3, "For",     "y"],
            ["d", "UI",   3, "Agt",     "y"]]
        if self.forms == 1:
            cols.append(["e", "UD", 4.1, " Ass",    "y"])
        elif self.forms == 4:
            cols.extend([
                ["e", "UD", 4.1, " Skp",    "y"],
                ["f", "UD", 4.1, " 3rd",    "y"],
                ["g", "UD", 4.1, " 2nd",    "y"],
                ["h", "UD", 4.1, "Lead",    "y"],
                ["i", "UD", 4.1, " Avg",    "y"],
                ["j", "UD", 4.1, " Own",    "y"],
                ["k", "UD", 4.1, " Adj",    "y"]])
        cols.extend([
            ["l", "UD", 4.1, " ATD",    "y"],
            ["m", "NA",  70, "Remarks", "y"]])
        RepPrt(self.mf, name=__name__, tables=data, heads=head, cols=cols,
            ttype="D", view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
        self.mf.closeLoop()

    def getHistory(self, player):
        sdate = 0
        acc = self.sql.getRec(tables="bwlflm", cols=["bfm_date"],
            where=[("bfm_fmat", "=", self.fmat), ("bfm_round", "=", 1)],
            order="bfm_date")
        for a in acc:
            if dateDiff(a[0], self.date, ptype="months") < 4:
                sdate = a[0]
                break
        if not sdate:
            print "ERROR"
        rounds = self.sql.getRec(tables="bwlflm", cols=["bfm_round",
            "bfm_date"], where=[("bfm_fmat", "=", self.fmat), ("bfm_date",
            ">=", sdate, "and", "bfm_date", "<=", self.date)],
            group="bfm_round", order="bfm_round")
        cnt = 0
        his = 0
        tot = 0
        for rnd in rounds:
            pl = self.sql.getRec(tables=["bwlfls", "bwlflt"],
                cols=["bft_position", "bft_rating1", "bft_rating2",
                "bft_rating3", "bft_rating4", "bft_skip"],
                where=[("bfs_fmat", "=", self.fmat), ("bfs_league", "=", "M"),
                ("bft_date", "=", rnd[1]), ("bft_player", "=", player),
                ("bft_fmat=bfs_fmat",), ("bft_team=bfs_code",)], limit=1)
            if not pl:
                # Did not play
                continue
            if not pl[1] and not pl[2] and not pl[3] and not pl[4]:
                # Not yet assessed
                continue
            cnt += 1.0
            his += pl[pl[0]]
            team = self.sql.getRec(tables="bwlflt", where=[("bft_fmat", "=",
                self.fmat), ("bft_date", "=", rnd[1]), ("bft_skip", "=",
                pl[5])])
            tmp = 0
            for tm in team:
                tot += tm[pl[0]+7]
        if tot:
            return round((tot - his) / 3.0 / cnt, 1)

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

    def getName(self, snam, fnam):
        for i in fnam.split():
            snam = "%s %s" % (snam, i[0].capitalize())
        return snam
