#! /usr/bin/python2

"""
SYNOPSIS
    Bulk Mailing Utility.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import imghdr, os, sys, time, urllib
from xml.dom.minidom import parseString
try:
    from markdown import markdown
    MARKDOWN = True
except:
    MARKDOWN = False
try:
    from PIL import Image as PILImage
    PIL = True
except:
    PIL = False
try:
    import xlrd
    XLRD = True
except:
    XLRD = False
from TartanClasses import CsvCvt, GetCtl, ProgressBar, ShowEmail
from TartanClasses import SplashScreen, Sql, TartanDialog
from tartanFunctions import askQuestion, sendMail, showError

class bm1010:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        gc = GetCtl(self.mf)
        # Get System Details
        ctlsys = gc.getCtl("ctlsys")
        if not ctlsys:
            return "no"
        self.smtp = ctlsys["sys_smtp"]
        self.sms = ctlsys["sys_sms"]
        if not self.smtp and not self.sms:
            showError(self.mf.body, "SMTP/SMS Error",
                "There is NO SMTP Server nor SMS Service in the System Record")
            return "no"
        # Get Company Details
        ctlmst = gc.getCtl("ctlmst", self.conum)
        if not ctlmst:
            return "no"
        self.fadd = ctlmst["ctm_email"]
        if self.smtp and not self.fadd:
            showError(self.mf.body, "From Error",
                "There is NO Email Address on the Company Record!")
            return "no"
        tab = ["emllog"]
        mod = []
        for x in range(0, 21, 2):
            mod.append(ctlmst["ctm_modules"][x:x+2])
        if mod.count("CR"):
            tab.append("crsmst")
        if mod.count("DR"):
            tab.extend(["drsact", "drstyp", "drsmst"])
        if mod.count("ML"):
            tab.extend(["memctc", "memmst", "memcat", "memkon"])
        self.sql = Sql(self.mf.dbm, tables=tab, prog=__name__)
        if mod.count("CR"):
            self.crs = self.sql.getRec(tables="crsmst", cols=["count(*)"],
                where=[("crm_cono", "=", self.conum)], limit=1)
        else:
            self.crs = [0]
        if mod.count("DR"):
            self.drs = self.sql.getRec(tables="drsmst", cols=["count(*)"],
                where=[("drm_cono", "=", self.conum)], limit=1)
        else:
            self.drs = [0]
        if mod.count("ML"):
            self.mem = self.sql.getRec(tables="memmst", cols=["count(*)"],
                where=[("mlm_cono", "=", self.conum)], limit=1)
        else:
            self.mem = [0]
        self.mess = ""
        self.embed = []
        self.attach = []

    def mainProcess(self):
        if not self.smtp:
            self.dtyp = "S"
            tit = ("%03i %s" % (self.conum, self.conam),
                "Bulk SMS Utility")
            r1s = (("SMS", "S"),)
        elif not self.sms:
            self.dtyp = "E"
            tit = ("%03i %s" % (self.conum, self.conam),
                "Bulk Mail Utility")
            r1s = (("E-Mail", "E"),)
        else:
            self.dtyp = ""
            tit = ("%03i %s" % (self.conum, self.conam),
                "Bulk Mail/SMS Utility")
            r1s = (("E-Mail", "E"), ("SMS", "S"))
        self.fle = {
            "stype": "F",
            "types": "fle",
            "multi": True}
        self.r2s = []
        if self.drs[0]:
            self.r2s.append(("Debtors", "D"))
        if self.crs[0]:
            self.r2s.append(("Creditors", "C"))
        if self.mem[0]:
            self.r2s.append(("Members", "M"))
        self.r2s.append(("CSV File", "V"))
        if XLRD:
            self.r2s.append(("XLS File", "X"))
        r3s = (("Yes", "Y"), ("No", "N"))
        if self.dtyp:
            fld = [(("T",0,0,0),("ORB",r1s),0,"Delivery Type")]
        else:
            fld = [(("T",0,0,0),("IRB",r1s),0,"Delivery Type","",
                "E","Y",self.doType,None,None,None)]
        fld.extend([
            (("T",0,1,0),("IRB",self.r2s),0,"List to Use","",
                self.r2s[0][1],"Y",self.doList,None,None,None),
            (("T",0,2,0),("IRB",r3s),0,"Skip Delivery Errors","",
                "Y","N",self.doSkip,None,None,None),
            (("T",0,3,0),"INA",(75,100),"Subject","",
                "","N",self.doSubject,None,None,("notblank",)),
            (("T",0,4,0),"IFF",75,"In-line Attachment","",
                "","N",self.doEmbed,self.fle,None,("fle","blank")),
            (("T",0,5,0),"IFF",75,"Separate Attachment","",
                "","N",self.doAttach,self.fle,None,("fle","blank")),
            (("T",0,6,0),"ITV",(75,25),"Message","",
                "","N",self.doMessage,None,None,None,None,
                """{{name}} will be substituted by the contact person's name e.g.
 
Dear {{name}} would become
 
Dear John Smith ...""")])
        tnd = ((self.doEnd, "y"), )
        txt = (self.doExit, )
        but = (
            ("Preview",None,self.doPreview,0,("T",0,7),
                (("T",0,0),("T",0,2),("T",0,6))),
            ("Send",None,self.doExecute,0,("T",0,7),
                (("T",0,0),("T",0,2),("T",0,6))),
            ("Quit",None,self.doExit,1,None,None))
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, butt=but)
        if self.dtyp:
            self.df.loadEntry("T", 0, 0, data=self.dtyp)

    def doType(self, frt, pag, r, c, p, i, w):
        self.dtyp = w
        idx = 0
        if self.dtyp == "S":
            if self.drs[0]:
                self.df.topEntry[0][1][idx][0].hide()
                idx += 1
            if self.crs[0]:
                self.df.topEntry[0][1][idx][0].hide()
            if self.mem[0]:
                self.df.topf[0][1][5] = "M"
            else:
                self.df.topf[0][1][5] = "V"
        elif self.sms:
            if self.drs[0]:
                self.df.topEntry[0][1][idx][0].show()
                idx += 1
            if self.crs[0]:
                self.df.topEntry[0][1][idx][0].show()
            self.df.topf[0][1][5] = self.r2s[0][1]

    def doList(self, frt, pag, r, c, p, i, w):
        self.ulist = w
        self.doFilter()
        if self.fltexit:
            return "rf"
        if self.dtyp == "S":
            return "sk4"

    def doFilter(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Bulk Mail Filter")
        if self.ulist == "M":
            cod = {
                "stype": "R",
                "tables": ("memctc",),
                "cols": (
                    ("mcc_code", "", 0, "Cd"),
                    ("mcc_desc", "", 0, "Description", "Y")),
                "where": [("mcc_cono", "=", self.conum)],
                "whera": [["T", "mcc_type", 0]],
                "order": "mcc_code",
                "size": (400, 600)}
            r1s = (("All", "A"),("Main","B"),("Sports","C"),("Debentures","D"))
            fld = (
                (("T",0,0,0),("IRB",r1s),0,"Category","Category",
                    "A","Y",self.doFltCat,None,None,None),
                (("T",0,1,0),"IUI",2,"Code","Code",
                    0,"Y",self.doFltCod,cod,None,None),
                (("T",0,1,0),"ONA",30,""))
        elif self.ulist in ("C", "D"):
            r1s = (("Manager","M"),("Accounts","A"),("Sales","S"))
            fld = [
                (("T",0,1,0),("IRB",r1s),0,"Email To","Email To",
                    "S","N",self.doFltTo,None,None,None)]
            if self.ulist == "D":
                daa = self.sql.getRec(tables="drsmst", cols=["drm_bus_activity",
                    "count(*)"], where=[("drm_cono", "=", self.conum)],
                    group="drm_bus_activity", order="drm_bus_activity")
                act = {
                    "stype": "C",
                    "titl": "Valid Activities",
                    "head": ("Act", "Quantity"),
                    "data": daa,
                    "typs": (("NA", 3), ("UI", 8)),
                    "size": (400, 400)}
                dab = self.sql.getRec(tables="drsmst", cols=["drm_bus_type",
                    "count(*)"], where=[("drm_cono", "=", self.conum)],
                    group="drm_bus_type", order="drm_bus_type")
                typ = {
                    "stype": "C",
                    "titl": "Valid Types",
                    "head": ("Typ", "Quantity"),
                    "data": dab,
                    "typs": (("NA", 3), ("UI", 8)),
                    "size": (400, 400)}
                fld.append((("T",0,2,0),"INA",3,"Activity","Account Activity",
                    "","N",self.doFltAct,act,None,None))
                fld.append((("T",0,2,0),"ONA",30,""))
                fld.append((("T",0,3,0),"INA",3,"Type","Account Type",
                    "","N",self.doFltTyp,typ,None,None))
                fld.append((("T",0,3,0),"ONA",30,""))
        else:
            fle = {
                "stype": "F",
                "types": "fle"}
            if sys.platform == "win32":
                self.lastdir = os.path.join(self.mf.rcdic["wrkdir"], "impdir")
            else:
                self.lastdir = os.path.join(self.mf.rcdic["wrkdir"], ".impdir")
            if os.path.exists(self.lastdir):
                infle = open(self.lastdir, "r")
                fle["initd"] = infle.readline().replace("\n", "")
                infle.close()
            else:
                fle["initd"] = self.mf.rcdic["wrkdir"]
            if self.ulist == "V":
                fle["ftype"] = (("CSV Files", "*.csv"),)
            else:
                fle["ftype"] = (("XLS Files", "*.xls"),)
            r1s = (("Yes", "Y"), ("No", "N"))
            fld = [
                (("T",0,0,0),"IFF",50,"File Name","",
                    "","N",self.doFltImp,fle,None,("file",)),
                (("T",0,1,0),("IRB",r1s),0,"Ignore Invalid Lines","",
                    "Y","N",self.doFltIgn,None,None,None),
                (("T",0,2,0),"IUI",3,"Recipient's Name Column","",
                    0,"N",self.doFltNC,None,None,("efld",))]
            if self.dtyp == "S":
                fld.append(
                    (("T",0,3,0),"IUI",3,"Recipient's Mobile Column","",
                        1,"N",self.doFltAC,None,None,("efld",)))
            else:
                fld.append(
                    (("T",0,3,0),"IUI",3,"Recipient's Email Address Column","",
                        1,"N",self.doFltAC,None,None,("efld",)))
        state = self.df.disableButtons()
        self.ff = TartanDialog(self.mf, title=tit, newscn=(1,1), tops=True,
            eflds=fld, tend=((self.doFltEnd,"n"),), txit=(self.doFltExit,))
        self.mf.startLoop()
        self.df.enableButtons(state=state)

    def doFltCat(self, frt, pag, r, c, p, i, w):
        if w == "A":
            self.fltcat = ""
            self.fltcod = ""
            self.ff.loadEntry(frt, pag, p+2, "All Categories and Codes")
            return "sk2"
        self.fltcat = w
        chk = self.sql.getRec(tables="memctc", where=[("mcc_cono", "=",
            self.conum), ("mcc_type", "=", self.fltcat)], limit=1)
        if not chk:
            return "Invalid Membership Category"

    def doFltCod(self, frt, pag, r, c, p, i, w):
        self.fltcod = w
        if self.fltcod:
            chk = self.sql.getRec(tables="memctc", where=[("mcc_cono", "=",
                self.conum), ("mcc_type", "=", self.fltcat), ("mcc_code", "=",
                self.fltcod)], limit=1)
            if not chk:
                return "Invalid Category Code"
            self.ff.loadEntry(frt, pag, p+1, chk[3])
        else:
            self.ff.loadEntry(frt, pag, p+1, "All Codes")

    def doFltTo(self, frt, pag, r, c, p, i, w):
        self.fltto = w
        if self.ulist == "C":
            self.flttyp = ""

    def doFltAct(self, frt, pag, r, c, p, i, w):
        self.fltact = w
        if not self.fltact:
            self.ff.loadEntry(frt, pag, p+1, "All Activities")
        else:
            chk = self.sql.getRec(tables="drsact", where=[("dac_code", "=",
                self.fltact)], limit=1)
            if not chk:
                return "Invalid Activity Code"
            self.ff.loadEntry(frt, pag, p+1, chk[1])

    def doFltTyp(self, frt, pag, r, c, p, i, w):
        self.flttyp = w
        if not self.flttyp:
            self.ff.loadEntry(frt, pag, p+1, "All Types")
        else:
            chk = self.sql.getRec(tables="drstyp", where=[("dtp_code", "=",
                self.flttyp)], limit=1)
            if not chk:
                return "Invalid Type Code"
            self.ff.loadEntry(frt, pag, p+1, chk[1])

    def doFltImp(self, frt, pag, r, c, p, i, w):
        self.impfle = w
        try:
            infle = open(self.lastdir, "w")
            infle.write(os.path.dirname(os.path.normpath(self.impfle)))
            infle.close()
        except:
            pass

    def doFltIgn(self, frt, pag, r, c, p, i, w):
        self.impign = w
        for x in range(2, len(self.ff.t_work[0][0])):
            self.ff.loadEntry("T", 0, p+x-1, data=x-2)

    def doFltNC(self, frt, pag, r, c, p, i, w):
        self.namcol = w

    def doFltAC(self, frt, pag, r, c, p, i, w):
        self.addcol = w

    def doFltEnd(self, widget=None):
        self.doFltClose()
        self.fltexit = False
        if self.ulist == "V":
            ifl = open(self.impfle, "r")
            data = ifl.readlines()
            self.impdat = []
            for num, line in enumerate(data):
                try:
                    cvt = CsvCvt(line).output
                    nam = cvt[self.namcol]
                    eml = cvt[self.addcol]
                    if self.dtyp == "E" and self.df.doCheckEmail(eml):
                        if self.impign == "Y":
                            continue
                        showError(self.mf.body, "Address Error",
                            "This '%s' is Not a Valid Email Address" % eml)
                        self.fltexit = True
                        break
                    self.impdat.append([eml, nam])
                except:
                    pass
        elif self.ulist == "X":
            workbk = xlrd.open_workbook(self.impfle)
            worksh = workbk.sheet_by_index(0)
            self.impdat = []
            for row in range(0, worksh.nrows):
                try:
                    nam = worksh.cell_value(rowx=row, colx=self.namcol)
                    eml = worksh.cell_value(rowx=row, colx=self.addcol)
                    if self.dtyp == "E" and self.df.doCheckEmail(eml):
                        if self.impign == "Y":
                            continue
                        showError(self.mf.body, "Address Error",
                            "This '%s' is Not a Valid Email Address" % eml)
                        self.fltexit = True
                        break
                    self.impdat.append([eml, nam])
                except:
                    pass

    def doFltExit(self, widget=None):
        self.fltexit = True
        self.doFltClose()

    def doFltClose(self):
        self.ff.closeProcess()
        self.mf.closeLoop()

    def doSkip(self, frt, pag, r, c, p, i, w):
        self.skip = w

    def doSubject(self, frt, pag, r, c, p, i, w):
        self.subj = w
        self.fle["ftype"] = (("All", "*"),)

    def doEmbed(self, frt, pag, r, c, p, i, w):
        if w:
            w = w.split(",")
            for f in w:
                if not imghdr.what(f):
                    showError(self.mf.body, "Invalid File Type",
                """Valid File Types are as Follows:

rgb   SGI ImgLib Files
gif   GIF 87a and 89a Files
pbm   Portable Bitmap Files
pgm   Portable Graymap Files
ppm   Portable Pixmap Files
tiff  TIFF Files
rast  Sun Raster Files
xbm   X Bitmap Files
jpeg  JPEG data in JFIF or Exif formats
bmp   BMP files
png   Portable Network Graphics""")
                    return "Invalid File Type"
            self.embed = w
        else:
            self.embed = []
        if self.embed and PIL:
            for n, i in enumerate(self.embed):
                try:
                    img = PILImage.open(i)
                    siz = img.size
                    if siz[0] > 800:
                        new = img.resize((800, siz[1]*800/siz[0]))
                        i = os.path.join(self.mf.wrkdir, os.path.basename(i))
                        new.save(i)
                        self.embed[n] = i
                except:
                    pass
        self.fle["ftype"] = (("Attachment", "*"),)

    def doAttach(self, frt, pag, r, c, p, i, w):
        if w:
            self.attach = w.split(",")
        else:
            self.attach = []

    def doMessage(self, frt, pag, r, c, p, i, w):
        if not w and not self.embed and not self.attach:
            return "Invalid Message"
        self.mess = w
        if MARKDOWN:
            self.html = markdown(self.mess)
        else:
            self.html = mess

    def doPreview(self, widget=None):
        buf = self.df.topEntry[0][6].get_buffer()
        mess = buf.get_text(buf.get_start_iter(), buf.get_end_iter())
        if not self.embed and not self.attach and not mess:
            return
        mess = mess.replace("{{name}}", "Tom Jones")
        state = self.df.disableButtons()
        ShowEmail(self.df.window, mess, self.embed, self.attach)
        self.df.enableButtons(state=state)

    def doExecute(self, widget=None):
        buf = self.df.topEntry[0][6].get_buffer()
        self.mess = buf.get_text(buf.get_start_iter(), buf.get_end_iter())
        if not self.embed and not self.attach and not self.mess:
            return
        if MARKDOWN:
            self.html = markdown(self.mess)
        else:
            self.html = self.mess
        self.doEnd()

    def doEnd(self):
        self.df.closeProcess()
        if self.dtyp == "S":
            self.doSendSMS()
        else:
            self.doSendEmail()
        self.closeProcess()

    def doSendSMS(self):
        self.url = "http://www.mymobileapi.com/api5/http5.aspx?"
        user = self.sms.split("|")
        self.par = {"username": user[0], "password": user[1]}
        xml = self.doExecuteSMS("credits")
        if not xml:
            print "XML", xml
            return
        dom = parseString(xml)
        tag = dom.getElementsByTagName("credits")[0].toxml()
        units = int(tag.replace("<credits>","").replace("</credits>",""))
        if self.ulist == "M":
            tab = ["memmst", "memkon", "memctk"]
            col = ["mlm_surname", "mlm_names", "mlk_detail"]
            whr = [
                ("mlm_cono", "=", self.conum),
                ("mlm_state", "=", "A"),
                ("mlk_cono=mlm_cono",),
                ("mlk_memno=mlm_memno",),
                ("mck_code=mlk_code",),
                ("mck_type", "=", "M")]
            if self.fltcat:
                tab.append("memcat")
                whr.append(("mlc_cono=mlm_cono",))
                whr.append(("mlc_memno=mlm_memno",))
                whr.append(("mlc_type", "=", self.fltcat))
                if self.fltcod:
                    whr.append(("mlc_code", "=", self.fltcod))
            odr = "mlm_surname, mlm_names"
            get = self.sql.getRec(tables=tab, cols=col, where=whr, order=odr)
            accs = []
            for g in get:
                accs.append((g[2], "%s %s" % (g[1].split()[0], g[0])))
        elif self.ulist in ("V", "X"):
            accs = self.impdat
        if units < len(accs):
            showError(self.mf.body, "Out of Units",
                "There Are Insufficient Units, %s, in the Account" % int(dat))
            return
        for acc in accs:
            cell = acc[0].strip().replace(" ", "")
            cell = cell.replace("(","").replace(")","")
            cell = cell.replace("{","").replace("}","")
            cell = cell.split(",")
            cont = acc[1].strip()
            if not cont:
                name = "Sir/Madam"
            else:
                name = self.doNames(cont)
            mess = self.mess.replace("{{name}}", name)
            for num in cell:
                if len(num) in (10, 12):
                    xml = self.doExecuteSMS("sendparam", num, mess)

    def doNames(self, name):
        n = name.split()
        new = ""
        while n:
            new = new + n.pop(0).capitalize() + " "
        idx = new.find("-")
        if idx != -1:
            new = new.replace("-%s" % new[idx+1], "-%s" % new[idx+1].upper())
        return new.strip()

    def doExecuteSMS(self, stype, num=None, dat=None):
        par = self.par.copy()
        par["type"] = stype
        if not dat:
            params = urllib.urlencode(par)
        else:
            par["numto"] = num
            par["data1"] = dat
            params = urllib.urlencode(par)
        try:
            f = urllib.urlopen(self.url, params)
            s = f.read()
            f.close()
        except:
            s = None
        return s

    def doSendEmail(self):
        if self.ulist == "D":
            whr = [("drm_cono", "=", self.conum)]
            if self.fltto == "M":
                col = ["drm_mgr_email", "drm_mgr"]
                whr.append(("drm_mgr_email", "<>", ""))
            elif self.fltto == "A":
                col = ["drm_acc_email", "drm_acc"]
                whr.append(("drm_acc_email", "<>", ""))
            else:
                col = ["drm_sls_email", "drm_sls"]
                whr.append(("drm_sls_email", "<>", ""))
            if self.fltact:
                whr.append(("drm_bus_activity", "=", self.fltact))
            if self.flttyp:
                whr.append(("drm_bus_type", "=", self.flttyp))
            accs = self.sql.getRec(tables="drsmst", cols=col, where=whr)
        elif self.ulist == "C":
            whr = [("crm_cono", "=", self.conum)]
            if self.fltto == "M":
                col = ["crm_mgr_email", "crm_mgr"]
                whr.append(("crm_mgr_email", "<>", ""))
            elif self.fltto == "A":
                col = ["crm_acc_email", "crm_acc"]
                whr.append(("crm_acc_email", "<>", ""))
            else:
                col = ["crm_ord_email", "crm_ord"]
                whr.append(("crm_ord_email", "<>", ""))
            accs = self.sql.getRec(tables="crsmst", cols=col, where=whr)
        elif self.ulist == "M":
            tab = ["memmst", "memkon"]
            col = ["mlm_surname", "mlm_names", "mlk_detail"]
            whr = [
                ("mlm_cono", "=", self.conum),
                ("mlm_state", "=", "A"),
                ("mlk_cono=mlm_cono",),
                ("mlk_memno=mlm_memno",),
                ("mlk_code", "=", 5)]
            if self.fltcat:
                tab.append("memcat")
                whr.append(("mlc_cono=mlm_cono",))
                whr.append(("mlc_memno=mlm_memno",))
                whr.append(("mlc_type", "=", self.fltcat))
                if self.fltcod:
                    whr.append(("mlc_code", "=", self.fltcod))
            odr = "mlm_surname, mlm_names"
            get = self.sql.getRec(tables=tab, cols=col, where=whr, order=odr)
            accs = []
            for g in get:
                accs.append((g[2], "%s %s" % (g[1].split()[0], g[0])))
        elif self.ulist in ("V", "X"):
            accs = self.impdat
        pb = ProgressBar(self.mf.body, max=len(accs), typ="E-Mailing")
        for num, acc in enumerate(accs):
            pb.displayProgress(num)
            mail = acc[0].split(",")
            if not acc[1]:
                name = "Sir/Madam"
            else:
                name = self.doNames(acc[1])
            mess = self.mess.replace("{{name}}", name)
            html = self.html.replace("{{name}}", name)
            ok = False
            while not ok:
                if self.mf.display:
                    sp = SplashScreen(self.mf.body, "E-Mailing the Report "\
                        "to %s\n\nPlease Wait........" % mail[0])
                try:
                    sendMail(self.smtp, self.fadd, mail, self.subj,
                        mess=(mess, html), attach=self.attach,
                        embed=self.embed)
                    if sp:
                        sp.closeSplash()
                    ok = "OK"
                except (KeyboardInterrupt, SystemExit):
                    if sp:
                        sp.closeSplash()
                    ok = "ABORTED"
                except:
                    if sp:
                        sp.closeSplash()
                    if self.skip == "Y":
                        ok = "SKIPPED"
                    else:
                        ok = askQuestion(self.mf.body, "E-Mail Error",
                        "Problem Delivering This Message.\n\n"\
                        "Would You Like to Retry?")
                    if ok == "yes":
                        ok = False
                    else:
                        ok = "FAILED"
            # Log the email attempt into table emllog.
            tadd = ""
            for t in mail:
                if not tadd:
                    tadd = t
                else:
                    tadd = tadd + ", " + t
            self.sql.insRec("emllog", data=[self.fadd.strip(), tadd.strip(),
                self.subj.strip(), "%04i-%02i-%02i %02i:%02i" % \
                time.localtime()[0:5], ok])
            self.mf.dbm.commitDbase()
        pb.closeProgress()

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

