#! /usr/bin/env python

"""
SYNOPSIS
    Creditors Masterfile Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import FileImport, GetCtl, ProgressBar, PwdConfirm, Sql
from TartanClasses import TabPrt, TartanDialog
from tartanFunctions import genAccNum, showError

class cr1010:
    def __init__(self, mf, coy, nam, usr, args=None):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.user = usr
        self.args = args
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["chglog", "ctlnot", "crsmst", "crstrn",
            "genmst", "strpom"], prog=__name__)
        gc = GetCtl(self.mf)
        crsctl = gc.getCtl("crsctl", self.conum)
        if not crsctl:
            return "no"
        self.glint = crsctl["ctc_glint"]
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Creditors Masterfile Maintenance (%s)" % __name__)
        crm = {
            "stype": "R",
            "tables": ("crsmst",),
            "cols": (
                ("crm_acno", "", 0, "Acc-Num"),
                ("crm_name", "", 0, "Name", "Y")),
            "where": [("crm_cono", "=", self.conum)]}
        glm = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "where": [("glm_cono", "=", self.conum)]}
        tag = (
            ("Page-_1",None,("T",1,1),("T",0,1)),
            ("Page-_2",None,("T",1,1),("T",0,1)))
        r1s = (("Open Item","O"),("Balance Forward","B"))
        r2s = (("Monthly","M"),("Daily","D"))
        r3s = (("Yes","Y"),("No","N"))
        self.fld = [
            (("T",0,0,0),"INA",7,"Acc-Num","Account Number",
                "","Y",self.doAcno,crm,None,("notblank",)),
            (("T",0,0,0),"INA",30,"Name","Name",
                "","N",None,None,self.doDelete,("notblank",)),
            (("T",1,0,0),"INA",30,"Address Line 1","Address Line 1",
                "","N",None,None,None,("notblank",)),
            (("T",1,1,0),"INA",30,"Address Line 2","Address Line 2",
                "","N",None,None,None,("efld",)),
            (("T",1,2,0),"INA",30,"Address Line 3","Address Line 3",
                "","N",None,None,None,("efld",)),
            (("T",1,3,0),"INA",4,"Postal Code","Postal Code",
                "","N",None,None,None,("efld",)),
            (("T",1,4,0),"INA",20,"Telephone Number","Telephone Number",
                "","N",None,None,None,("efld",)),
            (("T",1,5,0),"INA",20,"Fax Number","Fax Number",
                "","N",None,None,None,("efld",)),
            (("T",1,6,0),"INA",30,"Manager's Name","Manager's Name",
                "","N",None,None,None,("efld",)),
            (("T",1,6,0),"ITX",30,"E-Mail","Manager's E-Mail",
                "","N",None,None,None,("email",)),
            (("T",1,7,0),"INA",30,"Account's Contact","Account's Contact Name",
                "","N",None,None,None,("efld",)),
            (("T",1,7,0),"ITX",30,"E-Mail","Account's E-Mail",
                "","N",None,None,None,("email",)),
            (("T",1,8,0),"INA",30,"Order's Contact","Order's Contact Name",
                "","N",None,None,None,("efld",)),
            (("T",1,8,0),"ITX",30,"E-Mail","Order's E-Mail",
                "","N",None,None,None,("email",)),
            (("T",1,9,0),"Id1",10,"Date Account Opened","Date Account Opened",
                self.sysdtw,"N",None,None,None,("efld",)),
            (("T",1,10,0),"INA",10,"V.A.T Number","V.A.T Number",
                "","N",None,None,None,("efld",)),
            (("T",2,0,0),("IRB",r2s),0,"Terms Base","Terms Base",
                "M","N",self.doTermsBase,None,None,None),
            (("T",2,1,0),"IUI",2,"Statement Day","Statement Day",
                "","N",None,None,None,("between",1,30)),
            (("T",2,2,0),"IUI",3,"Terms","Terms",
                "","N",None,None,None,("between",0,30)),
            (("T",2,3,0),"IUI",7,"Credit Limit","Credit Limit",
                "","N",None,None,None,("efld",)),
            (("T",2,4,0),"IUD",5.2,"Trade Discount","Trade Discount",
                "","N",None,None,None,("efld",)),
            (("T",2,5,0),"IUD",5.2,"Settlement Discount",
                "Settlement Discount","","N",None,None,None,("efld",)),
            (("T",2,6,0),("IRB",r3s),0,"Payment Indicator","Payment Indicator",
                "Y","N",None,None,None,None),
            (("T",2,7,0),"INA",20,"Bank Name","Bank Name",
                "","N",None,None,None,("efld",)),
            (("T",2,8,0),"IUI",8,"Bank Branch","Bank Branch",
                "","N",None,None,None,("efld",)),
            (("T",2,9,0),"INA",16,"Bank Account","Bank Account",
                "","N",None,None,None,("efld",))]
        if self.glint == "Y":
            self.fld.append((("T",2,10,0),"IUI",7,"G/L Account Number",
                "G/L Account Number","","N",self.doGlac,glm,None,("efld",)))
        else:
            self.fld.append((("T",2,10,0),"OUI",7,"G/L Account Number"))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,2),("T",2,0)),
            ("Import",None,self.doImport,0,("T",0,1),("T",0,2),
                "Import Account Details from a CSV or XLS File.",1),
            ("Print",None,self.doPrint,0,("T",0,2),("T",2,0)),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",2,0)),
            ("Quit",None,self.doQuit,1,None,None))
        tnd = ((self.doEnd,"N"), (self.doEnd,"N"), (self.doAccept,"Y"))
        txt = (self.doExit, self.doExit, self.doExit)
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, tags=tag,
            butt=but, tend=tnd, txit=txt, clicks=self.doClick)
        if self.args:
            self.df.doKeyPressed("T", 0, 0, data=self.args)

    def doClick(self, *opts):
        if self.df.pag == 0:
            return
        if self.click:
            self.df.loadEntry("T", self.click[0], self.click[1],
                data=self.df.t_work[self.click[0]][0][self.click[1]])
        self.click = opts[2]
        self.df.focusField("T", opts[2][0], opts[2][1] + 1)

    def doAcno(self, frt, pag, r, c, p, i, w):
        self.acno = w
        self.click = []
        self.old = self.sql.getRec(tables="crsmst", where=[("crm_cono", "=",
            self.conum), ("crm_acno", "=", self.acno)], limit=1)
        if not self.old:
            self.new = True
            pw = PwdConfirm(self.mf, conum=self.conum, system="CRS",
                code="NewAcc")
            if pw.flag == "no":
                if self.args:
                    return "xt"
                else:
                    return "New Account Locked"
        else:
            self.new = False
            self.df.loadEntry("T", pag, p+1, data=self.old[2])
            d = 3
            for pag in range(1, self.df.pgs+1):
                for x in range(0, self.df.topq[pag]):
                    self.df.loadEntry("T", pag, x, data=self.old[d])
                    d = d + 1

    def doTermsBase(self, frt, pag, r, c, p, i, w):
        if w == "D":
            self.df.loadEntry(frt, pag, p+1, 0)
            return "sk1"

    def doGlac(self, frt, pag, r, c, p, i, w):
        if w and not self.sql.getRec(tables="genmst", where=[("glm_cono", "=",
                self.conum), ("glm_acno", "=", w)], limit=1):
            return "Invalid General Ledger Account"

    def doDelete(self):
        trs = self.sql.getRec(tables="crstrn", cols=["count(*)"],
            where=[("crt_cono", "=", self.conum), ("crt_acno", "=", self.acno)],
            limit=1)
        if trs[0]:
            return "Transactions Exist, Not Deleted"
        pom = self.sql.getRec(tables="strpom", cols=["count(*)"],
            where=[("pom_cono", "=", self.conum), ("pom_acno", "=",
            self.acno)], limit=1)
        if pom[0]:
            return "Purchase Order Exists, Not Deleted"
        nte = self.sql.getRec(tables="ctlnot", cols=["count(*)"],
            where=[("not_cono", "=", self.conum), ("not_sys", "=",
            "CRS"), ("not_key", "=", self.acno)], limit=1)
        if nte[0]:
            return "Notes Exist, Not Deleted"
        self.sql.delRec("crsmst", where=[("crm_cono", "=", self.conum),
            ("crm_acno", "=", self.acno)])
        dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
        self.sql.insRec("chglog", data=["crsmst", "D", "%03i%-7s" % \
            (self.conum, self.acno), "", dte, self.user, "", "", 0])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        if self.df.pag == 0:
            self.df.focusField("T", 1, 1, clr=self.new)
        elif self.df.pag == 1:
            self.df.selPage("Page-_2")
            self.df.focusField("T", 2, 1, clr=self.new)

    def doEndAll(self):
        data = [self.conum]
        for x in range(0, 3):
            for y in range(0, len(self.df.t_work[x][0])):
                data.append(self.df.t_work[x][0][y])
        if self.new:
            self.sql.insRec("crsmst", data=data)
        if data != self.old:
            self.sql.updRec("crsmst", data=data, where=[("crm_cono", "=",
                self.conum), ("crm_acno", "=", self.acno)])
            dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
            for num, dat in enumerate(self.old):
                if dat != data[num]:
                    self.sql.insRec("chglog", data=["crsmst", "U",
                        "%03i%-7s" % (self.conum, self.acno),
                        self.sql.crsmst_col[num], dte, self.user, str(dat),
                        str(data[num]), 0])
        if self.args:
            self.doQuit()
        else:
            self.mf.dbm.commitDbase()
            self.df.last[0] = [0, 0]
            self.df.selPage("Page-_1")
            self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            if pag > 0 and pag != self.df.pag:
                if frt == "T":
                    self.df.last[pag][0] = col+1
                else:
                    self.df.last[pag][1] = col+1
                self.df.selPage(self.df.tags[pag-1][0])
            else:
                self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.doEndAll()

    def doPrint(self, widget=None):
        state = self.df.disableButtons()
        self.df.mstAlw.hide()
        rp = TabPrt(self.mf, self.conum, self.conam, name=__name__,
            tabs="crsmst", wher=[("crm_cono", "=", self.conum),
            ("crm_acno", "=", self.acno)])
        self.df.mstAlw.show()
        self.mf.gtkDispatch()
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doImport(self, widget=None):
        self.df.setWidget(self.df.B3, state="disabled")
        self.df.mstAlw.hide()
        fi = FileImport(self.mf, imptab="crsmst", impskp=["crm_cono"])
        sp = ProgressBar(self.mf.body, typ="Importing Creditor's Accounts",
            max=len(fi.impdat), esc=False)
        err = None
        for num, line in enumerate(fi.impdat):
            sp.displayProgress(num)
            if not line[0]:
                if not line[1]:
                    err = "Blank Account Number and Blank Name"
                    break
                for x in range(1, 100):
                    line[0] = genAccNum(line[1], x, 7)
                    chk = self.sql.getRec(tables="crsmst", where=[("crm_cono",
                        "=", self.conum), ("crm_acno", "=", line[0])], limit=1)
                    if not chk:
                        break
            chk = self.sql.getRec(tables="crsmst", where=[("crm_cono",
                "=", self.conum), ("crm_acno", "=", line[0])], limit=1)
            if chk:
                err = "%s %s Already Exists" % (fi.impcol[0][0], line[0])
                break
            if not line[1]:
                err = "Blank Name"
                break
            if not line[14]:
                line[14] = self.sysdtw
            if not line[16]:
                line[16] = "M"
            if not line[17]:
                line[17] = 30
            if not line[18]:
                line[18] = 30
            if not line[22]:
                line[22] = "Y"
            if self.glint and line[26]:
                chk = self.sql.getRec(tables="genmst", where=[("glm_cono",
                    "=", self.conum), ("glm_acno", "=", line[26])], limit=1)
                if not chk:
                    err = "Invalid General Ledger Account"
                    break
            line.insert(0, self.conum)
            self.sql.insRec("crsmst", data=line)
        sp.closeProgress()
        if err:
            err = "Line %s: %s" % ((num + 1), err)
            showError(self.mf.body, "Import Error", """%s
            
Please Correct your Import File and then Try Again.""" % err)
            self.mf.dbm.rollbackDbase()
        else:
            self.mf.dbm.commitDbase()
        self.df.mstAlw.show_all()
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.last[0] = [0, 0]
        self.df.selPage("Page-_1")
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        if self.df.pag == 0:
            self.doQuit()
        elif self.df.pag == 1:
            self.df.focusField("T", 0, 1)
        else:
            self.df.selPage("Page-_1")

    def doQuit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()
