#! /usr/bin/python

"""
SYNOPSIS
    Creditors Data Capture Screen and Execution.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import AgeAll, ASD, Batches, GetCtl, PwdConfirm, Sql
from TartanClasses import TartanDialog
from tartanFunctions import askQuestion, callModule, chkCtlAcc, paymentDate
from tartanFunctions import showError
from tartanWork import crtrtp

class cr2010:
    def __init__(self, mf, coy, nam, period, rtn, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = period
        self.rtn = rtn
        self.capnm = usr
        no = self.checkControls()
        if not no:
            self.setVariables()
            self.drawDialog()
            self.mf.startLoop()

    def checkControls(self):
        # Check for Valid Posting Routine
        if self.rtn not in (1, 2, 3, 4, 5):
            showError(self.mf.body, "Control Error", "Invalid Routine %s, "\
                "Only 1 - 5 Are Allowed" % str(self.rtn))
            return "no"
        # Create SQL Object
        self.sql = Sql(self.mf.dbm, ["ctlmst", "ctlvmf", "ctlvtf", "crsmst",
            "crstrn", "genint", "genmst", "gentrn"], prog=__name__)
        # Check for Intercompany Facility
        coy = self.sql.getRec(tables="ctlmst", cols=["count(*)"],
            where=[("ctm_cono", "<>", self.conum)], limit=1)
        if not coy[0]:
            self.incoac = False
        else:
            int = self.sql.getRec(tables="genint", cols=["cti_inco"],
                where=[("cti_cono", "=", self.conum)])
            if int:
                self.incoac = [self.conum]
                [self.incoac.append(coy[0]) for coy in int]
            else:
                self.incoac = False
        #
        self.gc = GetCtl(self.mf)
        crsctl = self.gc.getCtl("crsctl", self.conum)
        if not crsctl:
            return "no"
        self.glint = crsctl["ctc_glint"]
        self.glinp = crsctl["ctc_glinp"]
        if self.glint == "Y":
            ctlctl = self.gc.getCtl("ctlctl", self.conum)
            if not ctlctl:
                return "no"
            ctls = ["crs_ctl", "vat_ctl", "dis_rec"]
            if self.gc.chkRec(self.conum, ctlctl, ctls):
                return "no"
            self.crsctl = ctlctl["crs_ctl"]
            self.disrec = ctlctl["dis_rec"]
        self.batchHeader()
        if not self.bh.batno:
            return "no"

    def setVariables(self):
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.allcoy = self.conum
        self.taxdf = self.gc.getCtl("ctlmst", self.conum)["ctm_taxdf"]
        if self.rtn == 1:
            self.glt = 5
        elif self.rtn == 2:
            self.glt = 6
        elif self.rtn == 3:
            self.glt = 4
        elif self.rtn == 4:
            self.glt = 5
        elif self.rtn == 5:
            self.glt = 2

    def batchHeader(self):
        self.bh = Batches(self.mf, self.conum, self.conam, self.period,
            "CRS", self.rtn, glint=self.glint)
        self.bh.doBatchHeader()
        if not self.bh.batno:
            return

    def drawDialog(self):
        tit = ("%03i %s" % (self.conum, self.conam), "Creditors " + \
            crtrtp[self.rtn - 1][1] + " Data Capture (%s)" % __name__)
        crm = {
            "stype": "R",
            "tables": ("crsmst",),
            "cols": (
                ("crm_acno", "", 0, "Acc-Num"),
                ("crm_name", "", 0, "Name", "Y")),
            "where": [("crm_cono", "=", self.conum)]}
        coy = {
            "stype": "R",
            "tables": ("ctlmst",),
            "cols": (
                ("ctm_cono", "", 0, "Coy"),
                ("ctm_name", "", 0, "Name", "Y"))}
        glm = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "whera": [["C", "glm_cono", 0, 2]]}
        vtm = {
            "stype": "R",
            "tables": ("ctlvmf",),
            "cols": (
                ("vtm_code", "", 0, "C"),
                ("vtm_desc", "", 0, "Description", "Y")),
            "where": [("vtm_cono", "=", self.conum)]}
        viw = {
            "stype": "R",
            "tables": ("crstrn",),
            "cols": (
                ("crt_acno", "", 0, ""),
                ("crt_ref1", "", 0, ""),
                ("crt_trdt", "", 0, ""),
                ("crt_tramt", "", 0, ""),
                ("crt_taxamt", "", 0, "")),
            "where": [
                ("crt_cono", "=", self.conum),
                ("crt_type", "=", self.rtn),
                ("crt_batch", "=", self.bh.batno)],
            "order": "crt_seq"}
        fld = [
            (("T",0,0,0),"OUI",9,"Batch %s Quantity" % self.bh.batno),
            (("T",0,0,0),"OSD",13.2,"Value"),
            (("C",1,0,0),"INA",7,"Acc-Num","Account Number",
                "r","Y",self.doCrsAcc,crm,None,None),
            [("C",1,0,1),"ONA",10,"Name"],
            (("C",1,0,2),"INa",9,"Reference","Reference Number One",
                "i","N",self.doRef1,None,None,("notblank",)),
            (("C",1,0,3),"ID1",10,"Date","Transaction Date",
                "r","N",self.doTrnDat,None,None,("efld",)),
            (("C",1,0,4),"ISD",13.2,"Amount","Transaction Amount",
                "","N",self.doTrnAmt,None,None,None)]
        if self.rtn in (2, 5):
            fld[3][2] = 22
            fld.extend([
                (("C",1,0,5),"ISD",13.2,"Discount",
                    "Discount Amount","","N",self.doDisAmt,None,None,None),
                [("C",1,0,6),"INA",(22,30),"Details","Transaction Details",
                    "","N",self.doTrnDet,None,None,None]])
        elif self.glint == "Y":
            if self.glinp == "E":
                tx1 = "Exc-Amount"
                tx2 = "Exclusive Amount"
            else:
                tx1 = "Inc-Amount"
                tx2 = "Inclusive Amount"
            fld[3][2] = 30
            fld.extend([
                (("C",1,0,5),"INA",(27,30),"Details","Transaction Details",
                    "","N",self.doTrnDet,None,None,None),
                [("T",2,0,0),"OSD",13.2,"Unallocated Balance"],
                [("C",2,0,0),"IUI",3,"Coy","Company Number",
                    self.conum,"N",self.doCoyNum,coy,None,None],
                (("C",2,0,1),"IUI",7,"Acc-Num","Account Number",
                    "","N",self.doGenAcc,glm,None,None),
                (("C",2,0,2),"ONA",19,"Description"),
                (("C",2,0,3),"IUA",1,"V","V.A.T Code",
                    "","N",self.doVatCode2,vtm,None,("notblank",)),
                (("C",2,0,4),"ISD",13.2,tx1,tx2,
                    "","N",self.doAllAmt,None,None,("efld",)),
                (("C",2,0,5),"ISD",11.2,"V.A.T","V.A.T Amount",
                    "","N",self.doAllVat,None,None,None),
                (("C",2,0,6),"INA",(26,30),"Details","Details",
                    "","N",self.doAllDet,None,None,("notblank",))])
            if not self.incoac:
                fld[9][1] = "OUI"
        else:
            fld[3][2] = 20
            fld.extend([
                (("C",1,0,5),"IUA",1,"V","V.A.T Code",
                    self.taxdf,"N",self.doVatCode1,vtm,None,("notblank",)),
                (("C",1,0,6),"ISD",11.2,"V.A.T","V.A.T Amount",
                    "","N",self.doVatAmt,None,None,None),
                (("C",1,0,7),"IUD",6.2,"Disc-%","Discount Percent",
                    "","N",self.doDisAmt,None,None,None),
                [("C",1,0,8),"INA",(18,30),"Details","Transaction Details",
                    "","N",self.doTrnDet,None,None,None]])
        but = (
            ("_Interrogate",None,self.queryCrs,0,("C",1,1),("C",1,2),
                "Interrogate Creditors Accounts",0),
            ("_View Entries",viw,None,0,("C",1,1),("C",1,2),
                "View Batch Transactions",0),
            ("_Maintain",None,self.maintainCrs,0,("C",1,1),("C",1,2),
                "Maintain Creditors Accounts",0),
            ("Age _Normal",None,self.doAgeNormal,0,None,None,
                "Only Show Unallocated Transactions",1),
            ("Age _History",None,self.doAgeHistory,0,None,None,
                "Show All Transactions, Including Already Allocated",1),
            ("Age _Automatic",None,self.doAgeAuto,0,None,None,
                "Automatically Allocate the Amount Starting With the "\
                "Oldest Unallocated Transaction",1),
            ("Age _Current",None,self.doAgeCurrent,0,None,None,
                "Leave the Transaction Unallocated",1),
            ("Cancel",None,self.doCancel,0,("C",2,1),("C",1,1),"",1))
        tag = [("Transaction",None,None,None)]
        cnd = [(None,"n"), (self.endPage1,"y")]
        cxt = [None, self.exitPage1]
        if self.rtn not in (2, 5) and self.glint == "Y":
            tag.append(("Allocation", None, None, None))
            cnd.append((self.endPage2,"y"))
            cxt.append(self.exitPage2)
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tags=tag,
            cend=cnd, cxit=cxt, butt=but)
        self.df.loadEntry("T", 0, 0, data=self.bh.batqty)
        self.df.loadEntry("T", 0, 1, data=self.bh.batval)
        self.df.focusField("C", 1, 1)

    def doCrsAcc(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="crsmst", cols=["crm_name", "crm_terms",
            "crm_termsb", "crm_stday", "crm_pydis", "crm_glac"],
            where=[("crm_cono", "=", self.conum), ("crm_acno", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Account Number"
        self.cracno = w
        self.name, self.term, self.base, self.stdt, self.pdis, self.glac = acc
        self.df.loadEntry("C", pag, p+1, data=self.name)

    def doRef1(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="crstrn", cols=["crt_batch"],
            where=[("crt_cono", "=", self.conum), ("crt_acno", "=",
            self.cracno), ("crt_ref1", "=", w)])
        if acc:
            return "Transaction Already Exists"
        self.ref1 = w

    def doTrnDat(self, frt, pag, r, c, p, i, w):
        if w < self.period[1][0] or w > self.period[2][0]:
            return "Invalid Date, Not in Financial Period"
        self.trndat = w

    def doTrnAmt(self, frt, pag, r, c, p, i, w):
        self.trnamt = w
        if self.glint == "Y" or self.rtn in (2, 5):
            self.vatcode = "N"
            self.vatamt = 0

    def doVatCode1(self, frt, pag, r, c, p, i, w):
        vat = self.sql.getRec(tables="ctlvmf", cols=["vtm_new"],
            where=[("vtm_cono", "=", self.conum), ("vtm_code", "=", w)],
            limit=1)
        if not vat:
            return "Invalid V.A.T Code"
        self.vatcode = w
        rate = vat[0]
        self.vatamt = round((self.trnamt * rate / (rate + 100)), 2)
        self.df.loadEntry(frt, pag, p+1, data=self.vatamt)
        if self.vatamt == 0:
            return "sk"

    def doVatAmt(self, frt, pag, r, c, p, i, w):
        if self.trnamt < 0 and w > 0:
            self.vatamt = float(ASD(0) - ASD(w))
        elif self.trnamt > 0 and w < 0:
            self.vatamt = float(ASD(0) - ASD(w))
        else:
            self.vatamt = w
        self.df.loadEntry(frt, pag, p, data=self.vatamt)

    def doDisAmt(self, frt, pag, r, c, p, i, w):
        if self.trnamt < 0 and w > 0:
            self.disamt = float(ASD(0) - ASD(w))
        elif self.trnamt > 0 and w < 0:
            self.disamt = float(ASD(0) - ASD(w))
        else:
            self.disamt = w
        self.df.loadEntry(frt, pag, p, data=self.disamt)

    def doTrnDet(self, frt, pag, r, c, p, i, w):
        self.trndet = w
        if self.rtn not in (2, 5) and self.glint == "Y":
            self.df.colf[2][6][5] = w
        self.allocated = float(0.00)

    def endPage1(self):
        self.cancel = False
        self.agecan = False
        self.batupd = False
        self.updateTables1()
        if self.cancel:
            return
        else:
            self.updateBatch()
            if self.trnamt == 0:
                self.df.advanceLine(1)
            else:
                self.allocated = float(0.0)
                if self.rtn in (2, 5) or self.glint == "N":
                    self.doCrsTrn()
                    self.mf.dbm.commitDbase()
                    self.df.selPage("Transaction")
                    self.df.advanceLine(1)
                else:
                    self.df.selPage("Allocation")
                    self.df.loadEntry("T", 2, 0, data=self.trnamt)
                    self.df.focusField("C", 2, 1)

    def updateTables1(self):
        if self.bh.multi == "Y":
            self.curdt = self.trndat / 100
        else:
            self.curdt = self.bh.curdt
        self.amt = self.trnamt
        self.vat = self.vatamt
        if self.rtn == 1:                          # Invoices
            self.recon = 0
            self.dis = 0.00
            self.per = self.pdis
        elif self.rtn == 2:                        # Receipts
            if self.trnamt == 0:
                self.recon = self.curdt
            else:
                self.recon = 0
            self.vat = 0.00
            self.dis = self.disamt
            self.per = 0.00
        elif self.rtn == 3:                        # Journals
            self.recon = 0
            self.dis = 0.00
            self.per = 0.00
        elif self.rtn == 4:                        # Credit Notes
            self.recon = 0
            self.amt = float(ASD(0) - ASD(self.amt))
            self.vat = float(ASD(0) - ASD(self.vat))
            self.dis = 0.00
            self.per = self.pdis
        elif self.rtn == 5:                        # Payments
            if self.trnamt == 0:
                self.recon = self.curdt
            else:
                self.recon = 0
            self.amt = float(ASD(0) - ASD(self.amt))
            self.vat = 0.00
            self.dis = float(ASD(0) - ASD(self.disamt))
            self.per = 0.00
        if self.rtn == 1:
            self.ageloop = False
            self.doAgeCurrent()
        else:
            state = self.df.disableButtons()
            self.mf.updateStatus("Choose an Ageing Option")
            for x in range(2, 7):
                exec "self.df.setWidget(self.df.B%s, 'normal')" % x
            self.ageloop = True
            self.mf.startLoop()
            self.df.enableButtons(state=state)
            if self.agecan:
                self.doCancel()
                return
        if self.glint == "N":
            return
        # General Ledger Control Transaction (Creditors)
        val = float(ASD(0) - ASD(self.amt) - ASD(self.dis))
        data = (self.conum, self.crsctl, self.curdt, self.trndat,
            self.glt, self.ref1, self.bh.batno, val, 0.00, self.trndet,
            "N", "", self.recon, self.sysdtw, 0)
        self.sql.insRec("gentrn", data=data)
        # General Ledger Control Transaction (Bank)
        if self.rtn in (2, 5):
            data = (self.conum, self.bh.acc, self.curdt, self.trndat,
                self.glt, self.ref1, self.bh.batno, self.amt, 0.00,
                self.trndet, "N", "", self.recon, self.sysdtw, 0)
            self.sql.insRec("gentrn", data=data)
            # General Ledger Control Transaction (Discount)
            if self.dis:
                data = (self.conum, self.disrec, self.curdt, self.trndat,
                    self.glt, self.ref1, self.bh.batno, self.dis, 0.00,
                    self.trndet, "N", "", self.recon, self.sysdtw, 0)
                self.sql.insRec("gentrn", data=data)

    def doAgeNormal(self, widget=None):
        self.doAgeing("N")

    def doAgeHistory(self, widget=None):
        self.doAgeing("H")

    def doAgeAuto(self, widget=None):
        self.doAgeing("O")

    def doAgeCurrent(self, widget=None):
        self.doAgeing("C")

    def doAgeing(self, atype):
        # Disable All Ageing Buttons
        for x in range(1, 5):
            exec "self.df.setWidget(self.df.B%s, 'disabled')" % x
        self.mf.updateStatus("Select a Transaction to Allocate Against")
        age = AgeAll(self.mf,  system="crs", agetyp=atype, agekey=[self.conum,
            self.cracno, self.rtn, self.ref1, self.curdt, self.amt, self.dis])
        self.agecan = age.cancel
        if self.ageloop:
            self.mf.closeLoop()

    def exitPage1(self):
        self.df.closeProcess()
        self.bh.doBatchTotal()
        self.mf.closeLoop()

    def doCoyNum(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="ctlmst", cols=["ctm_name"],
            where=[("ctm_cono", "=", w)], limit=1)
        if not acc:
            return "Invalid Company, Missing"
        if self.incoac and w not in self.incoac:
            return "Invalid Company, No Intercompany Record 1"
        if w != self.conum:
            acc = self.sql.getRec(tables="genint", where=[("cti_cono", "=", w),
                ("cti_inco", "=", self.conum)], limit=1)
            if not acc:
                return "Invalid Company, No Intercompany Record 2"
        self.allcoy = w
        self.df.loadEntry(frt, pag, p+1, data=self.glac)

    def doGenAcc(self, frt, pag, r, c, p, i, w):
        self.genacc = w
        acc = self.sql.getRec(tables="genmst", cols=["glm_desc", "glm_ind",
            "glm_vat"], where=[("glm_cono", "=", self.allcoy), ("glm_acno",
            "=", self.genacc)], limit=1)
        if not acc:
            return "Invalid G/L Account"
        if acc[1] == "N":
            cf = PwdConfirm(self.mf, conum=self.allcoy, system="GEN",
                code="NoPost")
            if not cf.pwd or cf.flag == "no":
                return "Postings Disallowed"
        else:
            chk = chkCtlAcc(self.mf, self.allcoy, self.genacc)
            if chk:
                return chk
        if not acc[2]:
            self.taxgl = self.taxdf
        else:
            self.taxgl = acc[2]
        self.df.loadEntry("C", pag, p+1, data=acc[0])
        self.df.loadEntry("C", pag, p+2, data=self.taxgl)

    def doVatCode2(self, frt, pag, r, c, p, i, w):
        ctlctl = self.gc.getCtl("ctlctl", self.allcoy, error=False)
        if not ctlctl:
            return "Missing ctlctl Record for Company"
        if self.gc.chkRec(self.conum, ctlctl, ["vat_ctl"]):
            return "Missing or Invalid Control Record"
        self.convat = ctlctl["vat_ctl"]
        acc = self.sql.getRec(tables="ctlvmf", cols=["vtm_new"],
            where=[("vtm_cono", "=", self.allcoy), ("vtm_code", "=", w)],
            limit=1)
        if not acc:
            return "Invalid V.A.T Code"
        self.vatcode = w
        self.vatrat = acc[0]

    def doAllAmt(self, frt, pag, r, c, p, i, w):
        if not w:
            incamt = float(ASD(self.trnamt) - ASD(self.allocated))
        elif self.glinp == "E":
            incamt = round((w * (100 + self.vatrat) / 100), 2)
        else:
            incamt = w
        self.allamt = round((incamt * 100 / (100 + self.vatrat)), 2)
        self.allvat =  float(ASD(incamt) - ASD(self.allamt))
        if self.glinp == "E":
            self.df.loadEntry(frt, pag, p, data=self.allamt)
        else:
            self.df.loadEntry(frt, pag, p, data=incamt)
        self.df.loadEntry(frt, pag, p+1, data=self.allvat)

    def doAllVat(self, frt, pag, r, c, p, i, w):
        if (self.allamt < 0 and w > 0) or (self.allamt > 0 and w < 0):
            w = float(ASD(0) - ASD(w))
        if self.glinp == "I" and w != self.allvat:
            self.allamt = float(ASD(self.allamt) + ASD(self.allvat) - ASD(w))
        self.allvat = w
        self.df.loadEntry(frt, pag, p, data=self.allvat)
        self.df.loadEntry(frt, pag, p+1, data=self.name)

    def doAllDet(self, frt, pag, r, c, p, i, w):
        self.alldet = w

    def doCancel(self, widget=None):
        if self.agecan:
            ok = "yes"
        else:
            ok = askQuestion(self.mf.body, head="Cancel",
                mess="Are You Certain You Want to Cancel This Entry?")
        if ok == "yes":
            self.cancel = True
            self.mf.dbm.rollbackDbase()
            if self.batupd:
                self.updateBatch(rev=True)
            if self.glint == "Y" and self.rtn not in (2, 5):
                self.df.clearFrame("C", 2)
            self.df.selPage("Transaction")
            row = int((self.df.last[1][1] - 1) / self.df.colq[1])
            col = (row * self.df.colq[1]) + 1
            self.df.focusField("C", 1, col)

    def endPage2(self):
        self.updateTables2()
        self.allocated = float(ASD(self.allocated) + ASD(self.allamt) + \
            ASD(self.allvat))
        if self.allocated == self.trnamt:
            self.doCrsTrn()
            self.mf.dbm.commitDbase()
            self.df.clearFrame("C", 2)
            self.df.selPage("Transaction")
            self.df.advanceLine(1)
        else:
            bal = float(ASD(self.trnamt) - ASD(self.allocated))
            self.df.loadEntry("T", 2, 0, data=bal)
            self.df.advanceLine(2)

    def updateTables2(self):
        if self.bh.multi == "Y":
            self.curdt = self.trndat / 100
        else:
            self.curdt = self.bh.curdt
        if self.rtn == 1:                          # Invoices
            self.debit()
        elif self.rtn == 3:                        # Journals
            self.debit()
        elif self.rtn == 4:                        # Credit Notes
            self.credit()

    def debit(self):
        amt = self.allamt
        vat = self.allvat
        self.restDebitCredit(amt, vat)

    def credit(self):
        amt = float(ASD(0) - ASD(self.allamt))
        vat = float(ASD(0) - ASD(self.allvat))
        self.restDebitCredit(amt, vat)

    def restDebitCredit(self, amt, vat):
        # VAT Transaction (ctlvtf)
        data = (self.allcoy, self.vatcode, "I", self.curdt, "C", self.rtn,
            self.bh.batno, self.ref1, self.trndat, self.cracno, self.alldet,
            amt, vat, self.sysdtw, 0, 0)
        self.sql.insRec("ctlvtf", data=data)
        if self.glint == "N":
            return
        # General Ledger Transaction (Expense)
        data = (self.allcoy, self.genacc, self.curdt, self.trndat,
            self.glt, self.ref1, self.bh.batno, amt, vat, self.alldet,
            self.vatcode, "", 0, self.sysdtw, 0)
        self.sql.insRec("gentrn", data=data)
        # General Ledger Transaction (Intercompany)
        if self.allcoy != self.conum:
            # General Ledger Transaction (Intercompany From)
            acc = self.sql.getRec(tables="genint", cols=["cti_acno"],
                where=[("cti_cono", "=", self.conum), ("cti_inco", "=",
                self.allcoy)], limit=1)[0]
            val = float(ASD(amt) + ASD(vat))
            data = (self.conum, acc, self.curdt, self.trndat, self.glt,
                self.ref1, self.bh.batno, val, 0.00, self.alldet, "N",
                "", 0, self.sysdtw, 0)
            self.sql.insRec("gentrn", data=data)
            # General Ledger Transaction (Intercompany To)
            acc = self.sql.getRec(tables="genint", cols=["cti_acno"],
                where=[("cti_cono", "=", self.allcoy), ("cti_inco", "=",
                self.conum)], limit=1)[0]
            val = float(ASD(0) - ASD(amt) - ASD(vat))
            data = (self.allcoy, acc, self.curdt, self.trndat, self.glt,
                self.ref1, self.bh.batno, val, 0.00, self.alldet, "N",
                "", 0, self.sysdtw, 0)
            self.sql.insRec("gentrn", data=data)
        if vat:
            # General Ledger Control Transaction (V.A.T.)
            data = (self.allcoy, self.convat, self.curdt, self.trndat,
                self.glt, self.ref1, self.bh.batno, vat, 0.00, self.alldet,
                "N", "", 0, self.sysdtw, 0)
            self.sql.insRec("gentrn", data=data)
        self.vat = float(ASD(self.vat) + ASD(vat))

    def exitPage2(self):
        self.df.focusField("C", 2, self.df.col)

    def doCrsTrn(self):
        # Creditors Ledger Transaction
        paydt = paymentDate(self.base, self.stdt, self.term, self.trndat)
        data = [self.conum, self.cracno, self.rtn, self.ref1, self.bh.batno,
            self.trndat, "", self.amt, self.vat, self.per, self.curdt, paydt,
            "Y", self.amt, self.trndet, self.vatcode, "N", self.sysdtw, 0]
        self.sql.insRec("crstrn", data=data)
        if not self.dis:
            return
        data = [self.conum, self.cracno, 6, self.ref1, self.bh.batno,
            self.trndat, "", self.dis, self.vat, 0, self.curdt, paydt,
            "Y", self.dis, self.trndet, self.vatcode, "N", self.sysdtw, 0]
        self.sql.insRec("crstrn", data=data)

    def updateBatch(self, rev=False):
        if rev:
            self.bh.batqty = self.bh.batqty - 1
            self.bh.batval = float(ASD(self.bh.batval) - ASD(self.trnamt))
        else:
            self.batupd = True
            self.bh.batqty = self.bh.batqty + 1
            self.bh.batval = float(ASD(self.bh.batval) + ASD(self.trnamt))
        self.df.loadEntry("T", 0, 0, data=self.bh.batqty)
        self.df.loadEntry("T", 0, 1, data=self.bh.batval)

    def queryCrs(self, widget=None):
        callModule(self.mf, self.df, "cr4010", coy=(self.conum, self.conam),
            period=None, user=self.capnm)

    def maintainCrs(self, widget=None):
        callModule(self.mf, self.df, "cr1010", coy=(self.conum, self.conam),
            period=None, user=self.capnm)
