#! /usr/bin/env python

"""
SYNOPSIS
    Creditors Ledger Transactions Due For Payment.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from tartanWork import crtrtp
from TartanClasses import ASD, CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, doPrinter, getTrn, getModName
from tartanFunctions import projectDate, showError

class cr3030:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["crsmst", "crstrn"], prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Creditors Transactions Due For Payment (%s)" % __name__)
        r1s = (("Monthly","M"),("Daily","D"),("Both","B"))
        fld = (
            (("T",0,1,0),("IRB",r1s),0,"Terms Base","Terms Base",
                "M","Y",self.doPerTyp,None,None,None),
            (("T",0,0,0),"ID1",10,"Payment Date","Payment Date",
                self.sysdtw,"Y",self.doPayDate,None,None,("efld",)),
            (("T",0,7,0),"IUI",2,"Days Leeway","Days Leeway",
                0,"Y",self.doLeeway,None,None, ("between",0,30)))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))
        self.t = None

    def doPerTyp(self, frt, pag, r, c, p, i, w):
        self.ptyp = w

    def doPayDate(self, frt, pag, r, c, p, i, w):
        if w == 0:
            w = self.sysdtw
        self.pdatw = w
        self.pdatd = self.df.t_disp[pag][r][p]
        self.df.loadEntry(frt, pag, p, data=self.pdatw)

    def doLeeway(self, frt, pag, r, c, p, i, w):
        self.leeway = w
        if self.leeway:
            self.pdatw = projectDate(self.pdatw, self.leeway)

    def doEnd(self):
        self.df.closeProcess()
        whr = [("crm_cono", "=", self.conum), ("crm_pyind", "<>", "N")]
        if self.ptyp != "B":
            whr.append(("crm_termsb", "=", self.ptyp))
        mst = self.sql.getRec(tables="crsmst", cols=["crm_acno", "crm_name",
            "crm_bname", "crm_bibt", "crm_bacc"], where=whr, order="crm_acno")
        if not mst:
            showError(self.mf.body, "Selection Error",
            "No Accounts Selected")
        else:
            self.printReport(mst)
        self.closeProcess()

    def printReport(self, mst):
        p = ProgressBar(self.mf.body, max=len(mst), esc=True)
        self.head = ("%03u %-30s %14s %35s %14s %10s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        pdfnam = getModName(self.mf.wrkdir, __name__, self.conum, ext="pdf")
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, pdfnam, self.head)
        self.ctots = [0,0,0,0,0]
        self.gtots = [0,0,0,0,0]
        self.pgnum = 0
        self.pglin = 99
        for num, rec in enumerate(mst):
            can = p.displayProgress(num)
            if can == "yes":
                self.t.textLine()
                self.t.textLine("Print Job ABORTED")
                break
            acno = CCD(rec[0], "NA", 7)
            name = CCD(rec[1], "NA", 30)
            if rec[2] and rec[3] and rec[4]:
                ptyp = "E"
            else:
                ptyp = "C"
            col, all = getTrn(self.mf.dbm, "crs", whr=[("crt_cono", "=",
                self.conum), ("crt_acno", "=", acno.work), ("crt_paydt", "<=",
                self.pdatw), ("crt_payind", "=", "Y")], neg=False, zer="N")
            if not all:
                continue
            if not self.pgnum:
                self.pageHeading(acno.disp, name.disp, ptyp)
            else:
                self.newAccount(acno.disp, name.disp, ptyp)
            for dat in all:
                ref1 = CCD(dat[col.index("crt_ref1")], "Na", 9)
                ref2 = CCD(dat[col.index("crt_ref2")], "Na", 9)
                trtp = CCD(dat[col.index("crt_type")], "UI", 1)
                trdt = CCD(dat[col.index("crt_trdt")], "d1", 10)
                disper = CCD(dat[col.index("crt_disper")], "SD", 7.2)
                tramt = CCD(dat[col.index("crt_tramt")], "SD", 13.2)
                paid = CCD(dat[col.index("paid")], "SD", 13.2)
                trbal = CCD(dat[col.index("balance")], "SD", 13.2)
                sett = round((trbal.work * disper.work / 100) ,2)
                sett = CCD(sett, "SD", 13.2)
                nett = float(ASD(trbal.work) - ASD(sett.work))
                nett = CCD(nett, "SD", 13.2)
                if self.pglin > self.lpp:
                    if self.pgnum:
                        self.c.drawText(self.t)
                        self.c.showPage()
                    self.pageHeading(acno.disp, name.disp, ptyp)
                self.t.textLine("%s %s %s %s %s %s %s %s %s %s" % \
                    (ref1.disp, ref2.disp, crtrtp[trtp.work-1][0], trdt.disp,
                    disper.disp, tramt.disp, paid.disp, trbal.disp, sett.disp,
                    nett.disp))
                self.pglin = self.pglin + 1
                self.ctots[0] = float(ASD(self.ctots[0]) + ASD(tramt.work))
                self.ctots[1] = float(ASD(self.ctots[1]) + ASD(paid.work))
                self.ctots[2] = float(ASD(self.ctots[2]) + ASD(trbal.work))
                self.ctots[3] = float(ASD(self.ctots[3]) + ASD(sett.work))
                self.ctots[4] = float(ASD(self.ctots[4]) + ASD(nett.work))
                self.gtots[0] = float(ASD(self.gtots[0]) + ASD(tramt.work))
                self.gtots[1] = float(ASD(self.gtots[1]) + ASD(paid.work))
                self.gtots[2] = float(ASD(self.gtots[2]) + ASD(trbal.work))
                self.gtots[3] = float(ASD(self.gtots[3]) + ASD(sett.work))
                self.gtots[4] = float(ASD(self.gtots[4]) + ASD(nett.work))
            if self.t:
                self.accountTotal()
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
            
    def pageHeading(self, acno, name, ptyp):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-44s %-10s %49s %5s" % \
            ("Creditors Transactions Due For Payment as at",
            self.pdatd, "Page", self.pgnum))
        self.t.textLine()
        self.t.textLine("%-21s%-1s%-3s%-9s%-10s%-3s%-12s%-2s%-1s" % \
            ("(Options: Terms-Base", self.ptyp, "", "Pay-Date", self.pdatd,
            "", "Days-Leeway", self.leeway, ")"))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 5
        self.newAccount(acno, name, ptyp)

    def newAccount(self, acno, name, ptyp):
        if self.pglin > (self.lpp - 5):
            self.c.drawText(self.t)
            self.c.showPage()
            self.pageHeading(acno, name, ptyp)
        else:
            self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.t.textLine("%-7s %-7s %-30s (%s)" % ("Account",acno,name,ptyp))
            self.t.textLine()
            self.t.textLine("%-9s %-9s %-3s %-10s %-6s %-13s %-13s %-13s "\
                "%-13s %-13s" % ("Reference", "Ref-Num-2", "Typ", "   Date",
                " Dis-%", "       Amount", "    Pd-Amount", "   Due-Amount",
                "   Set-Amount", "   Net-Amount"))
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.t.setFont("Tartan", self.cpi, self.ldg)
            self.pglin = self.pglin + 5

    def accountTotal(self):
        j = CCD(self.ctots[0], "SD", 13.2)
        k = CCD(self.ctots[1], "SD", 13.2)
        l = CCD(self.ctots[2], "SD", 13.2)
        m = CCD(self.ctots[3], "SD", 13.2)
        n = CCD(self.ctots[4], "SD", 13.2)
        self.t.textLine()
        self.t.textLine("%-42s %13s %13s %13s %13s %13s" % \
            ("Account Totals", j.disp, k.disp, l.disp, m.disp, n.disp))
        self.pglin = self.pglin + 3
        self.ctots = [0,0,0,0,0,0]
    
    def grandTotal(self):
        j = CCD(self.gtots[0], "SD", 13.2)
        k = CCD(self.gtots[1], "SD", 13.2)
        l = CCD(self.gtots[2], "SD", 13.2)
        m = CCD(self.gtots[3], "SD", 13.2)
        n = CCD(self.gtots[4], "SD", 13.2)
        self.t.textLine()
        self.t.textLine("%-42s %13s %13s %13s %13s %13s" % \
            ("Grand Totals", j.disp, k.disp, l.disp, m.disp, n.disp))
        self.t.textLine()

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

