#! /usr/bin/env python

"""
SYNOPSIS
    Creditors Ledger Reconciliation Statements.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from tartanWork import crtrtp
from TartanClasses import ASD, CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, doPrinter, getTrn, getModName
from tartanFunctions import getSingleRecords, showError

class cr3070:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.curdt = self.sysdtw / 100
        self.t = None
        self.sql = Sql(self.mf.dbm, ["crsmst", "crstrn"], prog=__name__)

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Creditors Reconciliation Statements (%s)" % __name__)
        r1s = (("Yes","Y"),("Singles","S"))
        r2s = (("Number","N"),("Name","M"))
        fld = (
            (("T",0,0,0),"ID2",7,"Period","Period",
                self.curdt,"Y",self.doPeriod,None,None,("efld",)),
            (("T",0,1,0),("IRB",r1s),0,"Whole File","Whole File",
                "S","Y",self.doWhole,None,None,None),
            (("T",0,2,0),("IRB",r2s),0,"Sort Order","Sort Order",
                "N","Y",self.doSort,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"), save="N")

    def doPeriod(self, frt, pag, r, c, p, i, w):
        self.curdt = w
        self.curdd = self.df.t_disp[pag][0][i]

    def doWhole(self, frt, pag, r, c, p, i, w):
        self.whole = w
        if self.whole == "S":
            return "sk1"

    def doSort(self, frt, pag, r, c, p, i, w):
        if w == "N":
            self.sort = "crm_acno"
        else:
            self.sort = "crm_name"
         
    def doEnd(self):
        self.df.closeProcess()
        self.view = self.df.repprt[0][1]
        self.prtnam = self.df.repprt[0][2]
        self.emadd = self.df.repprt[1][2]
        self.archive = self.df.repprt[2][1]
        if self.archive == "Y":
            dir = self.mf.arcdir
        else:
            dir = self.mf.wrkdir
        self.pdfnam = getModName(dir, __name__, self.conum, ext="pdf")
        self.prnt = "N"
        if self.whole == "S":
            all = getSingleRecords(self.mf, "crsmst",
            ("crm_acno", "crm_name"), where=[("crm_cono", "=", self.conum)])
        else:
            all = self.sql.getRec(tables="crsmst", where=[("crm_cono",
                "=", self.conum)], order=self.sort)
            if not all:
                showError(self.mf.body, "Error",
                "No Accounts Available")
        if all:
            num = len(all)
            p = ProgressBar(self.mf.body, max=num, esc=True)
            for x in range(0, num):
                can = p.displayProgress(x)
                if can == "yes":
                    if self.t:
                        self.t.textLine()
                        self.t.textLine("Print Job ABORTED")
                    break
                self.doProcess(all[x])
            p.closeProgress()
        if self.prnt == "Y":
            self.c.save()
            doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                head=self.tit, view=self.view, prtnam=self.prtnam,
                mail=self.emadd)
        self.closeProcess()
        
    def doProcess(self, crm):
        self.pgnum = 0
        self.totbal = 0
        cmc = self.sql.crsmst_col
        self.acno = CCD(crm[cmc.index("crm_acno")], "NA", 7)
        whr = [("crt_cono", "=", self.conum), ("crt_acno", "=", self.acno.work)]
        ord = "crt_curdt, crt_trdt, crt_ref1"
        self.ctc, self.crt = getTrn(self.mf.dbm, "crs", dte=self.curdt,
            whr=whr, ord=ord)
        if self.crt:
            if self.prnt == "N":
                self.printSetup()
            self.name = CCD(crm[cmc.index("crm_name")], "NA", 30)
            self.add1 = CCD(crm[cmc.index("crm_add1")], "NA", 30)
            self.add2 = CCD(crm[cmc.index("crm_add2")], "NA", 30)
            self.add3 = CCD(crm[cmc.index("crm_add3")], "NA", 30)
            self.pcod = CCD(crm[cmc.index("crm_pcod")], "NA", 4)
            self.terms = CCD(crm[cmc.index("crm_terms")], "UI", 3)
            self.termsb = CCD(crm[cmc.index("crm_termsb")], "NA", 1)
            self.stday = CCD(crm[cmc.index("crm_stday")], "UI", 2)
            self.pydis = CCD(crm[cmc.index("crm_pydis")], "SD", 7.2)
            self.printHeader()
            self.printBody()
            self.printTrailer()

    def printSetup(self):
        self.head = ("%03u %-30s %13s %15s %14s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.prnt = "Y"
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)

    def printHeader(self):
        self.pageHeading()
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-42s %-34s %-18s %-7s" % \
            (self.name.disp, "", "Acc-Num:", self.acno.disp))
        self.t.textLine("%-42s %-34s %-18s %-7s" % \
            (self.add1.disp, "", "Period", self.curdd))
        self.t.textLine("%-42s %-34s %-24s %-5s" % \
            (self.add2.disp, "", "Page:", self.pgnum))
        self.t.textLine("%-42s %-34s %-15s %-10s" % \
            (self.add3.disp, "", "", ""))
        self.t.textLine("%-42s %-34s" % \
            (self.pcod.disp, ""))
        self.t.textLine()
        self.t.textLine("%-11s %-17s %-5s %-17s %-14s %-20s %-6s %-7s" % \
            ("Terms Base:", self.termsb.disp, "Days:", self.terms.disp,
            "Statement Day:", self.stday.disp, "Disc-%:", self.pydis.disp))
        self.t.textLine()
        self.t.textLine("%-10s  %-9s  %-9s  %-16s %-13s %-13s %-13s %-13s" % \
            ("   Date", "Reference", "Ref-Num-2", "Type", "Orig-Amount",
            "Paid-Amount", " Bal-Amount", "    Balance"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        if self.pgnum > 1:
            j = CCD(self.totbal, "SD", 13.2)
            self.t.textLine("%-31s %-5s %-67s %s" % \
                ("", "B/Fwd", "", j.disp))
            self.pglin = 11
        else:
            self.pglin = 10
        self.t.setFont("Tartan", self.cpi, self.ldg)

    def printBody(self):
        for tr in self.crt:
            trtp = CCD(tr[self.ctc.index("crt_type")], "UI", 1)
            batch = CCD(tr[self.ctc.index("crt_batch")], "Na", 7)
            acno = CCD(tr[self.ctc.index("crt_acno")], "NA", 7)
            ref1 = CCD(tr[self.ctc.index("crt_ref1")], "Na", 9)
            ref2 = CCD(tr[self.ctc.index("crt_ref2")], "Na", 9)
            trdt = CCD(tr[self.ctc.index("crt_trdt")], "d1", 10)
            detail = CCD(tr[self.ctc.index("crt_desc")], "NA", 30)
            tramt = CCD(tr[self.ctc.index("crt_tramt")], "SD", 13.2)
            pay = CCD(tr[self.ctc.index("paid")], "SD", 13.2)
            bal = CCD(tr[self.ctc.index("balance")], "SD", 13.2)
            if self.pglin > self.lpp:
                self.printTrailer()
                self.printHeader()
            self.totbal = float(ASD(self.totbal) + ASD(bal.work))
            totbal = CCD(self.totbal, "SD", 13.2)
            typ = crtrtp[trtp.work-1][1]+(" "*(15-len(crtrtp[trtp.work-1][1])))
            self.t.textLine("%s  %s  %s  %s %s %s %s %s" % (trdt.disp,
                ref1.disp, ref2.disp, typ, tramt.disp, pay.disp, bal.disp,
                totbal.disp))
            self.pglin = self.pglin + 1

    def printTrailer(self):
        self.c.drawText(self.t)
        self.c.showPage()

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

