#! /usr/bin/env python

"""
SYNOPSIS
    Creditors Ledger Print Labels. L7159

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import CCD, GetCtl, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, doPrinter, getModName, getSingleRecords
from tartanFunctions import makeArray, showError

class cr3090:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "crsmst", prog=__name__)
        gc = GetCtl(self.mf)
        crsctl = gc.getCtl("crsctl", self.conum)
        if not crsctl:
            return "no"
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None
        self.labels = {"L7159": (3, 8, 32)}

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Print Creditors Labels (%s)" % __name__)
        r1s = (("Yes","Y"),("Singles","S"))
        r2s = (("Number","N"),("Name","M"),("Postal Code","P"))
        lab = []
        for l in self.labels:
            lab.append(l)
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Whole File","",
                "S","Y",self.doWhole,None,None,None),
            (("T",0,1,0),("IRB",r2s),0,"Sort Order","",
                "N","Y",self.doSort,None,None,None),
            (("T",0,2,0),"IUA",5,"Avery A4 Code","",
                "L7159","Y",self.doAvery,None,None,("in",lab)),
            (("T",0,3,0),"IUI",2,"First Label Position","",
                1,"Y",self.doStart,None,None,("notzero",)))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"))
    def doWhole(self, frt, pag, r, c, p, i, w):
        self.whole = w

    def doSort(self, frt, pag, r, c, p, i, w):
        self.sort = w

    def doAvery(self, frt, pag, r, c, p, i, w):
        self.across, self.down, self.width = self.labels[w]

    def doStart(self, frt, pag, r, c, p, i, w):
        max = (self.across * self.down) + 1
        if not w in range (1, max):
            return "Out of range"
        self.start = w

    def doEnd(self):
        self.df.closeProcess()
        self.view = self.df.repprt[0][1]
        self.prtnam = self.df.repprt[0][2]
        self.emadd = self.df.repprt[1][2]
        self.prnt = False
        tab = ["crsmst"]
        whr = [("crm_cono", "=", self.conum)]
        if self.sort == "N":
            ord = "crm_acno"
        elif self.sort == "M":
            ord = "crm_name"
        else:
            ord = "crm_pcod"
        if self.whole == "S":
            all = getSingleRecords(self.mf, "crsmst",
                ("crm_acno", "crm_name"), where=whr, order=ord)
        else:
            all = self.sql.getRec(tables=tab, where=whr, order=ord)
        if not all:
            showError(self.mf.body, "Error", "No Accounts Selected")
        else:
            p = ProgressBar(self.mf.body, max=len(all), esc=True)
            for num, dat in enumerate(all):
                can = p.displayProgress(num)
                if can == "yes":
                    if self.t:
                        self.t.textLine()
                        self.t.textLine("Print Job ABORTED")
                    break
                self.doProcess(dat)
            p.closeProgress()
        if self.prnt:
            if self.vcount == 0 and self.hcount == 0:
                pass
            else:
                self.pageHeading()
                self.printPage()
                self.c.drawText(self.t)
                self.c.showPage()
            self.c.save()
            doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                head=self.tit, view=self.view, prtnam=self.prtnam,
                mail=self.emadd)
        self.closeProcess()

    def doProcess(self, dat):
        if not self.prnt:
            self.printSetup()
            self.prnt = True
        col = self.sql.crsmst_col
        self.acno = CCD(dat[col.index("crm_acno")], "NA", self.width)
        self.name = CCD(dat[col.index("crm_name")], "TX", self.width)
        self.add1 = CCD(dat[col.index("crm_add1")], "TX", self.width)
        self.add2 = CCD(dat[col.index("crm_add2")], "TX", self.width)
        self.add3 = CCD(dat[col.index("crm_add3")], "TX", self.width)
        self.add4 = ""
        self.add5 = ""
        self.pcod = CCD(dat[col.index("crm_pcod")], "TX", self.width)
        self.loadStorage()
        if self.storagefull == "Y":
            self.pageHeading()
            self.printPage()
            self.c.drawText(self.t)
            self.c.showPage()
            self.clearStorage()

    def printSetup(self):
        self.c, cpi, inch, suc, duc = doReportlab(self.mf.prgdir, self.pdfnam)
        self.makeStorage()
        self.viewcnt = 0
        self.linecnt = 0
        if self.start == 1:
            self.hcount = 0
            self.vcount = 0
        else:
            self.hcount = ((self.start % self.across) + \
                (self.across - 1)) % self.across
            self.vcount = ((self.start + (self.across - 1)) / self.across) - 1

    def pageHeading(self):
        self.t = self.c.beginText(15, 20)
        self.t.setFont("Tartan", 10, 12.7)

    def printPage(self):
        self.t.textLine()
        spc = " " * self.width
        for x in range(0, self.down):
            txacno = ""
            txname = ""
            txadd1 = ""
            txadd2 = ""
            txadd3 = ""
            txadd4 = ""
            txadd5 = ""
            for y in range(0, self.across):
                txacno = txacno + self.xacno[x][y] + \
                        spc[0:self.width - len(self.xacno[x][y])]
                txname = txname + self.xname[x][y] + \
                        spc[0:self.width - len(self.xname[x][y])]
                txadd1 = txadd1 + self.xadd1[x][y] + \
                        spc[0:self.width - len(self.xadd1[x][y])]
                txadd2 = txadd2 + self.xadd2[x][y] + \
                        spc[0:self.width - len(self.xadd2[x][y])]
                txadd3 = txadd3 + self.xadd3[x][y] + \
                        spc[0:self.width - len(self.xadd3[x][y])]
                txadd4 = txadd4 + self.xadd4[x][y] + \
                        spc[0:self.width - len(self.xadd4[x][y])]
                txadd5 = txadd5 + self.xadd5[x][y] + \
                        spc[0:self.width - len(self.xadd5[x][y])]
            self.t.textLine(" %s" % txacno)
            self.t.textLine(" %s" % txname)
            self.t.textLine(" %s" % txadd1)
            self.t.textLine(" %s" % txadd2)
            self.t.textLine(" %s" % txadd3)
            self.t.textLine(" %s" % txadd4)
            self.t.textLine(" %s" % txadd5)
            self.t.textLine()
        self.clearStorage()
            
    def makeStorage(self):
        self.xacno = makeArray(self.across, self.down, 1, "S")
        self.xname = makeArray(self.across, self.down, 1, "S")
        self.xadd1 = makeArray(self.across, self.down, 1, "S")
        self.xadd2 = makeArray(self.across, self.down, 1, "S")
        self.xadd3 = makeArray(self.across, self.down, 1, "S")
        self.xadd4 = makeArray(self.across, self.down, 1, "S")
        self.xadd5 = makeArray(self.across, self.down, 1, "S")
        self.clearStorage()
            
    def clearStorage(self):
        for x in range(self.down):
            for y in range(self.across):
                self.xacno[x][y] = ""
                self.xname[x][y] = ""
                self.xadd1[x][y] = ""
                self.xadd2[x][y] = ""
                self.xadd3[x][y] = ""
                self.xadd4[x][y] = ""
                self.xadd5[x][y] = ""
        self.hcount = 0
        self.vcount = 0
        self.storagefull = "N"

    def loadStorage(self):
        self.xacno[self.vcount][self.hcount] = "%25s" % self.acno.work
        self.xname[self.vcount][self.hcount] = self.name.disp.strip()
        addr = []
        if self.add1.work:
            addr.append(self.add1.disp.strip())
        if self.add2.work:
            addr.append(self.add2.disp.strip())
        if self.add3.work:
            addr.append(self.add3.disp.strip())
        addr.append(self.pcod.disp.strip())
        for x in range(len(addr), 5):
            addr.append("")
        self.xadd1[self.vcount][self.hcount] = addr[0]
        self.xadd2[self.vcount][self.hcount] = addr[1]
        self.xadd3[self.vcount][self.hcount] = addr[2]
        self.xadd4[self.vcount][self.hcount] = addr[3]
        self.xadd5[self.vcount][self.hcount] = addr[4]
        if self.hcount == (self.across - 1) and self.vcount == (self.down - 1):
            self.storagefull = "Y"
        elif self.hcount != (self.across - 1):
            self.hcount = self.hcount + 1
        elif self.vcount != (self.down - 1):
            self.vcount = self.vcount + 1
            self.hcount = 0

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

