#! /usr/bin/env python

"""
SYNOPSIS
    Creditors Ledger Purchase History.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import Balances, CCD, CreateChart, ProgressBar, Sql
from TartanClasses import TartanDialog
from tartanFunctions import doReportlab, doWriteExport, getModName, doPrinter
from tartanFunctions import showError
from tartanWork import mthnam

class cr3110:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "crsmst", prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i) %6s" % (t[0],
            t[1], t[2], t[3], t[4], __name__)
        self.head = ("%03u %-30s %s" % (self.conum, self.conam, "%s"))
        self.forms = [("NA", 7), ("NA", 30)] + [("SL", 10)] * 12
        self.gtots = [0] * 12
        self.mchart = []
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Creditor's Purchase History (%s)" % __name__)
        r1s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),"ID2",7,"Ending Period","Ending Period",
                (self.sysdtw/100),"Y",self.doDat,None,None,("efld",)),
            (("T",0,1,0),("IRB",r1s),0,"Ignore Zero Purch","Ignore Zero Purch",
                "Y","Y",self.doZer,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"))

    def doDat(self, frt, pag, r, c, p, i, w):
        self.coffw = w
        self.coffd = self.df.t_disp[pag][r][p]

    def doZer(self, frt, pag, r, c, p, i, w):
        self.zer = w

    def doEnd(self):
        self.df.closeProcess()
        all = self.sql.getRec(tables="crsmst", cols=["crm_acno", "crm_name"],
            where=[("crm_cono", "=", self.conum)], order="crm_acno")
        if not all:
            showError(self.mf.body, "Transaction Error",
            "No Transactions Selected")
        else:
            self.start = self.coffw - 99
            if self.start % 100 == 13:
                self.start = self.start + 88
            s = self.start % 100
            self.mthhead = ""
            self.colsh = ["Acc-Num", "Name"]
            for x in range(12):
                if self.mthhead:
                    self.mthhead = "%s %9s " % (self.mthhead, mthnam[s][1])
                else:
                    self.mthhead = "%9s " % mthnam[s][1]
                self.colsh.append(mthnam[s][1])
                s += 1
                if s == 13:
                    s = 1
            if self.df.repprt[0][2] == "export":
                self.exportReport(all)
            else:
                self.printReport(all)
        self.closeProcess()

    def exportReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        expnam = getModName(self.mf.wrkdir, __name__, self.conum)
        self.expheads = [self.head % self.sysdttm]
        self.expheads.append("Creditor's Purchase History for the 12 Months "\
            "Period to %s" % self.coffd)
        self.expheads.append("Options: Ignore-Zeros %s" % self.zer)
        self.expcolsh = [self.colsh]
        self.expforms = self.forms
        self.expdatas = []
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                p.closeProgress()
                return
            vals = self.getValues(dat)
            if not vals:
                continue
            line = ["BODY", [vals[0].work, vals[1].work]]
            for val in vals[3]:
                line[1].append(val)
            self.expdatas.append(line)
        p.closeProgress()
        self.grandTotal()
        doWriteExport(xtype=self.df.repprt[0][1], name=expnam, sheet=__name__,
            heads=self.expheads, colsh=self.expcolsh, forms=self.expforms,
            datas=self.expdatas, rcdic=self.mf.rcdic)

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        chrs = len(self.colsh)
        for f in self.forms:
            chrs += int(f[1])
        if f[0][0].lower() == "s":
            chrs -= 2
        else:
            chrs -= 1
        pad = chrs - 35 - len(self.sysdttm)
        self.head1 = self.head % (" " * pad + self.sysdttm)
        self.head2 = "Creditor's Purchase History for the 12 Months "\
            "Period to %s%s" % (self.coffd, "%s%s")
        pad = chrs - len(self.head2) + 4 - 11 # %s%s and ' Page     1'
        self.head2 = self.head2 % (" " * pad, " Page %5s")
        pdfnam = getModName(self.mf.wrkdir, __name__, self.conum, ext="pdf")
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, pdfnam, self.head1)
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            vals = self.getValues(dat)
            if not vals:
                continue
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            self.t.textLine("%s %s %s" % (vals[0].disp, vals[1].disp, vals[2]))
            self.pglin = self.pglin + 1
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
        if self.df.repprt[0][1] == "X":
            return
        CreateChart(self.mf, self.conum, self.conam, [self.start, self.coffw],
            [self.tit, "Values"], None, self.mchart, ylab="Values")

    def getValues(self, data):
        acc = CCD(data[0], "NA", 7)
        name = CCD(data[1], "NA", 30)
        bals = Balances(self.mf, "CRS", self.conum, self.coffw, (acc.work,))
        this, hist = bals.doCrsDrsHist()
        if not this:
            return
        purchd = ""
        purchw = []
        purtot = 0
        for x in range(11, -1, -1):
            amt = CCD(round(hist[0][x], 0), "SL", 10)
            self.gtots[x] = self.gtots[x] + amt.work
            purtot = purtot + amt.work
            if purchd:
                purchd = "%s %9s" % (purchd, amt.disp)
            else:
                purchd = "%9s" % amt.disp
            purchw.append(amt.work)
        if self.zer == "Y" and purtot == 0:
            return
        return acc, name, purchd, purchw
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head1)
        self.t.textLine()
        self.t.textLine(self.head2 % self.pgnum)
        self.t.textLine()
        self.t.textLine("(%-23s%1s)" % ("Options: Ignore Zeros:-", self.zer))
        self.t.textLine()
        self.t.textLine("%-7s %-30s %s" % ("Acc-Num", "Name", self.mthhead))
        self.t.textLine("%s" % (self.suc * len(self.head1)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def grandTotal(self):
        if self.df.repprt[0][2] == "export":
            self.expdatas.append(["ULINES"])
            self.expdatas.append(["TOTAL", ["", "Grand Totals"] + self.gtots])
            self.expdatas.append(["ULINED"])
            return
        purchd = ""
        mchart = ["T", "Grand Totals"]
        for x in range(11,-1,-1):
            a = CCD(self.gtots[x], "SL", 10)
            if purchd:
                purchd = "%s %s" % (purchd, a.disp)
            else:
                purchd = "%s" % a.disp
            mchart.append(a.work)
        self.mchart.append(mchart)
        self.t.textLine("%s" % (self.suc * len(self.head1)))
        self.t.textLine("%-7s %-30s %s" % (" ", "Grand Totals", purchd))

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

