#! /usr/bin/env python

"""
SYNOPSIS
    Creditors Interrogation.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, NotesCreate, RepPrt, SChoice
from TartanClasses import SelectChoice, SplashScreen, Sql, TabPrt
from TartanClasses import TartanDialog
from tartanFunctions import getTrn
from tartanWork import crtrtp, mthnam

"""
30/60 days etc is done by crt_curdt and cra_curdt
last purchase date is done by last crt_trdt and crt_type = 1
last payment date is done by last crt_trdt and crt_type = 5
"""

class cr4010:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.capnm = usr
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["crsmst", "crstrn"], prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) +  (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.curdt = self.sysdtw / 100
        self.paidup = "N"

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
                "Creditors Interrogation (%s)" % __name__)
        crm = {
            "stype": "R",
            "tables": ("crsmst",),
            "cols": (
                ("crm_acno", "", 0, "Acc-Num"),
                ("crm_name", "", 0, "Name", "Y")),
            "where": [("crm_cono", "=", self.conum)]}
        tag = (
            ("Basic-_1", self.doTagSelect, ("T",0,2), ("T",0,1)),
            ("Basic-_2", self.doTagSelect, ("T",0,2), ("T",0,1)),
            ("Balances", self.doTagSelect, ("T",0,2), ("T",0,1)),
            ("History", self.doTagSelect, ("T",0,2), ("T",0,1)),
            ("Trans", self.doTrans1, ("T",0,2), ("T",0,1)))
        r1s = (("No","N"),("Yes","Y"))
        fld = (
            (("T",0,0,0),"INA",7,"Acc-Num","Account Number",
                "","N",self.doAccNum,crm,None,None),
            (("T",0,0,0),"ONA",30,"Name"),
            (("T",1,0,0),"ONA",30,"Address Line 1"),
            (("T",1,1,0),"ONA",30,"Address Line 2"),
            (("T",1,2,0),"ONA",30,"Address Line 3"),
            (("T",1,3,0),"ONA",4,"Postal Code"),
            (("T",1,4,0),"ONA",20,"Telephone"),
            (("T",1,5,0),"ONA",20,"Fax"),
            (("T",1,6,0),"ONA",30,"Manager"),
            (("T",1,6,0),"OTX",30,"E-Mail"),
            (("T",1,7,0),"ONA",30,"Accounts"),
            (("T",1,7,0),"OTX",30,"E-Mail"),
            (("T",1,8,0),"ONA",30,"Orders"),
            (("T",1,8,0),"OTX",30,"E-Mail"),
            (("T",1,9,0),"Od1",10,"Date Opened"),
            (("T",1,10,0),"ONA",10,"V.A.T. Number"),
            (("T",2,0,0),"OUA",1,"Terms Base"),
            (("T",2,1,0),"OUI",2,"Statement Day"),
            (("T",2,2,0),"OUI",3,"Terms"),
            (("T",2,3,0),"OUI",5,"Credit Limit"),
            (("T",2,4,0),"OUD",5.2,"Trade Discount"),
            (("T",2,5,0),"OUD",5.2,"Settlement Discount"),
            (("T",2,6,0),"OUA",1,"Payment Indicator"),
            (("T",2,7,0),"OUA",20,"Bank Name"),
            (("T",2,8,0),"OUI",8,"Bank Branch"),
            (("T",2,9,0),"OUA",16,"Bank Account"),
            (("T",2,10,0),"OUI",7,"G/L Account"),
            (("T",3,0,0),"Od1",10,"Date Last Purchased"),
            (("T",3,1,0),"Od1",10,"Date Last Paid"),
            (("T",3,2,0),"OSD",12.2,"Total Balance"),
            (("T",3,3,0),"OSD",12.2,"Current"),
            (("T",3,4,0),"OSD",12.2,"30 Days"),
            (("T",3,5,0),"OSD",12.2,"60 Days"),
            (("T",3,6,0),"OSD",12.2,"90 Days"),
            (("T",3,7,0),"OSD",12.2,"120 Days"),
            (("C",4,0,0),"OSD",13.2,"Purchases","","","N",
                None,None,None,None,("Months",13)),
            (("C",4,0,1),"OSD",13.2,"Payments"),
            (("T",5,0,0),("IRB",r1s),0,"History", "History",
                "N","N",self.doTrans2,None,None,None))
        but = (
            ("Clear",None,self.doClear,0,("T",0,0),("T",0,1)),
            ("Notes",None,self.doNotes,0,("T",0,0),("T",0,1)),
            ("Print",None,self.doPrint,0,("T",0,0),("T",0,1)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEndTop, "N"), None, None, None, None, None)
        txt = (self.doExit, None, None, None, None, self.doExit)
        self.df = TartanDialog(self.mf, title=self.tit, tags=tag, eflds=fld,
            butt=but, tend=tnd, txit=txt)
        yer = self.sysdtw / 10000
        mon = (self.sysdtw % 10000) / 100
        self.hist_tit = []
        for x in range(1, 14):
            if x == 13:
                txt = "Last 12 Month Total"
                self.df.colLabel[4][x-1].set_text(txt)
                self.hist_tit.append(txt)
            else:
                nam = mthnam[mon][1]
                nam = nam + (" " * (15-len(nam))) + str(yer)
                self.df.colLabel[4][x-1].set_text(nam)
                self.hist_tit.append(nam)
            if x != 13:
                mon = mon - 1
                if mon == 0:
                    mon = mon + 12
                    yer = yer - 1

    def doAccNum(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="crsmst", where=[("crm_cono", "=",
            self.conum), ("crm_acno", "=", w)], limit=1)
        if not acc:
            return "Invalid Account Number"
        self.acno = w
        self.name = acc[self.sql.crsmst_col.index("crm_name")]
        self.df.loadEntry("T",0, 1, data=self.name)
        seq = 3
        for x in range(0, self.df.topq[1]):
            self.df.loadEntry("T", 1, x, data=acc[seq])
            seq += 1
        for x in range(0, self.df.topq[2]):
            self.df.loadEntry("T", 2, x, data=acc[seq])
            seq += 1
        self.loadBalances()
        self.mf.updateStatus("")

    def doEndTop(self):
        self.df.last[0] = [0, 0]
        self.df.setWidget(self.df.topEntry[0][0], state="disabled")

    def loadBalances(self):
        bals = Balances(self.mf, "CRS", self.conum, (self.sysdtw/100),
            (self.acno,))
        obal, tbal, ages = bals.doAllBals()
        this, hist = bals.doCrsDrsHist()
        self.purch = hist[0]
        self.pays = hist[1]
        last = self.sql.getRec(tables="crstrn", cols=["max(crt_trdt)"],
            where=[("crt_cono", "=", self.conum), ("crt_acno", "=",
            self.acno), ("crt_type", "=", 1)], limit=1)
        if not last or not last[0]:
            lastpurd = 0
        else:
            lastpurd = last[0]
        last = self.sql.getRec(tables="crstrn", cols=["max(crt_trdt)"],
            where=[("crt_cono", "=", self.conum), ("crt_acno", "=",
            self.acno), ("crt_type", "=", 5)], limit=1)
        if not last or not last[0]:
            lastpayd = 0
        else:
            lastpayd = last[0]
        for x in range (0, 8):
            if x == 0:
                data=lastpurd
            elif x == 1:
                data=lastpayd
            elif x == 2:
                data=tbal
            else:
                data=ages[x-3]
            self.df.loadEntry("T", 3, x, data=data)
        p = 0
        for x in range (0, 13):
            i = 0
            self.df.loadEntry("C", 4, p, data=self.purch[x])
            p = p + 1
            i = i + 1
            pay = float(ASD(0) - ASD(self.pays[x]))
            self.df.loadEntry("C", 4, p, data=pay)
            p = p + 1
            i = i + 1

    def doTagSelect(self):
        self.mf.updateStatus("")

    def doTrans1(self, *args):
        self.df.focusField("T", 5, 1)

    def doTrans2(self, frt, pag, r, c, p, i, w):
        self.paidup = w
        whr = [("crt_cono", "=", self.conum), ("crt_acno", "=", self.acno)]
        if self.paidup == "Y":
            col, all = getTrn(self.mf.dbm, "crs", whr=whr)
        else:
            col, all = getTrn(self.mf.dbm, "crs", dte=self.curdt, whr=whr)
        if all:
            data = []
            for dat in all:
                data.append([
                    dat[col.index("crt_trdt")],
                    dat[col.index("crt_curdt")],
                    dat[col.index("crt_batch")],
                    dat[col.index("crt_type")],
                    dat[col.index("crt_ref1")],
                    dat[col.index("crt_tramt")],
                    dat[col.index("paid")],
                    dat[col.index("balance")],
                    dat[col.index("crt_desc")]])
            tit = "Transactions for Account: %s - %s" % (self.acno, self.name)
            col = (
                ("crt_trdt", "   Date", 10, "D1", "N"),
                ("crt_curdt", "Curr-Dt", 7, "D2", "N"),
                ("crt_batch", "Batch", 7, "Na", "N"),
                ("crt_type", "Typ", 3, ("XX", crtrtp), "N"),
                ("crt_ref1", "Reference", 9, "Na", "Y"),
                ("crt_tramt", "    Amount", 11.2, "SD", "N"),
                ("alloc", " Allocated", 11.2, "SD", "N"),
                ("balan", "   Balance", 11.2, "SD", "N"),
                ("crt_desc", "Details", 30, "NA", "N"))
            state = self.df.disableButtons()
            while True:
                rec = SelectChoice(self.df.nb.Page4, tit, col, data)
                # Display all transaction details
                if rec.selection:
                    self.df.mstAlw.hide()
                    sp = SplashScreen(self.df.nb.Page5,
                        "Displaying Document.... Please Wait....")
                    whr = [
                        ("crt_cono", "=", self.conum),
                        ("crt_acno", "=", self.acno),
                        ("crt_type", "=", rec.selection[4]),
                        ("crt_ref1", "=", rec.selection[5])]
                    TabPrt(self.mf, tabs="crstrn", wher=whr, pdia=False)
                    sp.closeSplash()
                    self.df.mstAlw.show_all()
                else:
                    break
            self.df.enableButtons(state=state)
        self.doTrans1()

    def doNotes(self, widget=None):
        state = self.df.disableButtons()
        self.df.mstAlw.hide()
        nf = NotesCreate(self.mf, self.conum, self.conam, self.capnm, "CRS",
            self.acno)
        self.df.mstAlw.show()
        self.df.enableButtons(state=state)

    def doClear(self, widget=None):
        self.df.selPage("Basic-_1")
        self.df.focusField("T", 0, 1)

    def doPrint(self, widget=None):
        titl = "Select the required print options"
        data = (
            ("1", "Information"),
            ("2", "Transactions"),
            ("3", "Both"))
        ss = SChoice(self.mf, titl=titl, data=data)
        self.mf.updateStatus("")
        if ss.selection:
            state = self.df.disableButtons()
            self.df.mstAlw.hide()
            if ss.selection[0] in ("1", "3"):
                table = "crsmst"
                whr = [("crm_cono", "=", self.conum),
                    ("crm_acno", "=", self.acno)]
                rp = TabPrt(self.mf, self.conum, self.conam, name=__name__,
                    tabs=table, wher=whr, keys=[self.acno])
                view = rp.view
                mail = rp.mail
                exit = rp.exit
            else:
                view = None
                mail = None
                exit = "n"
            if ss.selection[0] in ("2", "3") and exit == "n":
                heads = ["Creditor's Transactions",
                    "Account: %s  Name: %s" % (self.acno, self.name)]
                whr = [("crt_cono", "=", self.conum),
                    ("crt_acno", "=", self.acno)]
                if self.paidup == "Y":
                    col, all = getTrn(self.mf.dbm, "crs", whr=whr)
                else:
                    col, all = getTrn(self.mf.dbm, "crs", dte=self.curdt,
                        whr=whr)
                cols = []
                data = []
                dic = self.sql.crstrn_dic
                for num, rec in enumerate(all):
                    dat = []
                    for nam in [
                        "crt_ref1",
                        "crt_trdt",
                        "crt_type",
                        "crt_tramt",
                        "paid",
                        "balance",
                        "crt_desc"]:
                        if not num:
                            if nam == "paid":
                                cols.append(["paid", "SD", 11.2, "Paid"])
                            elif nam == "balance":
                                cols.append(["balance", "SD", 11.2, "Balance"])
                            else:
                                cols.append([nam, dic[nam][2], dic[nam][3],
                                    dic[nam][5]])
                        dat.append(rec[col.index(nam)])
                    data.append(dat)
                gtots=["crt_tramt", "paid", "balance"]
                if view:
                    prtdia = False
                else:
                    prtdia = ("Y","V")
                rp = RepPrt(self.mf, conum=self.conum, conam=self.conam,
                    name=__name__, ttype="D", tables=data, heads=heads,
                    cols=cols, trtp=["crt_type", crtrtp], gtots=gtots,
                    prtdia=prtdia, view=view, mail=mail)
            self.df.mstAlw.show()
            self.mf.gtkDispatch()
            self.df.enableButtons(state=state)
            self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

