#! /usr/bin/env python

"""
SYNOPSIS
    Creditors Control File Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import Sql, TartanDialog

class crc110:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.user = usr
        no = self.setVariables()
        if not no:
            self.drawDialog()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["crsctl", "chglog"], prog=__name__)
        if self.sql.error:
            return "no"
        self.acc = self.sql.getRec(tables="crsctl", where=[("ctc_cono", "=",
            self.conum)], limit=1)
        if not self.acc:
            self.new = True
            self.acc = [self.conum, "N", "E", "", 0, 0]
        else:
            self.new = False

    def drawDialog(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Creditors Control File Maintenance (%s)" % __name__)
        ctl = {
            "stype": "R",
            "tables": ("ctlctl", "genmst"),
            "cols": (
                ("ctl_code", "", 0, "Ctl-Code"),
                ("ctl_conacc", "", 0, "G/L-Num"),
                ("glm_desc", "", 30, "Description")),
            "where": [("ctl_cono", "=", self.conum), ("ctl_code", "like",
                "bank_%"), ("glm_cono=ctl_cono",), ("glm_acno=ctl_conacc",)],
            "index": 1}
        r1s = (("Yes","Y"),("No","N"))
        r2s = (("Exclusive","E"),("Inclusive","I"))
        self.fld = (
            (("T",0,0,0),("IRB",r1s),0,"G/L Integration","G/L Integration",
                self.acc[1],"N",self.doGlint,None,None,None),
            (("T",0,1,0),("IRB",r2s),0,"G/L Input Method","G/L Input Method",
                self.acc[2],"N",None,None,None,None))
            #(("T",0,2,0),"INA",4,"Best Account Code","Best",
            #    self.acc[3],"N",self.doBest,None,None,("efld",)),
            #(("T",0,3,0),"IUI",2,"Best Account Type","BT",
            #    self.acc[4],"N",self.doBest,None,None,("efld",)),
            #(("T",0,4,0),"IUI",7,"Bank Account Code","Bank-Ac",
            #    self.acc[5],"N",self.doBank,ctl,None,("efld",)))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,1),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)
        if not self.new:
            for n, f in enumerate(self.acc[1:3]):
                self.df.loadEntry("T", 0, n, data=f)

    def doGlint(self, frt, pag, r, c, p, i, w):
        self.glint = w
        if self.glint == "N":
            self.df.loadEntry(frt, pag, p+1, data="E")
            return "sk1"

    def doBest(self, frt, pag, r, c, p, i, w):
        if not w or self.glint == "N":
            self.df.loadEntry(frt, pag, p+1, data=0)
            return "nd"

    def doBank(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="genmst", cols=["glm_desc"],
            where=[("glm_cono", "=", self.conum), ("glm_acno", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Bank Account"

    def doEnd(self):
        data = [self.conum]
        for x in range(0, len(self.df.t_work[0][0])):
            data.append(self.df.t_work[0][0][x])
        data.extend(["", 0, 0])
        if self.new:
            self.sql.insRec("crsctl", data=data)
        elif data != self.acc:
            self.sql.updRec("crsctl", data=data, where=[("ctc_cono", "=",
                self.conum)])
            dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
            for num, dat in enumerate(self.acc):
                if dat != data[num]:
                    self.sql.insRec("chglog", data=["crsctl", "U",
                        "%03i" % self.conum, self.sql.crsctl_col[num],
                        dte, self.user, str(dat), str(data[num]), 0])
        self.mf.dbm.commitDbase()
        self.doExit()

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()
