#! /usr/bin/env python

"""
SYNOPSIS
    Debtors Ledger Master Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError

class dr3060:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["drsact", "drstyp", "drsmst"],
            prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Debtors Master Listing (%s)" % __name__)
        daa = self.sql.getRec(tables="drsmst", cols=["drm_bus_activity",
            "count(*)"], where=[("drm_cono", "=", self.conum)],
            group="drm_bus_activity", order="drm_bus_activity")
        act = {
            "stype": "C",
            "titl": "Valid Activities",
            "head": ("Act", "Quantity"),
            "data": daa,
            "typs": (("NA", 3), ("UI", 8)),
            "size": (400, 400)}
        dab = self.sql.getRec(tables="drsmst", cols=["drm_bus_type",
            "count(*)"], where=[("drm_cono", "=", self.conum)],
            group="drm_bus_type", order="drm_bus_type")
        typ = {
            "stype": "C",
            "titl": "Valid Types",
            "head": ("Typ", "Quantity"),
            "data": dab,
            "typs": (("NA", 3), ("UI", 8)),
            "size": (400, 400)}
        r1s = (("Number","N"),("Name","M"))
        r2s = (("Yes","Y"),("No","N"))
        r3s = (("Manager","M"),("Accounts","A"),("Sales","S"))
        fld = (
            (("T",0,0,0),"INA",3,"Activity","Account Activity",
                "","N",self.doAct,act,None,None),
            (("T",0,0,0),"ONA",30,""),
            (("T",0,1,0),"INA",3,"Type","Account Type",
                "","N",self.doTyp,typ,None,None),
            (("T",0,1,0),"ONA",30,""),
            (("T",0,2,0),("IRB",r1s),0,"Sort Order","Sort Order",
                "N","Y",self.doSort,None,None,None),
            (("T",0,3,0),("IRB",r2s),0,"Full Address","Full Address",
                "N","Y",self.doAddress,None,None,None),
            (("T",0,4,0),("IRB",r3s),0,"Contact Details","Contact Details",
                "M","Y",self.doContact,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doAct(self, frt, pag, r, c, p, i, w):
        self.act = w
        if not self.act:
            self.df.loadEntry(frt, pag, p+1, "All Activities")
        else:
            chk = self.sql.getRec(tables="drsact", where=[("dac_code", "=",
                self.act)], limit=1)
            if not chk:
                return "Invalid Activity Code"
            self.df.loadEntry(frt, pag, p+1, chk[1])

    def doTyp(self, frt, pag, r, c, p, i, w):
        self.typ = w
        if not self.typ:
            self.df.loadEntry(frt, pag, p+1, "All Types")
        else:
            chk = self.sql.getRec(tables="drstyp", where=[("dtp_code", "=",
                self.typ)], limit=1)
            if not chk:
                return "Invalid Type Code"
            self.df.loadEntry(frt, pag, p+1, chk[1])

    def doSort(self, frt, pag, r, c, p, i, w):
        self.sort = w

    def doAddress(self, frt, pag, r, c, p, i, w):
        self.address = w

    def doContact(self, frt, pag, r, c, p, i, w):
        self.condet = w

    def doEnd(self):
        self.df.closeProcess()
        whr = [("drm_cono", "=", self.conum)]
        if self.act:
            whr.append(("drm_bus_activity", "=", self.act))
        if self.typ:
            whr.append(("drm_bus_type", "=", self.typ))
        if self.sort == "N":
            ord = "drm_chain, drm_acno"
        else:
            ord = "drm_name"
        all = self.sql.getRec(tables="drsmst", where=whr, order=ord)
        if not all:
            showError(self.mf.body, "Selection Error",
            "No Records Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %145s %6s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        pdfnam = getModName(self.mf.wrkdir, __name__, self.conum, ext="pdf")
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        col = self.sql.drsmst_col
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            chn = CCD(dat[col.index("drm_chain")], "UI", 3)
            acno = CCD(dat[col.index("drm_acno")], "NA", 7)
            name = CCD(dat[col.index("drm_name")], "NA", 30)
            add1 = CCD(dat[col.index("drm_add1")], "NA", 30)
            add2 = CCD(dat[col.index("drm_add2")], "NA", 30)
            add3 = CCD(dat[col.index("drm_add3")], "NA", 30)
            pcod = CCD(dat[col.index("drm_pcod")], "NA", 4)
            tel = CCD(dat[col.index("drm_tel")], "NA", 12)
            fax = CCD(dat[col.index("drm_fax")], "NA", 12)
            if self.condet == "M":
                contact = dat[col.index("drm_mgr")]
                email = dat[col.index("drm_mgr_email")].split(",")
            elif self.condet == "A":
                contact = dat[col.index("drm_acc")]
                email = dat[col.index("drm_acc_email")].split(",")
            else:
                contact = dat[col.index("drm_sls")]
                email = dat[col.index("drm_sls_email")].split(",")
            nameml = CCD("%s <%s>" % (contact, email[0]), "NA", 40)
            delv = CCD(dat[col.index("drm_delivery")], "Na", 7)
            area = CCD(dat[col.index("drm_area")], "NA", 3)
            rep = CCD(dat[col.index("drm_rep")], "Na", 3)
            bact = CCD(dat[col.index("drm_bus_activity")], "UA", 3)
            btyp = CCD(dat[col.index("drm_bus_type")], "UA", 3)
            prices = CCD(dat[col.index("drm_prices")], "UI", 1)
            rfterm = CCD(dat[col.index("drm_rfterms")], "UI", 3)
            rjterm = CCD(dat[col.index("drm_rjterms")], "UI", 3)
            limit = CCD(dat[col.index("drm_limit")], "UI", 7)
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            self.t.textLine("%s %s %s %s %s %s %s %s %s %s %s %s %s "\
                "%s %s %s %s" % (chn.disp, acno.disp, name.disp,
                add1.disp, pcod.disp, tel.disp, fax.disp, nameml.disp,
                delv.disp, area.disp, rep.disp, bact.disp,
                btyp.disp, prices.disp, rfterm.disp, rjterm.disp,
                limit.disp))
            if self.address == "Y" and add2.work:
                self.t.textLine("%42s %-30s" % ("", add2.disp))
                self.pglin = self.pglin + 1
            if self.address == "Y" and add3.work:
                self.t.textLine("%42s %-30s" % ("", add3.disp))
                self.pglin = self.pglin + 1
            self.pglin = self.pglin + 1
        p.closeProgress()
        if self.t:
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])

    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-32s %-10s %137s %5s" % \
            ("Debtors Ledger Master List as at", self.sysdtd,
                "Page", self.pgnum))
        self.t.textLine()
        self.t.textLine("(%-15s%1s)" % ("Options: Sort-",
            self.df.t_disp[0][0][0]))
        self.t.textLine()
        if self.condet == "M":
            det = "Manager's Contact Details"
        elif self.condet == "A":
            det = "Accounts Contact Details"
        else:
            det = "Orders Contact Details"
        self.t.textLine("%-3s %-7s %-30s %-30s %-4s %-12s %-12s %-40s %-7s "\
            "%-3s %-3s %-3s %-3s %1s %-3s %-3s %-7s" % ("Chn", "Acc-Num",
            "Name", "Address", "Code", "Tel-Number", "Fax-Number", "%s" % det,
            "Deliver", "Are", "Rep", "Act", "Typ", "P", "Rft","Rjt","C-Limit"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

