#! /usr/bin/env python

"""
SYNOPSIS
    Debtors Interrogation.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, GetCtl, NotesCreate, PrintInvoice
from TartanClasses import RepPrt, SChoice, Sql, SelectChoice, SplashScreen
from TartanClasses import TabPrt, TartanDialog
from tartanFunctions import getTrn
from tartanWork import drtrtp, mthnam

class dr4010:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.capnm = usr
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        tables = [
            "ctlare", "ctlrep", "ctlmes",
            "drsact", "drschn", "drsmst", "drstrn", "drstyp",
            "slsiv1"]
        self.sql = Sql(self.mf.dbm, tables, prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) +  (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.curdt = self.sysdtw / 100
        gc = GetCtl(self.mf)
        drsctl = gc.getCtl("drsctl", self.conum)
        if not drsctl:
            return "no"
        self.chains = drsctl["ctd_chain"]
        self.paidup = "N"

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
                "Debtors Interrogation (%s)" % __name__)
        drc = {
            "stype": "R",
            "tables": ("drschn",),
            "cols": (
                ("chm_chain", "", 0, "Num"),
                ("chm_name", "", 0, "Name", "Y")),
            "where": [("chm_cono", "=", self.conum)]}
        drm = {
            "stype": "R",
            "tables": ("drsmst",),
            "cols": (
                ("drm_acno", "", 0, "Acc-Num"),
                ("drm_name", "", 0, "Name", "Y"),
                ("drm_add1", "", 0, "Address Line 1"))}
        if self.chains == "Y":
            drm["where"] = [("drm_cono", "=", self.conum)]
            drm["whera"] = [["T", "drm_chain", 0]]
        else:
            drm["where"] = [
            ("drm_cono", "=", self.conum),
            ("drm_chain", "=", 0)]
        tag = (
            ("Basic-_1", self.doTagSelect, ("T",0,0), ("T",0,1)),
            ("Basic-_2", self.doTagSelect, ("T",0,0), ("T",0,1)),
            ("Balances", self.doTagSelect, ("T",0,0), ("T",0,1)),
            ("History", self.doTagSelect, ("T",0,0), ("T",0,1)),
            ("Trans", self.doTrans1, ("T",0,0), ("T",0,1)))
        r1s = (("No","N"),("Yes","Y"))
        fld = [
            [("T",0,0,0),"IUI",3,"Chain Store","Chain Store",
                "","N",self.doChain,drc,None,("efld",)],
            [("T",0,0,0),"INA",7,"Acc-Num","Account Number",
                "","N",self.doAcno,drm,None,("notblank",)],
            [("T",0,0,0),"ONA",30,"Name"],
            [("T",0,0,0),"INA",7,"Acc-Num","Account Number",
                "","N",self.doAcno,drm,None,("notblank",)],
            [("T",0,0,0),"ONA",30,"Name"],
            (("T",1,0,0),"ONA",30,"Name"),
            (("T",1,1,0),"ONA",30,"Address Line 1"),
            (("T",1,2,0),"ONA",30,"Address Line 2"),
            (("T",1,3,0),"ONA",30,"Address Line 3"),
            (("T",1,4,0),"ONA",4,"Postal Code"),
            (("T",1,5,0),"ONA",20,"Telephone"),
            (("T",1,6,0),"ONA",20,"Fax"),
            (("T",1,7,0),"ONA",30,"Manager"),
            (("T",1,7,0),"OTX",40,"E-Mail"),
            (("T",1,8,0),"ONA",30,"Accounts Contact"),
            (("T",1,8,0),"OTX",40,"E-Mail"),
            (("T",1,9,0),"ONA",30,"Sales Contact"),
            (("T",1,9,0),"OTX",40,"E-Mail"),
            (("T",1,10,0),"Od1",10,"Date Opened"),
            (("T",1,11,0),"Od1",10,"Date Registered"),
            (("T",1,12,0),"ONA",10,"V.A.T Number"),
            (("T",1,13,0),"ONA",7,"Delivery Code"),
            (("T",1,14,0),"ONA",3,"Area"),
            (("T",1,14,20),"ONA",26,""),
            (("T",1,15,0),"ONa",3,"Rep Code"),
            (("T",1,15,0),"ONA",26,""),
            (("T",1,16,0),"ONA",3,"Business Activity"),
            (("T",1,16,0),"ONA",26,""),
            (("T",1,17,0),"ONA",3,"Business Type"),
            (("T",1,17,0),"ONA",26,""),
            (("T",2,0,0),"OUI",1,"Price Level"),
            (("T",2,1,0),"OUD",5.2,"Discount Percentage"),
            (("T",2,2,0),"OUD",5.2,"Interest Percentage"),
            (("T",2,3,0),"OUI",3,"Referral Terms"),
            (("T",2,4,0),"OUI",3,"Rejected Terms"),
            (("T",2,5,0),"OUI",5,"Credit Limit"),
            (("T",2,6,0),"OUA",1,"Stop Indicator"),
            (("T",2,7,0),"OUI",3,"Invoice Message"),
            (("T",2,7,0),"ONA",40,""),
            (("T",2,8,0),"OUI",3,"Statement Message"),
            (("T",2,8,0),"ONA",40,""),
            (("T",2,9,0),"OUA",1,"Credit Rating"),
            (("T",3,0,0),"Od1",10,"Date Last Sale"),
            (("T",3,1,0),"Od1",10,"Date Last Paid"),
            (("T",3,2,0),"OSD",12.2,"Total Balance"),
            (("T",3,3,0),"OSD",12.2,"Current"),
            (("T",3,4,0),"OSD",12.2,"30 Days"),
            (("T",3,5,0),"OSD",12.2,"60 Days"),
            (("T",3,6,0),"OSD",12.2,"90 Days"),
            (("T",3,7,0),"OSD",12.2,"120 Days"),
            (("C",4,0,0),"OSD",13.2,"Sales","","","N",
                None,None,None,None,("Months",13)),
            (("C",4,0,1),"OSD",13.2,"Receipts"),
            (("T",5,0,0),("IRB",r1s),1,"History","History",
                "N","N",self.doTrans2,None,None,None,None)]
        but = (
            ("Clear",None,self.doClear,0,("T",0,0),("T",0,1)),
            ("Notes",None,self.doNotes,0,("T",0,0),("T",0,1)),
            ("Print",None,self.doPrint,0,("T",0,0),("T",0,1)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEndTop, "N"), None, None, None, None, None)
        txt = (self.doExit, None, None, None, None, self.doExit)
        if self.chains == "Y":
            del fld[3]
            del fld[3]
        else:
            del fld[0]
            del fld[0]
            del fld[0]
            self.chain = 0
        self.df = TartanDialog(self.mf, title=self.tit, screen=self.mf.body,
            tags=tag, eflds=fld, butt=but, tend=tnd, txit=txt)
        yer = self.sysdtw / 10000
        mon = (self.sysdtw % 10000) / 100
        self.hist_tit = []
        for x in range(1, 14):
            if x == 13:
                txt = "Last 12 Month Total"
                self.df.colLabel[4][x-1].set_text(txt)
                self.hist_tit.append(txt)
            else:
                nam = mthnam[mon][1]
                nam = nam + (" " * (15-len(nam))) + str(yer)
                self.df.colLabel[4][x-1].set_text(nam)
                self.hist_tit.append(nam)
            if x != 13:
                mon = mon - 1
                if mon == 0:
                    mon = mon + 12
                    yer = yer - 1

    def doChain(self, frt, pag, r, c, p, i, w):
        if w:
            acc = self.sql.getRec(tables="drschn", where=[("chm_cono", "=",
                self.conum), ("chm_chain", "=", w)], limit=1)
            if not acc:
                return "Invalid Chain Store"
        self.chain = w

    def doAcno(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="drsmst", where=[("drm_cono", "=",
            self.conum), ("drm_chain", "=", self.chain), ("drm_acno", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Account Number"
        self.acno = w
        self.name = acc[self.sql.drsmst_col.index("drm_name")]
        self.df.loadEntry("T",pag,p+1,data=acc[3])
        d = 3
        for pag in range(1, 3):
            for x in range(0, self.df.topq[pag]):
                if pag == 1 and x in (18, 20, 22, 24):
                    continue
                elif pag == 2 and x in (8, 10):
                    continue
                elif pag == 2 and x == 11 and not acc[d]:
                    data = "N"
                else:
                    data = acc[d]
                self.df.loadEntry("T", pag, x, data=data)
                d = d + 1
        self.loadArea()
        self.loadRep()
        self.loadAct()
        self.loadTyp()
        self.loadMes()
        self.loadBalances()
        self.mf.updateStatus("")

    def doEndTop(self):
        self.df.last[0] = [0, 0]
        self.df.setWidget(self.df.topEntry[0][0], state="disabled")

    def loadArea(self):
        acc = self.sql.getRec(tables="ctlare", where=[("arm_cono", "=",
            self.conum), ("arm_area", "=", self.df.t_work[1][0][17])],
            cols=["arm_name"], limit=1)
        if acc:
            self.df.loadEntry("T", 1, 18, data=acc[0])

    def loadRep(self):
        acc = self.sql.getRec(tables="ctlrep", where=[("rpm_cono", "=",
            self.conum), ("rpm_rep", "=", self.df.t_work[1][0][19])],
            cols=["rpm_name"], limit=1)
        if acc:
            self.df.loadEntry("T", 1, 20, data=acc[0])

    def loadAct(self):
        acc = self.sql.getRec(tables="drsact", where=[("dac_code", "=",
            self.df.t_work[1][0][21])], cols=["dac_desc"], limit=1)
        if acc:
            self.df.loadEntry("T", 1, 22, data=acc[0])

    def loadTyp(self):
        acc = self.sql.getRec(tables="drstyp", where=[("dtp_code", "=",
            self.df.t_work[1][0][23])], cols=["dtp_desc"], limit=1)
        if acc:
            self.df.loadEntry("T", 1, 24, data=acc[0])

    def loadMes(self):
        acc = self.sql.getRec(tables="ctlmes", where=[("mss_system", "=",
            "INV"), ("mss_message", "=", self.df.t_work[2][0][7])],
            cols=["mss_detail"], limit=1)
        if acc:
            self.df.loadEntry("T", 2, 8, data=acc[0])
        acc = self.sql.getRec(tables="ctlmes", where=[("mss_system", "=",
            "STA"), ("mss_message", "=", self.df.t_work[2][0][9])],
            cols=["mss_detail"], limit=1)
        if acc:
            self.df.loadEntry("T", 2, 10, data=acc[0])

    def loadBalances(self):
        bals = Balances(self.mf, "DRS", self.conum, (self.sysdtw/100),
            (self.chain, self.acno))
        obal, tbal, ages = bals.doAllBals()
        this, hist = bals.doCrsDrsHist()
        self.sale = hist[0]
        self.pays = hist[1]
        last = self.sql.getRec(tables="drstrn", cols=["max(drt_trdt)"],
            where=[("drt_cono", "=", self.conum), ("drt_chain", "=",
            self.chain), ("drt_acno", "=", self.acno), ("drt_type", "=", 1)],
            limit=1)
        if not last or not last[0]:
            lastsald = 0
        else:
            lastsald = last[0]
        last = self.sql.getRec(tables="drstrn", cols=["max(drt_trdt)"],
            where=[("drt_cono", "=", self.conum), ("drt_chain", "=",
            self.chain), ("drt_acno", "=", self.acno), ("drt_type", "=", 5)],
            limit=1)
        if not last or not last[0]:
            lastpayd = 0
        else:
            lastpayd = last[0]
        for x in range (0, 8):
            if x == 0:
                data=lastsald
            elif x == 1:
                data=lastpayd
            elif x == 2:
                data=tbal
            else:
                data=ages[x-3]
            self.df.loadEntry("T", 3, x, data=data)
        p = 0
        for x in range (0, 13):
            i = 0
            self.df.loadEntry("C", 4, p, data=self.sale[x])
            p = p + 1
            i = i + 1
            pay = float(ASD(0) - ASD(self.pays[x]))
            self.df.loadEntry("C", 4, p, data=pay)
            p = p + 1
            i = i + 1

    def doTagSelect(self):
        self.mf.updateStatus("")

    def doTrans1(self, widget=None):
        self.df.focusField("T", 5, 1)

    def doTrans2(self, frt, pag, r, c, p, i, w):
        self.paidup = w
        whr = [("drt_cono", "=", self.conum), ("drt_chain", "=", self.chain),
            ("drt_acno", "=", self.acno)]
        if self.paidup == "Y":
            col, all = getTrn(self.mf.dbm, "drs", whr=whr)
        else:
            col, all = getTrn(self.mf.dbm, "drs", dte=self.curdt, whr=whr)
        if all:
            data = []
            for dat in all:
                data.append([
                    dat[col.index("drt_trdt")],
                    dat[col.index("drt_curdt")],
                    dat[col.index("drt_batch")],
                    dat[col.index("drt_type")],
                    dat[col.index("drt_ref1")],
                    dat[col.index("drt_ref2")],
                    dat[col.index("drt_tramt")],
                    dat[col.index("paid")],
                    dat[col.index("balance")],
                    dat[col.index("drt_desc")]])
            if self.chains == "Y":
                tit = "Transactions for Account: %s %s - %s" % \
                    (self.chain, self.acno, self.name)
            else:
                tit = "Transactions for Account: %s - %s" % \
                    (self.acno, self.name)
            col = (
                ("drt_trdt", "   Date", 10, "D1", "N"),
                ("drt_curdt", "Curr-Dt", 7, "D2", "N"),
                ("drt_batch", "Batch", 7, "Na", "N"),
                ("drt_type", "Typ", 3, ("XX", drtrtp), "N"),
                ("drt_ref1", "Reference", 9, "Na", "Y"),
                ("drt_ref2", "Ref-Num-2", 9, "Na", "Y"),
                ("drt_tramt", "    Amount", 11.2, "SD", "N"),
                ("alloc", " Allocated", 11.2, "SD", "N"),
                ("balan", "   Balance", 11.2, "SD", "N"),
                ("drt_desc", "Details", 30, "NA", "N"))
            state = self.df.disableButtons()
            while True:
                rec = SelectChoice(self.df.nb.Page5, tit, col, data)
                # If Invoice or Credit Note, attempt to display the document
                # else display all transaction details
                if rec.selection:
                    self.df.mstAlw.hide()
                    sp = SplashScreen(self.df.nb.Page5,
                        "Displaying Document.... Please Wait....")
                    if rec.selection[4] in (1, 4):
                        if rec.selection[4] == 1:
                            typ = "I"
                        else:
                            typ = "C"
                        try:
                            doc = int(rec.selection[5])
                            inv = self.sql.getRec(tables="slsiv1",
                                where=[("si1_cono", "=", self.conum),
                                ("si1_rtn", "=", typ), ("si1_docno", "=", doc)],
                                limit=1)
                        except:
                            inv = []
                    else:
                        inv = []
                    if inv:
                        PrintInvoice(self.mf, self.conum, typ, doc,
                            prtnam="view", copy="y")
                    else:
                        whr = [
                            ("drt_cono", "=", self.conum),
                            ("drt_chain", "=", self.chain),
                            ("drt_acno", "=", self.acno),
                            ("drt_type", "=", rec.selection[4]),
                            ("drt_ref1", "=", rec.selection[5])]
                        TabPrt(self.mf, tabs="drstrn", wher=whr, pdia=False)
                    sp.closeSplash()
                    self.df.mstAlw.show_all()
                else:
                    break
            self.df.enableButtons(state=state)
        self.doTrans1()

    def doNotes(self, widget=None):
        state = self.df.disableButtons()
        self.df.mstAlw.hide()
        nf = NotesCreate(self.mf, self.conum, self.conam, self.capnm, "DRS",
            "%03i%s" % (self.chain, self.acno))
        self.df.mstAlw.show()
        self.df.enableButtons(state=state)

    def doClear(self, widget=None):
        self.df.selPage("Basic-_1")
        self.df.focusField("T", 0, 1)

    def doPrint(self, widget=None):
        titl = "Select the required print options"
        data = (
            ("1", "Information"),
            ("2", "Transactions"),
            ("3", "Both"))
        ss = SChoice(self.mf, titl=titl, data=data)
        self.mf.updateStatus("")
        if ss.selection:
            state = self.df.disableButtons()
            self.df.mstAlw.hide()
            if ss.selection[0] in ("1", "3"):
                tab = "drsmst"
                whr = [("drm_cono", "=", self.conum), ("drm_chain", "=",
                    self.chain), ("drm_acno", "=", self.acno)]
                rp = TabPrt(self.mf, self.conum, self.conam, name=__name__,
                    tabs=tab, wher=whr, keys=[self.chain, self.acno])
                view = rp.view
                mail = rp.mail
                exit = rp.exit
            else:
                view = None
                mail = None
                exit = "n"
            if ss.selection[0] in ("2", "3") and exit == "n":
                heads = ["Debtor's Transactions",
                    "Chain: %s  Account: %s  Name: %s" % \
                    (self.chain, self.acno, self.name)]
                whr = [("drt_cono", "=", self.conum), ("drt_chain", "=",
                    self.chain), ("drt_acno", "=", self.acno)]
                if self.paidup == "Y":
                    col, all = getTrn(self.mf.dbm, "drs", whr=whr)
                else:
                    col, all = getTrn(self.mf.dbm, "drs", dte=self.curdt,
                        whr=whr)
                cols = []
                data = []
                dic = self.sql.drstrn_dic
                for num, rec in enumerate(all):
                    dat = []
                    for nam in [
                        "drt_ref1",
                        "drt_ref2",
                        "drt_trdt",
                        "drt_type",
                        "drt_tramt",
                        "paid",
                        "balance",
                        "drt_desc"]:
                        if not num:
                            if nam == "paid":
                                cols.append(["paid", "SD", 11.2, "Paid"])
                            elif nam == "balance":
                                cols.append(["balance", "SD", 11.2, "Balance"])
                            else:
                                cols.append([nam, dic[nam][2], dic[nam][3],
                                dic[nam][5]])
                        dat.append(rec[col.index(nam)])
                    data.append(dat)
                gtots=["drt_tramt", "paid", "balance"]
                if view:
                    prtdia = False
                else:
                    prtdia = ("Y","V")
                rp = RepPrt(self.mf, conum=self.conum, conam=self.conam,
                    name=__name__, ttype="D", tables=data, heads=heads,
                    cols=cols, trtp=["drt_type", drtrtp], gtots=gtots,
                    prtdia=prtdia, view=view, mail=mail)
            self.df.mstAlw.show()
            self.df.enableButtons(state=state)
            self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

