#! /usr/bin/python

"""
SYNOPSIS
    Debtors Ledger Change Age Allocations.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import AgeTrans, ASD, CCD, GetCtl, Sql, TartanDialog
from tartanFunctions import getTrn

class dr6020:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.dataHeader()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["drsmst", "drschn", "drstrn", "drsage"],
            prog=__name__)
        gc = GetCtl(self.mf)
        drsctl = gc.getCtl("drsctl", self.conum)
        if not drsctl:
            return "no"
        self.chains = drsctl["ctd_chain"]
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.curdt = self.sysdtw / 100

    def dataHeader(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Debtors Change Allocations (%s)" % __name__)
        drc = {
            "stype": "R",
            "tables": ("drschn",),
            "cols": (
                ("chm_chain", "", 0, "Num"),
                ("chm_name", "", 0, "Name", "Y")),
            "where": [("chm_cono", "=", self.conum)]}
        drm = {
            "stype": "R",
            "tables": ("drsmst",)}
        if self.chains == "Y":
            drm["cols"] = (
                ("drm_chain", "", 0, "Chn"),
                ("drm_acno", "", 0, "Acc-Num"),
                ("drm_name", "", 0, "Name", "Y"),
                ("drm_add1", "", 0, "Address Line 1"))
            drm["where"] = [("drm_cono", "=", self.conum)]
            drm["whera"] = [["T", "drm_chain", 0]]
        else:
            drm["cols"] = (
                ("drm_acno", "", 0, "Acc-Num"),
                ("drm_name", "", 0, "Name", "Y"),
                ("drm_add1", "", 0, "Address Line 1"))
            drm["where"] = [
                ("drm_cono", "=", self.conum),
                ("drm_chain", "=", 0)]
        fld = [
            [["T",0,0,0],"ID2",7,"Period","Current Financial Period",
                self.curdt,"N",self.doCurdt,None,None,("efld",)],
            [["T",0,1,0],"INA",7,"Acc-Num","Account Number",
                "","N",self.doAccno,drm,None,("notblank",)],
            [["T",0,1,0],"ONA",30,"Name"]]
        if self.chains == "Y":
            fld.insert(1, [["T",0,0,0],"IUI",3,"Chain","Chain Store",
                "","N",self.doChain,drc,None,("efld",)])
        else:
            self.chain = 0
        tnd = ((self.endTop, "n"),)
        txt = (self.exitTop,)
        self.but = (
            ("Normal", None, self.doReAgeNormal, 0, None, None,
                "Only Show Unallocated Transactions"),
            ("History", None, self.doReAgeHistory, 0, None, None,
                "Show All Transactions, Including Already Allocated"),
            ("Automatic", None, self.doReAgeAuto, 0, None, None,
                "Automatically Re-Age the Account Based on Date"))
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, butt=self.but)

    def doCurdt(self, frt, pag, r, c, p, i, w):
        self.curdt = w

    def doChain(self, frt, pag, r, c, p, i, w):
        if w:
            acc = self.sql.getRec(tables="drschn", cols=["chm_chain"],
                where=[("chm_cono", "=", self.conum), ("chm_chain", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Chain Store"
        self.chain = w

    def doAccno(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="drsmst", cols=["drm_name"],
            where=[("drm_cono", "=", self.conum), ("drm_chain", "=",
            self.chain), ("drm_acno", "=", w)], limit=1)
        if not acc:
            return "Invalid Account Number"
        self.acno = w
        self.key = None
        self.df.loadEntry("T", pag, p+1, data=acc[0])
        self.mf.mess.set_text("Select Routine")
        for b in range(3):
            exec "self.df.setWidget(self.df.B%s, 'normal')" % b
        self.mf.startLoop()
        if self.key in ("normal", "history"):
            return "nd"
        elif self.key == "cancel":
            return "nc"

    def doCancel(self, widget=None):
        if self.key == "move":
            self.mf.dbm.rollbackDbase()
            self.df.focusField("T", 0, 1)
        else:
            self.key = "cancel"
            self.mf.closeLoop()

    def doReAgeNormal(self, widget=None):
        self.key = "normal"
        self.doReAge()

    def doReAgeHistory(self, widget=None):
        self.key = "history"
        self.doReAge()

    def doReAge(self):
        self.mf.updateStatus("")
        for b in range(3):
            exec "self.df.setWidget(self.df.B%s, 'disabled')" % b
        col = ["drt_trdt","drt_ref1","drt_type","drt_tramt","paid","balance"]
        whr = [("drt_cono", "=", self.conum), ("drt_chain", "=", self.chain),
            ("drt_acno", "=", self.acno)]
        if self.key == "normal":
            dtc, all = getTrn(self.mf.dbm, "drs", whr=whr, zer="N")
        else:
            dtc, all = getTrn(self.mf.dbm, "drs", whr=whr)
        if all:
            data = []
            for rec in all:
                dat = []
                for c in col:
                    dat.append(rec[dtc.index(c)])
                data.append(dat)
            age = AgeTrans(self.mf, "drs", data, 0, exit=False)
            if not age.ccl and age.data:
                if age.total.work:
                    totl = age.total.work
                    while not age.ccl:
                        age.doAllocate()
                        if age.ccl or not age.total.work:
                            break
                if age.ccl:
                    self.key = "cancel"
                    age.data = []
                for tr in age.data:
                    if tr[6]:
                        w = whr[:]
                        w.extend([("drt_type", "=", tr[2]),
                            ("drt_ref1", "=", tr[1])])
                        self.sql.insRec("drsage", data=[self.conum, self.chain,
                            self.acno, tr[2], tr[1], self.curdt, tr[2], tr[1],
                            tr[6], 0])
            else:
                self.key = "cancel"
        self.mf.closeLoop()

    def doReAgeAuto(self, widget=None):
        self.mf.updateStatus("Re-Ageing .... Please Wait!")
        self.key = "normal"
        for b in range(3):
            exec "self.df.setWidget(self.df.B%s, 'disabled')" % b
        self.sql.delRec("drsage", where=[("dra_cono", "=", self.conum),
            ("dra_chain", "=", self.chain), ("dra_acno", "=", self.acno)])
        all = self.sql.getRec(tables="drstrn", where=[("drt_cono", "=",
            self.conum), ("drt_chain", "=", self.chain), ("drt_acno", "=",
            self.acno)])
        self.sql.delRec("drstrn", where=[("drt_cono", "=", self.conum),
            ("drt_chain", "=", self.chain), ("drt_acno", "=", self.acno)])
        for rec in all:
            self.sql.insRec("drstrn", data=rec)
        col, cr = getTrn(self.mf.dbm, "drs", whr=[("drt_cono", "=",
            self.conum), ("drt_chain", "=", self.chain), ("drt_acno", "=",
            self.acno), ("drt_tramt", "<", 0)], zer="N")
        if cr:
            for ctr in cr:
                ccdt = ctr[col.index("drt_curdt")]
                cbal = CCD(ctr[col.index("balance")], "SD", 13.2).work
                camt = cbal
                col, dr = getTrn(self.mf.dbm, "drs", whr=[("drt_cono", "=",
                    self.conum), ("drt_chain", "=", self.chain), ("drt_acno",
                    "=", self.acno), ("drt_tramt", ">", 0)], neg=False, zer="N")
                if dr:
                    for dtr in dr:
                        dbal = CCD(dtr[col.index("balance")], "SD", 13.2).work
                        damt = float(ASD(dbal) + ASD(camt))
                        if damt < 0:
                            camt = damt
                            damt = 0.00
                        else:
                            camt = 0.00
                        diff = float(ASD(dbal) - ASD(damt))
                        if diff:
                            self.sql.insRec("drsage", data=[self.conum,
                                self.chain, self.acno,
                                dtr[col.index("drt_type")],
                                dtr[col.index("drt_ref1")], ccdt,
                                ctr[col.index("drt_type")],
                                ctr[col.index("drt_ref1")], diff, 0])
                        if not camt:
                            break
                diff = float(ASD(cbal) - ASD(camt))
                if diff:
                    self.sql.insRec("drsage", data=[self.conum,
                        self.chain, self.acno,
                        ctr[col.index("drt_type")],
                        ctr[col.index("drt_ref1")], ccdt,
                        ctr[col.index("drt_type")],
                        ctr[col.index("drt_ref1")], diff, 0])
        self.mf.closeLoop()
    
    def endTop(self):
        self.df.focusField("T", 0, 1)

    def exitTop(self):
        self.mf.dbm.commitDbase(ask=True)
        self.df.closeProcess()
        self.mf.closeLoop()

