#! /usr/bin/env python

"""
SYNOPSIS
    Debtors Chainstores File Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import GetCtl, Sql, TartanDialog

class drc210:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlvmf", "drschn"], prog=__name__)
        gc = GetCtl(self.mf)
        drsctl = gc.getCtl("drsctl", self.conum)
        if not drsctl:
            return "no"
        if drsctl["ctd_chain"] == "N":
            return "no"

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Debtors Chainstores File Maintenance (%s)" % __name__)
        chm = {
            "stype": "R",
            "tables": ("drschn",),
            "cols": (
                ("chm_chain", "", 0, "Chn"),
                ("chm_name", "", 0, "Name", "Y")),
            "where": [("chm_cono", "=", self.conum)]}
        vtm = {
            "stype": "R",
            "tables": ("ctlvmf",),
            "cols": (
                ("vtm_code", "", 0, "C"),
                ("vtm_desc", "", 0, "Description", "Y")),
            "where": [("vtm_cono", "=", self.conum)]}
        self.fld = (
            (("T",0,0,0),"IUI",3,"Chn","Chainstore Number",
                "","N",self.doChain,chm,None,("notzero",)),
            (("T",0,1,0),"INA",30,"Name","Name",
                "","N",None,None,self.doDelete,("notblank",)),
            (("T",0,2,0),"INA",30,"Address Line 1","Address Line 1",
                "","N",None,None,None,("notblank",)),
            (("T",0,3,0),"INA",30,"Address Line 2","Address Line 2",
                "","N",None,None,None,("efld",)),
            (("T",0,4,0),"INA",30,"Address Line 3","Address Line 3",
                "","N",None,None,None,("efld",)),
            (("T",0,5,0),"INA",4,"Postal Code","Postal Code",
                "","N",None,None,None,("efld",)),
            (("T",0,6,0),"INA",20,"Telephone","Telephone",
                "","N",None,None,None,("efld",)),
            (("T",0,7,0),"INA",20,"Fax","Fax",
                "","N",None,None,None,("efld",)),
            (("T",0,8,0),"ITX",50,"E-Mail Address","E-Mail Address",
                "","N",None,None,None,("email",)),
            (("T",0,9,0),"INA",30,"Contact","Contact",
                "","N",None,None,None,("efld",)),
            (("T",0,10,0),"IUA",1,"Vat Code","Vat Code",
                "","N",self.doVat,vtm,None,("notblank",)),
            (("T",0,10,0),"ONA",30,""))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,2),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)

    def doChain(self, frt, pag, r, c, p, i, w):
        self.chain = w
        acc = self.sql.getRec(tables="drschn", where=[("chm_cono", "=",
            self.conum), ("chm_chain", "=", self.chain)], limit=1)
        if not acc:
            self.new = "Y"
        else:
            self.new = "N"
            vat = self.sql.getRec(tables="ctlvmf", cols=["vtm_desc"],
                where=[("vtm_cono", "=", self.conum), ("vtm_code", "=",
                acc[self.sql.drschn_col.index("chm_vatind")])], limit=1)
            if not vat:
                acc.append("Invalid Vat Record")
            else:
                acc.append(vat[0])
            for x in range(0, self.df.topq[pag]):
                self.df.loadEntry(frt, pag, p+x, data=acc[x+1])

    def doVat(self, frt, pag, r, c, p, i, w):
        vat = self.sql.getRec(tables="ctlvmf", cols=["vtm_desc"],
            where=[("vtm_cono", "=", self.conum), ("vtm_code", "=", w)],
            limit=1)
        if not vat:
            return "Invalid Vat Record"
        self.df.loadEntry(frt, pag, p+1, data=vat[0])

    def doDelete(self):
        self.sql.delRec("drschn", where=[("chm_cono", "=", self.conum),
            ("chm_chain", "=", self.chain)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        dat = [self.conum]
        for x in range(0, len(self.df.t_work[0][0])):
            if x == 11:
                continue
            dat.append(self.df.t_work[0][0][x])
        if self.new == "Y":
            self.sql.insRec("drschn", data=dat)
        else:
            self.sql.updRec("drschn", data=dat, where=[("chm_cono", "=",
            self.conum), ("chm_chain", "=", self.chain)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

