#! /usr/bin/env python

"""
SYNOPSIS
    Debtors Areas File Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import Sql, TartanDialog 
class drc310:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlare", "ctlrep"], prog=__name__)

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Debtors Areas File Maintenance (%s)" % __name__)
        arm = {
            "stype": "R",
            "tables": ("ctlare",),
            "cols": (
                ("arm_area", "", 0, "Are"),
                ("arm_name", "", 0, "Name", "Y"),
                ("arm_rep", "", 0, "Rep")),
            "where": [("arm_cono", "=", self.conum)]}
        rpm = {
            "stype": "R",
            "tables": ("ctlrep",),
            "cols": (
                ("rpm_rep", "", 0, "Rep"),
                ("rpm_name", "", 0, "Name", "Y")),
            "where": [("rpm_cono", "=", self.conum)]}
        self.fld = (
            (("T",0,0,0),"INA",3,"Area","Area Number",
                "","N",self.doArea,arm,None,("notblank",)),
            (("T",0,1,0),"INA",30,"Name","Name",
                "","N",None,None,self.doDelete,("notblank",)),
            (("T",0,2,0),"INa",3,"Rep Number","Rep Number",
                "","N",self.doRep,rpm,None,("efld",)),
            (("T",0,2,0),"ONA",30,""))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,2),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)

    def doArea(self, frt, pag, r, c, p, i, w):
        self.area = w
        acc = self.sql.getRec(tables="ctlare", where=[("arm_cono", "=",
            self.conum), ("arm_area", "=", self.area)], limit=1)
        if not acc:
            self.new = "Y"
        else:
            self.new = "N"
            rep = self.sql.getRec(tables="ctlrep", cols=["rpm_name"],
                where=[("rpm_cono", "=", self.conum), ("rpm_rep", "=",
                acc[self.sql.ctlare_col.index("arm_rep")])], limit=1)
            if not rep:
                acc.append("Invalid Rep Record")
            else:
                acc.append(rep[0])
            for x in range(0, self.df.topq[pag]):
                self.df.loadEntry(frt, pag, p+x, data=acc[x+1])

    def doRep(self, frt, pag, r, c, p, i, w):
        if w:
            rep = self.sql.getRec(tables="ctlrep", cols=["rpm_name"],
                where=[("rpm_cono", "=", self.conum), ("rpm_rep", "=", w)],
                limit=1)
            if not rep:
                return "Invalid Rep Record"
            self.df.loadEntry("T", 0, 3, data=rep[0])

    def doDelete(self):
        dat = (self.conum, self.df.t_work[0][0][0])
        self.sql.delRec("ctlare", where=[("arm_cono", "=", self.conum),
            ("arm_area", "=", self.area)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        dat = []
        dat.append(self.conum)
        for x in range(0, len(self.df.t_work[0][0])):
            if x == 3:
                continue
            dat.append(self.df.t_work[0][0][x])
        if self.new == "Y":
            self.sql.insRec("ctlare", data=dat)
        else:
            self.sql.updRec("ctlare", data=dat, where=[("arm_cono", "=",
                self.conum), ("arm_area", "=", self.area)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

