#! /usr/bin/env python

"""
SYNOPSIS
    Debtors Reps Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import RepPrt, Sql, TartanDialog

class drc420:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "ctlrep", prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Debtors Reps Master Listing (%s)" % __name__)
        r1s = (("Number","N"),("Name","A"))
        r2s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Sort Order","Sort Order",
                "N","Y",self.doSort,None,None,None),)
        tnd = ((self.doEnd,"Y"), )
        txt = (self.closeProcess, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"))

    def doSort(self, frt, pag, r, c, p, i, w):
        self.sort = w

    def doEnd(self):
        self.df.closeProcess()
        name = __name__
        tables = ["ctlrep"]
        heads = ["Salemen's Master Listing"]
        cols = []
        dics = self.sql.ctlrep_dic
        for col in self.sql.ctlrep_col:
            if col in ("rpm_cono",):
                continue
            dat = [col, dics[col][2], dics[col][3], dics[col][4]]
            cols.append(dat)
        if self.sort == "N":
            ord = "rpm_rep"
        else:
            ord = "rpm_name, rpm_rep"
        whr = [("rpm_cono", "=", self.conum)]
        opts = "%-5s%-6s" % ("Sort-", self.sort)
        rp = RepPrt(self.mf, name=name, tables=tables, heads=heads, cols=cols,
            order=ord, opts=opts, where=whr, conum=self.conum, conam=self.conam,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
        self.mf.closeLoop()

    def closeProcess(self):
        self.df.closeProcess()
        self.mf.closeLoop()

