#! /usr/bin/env python

"""
SYNOPSIS
    Debtors Delivery Addresses Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import RepPrt, Sql, TartanDialog
from tartanFunctions import showError

class drc520:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "drsdel", prog=__name__)

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Debtors Delivery Address Listing (%s)" % __name__)
        r1s = (("Yes","Y"),("No","N"))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.closeProcess, )
        self.df = TartanDialog(self.mf, title=tit, eflds=[], tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doEnd(self):
        all = self.sql.getRec(tables="drsdel", order="del_code")
        if not all:
            showError(self.mf.body, "Transaction Error",
            "No Transactions Selected")
        else:
            self.printReport(all)

    def printReport(self, all):
        self.df.closeProcess()
        tables = []
        for x in range(0, len(all)):
            lin = []
            lin.append(all[x][0])
            lin.append(all[x][1])
            lin.append(all[x][2])
            tables.append(lin)
            lin = []
            if all[x][3] or all[x][4]:
                lin.append("")
                lin.append(all[x][3])
                lin.append(all[x][4])
                tables.append(lin)
            lin = []
            for z in range(0, 3):
                lin.append("")
            tables.append(lin)
        heads = ["Debtors Delivery Address Listing"]
        cols = [["a","NA",7.0,"Add-Cod"],
                ["b","NA",30.0,"Address-Line"],
                ["c","NA",30.0,"Address-Line"]]
        state = self.df.disableButtons()
        rp = RepPrt(self.mf, name=__name__, tables=tables, heads=heads,
            cols=cols, conum=self.conum, ttype="D", conam=self.conam,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
        self.df.enableButtons(state=state)
        self.mf.closeLoop()

    def closeProcess(self):
        self.df.closeProcess()
        self.mf.closeLoop()

