#! /usr/bin/python

"""
SYNOPSIS
    General Ledger Accounts Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, time
from TartanClasses import CsvCvt, FileImport, GetCtl, ProgressBar
from TartanClasses import SplashScreen, Sql, TartanDialog
from tartanFunctions import showError

class gl1010:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.user = usr
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        gc = GetCtl(self.mf)
        ctlmst = gc.getCtl("ctlmst", self.conum)
        if not ctlmst:
            return "no"
        self.taxdf = ctlmst["ctm_taxdf"]
        if not self.taxdf:
            self.taxdf = "N"
        self.mods = []
        for x in range(0, 21, 2):
            self.mods.append(ctlmst["ctm_modules"][x:x+2])
        tabs = ["ctlctl", "ctlvmf", "genmst", "genrpt", "gentrn", "chglog"]
        if "CR" in self.mods:
            tabs.append("crsctl")
        if "DR" in self.mods:
            tabs.append("drsctl")
        if "ST" in self.mods:
            tabs.append("strctl")
        self.sql = Sql(self.mf.dbm, tabs, prog=__name__)
        if self.sql.error:
            return self.sql.error
        chk = self.sql.getRec(tables="genmst", cols=["count(*)"],
            where=[("glm_cono", "=", self.conum)], limit=1)
        if chk[0]:
            self.newgen = False
        else:
            self.newgen = True

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "General Ledger File Maintenance (%s)" % __name__)
        glm = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "where": [("glm_cono", "=", self.conum)]}
        vat = {
            "stype": "R",
            "tables": ("ctlvmf",),
            "cols": (
                ("vtm_code", "", 0, "Acc-Num"),
                ("vtm_desc", "", 0, "Description", "Y")),
            "where": [("vtm_cono", "=", self.conum)]}
        r1s = (("Profit & Loss","P"), ("Balance Sheet","B"))
        r2s = (("Yes","Y"), ("No","N"))
        fld = [
            (("T",0,0,0),"IUI",7,"Acc-Num","Account Number",
                "","Y",self.doAccNum,glm,None,("notzero",)),
            (("T",0,1,0),("IRB",r1s),0,"Account Type","Account Type",
                "P","N",self.doTypCod,None,self.doDelete,None),
            (("T",0,2,0),"INA",30,"Description","Account Description",
                "","N",None,None,None,("notblank",)),
            (("T",0,3,0),("IRB",r2s),0,"Allow Postings","Allow Postings",
                "Y","N",None,None,None,None),
            [("T",0,4,0),"INA",1,"Tax Default","Tax Default",
                "","N",self.doVatCod,vat,None,("notblank",)]]
        but = (
            ("Accept",None,self.doAccept,0,("T",0,2),("T",0,0)),
            ("Import",None,self.doImport,0,("T",0,1),("T",0,2),"""Import a Chart of Accounts from a CSV or XLS File.""",1),
            ("Populate",None,self.doPopulate,0,None,None,"""Generate a Chart of Accounts with Accompanying Control Records and Financial Statement Report. This Only Applies to Unpopulated (NEW) Ledgers.""",1),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, butt=but,
            tend=tnd, txit=txt)
        if self.newgen:
            self.df.setWidget(self.df.B2, state="normal")

    def doAccNum(self, frt, pag, r, c, p, i, w):
        self.acno = w
        self.old = self.sql.getRec(tables="genmst", where=[("glm_cono", "=",
            self.conum), ("glm_acno", "=", self.acno)], limit=1)
        if not self.old:
            self.new = True
        else:
            self.new = False
            for x in range(0, self.df.topq[pag]):
                self.df.loadEntry(frt, pag, p+x, data=self.old[x+1])

    def doTypCod(self, frt, pag, r, c, p, i, w):
        if self.new:
            if w == "P":
                self.df.topf[pag][4][5] = self.taxdf
            else:
                self.df.topf[pag][4][5] = "N"
        elif not self.df.topf[pag][4][5]:
            self.df.topf[pag][4][5] = "N"

    def doVatCod(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="ctlvmf", cols=["vtm_desc"],
            where=[("vtm_cono", "=", self.conum), ("vtm_code", "=", w)],
            limit=1)
        if not acc:
            return "Invalid VAT Code"

    def doDelete(self):
        t = self.sql.getRec(tables="gentrn", cols=["count(*)"],
            where=[("glt_cono", "=", self.conum), ("glt_acno", "=", self.acno)],
            limit=1)
        if t[0]:
            return "Transactions Exist, Not Deleted"
        dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
        self.sql.delRec("genmst", where=[("glm_cono", "=", self.conum),
            ("glm_acno", "=", self.acno)])
        self.sql.delRec("genbal", where=[("glo_cono", "=", self.conum),
            ("glo_acno", "=", self.acno)])
        self.sql.delRec("genbud", where=[("glb_cono", "=", self.conum),
            ("glb_acno", "=", self.acno)])
        dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
        self.sql.insRec("chglog", data=["genmst", "D", "%03i%07i" % \
            (self.conum, self.acno), "", dte, self.user, "", "", 0])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        data = [self.conum, self.acno,
            self.df.t_work[0][0][1], self.df.t_work[0][0][2],
            self.df.t_work[0][0][3], self.df.t_work[0][0][4]]
        if self.new:
            self.sql.insRec("genmst", data=data)
        elif data != self.old:
            self.sql.updRec("genmst", data=data, where=[("glm_cono", "=",
                self.conum), ("glm_acno", "=", self.acno)])
            dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
            for num, dat in enumerate(self.old):
                if dat != data[num]:
                    self.sql.insRec("chglog", data=["genmst", "U",
                        "%03i%07i" % (self.conum, self.acno),
                        self.sql.genmst_col[num], dte, self.user,
                        str(dat), str(data[num]), 0])
        self.mf.dbm.commitDbase()
        self.df.setWidget(self.df.B3, state="disabled")
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doImport(self, widget=None):
        self.df.setWidget(self.df.B3, state="disabled")
        self.df.mstAlw.hide()
        fi = FileImport(self.mf, imptab="genmst", impskp=["glm_cono"])
        sp = ProgressBar(self.mf.body, typ="Importing Creditor's Accounts",
            max=len(fi.impdat), esc=False)
        err = None
        for num, line in enumerate(fi.impdat):
            sp.displayProgress(num)
            chk = self.sql.getRec(tables="genmst", where=[("glm_cono",
                "=", self.conum), ("glm_acno", "=", line[0])], limit=1)
            if chk:
                err = "%s %s Already Exists" % (fi.impcol[0][0], line[0])
                break
            if line[1] not in ("B", "P"):
                err = "Invalid %s %s, Only B or P" % (fi.impcol[1][0], line[1])
                break
            if not line[2]:
                err = "Blank Description"
                break
            if line[3] not in ("Y", "N"):
                err = "Invalid %s %s" % (fi.impcol[3][0], line[3])
                break
            chk = self.sql.getRec(tables="ctlvmf", where=[("vtm_cono",
                "=", self.conum), ("vtm_code", "=", line[4])], limit=1)
            if not chk:
                err = "%s %s Does Not Exist" % (fi.impcol[4][0], line[4])
                break
            line.insert(0, self.conum)
            self.sql.insRec("genmst", data=line)
        sp.closeProgress()
        if err:
            err = "Line %s: %s" % ((num + 1), err)
            showError(self.mf.body, "Import Error", """%s
            
Please Correct your Import File and then Try Again.""" % err)
            self.mf.dbm.rollbackDbase()
        else:
            self.mf.dbm.commitDbase()
        self.df.mstAlw.show_all()
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doPopulate(self, widget=None):
        self.igcrs = "N"
        self.igdrs = "N"
        self.igstr = "N"
        self.df.setWidget(self.df.B3, state="disabled")
        self.df.mstAlw.hide()
        tit = ("Integrate Subsiduary Ledgers",)
        r1s = (("Yes", "Y"), ("No", "N"))
        fld = []
        col = 0
        if "CR" in self.mods:
            fld.append((("T",0,col,0),("IRB",r1s),0,"Creditor's Ledger",
                "Creditor's Ledger","N","N",self.doIgCrs,None,None,None,None))
            col += 1
        if "DR" in self.mods:
            fld.append((("T",0,col,0),("IRB",r1s),0,"Debtor's Ledger",
                "Debtor's Ledger","N","N",self.doIgDrs,None,None,None,None))
            col += 1
        if "ST" in self.mods:
            fld.append((("T",0,col,0),("IRB",r1s),0,"Stores's Ledger",
                "Stores's Ledger","N","N",self.doIgStr,None,None,None,None))
        if fld:
            self.ig = TartanDialog(self.mf, title=tit, newscn=(0,1), tops=True,
                eflds=fld, tend=((self.doIgEnd,"y"),), txit=(self.doIgExit,))
            self.mf.startLoop()
            if self.igexit:
                self.doExit()
                return
        sp = SplashScreen(self.mf.body,
            "Populating Records\n\nPlease Wait ...")
        # genmst
        input = open(os.path.join(self.mf.prgdir, "csv", "genmst.dat"), "r")
        data = input.readlines()
        for num, line in enumerate(data):
            cvt = CsvCvt(line).output
            cvt.insert(0, self.conum)
            cvt.append("Y")
            if cvt[2] == "B":
                cvt.append("N")
            else:
                cvt.append(self.taxdf)
            self.sql.insRec("genmst", data=cvt)
        # genrpt
        input = open(os.path.join(self.mf.prgdir, "csv", "genrpt.dat"), "r")
        data = input.readlines()
        for num, line in enumerate(data):
            cvt = CsvCvt(line).output
            cvt.insert(0, self.conum)
            self.sql.insRec("genrpt", data=cvt)
        # ctlctl
        crsctl = 0
        drsctl = 0
        stksoh = 0
        input = open(os.path.join(self.mf.prgdir, "csv", "ctlctl.dat"), "r")
        data = input.readlines()
        for num, line in enumerate(data):
            cvt = CsvCvt(line).output
            cvt.insert(0, self.conum)
            self.sql.insRec("ctlctl", data=cvt)
            if cvt[1] == "crs_ctl":
                crsctl = int(cvt[3])
            elif cvt[1] == "drs_ctl":
                drsctl = int(cvt[3])
            elif cvt[1] == "stk_soh":
                stksoh = int(cvt[3])
        if "CR" in self.mods:
            self.sql.delRec("crsctl", where=[("ctc_cono", "=", self.conum)])
            self.sql.insRec("crsctl", data=[self.conum, self.igcrs, "E", "",
                0, 0])
            if self.igcrs == "Y" and crsctl:
                self.sql.updRec("genmst", cols=["glm_ind"], data=["N"],
                    where=[("glm_cono", "=", self.conum), ("glm_acno", "=",
                    crsctl)])
        if "DR" in self.mods:
            self.sql.delRec("drsctl", where=[("ctd_cono", "=", self.conum)])
            self.sql.insRec("drsctl", data=[self.conum, self.igdrs, "E", "N",
                "N", "N", "Y"])
            if self.igdrs == "Y" and drsctl:
                self.sql.updRec("genmst", cols=["glm_ind"], data=["N"],
                    where=[("glm_cono", "=", self.conum), ("glm_acno", "=",
                    drsctl)])
        if "ST" in self.mods:
            self.sql.delRec("strctl", where=[("cts_cono", "=", self.conum)])
            self.sql.insRec("strctl", data=[self.conum,self.igstr,"N",1])
            if self.igstr == "Y" and stksoh:
                self.sql.updRec("genmst", cols=["glm_ind"], data=["N"],
                    where=[("glm_cono", "=", self.conum), ("glm_acno", "=",
                    stksoh)])
        sp.closeSplash()
        self.mf.dbm.commitDbase()
        self.df.mstAlw.show_all()
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doIgCrs(self, frt, pag, r, c, p, i, w):
        self.igcrs = w

    def doIgDrs(self, frt, pag, r, c, p, i, w):
        self.igdrs = w

    def doIgStr(self, frt, pag, r, c, p, i, w):
        self.igstr = w

    def doIgEnd(self, widget=None):
        self.igexit = False
        self.ig.closeProcess()
        self.mf.closeLoop()

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.last[0] = [0, 0]
        self.df.focusField("T", 0, 1)

    def doIgExit(self, widget=None):
        self.igexit = True
        self.ig.closeProcess()
        self.mf.closeLoop()

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

