#! /usr/bin/python

"""
SYNOPSIS
    General Ledger Report Generator Record Maintenance.
    This program is used to create records used by the general ledger
    financial reports program i.e. gl3050. With these records you can
    design your own layouts of this report.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import RepPrt, Sql, TartanDialog
from tartanFunctions import askQuestion, showError

class gl1030:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "genrpt", prog=__name__)
        if self.sql.error:
            return self.sql.error
        self.genrpt_fd = self.sql.genrpt_dic
        self.genrpt_cl = self.sql.genrpt_col
        self.pags = ["","Q","H","L","G","S","T","U","C","P"]
        self.head = ["glr_desc", "glr_high", "glr_ffeed", "glr_ignore"]
        self.ledger = [
            "glr_from", "glr_to", "glr_high", "glr_obal", "glr_accum",
            "glr_print", "glr_norm", "glr_acbal", "glr_ignore",
            "glr_store", "glr_snum1", "glr_acstr"]
        self.group = [
            "glr_group", "glr_desc", "glr_from", "glr_to", "glr_high",
            "glr_obal", "glr_accum", "glr_print", "glr_norm", "glr_acbal",
            "glr_ignore", "glr_store", "glr_snum1", "glr_acstr", "glr_label"]
        self.store = [
            "glr_desc", "glr_high", "glr_print", "glr_norm", "glr_acbal",
            "glr_clear", "glr_snum1", "glr_strper"]
        self.total = [
            "glr_desc", "glr_high", "glr_total", "glr_print", "glr_norm",
            "glr_clear", "glr_store", "glr_snum1", "glr_acstr", "glr_label"]
        self.uline = ["glr_high", "glr_uline"]
        self.calc = [
            "glr_desc", "glr_high", "glr_print", "glr_norm", "glr_store",
            "glr_snum1", "glr_acbal", "glr_cbase", "glr_ctype", "glr_snum2",
            "glr_camnt", "glr_snum3"]
        self.percent = ["glr_desc", "glr_high", "glr_snum1", "glr_snum2"]

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "General Ledger Report Generator (%s)" % __name__)
        data = [
            ("H","Heading"),
            ("L","Ledger Accounts"),
            ("G","Group of Accounts"),
            ("S","Stored Amount"),
            ("T","Total"),
            ("U","Underline"),
            ("C","Calculation"),
            ("P","Percentage")]
        typ = {
            "stype": "C",
            "titl": "Select the Required Type",
            "data": data}
        glr = {
            "stype": "R",
            "tables": ("genrpt",),
            "cols": (
                ("glr_cono", "", 0, "Coy"),
                ("glr_repno", "", 0, "Rep"),
                ("glr_type", "", 0, "T"),
                ("glr_desc", "", 0, "Description", "Y")),
            "where": [("glr_seq", "=", 0)],
            "whera": (("T", "glr_cono", 0, 0),),
            "order": "glr_cono, glr_repno",
            "index": 1}
        gls = {
            "stype": "R",
            "tables": ("genrpt",),
            "cols": (
                ("glr_seq", "", 0, "Seq"),
                ("glr_type", "", 0, "T"),
                ("glr_group", "", 0, "Grp"),
                ("glr_from", "", 0, "From-Ac"),
                ("glr_to", "", 0, "  To-Ac"),
                ("glr_desc", "", 0, "Description", "Y")),
            "where": [("glr_seq", ">", 0)],
            "whera": (("T", "glr_cono", 0, 0),("T", "glr_repno", 1, 0))}
        glm = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "where": [],
            "group": "glm_acno, glm_desc"}
        ryn = (("Yes","Y"),("No","N"))
        rys = (("Yes","Y"),("No","N"),("Debit","+"),("Credit","-"))
        rns = (("Positive","P"),("Negative","N"))
        ras = (("Add","A"),("Subtract","S"),("Ignore","I"))
        rcb = (("Percentage","P"),("Amount","A"),("Store","S"))
        rct = (("Plus","+"),("Minus","-"),("Multiply","*"),("Divide","/"))
        rsd = (("Single","S"),("Double","D"),("Blank","B"))
        pts = (
            "Accumulate Month Values",
            "Add, Subtract or Ignore",
            "Calculation Base",
            "Calculation Type",
            "Clear Stored Value",
            "Clear Total",
            "Highlight",
            "Ignore Account Type",
            "Include Opening Balance",
            "Normal Sign",
            "Percentage of Stored Value",
            "Print Values",
            "Store Amount")
        tag = [
            ("Sequence",None,None,None),
            ("Heading",None,None,None),
            ("Ledger",None,None,None),
            ("Group",None,None,None),
            ("Stored",None,None,None),
            ("Total",None,None,None),
            ("Uline",None,None,None),
            ("Calc",None,None,None),
            ("Percent",None,None,None)]
        fld = (
            (("T",0,0,0),"IUI",3,"Company","Company Number, 0=All",
                self.conum,"Y",self.doCoyNum,None,None,
                ("in", (self.conum, 0))),
            (("T",0,0,0),"IUI",3,"Report","Report Number",
                "","N",self.doRepNum,glr,None,("notzero",)),
            (("T",0,0,0),"IUA",1,"Type","Report Type (B, P, O)",
                "P","N",self.doRepTyp,None,self.doDelRpt,("in", ("O","P","B"))),
            (("T",0,0,0),"INA",30,"Heading","Report Heading",
                "","N",None,None,None,("notblank",)),
            (("T",1,0,0),"IUD",7.2,"Seq-Num","Sequence Number",
                "","N",self.doSeqNum,gls,None,("notzero",)),
            (("T",1,1,0),"IUA",1,"Sequence Type","Sequence Type",
                "","N",self.doSeqType,typ,self.doDelSeq,
                ("in",("H","L","G","T","S","U","P","C"))),
            (("T",2,0,0),"INA",30,"Description","Description",
                "","N",None,None,None,("notblank",)),
            (("T",2,1,0),("IRB",ryn),0,pts[6],pts[6],
                "Y","N",None,None,None,None),
            (("T",2,2,0),("IRB",ryn),0,"New Page","New Page (Y/N)",
                "N","N",None,None,None,None),
            (("T",2,3,0),("IRB",ryn),0,pts[7],pts[7],
                "N","N",None,None,None,None),
            (("T",3,0,0),"IUI",7,"From Account","From Account Number",
                "","N",None,glm,None,("notzero",)),
            (("T",3,1,0),"IUI",7,"To Account","To Account Number",
                "","N",None,glm,None,("efld",)),
            (("T",3,2,0),("IRB",ryn),0,pts[6],pts[6],
                "N","N",None,None,None,None),
            (("T",3,3,0),("IRB",ryn),0,pts[8],pts[8],
                "Y","N",None,None,None,None),
            (("T",3,4,0),("IRB",ryn),0,pts[0],pts[0],
                "N","N",None,None,None,None),
            (("T",3,5,0),("IRB",rys),0,pts[11],pts[11],
                "Y","N",None,None,None,None),
            (("T",3,6,0),("IRB",rns),0,pts[9],pts[9],
                "P","N",None,None,None,None),
            (("T",3,7,0),("IRB",ras),0,pts[1],pts[1],
                "A","N",None,None,None,None),
            (("T",3,8,0),("IRB",ryn),0,pts[7],pts[7],
                "N","N",None,None,None,None),
            (("T",3,9,0),("IRB",ryn),0,pts[12],pts[12],
                "N","N",self.doStore,None,None,None),
            (("T",3,10,0),"IUI",2,"Storage Number","Storage Number",
                "","N",None,None,None,("notzero",)),
            (("T",3,11,0),("IRB",ras),0,pts[1],pts[1],
                "A","N",None,None,None,None),
            (("T",4,0,0),"IUI",3,"Group Number","Group Number",
                "","N",self.doGrpNum,None,None,("efld",)),
            (("T",4,1,0),"INA",30,"Description","Description",
                "","N",None,None,None,("notblank",)),
            (("T",4,2,0),"IUI",7,"From Account","From Account Number",
                "","N",None,glm,None,("notzero",)),
            (("T",4,3,0),"IUI",7,"To Account","To Account Number",
                "","N",self.doGrpTo,glm,None,("efld",)),
            (("T",4,4,0),("IRB",ryn),0,pts[6],pts[6],
                "N","N",None,None,None,None),
            (("T",4,5,0),("IRB",ryn),0,pts[8],pts[8],
                "Y","N",None,None,None,None),
            (("T",4,6,0),("IRB",ryn),0,pts[0],pts[0],
                "N","N",None,None,None,None),
            (("T",4,7,0),("IRB",rys),0,pts[11],pts[11],
                "Y","N",None,None,None,None),
            (("T",4,8,0),("IRB",rns),0,pts[9],pts[9],
                "P","N",None,None,None,None),
            (("T",4,9,0),("IRB",ras),0,pts[1],pts[1],
                "A","N",None,None,None,None),
            (("T",4,10,0),("IRB",ryn),0,pts[7],pts[7],
                "N","N",None,None,None,None),
            (("T",4,11,0),("IRB",ryn),0,pts[12],pts[12],
                "N","N",self.doStore,None,None,None),
            (("T",4,12,0),"IUI",2,"Storage Number","Storage Number",
                "","N",None,None,None,("notzero",)),
            (("T",4,13,0),("IRB",ras),0,pts[1],pts[1],
                "A","N",None,None,None,None),
            (("T",4,14,0),"INA",10,"Chart Label","Chart Label",
                "","N",None,None,None,None),
            (("T",5,0,0),"INA",30,"Description","Description",
                "","N",None,None,None,("notblank",)),
            (("T",5,1,0),("IRB",ryn),0,pts[6],pts[6],
                "N","N",None,None,None,None),
            (("T",5,2,0),("IRB",rys),0,pts[11],pts[11],
                "Y","N",None,None,None,None),
            (("T",5,3,0),("IRB",rns),0,pts[9],pts[9],
                "P","N",None,None,None,None),
            (("T",5,4,0),("IRB",ras),0,pts[1],pts[1],
                "A","N",None,None,None,None),
            (("T",5,5,0),("IRB",ryn),0,pts[4],pts[4],
                "N","N",None,None,None,None),
            (("T",5,6,0),"IUI",2,"Storage Number","Storage Number",
                "","N",None,None,None,("notzero",)),
            (("T",5,7,0),"IUD",6.2,pts[10],pts[10],
                0,"N",None,None,None,("notzero",)),
            (("T",6,0,0),"INA",30,"Description","Description",
                "","N",None,None,None,("efld",)),
            (("T",6,1,0),("IRB",ryn),0,pts[6],pts[6],
                "Y","N",None,None,None,None),
            (("T",6,2,0),"IUI",1,"Total Level","Total Level",
                "","N",None,None,None,("between",1,9)),
            (("T",6,3,0),("IRB",rys),0,pts[11],pts[11],
                "Y","N",None,None,None,None),
            (("T",6,4,0),("IRB",rns),0,pts[9],pts[9],
                "P","N",None,None,None,None),
            (("T",6,5,0),("IRB",ryn),0,pts[5],pts[5],
                "Y","N",None,None,None,None),
            (("T",6,6,0),("IRB",ryn),0,pts[12],pts[12],
                "N","N",self.doStore,None,None,None),
            (("T",6,7,0),"IUI",2,"Storage Number","Storage Number",
                "","N",None,None,None,("notzero",)),
            (("T",6,8,0),("IRB",ras),0,pts[1],pts[1],
                "A","N",None,None,None,None),
            (("T",6,9,0),"INA",10,"Chart Label","Chart Label",
                "","N",None,None,None,None),
            (("T",7,0,0),("IRB",ryn),0,pts[6],pts[6],
                "Y","N",None,None,None,None),
            (("T",7,1,0),("IRB",rsd),0,"Underline Type","Underline Type",
                "S","N",None,None,None,None),
            (("T",8,0,0),"INA",30,"Description","Description",
                "","N",None,None,None,("notblank",)),
            (("T",8,1,0),("IRB",ryn),0,pts[6],pts[6],
                "N","N",None,None,None,None),
            (("T",8,2,0),("IRB",rys),0,pts[11],pts[11],
                "Y","N",None,None,None,None),
            (("T",8,3,0),("IRB",rns),0,pts[9],pts[9],
                "P","N",None,None,None,None),
            (("T",8,4,0),("IRB",ryn),0,pts[12],pts[12],
                "N","N",self.doStore,None,None,None),
            (("T",8,5,0),"IUI",2,"Storage Number","Storage Number",
                "","N",None,None,None,("notzero",)),
            (("T",8,6,0),("IRB",ras),0,pts[1],pts[1],
                "A","N",None,None,None,None),
            (("T",8,7,0),("IRB",rcb),0,pts[2],pts[2],
                "A","N",self.doPAS,None,None,None),
            (("T",8,8,0),("IRB",rct),0,pts[3],pts[3],
                "+","N",None,None,None,None),
            (("T",8,9,0),"IUI",2,"Storage Number","Storage Number One",
                "","N",self.doStore1,None,None,("notzero",)),
            (("T",8,10,0),"ISD",13.2,"Amount","Percent/Amount",
                "","N",self.doAmount,None,None,("efld",)),
            (("T",8,11,0),"IUI",2,"Storage Number","Storage Number Two",
                "","N",None,None,None,("efld",)),
            (("T",9,0,0),"INA",30,"Description","Description",
                "","N",None,None,None,("notblank",)),
            (("T",9,1,0),("IRB",ryn),0,pts[6],pts[6],
                "N","N",None,None,None,None),
            (("T",9,2,0),"IUI",2,"Storage Number","Storage Number One",
                "","N",None,None,None,("notzero",)),
            (("T",9,3,0),"IUI",2,"Storage Number","Storage Number Two",
                "","N",None,None,None,("notzero",)))
        tnd = (
            (self.doT0End,"y"), (self.doT1End,"n"), (self.doT2End,"y"),
            (self.doT2End,"y"), (self.doT2End,"y"), (self.doT2End,"y"),
            (self.doT2End,"y"), (self.doT2End,"y"), (self.doT2End,"y"),
            (self.doT2End,"y"))
        txt = (
            self.doT0Exit, self.doT1Exit, self.doT2Exit, self.doT2Exit,
            self.doT2Exit, self.doT2Exit, self.doT2Exit, self.doT2Exit,
            self.doT2Exit, self.doT2Exit)
        row = None
        but = (
            ("Re-Sequence",None,self.doReSeq,0,("T",1,1),("T",1,2)),
            ("Copy",None,self.doCpyRpt,0,("T",0,3),(("T",0,2),("T",0,4))),
            ("Print",None,self.doPrint,0,("T",1,1),("T",1,2)),
            ("Quit",None,self.doT0Exit,1,None,None))
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, tags=tag, butt=but)

    def doCoyNum(self, frt, pag, r, c, p, i, w):
        self.selcoy = w
        if self.selcoy == 0:
            self.df.topf[3][0][8]["where"] = None
            self.df.topf[3][1][8]["where"] = None
            self.df.topf[4][2][8]["where"] = None
            self.df.topf[4][3][8]["where"] = None
        else:
            self.df.topf[3][0][8]["where"] = [("glm_cono", "=", self.selcoy)]
            self.df.topf[3][1][8]["where"] = [("glm_cono", "=", self.selcoy)]
            self.df.topf[4][2][8]["where"] = [("glm_cono", "=", self.selcoy)]
            self.df.topf[4][3][8]["where"] = [("glm_cono", "=", self.selcoy)]

    def doRepNum(self, frt, pag, r, c, p, i, w):
        self.repno = w
        rep = self.doReadRep(self.selcoy, self.repno, 0)
        if not rep:
            self.newrep = "y"
            self.df.setWidget(self.df.B1, "normal")
        else:
            self.newrep = "n"
            self.df.setWidget(self.df.B1, "disabled")
            self.df.loadEntry(frt, pag, p+1, data=rep[3])
            self.df.loadEntry(frt, pag, p+2, data=rep[4])

    def doRepTyp(self, frt, pag, r, c, p, i, w):
        self.df.setWidget(self.df.B1, "disabled")

    def doCpyRpt(self, widget=None):
        if self.newrep != "y":
            showError(self.mf.body, "Invalid Copy Request",
                "You can only Copy a report when Creating a New report, "\
                "not when Changing an Existing report!")
            return "no"
        tit = ("Copy Existing Report Layout",)
        coy = {
            "stype": "R",
            "tables": ("ctlmst",),
            "cols": (
                ("ctm_cono", "", 0, "Com"),
                ("ctm_name", "", 0, "Name", "Y"))}
        glr = {
            "stype": "R",
            "tables": ("genrpt",),
            "cols": (
                ("glr_cono", "", 0, "Coy"),
                ("glr_repno", "", 0, "Rep"),
                ("glr_type", "", 0, "T"),
                ("glr_desc", "", 0, "Description", "Y")),
            "where": [("glr_seq", "=", 0)],
            "whera": (("T", "glr_cono", 0, 0),),
            "index": 1}
        fld = (
            (("T",0,0,0),"IUI",3,"Company Number","Company Number",
                "","N",self.doCpyCoy,coy,None,None),
            (("T",0,1,0),"IUI",3,"Report Number","Report Number",
                "","N",self.doRptNum,glr,None,None))
        state = self.df.disableButtons()
        self.cp = TartanDialog(self.mf, newscn=(0, 1), tops=True, title=tit,
            eflds=fld, tend=((self.doCpyEnd, "n"),),
        txit=(self.doCpyExit,))
        self.mf.startLoop()
        self.df.enableButtons(state=state)
        self.df.focusField("T", 0, 1)

    def doCpyCoy(self, frt, pag, r, c, p, i, w):
        self.cpycoy = w

    def doRptNum(self, frt, pag, r, c, p, i, w):
        self.cpynum = w

    def doCpyEnd(self):
        rpt = self.sql.getRec(tables="genrpt", where=[("glr_cono", "=",
            self.cpycoy), ("glr_repno", "=", self.cpynum)])
        if rpt:
            for rec in rpt:
                rec[0] = self.selcoy
                rec[1] = self.repno
                self.sql.insRec("genrpt", data=rec)
            self.mf.dbm.commitDbase()
        self.doCpyCloseProcess()

    def doCpyExit(self):
        self.doCpyCloseProcess()

    def doCpyCloseProcess(self):
        self.cp.closeProcess()
        self.mf.closeLoop()

    def doDelRpt(self):
        self.sql.delRec("genrpt", where=[("glr_cono", "=", self.selcoy),
            ("glr_repno", "=", self.repno)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doT0End(self):
        rtype = self.df.t_work[0][0][2]
        desc = self.df.t_work[0][0][3]
        if self.newrep == "y":
            data = [self.selcoy, self.repno, 0, rtype, desc]
            for x in range(5, len(self.genrpt_cl)):
                if self.genrpt_fd[self.genrpt_cl[x]][2][1] in ("A", "a"):
                    data.append("")
                else:
                    data.append(0)
            self.sql.insRec("genrpt", data=data)
        else:
            self.sql.updRec("genrpt", cols=["glr_type", "glr_desc"],
                data=[rtype, desc], where=[("glr_cono", "=", self.selcoy),
                ("glr_repno", "=", self.repno), ("glr_seq", "=", 0)])
        self.df.selPage("Sequence")
        self.df.focusField("T", 1, 1)

    def doReSeq(self, widget=None):
        whr = [("glr_cono", "=", self.selcoy), ("glr_repno", "=", self.repno)]
        all = self.sql.getRec(tables="genrpt", where=whr, order="glr_seq")
        self.sql.delRec("genrpt", where=whr)
        for seq, rec in enumerate(all):
            rec = list(rec)
            rec[2] = float(seq)
            self.sql.insRec("genrpt", data=rec)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doPrint(self, widget=None):
        self.df.setWidget(self.df.topEntry[self.df.pag][self.df.pos],
            "disabled")
        table = ["genrpt"]
        heads = ["General Ledger Report %s Layout" % self.repno]
        state = self.df.disableButtons()
        rp = RepPrt(self.mf, name="rp1030", tables=table, heads=heads,
            where=[("glr_cono", "=", self.selcoy), ("glr_repno", "=",
            self.repno)], order="glr_seq asc")
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doSeqNum(self, frt, pag, r, c, p, i, w):
        self.seqno = w
        self.seq = self.doReadRep(self.selcoy, self.repno, self.seqno)
        if not self.seq:
            self.newseq = "y"
        else:
            self.newseq = "n"
            self.df.loadEntry(frt, pag, p+1, data=self.seq[3])

    def doSeqType(self, frt, pag, r, c, p, i, w):
        self.stype = w
        if self.newseq == "y":
            return
        if self.stype == self.seq[3]:
            return
        yn = askQuestion(self.mf.body, "Change Type?", "Change the Type?",
            default="no")
        if yn == "no":
            return "No Change"
        self.newseq = "c"

    def doDelSeq(self):
        if self.newseq == "y":
            showError(self.mf.body, "Invalid Delete Request",
                "You can only delete Existing report lines.")
            return
        self.sql.delRec("genrpt", where=[("glr_cono", "=", self.selcoy),
            ("glr_repno", "=", self.repno), ("glr_seq", "=", self.seqno)])
        self.df.clearFrame("T", 1)
        self.df.focusField("T", 1, 1)
        return "nf"

    def doT1End(self):
        for x in range(2, 10):
            self.df.clearFrame("T", x)
        pag = self.pags.index(self.stype)
        if self.newseq == "n":
            cl = self.genrpt_cl
            fld = self.doLoadTypes()
            for x in range(0, len(fld)):
                data = self.seq[cl.index(fld[x])]
                self.df.loadEntry("T", pag, x, data=data)
        self.df.selPage(self.df.tags[pag - 1][0])

    def doStore(self, frt, pag, r, c, p, i, w):
        if w == "N":
            self.df.loadEntry(frt, pag, p+1, data=0)
            self.df.loadEntry(frt, pag, p+2, data="I")
            return "sk2"

    def doGrpNum(self, frt, pag, r, c, p, i, w):
        if not w:
            gno = self.sql.getRec(tables="genrpt", cols=["max(glr_group)"],
                where=[("glr_cono", "=", self.selcoy), ("glr_repno", "=",
                self.repno), ("glr_type", "=", "G")], limit=1)
            self.gno = gno[0] + 1
            self.df.loadEntry(frt, pag, p, data=self.gno)
        else:
            self.gno = w
        col = ["glr_desc", "glr_high", "glr_obal", "glr_accum",
            "glr_print", "glr_norm", "glr_acbal", "glr_ignore",
            "glr_store", "glr_snum1", "glr_acstr", "glr_label"]
        grp = ""
        for c in col:
            grp = "%s%s," % (grp, c)
        grp = grp[:-1]
        self.grp = self.sql.getRec(tables="genrpt", cols=col,
            where=[("glr_cono", "=", self.selcoy), ("glr_repno", "=",
            self.repno), ("glr_group", "=", self.gno)], group=grp)
        if self.grp:
            for n, f in enumerate(col):
                data = self.grp[0][n]
                self.df.loadEntry("T", pag, self.group.index(f), data=data)

    def doGrpTo(self, frt, pag, r, c, p, i, w):
        pass

    def doPAS(self, frt, pag, r, c, p, i, w):
        self.pas = w

    def doStore1(self, frt, pag, r, c, p, i, w):
        if self.pas == "S":
            self.df.loadEntry(frt, pag, p+1, data=0)
            return "sk1"

    def doAmount(self, frt, pag, r, c, p, i, w):
        self.df.loadEntry(frt, pag, p+1, data=0)
        return "sk1"

    def doT2End(self):
        ff = self.doLoadTypes()
        data = [self.selcoy, self.repno, self.seqno, self.stype]
        for nm, cl in enumerate(self.genrpt_cl):
            if nm < 4:
                continue
            if cl in ff:
                data.append(
                  self.df.t_work[self.pags.index(self.stype)][0][ff.index(cl)])
            elif self.genrpt_fd[cl][2][1] in ("A", "a"):
                data.append("")
            else:
                data.append(0)
        if self.newseq in ("c", "n"):
            self.doDelSeq()
        self.sql.insRec("genrpt", data=data)
        if self.stype == "G" and self.grp:
            grp = self.sql.getRec(tables="genrpt", where=[("glr_cono", "=",
                self.selcoy), ("glr_repno", "=", self.repno), ("glr_group",
                "=", self.gno)], order="glr_seq")
            for g in grp:
                if g[2] == self.seqno:
                    continue
                data[2] = g[2]
                data[8] = g[8]
                data[9] = g[9]
                self.sql.updRec("genrpt", data=data, where=[("glr_cono", "=",
                    self.selcoy), ("glr_repno", "=", self.repno), ("glr_group",
                    "=", self.gno), ("glr_seq", "=", g[2])])

        self.doT2Exit()
        if self.newseq == "y":
            self.df.loadEntry("T", 1, 0, data=(self.seqno + 1))
        else:
            self.df.clearEntry("T", 1, 1)
        self.df.clearEntry("T", 1, 2)
        self.df.focusField("T", 1, 1)

    def doLoadTypes(self):
        if self.stype == "H":
            fld = self.head
        elif self.stype == "L":
            fld = self.ledger
        elif self.stype == "G":
            fld = self.group
        elif self.stype == "S":
            fld = self.store
        elif self.stype == "T":
            fld = self.total
        elif self.stype == "U":
            fld = self.uline
        elif self.stype == "C":
            fld = self.calc
        elif self.stype == "P":
            fld = self.percent
        return fld

    def doT0Exit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

    def doT1Exit(self):
        # Check for duplicate group numbers
        errmess = """Group Number %s at Sequence Line %s is Duplicated or is Not Contiguos i.e. All Group Lines Must Follow One Another!

Please Delete, or Correct This Line!"""
        seq = self.sql.getRec(tables="genrpt", cols=["glr_type", "glr_seq",
            "glr_group"], where=[("glr_cono", "=", self.selcoy), ("glr_repno",
            "=", self.repno)], order="glr_seq")
        gp = []
        lg = 0
        for t, s, g in seq:
            if t == "G":
                if not lg and g in gp:
                    showError(self.mf.body, "Group Error", errmess % (g, s))
                    self.df.loadEntry("T", 1, 0, data=s)
                    self.df.focusField("T", 1, 1)
                    return
                if lg and not lg == g:
                    if g in gp:
                        showError(self.mf.body, "Group Error", errmess % (g, s))
                        self.df.loadEntry("T", 1, 0, data=s)
                        self.df.focusField("T", 1, 1)
                        return
                    lg = g
                    gp.append(g)
                    continue
                if not lg:
                    lg = g
                    gp.append(g)
                    continue
            else:
                lg = 0
        self.mf.dbm.commitDbase(ask=True, mess="Save All Changes?")
        self.df.focusField("T", 0, 1)

    def doT2Exit(self):
        self.df.selPage("Sequence")

    def doReadRep(self, coy, rep, seq):
        rep = self.sql.getRec(tables="genrpt", where=[("glr_cono", "=", coy),
            ("glr_repno", "=", rep), ("glr_seq", "=", seq)], limit=1)
        return rep
