#! /usr/bin/python

"""
SYNOPSIS
    General Ledger Bank Reconciliation Codes.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import GetCtl, Sql, TartanDialog
from tartanFunctions import chkCtlAcc

class gl1060:
    def __init__(self, mf, coy, nam, args=None):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.args = args
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        # Create SQL Object
        self.sql = Sql(self.mf.dbm, ["assgrp", "crsmst", "ctlctl", "ctlmst",
            "ctlvmf", "drsmst", "drschn", "genint", "genmst", "genrcc",
            "rtlprm"], prog=__name__)
        if self.sql.error:
            return self.sql.error
        # Check for Intercompany Facility
        coy = self.sql.getRec(tables="ctlmst", cols=["count(*)"],
            where=[("ctm_cono", "<>", self.conum)], limit=1)
        if not coy[0]:
            self.incoac = False
        else:
            int = self.sql.getRec(tables="genint", cols=["cti_inco"],
                where=[("cti_cono", "=", self.conum)])
            if int:
                self.incoac = [self.conum]
                [self.incoac.append(coy[0]) for coy in int]
            else:
                self.incoac = False
        #
        self.gc = GetCtl(self.mf)

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "General Ledger Bank Recon Codes (%s)" % __name__)
        ctl = {
            "stype": "R",
            "tables": ("ctlctl", "genmst"),
            "cols": (
                ("ctl_code", "", 0, "Ctl-Code"),
                ("ctl_conacc", "", 0, "G/L-Num"),
                ("glm_desc", "", 30, "Description")),
            "where": [("ctl_cono", "=", self.conum), ("ctl_code", "like",
                "bank_%"), ("glm_cono=ctl_cono",), ("glm_acno=ctl_conacc",)],
            "index": 1}
        rcc = {
            "stype": "R",
            "tables": ("genrcc",),
            "cols": (
                ("grc_memo", "", 0, "Code"),
                ("grc_desc1", "", 0, "Details"),
                ("grc_rtn", "", 0, "R"),
                ("grc_acoy", "", 0, "Coy"),
                ("grc_aacc", "", 0, "Acc-Num"),
                ("grc_acrs", "", 0, "Crs-Acc"),
                ("grc_achn", "", 0, "Chn"),
                ("grc_adrs", "", 0, "Drs-Acc"),
                ("grc_vat", "", 0, "V")),
            "where": [("grc_cono", "=", self.conum)],
            "whera": [["T", "grc_acno", 0]]}
        rct = {
            "stype": "R",
            "tables": ("genrct",),
            "cols": (
                ("grt_memo", "", 75, "Details"),
                ("grt_amount", "SD", 13.2, "Value")),
            "where": [
                ("grt_cono", "=", self.conum),
                ("grt_flag", "=", "N")],
            "whera": [["T", "grt_acno", 0, 0]],
            "group": "grt_memo",
            "order": "grt_memo",
            "comnd": self.doSameField}
        coy = {
            "stype": "R",
            "tables": ("ctlmst",),
            "cols": (
                ("ctm_cono", "", 0, "Coy"),
                ("ctm_name", "", 0, "Name", "Y"))}
        glm = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "whera": [["T", "glm_cono", 7]]}
        chn = {
            "stype": "R",
            "tables": ("drschn",),
            "cols": (
                ("chm_chain", "", 0, "Num"),
                ("chm_name", "", 0, "Name", "Y")),
            "whera": [["T", "chm_cono", 7]]}
        self.crm = {
            "stype": "R",
            "tables": ("crsmst",),
            "cols": (
                ("crm_acno", "", 0, "Acc-Num"),
                ("crm_name", "", 0, "Name", "Y"),
                ("crm_add1", "", 0, "Address Line 1")),
            "whera": [["T", "crm_cono", 7]]}
        self.drm = {
            "stype": "R",
            "tables": ("drsmst",),
            "cols": (
                ("drm_acno", "", 0, "Acc-Num"),
                ("drm_name", "", 0, "Name", "Y"),
                ("drm_add1", "", 0, "Address Line 1")),
            "whera": [
                ["T", "drm_cono", 7],
                ["T", "drm_chain", 11]]}
        vat = {
            "stype": "R",
            "tables": ("ctlvmf",),
            "cols": (
                ("vtm_code", "", 0, "Acc-Num"),
                ("vtm_desc", "", 0, "Description", "Y")),
            "whera": [["T", "vtm_cono", 7]]}
        r1s = (("Payment","P"), ("Deposit","D"))
        r2s = (("Oldest","O"), ("Current","C"))
        fld = [
            [("T",0,0,0),"IUI",7,"Bank Account","Bank Account",
                "","Y",self.doAccNum,ctl,None,None],
            (("T",0,0,0),"ONA",30,""),
            [("T",0,1,0),"IUI",5,"Memo Code","Memo Code (Blank for Next)",
                "","Y",self.doMemo,rcc,None,None],
            [("T",0,2,0),"INA",50,"Memo Desc Contains","Memo Desc Contains",
                "","N",self.doDesc1,rct,self.doDelete,("notblank",)],
            (("T",0,3,0),"INA",50,"      and Contains","Desc Also Contains",
                "","N",self.doDesc2,rct,None,None),
            (("T",0,4,0),"INA",50,"      and Contains","Desc Also Contains",
                "","N",self.doDesc3,rct,None,None),
            (("T",0,5,0),("IRB",r1s),0,"Transaction Type","Transaction Type",
                "P","N",self.doTypCod,None,None,None),
            (("T",0,6,0),"IUI",7,"Allocation Company","Allocation Company",
                self.conum,"N",self.doAllCoy,coy,None,("notzero",)),
            (("T",0,6,0),"ONA",30,""),
            (("T",0,7,0),"IUI",7,"Allocation Account","Allocation Account",
                "","N",self.doAllAcc,glm,None,("notzero",)),
            (("T",0,7,0),"ONA",30,""),
            [["T",0,8,0],"IUI",3,"Chn","Chain Store Code",
                0,"N",self.doDrsChn,chn,None,("efld",)],
            (("T",0,8,0),"ONA",30,""),
            [("T",0,9,0),"INA",7,"Acc-Num","Account Number",
                "","N",self.doCrsDrsAcc,self.crm,None,("notblank",)],
            (("T",0,9,0),"ONA",30,""),
            (("T",0,10,0),("IRB",r2s),0,"Ageing Code","Ageing Code",
                "O","N",self.doCrsDrsAge,None,None,None,None,
                """
The Ageing Codes are as follows:

Oldest  - Automatically age against the oldest outstanding transactions.

Current - Do not age the transaction.
"""),
            (("T",0,11,0),"INA",1,"Vat Code","Vat Code",
                "","N",self.doVatCod,vat,None,None)]
        but = [
            ("Cancel",None,self.doCancel,0,("T",0,4),("T",0,1)),
            ("Quit",None,self.doExit,1,None,None)]
        if self.args:
            for f in (0, 2, 3):
                if f == 3:
                    fld[f][4] += "(noesc)"
                else:
                    fld[f][7] = None
                fld[f][8] = None
                fld[f][9] = None
            del but[0]
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        if self.args:
            foc = False
        else:
            foc = True
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, butt=but,
            tend=tnd, txit=txt, focus=foc)
        if self.args:
            self.df.loadEntry("T", 0,  0, data=self.args[0])
            self.doAccNum("T", 0, 0, 1, 0, 0, self.args[0])
            self.df.loadEntry("T", 0,  2, data=0)
            self.doMemo("T", 0, 0, 3, 2, 2, 0)
            self.df.loadEntry("T", 0,  3, data=self.args[1][2])
            self.df.focusField("T", 0, 4)

    def doAccNum(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables=["ctlctl", "genmst"], cols=["glm_desc"],
            where=[("glm_cono", "=", self.conum), ("glm_acno", "=", w),
            ("ctl_cono=glm_cono",), ("ctl_conacc=glm_acno",), ("ctl_code",
            "like", "bank_%")], limit=1)
        if not acc:
            return "Invalid Bank Account"
        self.acno = w
        self.df.loadEntry(frt, pag, p+1, data=acc[0])

    def doSameField(self, frt, pag, r, c, p, i, w):
        self.df.loadEntry("T", 0, self.df.pos, data=w[0])
        self.df.focusField("T", 0, self.df.col)

    def doMemo(self, frt, pag, r, c, p, i, w):
        if not w:
            w = self.sql.getRec(tables="genrcc", cols=["max(grc_memo)"],
                where=[("grc_cono", "=", self.conum), ("grc_acno", "=",
                self.acno)], limit=1)
            if not w or not w[0]:
                self.memo = 1
            else:
                self.memo = w[0] + 1
            self.df.loadEntry(frt, pag, p, data=self.memo)
        else:
            self.memo = w
        mem = self.sql.getRec(tables="genrcc", where=[("grc_cono", "=",
            self.conum), ("grc_acno", "=", self.acno), ("grc_memo", "=",
            self.memo)], limit=1)
        if not mem:
            self.new = "y"
        else:
            self.new = "n"
            col = self.sql.genrcc_col
            self.desc1  = mem[col.index("grc_desc1")]
            self.desc2  = mem[col.index("grc_desc2")]
            self.desc3  = mem[col.index("grc_desc3")]
            self.rtn    = mem[col.index("grc_rtn")]
            self.acoy   = mem[col.index("grc_acoy")]
            self.aacc   = mem[col.index("grc_aacc")]
            self.acrs  = mem[col.index("grc_acrs")]
            self.achn  = mem[col.index("grc_achn")]
            self.aage  = mem[col.index("grc_aage")]
            self.adrs  = mem[col.index("grc_adrs")]
            self.vat    = mem[col.index("grc_vat")]
            self.df.loadEntry(frt, pag, 3, data=self.desc1)
            self.df.loadEntry(frt, pag, 4, data=self.desc2)
            self.df.loadEntry(frt, pag, 5, data=self.desc3)
            self.df.loadEntry(frt, pag, 6, data=self.rtn)
            self.df.loadEntry(frt, pag, 7, data=self.acoy)
            self.df.loadEntry(frt, pag, 8, data=self.getCoyName())
            self.df.loadEntry(frt, pag, 9, data=self.aacc)
            self.df.loadEntry(frt, pag, 10, data=self.getAccDesc())
            # Creditor and Debtor Details
            if self.acrs:
                self.crsdrs = "crs"
                self.df.loadEntry(frt, pag, 11, data=0)
                self.df.loadEntry(frt, pag, 12, data="")
                self.df.loadEntry(frt, pag, 13, data=self.acrs)
                self.df.loadEntry(frt, pag, 14, data=self.getCrsName())
                self.df.loadEntry(frt, pag, 15, data=self.aage)
            elif self.adrs:
                self.crsdrs = "drs"
                self.df.loadEntry(frt, pag, 11, data=self.achn)
                if self.achn:
                    self.df.loadEntry(frt, pag, 12, data=self.getChainName())
                else:
                    self.df.loadEntry(frt, pag, 12, data="")
                self.df.loadEntry(frt, pag, 13, data=self.adrs)
                self.df.loadEntry(frt, pag, 14, data=self.getDrsName())
                self.df.loadEntry(frt, pag, 15, data=self.aage)
            else:
                self.crsdrs = None
                self.df.loadEntry(frt, pag, 11, data=0)
                self.df.loadEntry(frt, pag, 12, data="")
                self.df.loadEntry(frt, pag, 13, data="")
                self.df.loadEntry(frt, pag, 14, data="")
                self.df.loadEntry(frt, pag, 15, data="")
            # VAT Code
            self.df.loadEntry(frt, pag, 16, data=self.vat)

    def doDesc1(self, frt, pag, r, c, p, i, w):
        self.desc1 = w

    def doDesc2(self, frt, pag, r, c, p, i, w):
        self.desc2 = w
        if not self.desc2:
            self.desc3 = ""
            self.df.loadEntry(frt, pag, p+1, data=self.desc3)
            if self.args:
                if self.args[1][-1] < 0:
                    self.rtn = "P"
                else:
                    self.rtn = "D"
                self.df.loadEntry(frt, pag, p+2, data=self.rtn)
                return "sk2"
            return "sk1"

    def doDesc3(self, frt, pag, r, c, p, i, w):
        self.desc3 = w
        if self.args:
            if self.args[1][-1] < 0:
                self.rtn = "P"
            else:
                self.rtn = "D"
            self.df.loadEntry(frt, pag, p+1, data=self.rtn)
            return "sk1"

    def doTypCod(self, frt, pag, r, c, p, i, w):
        self.rtn = w
        if not self.incoac:
            self.acoy = self.conum
            if self.doChkLoadCtls():
                return "rf"
            else:
                self.df.loadEntry(frt, pag, p+1, data=self.acoy)
                self.df.loadEntry(frt, pag, p+2, data=self.getCoyName())
                return "sk2"

    def doAllCoy(self, frt, pag, r, c, p, i, w):
        self.acoy = w
        name = self.getCoyName()
        if not name:
            return "Invalid Company Number"
        self.df.loadEntry(frt, pag, p+1, data=name)
        return self.doChkLoadCtls()

    def doChkLoadCtls(self):
        # Check for Intercompany Records
        if self.acoy != self.conum:
            acc = self.sql.getRec(tables="genint", where=[("cti_cono", "=",
                self.conum), ("cti_inco", "=", self.acoy)], limit=1)
            if not acc:
                return "Invalid Company, No Intercompany Record 1"
            acc = self.sql.getRec(tables="genint", where=[("cti_cono", "=",
                self.acoy), ("cti_inco", "=", self.conum)], limit=1)
            if not acc:
                return "Invalid Company, No Intercompany Record 2"
        # Get Company Details
        ctlmst = self.gc.getCtl("ctlmst", self.acoy)
        if not ctlmst:
            return "rf"
        # Set Company VAT Default
        self.taxdf = ctlmst["ctm_taxdf"]
        if not self.taxdf:
            self.taxdf = "N"
        # Check for Integrated Systems
        mod = []
        for x in range(0, 21, 2):
            mod.append(ctlmst["ctm_modules"][x:x+2])
        # Load Ass Integration
        if mod.count("AR"):
            ctl = self.gc.getCtl("assctl", self.acoy)
            if not ctl:
                return "rf"
            self.ass_gl = ctl["cta_glint"]
            self.assctl = []
            ass = self.sql.getRec(tables="assgrp", cols=["asg_assacc"],
                where=[("asg_cono", "=", self.acoy)])
            for a in ass:
                self.assctl.append(a[0])
        else:
            self.ass_gl = "N"
        # Load Crs Integration
        if mod.count("CR"):
            ctl = self.gc.getCtl("crsctl", self.acoy)
            if not ctl:
                return "rf"
            self.crs_gl = ctl["ctc_glint"]
        else:
            self.crs_gl = "N"
        # Load Drs Integration and Chain Store Flag
        if mod.count("DR"):
            ctl = self.gc.getCtl("drsctl", self.acoy)
            if not ctl:
                return "rf"
            self.drs_gl = ctl["ctd_glint"]
            self.drs_ch = ctl["ctd_chain"]
        else:
            self.drs_gl = "N"
        # Load Rtl Integration
        if mod.count("RT"):
            ctl = self.gc.getCtl("rtlctl", self.acoy)
            if not ctl:
                return "rf"
            self.rtl_gl = ctl["ctr_glint"]
            self.rtlctl = []
            rtl = self.sql.getRec(tables="rtlprm", cols=["rtp_rtlacc"],
                where=[("rtp_cono", "=", self.acoy)])
            for r in rtl:
                self.rtlctl.append(r[0])
        else:
            self.rtl_gl = "N"
        # Load Rca Integration
        if mod.count("RC"):
            ctl = self.gc.getCtl("rcactl", self.acoy)
            if not ctl:
                return "rf"
            self.rca_gl = ctl["cte_glint"]
        else:
            self.rca_gl = "N"
        # Check for Control Records e.g. VAT and Discounts
        self.ctlctl = self.gc.getCtl("ctlctl", self.acoy)
        if not self.ctlctl:
            return "rf"
        ctls = ["vat_ctl"]
        if self.crs_gl == "Y":
            ctls.extend(["crs_ctl", "dis_rec"])
        if self.drs_gl == "Y":
            ctls.extend(["drs_ctl", "dis_all"])
        if self.gc.chkRec(self.acoy, self.ctlctl, ctls):
            return "rf"
        if self.rca_gl == "Y":
            self.rcactl = [self.ctlctl["rca_own"], self.ctlctl["rca_tnt"]]

    def doAllAcc(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="genmst", cols=["glm_desc", "glm_vat"],
            where=[("glm_cono", "=", self.acoy), ("glm_acno", "=", w)], limit=1)
        if not acc:
            return "Invalid Account Number"
        self.crsdrs = None
        if self.crs_gl == "Y" and w == self.ctlctl["crs_ctl"]:
            self.crsdrs = "crs"
            self.df.topf[0][p+4][8] = self.crm
        elif self.drs_gl == "Y" and w == self.ctlctl["drs_ctl"]:
            self.crsdrs = "drs"
            self.df.topf[0][p+4][8] = self.drm
        elif self.ass_gl == "Y" and w in self.assctl:
            return "This Is An Integrated Asset Control Account"
        elif self.rtl_gl == "Y" and w in self.rtlctl:
            return "This Is An Integrated Rental Control Account"
        elif self.rca_gl == "Y" and w in self.rcactl:
            return "This Is An Integrated Rental Control Account"
        self.aacc = w
        self.df.loadEntry(frt, pag, p+1, data=acc[0])
        if self.crsdrs:
            if self.crsdrs == "crs" or self.drs_ch == "N":
                self.achn = 0
                self.df.topf[0][p+2][1] = "OUI"
                self.df.loadEntry(frt, pag, p+2, data=self.achn)
                self.df.loadEntry(frt, pag, p+3, data="")
                return "sk3"
            elif self.crsdrs == "drs":
                self.df.topf[0][p+2][1] = "IUI"
                return "sk1"
        else:
            self.acrs = ""
            self.achn = 0
            self.adrs = ""
            self.aage = ""
            self.df.loadEntry(frt, pag, p+2, data=0)
            self.df.loadEntry(frt, pag, p+3, data="")
            self.df.loadEntry(frt, pag, p+4, data="")
            self.df.loadEntry(frt, pag, p+5, data="")
            self.df.loadEntry(frt, pag, p+6, data="")
            if not self.df.t_work[pag][0][p+7]:
                if acc[1]:
                    self.df.loadEntry(frt, pag, p+7, data=acc[1])
                else:
                    self.df.loadEntry(frt, pag, p+7, data=self.taxdf)
            return "sk6"

    def doDrsChn(self, frt, pag, r, c, p, i, w):
        self.achn = w
        name = self.getChainName()
        if not name:
            return "Invalid Account Number"
        self.df.loadEntry(frt, pag, p+1, data=name)

    def doCrsDrsAcc(self, frt, pag, r, c, p, i, w):
        if self.crsdrs == "crs":
            self.acrs = w
            name = self.getCrsName()
        else:
            self.adrs = w
            name = self.getDrsName()
        if not name:
            return "Invalid %s Account Number" % self.crsdrs.capitalize()
        self.df.loadEntry(frt, pag, p+1, data=name)

    def doCrsDrsAge(self, frt, pag, r, c, p, i, w):
        self.aage = w
        self.vat = "N"
        self.df.loadEntry(frt, pag, p+1, data=self.vat)
        return "sk1"

    def doVatCod(self, frt, pag, r, c, p, i, w):
        vat = self.sql.getRec(tables="ctlvmf", cols=["vtm_desc"],
            where=[("vtm_cono", "=", self.acoy), ("vtm_code", "=", w)], limit=1)
        if not vat:
            return "Invalid VAT Code"
        self.vat = w

    def doDelete(self):
        self.sql.delRec("genrcc", where=[("grc_cono", "=", self.conum),
            ("grc_acno", "=", self.acno), ("grc_memo", "=", self.memo)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 3)

    def doEnd(self):
        data = [self.conum, self.acno, self.memo, self.desc1, self.desc2,
            self.desc3, self.rtn, self.acoy, self.aacc]
        if self.crsdrs:
            if self.crsdrs == "crs":
                data.extend([self.acrs, 0, "", self.aage, self.vat])
            else:
                data.extend(["", self.achn, self.adrs, self.aage, self.vat])
        else:
                data.extend(["", 0, "", "", self.vat])
        if self.new == "y":
            self.sql.insRec("genrcc", data=data)
        else:
            self.sql.updRec("genrcc", data=data, where=[("grc_cono", "=",
                self.conum), ("grc_acno", "=", self.acno), ("grc_memo", "=",
                self.memo)])
        self.mf.dbm.commitDbase()
        if self.args:
            self.doExit()
        else:
            for x in range(2, self.df.topq[0]):
                self.df.loadEntry("T", 0, x, data="")
            self.df.focusField("T", 0, 3)

    def getCoyName(self):
        coy = self.sql.getRec(tables="ctlmst", cols=["ctm_name"],
            where=[("ctm_cono", "=", self.acoy)], limit=1)
        if coy:
            return coy[0]

    def getAccDesc(self):
        acc = self.sql.getRec(tables="genmst", cols=["glm_desc"],
            where=[("glm_cono", "=", self.acoy), ("glm_acno", "=", self.aacc)],
            limit=1)
        if acc:
            return acc[0]

    def getCrsName(self):
        acc = self.sql.getRec(tables="crsmst", cols=["crm_name"],
            where=[("crm_cono", "=", self.acoy), ("crm_acno", "=", self.acrs)],
            limit=1)
        if acc:
            return acc[0]

    def getChainName(self):
        acc = self.sql.getRec(tables="drschn", cols=["chm_name"],
            where=[("chm_cono", "=", self.acoy), ("chm_chain", "=", self.achn)],
            limit=1)
        if acc:
            return acc[0]

    def getDrsName(self):
        acc = self.sql.getRec(tables="drsmst", cols=["drm_name"],
            where=[("drm_cono", "=", self.acoy), ("drm_chain", "=", self.achn),
            ("drm_acno", "=", self.adrs)], limit=1)
        if acc:
            return acc[0]

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()
