#! /usr/bin/python

"""
SYNOPSIS
    General Ledger Transaction Audit Trail.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, doWriteExport, getModName, doPrinter
from tartanFunctions import showError
from tartanWork import gltrtp

class gl3020:
    def __init__(self, mf, coy, nam, per):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = per
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlmst", "genmst", "gentrn"],
            prog=__name__)
        # Check for Multiple Companies
        coy = self.sql.getRec(tables="ctlmst", cols=["count(*)"], limit=1)
        if coy[0]:
            self.multi = "Y"
        else:
            self.multi = "N"
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i) %6s" % (t[0],
            t[1], t[2], t[3], t[4], __name__)
        self.head = ("%03u %-30s %s" % (self.conum, self.conam, "%s"))
        self.colsh = ["TP", "BatchNo", "Coy", "Acc-Num", "Description",
            "Reference", "Date", "Debits", "Credits", "Tax-Amount", "T",
            "Remarks"]
        self.forms = [("UI", 2, False, False, True), ("Na", 7),
            ("UI", 3, False, False, True), ("UI", 7, False, False, True),
            ("NA", 30), ("Na", 9), ("D1", 10), ("SD", 13.2), ("SD", 13.2),
            ("SD", 13.2), ("UA", 1), ("NA", 30)]
        self.s_per = self.period[1][0] / 100
        self.e_per = self.period[2][0] / 100
        self.gqt = [0] * (len(gltrtp) + 1)
        self.gdr = [0] * (len(gltrtp) + 1)
        self.gcr = [0] * (len(gltrtp) + 1)
        self.gvt = [0] * (len(gltrtp) + 1)
        self.totind = "N"
        self.other = "N"
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "General Ledger Audit Trail (%s)" % __name__)
        data = ["All Types"]
        for x in range(1, len(gltrtp)+1):
            data.append(gltrtp[x-1][1])
        btt = {
            "stype": "C",
            "titl": "Valid Types",
            "data": data,
            "retn": "I"}
        btm = {
            "stype": "R",
            "tables": ("gentrn",),
            "cols": (
                ("glt_batch", "", 0, "Bat-Num"),
                ("glt_type", ("xx", gltrtp), 20, "Type"),
                ("glt_curdt", "", 0, "Cur-Dat")),
            "where": [],
            "group": "glt_batch, glt_type, glt_curdt",
            "order": "glt_type, glt_curdt, glt_batch"}
        r1s = (("Financial","F"),("Capture","C"))
        r2s = (("Yes","Y"),("No","N"))
        fld = [
            (("T",0,0,0),("IRB",r1s),0,"Period Type","Period Type",
                "F","Y",self.doPtyp,None,None,None),
            (("T",0,1,0),"Id2",7,"Starting Period","Starting Period",
                self.s_per,"Y",self.doStartPer,None,None,("efld",)),
            (("T",0,2,0),"Id2",7,"Ending Period","Ending Period",
                self.e_per,"Y",self.doEndPer,None,None,("efld",)),
            (("T",0,3,0),"Id1",10,"Starting Date","Starting Date",
                self.sysdtw,"Y",self.doStartDat,None,None,("efld",)),
            (("T",0,4,0),"Id1",10,"Ending Date","Ending Date",
                self.sysdtw,"Y",self.doEndDat,None,None,("efld",)),
            (("T",0,5,0),"IUI",1,"Type","Transaction Type",
                "","Y",self.doBatTyp,btt,None,None),
            (("T",0,6,0),"INa",7,"Batch-Number","Batch Number",
                "","Y",self.doBatNum,btm,None,None),
            (("T",0,7,0),("IRB",r2s),0,"Totals Only (Y/N)","Totals Only (Y/N)",
                "Y","Y",self.doTots,None,None,None)]
        if self.multi == "Y":
            fld.append(
            (("T",0,8,0),("IRB",r2s),10,"Include Other Companies","Include "\
                "Other Companies","N","N",self.doCoy,None,None,None))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"))

    def doPtyp(self, frt, pag, r, c, p, i, w):
        self.ptyp = w
        if self.ptyp == "C":
            return "sk2"

    def doStartPer(self, frt, pag, r, c, p, i, w):
        if w == 0:
            w = self.s_per
        elif w < self.s_per or w > self.e_per:
            return "Invalid Period"
        self.sperw = w
        self.sperd = self.df.t_disp[pag][0][i]
        self.df.loadEntry(frt, pag, p, data=self.sperw)

    def doEndPer(self, frt, pag, r, c, p, i, w):
        if w == 0:
            w = self.e_per
        elif w < self.sperw or w > self.e_per:
            return "Invalid Period"
        self.eperw = w
        self.eperd = self.df.t_disp[pag][0][i]
        self.df.loadEntry(frt, pag, p, self.eperw)
        return "sk2"

    def doStartDat(self, frt, pag, r, c, p, i, w):
        self.sdatw = w
        self.sperd = self.df.t_disp[pag][0][i]
        self.df.loadEntry(frt, pag, p, data=self.sdatw)

    def doEndDat(self, frt, pag, r, c, p, i, w):
        if w < self.sdatw:
            return "Invalid Date"
        self.edatw = w
        self.eperd = self.df.t_disp[pag][0][i]
        self.df.loadEntry(frt, pag, p, self.edatw)

    def doBatTyp(self, frt, pag, r, c, p, i, w):
        if w > len(gltrtp):
            return "Invalid Batch Type"
        self.btyp = w
        whr = [("glt_cono", "=", self.conum)]
        if self.ptyp == "F":
            whr.append(("glt_curdt", "between", self.sperw, self.eperw))
        else:
            whr.append(("glt_capdt", "between", self.sdatw, self.edatw))
        if self.btyp:
            whr.append(("glt_type", "=", self.btyp))
        self.df.topf[pag][i+1][8]["where"] = whr

    def doBatNum(self, frt, pag, r, c, p, i, w):
        self.batch = w

    def doTots(self, frt, pag, r, c, p, i, w):
        self.totsonly = w
        if self.totsonly == "Y":
            if self.multi == "Y":
                self.df.topEntry[0][9][2][0].hide()
                self.df.topEntry[0][9][3][0].hide()
            else:
                self.df.topEntry[0][8][2][0].hide()
                self.df.topEntry[0][8][3][0].hide()
        else:
            if self.multi == "Y":
                self.df.topEntry[0][9][2][0].show()
                self.df.topEntry[0][9][3][0].show()
            else:
                self.df.topEntry[0][8][2][0].show()
                self.df.topEntry[0][8][3][0].show()

    def doCoy(self, frt, pag, r, c, p, i, w):
        self.other = w

    def doEnd(self):
        self.df.closeProcess()
        col = ["glt_cono", "glt_acno", "glt_trdt", "glt_type", "glt_refno",
            "glt_batch", "glt_tramt", "glt_taxamt", "glt_desc", "glt_taxind",
            "glm_desc"]
        if self.other == "Y":
            self.doSelCoy()
            if self.con == "X":
                self.closeProcess()
                return
            whr = [("glt_cono", "in", tuple(self.con))]
        else:
            whr = [("glt_cono", "=", self.conum)]
        if self.ptyp == "F":
            whr.append(("glt_curdt", "between", self.sperw,
            self.eperw))
        else:
            whr.append(("glt_capdt", "between", self.sdatw, self.edatw))
        if self.btyp:
            whr.append(("glt_type", "=", self.btyp))
        if self.batch:
            whr.append(("glt_batch", "=", self.batch))
        whr.extend([("glt_cono=glm_cono",), ("glt_acno=glm_acno",)])
        ord = "glt_type, glt_batch, glt_refno"
        all = self.sql.getRec(tables=["genmst", "gentrn"], cols=col, where=whr,
            order=ord)
        if not all:
            showError(self.mf.body, "Transaction Error",
            "No Transactions Selected")
        elif self.df.repprt[0][2] == "export":
            self.exportReport(all)
        else:
            self.printReport(all)
        self.closeProcess()

    def doSelCoy(self):
        tit = ("Company Selection",)
        self.coys = self.sql.getRec(tables="ctlmst", cols=["ctm_cono",
            "ctm_name"], order="ctm_cono")
        coy = {
            "stype": "C",
            "titl": "",
            "data": self.coys,
            "mode": "M",
            "comnd": self.doCoyCmd}
        r1s = (("Yes","Y"),("Include","I"),("Exclude","E"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"All Companies","All Companies",
                "Y","N",self.doAllCoy,None,None,None,None),
            (("T",0,1,0),"INA",30,"Companies","Companies",
                "","N",self.doCoySel,coy,None,None,None))
        self.cf = TartanDialog(self.mf, screen=self.mf.body, tops=True,
            title=tit, eflds=fld, tend=((self.doCoyEnd, "y"),),
            txit=(self.doCoyExit,))
        self.mf.startLoop()

    def doAllCoy(self, frt, pag, r, c, p, i, w):
        self.con = w
        if self.con == "Y":
            self.con = []
            for coy in self.coys:
                self.con.append(coy[0])
            return "nd"
        elif self.con == "I":
            self.cf.topf[pag][i+1][8]["titl"] = "Select Companies to Include"
        else:
            self.cf.topf[pag][i+1][8]["titl"] = "Select Companies to Exclude"

    def doCoyCmd(self, frt, pag, r, c, p, i, w):
        c = ""
        for co in w:
            if int(co[0]):
                c = c + str(int(co[0])) + ","
        if len(c) > 1:
            c = c[:-1]
        self.cf.loadEntry(frt, pag, p, data=c)

    def doCoySel(self, frt, pag, r, c, p, i, w):
        if w[-1:
           ] == ",": w = w[:-1]
        self.coy = w.split(",")

    def doCoyEnd(self):
        if self.con == "I":
            self.con = self.coy
        elif self.con == "E":
            self.con = []
            for co in self.coys:
                self.con.append(int(co[0]))
            for co in self.coy:
                del self.con[self.con.index(int(co))]
            self.con.sort()
        self.doCoyClose()

    def doCoyExit(self):
        self.con = "X"
        self.doCoyClose()

    def doCoyClose(self):
        self.cf.closeProcess()
        self.mf.closeLoop()

    def exportReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        expnam = getModName(self.mf.wrkdir, __name__, self.conum)
        self.expheads = [self.head % self.sysdttm]
        self.expheads.append("General Ledger Audit Trail for Period "\
            "%s to %s" % (self.sperd, self.eperd))
        self.expcolsh = [self.colsh]
        self.expforms = self.forms
        self.expdatas = []
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                p.closeProgress()
                return
            vals = self.getValues(dat)
            if not vals:
                continue
            cono, acno, trdt, trtp, refno, batch, debit, credit, taxamt, \
                detail, taxind, desc = vals
            line = ["BODY", [trtp.work, batch.work, cono.work, acno.work,
                desc.work, refno.work, trdt.work, debit.work, credit.work,
                taxamt.work, taxind.work, detail.work]]
            self.expdatas.append(line)
        p.closeProgress()
        self.grandTotal()
        doWriteExport(xtype=self.df.repprt[0][1], name=expnam, sheet=__name__,
            heads=self.expheads, colsh=self.expcolsh, forms=self.expforms,
            datas=self.expdatas, rcdic=self.mf.rcdic)

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        if self.totsonly == "Y":
            self.head = ("%03u %-30s %49s %6s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        else:
            self.head = ("%03u %-30s %91s %10s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        pdfnam = getModName(self.mf.wrkdir, __name__, self.conum, ext="pdf")
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, pdfnam, self.head)
        self.bqty = 0
        self.bdrs = 0
        self.bcrs = 0
        self.bvat = 0
        self.tqty = 0
        self.tdrs = 0
        self.tcrs = 0
        self.tvat = 0
        self.trtp = 0
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            vals = self.getValues(dat)
            if not vals:
                continue
            cono, acno, trdt, trtp, refno, batch, debit, credit, taxamt, \
                detail, taxind, desc = vals
            if self.trtp == 0:
                self.trtp = trtp.work
                self.batch = batch.work
            if trtp.work != self.trtp:
                self.batchTotal()
                self.typeTotal()
                self.trtp = trtp.work
                self.batch = batch.work
                self.pglin = 99
            if batch.work != self.batch:
                self.batchTotal()
                self.batch = batch.work
                if self.totsonly != "Y":
                    self.typeHeading()
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            if self.totsonly != "Y":
                self.t.textLine("%s %s %s %s %s %s %s %s %s %s" % \
                    (cono.disp, acno.disp, desc.disp, refno.disp,
                    trdt.disp, debit.disp, credit.disp, taxamt.disp,
                    taxind.disp, detail.disp))
                self.pglin = self.pglin + 1
            # Batch Totals
            self.bqty += 1
            self.bdrs = float(ASD(self.bdrs) + ASD(debit.work))
            self.bcrs = float(ASD(self.bcrs) + ASD(credit.work))
            self.bvat = float(ASD(self.bvat) + ASD(taxamt.work))
            # Type Totals
            self.tqty += 1
            self.tdrs = float(ASD(self.tdrs) + ASD(debit.work))
            self.tcrs = float(ASD(self.tcrs) + ASD(credit.work))
            self.tvat = float(ASD(self.tvat) + ASD(taxamt.work))
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.batchTotal()
                self.typeTotal()
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])

    def getValues(self, data):
        cono = CCD(data[0], "UI", 3)
        acno = CCD(data[1], "UI", 7)
        trdt = CCD(data[2], "D1", 10)
        trtp = CCD(data[3], "UI", 2)
        refno = CCD(data[4], "Na", 9)
        batch = CCD(data[5], "Na", 7)
        if data[6] < 0:
            debit = CCD(0, "SD", 13.2)
            credit = CCD(data[6], "SD", 13.2)
        else:
            debit = CCD(data[6], "SD", 13.2)
            credit = CCD(0, "SD", 13.2)
        taxamt = CCD(data[7], "SD", 13.2)
        detail = CCD(data[8], "NA", 30)
        taxind = CCD(data[9], "NA", 1)
        desc = CCD(data[10], "NA", 30)
        self.gqt[trtp.work - 1] = self.gqt[trtp.work - 1] + 1
        self.gdr[trtp.work - 1] = float(ASD(self.gdr[trtp.work - 1]) + \
            ASD(debit.work))
        self.gcr[trtp.work - 1] = float(ASD(self.gcr[trtp.work - 1]) + \
            ASD(credit.work))
        self.gvt[trtp.work - 1] = float(ASD(self.gvt[trtp.work - 1]) + \
            ASD(taxamt.work))
        return (cono, acno, trdt, trtp, refno, batch, debit, credit, taxamt,
            detail, taxind, desc)

    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        if self.totsonly == "Y":
            self.t.textLine(
            "%-37s %-10s %-2s %-10s %22s %5s" % \
            ("General Ledger Audit Trail for Period",
            self.sperd, "to", self.eperd, "Page", self.pgnum))
        else:
            self.t.textLine(
            "%-37s %-10s %-2s %-10s %68s %5s" % \
            ("General Ledger Audit Trail for Period",
            self.sperd, "to", self.eperd, "Page", self.pgnum))
        self.t.textLine()
        self.pglin = 4
        if self.totind == "N":
            self.typeHeading()
        else:
            self.t.textLine("%-14s" % "Totals Summary")
            self.t.textLine()
            if self.totsonly == "Y":
                self.t.textLine(
                "%-27s%-8s  %-13s %-13s %-13s %-13s" %
                ("Document Type", "Quantity", "      Debits",
                "     Credits", "  Difference", "      V.A.T."))
            else:
                self.t.textLine(
                "%-34s%-8s  %-13s %-13s %-13s %-13s" %
                ("Document Type", "Quantity", "      Debits",
                "     Credits", "  Difference", "      V.A.T."))
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.t.setFont("Tartan", self.cpi, self.ldg)
            self.pglin = self.pglin + 4

    def typeHeading(self):
        if self.totsonly != "Y":
            batch = self.batch
        else:
            batch = "Various"
        if self.lpp - self.pglin < 7:
            self.c.drawText(self.t)
            self.c.showPage()
            self.pageHeading()
            return
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%-7s %7s %-10s %3s" % \
            ("Batch", batch, "    Type", gltrtp[self.trtp - 1][1]))
        self.t.textLine()
        if self.totsonly == "Y":
            self.t.textLine(
            "%-36s %-13s %-13s %-13s %-13s" %
            ("Details", "      Debits", "     Credits", "  Difference",
                "      V.A.T."))
        else:
            self.t.textLine(
            "%-3s %-7s %-30s %-9s %-10s %-13s %-13s %-13s %-1s %-30s" % \
            ("Coy", "Acc-Num", "Description", "Reference", "   Date",
            "      Debits", "     Credits", "  Tax-Amount", "T", "Remarks"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = self.pglin + 4

    def batchTotal(self):
        j = CCD(self.bdrs, "SD", 13.2)
        k = CCD(self.bcrs, "SD", 13.2)
        l = CCD(float(ASD(j.work) + ASD(k.work)), "SD", 13.2)
        m = CCD(self.bvat, "SD", 13.2)
        if self.totsonly == "Y":
            self.t.textLine("%-36s %13s %13s %13s %13s" % \
                ("Batch " + self.batch + " Totals", j.disp,
                k.disp, l.disp, m.disp))
            self.pglin = self.pglin + 1
        else:
            self.t.textLine()
            self.pglin = self.pglin + 1
            self.t.textLine("%-11s %-51s %13s %13s %13s" % (" ",
                "Batch " + self.batch + " Totals", j.disp, k.disp, m.disp))
            self.t.textLine()
            self.pglin = self.pglin + 1
        self.bqty = 0
        self.bcrs = 0
        self.bdrs = 0
        self.bvat = 0

    def typeTotal(self):
        j = CCD(self.tdrs, "SD", 13.2)
        k = CCD(self.tcrs, "SD", 13.2)
        l = CCD(float(ASD(j.work) + ASD(k.work)), "SD", 13.2)
        m = CCD(self.tvat, "SD", 13.2)
        if self.totsonly == "Y":
            self.t.textLine()
            self.t.textLine("%-36s %13s %13s %13s %13s" % \
                ("Type Totals", j.disp, k.disp, l.disp, m.disp))
        else:
            self.t.textLine("%-11s %-51s %13s %13s %13s" % \
            (" ", "Type-Totals", j.disp, k.disp, m.disp))
        self.t.textLine()
        self.tqty = 0
        self.tcrs = 0
        self.tdrs = 0
        self.tvat = 0
    
    def grandTotal(self):
        tot = [0, 0, 0, 0, 0]
        if self.df.repprt[0][2] == "export":
            for x in range(0, len(gltrtp)):
                tot[0] = float(ASD(tot[0]) + ASD(self.gdr[x]))
                tot[1] = float(ASD(tot[1]) + ASD(self.gcr[x]))
                tot[2] = float(ASD(tot[2]) + ASD(self.gvt[x]))
            self.expdatas.append(["ULINES"])
            self.expdatas.append(["TOTAL", ["", "", "", "", "Grand Totals",
                "", "", tot[0], tot[1], tot[2], ""]])
            self.expdatas.append(["ULINED"])
            return
        self.c.drawText(self.t)
        self.c.showPage()
        self.totind = "Y"
        self.pageHeading()
        for x in range(0, len(gltrtp)):
            qt = CCD(self.gqt[x], "SI", 8)
            dr = CCD(self.gdr[x], "SD", 13.2)
            cr = CCD(self.gcr[x], "SD", 13.2)
            df = float(ASD(dr.work) + ASD(cr.work))
            df = CCD(df, "SD", 13.2)
            vt = CCD(self.gvt[x], "SD", 13.2)
            if self.totsonly == "Y":
                self.t.textLine("%-27s %s %s %s %s %s" % \
                (gltrtp[x][1], qt.disp, dr.disp, cr.disp, df.disp, vt.disp))
            else:
                self.t.textLine("%-34s %s %s %s %s %s" % \
                (gltrtp[x][1], qt.disp, dr.disp, cr.disp, df.disp, vt.disp))
            tot[0] = tot[0] + qt.work
            tot[1] = float(ASD(tot[1]) + ASD(dr.work))
            tot[2] = float(ASD(tot[2]) + ASD(cr.work))
            tot[3] = float(ASD(tot[3]) + ASD(df.work))
            tot[4] = float(ASD(tot[4]) + ASD(vt.work))
        self.t.textLine()
        qt = CCD(tot[0], "SI", 8)
        dr = CCD(tot[1], "SD", 13.2)
        cr = CCD(tot[2], "SD", 13.2)
        df = CCD(tot[3], "SD", 13.2)
        vt = CCD(tot[4], "SD", 13.2)
        if self.totsonly == "Y":
            self.t.textLine("%-27s %s %s %s %s %s" % \
            ("Grand Totals", qt.disp, dr.disp, cr.disp, df.disp, vt.disp))
        else:
            self.t.textLine("%-34s %s %s %s %s %s" % \
            ("Grand Totals", qt.disp, dr.disp, cr.disp, df.disp, vt.disp))
        self.t.textLine()

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

