#! /usr/bin/python

"""
SYNOPSIS
    General Ledger Account Statements.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""
import time
from TartanClasses import CCD, ASD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doPrinter, doReportlab, doWriteExport, getModName
from tartanFunctions import getSingleRecords, showError
from tartanWork import gltrtp

class gl3030:
    def __init__(self, mf, coy, nam, per, auto=False):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = per
        self.auto = auto
        no = self.setVariables()
        if not no:
            if self.auto:
                self.whole = "Y"
                self.sacno = 0
                self.eacno = 0
                self.sdate = CCD(self.s_per, "D2", 7)
                self.edate = CCD(self.e_per, "D2", 7)
                self.pages = "N"
                self.doEnd()
            else:
                self.mainProcess()
                self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["genbal", "genmst", "gentrn"],
            prog=__name__)
        if self.sql.error:
            return self.sql.error
        t = time.localtime()
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i) %6s" % (t[0],
            t[1], t[2], t[3], t[4], __name__)
        self.head = ("%03u %-30s %s" % (self.conum, self.conam, "%s"))
        self.colsh = ["Coy", "Acc-Num", "Description", "Date", "Reference",
            "Typ", "Batch", "Remarks", "Debit", "Credit", "Balance"]
        self.forms = [("UI", 3), ("UI", 7), ("NA", 30), ("D1", 10),
            ("Na", 9), ("NA", 3), ("Na", 7), ("NA", 30), ("SD", 13.2),
            ("SD", 13.2), ("SD", 13.2)]
        self.s_per = self.period[1][0] / 100
        self.e_per = self.period[2][0] / 100
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "General Ledger Account Statements (%s)" % __name__)
        glm = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "where": [("glm_cono", "=", self.conum)],
            "order": "glm_acno"}
        r1s = (("Yes","Y"),("Range","R"),("Singles", "S"))
        r2s = (("Yes","Y"), ("No","N"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Whole File","Whole File",
                "S","Y",self.doWhole,None,None,None),
            (("T",0,1,0),"IUI",7,"From Account","From Account",
                "","Y",self.doStartAcc,glm,None,("efld",)),
            (("T",0,2,0),"IUI",7,"To Account","To Account",
                0,"Y",self.doEndAcc,glm,None,("efld",)),
            (("T",0,3,0),"Id2",7,"Starting Period","Starting Period",
                self.s_per,"Y",self.doStartPer,None,None,("efld",)),
            (("T",0,4,0),"Id2",7,"Ending Period","Ending Period",
                self.e_per,"Y",self.doEndPer,None,None,("efld",)),
            (("T",0,5,0),("IRB",r2s),7,"Separate Pages","Separate Pages",
                "N","Y",self.doPages,None,None,None))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"))

    def doWhole(self, frt, pag, r, c, p, i, w):
        self.whole = w
        if self.whole in ("Y", "S"):
            self.sacno = 0
            self.eacno = 0
            self.df.loadEntry("T", 0, 1, data=self.sacno)
            self.df.loadEntry("T", 0, 2, data=self.eacno)
            return "sk2"

    def doStartAcc(self, frt, pag, r, c, p, i, w):
        if w == 0:
            ok = "yes"
        else:
            ok = self.readAcno(w)
        if ok != "yes":
            return "Invalid Account Number"
        self.sacno = w

    def doEndAcc(self, frt, pag, r, c, p, i, w):
        if w == 0:
            self.eacno = 9999999
            ok = "yes"
        else:
            self.eacno = w
            ok = self.readAcno(w)
        if ok != "yes":
            return "Invalid Account Number"

    def readAcno(self, acno):
        nam = self.sql.getRec(tables="genmst", cols=["glm_desc"],
            where=[("glm_cono", "=", self.conum), ("glm_acno", "=", acno)],
            limit=1)
        if not nam:
            return "no"
        else:
            return "yes"

    def doStartPer(self, frt, pag, r, c, p, i, w):
        if w < self.s_per or w > self.e_per:
            return "Invalid Period, Outside Financial Period"
        self.sdate = CCD(w, "D2", 7)

    def doEndPer(self, frt, pag, r, c, p, i, w):
        if w < self.sdate.work:
            return "Invalid Period, Before Start Period"
        if w > self.e_per:
            return "Invalid Period, Outside Financial Period"
        self.edate = CCD(w, "D2", 7)

    def doPages(self, frt, pag, r, c, p, i, w):
        self.pages = w

    def doEnd(self):
        if not self.auto:
            self.df.closeProcess()
        if self.whole == "S":
            all = getSingleRecords(self.mf, "genmst",
                ("glm_acno", "glm_desc"), where=[("glm_cono", "=", self.conum)])
        elif self.whole == "R":
            all = self.sql.getRec(tables="genmst", where=[("glm_cono", "=",
                self.conum), ("glm_acno", "between", self.sacno, self.eacno)],
                order="glm_acno")
        else:
            all = self.sql.getRec(tables="genmst", where=[("glm_cono", "=",
                self.conum)], order="glm_acno")
        if not all:
            showError(self.mf.body, "Error", "No Accounts Selected")
        elif not self.auto and self.df.repprt[0][2] == "export":
            self.exportReport(all)
        else:
            self.printReport(all)
        self.doExit()

    def exportReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        expnam = getModName(self.mf.wrkdir, __name__, self.conum)
        self.expheads = [self.head % self.sysdttm]
        self.expheads.append("General Ledger Account Statements for Period "\
            "%s to %s" % (self.sdate.disp, self.edate.disp))
        self.expcolsh = [self.colsh]
        self.expforms = self.forms
        self.expdatas = []
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                p.closeProgress()
                return
            self.acno, self.desc, acctot = self.getAccValues(dat)
            trn = self.sql.getRec(tables="gentrn", where=[("glt_cono", "=",
                dat[0]), ("glt_acno", "=", dat[1]), ("glt_curdt", ">=",
                self.sdate.work), ("glt_curdt", "<=", self.edate.work)],
                order="glt_acno, glt_curdt, glt_trdt, glt_type, glt_refno, "\
                "glt_batch")
            prt = False
            if acctot:
                prt = True
                self.expdatas.append(["BODY", [self.conum, self.acno.work,
                    self.desc.work, ((self.sdate.work * 100) + 1), "", "",
                    "", "Opening Balance", "", "", acctot]])
            for acc in trn:
                prt = True
                trdt, refno, trtp, batch, amt, dbt, crt, detail, curdt, \
                    curmth = self.getTrnValues(acc)
                acctot = float(ASD(acctot) + ASD(amt.work))
                self.expdatas.append(["BODY", [self.conum, self.acno.work,
                    self.desc.work, trdt.work, refno.work,
                    gltrtp[(trtp.work - 1)][0], batch.work, detail.work,
                    dbt.work, crt.work, acctot]])
            if prt:
                self.expdatas.append(["BLANK"])
        p.closeProgress()
        doWriteExport(xtype=self.df.repprt[0][1], name=expnam, sheet=__name__,
            heads=self.expheads, colsh=self.expcolsh, forms=self.expforms,
            datas=self.expdatas, rcdic=self.mf.rcdic)

    def printReport(self, all):
        self.pgnum = 0
        self.pglin = 99
        if not self.auto:
            p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %59s %10s" % (self.conum, self.conam,
            self.sysdttm, __name__))
        pdfnam = getModName(self.mf.wrkdir, __name__, self.conum, ext="pdf")
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, \
            self.duc = doReportlab(self.mf.prgdir, pdfnam, self.head)
        for seq, dat in enumerate(all):
            if not self.auto:
                can = p.displayProgress(seq)
                if can == "yes":
                    if self.t:
                        self.t.textLine()
                        self.t.textLine("Print Job ABORTED")
                    break
            self.acno, self.desc, acctot = self.getAccValues(dat)
            trn = self.sql.getRec(tables="gentrn", where=[("glt_cono", "=",
                dat[0]), ("glt_acno", "=", dat[1]), ("glt_curdt", ">=",
                self.sdate.work), ("glt_curdt", "<=", self.edate.work)],
                order="glt_acno, glt_curdt, glt_trdt, glt_type, glt_refno, "\
                "glt_batch")
            if acctot or trn:
                if not self.pgnum or self.pages == "Y":
                    self.pageHeading()
                else:
                    self.newAccount()
            if acctot:
                w1 = CCD(acctot, "SD", 13.2)
                self.t.textLine("%-10s %-9s %-3s %-7s %-30s %-13s %-13s "\
                    "%-13s" % (self.sdate.disp + "-01", "", "", "",
                    "Opening Balance", "", "", w1.disp))
                self.pglin = self.pglin + 1
            for acc in trn:
                trdt, refno, trtp, batch, amt, dbt, crt, detail, curdt, \
                    curmth = self.getTrnValues(acc)
                if self.pglin > self.lpp:
                    if self.pgnum != 0:
                        self.c.drawText(self.t)
                        self.c.showPage()
                    self.pageHeading()
                    bf = CCD(acctot, "SD", 13.2)
                    if bf.work:
                        self.t.textLine("%32s %-30s %27s %13s" % ("",
                            "Brought Forward", "", bf.disp))
                        self.pglin = self.pglin + 1
                acctot = float(ASD(acctot) + ASD(amt.work))
                w1 = CCD(acctot, "SD", 13.2)
                self.t.textLine("%-10s %-9s %-3s %-7s %-30s %-13s %-13s %-13s"\
                    % (trdt.disp, refno.disp, gltrtp[(trtp.work - 1)][0],
                    batch.disp, detail.disp, dbt.disp, crt.disp, w1.disp))
                self.pglin = self.pglin + 1
            if self.pages == "Y" and (acctot or trn):
                self.c.drawText(self.t)
                self.c.showPage()
                self.pglin = 99
        if not self.auto:
            p.closeProgress()
        if self.t:
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        if not self.auto:
            doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
                view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
                mail=self.df.repprt[1][2])

    def getAccValues(self, data):
        dic = self.sql.genmst_dic
        acno = CCD(data[dic["glm_acno"][1]], "UI", 7)
        desc = CCD(str(data[dic["glm_desc"][1]]), "NA", 30)
        o = self.sql.getRec(tables="genbal", cols=["glo_cyr"],
            where=[("glo_cono", "=", self.conum), ("glo_acno", "=",
            acno.work), ("glo_trdt", "=", self.period[1][0])], limit=1)
        if o:
            b = CCD(o[0], "SD", 13.2)
        else:
            b = CCD(0, "SD", 13.2)
        ob = b.work
        o = self.sql.getRec(tables="gentrn", cols=["round(sum(glt_tramt), 2)"],
            where=[("glt_cono", "=", self.conum), ("glt_acno", "=",
            acno.work), ("glt_curdt", ">=", self.s_per), ("glt_curdt",
            "<", self.sdate.work)], limit=1)
        if o and o[0]:
            b = CCD(o[0], "SD", 13.2)
        else:
            b = CCD(0, "SD", 13.2)
        acctot = float(ASD(ob) + ASD(b.work))
        return (acno, desc, acctot)

    def getTrnValues(self, data):
        tic = self.sql.gentrn_dic
        trdt = CCD(data[tic["glt_trdt"][1]], "D1", 10)
        refno = CCD(data[tic["glt_refno"][1]], "Na", 9)
        trtp = CCD(data[tic["glt_type"][1]], "UI", 1)
        batch = CCD(data[tic["glt_batch"][1]], "Na", 7)
        amt = CCD(data[tic["glt_tramt"][1]], "SD", 13.2)
        if amt.work < 0:
            dbt = CCD(0, "SD", 13.2)
            crt = CCD(amt.work, "SD", 13.2)
        else:
            dbt = CCD(amt.work, "SD", 13.2)
            crt = CCD(0, "SD", 13.2)
        detail = CCD(data[tic["glt_desc"][1]], "NA", 30)
        curdt = CCD(data[tic["glt_curdt"][1]], "UI", 6)
        curmth = int(curdt.disp[4:6])
        return (trdt, refno, trtp, batch, amt, dbt, crt, detail, curdt, curmth)
    
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        if self.sacno == 0:
            sacc = "  First"
        else:
            sacc = self.df.t_disp[0][0][1]
        if self.eacno == 0:
            eacc = "Last"
        else:
            eacc = self.df.t_disp[0][0][2]
        self.t.textLine("%-38s %-7s %2s %-7s %-10s %7s %2s %7s %11s %5s" % \
            ("General Ledger Statements for Accounts", sacc, "to", eacc,
            "for Period", self.sdate.disp, "to", self.edate.disp, "Page",
            self.pgnum))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 3
        self.newAccount()

    def newAccount(self):
        if self.pglin > (self.lpp - 7):
            self.c.drawText(self.t)
            self.c.showPage()
            self.pageHeading()
        else:
            self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
            self.t.textLine()
            self.t.textLine("%-7s %-7s %-30s" % ("Account", self.acno.disp,
                self.desc.disp))
            self.t.textLine()
            self.t.textLine("%-10s %-9s %-3s %-7s %-30s %-13s %-13s %-13s" % \
                ("   Date", "Reference", "Typ", "Batch", "Remarks",
                "       Debit", "      Credit", "     Balance"))
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.t.setFont("Tartan", self.cpi, self.ldg)
            self.pglin = self.pglin + 5

    def doExit(self):
        if not self.auto:
            self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    import sys
    print sys.argv
    coy, per = int(sys.argv[1]), int(sys.argv[2])
    from TartanClasses import MainFrame
    from tartanFunctions import getPeriods
    mf = MainFrame(display=False)
    mf.dbm.openDbase()
    sql = Sql(mf.dbm, "ctlmst")
    nam = sql.getRec(tables="ctlmst", cols=["ctm_name"], where=[("ctm_cono",
        "=", coy)], limit=1)
    start, end, fin = getPeriods(mf, coy, per, check=True)
    per = (per, (start.work, start.disp), (end.work, end.disp))
    gl3030(mf, coy, nam[0], per, True)
    mf.dbm.closeDbase()
