#! /usr/bin/python

"""
SYNOPSIS
    General Ledger Financial Statements.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import copy, time
from TartanClasses import ASD, CCD, CreateChart, FinReport, ProgressBar, Sql
from TartanClasses import TartanDialog
from tartanFunctions import doPrinter, doReportlab, doWriteExport, getModName
from tartanFunctions import mthendDate, showError
from tartanWork import mthnam

class gl3050:
    def __init__(self, mf, coy, nam, per):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = per
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlmst", "genctl", "gendtm", "genrpt",
            "genrpc", "genstr"], prog=__name__)
        t = time.localtime()
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i) %s" % \
            (t[0], t[1], t[2], t[3], t[4], __name__)
        self.s_per = self.period[1][0] / 100
        self.e_per = self.period[2][0] / 100
        self.titles = {
            1: ["Acc-Num", "UI", 7, False, False, True],
            2: ["Description", "NA", 30],
            3: ["Actual", "SD", 13.2],
            4: ["Budget", "SL", 10],
            5: ["Variance", "SL", 10],
            6: ["Var-%", "SD", 7.2],
            7: ["Last-Year", "SD", 13.2],
            8: ["Open-Bal", "SD", 13.2],
            9: ["Close-Bal", "SD", 13.2]}
        m = self.s_per % 100
        for x in range(0, 12):
            self.titles[x+10] = [mthnam[m][1], "SD", 13.2]
            m += 1
            if  m > 12:
                m = 1
        self.colss = [7, 2, 3, 4, 5, 6]
        self.colsl = [2, 3, 4, 5, 6, 3, 4, 5, 6, 7]
        self.colsm = [2, 8, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 9]
        self.colsc = []
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "General Ledger Financial Statements (%s)" % __name__)
        stm = {
            "stype": "R",
            "tables": ("genstr",),
            "cols": (
                ("gls_strm", "", 0, "Num"),
                ("gls_desc", "", 0, "Description", "Y"),
                ("count(*)", "UI", 3, "Qty")),
            "where": [("gls_cono", "=", self.conum)],
            "group": "gls_strm"}
        rpt = {
            "stype": "R",
            "tables": ("genrpt",),
            "cols": (
                ("glr_repno", "", 0, "Num"),
                ("glr_type", "", 0, "T"),
                ("glr_desc", "", 0, "Description", "Y")),
            "where": [
                ("glr_cono", "in", (0, self.conum)),
                ("glr_seq", "=", 0),
                ("glr_type", "in", ("P", "B", "O"))],
            "group": "glr_repno, glr_type, glr_desc"}
        det = {
            "stype": "R",
            "tables": ("gendtm",),
            "cols": (
                ("gdm_code", "", 0, "T"),
                ("gdm_desc", "", 0, "Description", "Y")),
            "where": [("gdm_cono", "=", self.conum)]}
        r1s = (
            ("Short","S"),
            ("Long","L"),
            ("Month","M"),
            ("Custom","C"))
        r2s = (("Yes","Y"),("No","N"))
        r3s = (
            ("Values","V"),
            ("Budgets","B"),
            ("Combined","C"),
            ("Variances","X"),
            ("Detail","D"))
        fld = (
            (("T",0,0,0),"ID2",7,"Ending Period","Ending Period (YYYY/MM)",
                self.e_per,"Y",self.doRepPer,None,None,None),
            (("T",0,1,0),"IUI",3,"Stream Number","",
                0,"Y",self.doRepStr,stm,None,None),
            (("T",0,2,0),("IRB",r1s),0,"Report Type","",
                "S","Y",self.doType,None,None,None),
            (("T",0,3,0),("IRB",r2s),0,"Consolidate","",
                "N","Y",self.doCons,None,None,None),
            (("T",0,4,0),"IUI",3,"Report Number","",
                1,"Y",self.doRepNum,rpt,None,("notzero",)),
            (("T",0,5,0),("IRB",r2s),0,"General Report","",
                "N","N",self.doRepGen,None,None,None),
            (("T",0,6,0),("IRB",r3s),0,"Contents","",
                "V","Y",self.doRepVal,None,None,None),
            (("T",0,7,0),"IUI",3,"Detail Code","",
                "","Y",self.doRepDet,det,None,("notzero",)),
            (("T",0,7,0),"ONA",30,""),
            (("T",0,8,0),("IRB",r2s),0,"Ignore Zeros","",
                "Y","Y",self.doIgnore,None,None,None),
            (("T",0,9,0),("IRB",r2s),0,"Print Options","",
                "Y","Y",self.doOptions,None,None,None),
            (("T",0,10,0),("IRB",r2s),0,"Account Numbers","",
                "Y","Y",self.doNumber,None,None,None))
        tnd = ((self.doMainEnd,"y"), )
        txt = (self.doMainExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"))

    def doRepPer(self, frt, pag, r, c, p, i, w):
        if w < self.s_per or w > self.e_per:
            return "Invalid Period"
        self.end = w
        self.endd = self.df.t_disp[pag][0][i]
        yed = CCD(mthendDate((w * 100) + 1), "D1", 10)
        self.yed = "%s %s %s" % ((yed.work % 100),
            mthnam[((yed.work % 10000) / 100)][1], yed.work / 10000)
        self.cyr = "  %4s" % (self.end / 100)
        self.pyr = "  %4s" % ((self.end / 100) - 1)

    def doRepStr(self, frt, pag, r, c, p, i, w):
        self.strm = w
        if self.strm:
            chk = self.sql.getRec(tables="genstr", where=[("gls_cono", "=",
                self.conum), ("gls_strm", "=", self.strm)], limit=1)
            if not chk:
                return "Invalid Stream Number"
            return "nd"

    def doType(self, frt, pag, r, c, p, i, w):
        self.typ = w
        chk = self.sql.getRec(tables="ctlmst", cols=["count(*)"], limit=1)
        if chk[0] == 1:
            self.con = "N"
            self.df.loadEntry(frt, pag, p+1, data=self.con)
            return "sk1"

    def doCons(self, frt, pag, r, c, p, i, w):
        self.con = w

    def doRepNum(self, frt, pag, r, c, p, i, w):
        if self.con == "Y":
            whr = [("glr_cono", "=", 0), ("glr_repno", "=", w)]
        else:
            whr = [("glr_cono", "in", (0, self.conum)), ("glr_repno", "=", w)]
        whr.append(("glr_seq", "=", 0))
        chk = self.sql.getRec(tables="genrpt", cols=["glr_cono"], where=whr,
            order="glr_cono")
        if not chk:
            return "Invalid Report Number"
        self.rep = w
        self.gen = "N"
        for c in chk:
            if c[0] == 0:
                self.gen = "Y"
                if self.con == "Y":
                    break
            else:
                self.gen = "N"
        if self.gen == "Y":
            err = self.getRepDetails()
            if err:
                return err
            else:
                self.df.loadEntry(frt, pag, p+1, data=self.gen)
                return "sk1"
        elif self.con == "Y":
            return "Invalid Report, Not A General Report"

    def doRepGen(self, frt, pag, r, c, p, i, w):
        if self.con == "Y" and w == "N":
            return "Invalid"
        self.gen = w
        return self.getRepDetails()

    def doRepVal(self, frt, pag, r, c, p, i, w):
        if w in ("B", "C", "X") and self.typ != "M":
            return "Invalid"
        self.val = w
        if self.val != "D":
            return "sk"

    def doRepDet(self, frt, pag, r, c, p, i, w):
        det = self.sql.getRec(tables="gendtm", cols=["gdm_desc"],
            where=[("gdm_cono", "=", self.conum), ("gdm_code", "=", w)],
            limit=1)
        if not det:
            return "Invalid Code"
        self.det = w
        self.df.loadEntry(frt, pag, p+1, data=det[0])

    def doIgnore(self, frt, pag, r, c, p, i, w):
        self.zer = w
        if self.typ == "C":
            self.df.topf[0][10][5] = "N"
        else:
            self.df.topf[0][10][5] = "Y"

    def doOptions(self, frt, pag, r, c, p, i, w):
        self.opt = w
        if self.typ not in ("L", "M"):
            self.df.loadEntry(frt, pag, p+1, data="")
            return "sk1"

    def doNumber(self, frt, pag, r, c, p, i, w):
        self.num = w

    def doMainEnd(self):
        self.df.closeProcess()
        if self.strm:
            reps = self.sql.getRec(tables="genstr", where=[("gls_cono", "=",
                self.conum), ("gls_strm", "=", self.strm)])
            if reps:
                col = self.sql.genstr_col
                for rep in reps:
                    seq = rep[col.index("gls_seq")]
                    self.typ = rep[col.index("gls_typ")]
                    self.typ = rep[col.index("gls_typ")]
                    self.con = rep[col.index("gls_con")]
                    self.rep = rep[col.index("gls_rep")]
                    self.gen = rep[col.index("gls_gen")]
                    err = self.getRepDetails()
                    if err:
                        continue
                    self.val = rep[col.index("gls_val")]
                    self.det = rep[col.index("gls_det")]
                    self.zer = rep[col.index("gls_zer")]
                    self.opt = rep[col.index("gls_opt")]
                    if not self.opt:
                        self.opt = "N"
                    self.num = rep[col.index("gls_num")]
                    if not self.num:
                        self.num = "Y"
                    if not rep[col.index("gls_prnt")]:
                        self.view = "N"
                        self.prtnam = "None"
                        self.flenam = getModName(self.mf.wrkdir, __name__,
                            "%s_%s" % (self.conum, seq), ext="pdf")
                    elif rep[col.index("gls_prnt")] == "Export":
                        self.view = "X"
                        self.prtnam = "export"
                        self.flenam = getModName(self.mf.wrkdir, __name__,
                            "%s_%s" % (self.conum, seq))
                    else:
                        self.view = "N"
                        self.prtnam = rep[col.index("gls_prnt")]
                        self.flenam = getModName(self.mf.wrkdir, __name__,
                            "%s_%s" % (self.conum, seq), ext="pdf")
                    self.mail = rep[col.index("gls_mail")]
                    if self.typ == "C":
                        if self.doReadCustomReport("stream") or not self.cols:
                            continue
                    self.doCreateReport()
        else:
            self.view = self.df.repprt[0][1]
            self.prtnam = self.df.repprt[0][2]
            self.mail = self.df.repprt[1][2]
            if self.prtnam == "export":
                self.flenam = getModName(self.mf.wrkdir, __name__, self.conum)
            else:
                self.flenam = getModName(self.mf.wrkdir, __name__, self.conum,
                    ext="pdf")
            if self.con == "Y":
                self.doConCoy()
                if self.con == "X":
                    self.mf.closeLoop()
                    return
            if self.typ == "C":
                self.doCustomReport()
                if not self.cols:
                    self.mf.closeLoop()
                    return
            self.doCreateReport()
        self.mf.closeLoop()

    def getRepDetails(self):
        if self.gen == "Y":
            self.rco = 0
        else:
            self.rco = self.conum
        chk = self.sql.getRec(tables="genrpt", cols=["glr_type", "glr_desc"],
            where=[("glr_cono", "=", self.rco), ("glr_repno", "=", self.rep),
            ("glr_seq", "=", 0)], limit=1)
        if not chk:
            return "Invalid Report Number or Type"
        self.rptyp, self.rpdes = chk

    def doConCoy(self):
        tit = ("Companies to Consolidate",)
        self.coys = self.sql.getRec(tables="ctlmst", cols=["ctm_cono",
            "ctm_name"], order="ctm_cono")
        coy = {
            "stype": "C",
            "titl": "",
            "data": self.coys,
            "mode": "M",
            "comnd": self.doCoyCmd}
        r1s = (("Yes","Y"),("Include","I"),("Exclude","E"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"All Companies","All Companies",
                "Y","N",self.doAllCoy,None,None,None,None),
            (("T",0,1,0),"INA",(30,100),"Companies","Companies",
                "","N",self.doCoySel,coy,None,None,None))
        state = self.df.disableButtons()
        self.cf = TartanDialog(self.mf, screen=self.mf.body, tops=True,
            title=tit, eflds=fld, tend=((self.doCoyEnd, "y"),),
            txit=(self.doCoyExit,))
        self.mf.startLoop()
        self.df.enableButtons(state=state)

    def doAllCoy(self, frt, pag, r, c, p, i, w):
        self.con = w
        if self.con == "Y":
            return "nd"
        if self.con == "I":
            self.cf.topf[pag][1][8]["titl"] = "Select Companies to Include"
        else:
            self.cf.topf[pag][1][8]["titl"] = "Select Companies to Exclude"

    def doCoyCmd(self, frt, pag, r, c, p, i, w):
        c = ""
        for co in w:
            if int(co) != 0:
                c = c + str(int(co)) + ","
        if len(c) > 1:
            c = c[:-1]
        self.cf.loadEntry(frt, pag, p, data=c)

    def doCoySel(self, frt, pag, r, c, p, i, w):
        if not w:
            return "Invalid List of Companies"
        if w[-1:] == ",":
            w = w[:-1]
        self.coy = w.split(",")

    def doCoyEnd(self):
        if self.con == "I":
            self.con = self.coy
        elif self.con == "E":
            self.con = []
            for co in self.coys:
                self.con.append(int(co[0]))
            for co in self.coy:
                del self.con[self.con.index(int(co))]
            self.con.sort()
        self.doCoyClose()

    def doCoyExit(self):
        self.con = "X"
        self.doCoyClose()

    def doCoyClose(self):
        self.cf.closeProcess()
        self.mf.closeLoop()

    def doCustomReport(self):
        tit = ("Custom Report Headings and Columns to Print",)
        rpc = {
            "stype": "C",
            "titl": "Available Custom Reports",
            "head": ("No", "Heading-1", "Heading-2"),
            "data": self.doLoadReports()}
        var = {
            "stype": "C",
            "titl": "Available Variables",
            "head": ("Code", "Description"),
            "data": (
                ("C-Year", "Prints the Current Year e.g. 2009"),
                ("P-Year", "Prints the Previous Year e.g. 2008)"))}
        data = []
        for d in self.titles:
            if d > 7:
                continue
            data.append(self.titles[d][0])
        col = {
            "stype": "C",
            "titl": "Available Columns",
            "data": data}
        check = [""]
        check.extend(data[:])
        self.tag = (
            ("Headings",self.chgPage1,None,None),
            ("Columns",self.chgPage2,None,None))
        fld = [
            [("T",1,0,0),"IUI",2,"Custom Number","",
                "","Y",self.doCusNo,rpc,None,("notblank",)],
            [("T",1,1,0),"INA",50,"Company Name","",
                "","N",self.doHeader,None,self.doDelete,("notblank",)]]
        if self.rptyp in ("B", "P"):
            fld.extend([
            [("T",1,2,0),"ITX",50,"B/S Heading","",
                "","N",self.doHeader,None,None,("notblank",),None,
                "Use %ymd to Insert a Date in the Heading Line e.g. "\
                "28 February 2008"],
            [("T",1,3,0),"ITX",50,"P&L Heading","",
                "","N",self.doHeader,None,None,("notblank",),None,
                "Use %ymd to Insert a Date in the Heading Line e.g. "\
                "28 February 2008"]])
        else:
            fld.append(
            [("T",1,2,0),"ITX",50,"Other Heading","",
                "","N",self.doHeader,None,None,("notblank",),None,
                "Use %ymd to Insert a Date in the Heading Line e.g. "\
                "28 February 2008"])
        fld.extend([
            [("T",2,0,0),"IUI",2,"Position","",
                0,"N",self.doPosition,None,None,("between",0,6)],
            [("C",2,0,0),"OUI",2,"P","P",
                "","N",None,None,None,None],
            [("C",2,0,1),"INA",20,"Column","",
                "","N",self.doColumn,col,None,("in", check)],
            [("C",2,0,2),"INA",20,"Title","",
                "","N",None,var,None,("efld",)]])
        but = (
            ("Accept",None,self.doAccept,2,None,None),
            ("Quit",None,self.doQuit,1,None,None))
        txit = (None,self.doTopExit,self.doTopExit)
        tend = (None,(self.doTopEnd,"n"),(self.doTopEnd,"n"))
        cxit = (None,None,self.doColExit)
        cend = (None,None,(self.doColEnd,"n"))
        state = self.df.disableButtons()
        self.cc = TartanDialog(self.mf, newscn=(0, 1), tops=True, title=tit,
            tags=self.tag, eflds=fld, rows=(0,0,7), butt=but, tend=tend,
            txit=txit, cend=cend, cxit=cxit)
        self.mf.startLoop()
        self.df.enableButtons(state=state)

    def doCusNo(self, frt, pag, r, c, p, i, w):
        self.cusno = w
        self.doReadCustomReport()

    def doReadCustomReport(self, ctype=None):
        self.cols = []
        self.whr = [("glc_cono", "=", self.conum), ("glc_cusno", "=",
            self.cusno)]
        rpt = self.sql.getRec(tables="genrpc", where=self.whr, limit=1)
        self.desc = True
        if rpt:
            if ctype:
                pads = 0
            self.new = False
            self.heds = [rpt[self.sql.genrpc_col.index("glc_head1")]]
            self.heds.append(rpt[self.sql.genrpc_col.index("glc_head2")])
            self.heds.append(rpt[self.sql.genrpc_col.index("glc_head3")])
            self.heds.append(rpt[self.sql.genrpc_col.index("glc_head4")])
            for x in range(6, 19, 2):
                if not rpt[x]:
                    break
                for idx in self.titles:
                    if self.titles[idx][0] == rpt[x]:
                        self.cols.append((idx, rpt[x], rpt[x+1]))
                        break
                if ctype:
                    pads = pads + int(self.titles[idx][2]) + 2
            if ctype:
                if pads < 80:
                    self.titles[2][2] = int(self.titles[2][2]) + 79 - pads
                for col in self.cols:
                    if col[0] == 2 and not col[2]:
                        self.desc = False
                return
        elif ctype:
            return "error"
        else:
            self.new = True
            self.heds = [
                self.conam,
                "Balance Sheet as at %ymd",
                "Income Statement for the Year Ended %ymd"]
            if self.rptyp in ("B", "P"):
                self.heds.append("")
            else:
                self.heds.append(self.rpdes + " as at %ymd")
        # Populate Fields
        self.cc.topf[1][1][5] = self.heds[0]
        if not self.new:
            self.cc.loadEntry("T", 1, 1, data=self.heds[0])
        if self.rptyp in ("B", "P"):
            self.cc.topf[1][2][5] = self.heds[1]
            self.cc.topf[1][3][5] = self.heds[2]
            if not self.new:
                self.cc.loadEntry("T", 1, 2, data=self.heds[1])
                self.cc.loadEntry("T", 1, 3, data=self.heds[2])
        else:
            self.cc.topf[1][2][5] = self.heds[3]
            if not self.new:
                self.cc.loadEntry("T", 1, 2, data=self.heds[3])
        if not self.new:
            for num, (idx, nam, tit) in enumerate(self.cols):
                self.cc.loadEntry("C", 2, ((num * 3)+1), data=nam)
                self.cc.loadEntry("C", 2, ((num * 3)+2), data=tit)
            # Enable Tags and Accept Button
            self.cc.enableTag(0)
            self.cc.enableTag(1)
            self.cc.B0.set_sensitive(True)

    def chgPage1(self, *args):
        self.cc.focusField("T", 1, 1, clr=False)

    def chgPage2(self, *args):
        self.cc.focusField("T", 2, 1, clr=False)

    def doHeader(self, frt, pag, r, c, p, i, w):
        if self.rptyp in ("B", "P"):
            self.heds[p] = w
        else:
            self.heds[3] = w

    def doDelete(self, widget=None):
        self.sql.delRec("genrpc", where=self.whr)
        self.mf.dbm.commitDbase()
        self.cc.topf[1][0][8]["data"] = self.doLoadReports()
        self.cc.focusField("T", 1, 1)

    def doLoadReports(self):
        data = []
        acc = self.sql.getRec(tables="genrpc", where=[("glc_cono", "=",
            self.conum)], order="glc_cusno")
        for a in acc:
            if self.rptyp == "B":
                data.append([a[1], a[2], a[3]])
            elif self.rptyp == "P":
                data.append([a[1], a[2], a[4]])
            else:
                data.append([a[1], a[2], a[5]])
        return data

    def doPosition(self, frt, pag, r, c, p, i, w):
        self.seq = w

    def doColumn(self, frt, pag, r, c, p, i, w):
        if not w:
            for x in range(p, 20):
                if not x % 3:
                    continue
                self.cc.loadEntry(frt, pag, x, data="")
            self.seq -= 1
            return "sk1"
        if w == "Description":
            self.desc = True
        if self.new or not self.cc.c_work[pag][r][i+1]:
            self.cc.loadEntry(frt, pag, p+1, data=w)

    def doTopEnd(self):
        if self.cc.pag == 1:
            for pos in range(7):
                self.cc.loadEntry("C", 2, (pos * 3), data=pos)
            self.cc.selPage("Columns")
        elif self.cc.pag == 2:
            self.cc.focusField("C", 2, ((self.seq * 3) + 1), clr=False)

    def doTopExit(self):
        if self.cc.pag == 1:
            self.doQuit()
        elif self.cc.pag == 2:
            if not self.desc:
                showError(self.mf.body, "Description Error",
                    "You Must Select the Description Column!")
                self.cc.focusField("T", 2, 1, clr=False)
                return
            pads = 0
            self.cols = []
            data = [self.conum, self.cusno] + self.heds
            for row in self.cc.c_work[2]:
                if row[1]:
                    data.extend([row[1], row[2]])
                    for d in self.titles:
                        if self.titles[d][0] == row[1]:
                            self.cols.append((d, row[1], row[2]))
                            pads = pads + int(self.titles[d][2]) + 2
                            break
                    if d == 2 and not row[2]:
                        self.desc = False
            for x in range(len(data), 20):
                data.append("")
            pads = pads - 2
            if pads < 80:
                self.titles[2][2] = int(self.titles[2][2]) + 79 - pads
            if self.new:
                self.sql.insRec("genrpc", data=data)
            else:
                self.sql.updRec("genrpc", data=data, where=self.whr)
            self.mf.dbm.commitDbase()
            self.cc.closeProcess()
            self.mf.closeLoop()

    def doColEnd(self):
        if self.seq != 6:
            self.seq += 1
        self.cc.loadEntry("T", 2, 0, data=self.seq)
        self.cc.focusField("T", 2, 1, clr=False)

    def doColExit(self):
        self.cc.focusField("T", 2, 1, clr=False)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.cc.doCheckFields(("T",1,None))
        if mes:
            self.cc.selPage(self.tag[pag - 1][0])
            self.cc.focusField(frt, pag, (col+1), err=mes, clr=False)
        else:
            self.cc.pag = 2
            self.doTopExit()

    def doQuit(self, widget=None):
        self.cols = []
        self.cc.closeProcess()
        self.mf.closeLoop()

    def doCreateReport(self):
        if self.gen == "Y":
            rco = 0
        else:
            rco = self.conum
        if self.val != "D":
            self.det = None
        elif not self.det:
            self.val = "V"
            self.det = None
        ####################################################################
        # RTYPE Determines the basis of the sign check i.e. MTD or YTD value
        # Maybe this should be an interactive request (Y=YTD, M=MTD)
        ####################################################################
        if self.typ in ("S", "L"):
            rtype = "Y"
        else:
            rtype = "Y"
        ####################################################################
        self.fin = FinReport(self.mf, self.conum, self.period, rco, self.rep,
            self.end, vcode=self.val, dcode=self.det, consol=self.con,
            rtype=rtype)
        if not self.fin.allFields:
            return
        if self.prtnam == "export":
            self.doExportReport()
        else:
            self.doPrintReport()
            # For type M and a Manual or Auto Chart is selected
            if not self.strm and (self.achart or self.mchart):
                CreateChart(self.mf, self.conum, self.conam, [self.s_per,
                self.end], [self.tit, self.des1], self.achart, self.mchart)

    def doPrintReport(self):
        p = ProgressBar(self.mf.body, max=len(self.fin.allFields), esc=True)
        self.head = []
        self.linb = ""
        self.lind = ""
        self.linh = ""
        self.linu = ""
        self.linw = ""
        self.last = False
        ulc = "X"                           # Temporary Underline Character
        if self.typ == "S":
            self.head.append("%03u %-30s %57s" % (self.conum, self.conam,
                self.sysdttm))
            self.head.append("")
            self.head.append("")
            self.head.append("%-13s  %-30s  %-13s  %-10s  %-10s  %-7s" % \
                ("   Last-Year", "Description", "      Actual", "   Budget",
                " Variance", " Var-%"))
            self.linu = "%s  %31s  %s  %s  %s  %s" % \
                (ulc*12,"",ulc*12,ulc*10,ulc*10,ulc*7)
            self.width = self.head[0]
        elif self.typ == "L":
            if self.num == "Y":
                self.head.append("%03u %-30s %114s" % (self.conum, self.conam,
                    self.sysdttm))
            else:
                self.head.append("%03u %-30s %105s" % (self.conum, self.conam,
                    self.sysdttm))
            self.head.append("")
            self.head.append("")
            if self.num == "Y":
                self.head.append("%-7s  %-30s  %-45s   %-45s" % ("", "",
                    "*************** Current-Month ***************",
                    "*************** Year-to-Date ****************"))
            else:
                self.head.append("%-30s  %-45s   %-45s" % ("",
                    "*************** Current-Month ***************",
                    "*************** Year-to-Date ****************"))
            if self.num == "Y":
                hdr = "%-7s  "
                hdt = ["Acc-Num"]
                lin = "%7s  "
                ldt = [""]
            else:
                hdr = ""
                hdt = []
                lin = ""
                ldt = []
            hdr = hdr + "%-30s  %-13s  %-10s  %-10s  %-7s  %-13s  %-10s  "\
                "%-10s  %-7s  %-13s"
            hdt.extend(["Description",
                "      Actual", "   Budget", " Variance", " Var-%",
                "      Actual", "   Budget", " Variance", " Var-%",
                "   Last-Year"])
            self.head.append(hdr % tuple(hdt))
            lin = lin + "%30s  %s  %s  %s  %s   %s  %s  %s  %s   %s"
            self.linu = lin % tuple(ldt + ["", ulc*12, ulc*10, ulc*10, ulc*7,
                ulc*12, ulc*10, ulc*10, ulc*7, ulc*12])
            self.linb = lin % tuple(ldt + ["", " "*12, " "*10, " "*10, " "*7,
                ulc*12, ulc*10, ulc*10, ulc*7, ulc*12])
            self.width = self.head[0]
        elif self.typ == "M":
            if self.num == "Y":
                self.head.append("%03u %-30s %156s" % (self.conum, self.conam,
                    self.sysdttm))
            else:
                self.head.append("%03u %-30s %148s" % (self.conum, self.conam,
                    self.sysdttm))
            self.head.append("")
            self.head.append("")
            if self.val == "V":
                self.des1   = "Actuals"
            elif self.val == "B":
                self.des1 = "Budgets"
            elif self.val == "C":
                self.des1 = "Actuals and Budgets"
            elif self.val == "X":
                self.des1 = "Variances to Budget"
            elif self.val == "D":
                self.des1 = "Details"
            self.lastmth = self.end % 100
            m = self.s_per % 100
            self.text = ""
            space = " " * 20
            for x in range(0, 12):
                self.text = self.text + space[0:(9-len(mthnam[m][1]))] + \
                    mthnam[m][1] + "  "
                m += 1
                if  m > 12:
                    m = 1
            if self.num == "Y":
                self.head.append("%-7s %-30s %-10s%-132s%-10s" % ("Acc-Num",
                    "Description", " Open-Bal  ", self.text, "Close-Bal  "))
                self.linu = "%7s %30s %s  %s  %s  %s  %s  %s  %s  %s  %s  %s  "\
                    "%s  %s  %s  %s" % ("", "",ulc*9,ulc*9,ulc*9,ulc*9,ulc*9,
                    ulc*9,ulc*9,ulc*9,ulc*9,ulc*9,ulc*9,ulc*9,ulc*9,ulc*9)
            else:
                self.head.append("%-30s %-10s%-132s%-10s" % ("Description",
                    " Open-Bal  ", self.text, "Close-Bal  "))
                self.linu = "%30s %s  %s  %s  %s  %s  %s  %s  %s  %s  %s  "\
                    "%s  %s  %s  %s" % ("",ulc*9,ulc*9,ulc*9,ulc*9,ulc*9,
                    ulc*9,ulc*9,ulc*9,ulc*9,ulc*9,ulc*9,ulc*9,ulc*9,ulc*9)
            self.width = self.head[0]
        else:
            head = ""
            self.text = ''
            stop = None
            for col in self.cols:
                text = self.titles[col[0]]
                if text[0] in ("Acc-Num", "Description"):
                    hd = "%-" + str(int(text[2])) + "s  "
                    head = head + hd % col[2]
                else:
                    hd = "%" + str(int(text[2]) - 1) + "s   "
                    head = head + hd % col[2]
                if not self.text:
                    self.text = '"%s"' % col[2]
                else:
                    self.text = '%s,"%s"' % (self.text, col[2])
                self.lind = self.lind + "%" + str(int(text[2])) + "s  "
                if not stop:
                    self.linh = self.linh + "%-" + str(int(text[2])) + "s  "
                self.linu = self.linu + "%" + str(int(text[2]) - 1) + "s   "
                if text[0] == "Acc-Num":
                    if not stop:
                        self.linh = self.linh % " "
                    self.linu = self.linu % " "
                    self.linw = self.linw + "%s,"
                elif text[0] == "Description":
                    self.linu = self.linu % " "
                    self.linw = self.linw + '"%s",'
                    stop = True
                else:
                    if not stop:
                        self.linh = self.linh % " "
                    self.linu = self.linu % ((int(text[2]) - 1) * ulc)
                    self.linw = self.linw + "%s,"
            head = head[:-2]
            head = head.replace("C-Year", self.cyr)
            self.text = self.text.replace("C-Year", self.cyr)
            head = head.replace("P-Year", self.pyr)
            self.text = self.text.replace("P-Year", self.pyr)
            self.lind = self.lind[:-2]
            self.linh = self.linh[:-2]
            self.linw = self.linw[:-1]
            self.heds.append(head)
            self.width = head
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.flenam, self.width)
        ############################################
        # Replace Temporary Underline Character
        ############################################
        self.lind = self.lind.replace("X", self.suc)
        self.linh = self.linh.replace("X", self.suc)
        self.linw = self.linw.replace("X", self.suc)
        ############################################
        self.pgnum = 0
        self.pglin = 99
        self.achart = []
        self.mchart = []
        for num, dat in enumerate(self.fin.allFields):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading(dat)
            if dat[0] == "H":
                self.doHeading(dat)
            elif dat[0] in ("C","G","L","P","S","T"):
                self.doValues(dat)
            elif dat[0] == "U":
                self.doUnderline(dat)
        p.closeProgress()
        if self.t:
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.flenam,
            head=self.emlhead, view=self.view, prtnam=self.prtnam,
            mail=self.mail)

    def doExportReport(self):
        self.expheads = ["%03u %-30s %s" % (self.conum, self.conam,
            self.sysdttm)]
        self.expheads.append("TYPE")
        self.expcolsh = [[]]
        self.expforms = []
        if self.typ == "S":
            for h in self.colss:
                self.expcolsh[0].append(self.titles[h][0])
                self.expforms.append(self.titles[h][1:])
        elif self.typ == "L":
            if self.num == "Y":
                self.colsl.insert(0, 1)
                self.expcolsh = [["", "", ["************** Current-Month "\
                    "**************", 2, 5], ["************** Year-to-Date "\
                    "**************", 6, 9], ""], []]
            else:
                self.expcolsh = [["", ["************** Current-Month "\
                    "**************", 1, 4], ["************** Year-to-Date "\
                    "**************", 5, 8], ""], []]
            for h in self.colsl:
                self.expcolsh[1].append(self.titles[h][0])
                self.expforms.append(self.titles[h][1:])
        elif self.typ == "M":
            if self.num == "Y":
                self.colsm.insert(0, 1)
            for h in self.colsm:
                self.expcolsh[0].append(self.titles[h][0])
                self.expforms.append(self.titles[h][1:])
            if self.val == "V":
                self.des1   = "Actuals"
            elif self.val == "B":
                self.des1 = "Budgets"
            elif self.val == "C":
                self.des1 = "Actuals and Budgets"
            elif self.val == "X":
                self.des1 = "Variance to Budget"
            elif self.val == "D":
                self.des1 = "Details"
            self.lastmth = self.end % 100
        else:
            for col in self.cols:
                self.expcolsh[0].append(col[2])
                self.expforms.append(self.titles[col[0]][1:])
        self.expdatas = []
        p = ProgressBar(self.mf.body, max=len(self.fin.allFields), esc=True)
        for num, dat in enumerate(self.fin.allFields):
            can = p.displayProgress(num)
            if can == "yes":
                p.closeProgress()
                return
            if not num:
                self.pageHeading(dat)
            if dat[0] == "H":
                self.doHeading(dat)
            elif dat[0] in ("C","G","L","P","S","T"):
                self.doValues(dat)
            elif dat[0] == "U":
                self.doUnderline(dat)
        p.closeProgress()
        if self.strm and self.prtnam == "export":
            view = False
        else:
            view = True
        doWriteExport(xtype=self.view, name=self.flenam, sheet=__name__,
            heads=self.expheads, colsh=self.expcolsh, forms=self.expforms,
            datas=self.expdatas, rcdic=self.mf.rcdic, view=view)

    def doHeading(self, line):
        if line[3] == "Y":
            if self.prtnam == "export":
                if self.oldtyp != line[1]:
                    self.pageHeading(line)
            else:
                self.c.drawText(self.t)
                self.c.showPage()
                self.pageHeading(line)
        if self.prtnam == "export":
            data = []
            for num, dat in enumerate(self.expforms):
                if int(self.expforms[num][1]) < 30:
                    data.append("")
                    continue
                data.append(line[5])
                break
            if self.expdatas[-1][0] not in ("PAGE", "BLANK"):
                self.expdatas.append(["BLANK"])
            self.expdatas.append(["HEAD", data])
            self.expdatas.append(["BLANK"])
        else:
            if line[2] == "Y":
                self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
            if self.typ == "S":
                self.t.textLine("%13s  %s" % ("", line[5]))
            elif self.typ == "L":
                if self.num == "Y":
                    self.t.textLine("%7s  %s" % ("", line[5]))
                else:
                    self.t.textLine("%s" % (line[5]))
            elif self.typ == "M":
                if self.num == "Y":
                    self.t.textLine("%7s %s" % ("", line[5]))
                else:
                    self.t.textLine("%s" % (line[5]))
            elif self.typ == "C":
                self.t.textLine(self.linh % line[5])
            self.t.setFont("Tartan", self.cpi, self.ldg)
            self.t.textLine()
            self.pglin = self.pglin + 2

    def doValues(self, line):
        typ = line[0]
        acc = CCD(line[4], "UL", 7)
        if acc.work == 0:
            acc.disp = "       "
        if self.typ == "C":
            des = CCD(line[5], "NA", self.titles[2][2])
        else:
            des = CCD(line[5], "NA", 30)
        sgn = line[12]
        if self.typ in ("S", "L", "C"):
            lyr = CCD(line[8][1], "SD", 13.2)
            if self.typ == "L":
                mth = CCD(line[8][2], "SD", 13.2)
                bud = CCD(long(line[8][3]), "SL", 10)
                if typ != "P":
                    if sgn == "N":
                        vmt = long(long(mth.work) - bud.work)
                    else:
                        vmt = long(bud.work - long(mth.work))
                    vmt = CCD(vmt, "SL", 10)
                    if bud.work != 0:
                        vmtper = round(vmt.work * 100.0 / bud.work, 2)
                    else:
                        vmtper = 0
                    vmtper = CCD(vmtper, "SD", 7.2)
                    if vmtper.err:
                        vmtper = CCD(999.99, "SD", 7.2)
                else:
                    vmt = CCD(0, "NA", 10)
                    vmtper = CCD(0, "NA", 7)
            ytd = CCD(line[8][4], "SD", 13.2)
            btd = CCD(long(line[8][5]), "SL", 10)
            if typ != "P":
                if sgn == "N":
                    vtd = long(long(ytd.work) - btd.work)
                else:
                    vtd = long(btd.work - long(ytd.work))
                vtd = CCD(vtd, "SL", 10)
                if btd.work != 0:
                    vtdper = round(vtd.work * 100.0 / btd.work, 2)
                else:
                    vtdper = 0
                vtdper = CCD(vtdper, "SD", 7.2)
                if vtdper.err:
                    vtdper = CCD(999.99, "SD", 7.2)
            else:
                vtd = CCD(0, "NA", 10)
                vtdper = CCD(0, "NA", 7)
        else:
            for x in range(0, 14):
                if self.val == "X" and sgn == "N":
                    line[9][x] = float(ASD(0) - ASD(line[9][x]))
                exec "mp%s = CCD(long(line[9][x]), 'SL', 10.0)" % x
                exec "mx%s = CCD(line[9][x], 'SD', 13.2)" % x
        if self.prtnam != "export" and line[2] == "Y":
            self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        if self.prtnam == "export":
            if typ == "T":
                txt = "TOTAL"
            else:
                txt = "BODY"
        if self.typ == "S":
            if self.zer == "Y" and not lyr.work and not ytd.work \
                               and not btd.work and not vtd.work:
                pass
            elif self.prtnam == "export":
                self.expdatas.append([txt, [lyr.work, des.work, ytd.work,
                    btd.work, vtd.work, vtdper.work]])
            else:
                self.t.textLine("%s  %s  %s  %s  %s  %s" % (lyr.disp,
                    des.disp,ytd.disp,btd.disp,vtd.disp,vtdper.disp))
                self.pglin += 1
                self.last = True
        elif self.typ == "L":
            if self.zer == "Y" and not mth.work and not bud.work \
                               and not vmt.work and not ytd.work \
                               and not btd.work and not vtd.work \
                               and not lyr.work:
                pass
            elif line[1] == "B":
                if self.prtnam == "export":
                    if self.num == "Y":
                        self.expdatas.append([txt, [acc.work, des.work, "",
                            "", "", "", ytd.work, btd.work, vtd.work,
                            vtdper.work, lyr.work]])
                    else:
                        self.expdatas.append([txt, [des.work, "", "", "",
                            "", ytd.work, btd.work, vtd.work, vtdper.work,
                            lyr.work]])
                else:
                    if self.num == "Y":
                        self.t.textLine("%s  %s  %13s  %-10s  %-10s  "\
                            "%-7s  %s  %s  %s  %s  %s" % (acc.disp,
                            des.disp, "", "", "", "", ytd.disp, btd.disp,
                            vtd.disp, vtdper.disp, lyr.disp))
                    else:
                        self.t.textLine("%s  %13s  %-10s  %-10s  %-7s  "\
                            "%s  %s  %s  %s  %s" % (des.disp, "", "", "",
                            "", ytd.disp, btd.disp, vtd.disp, vtdper.disp,
                            lyr.disp))
                    self.pglin += 1
                    self.last = True
            else:
                if self.prtnam == "export":
                    if self.num == "Y":
                        self.expdatas.append([txt, [acc.work, des.work,
                            mth.work, bud.work, vmt.work, vmtper.work,
                            ytd.work, btd.work, vtd.work, vtdper.work,
                            lyr.work]])
                    else:
                        self.expdatas.append([txt, [des.work,
                            mth.work, bud.work, vmt.work, vmtper.work,
                            ytd.work, btd.work, vtd.work, vtdper.work,
                            lyr.work]])
                else:
                    if self.num == "Y":
                        self.t.textLine("%s  %s  %s  %s  %s  %s  %s  %s  "\
                            "%s  %s  %s" % (acc.disp, des.disp, mth.disp,
                            bud.disp, vmt.disp, vmtper.disp, ytd.disp,
                            btd.disp, vtd.disp, vtdper.disp, lyr.disp))
                    else:
                        self.t.textLine("%s  %s  %s  %s  %s  %s  %s  %s  "\
                            "%s  %s" % (des.disp, mth.disp, bud.disp,
                            vmt.disp, vmtper.disp, ytd.disp, btd.disp,
                            vtd.disp, vtdper.disp, lyr.disp))
                    self.pglin += 1
                    self.last = True
        elif self.typ == "M":
            if self.zer == "Y":
                p = "n"
                for x in range(0, 14):
                    if self.prtnam == "export":
                        exec "a = mx%s.work" % x
                    else:
                        exec "a = mp%s.work" % x
                    if a != 0:
                        p = "y"
            else:
                p = "y"
            if p == "n":
                pass
            elif self.prtnam == "export":
                if self.num == "Y":
                    self.expdatas.append([txt, [acc.work, des.work, mx0.work,
                        mx1.work, mx2.work, mx3.work, mx4.work, mx5.work,
                        mx6.work, mx7.work, mx8.work, mx9.work, mx10.work,
                        mx11.work, mx12.work, mx13.work]])
                else:
                    self.expdatas.append([txt, [des.work, mx0.work, mx1.work,
                        mx2.work, mx3.work, mx4.work, mx5.work, mx6.work,
                        mx7.work, mx8.work, mx9.work, mx10.work, mx11.work,
                        mx12.work, mx13.work]])
            else:
                if self.num == "Y":
                    self.t.textLine(
                        "%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s" % \
                        (acc.disp, des.disp, mp0.disp, mp1.disp, mp2.disp,
                        mp3.disp, mp4.disp, mp5.disp, mp6.disp, mp7.disp,
                        mp8.disp, mp9.disp, mp10.disp, mp11.disp, mp12.disp,
                        mp13.disp))
                else:
                    self.t.textLine(
                        "%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s" % \
                        (des.disp, mp0.disp, mp1.disp, mp2.disp, mp3.disp,
                        mp4.disp, mp5.disp, mp6.disp, mp7.disp, mp8.disp,
                        mp9.disp, mp10.disp, mp11.disp, mp12.disp, mp13.disp))
                self.pglin += 1
                self.last = True
                if line[10]:
                    self.achart.append([typ, line[10], mp1.work, mp2.work,
                        mp3.work, mp4.work, mp5.work, mp6.work, mp7.work,
                        mp8.work, mp9.work, mp10.work, mp11.work, mp12.work])
                if typ in ("L", "T", "G") and des.work:
                    dsc = des.work
                    for x in (47, 123, 125):
                        dsc = dsc.replace(chr(x), "%s%s" % (chr(47), chr(x)))
                    self.mchart.append([typ, dsc, mp1.work, mp2.work, mp3.work,
                        mp4.work, mp5.work, mp6.work, mp7.work, mp8.work,
                        mp9.work, mp10.work, mp11.work, mp12.work])
        elif self.typ == "C":
            disp = []
            work = []
            zero = True
            for col in self.cols:
                if self.titles[col[0]][0] == "Acc-Num":
                    disp.append(acc.disp)
                    work.append(acc.work)
                elif self.titles[col[0]][0] == "Description":
                    disp.append(des.disp)
                    work.append(des.work)
                elif self.titles[col[0]][0] == "Actual":
                    disp.append(ytd.disp)
                    work.append(ytd.work)
                    if ytd.work:
                        zero = False
                elif self.titles[col[0]][0] == "Budget":
                    disp.append(btd.disp)
                    work.append(btd.work)
                    if btd.work:
                        zero = False
                elif self.titles[col[0]][0] == "Variance":
                    disp.append(vtd.disp)
                    work.append(vtd.work)
                elif self.titles[col[0]][0] == "Var-%":
                    disp.append(vtdper.disp)
                    work.append(vtdper.work)
                elif self.titles[col[0]][0] == "Last-Year":
                    disp.append(lyr.disp)
                    work.append(lyr.work)
                    if lyr.work:
                        zero = False
            if self.zer == "Y" and zero:
                pass
            elif self.prtnam == "export":
                self.expdatas.append([txt, work])
            else:
                self.t.textLine(self.lind % tuple(disp))
                self.pglin += 1
                self.last = True
        if self.prtnam != "export":
            self.t.setFont("Tartan", self.cpi, self.ldg)

    def doUnderline(self, line):
        if self.prtnam == "export":
            if line[5] == "Blank":
                self.expdatas.append(["BLANK"])
            elif line[5] == "Double":
                self.expdatas.append(["ULINED"])
            else:
                self.expdatas.append(["ULINES"])
            return
        if not self.last:
            return
        if line[5] == "Blank":
            self.t.textLine()
        else:
            if line[5] == "Double":
                uc = self.duc
            else:
                uc = self.suc
            if self.typ == "L" and line[1] == "B":
                self.t.textLine(self.linb.replace("X", uc))
            elif self.typ == "C":
                self.t.textLine(self.linu.replace("X", uc))
            else:
                self.t.textLine(self.linu.replace("X", uc))
        self.pglin += 1
        self.last = False

    def pageHeading(self, line):
        if self.prtnam == "export":
            if self.typ == "C":
                if self.rptyp in ("B", "P"):
                    if line[11] == "Balance Sheet":
                        head = self.heds[1].replace("%ymd", self.yed)
                    elif line[11] == "Profit and Loss":
                        head = self.heds[2].replace("%ymd", self.yed)
                else:
                    head = self.heds[3].replace("%ymd", self.yed)
                expheads = copy.deepcopy(self.expheads)
                expcolsh = copy.deepcopy(self.expcolsh)
            else:
                head = "%s for period %s to %s" % (line[11],
                    self.period[1][1][:-3], self.endd)
                if self.val == "D":
                    head = "%s (%s of %s)" % (head,
                        self.fin.ddet[0].strip(), self.fin.ddet[1].strip())
                elif self.typ == "M":
                    head = "%s (%s)" % (head, self.des1)
                expheads = copy.deepcopy(self.expheads)
                expheads[1] = head
                if self.opt == "Y" and len(self.expheads) < 3:
                    self.printOptions()
                expcolsh = copy.deepcopy(self.expcolsh)
                if self.typ == "L":
                    if self.num == "Y":
                        start = 2
                    else:
                        start = 1
                    if line[1] == "B":
                        expcolsh[0][start] = ""
                        expcolsh[1][start] = ""
                        expcolsh[1][start + 1] = ""
                        expcolsh[1][start + 2] = ""
                        expcolsh[1][start + 3] = ""
            expheads[1] = head
            self.expdatas.append(["PAGE", [expheads, expcolsh]])
            self.oldtyp = line[1]
            return
        self.pgnum += 1
        self.pglin = 0
        if self.typ == "C":
            self.t = self.c.beginText(20, 30)
            self.t.setFont("Tartan-Bold", 18, self.ldg)
            self.t.textLine(self.heds[0])
            self.t.textLine()
            self.t.setFont("Tartan-Bold", 14, self.ldg)
            if self.rptyp in ("B", "P"):
                if line[11] == "Balance Sheet":
                    text = self.heds[1].replace("%ymd", self.yed)
                elif line[11] == "Profit and Loss":
                    text = self.heds[2].replace("%ymd", self.yed)
            else:
                text = self.heds[3].replace("%ymd", self.yed)
            self.t.textLine(text)
            if self.opt == "Y":
                self.printOptions()
            else:
                self.t.textLine()
            self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
            self.t.textLine(self.heds[4])
            if self.desc:
                self.t.textLine("%s" % (self.suc * len(self.width)))
            else:
                self.t.textLine()
            self.pglin += 5
            self.emlhead = "Financials for %s as at %s" % (self.conam, self.yed)
        else:
            self.t = self.c.beginText(self.ogn[0], self.ogn[1])
            self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
            for num, data in enumerate(self.head):
                if num == 2:
                    head = "%s for period %s to %s" % (line[11],
                        self.period[1][1][:-3], self.endd)
                    if self.val == "D":
                        head = "%s (%s of %s)" % (head,
                            self.fin.ddet[0].strip(), self.fin.ddet[1].strip())
                    elif self.typ == "M":
                        head = "%s (%s)" % (head, self.des1)
                    if self.typ == "S":
                        self.t.textLine("%-81s %4s %5s" % (head[:81], "Page",
                            self.pgnum))
                    elif self.typ == "L":
                        if self.num == "Y":
                            self.t.textLine("%-138s %4s %5s" % (head, "Page",
                                self.pgnum))
                        else:
                            self.t.textLine("%-129s %4s %5s" % (head, "Page",
                                self.pgnum))
                    elif self.typ == "M":
                        if self.num == "Y":
                            self.t.textLine("%-180s %4s %5s" % (head, "Page",
                                self.pgnum))
                        else:
                            self.t.textLine("%-172s %4s %5s" % (head, "Page",
                                self.pgnum))
                    else:
                        self.t.setFont("Tartan-Bold", 14, self.ldg)
                        self.t.textLine(head)
                        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
                    self.pglin += 1
                    self.emlhead = head
                    if self.opt == "Y":
                        self.printOptions()
                    self.t.textLine()
                    self.pglin += 1
                elif line[1] == "B" and num == 3:
                    self.t.textLine(data.replace(
                        "*************** Current-Month ***************",
                        "                                             "))
                    self.pglin += 1
                elif line[1] == "B" and num == 4:
                    self.t.textLine(data.replace(
                        "Description                           "\
                        "Actual      Budget    Variance    Var-%",
                        "Description                           "\
                        "                                       "))
                    self.pglin += 1
                else:
                    self.t.textLine(data)
                    self.pglin += 1
            self.t.textLine("%s" % (self.suc * len(self.width)))
            self.pglin += 1
        self.t.setFont("Tartan", self.cpi, self.ldg)

    def printOptions(self):
        txt = "Options:- "
        if self.strm:
            txt = txt + "Stream: %s " % self.strm
        txt = txt + "Report: %s General: %s " % (self.rep, self.gen)
        if self.typ == "M":
            txt = txt + "Cut-Off: %s " % mthnam[self.lastmth][1]
        if self.val == "V":
            txt = txt + "Content: Actuals"
        elif self.val == "B":
            txt = txt + "Content: Budgets"
        elif self.val == "C":
            txt = txt + "Content: Combination"
        elif self.val == "X":
            txt = txt + "Content: Variances to Budget"
        elif self.val == "D":
            det = self.sql.getRec(tables=["gendtt"], cols=["gdt_value"],
                where=[("gdt_cono", "=", self.conum), ("gdt_code", "=",
                self.det), ("gdt_curdt", "=", self.end)], limit=1)
            if not det:
                mtd = CCD(0, "SD", 13.2).disp
            else:
                mtd = CCD(det[0], "SD", 13.2).disp
            det = self.sql.getRec(tables=["gendtm", "gendtt"],
                cols=["gdm_desc", "sum(gdt_value)"], where=[("gdm_cono", "=",
                self.conum), ("gdm_code", "=", self.det),
                ("gdt_cono=gdm_cono",), ("gdt_cono=gdm_cono",),
                ("gdt_code=gdm_code",), ("gdt_curdt", "between", self.s_per,
                self.end)], group="gdm_desc", limit=1)
            des = det[0]
            ytd = CCD(det[1], "SD", 13.2).disp
            if self.typ in ("C", "M", "S"):
                txt = txt + "Detail: %s - Y.T.D. %s" % (des, ytd)
            elif self.typ == "L":
                txt = txt + "Detail: %s - Month %s  Y.T.D. %s" % (des, mtd, ytd)
        if self.con == "N":
            pass
        elif self.con == "Y":
            txt = txt + "Companies: All "
        else:
            con = str(tuple(self.con)).replace("'", "").replace(" ", "")
            txt = txt + "Companies: %s " % con
        if self.prtnam == "export":
            self.expheads.append(txt)
            return
        self.t.textLine()
        self.pglin += 1
        if self.typ == "C":
            self.t.textLine(txt)
            self.t.textLine()
            self.pglin += 1
        else:
            self.t.textLine(txt)
        self.pglin += 1

    def doMainExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

