#! /usr/bin/python

"""
SYNOPSIS
    General Ledger Interrogation.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, NotesCreate, RepPrt, SplashScreen, Sql
from TartanClasses import SRec, TabPrt, TartanDialog
from tartanFunctions import getPeriods, doReportlab, getModName, doPrinter
from tartanWork import gltrtp, mthnam

class gl4010:
    def __init__(self, mf, coy, nam, per, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.capnm = usr
        self.period = per
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["genmst", "genbal", "genbud", "gentrn"],
            prog=__name__)
        if self.sql.error:
            return self.sql.error
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.s_per = self.period[1][0] / 100
        self.e_per = self.period[2][0] / 100
        if self.period[0] == 0:
            s = e = None
        else:
            s, e, f = getPeriods(self.mf, self.conum, (self.period[0] - 1))
            if (s, e, f) == (None, None, None):
                return "no"
        if s == None or e == None:
            self.s_lyr = self.period[1][0] - 10000
            self.s_pyr = self.s_per - 100
            self.e_pyr = self.e_per - 100
        else:
            self.s_lyr = s.work
            self.s_pyr = s.work / 100
            self.e_pyr = e.work / 100
        self.trnper = 0
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "General Ledger Interrogation (%s)" % __name__)
        glm = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "where": [("glm_cono", "=", self.conum)],
            "order": "glm_acno"}
        fld = (
            (("T",0,0,0,7),"IUI",7,"Acc-Num","Account Number",
                "","N",self.doAccNum,glm,None,None),
            (("T",0,0,0),"ONA",30,"Description"),
            (("T",0,0,0),"OUA",1,"Type"),
            (("T",0,0,0),"OUA",1,"Indicator"),
            (("C",1,0,0),"OSD",13.2,"Actual","","","N",
                None,None,None,None,("Months",14)),
            (("C",1,0,1),"OSD",13.2,"Budget"),
            (("C",1,0,2),"OSD",13.2,"Variance"),
            (("C",1,0,3),"OSD",13.2,"Last-Year"),
            (("T",2,0,0),"Id2",7,"Period (YYYYMM)","Financial Period (0=All)",
                self.trnper,"Y",self.doPeriod,None,None,None))
        tag = (
            ("Balances",self.doBals,("T",0,2),("T",0,1)),
            ("Transactions",self.doTrans1,("T",0,2),("T",0,1)))
        but = (
            ("Clear",None,self.doClear,0,("T",0,0),("T",0,1)),
            ("Notes",None,self.doNotes,0,("T",0,0),("T",0,1)),
            ("Print",None,self.doPrint,1,("T",0,0),("T",0,1)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doTopEnd, "N"), None, (self.doTrans2, "N"))
        txt = (self.doExit, None, self.doExit)
        self.df = TartanDialog(self.mf, title=self.tit, tags=tag, eflds=fld,
            butt=but, tend=tnd, txit=txt)
        mon = self.s_per % 100
        yer = self.s_per / 100
        for x in range(0, 14):
            if x == 0:
                txt = "Opening Balance"
                self.df.colLabel[1][x].set_text(txt)
            elif x == 13:
                txt = "Closing Balance"
                self.df.colLabel[1][x].set_text(txt)
            else:
                nam = mthnam[mon][1]
                nam = nam + (" " * (11-len(nam))) + str(yer)
                self.df.colLabel[1][x].set_text(nam)
            if x != 0 and x != 13:
                mon = mon + 1
                if mon > 12:
                    mon = mon - 12
                    yer = yer + 1
        txt = "Closing Balance"
        self.df.colLabel[1][13].set_text(txt)

    def doAccNum(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="genmst", cols=["glm_desc", "glm_type",
            "glm_ind"], where=[("glm_cono", "=", self.conum), ("glm_acno",
            "=", w)], limit=1)
        if not acc:
            return "Invalid Account"
        self.acno = w
        self.desc = acc[0]
        for x in range(1, self.df.topq[pag]):
            self.df.loadEntry(frt, pag, p+x, data=acc[x-1])
        gp = self.sql.getRec(tables="genbal", cols=["glo_cyr"],
            where=[("glo_cono", "=", self.conum), ("glo_acno", "=", self.acno),
            ("glo_trdt", "=", self.s_lyr)])
        gc = self.sql.getRec(tables="genbal", cols=["glo_cyr"],
            where=[("glo_cono", "=", self.conum), ("glo_acno", "=", self.acno),
            ("glo_trdt", "=", self.period[1][0])])
        gt = self.sql.getRec(tables="gentrn", cols=["glt_type", "glt_curdt",
            "glt_tramt"], where=[("glt_cono", "=", self.conum), ("glt_acno",
            "=", self.acno), ("glt_curdt", "between", self.s_per, self.e_per)])
        gb = self.sql.getRec(tables="genbud", cols=["glb_curdt", "glb_tramt"],
            where=[("glb_cono", "=", self.conum), ("glb_acno", "=", self.acno),
            ("glb_curdt", "between", self.s_per, self.e_per)])
        gl = self.sql.getRec(tables="gentrn", cols=["glt_curdt", "glt_tramt"],
            where=[("glt_cono", "=", self.conum), ("glt_acno", "=", self.acno),
            ("glt_curdt", "between", self.s_pyr, self.e_pyr)])
        act = [0] * 14
        bud = [0] * 14
        lyr = [0] * 14
        if gc:
            self.obal = gc[0][0]
            act[0] = float(ASD(act[0]) + ASD(self.obal))
            act[13] = float(ASD(act[13]) + ASD(self.obal))
            if gp:
                lyr[0] = float(ASD(lyr[0]) + ASD(float(gp[0][0])))
                lyr[13] = float(ASD(lyr[13]) + ASD(float(gp[0][0])))
        else:
            self.obal = 0
        if gt:
            for x in range(0, len(gt)):
                mon = int(str(gt[x][1])[4:6])
                yre = self.e_per % 100
                col = mon - yre
                if col <= 0:
                    col = col + 12
                act[col] = float(ASD(act[col]) + ASD(float(gt[x][2])))
                act[13] = float(ASD(act[13]) + ASD(float(gt[x][2])))
        if gb:
            for x in range(0, len(gb)):
                mon = int(str(gb[x][0])[4:6])
                yre = self.e_per % 100
                col = mon - yre
                if col <= 0:
                    col = col + 12
                bud[col] = float(ASD(bud[col]) + ASD(float(gb[x][1])))
                bud[13] = float(ASD(bud[13]) + ASD(float(gb[x][1])))
        if gl:
            for x in range(0, len(gl)):
                mon = int(str(gl[x][0])[4:6])
                yre = self.e_per % 100
                col = mon - yre
                if col <= 0:
                    col = col + 12
                lyr[col] = float(ASD(lyr[col]) + ASD(float(gl[x][1])))
                lyr[13] = float(ASD(lyr[13]) + ASD(float(gl[x][1])))
        p = 0
        for x in range (0, 14):
            i = 0
            self.df.loadEntry("C", 1, p, data=act[x])
            p = p + 1
            i = i + 1
            self.df.loadEntry("C", 1, p, data=bud[x])
            p = p + 1
            i = i + 1
            data=float(ASD(bud[x])-ASD(act[x]))
            self.df.loadEntry("C", 1, p, data=data)
            p = p + 1
            i = i + 1
            self.df.loadEntry("C", 1, p, data=lyr[x])
            p = p + 1
        self.df.last[0] = [0, 0]
        self.mf.updateStatus("")

    def doBals(self):
        self.mf.updateStatus("")

    def doTrans1(self, widget=None):
        self.df.focusField("T", 2, 1)

    def doPeriod(self, frt, pag, r, c, p, i, w):
        if w != 0 and (w < self.s_per or w > self.e_per):
            return "Invalid Period"
        self.trnper = w

    def doTrans2(self):
        tit = "Transactions for Account: %s - %s" % (self.acno, self.desc)
        gtt = ["gentrn"]
        gtc = (
            ("glt_trdt", "", 0, "   Date"),
            ("glt_refno", "", 0, "Reference", "Y"),
            ("glt_type", ("XX", gltrtp), 3, "Typ"),
            ("glt_batch", "", 0, "Batch"),
            ("glt_tramt", "", 0, "       Debit"),
            ("glt_tramt", "", 0, "      Credit"),
            ("balance", "SD", 13.2, "     Balance"),
            ("glt_desc", "", 0, "Remarks"),
            ("glt_seq", "", 0, "Sequence"))
        mthno, acctot = self.getObal()
        if mthno.work == 0:
            d = self.period[1][0]
        else:
            d = (self.trnper * 100) + 1
        if acctot:
            if acctot < 0:
                dr = 0
                cr = acctot
            else:
                dr = acctot
                cr = 0
            data = [[d, "B/Fwd", 4, "", dr, cr, acctot, "Opening Balance", 0]]
        else:
            data = []
        trn = self.getTrans()
        if trn:
            col = self.sql.gentrn_col
            for rec in trn:
                acctot = float(ASD(acctot) + ASD(rec[col.index("glt_tramt")]))
                if rec[col.index("glt_tramt")] < 0:
                    dr = 0
                    cr = rec[col.index("glt_tramt")]
                else:
                    dr = rec[col.index("glt_tramt")]
                    cr = 0
                data.append([
                    rec[col.index("glt_trdt")],
                    rec[col.index("glt_refno")],
                    rec[col.index("glt_type")],
                    rec[col.index("glt_batch")],
                    dr, cr, acctot,
                    rec[col.index("glt_desc")],
                    rec[col.index("glt_seq")]])
        state = self.df.disableButtons()
        while True:
            rec = SRec(self.mf, screen=self.df.nb.Page2, title=tit, tables=gtt,
                cols=gtc, where=data, wtype="D", sort=False, rows=15)
            # Display all transaction details
            if rec.selection:
                self.df.mstAlw.hide()
                sp = SplashScreen(self.df.nb.Page2,
                    "Displaying Document.... Please Wait....")
                whr = [("glt_seq", "=", rec.selection[8])]
                TabPrt(self.mf, tabs="gentrn", wher=whr, pdia=False)
                sp.closeSplash()
                self.df.mstAlw.show_all()
            else:
                break
        self.df.enableButtons(state=state)
        self.doTrans1()

    def doNotes(self, widget=None):
        state = self.df.disableButtons()
        self.df.mstAlw.hide()
        nf = NotesCreate(self.mf, self.conum, self.conam, self.capnm, "GEN",
            self.acno)
        self.df.mstAlw.show()
        self.df.enableButtons(state=state)

    def doTopEnd(self):
        self.df.last[0] = [0, 0]
        self.df.setWidget(self.df.topEntry[0][0], state="disabled")

    def doClear(self, widget=None):
        self.df.selPage("Balances")
        self.df.focusField("T", 0, 1)

    def doPrint(self, widget=None):
        self.df.mstAlw.hide()
        tit = ("Printer/Email Selection",)
        r1s = (("Info","I"), ("Trans","T"))
        fld = (
            (("T",0,1,0),("IRB",r1s),0,"Selection","Selection",
                "I","N",self.doPrtSel,None,None,None),)
        self.pr = TartanDialog(self.mf, screen=self.mf.body, tops=True,
            title=tit, eflds=fld, tend=((self.doPrtEnd,"y"),),
            txit=(self.doPrtExit,), view=("Y","V"), mail=("Y","N"))
        self.mf.startLoop()
        self.df.mstAlw.show()

    def doPrtSel(self, frt, pag, r, c, p, i, w):
        self.sel = w
        if self.sel == "I":
            hide = True
        else:
            hide = False
        for x in (2, 3):
            if hide:
                self.pr.topEntry[0][1][x][0].hide()
            else:
                self.pr.topEntry[0][1][x][0].show()

    def doPrtEnd(self, widget=None):
        self.pr.closeProcess()
        self.head = ("%03u %-30s %57s %10s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        self.num = self.df.t_disp[0][0][0]
        self.dsc = self.df.t_disp[0][0][1]
        atype = self.df.t_disp[0][0][2]
        ind = self.df.t_disp[0][0][3]
        if self.pr.t_work[0][0][0] == "I":
            self.pageHeading()
            self.t.textLine("%-8s %-12s %-6s %-35s %-6s %-7s %-11s %s" % \
                ("Acc No:", self.num, "Desc:", self.dsc, "Type:", atype,
                "Indicator:", ind))
            self.t.textLine()
            self.t.textLine("%-21s %-20s %-18s %-19s %-18s" % \
                ("", "Actual", "Budget", "Variance", "Last Year"))
            self.t.textLine()
            m = self.s_per % 100
            for x in range (0, 14):
                if x == 0:
                    mon = "Opening"
                elif x == 13:
                    mon = "Closing"
                else:
                    mon = mthnam[m][1]
                    m = m + 1
                    if m > 12:
                        m = m - 12
                act = self.df.c_disp[1][x][0]
                bud = self.df.c_disp[1][x][1]
                var = self.df.c_disp[1][x][2]
                lyr = self.df.c_disp[1][x][3]
                self.t.textLine("%-15s %-20s %-20s %-20s %-20s" % \
                    (mon, act, bud, var, lyr))
                if x == 0 or x == 12:
                    self.t.textLine()
            if self.t:
                self.c.drawText(self.t)
            self.c.showPage()
            self.c.save()
            doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                head=self.tit, view=self.pr.repprt[0][1],
                prtnam=self.pr.repprt[0][2], mail=self.pr.repprt[1][2])
        else:
            trn = self.getTrans()
            if trn:
                heads = ["General Ledger Interrogation as at %s" % self.sysdtd]
                mthno, acctot = self.getObal()
                if mthno.work == 0:
                    x = "(For all months)"
                    d = self.period[1][0]
                else:
                    x = "(For month " + mthno.disp + ")"
                    d = (self.trnper * 100) + 1
                heads.append("Account %s %s %s" % (self.num, self.dsc, x))
                cols = [
                    ["glt_trdt", "D1", 10, "   Date"],
                    ["glt_refno", "Na", 9, "Reference"],
                    ["glt_type", "NA", 3, "Typ"],
                    ["glt_batch", "Na", 7, "BatchNo"],
                    ["debit", "SD", 13.2, "       Debit "],
                    ["credit", "SD", 13.2, "      Credit "],
                    ["balance", "SD", 13.2, "     Balance "],
                    ["glt_desc", "NA", 30, "Remarks"]]
                if acctot:
                    if acctot < 0:
                        dr = 0
                        cr = acctot
                    else:
                        dr = acctot
                        cr = 0
                    data = [(d,"B/Fwd","","",dr,cr,acctot,"Opening Balance")]
                else:
                    data = []
                col = self.sql.gentrn_col
                for num, rec in enumerate(trn):
                    trdt = rec[col.index("glt_trdt")]
                    refno = rec[col.index("glt_refno")]
                    trtp = gltrtp[(rec[col.index("glt_type")]) - 1][0]
                    batch = rec[col.index("glt_batch")]
                    tramt = rec[col.index("glt_tramt")]
                    if tramt < 0:
                        debit = 0
                        credit = tramt
                    else:
                        debit = tramt
                        credit = 0
                    detail = rec[col.index("glt_desc")]
                    acctot = float(ASD(acctot) + ASD(tramt))
                    data.append((trdt, refno, trtp, batch, debit, credit,
                        acctot, detail))
                RepPrt(self.mf, name=__name__, conum=self.conum,
                    conam=self.conam, tables=data, heads=heads, cols=cols,
                    gtots=["debit", "credit"], view=self.pr.repprt[0][1],
                    prtnam=self.pr.repprt[0][2], mail=self.pr.repprt[1][2],
                    ttype="D")
        self.mf.closeLoop()

    def doPrtExit(self, widget=None):
        self.pr.closeProcess()
        self.mf.closeLoop()

    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-34s %-10s %51s %5s" % \
            ("General Ledger Interrogation as at", self.sysdtd,
            "Page", self.pgnum))
        self.t.textLine("%-95s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 4

    def getObal(self):
        acbal = self.obal
        if self.trnper:
            o = self.sql.getRec(tables="gentrn",
                cols=["round(sum(glt_tramt), 2)"],
                where=[("glt_cono", "=", self.conum), ("glt_acno", "=",
                self.acno), ("glt_curdt", ">=", self.s_per), ("glt_curdt",
                "<", self.trnper)], limit=1)
            if o[0]:
                acbal = float(ASD(acbal) + ASD(o[0]))
        mthno = CCD(self.trnper % 100, "UI", 2.0)
        return mthno, acbal

    def getTrans(self):
        ord = "glt_acno, glt_curdt, glt_trdt, glt_type, glt_refno, glt_batch"
        if not self.trnper:
            whr = [("glt_cono", "=", self.conum), ("glt_acno", "=",
                self.acno), ("glt_curdt", "between", self.s_per,
                self.e_per)]
        else:
            whr = [("glt_cono", "=", self.conum), ("glt_acno", "=",
                self.acno), ("glt_curdt", "=", self.trnper)]
        return self.sql.getRec(tables="gentrn", where=whr, order=ord)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

