#! /usr/bin/python

"""
SYNOPSIS
    General Ledger Copy Accounts From One Company to Another.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import ProgressBar, Sql, TartanDialog

class gl6020:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlctl", "genbud", "genbal", "gendtm",
            "gendtt", "genjlm", "genjlt", "genmst", "gentrn", "genrpt",
            "genstr"], prog=__name__)

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "General Ledger Account Copy (%s)" % __name__)
        coy = {
            "stype": "R",
            "tables": ("ctlmst",),
            "cols": (
                ("ctm_cono", "", 0, "Coy-Num"),
                ("ctm_name", "", 0, "Name", "Y")),
            "where": [("ctm_cono", "!=", self.conum)]}
        r1s = (("Yes", "Y"), ("No", "N"))
        fld = (
            (("T",0,0,0),"IUI",3,"Copy From Company","Copy From Company",
                "","N",self.doCoyNum,coy,None,("notzero",)),
            (("T",0,1,0),("IRB",r1s),1,"Include Control Records",
                "Include Control Records","N","N",None,None,None,None),
            (("T",0,2,0),("IRB",r1s),1,"Include Opening Balances",
                "Include Opening Balances","N","N",None,None,None,None),
            (("T",0,3,0),("IRB",r1s),1,"Include Transactions",
                "Include Transactions","N","N",None,None,None,None),
            (("T",0,4,0),("IRB",r1s),1,"Include Budgets","Include Budgets",
                "N","N",None,None,None,None),
            (("T",0,5,0),("IRB",r1s),1,"Include Standard Journals",
                "Include Standard Journals","N","N",None,None,None,None),
            (("T",0,6,0),("IRB",r1s),1,"Include Reports","Include Reports",
                "N","N",None,None,None,None),
            (("T",0,7,0),("IRB",r1s),1,"Include Detail Records",
                "Include Detail Records","N","N",None,None,None,None),
            (("T",0,8,0),("IRB",r1s),1,"Include Stream Records",
                "Include Stream Records","N","N",None,None,None,None))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt)

    def doCoyNum(self, frt, pag, r, c, p, i, w):
        if w == 0:
            return "Invalid Company Number"
        acc = self.sql.getRec(tables="ctlmst", cols=["ctm_name"],
            where=[("ctm_cono", "=", w)], limit=1)
        if not acc:
            return "Invalid Company Number"
        acc = self.sql.getRec(tables="genmst", cols=["count(*)"],
            where=[("glm_cono", "=", w)], limit=1)
        if not acc[0]:
            return "This Company Has No Masterfile Records"
        self.oldco = w

    def doEnd(self):
        self.df.closeProcess()
        tabels = [("genmst", "glm")]
        if self.df.t_work[0][0][1] == "Y":
            tabels.append(("ctlctl", "ctl"))
        if self.df.t_work[0][0][2] == "Y":
            tabels.append(("genbal", "glo"))
        if self.df.t_work[0][0][3] == "Y":
            tabels.append(("gentrn", "glt"))
        if self.df.t_work[0][0][4] == "Y":
            tabels.append(("genbud", "glb"))
        if self.df.t_work[0][0][5] == "Y":
            tabels.append(("genjlm", "gjm"))
            tabels.append(("genjlt", "gjt"))
        if self.df.t_work[0][0][6] == "Y":
            tabels.append(("genrpt", "glr"))
        if self.df.t_work[0][0][7] == "Y":
            tabels.append(("gendtm", "gdm"))
            tabels.append(("gendtt", "gdt"))
        if self.df.t_work[0][0][8] == "Y":
            tabels.append(("genstr", "gls"))
        for tab, ref in tabels:
            self.sql.delRec(tab, where=[("%s_cono" % ref, "=", self.conum)])
            all = self.sql.getRec(tables=tab, where=[("%s_cono" % ref, "=",
                self.oldco)])
            if all:
                p = ProgressBar(self.mf.body, max=len(all),
                    typ="Copying %s" % tab)
                for num, acc in enumerate(all):
                    p.displayProgress(num)
                    exec "acc[self.sql.%s_col.index('%s_cono')] = %s" % (tab,
                        ref, self.conum)
                    exec "self.sql.insRec('%s', data=acc)" % tab
                p.closeProgress()
        self.mf.dbm.commitDbase(ask=True)
        self.mf.closeLoop()

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

