#! /usr/bin/python

"""
SYNOPSIS
    General Ledger Check Integration Control Balances.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, GetCtl, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter

class gl6030:
    def __init__(self, mf, coy, nam, per):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = per
        no = self.checkControls()
        if not no:
            no = self.setVariables()
            if not no:
                self.mainProcess()
                self.mf.startLoop()

    def checkControls(self):
        gc = GetCtl(self.mf)
        ctlctl = gc.getCtl("ctlctl", self.conum)
        if not ctlctl:
            return "no"
        crsctl = gc.getCtl("crsctl", self.conum, error=False)
        if crsctl and crsctl["ctc_glint"] == "Y" and "crs_ctl" in ctlctl:
            self.crint = "Y"
            self.crs_ctl = ctlctl["crs_ctl"]
        else:
            self.crint = "N"
            self.crs_ctl = None
        drsctl = gc.getCtl("drsctl", self.conum, error=False)
        if drsctl and drsctl["ctd_glint"] == "Y" and "drs_ctl" in ctlctl:
            self.drint = "Y"
            self.drs_ctl = ctlctl["drs_ctl"]
        else:
            self.drint = "N"
            self.drs_ctl = None
        memctl = gc.getCtl("memctl", self.conum, error=False)
        if memctl and memctl["mcm_glint"] == "Y" and "mem_ctl" in ctlctl:
            self.mlint = "Y"
            self.mem_ctl = ctlctl["mem_ctl"]
        else:
            self.mlint = "N"
            self.mem_ctl = None
        strctl = gc.getCtl("strctl", self.conum, error=False)
        if strctl and strctl["cts_glint"] == "Y" and "stk_soh" in ctlctl:
            self.stint = "Y"
            self.str_ctl = ctlctl["stk_soh"]
        else:
            self.stint = "N"
            self.str_ctl = None
        assctl = gc.getCtl("assctl", self.conum, error=False)
        if not assctl:
            self.arint = "N"
        else:
            self.arint = assctl["cta_glint"]
        rtlctl = gc.getCtl("rtlctl", self.conum, error=False)
        if not rtlctl:
            self.rtint = "N"
        else:
            self.rtint = rtlctl["ctr_glint"]
        wagctl = gc.getCtl("wagctl", self.conum, error=False)
        if wagctl and wagctl["ctw_glint"] == "Y" and "wag_slc" in ctlctl:
            self.wgint = "Y"
            self.sln_ctl = ctlctl["wag_slc"]
        else:
            self.wgint = "N"
            self.sln_ctl = None

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["genbal", "gentrn"],
            prog=__name__)
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.start = self.period[1][0]
        self.s_per = self.start / 100
        self.e_per = self.period[2][0] / 100

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "General Ledger Check Integrated Controls (%s)" % __name__)
        fld = (
            (("T",0,0,0),"ID2",10,"Cut Off Period","Cut Off Period",
                self.e_per,"N",self.doPeriod,None,None,("efld",)),)
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doPeriod(self, frt, pag, r, c, p, i, w):
        if w < self.s_per or w > self.e_per:
            return "Invalid Period"
        self.period = w

    def doEnd(self):
        self.doProcess()
        self.doExit()

    def doProcess(self):
        if self.crint == "Y":
            self.gcr_bal = 0.0
            o = self.sql.getRec(tables="genbal", cols=["glo_cyr"],
                where=[("glo_cono", "=", self.conum), ("glo_acno", "=",
                self.crs_ctl), ("glo_trdt", "=", self.start)], limit=1)
            if o:
                b = CCD(o[0], "SD", 13.2)
            else:
                b = CCD(0, "SD", 13.2)
            self.gcr_bal = float(ASD(self.gcr_bal) + ASD(b.work))
            o = self.sql.getRec(tables="gentrn",
                cols=["round(sum(glt_tramt), 2)"], where=[("glt_cono", "=",
                self.conum), ("glt_acno", "=", self.crs_ctl), ("glt_curdt",
                ">=", self.s_per), ("glt_curdt", "<=", self.period)], limit=1)
            if o and o[0]:
                b = CCD(float(o[0]), "SD", 13.2)
            else:
                b = CCD(0, "SD", 13.2)
            self.gcr_bal = float(ASD(self.gcr_bal) + ASD(b.work))
            self.gcr_bal = float(ASD(0) - ASD(self.gcr_bal))
            bal = self.sql.getRec(tables="crstrn",
                cols=["round(sum(crt_tramt), 2)"], where=[("crt_cono", "=",
                self.conum), ("crt_curdt", "<=", self.period)], limit=1)
            if bal and bal[0]:
                self.crs_bal = bal[0]
            else:
                self.crs_bal = 0.00
        else:
            self.gcr_bal = 0.00
            self.crs_bal = 0.00
        if self.drint == "Y":
            self.gdr_bal = 0.0
            o = self.sql.getRec(tables="genbal", cols=["glo_cyr"],
                where=[("glo_cono", "=", self.conum), ("glo_acno", "=",
                self.drs_ctl), ("glo_trdt", "=", self.start)], limit=1)
            if o:
                b = CCD(o[0], "SD", 13.2)
            else:
                b = CCD(0, "SD", 13.2)
            self.gdr_bal = float(ASD(self.gdr_bal) + ASD(b.work))
            o = self.sql.getRec(tables="gentrn",
                cols=["round(sum(glt_tramt), 2)"], where=[("glt_cono", "=",
                self.conum), ("glt_acno", "=", self.drs_ctl), ("glt_curdt",
                ">=", self.s_per), ("glt_curdt", "<=", self.period)], limit=1)
            if o and o[0]:
                b = CCD(float(o[0]), "SD", 13.2)
            else:
                b = CCD(0, "SD", 13.2)
            self.gdr_bal = float(ASD(self.gdr_bal) + ASD(b.work))
            bal = self.sql.getRec(tables="drstrn",
                cols=["round(sum(drt_tramt), 2)"], where=[("drt_cono", "=",
                self.conum), ("drt_curdt", "<=", self.period)], limit=1)
            if bal and bal[0]:
                self.drs_bal = bal[0]
            else:
                self.drs_bal = 0.00
        else:
            self.gdr_bal = 0.00
            self.drs_bal = 0.00
        if self.mlint == "Y":
            self.gml_bal = 0.0
            o = self.sql.getRec(tables="genbal", cols=["glo_cyr"],
                where=[("glo_cono", "=", self.conum), ("glo_acno", "=",
                self.mem_ctl), ("glo_trdt", "=", self.start)], limit=1)
            if o:
                b = CCD(o[0], "SD", 13.2)
            else:
                b = CCD(0, "SD", 13.2)
            self.gml_bal = float(ASD(self.gml_bal) + ASD(b.work))
            o = self.sql.getRec(tables="gentrn",
                cols=["round(sum(glt_tramt), 2)"], where=[("glt_cono", "=",
                self.conum), ("glt_acno", "=", self.mem_ctl), ("glt_curdt",
                ">=", self.s_per), ("glt_curdt", "<=", self.period)], limit=1)
            if o and o[0]:
                b = CCD(float(o[0]), "SD", 13.2)
            else:
                b = CCD(0, "SD", 13.2)
            self.gml_bal = float(ASD(self.gml_bal) + ASD(b.work))
            bal = self.sql.getRec(tables="memtrn",
                cols=["round(sum(mlt_tramt), 2)"], where=[("mlt_cono", "=",
                self.conum), ("mlt_curdt", "<=", self.period)], limit=1)
            if bal and bal[0]:
                self.mem_bal = bal[0]
            else:
                self.mem_bal = 0.00
        else:
            self.gml_bal = 0.00
            self.mem_bal = 0.00
        if self.stint == "Y":
            self.gst_bal = 0.0
            o = self.sql.getRec(tables="genbal", cols=["glo_cyr"],
                where=[("glo_cono", "=", self.conum), ("glo_acno", "=",
                self.str_ctl), ("glo_trdt", "=", self.start)], limit=1)
            if o:
                b = CCD(o[0], "SD", 13.2)
            else:
                b = CCD(0, "SD", 13.2)
            self.gst_bal = float(ASD(self.gst_bal) + ASD(b.work))
            o = self.sql.getRec(tables="gentrn",
                cols=["round(sum(glt_tramt), 2)"], where=[("glt_cono", "=",
                self.conum), ("glt_acno", "=", self.str_ctl), ("glt_curdt",
                ">=", self.s_per), ("glt_curdt", "<=", self.period)], limit=1)
            if o and o[0]:
                b = CCD(float(o[0]), "SD", 13.2)
            else:
                b = CCD(0, "SD", 13.2)
            self.gst_bal = float(ASD(self.gst_bal) + ASD(b.work))
            o = self.sql.getRec(tables="strtrn",
                cols=["round(sum(stt_cost), 2)"], where=[("stt_cono", "=",
                self.conum), ("stt_curdt", "<=", self.period)], limit=1)
            if o and o[0]:
                self.str_bal = o[0]
            else:
                self.str_bal = 0.00
        else:
            self.gst_bal = 0.00
            self.str_bal = 0.00
        if self.arint == "Y":
            self.gar_bal = 0.0
            grp = self.sql.getRec(tables="assgrp", cols=["asg_assacc",
                "asg_depacc"], where=[("asg_cono", "=", self.conum)])
            if grp:
                done = []
                for g in grp:
                    for acc in g:
                        if acc in done:
                            continue
                        o = self.sql.getRec(tables="genbal", cols=["glo_cyr"],
                            where=[("glo_cono", "=", self.conum), ("glo_acno",
                            "=", acc), ("glo_trdt", "=", self.start)],
                            limit=1)
                        if o:
                            b = CCD(o[0], "SD", 13.2)
                        else:
                            b = CCD(0, "SD", 13.2)
                        self.gar_bal = float(ASD(self.gar_bal) + ASD(b.work))
                        o = self.sql.getRec(tables="gentrn",
                            cols=["round(sum(glt_tramt), 2)"],
                            where=[("glt_cono", "=", self.conum), ("glt_acno",
                            "=", acc), ("glt_curdt", ">=", self.s_per),
                            ("glt_curdt", "<=", self.period)], limit=1)
                        if o and o[0]:
                            b = CCD(float(o[0]), "SD", 13.2)
                        else:
                            b = CCD(0, "SD", 13.2)
                        self.gar_bal = float(ASD(self.gar_bal) + ASD(b.work))
                        done.append(acc)
            o = self.sql.getRec(tables="asstrn",
                cols=["round(sum(ast_amt1), 2)"], where=[("ast_cono", "=",
                self.conum), ("ast_curdt", "<=", self.period)], limit=1)
            if o and o[0]:
                self.ass_bal = o[0]
            else:
                self.ass_bal = 0.00
        else:
            self.gar_bal = 0.00
            self.ass_bal = 0.00
        if self.rtint == "Y":
            self.grt_bal = 0.0
            prm = self.sql.getRec(tables="rtlprm", cols=["rtp_rtlacc"],
                where=[("rtp_cono", "=", self.conum)])
            if prm:
                done = []
                for acc in prm:
                    if acc[0] in done:
                        continue
                    o = self.sql.getRec(tables="genbal", cols=["glo_cyr"],
                        where=[("glo_cono", "=", self.conum), ("glo_acno",
                        "=", acc[0]), ("glo_trdt", "=", self.start)],
                        limit=1)
                    if o:
                        b = CCD(o[0], "SD", 13.2)
                    else:
                        b = CCD(0, "SD", 13.2)
                    self.grt_bal = float(ASD(self.grt_bal) + ASD(b.work))
                    o = self.sql.getRec(tables="gentrn",
                        cols=["round(sum(glt_tramt), 2)"],
                        where=[("glt_cono", "=", self.conum), ("glt_acno",
                        "=", acc[0]), ("glt_curdt", ">=", self.s_per),
                        ("glt_curdt", "<=", self.period)], limit=1)
                    if o and o[0]:
                        b = CCD(float(o[0]), "SD", 13.2)
                    else:
                        b = CCD(0, "SD", 13.2)
                    self.grt_bal = float(ASD(self.grt_bal) + ASD(b.work))
                    done.append(acc[0])
            o = self.sql.getRec(tables="rtltrn",
                cols=["round(sum(rtt_tramt), 2)"], where=[("rtt_cono", "=",
                self.conum), ("rtt_curdt", "<=", self.period)], limit=1)
            if o and o[0]:
                self.rtl_bal = o[0]
            else:
                self.rtl_bal = 0.00
        else:
            self.grt_bal = 0.00
            self.rtl_bal = 0.00
        if self.wgint == "Y":
            self.gsl_bal = 0.0
            o = self.sql.getRec(tables="genbal", cols=["glo_cyr"],
                where=[("glo_cono", "=", self.conum), ("glo_acno", "=",
                self.sln_ctl), ("glo_trdt", "=", self.start)], limit=1)
            if o:
                b = CCD(o[0], "SD", 13.2)
            else:
                b = CCD(0, "SD", 13.2)
            self.gsl_bal = float(ASD(self.gsl_bal) + ASD(b.work))
            o = self.sql.getRec(tables="gentrn",
                cols=["round(sum(glt_tramt), 2)"], where=[("glt_cono", "=",
                self.conum), ("glt_acno", "=", self.sln_ctl), ("glt_curdt",
                ">=", self.s_per), ("glt_curdt", "<=", self.period)], limit=1)
            if o and o[0]:
                b = CCD(float(o[0]), "SD", 13.2)
            else:
                b = CCD(0, "SD", 13.2)
            self.gsl_bal = float(ASD(self.gsl_bal) + ASD(b.work))
            o = self.sql.getRec(tables="wagltf",
                cols=["round(sum(wlt_amt), 2)"], where=[("wlt_cono", "=",
                self.conum), ("wlt_curdt", "<=", self.period)], limit=1)
            if o and o[0]:
                self.sln_bal = o[0]
            else:
                self.sln_bal = 0.00
        else:
            self.gsl_bal = 0.00
            self.sln_bal = 0.00
        self.printReport()

    def printReport(self):
        self.head = ("%03u %-30s %2s %33s %3s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        self.pageHeading()
        if self.crint == "Y":
            des = "Creditors Ledger"
            a1 = CCD(self.gcr_bal, "SD", 20.2)
            a2 = CCD(self.crs_bal, "SD", 20.2)
            a = float(ASD(a1.work) - ASD(a2.work))
            a3 = CCD(a, "SD", 20.2)
            self.t.textLine("%-20s %s %s %s" % (des, a1.disp, a2.disp, a3.disp))
        if self.drint == "Y":
            des = "Debtors Ledger"
            a1 = CCD(self.gdr_bal, "SD", 20.2)
            a2 = CCD(self.drs_bal, "SD", 20.2)
            a = float(ASD(a1.work) - ASD(a2.work))
            a3 = CCD(a, "SD", 20.2)
            self.t.textLine("%-20s %s %s %s" % (des, a1.disp, a2.disp, a3.disp))
        if self.mlint == "Y":
            des = "Members Ledger"
            a1 = CCD(self.gml_bal, "SD", 20.2)
            a2 = CCD(self.mem_bal, "SD", 20.2)
            a = float(ASD(a1.work) - ASD(a2.work))
            a3 = CCD(a, "SD", 20.2)
            self.t.textLine("%-20s %s %s %s" % (des, a1.disp, a2.disp, a3.disp))
        if self.stint == "Y":
            des = "Stores Ledger"
            a1 = CCD(self.gst_bal, "SD", 20.2)
            a2 = CCD(self.str_bal, "SD", 20.2)
            a = float(ASD(a1.work) - ASD(a2.work))
            a3 = CCD(a, "SD", 20.2)
            self.t.textLine("%-20s %s %s %s" % (des, a1.disp, a2.disp, a3.disp))
        if self.arint == "Y":
            des = "Assets Register"
            a1 = CCD(self.gar_bal, "SD", 20.2)
            a2 = CCD(self.ass_bal, "SD", 20.2)
            a = float(ASD(a1.work) - ASD(a2.work))
            a3 = CCD(a, "SD", 20.2)
            self.t.textLine("%-20s %s %s %s" % (des, a1.disp, a2.disp, a3.disp))
        if self.rtint == "Y":
            des = "Rentals Ledger"
            a1 = CCD(self.grt_bal, "SD", 20.2)
            a2 = CCD(self.rtl_bal, "SD", 20.2)
            a = float(ASD(a1.work) - ASD(a2.work))
            a3 = CCD(a, "SD", 20.2)
            self.t.textLine("%-20s %s %s %s" % (des, a1.disp, a2.disp, a3.disp))
        if self.wgint == "Y":
            des = "Staff Loans"
            a1 = CCD(self.gsl_bal, "SD", 20.2)
            a2 = CCD(self.sln_bal, "SD", 20.2)
            a = float(ASD(a1.work) - ASD(a2.work))
            a3 = CCD(a, "SD", 20.2)
            self.t.textLine("%-20s %s %s %s" % (des, a1.disp, a2.disp, a3.disp))
        self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
        self.doExit()
            
    def pageHeading(self, desc=None):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-48s %-12s %13s %5s" % \
            ("General Ledger Integrated Controls up to Period ",
            self.df.t_disp[0][0][1], "Page", self.pgnum))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.textLine("%-24s %-29s %-16s %-20s" % ("Description",
            "Control Account", "Ledger", "Difference"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 6

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()
