#! /usr/bin/python

"""
SYNOPSIS
    General Ledger Check Intercompany Balances.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, Sql, TartanDialog
from tartanFunctions import getModName, doPrinter, doReportlab

class gl6040:
    def __init__(self, mf, coy, nam, per):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = per
        no = self.getAccounts()
        if not no:
            self.setVariables()
            self.mainProcess()
            self.mf.startLoop()

    def getAccounts(self):
        self.sql = Sql(self.mf.dbm, ["ctlmst", "genint", "genbal", "gentrn"],
            prog=__name__)
        all = self.sql.getRec(tables="genint", cols=["cti_inco", "cti_acno"],
            where=[("cti_cono", "=", self.conum)])
        if not all:
            return "no"
        self.accs = []
        for acc in all:
            acn = self.sql.getRec(tables="genint", cols=["cti_acno"],
                where=[("cti_cono", "=", acc[0]), ("cti_inco", "=",
                self.conum)], limit=1)
            if acn:
                self.accs.append([acc[0], acc[1], acn[0]])

    def setVariables(self):
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.s_per = self.period[1][0] / 100
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "General Ledger Check Intercompany Accounts (%s)" % __name__)
        fld = (
            (("T",0,0,0),"ID1",10,"Reporting Date","Reporting Date",
                self.period[2][0],"N",self.doRepDte,None,None,None),)
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doRepDte(self, frt, pag, r, c, p, i, w):
        if w < self.period[1][0] or w > self.period[2][0]:
            return "Invalid Date, Not in Financial Period"
        self.e_per = w / 100

    def doEnd(self):
        self.printReport()
        self.doExit()

    def printReport(self):
        self.head = ("%03u %-30s %2s %35s %3s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        for acc in self.accs:
            if self.pglin > self.lpp:
                if self.pgnum != 0:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            n = self.sql.getRec(tables="ctlmst", cols=["ctm_name"],
                where=[("ctm_cono", "=", acc[0])], limit=1)
            if not n:
                continue
            n = CCD(n[0], "NA", 30)
            o = self.sql.getRec(tables="genbal", cols=["glo_cyr"],
                where=[("glo_cono", "=", self.conum), ("glo_acno", "=",
                acc[1]), ("glo_trdt", "=", self.period[1][0])], limit=1)
            if o:
                b = CCD(o[0], "SD", 13.2)
            else:
                b = CCD(0, "SD", 13.2)
            bal1 = b.work
            o = self.sql.getRec(tables="gentrn",
                cols=["round(sum(glt_tramt), 2)"], where=[("glt_cono", "=",
                self.conum), ("glt_acno", "=", acc[1]), ("glt_curdt", ">=",
                self.s_per), ("glt_curdt", "<=", self.e_per)], limit=1)
            if o and o[0]:
                b = CCD(float(o[0]), "SD", 13.2)
            else:
                b = CCD(0, "SD", 13.2)
            bal1 = float(ASD(bal1) + ASD(b.work))
            o = self.sql.getRec(tables="genbal", cols=["glo_cyr"],
                where=[("glo_cono", "=", acc[0]), ("glo_acno", "=", acc[2]),
                ("glo_trdt", "=", self.period[1][0])], limit=1)
            if o:
                b = CCD(o[0], "SD", 13.2)
            else:
                b = CCD(0, "SD", 13.2)
            bal2 = b.work
            o = self.sql.getRec(tables="gentrn",
                cols=["round(sum(glt_tramt), 2)"], where=[("glt_cono", "=",
                acc[0]), ("glt_acno", "=", acc[2]), ("glt_curdt", ">=",
                self.s_per), ("glt_curdt", "<=", self.e_per)], limit=1)
            if o and o[0]:
                b = CCD(float(o[0]), "SD", 13.2)
            else:
                b = CCD(0, "SD", 13.2)
            bal2 = float(ASD(bal2) + ASD(b.work))
            a0 = CCD(acc[0], "UI", 3)
            a1 = CCD(bal1, "SD", 13.2)
            a2 = CCD(bal2, "SD", 13.2)
            a = float(ASD(a1.work) + ASD(a2.work))
            a3 = CCD(a, "SD", 13.2)
            self.t.textLine("%-3s %-32s %15s %15s %15s" % \
                (a0.disp, n.disp, a1.disp, a2.disp, a3.disp))
            self.pglin = self.pglin + 1
        if self.t:
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
        self.doExit()
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-40s %-20s %15s %5s" % \
            ("General Ledger Integrated Controls as at ",
            self.df.t_disp[0][0][0], "Page", self.pgnum))
        self.t.textLine()
        self.t.textLine("%-3s %-32s%15s %15s %15s" % \
            ("Coy", "Company-Name", "Balance-1", "Balance-2", "Difference"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 6

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

