#! /usr/bin/env python

"""
SYNOPSIS
    General Ledger Control File Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import Sql, TartanDialog

class glc110:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.user = usr
        no = self.setVariables()
        if not no:
            self.drawDialog()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["genctl", "chglog"], prog=__name__)
        if self.sql.error:
            return "no"
        self.acc = self.sql.getRec(tables="genctl", where=[("ctg_cono", "=",
            self.conum)], limit=1)
        if not self.acc:
            self.new = True
            self.acc = [self.conum, "N", 0]
        else:
            self.new = False

    def drawDialog(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "General Ledger Control File Maintenance (%s)" % __name__)
        r1s = (("Yes","Y"),("No","N"))
        r2s = (("Exclusive","E"),("Inclusive","I"))
        self.fld = (
            (("T",0,0,0),("IRB",r1s),0,"Departments","Departments",
                self.acc[1],"N",self.doDeps,None,None,None),
            (("T",0,1,0),"IUI",1,"Number of Digits","Number of Digits",
                self.acc[2],"N",self.doDigits,None,None,("between",0,3)))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,1),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)
        if not self.new:
            for n, f in enumerate(self.acc[1:]):
                self.df.loadEntry("T", 0, n, data=f)

    def doDeps(self, frt, pag, r, c, p, i, w):
        self.depts = w
        if self.depts == "N":
            self.digits = 0
            self.df.loadEntry(frt, pag, p+1, data=0)
            return "sk1"

    def doDigits(self, frt, pag, r, c, p, i, w):
        if not w:
            return "Invalid Number of Digits"
        self.digits = w

    def doEnd(self):
        data = [self.conum]
        for x in range(0, len(self.df.t_work[0][0])):
            data.append(self.df.t_work[0][0][x])
        if self.new:
            self.sql.insRec("genctl", data=data)
        elif data != self.acc:
            self.sql.updRec("genctl", data=data, where=[("ctg_cono", "=",
                self.conum)])
            dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
            for num, dat in enumerate(self.acc):
                if dat != data[num]:
                    self.sql.insRec("chglog", data=["genctl", "U",
                        "%03i" % self.conum, self.sql.genctl_col[num],
                        dte, self.user, str(dat), str(data[num]), 0])
        self.mf.dbm.commitDbase()
        self.doExit()

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

