#! /usr/bin/python

"""
SYNOPSIS
    General Ledger Control Records Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import GetCtl, Sql, TartanDialog

class glc210:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.drawDialog()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlctl", "genmst"], prog=__name__)
        if self.sql.error:
            return "no"
        #-------------------------------
        # General Ledger Control Records
        #-------------------------------
        # Get Company Details
        gc = GetCtl(self.mf)
        ctlmst = gc.getCtl("ctlmst", self.conum)
        if not ctlmst:
            return "no"
        # Standard Controls
        self.glctrl = [
            ("bank_1", "Bank Control 1"),
            ("bank_2", "Bank Control 2"),
            ("bank_3", "Bank Control 3"),
            ("bank_4", "Bank Control 4"),
            ("bank_5", "Bank Control 5"),
            ("bank_6", "Bank Control 6"),
            ("bank_7", "Bank Control 7"),
            ("bank_8", "Bank Control 8"),
            ("bank_9", "Bank Control 9"),
            ("bank_10", "Bank Control 10"),
            ("p_cash", "Petty Cash Control"),
            ("crs_ctl", "Creditors Control"),
            ("drs_ctl", "Debtors Control"),
            ("ret_inc", "Retained Income"),
            ("vat_ctl", "VAT Control")]
        # Check for Integrated Systems
        mod = []
        for x in range(0, 21, 2):
            mod.append(ctlmst["ctm_modules"][x:x+2])
        # Load Ass Integration
        if mod.count("CR"):
            self.glctrl.extend([
                ("dis_rec", "Discount Received")])
        if mod.count("DR"):
            self.glctrl.extend([
                ("dis_all", "Discount Allowed")])
        if mod.count("RC"):
            self.glctrl.extend([
                ("rca_com", "Commission Raised"),
                ("rca_dep", "Deposit Control"),
                ("rca_fee", "Contract Fees Raised"),
                ("rca_orx", "Owners Charges"),
                ("rca_own", "Owners Control"),
                ("rca_tnt", "Tenants Control"),
                ("rca_trx", "Tenants Charges")])
        if mod.count("ST"):
            self.glctrl.extend([
                ("stk_soh", "Stock on Hand"),
                ("stk_susp", "Stock Reconciliation")])
        if mod.count("WG") or mod.count("SL"):
            self.glctrl.extend([
                ("wag_ctl", "Salaries Control"),
                ("wag_slc", "Staff Loans Control"),
                ("wag_sli", "Staff Loans Interest")])
        if mod.count("ML"):
            self.glctrl.extend([
                ("mem_ctl", "Members Control"),
                ("mem_pen", "Members Penalties")])

    def drawDialog(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "General Ledger Control Records Maintenance (%s)" % __name__)
        ctl = {
            "stype": "C",
            "titl": "Select Control",
            "data": self.glctrl}
        glm = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "where": [("glm_cono", "=", self.conum)]}
        r1s = (
            ("None", "N"),
            ("OFX", "O"),
            ("QIF", "Q"),
            ("S/Bank Online", "S"))
        r2s = (
            ("None", "N"),
            ("CCYYMMDD", "A"),
            ("DDMMCCYY", "B"),
            ("MMDD(CC)YY", "C"))
        fld = (
            (("T",0,0,0),"INA",10,"Code","Control Code",
                "","Y",self.doCode,ctl,None,None),
            (("T",0,0,25),"ONA",30,"Description"),
            (("T",0,1,0),"IUI",7,"G/L Acc-Num","G/L Account Number",
                "","N",self.doAccNum,glm,self.doDelete,("notzero",)),
            (("T",0,1,25),"ONA",30,"Description"),
            (("T",0,2,0),"INA",16,"Bank Account","Bank Account Number",
                "","N",self.doImpBnk,None,None,("efld",)),
            (("T",0,3,0),("IRB",r1s),0,"Import Format","Import Format",
                "N","N",self.doImpFmt,None,None,None),
            (("T",0,4,0),("IRB",r2s),0,"Date Format","Date Format",
                "A","N",self.doDteFmt,None,None,None))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,3),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,3),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, butt=but,
            tend=tnd, txit=txt)

    def doCode(self, frt, pag, r, c, p, i, w):
        self.code = w
        d1 = ""
        for ctl in self.glctrl:
            if self.code in ctl:
                d1 = ctl[1]
                break
        if not d1:
            return "Invalid Control Code"
        self.df.loadEntry(frt, pag, p+1, data=d1)
        acc = self.sql.getRec(tables="ctlctl", where=[("ctl_cono", "=",
            self.conum), ("ctl_code", "=", self.code)], limit=1)
        if not acc:
            self.new = "y"
        else:
            self.new = "n"
            col = self.sql.ctlctl_col
            desc = self.readAcno(acc[col.index("ctl_conacc")])
            if not desc:
                desc = [""]
            self.df.loadEntry(frt, pag, p+2, data=acc[col.index("ctl_conacc")])
            self.df.loadEntry(frt, pag, p+3, data=desc[0])
            self.df.loadEntry(frt, pag, p+4, data=acc[col.index("ctl_bankac")])
            self.df.loadEntry(frt, pag, p+5, data=acc[col.index("ctl_impfmt")])
            self.df.loadEntry(frt, pag, p+6, data=acc[col.index("ctl_dtefmt")])

    def doDelete(self):
        self.sql.delRec("ctlctl", where=[("ctl_cono", "=", self.conum),
            ("ctl_code", "=", self.code)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccNum(self, frt, pag, r, c, p, i, w):
        desc = self.readAcno(w)
        if not desc:
            return "Invalid Account Number"
        self.df.loadEntry(frt, pag, p+1, data=desc[0])
        if self.code[:4] != "bank":
            self.impfmt = "N"
            self.bankac = ""
            self.dtefmt = "N"
            return "nd"

    def readAcno(self, acno):
        self.acno = acno
        acc = self.sql.getRec(tables="genmst", cols=["glm_desc"],
            where=[("glm_cono", "=", self.conum), ("glm_acno", "=",
            self.acno)], limit=1)
        return acc

    def doImpBnk(self, frt, pag, r, c, p, i, w):
        self.bankac = w
        if not self.bankac:
            self.impfmt = "N"
            self.dtefmt = "N"
            self.df.loadEntry(frt, pag, p+1, data=self.impfmt)
            self.df.loadEntry(frt, pag, p+2, data=self.dtefmt)
            return "sk2"

    def doImpFmt(self, frt, pag, r, c, p, i, w):
        self.impfmt = w
        if self.impfmt == "N":
            self.dtefmt = "N"
            self.df.loadEntry(frt, pag, p+1, data=self.dtefmt)
            return "nd"

    def doDteFmt(self, frt, pag, r, c, p, i, w):
        if w not in ("A", "B", "C"):
            return "Invalid Date Format"
        self.dtefmt = w

    def doEnd(self):
        data = [self.conum, self.code, self.df.t_work[0][0][1], self.acno,
            self.bankac, self.impfmt, self.dtefmt]
        if self.new == "y":
            self.sql.insRec("ctlctl", data=data)
        else:
            self.sql.updRec("ctlctl", data=data, where=[("ctl_cono", "=",
            self.conum), ("ctl_code", "=", self.code)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

