#! /usr/bin/python

"""
SYNOPSIS
    General Ledger Intercompany Records Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import Sql, TartanDialog

class glc310:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlmst", "genmst", "genint"],
            prog=__name__)
        if self.sql.error:
            return self.sql.error

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "General Ledger Intercompany Records Maintenance (%s)" % __name__)
        coy = {
            "stype": "R",
            "tables": ("ctlmst",),
            "cols": (
                ("ctm_cono", "", 0, "Coy-Num"),
                ("ctm_name", "", 0, "Name", "Y")),
            "where": [("ctm_cono", "!=", self.conum)]}
        gl1 = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "where": [("glm_cono", "=", self.conum)]}
        gl2 = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "whera": [["T", "glm_cono", 0, 0]]}
        fld = (
            (("T",0,0,0,14),"IUI",3,"Coy-Num","Company Number",
                "","Y",self.doCoyNum,coy,None,None),
            (("T",0,0,17),"ONA",30,""),
            (("T",0,1,0,10),"IUI",7,"Acc-Num-1","G/L Account Number",
                "","N",self.doAccNum1,gl1,self.doDelete,("notzero",)),
            (("T",0,1,17),"ONA",30,""),
            (("T",0,2,0,10),"IUI",7,"Acc-Num-2","G/L Account Number",
                "","N",self.doAccNum2,gl2,None,("notzero",)),
            (("T",0,2,17),"ONA",30,""))
        but = (
            ("Cancel",None,self.doCancel,0,("T",0,3),("T",0,1)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, butt=but,
            tend=tnd, txit=txt)

    def doCoyNum(self, frt, pag, r, c, p, i, w):
        if w == 0:
            return "Invalid Company Number"
        if w == self.conum:
            return "Invalid Company, Same Company Not Allowed"
        self.cono2 = w
        acc = self.sql.getRec(tables="ctlmst", cols=["ctm_name"],
            where=[("ctm_cono", "=", self.cono2)], limit=1)
        if not acc:
            return "Invalid Company Number"
        self.df.loadEntry(frt, pag, 1, data=acc[0])
        acc1 = self.sql.getRec(tables="genint", cols=["cti_acno"],
            where=[("cti_cono", "=", self.conum), ("cti_inco", "=",
            self.cono2)], limit=1)
        if not acc1:
            self.new1 = "y"
        else:
            self.new1 = "n"
            self.acno1 = acc1[0]
            desc = self.readAcno(self.conum, self.acno1)
            self.df.loadEntry(frt, pag, 2, data=self.acno1)
            self.df.loadEntry(frt, pag, 3, data=desc[0])
        acc2 = self.sql.getRec(tables="genint", cols=["cti_acno"],
            where=[("cti_cono", "=", self.cono2), ("cti_inco", "=",
            self.conum)], limit=1)
        if not acc2:
            self.new2 = "y"
        else:
            self.new2 = "n"
            self.acno2 = acc2[0]
            desc = self.readAcno(self.cono2, self.acno2)
            self.df.loadEntry(frt, pag, 4, data=self.acno2)
            self.df.loadEntry(frt, pag, 5, data=desc[0])

    def doDelete(self):
        if self.new1 == "n":
            self.sql.delRec("genint", where=[("cti_cono", "=", self.conum),
                ("cti_inco", "=", self.cono2)])
        if self.new2 == "n":
            self.sql.delRec("genint", where=[("cti_cono", "=", self.cono2),
                ("cti_inco", "=", self.conum)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccNum1(self, frt, pag, r, c, p, i, w):
        desc = self.readAcno(self.conum, w)
        if not desc:
            return "Invalid Account Number"
        self.acno1 = w
        self.df.loadEntry(frt, pag, p+1, data=desc[0])

    def doAccNum2(self, frt, pag, r, c, p, i, w):
        desc = self.readAcno(self.cono2, w)
        if not desc:
            return "Invalid Account Number"
        self.acno2 = w
        self.df.loadEntry(frt, pag, p+1, data=desc[0])

    def readAcno(self, cono, acno):
        acc = self.sql.getRec(tables="genmst", cols=["glm_desc"],
            where=[("glm_cono", "=", cono), ("glm_acno", "=", acno)], limit=1)
        return acc

    def doEnd(self):
        data1 = [self.conum, self.cono2, self.acno1]
        data2 = [self.cono2, self.conum, self.acno2]
        if self.new1 == "y":
            self.sql.insRec("genint", data=data1)
        else:
            self.sql.updRec("genint", data=data1, where=[("cti_cono", "=",
                self.conum), ("cti_inco", "=", self.cono2)])
        if self.new2 == "y":
            self.sql.insRec("genint", data=data2)
        else:
            self.sql.updRec("genint", data=data2, where=[("cti_cono", "=",
                self.cono2), ("cti_inco", "=", self.conum)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

