#! /usr/bin/python

"""
SYNOPSIS
    Membership Maintenance.

    This file is part of Tartan Systems (TARTAN).

    The PHOTODIR and ACCESSCTL environment variables are as follows:

        PHOTODIR='path_to_photo_images'
        ACCESSCTL='["dbhost", "dbname", "dbuser", "dbpwd"]'

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, time
from TartanClasses import CCD, Dbase, GetCtl, NotesCreate, PwdConfirm, RepPrt
from TartanClasses import Sql, SChoice, SelectChoice, ShowImage, SplashScreen
from TartanClasses import TabPrt, TartanDialog 
from tartanFunctions import askQuestion, callModule, dateDiff, doChkCatChg
from tartanFunctions import getTrn, mthendDate, showError
from tartanWork import countries, mltrtp

class ml1010:
    def __init__(self, mf, conum, conam, period, user, lvl=0):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        self.period = period
        self.capnm = user
        self.level = lvl
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        tabs = ["chglog", "memadd", "memcat", "memctc", "memctk", "memctp",
            "memctl", "memcto", "memkon", "memlnk", "memmst", "memsta",
            "memtrn", "memtrs"]
        mc = GetCtl(self.mf)
        self.taxdf = mc.getCtl("ctlmst", self.conum)["ctm_taxdf"]
        memctl = mc.getCtl("memctl", self.conum)
        if not memctl:
           return "no"
        self.ldays = memctl["mcm_ldays"]
        self.lme = memctl["mcm_lme"]
        yy = self.lme / 10000
        mm = ((self.lme / 100) % 100) + 1
        while mm > 12:
            yy += 1
            mm -= 12
        self.tme = mthendDate((yy*10000) + (mm*100) + 1)
        if self.tme / 100 == self.period[2][0] / 100:
            self.ynd = True
        else:
            self.ynd = False
        self.curdt = self.tme / 100
        mm += 1
        while mm > 12:
            yy += 1
            mm -= 12
        self.nxtdt = (yy * 10000) + (mm * 100) + 1
        if "PHOTODIR" in os.environ:
            self.photo = os.environ["PHOTODIR"]
        else:
            self.photo = memctl["mcm_photo"]
        if not os.path.isdir(self.photo):
            self.photo = os.path.join(self.mf.prgdir, "img")
        if "ACCESSCTL" in os.environ:
            try:
                memctl["mcm_ahst"], memctl["mcm_adbn"], memctl["mcm_ausr"], \
                    memctl["mcm_apwd"] = eval(os.environ["ACCESSCTL"])
                self.acs = "Y"
            except:
                self.acs = memctl["mcm_acs"]
        else:
            self.acs = memctl["mcm_acs"]
        if self.acs == "Y":
            try:
                self.dba = Dbase(rcdic={
                    "dbase": "FBird",
                    "dbhost": "%s" % str(memctl["mcm_ahst"]),
                    "dbname": "%s" % str(memctl["mcm_adbn"]),
                    "dbuser": "%s" % str(memctl["mcm_ausr"]),
                    "dbpwd": "%s" % str(memctl["mcm_apwd"]),
                    "dbdir": "",
                    "wrkdir": self.mf.wrkdir})
                self.dba.openDbase()
                self.sqa = Sql(self.dba, prog=__name__)
            except:
                self.acs = "N"
        self.sql = Sql(self.mf.dbm, tabs, prog=__name__)
        t = time.localtime()
        self.sysdt = ((t[0] * 10000) + (t[1] * 100) + t[2])
        if dateDiff(self.tme, self.sysdt, "months") > 1:
            showError(self.mf.body, "Date Error",
                """More than Two Month Ends Have Not Been Run!!!

This Could Have Serious Date and Transaction Issues!!!

Please Contact your IT Manager and Report This!!!""")
        self.new = False
        self.edit = False
        self.idnum = 0
        self.proposer = 0
        self.seconder = 0
        self.years = 0
        self.select = "T"
        self.image = None

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Members Maintenance",)
        mlm = {
            "stype": "R",
            "tables": ("memmst",),
            "cols": (
                ("mlm_memno", "", 0, "Mem-No"),
                ("mlm_oldno", "", 0, "Old-No"),
                ("mlm_idnum", "", 0, "Identity-Numb"),
                ("mlm_gender", "", 0, "G"),
                ("mlm_state", "", 0, "S"),
                ("mlm_surname", "", 0, "Surname", "Y"),
                ("mlm_names", "", 0, "Names", "F")),
            "where": [("mlm_cono", "=", self.conum)],
            "order": "mlm_surname, mlm_names",
            "sort": False}
        data = []
        keys = countries.keys()
        keys.sort()
        for key in keys:
            data.append((key, countries[key][1]))
        nat = {
            "stype": "C",
            "titl": "Select the Required Country",
            "data": data}
        typ = {
            "stype": "C",
            "titl": "Select the Required Type",
            "data": (("A", "Fees"), ("B", "Category"), ("C", "Section"))}
        cat = {
            "stype": "R",
            "tables": ("memctc",),
            "cols": (
                ("mcc_code", "", 0, "Cd"),
                ("mcc_desc", "", 0, "Description", "Y")),
            "where": [("mcc_cono", "=", self.conum)],
            "whera": [["C", "mcc_type", 1, 4]]}
        occ = {
            "stype": "R",
            "tables": ("memcto",),
            "cols": (
                ("mco_code", "", 0, "Cd"),
                ("mco_desc", "", 0, "Description", "Y"))}
        kon = {
            "stype": "R",
            "tables": ("memctk",),
            "cols": (
                ("mck_code", "", 0, "Cd"),
                ("mck_desc", "", 0, "Description", "Y"))}
        r1s = (
            ("Male", "M"),
            ("Female", "F"))
        r2s = (
            ("Yes", "Y"),
            ("No", "N"))
        r3s = (
            ("Active", "A"),
            ("Deceased", "D"),
            ("Defaulted", "X"),
            ("Inactive", "I"),
            ("Resigned", "R"),
            ("Suspended", "S"))
        r4s = (
            ("Active", "A"),
            ("Inactive", "I"))
        r5s = [
            ("Status", "U"),
            ("Tartan", "T")]
        chk = self.sql.getRec(tables="memtrs", cols=["count(*)"],
            where=[("mst_cono", "=", self.conum)], limit=1)
        if chk[0]:
            r5s.append(("Sportsman", "S"))
        if self.acs == "Y":
            r5s.append(("Acces Control", "A"))
        tag = (
            ("Personal",self.doTags,None,("T",0,1)),
            ("Addresses",self.doTags,None,("T",0,1)),
            ("Contacts",self.doTags,None,("T",0,1)),
            ("Categories",self.doTags,None,("T",0,1)),
            ("Links",self.doTags,None,("T",0,1)),
            ("Movements",self.doTrans1,None,("T",0,1)),
            ("Photo",self.doPhoto,None,("T",0,1)))
        fld = [
            (("T",0,0,0),"I@mlm_memno",0,"","",
                "","N",self.doMemNo,mlm,None,None),
            (("T",0,0,0),"I@mlm_title",0,"","",
                "","N",self.doTitle,None,None,("notblank",)),
            (("T",0,0,0),"O@mlm_initial",0,""),
            (("T",0,0,0),"ITX",30,"Surname","Surname",
                "","N",self.doSurname,None,None,("notblank",)),
            (("T",1,0,0),"ITX",30,"Names","Names",
                "","N",self.doNames,None,None,("notblank",)),
            (("T",1,1,0),("IRB",r1s),0,"Gender","Gender",
                "M","N",self.doGender,None,None,None),
            (("T",1,2,0),"I@mlm_nation",0,"","",
                "","N",self.doNation,nat,None,("efld",)),
            (("T",1,2,0),"ONA",50,""),
            (("T",1,3,0),"I@mlm_dob",0,"","",
                "","N",self.doDOB,None,None,("efld",)),
            (("T",1,3,37,40),"OUI",3,"Age"),
            (("T",1,4,0),"I@mlm_idnum",0,"","",
                "","N",self.doID,None,None,("efld",)),
            (("T",1,5,0),"I@mlm_occup",0,"","",
                "","N",self.doOccCode,occ,None,("efld",)),
            (("T",1,5,0),"INA",30,"","",
                "","N",self.doOccDesc,None,None,("notblank",)),
            (("T",1,6,0),"I@mlm_proposer",0,"","",
                "","N",self.doProposer,mlm,None,("efld",)),
            (("T",1,6,0),"ONA",40,""),
            (("T",1,7,0),"I@mlm_seconder",0,"","",
                "","N",self.doSeconder,mlm,None,("efld",)),
            (("T",1,7,0),"ONA",40,""),
            (("T",1,8,0),"I@mlm_entry",0,"","",
                "","N",self.doEntry,None,None,("notzero",)),
            (("T",1,8,35,40),"OUI",3,"Years"),
            (("T",1,9,0),("IRB",r2s),0,"Payment Plan","Payment Plan",
                "N","N",self.doPayPlan,None,None,None),
            (("T",1,10,0),"O@mlm_oldno",0,""),
            (("T",1,11,0),("IRB",r3s),0,"Status","Status",
                "A","N",self.doStatus,None,None,None),
            (("T",1,12,0),"I@mlm_sdate",0,"","",
                "","N",self.doDate,None,None,("notzero",)),
            (("T",1,12,33),"OSD",13.2,"Balance")]
        if self.acs == "Y":
            fld.extend([
                (("T",1,13,0),("IRB",r4s),0,"ACS Status","ACS Status",
                    "","N",None,None,None,None),
                (("T",1,14,0),"OdT",16.0,"Date of Last Access")])
        fld.extend([
            (("T",2,0,0,35),"ITX",30,"Postal      Address Line 1",
                "Postal Address Line 1","","N",None,None,None,("efld",)),
            (("T",2,1,0,35),"ITX",30,"            Address Line 2",
                "Postal Address Line 2","","N",None,None,None,("efld",)),
            (("T",2,2,0,35),"ITX",30,"            Address Line 3",
                "Postal Address Line 2","","N",None,None,None,("efld",)),
            (("T",2,3,0,35),"ITX",30,"            City","City",
                "","N",None,None,None,("efld",)),
            (("T",2,4,0,35),"ITX",4,"            Postal Code","Postal Code",
                "","N",None,None,None,("efld",)),
            (("T",2,5,0,35),"ITX",30,"            Region","Region",
                "","N",None,None,None,("efld",)),
            (("T",2,6,0,35),"ITX",30,"            Country","Country",
                "","N",None,None,None,("efld",)),
            (("T",2,7,0,35),("IRB",r2s),0,"Residential Same as Postal","",
                "Y","N",self.doSameAdd,None,None,None),
            (("T",2,8,0,35),"ITX",30,"Residential Address Line 1",
                "Residential Address Line 1","","N",None,None,None,("efld",)),
            (("T",2,9,0,35),"ITX",30,"            Address Line 2",
                "Residential Address Line 1","","N",None,None,None,("efld",)),
            (("T",2,10,0,35),"ITX",30,"            Address Line 3",
                "Residential Address Line 1","","N",None,None,None,("efld",)),
            (("T",2,11,0,35),"ITX",30,"            City","City",
                "","N",None,None,None,("efld",)),
            (("T",2,12,0,35),"ITX",4,"            Postal Code","Postal Code",
                "","N",None,None,None,("efld",)),
            (("T",2,13,0,35),"ITX",30,"            Region","Region",
                "","N",None,None,None,("efld",)),
            (("T",2,14,0,35),"ITX",30,"            Country","Country",
                "","N",None,None,None,("efld",)),
            (("T",3,0,0),"IUI",1,"Contacts Sequence","Contacts Sequence",
                "","N",self.doSequence,None,None,("between",0,9)),
            (("C",3,0,0),"OUI",1,"S"),
            (("C",3,0,1),"I@mck_code",0,"","",
                "","N",self.doConCod,kon,self.doDelete,("efld",)),
            (("C",3,0,2),"O@mck_desc",0,""),
            (("C",3,0,3),"ITX",35,"Contact Details","Contact Details",
                "","N",None,None,None,("notblank",)),
            (("T",4,0,0),"IUI",1,"Category Sequence","Category Sequence",
                "","N",self.doSequence,None,None,("between",0,9)),
            (("C",4,0,0),"OUI",1,"S"),
            (("C",4,0,1),"I@mlc_type",0,"","",
               "","N",self.doCatTyp,typ,self.doDelete,("in",("A","B","C","D"))),
            (("C",4,0,2),"I@mlc_code",0,"","",
                "","N",self.doCatCod,cat,None,("efld",)),
            (("C",4,0,3),"O@mcc_freq",0,""),
            (("C",4,0,4),"O@mcc_desc",35,""),
            (("C",4,0,5),"ITX",20,"Notes","Notes",
                "","N",self.doCatNote,None,None,("efld",)),
            (("C",4,0,6),"I@mlc_start",0,"","",
                "","N",self.doCatStart,None,None,("efld",)),
            (("C",4,0,7),"I@mlc_end",0,"","",
                "","N",None,None,None,("efld",)),
            (("C",4,0,8),"O@mlc_last",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",5,0,0),"IUI",1,"Links Sequence","Links Sequence",
                "","N",self.doSequence,None,None,("between",0,9)),
            (("C",5,0,0),"OUI",1,"S"),
            (("C",5,0,1),"I@mll_lnkno",0,"","",
                "","N",self.doLnkNo,mlm,self.doDelete,("notzero",)),
            (("C",5,0,2),"O@mlm_title",0,""),
            (("C",5,0,3),"O@mlm_surname",0,""),
            (("C",5,0,4),"O@mlm_names",0,""),
            (("T",6,0,0),("IRB",r5s),0,"","Selection",
                "T","N",self.doTrans2,None,None,None,None)])
        row = [0, 0, 0, 10, 10, 10, 0, 0]
        tnd = (
            (self.doTopEnd,"n"),
            (self.doTopEnd,"y"),
            (self.doTopEnd,"y"),
            (self.doTopEnd,"n"),
            (self.doTopEnd,"n"),
            (self.doTopEnd,"n"),
            None,
            None)
        txt = (
            self.doExit,
            self.doTopExit,
            self.doTopExit,
            self.doTopExit,
            self.doTopExit,
            self.doTopExit,
            self.doTopExit,
            self.doTopExit)
        cnd = (
            None,
            None,
            None,
            (self.doColEnd,"n"),
            (self.doColEnd,"n"),
            (self.doColEnd,"n"),
            None,
            None)
        cxt = (
            None,
            None,
            None,
            self.doColExit,
            self.doColExit,
            self.doColExit,
            None,
            None)
        if self.level < 3:
            but = [("New",None,self.doNew,0,None,None,None,0)]
        else:
            but = [("New",None,self.doNew,0,("T",0,1),None,None,0)]
        but.extend([
            ("Edit",None,self.doEdit,2,None,None,None,0),
            ("Update",None,self.doUpdate,2,None,None,None,0),
            ("Notes",None,self.doNotes,2,None,None,None,0),
            ("Print",None,self.doPrint,2,None,None,None,0),
            ("Reset",None,self.doClear,2,None,None,None,0),
            ("Exit",None,self.doExit,1,None,None,None,1)])
        self.df = TartanDialog(self.mf, tops=False, title=tit, tags=tag,
            eflds=fld, rows=row, tend=tnd, txit=txt, cend=cnd, cxit=cxt,
            butt=but, clicks=self.doClick)

    def doClick(self, *opts):
        if not self.new and not self.edit:
            return
        if opts[2] == (1, 20):
            # ACS Status
            return
        if self.new and opts[2][0] == 1 and opts[2][1] > 16:
            # Status, Status Date and Balance
            return
        if opts[2][0] == 2 and opts[2][1] > 7:
            # Residential Address
            if self.df.t_work[2][0][7] == "Y":
                # Residential same as Postal
                return
        if self.click:
            self.df.loadEntry("T", self.click[0], self.click[1],
                data=self.df.t_work[self.click[0]][0][self.click[1]])
        self.click = opts[2]
        self.df.focusField("T", opts[2][0], opts[2][1] + 1)

    def doTags(self):
        if not self.new and not self.edit:
            return
        if self.df.topq[self.df.pag]:
            self.df.focusField("T", self.df.pag, 1)
        else:
            self.df.focusField("C", self.df.pag, 1)

    def doNew(self, widget=None):
        if self.df.pag == 0:
            self.doSetButtons((2, 5), (0, 1, 3, 4))
            self.new = True
            self.click = []
            self.memcat = []
            self.df.focusField("T", 0, 2)

    def doEdit(self, widget=None):
        if self.df.pag > 5:
            return
        self.click = []
        self.edit = True
        self.newocc = False
        self.doAllTags(state="disabled", exclude=(0, 1, 2, 3, 4))
        self.doSetButtons((2, 5), (0, 1, 3, 4))
        if self.df.pag == 0:
            self.df.focusField("T", self.df.pag, 2)
        else:
            self.df.focusField("T", self.df.pag, 1)

    def doDelete(self, widget=None):
        # Clear the Row
        self.df.clearLine(self.df.pag, self.df.row)
        # Resequence all Rows
        cnt = 0
        for row in self.df.c_work[self.df.pag]:
            if row[1]:
                pos = cnt * self.df.colq[self.df.pag]
                self.df.loadEntry("C", self.df.pag, pos, data=cnt)
                for num, fld in enumerate(row[1:]):
                    self.df.loadEntry("C", self.df.pag, pos+num+1, data=fld)
                cnt += 1
        for row in range(cnt, self.df.rows[self.df.pag]):
            self.df.clearLine(self.df.pag, row)
        # Return
        self.df.focusField("T", self.df.pag, 1)
        return "nf"

    def doMemNo(self, frt, pag, r, c, p, i, w):
        # Allow Copying of Membership Number
        self.df.topEntry[pag][p].set_sensitive(True)
        # Flags
        self.new = False
        self.edit = False
        # memmst table
        self.memmst = self.sql.getRec(tables="memmst", where=[("mlm_cono",
            "=", self.conum), ("mlm_memno", "=", w)], limit=1)
        if not self.memmst:
            return "Invalid Membership Number"
        self.memno = w
        # save memcat table
        self.memcat = self.sql.getRec(tables="memcat", where=[("mlc_cono",
            "=", self.conum), ("mlc_memno", "=", self.memno)],
            order="mlc_type, mlc_code")
        self.title = self.memmst[self.sql.memmst_col.index("mlm_title")]
        self.initial = self.memmst[self.sql.memmst_col.index("mlm_initial")]
        self.surname = self.memmst[self.sql.memmst_col.index("mlm_surname")]
        names = self.memmst[self.sql.memmst_col.index("mlm_names")]
        gender = self.memmst[self.sql.memmst_col.index("mlm_gender")]
        nation = self.memmst[self.sql.memmst_col.index("mlm_nation")]
        self.dob = self.memmst[self.sql.memmst_col.index("mlm_dob")]
        self.idnum = self.memmst[self.sql.memmst_col.index("mlm_idnum")]
        self.occup = self.memmst[self.sql.memmst_col.index("mlm_occup")]
        self.proposer = self.memmst[self.sql.memmst_col.index("mlm_proposer")]
        self.seconder = self.memmst[self.sql.memmst_col.index("mlm_seconder")]
        self.entry = self.memmst[self.sql.memmst_col.index("mlm_entry")]
        self.years = dateDiff(self.entry, self.sysdt, "years")
        payplan = self.memmst[self.sql.memmst_col.index("mlm_payplan")]
        oldno = self.memmst[self.sql.memmst_col.index("mlm_oldno")]
        self.oldsta = self.memmst[self.sql.memmst_col.index("mlm_state")]
        self.sdate = self.memmst[self.sql.memmst_col.index("mlm_sdate")]
        self.newsta = self.oldsta
        # Load All Fields
        self.df.loadEntry("T", 0, 1, data=self.title)
        self.df.loadEntry("T", 0, 2, data=self.initial)
        self.df.loadEntry("T", 0, 3, data=self.surname)
        self.df.loadEntry("T", 1, 0, data=names)
        self.df.loadEntry("T", 1, 1, data=gender)
        self.df.loadEntry("T", 1, 2, data=nation)
        self.df.loadEntry("T", 1, 3, data=countries[nation][1])
        self.df.loadEntry("T", 1, 4, data=self.dob)
        if not self.dob:
            self.age = 0
            self.df.loadEntry("T", 1, 5, data=0)
        else:
            self.age = dateDiff(self.dob, self.sysdt, "years")
            self.df.loadEntry("T", 1, 5, data=self.age)
        self.df.loadEntry("T", 1, 6, data=self.idnum)
        self.df.loadEntry("T", 1, 7, data=self.occup)
        acc = self.sql.getRec(tables="memcto", cols=["mco_desc"],
            where=[("mco_code", "=", self.occup)], limit=1)
        self.df.loadEntry("T", 1, 8, data=acc[0])
        for x, f in enumerate(("mlm_proposer", "mlm_seconder")):
            idx = (x * 2) + 9
            cnt = self.sql.memmst_col.index(f)
            self.df.loadEntry("T", 1, idx, data=self.memmst[cnt])
            acc = self.sql.getRec(tables="memmst", cols=["mlm_title",
                "mlm_initial", "mlm_surname"], where=[("mlm_cono", "=",
                self.conum), ("mlm_memno", "=", self.memmst[cnt])], limit=1)
            if not acc:
                self.df.loadEntry("T", 1, (idx + 1), data="%s" % "Unknown")
            else:
                self.df.loadEntry("T", 1, (idx + 1), data="%s %s %s" % (acc[0],
                    acc[1], acc[2]))
        self.df.loadEntry("T", 1, 13, data=self.entry)
        if not self.entry:
            self.df.loadEntry("T", 1, 14, data=0)
        else:
            self.df.loadEntry("T", 1, 14, data=dateDiff(self.entry, self.sysdt,
                "years"))
        self.df.loadEntry("T", 1, 15, data=payplan)
        self.df.loadEntry("T", 1, 16, data=oldno)
        self.df.loadEntry("T", 1, 17, data=self.oldsta)
        self.df.loadEntry("T", 1, 18, data=self.sdate)
        # Outstanding Balance including pro-forma
        if self.oldsta == "A":
            self.raiseExtraTrans()
        bal = self.sql.getRec(tables="memtrn", cols=["sum(mlt_tramt)"],
            where=[("mlt_cono", "=", self.conum), ("mlt_memno", "=",
            self.memno)], limit=1)
        if not bal[0]:
            self.df.loadEntry("T", 1, 19, data=0)
        else:
            self.df.loadEntry("T", 1, 19, data=bal[0])
        # Access Control System
        if self.acs == "Y":
            data = self.sqa.sqlRec(("Select a.mst_sq, a.tag_expiry from "\
                "tag a, employee b where b.emp_employeeno=? and "\
                "b.mst_sq=a.mst_sq", (str(self.memno),)), limit=1)
            if not data or data[1]:
                self.df.loadEntry("T", 1, 20, data="I")
                if data and data[1]:
                    self.df.loadEntry("T", 1, 21, data=data[1])
            else:
                self.df.loadEntry("T", 1, 20, data="A")
                data = self.sqa.sqlRec(("Select c.tr_date, c.tr_time from "\
                    "transack c where c.tr_mstsq=? order by c.tr_date desc, "\
                    "c.tr_time desc", (data[0],)), limit=1)
                if data:
                    dt = (data[0] * 10000) + (data[1] / 100)
                    self.df.loadEntry("T", 1, 21, data=dt)
                else:
                    self.df.loadEntry("T", 1, 21, data=0)
        # memadd table
        acc = self.sql.getRec(tables="memadd", where=[("mla_cono", "=",
            self.conum), ("mla_memno", "=", self.memno), ("mla_type", "=",
            "P")], limit=1)
        if acc:
            for n, f in enumerate(("mla_add1", "mla_add2", "mla_add3",
                    "mla_city", "mla_code", "mla_region", "mla_country")):
                cnt = self.sql.memadd_col.index(f)
                self.df.loadEntry("T", 2, n, data=acc[cnt])
        acc = self.sql.getRec(tables="memadd", where=[("mla_cono", "=",
            self.conum), ("mla_memno", "=", self.memno), ("mla_type", "=",
            "R")], limit=1)
        if not acc:
            self.df.loadEntry("T", 2, 7, data="Y")
        else:
            self.df.loadEntry("T", 2, 7, data="N")
            for n, f in enumerate(("mla_add1", "mla_add2", "mla_add3",
                    "mla_city", "mla_code", "mla_region", "mla_country")):
                cnt = self.sql.memadd_col.index(f)
                self.df.loadEntry("T", 2, n+8, data=acc[cnt])
        # memkon table
        acc = self.sql.getRec(tables=["memkon", "memctk"], cols=["mlk_code",
            "mck_desc", "mlk_detail"], where=[("mlk_cono", "=", self.conum),
            ("mlk_memno", "=", self.memno), ("mck_code=mlk_code",)])
        for n, c in enumerate(acc):
            self.df.loadEntry("C", 3, (n * 4), data=n)
            self.df.loadEntry("C", 3, ((n * 4) + 1), data=c[0])
            self.df.loadEntry("C", 3, ((n * 4) + 2), data=c[1])
            self.df.loadEntry("C", 3, ((n * 4) + 3), data=c[2])
            self.df.loadEntry("T", 3, 0, data=n+1)
        # memcat table
        for n, c in enumerate(self.memcat):
            ctyp = c[self.sql.memcat_col.index("mlc_type")]
            code = c[self.sql.memcat_col.index("mlc_code")]
            note = c[self.sql.memcat_col.index("mlc_note")]
            start = c[self.sql.memcat_col.index("mlc_start")]
            end = c[self.sql.memcat_col.index("mlc_end")]
            last = c[self.sql.memcat_col.index("mlc_last")]
            acc = self.sql.getRec(tables="memctc", cols=["mcc_freq",
                "mcc_desc"], where=[("mcc_cono", "=", self.conum),
                ("mcc_type", "=", ctyp), ("mcc_code", "=", code)],
                limit=1)
            if not acc:
                showError(self.mf.body, "Invalid Category",
                    "There in an Incorrect Category on this Member's Record, "\
                    "Please Rectify It as a Matter of Urgency")
                acc = ["", "Unknown"]
            self.df.loadEntry("C", 4, (n * 9), data=n)
            self.df.loadEntry("C", 4, ((n * 9) + 1), data=ctyp)
            self.df.loadEntry("C", 4, ((n * 9) + 2), data=code)
            self.df.loadEntry("C", 4, ((n * 9) + 3), data=acc[0])
            self.df.loadEntry("C", 4, ((n * 9) + 4), data=acc[1])
            self.df.loadEntry("C", 4, ((n * 9) + 5), data=note)
            self.df.loadEntry("C", 4, ((n * 9) + 6), data=start)
            self.df.loadEntry("C", 4, ((n * 9) + 7), data=end)
            self.df.loadEntry("C", 4, ((n * 9) + 8), data=last)
            self.df.loadEntry("T", 4, 0, data=n+1)
        # memlnk table
        memlnk = self.sql.getRec(tables="memlnk", cols=["mll_lnkno"],
            where=[("mll_cono", "=", self.conum), ("mll_memno", "=",
            self.memno)], order="mll_lnkno")
        for n, c in enumerate(memlnk):
            acc = self.sql.getRec(tables="memmst", cols=["mlm_title",
                "mlm_surname", "mlm_names"], where=[("mlm_cono", "=",
                self.conum), ("mlm_memno", "=", c[0])], limit=1)
            self.df.loadEntry("C", 5, (n * 5), data=n)
            self.df.loadEntry("C", 5, ((n * 5) + 1), data=c[0])
            self.df.loadEntry("C", 5, ((n * 5) + 2), data=acc[0])
            self.df.loadEntry("C", 5, ((n * 5) + 3), data=acc[1])
            self.df.loadEntry("C", 5, ((n * 5) + 4), data=acc[2])
        # Set Tags and Buttons
        for num, tag in enumerate(self.df.tags):    
            self.df.enableTag(num)
        self.df.setWidget(self.df.B0, "focus")
        if self.level < 3:
            self.doSetButtons((3, 4, 5), (0, 1, 2))
        else:
            self.doSetButtons((1, 3, 4, 5), (0, 2))
        self.mf.updateStatus("")
        return "nf"

    def doTitle(self, frt, pag, r, c, p, i, w):
        if not w[0].isupper():
            self.title = w.capitalize()
        else:
            self.title = w
        self.df.loadEntry(frt, pag, p, data=self.title)

    def doSurname(self, frt, pag, r, c, p, i, w):
        if not w[0].isupper():
            self.surname = w.capitalize()
        else:
            self.surname = w
        self.df.loadEntry(frt, pag, p, data=self.surname)

    def doNames(self, frt, pag, r, c, p, i, w):
        self.names = ""
        self.initial = ""
        for num, name in enumerate(w.split()):
            if num < 3:
                if not self.initial:
                    self.initial = name[0].upper()
                else:
                    self.initial = "%s %s" % (self.initial, name[0].upper())
            if not name[0].isupper():
                name = name.capitalize()
            if not self.names:
                self.names = name
            else:
                self.names = "%s %s" % (self.names, name)
        self.df.loadEntry("T", 0, 2, data=self.initial)
        self.df.loadEntry(frt, pag, p, data=self.names)

    def doGender(self, frt, pag, r, c, p, i, w):
        self.gender = w

    def doNation(self, frt, pag, r, c, p, i, w):
        self.nation = w
        try:
            self.df.loadEntry(frt, pag, p+1, data=countries[self.nation][1])
        except:
            return "Invalid Nationality"

    def doDOB(self, frt, pag, r, c, p, i, w):
        if not w and self.new:
            return "Invalid Date of Birth"
        self.dob = w
        if self.dob:
            self.age = dateDiff(w, self.sysdt, "years")
        else:
            self.age = 0
        self.df.loadEntry(frt, pag, p+1, data=self.age)

    def doID(self, frt, pag, r, c, p, i, w):
        if not w and self.nation == "ZA":
            return "Invalid Identity Number 1"
        if w and self.dob % 1000000 != w / 10000000:
            return "Invalid Identity Number 2"
        self.idnum = w

    def doOccCode(self, frt, pag, r, c, p, i, w):
        if not w:
            # Check for a Zero Record
            acc = self.sql.getRec(tables="memcto", where=[("mco_code", "=",
                w)], limit=1)
            if not acc:
                # Get Next Number
                acc = self.sql.getRec(tables="memcto", cols=["max(mco_code)"],
                    limit=1)
                if not acc or not acc[0]:
                    w = 1
                else:
                    w = acc[0] + 1
                self.df.loadEntry(frt, pag, p, data=w)
        acc = self.sql.getRec(tables="memcto", cols=["mco_desc"],
            where=[("mco_code", "=", w)], limit=1)
        if not acc:
            ok = askQuestion(self.mf.body, "Occupation",
                "This Occupation Code Does Not Exist, Create It?")
            if ok == "no":
                return "Invalid Occupation Code"
            else:
                self.occup = w
        else:
            self.occup = w
            self.newocc = False
            self.df.loadEntry(frt, pag, p+1, data=acc[0])
            return "sk1"

    def doOccDesc(self, frt, pag, r, c, p, i, w):
        self.newocc = w

    def doProposer(self, frt, pag, r, c, p, i, w):
        if not w:
            cf = PwdConfirm(self.mf, conum=self.conum, system="MEM",
                code="PropSec")
            if cf.flag == "no":
                return "Invalid Proposer"
            self.proposer = w
            return
        if self.seconder and w == self.seconder:
            return "Invalid Proposer, same as Seconder"
        acc = self.sql.getRec(tables="memmst", cols=["mlm_title",
            "mlm_initial", "mlm_surname", "mlm_state"], where=[("mlm_cono",
            "=", self.conum), ("mlm_memno", "=", w)], limit=1)
        if not acc:
            return "Invalid Proposer"
        if not acc[3] == "A":
            return "Invalid Proposer, Not Active"
        self.df.loadEntry(frt, pag, p+1, data="%s %s %s" % (acc[0],
            acc[1], acc[2]))
        self.proposer = w

    def doSeconder(self, frt, pag, r, c, p, i, w):
        if not w:
            cf = PwdConfirm(self.mf, conum=self.conum, system="MEM",
                code="PropSec")
            if cf.flag == "no":
                return "Invalid Proposer"
            self.seconder = w
            return
        if self.proposer and w == self.proposer:
            return "Invalid Seconder, same as Proposer"
        acc = self.sql.getRec(tables="memmst", cols=["mlm_title",
            "mlm_initial", "mlm_surname", "mlm_state"], where=[("mlm_cono",
            "=", self.conum), ("mlm_memno", "=", w)], limit=1)
        if not acc:
            return "Invalid Seconder"
        if not acc[3] == "A":
            return "Invalid Seconder, Not Active"
        self.df.loadEntry(frt, pag, p+1, data="%s %s %s" % (acc[0],
            acc[1], acc[2]))
        self.seconder = w

    def doEntry(self, frt, pag, r, c, p, i, w):
        self.entry = w
        self.years = dateDiff(self.entry, self.sysdt, "years")
        self.df.loadEntry(frt, pag, p+1, data=self.years)

    def doPayPlan(self, frt, pag, r, c, p, i, w):
        self.pplan = w
        if self.new:
            self.df.loadEntry(frt, pag, p+1, data=0)
            self.newsta = "A"
            self.df.loadEntry(frt, pag, p+2, data=self.newsta)
            self.sdate = self.entry
            self.df.loadEntry(frt, pag, p+3, data=self.sdate)
            self.df.loadEntry(frt, pag, p+4, data=0)
            if self.acs == "Y":
                self.df.loadEntry(frt, pag, p+5, data="I")
                self.df.loadEntry(frt, pag, p+6, data=0)
                return "sk6"
            else:
                return "sk4"

    def doStatus(self, frt, pag, r, c, p, i, w):
        self.newsta = w
        if self.newsta == self.oldsta:
            return "sk1"
        self.sdate = self.sysdt
        self.df.loadEntry(frt, pag, p+1, data=self.sdate)

    def doDate(self, frt, pag, r, c, p, i, w):
        self.sdate = w
        return "nd"

    def doSameAdd(self, frt, pag, r, c, p, i, w):
        if w == "Y":
            for x in range(p+1, p+8):
                self.df.loadEntry(frt, pag, x, data="")
            return "sk7"

    def doSequence(self, frt, pag, r, c, p, i, w):
        return self.doChkSeq(pag, w)

    def doConCod(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="memctk", cols=["mck_desc"],
            where=[("mck_code", "=", w)], limit=1)
        if not acc:
            return "Invalid Contact Code"
        for line in self.df.c_work[pag]:
            if line[0] == r:
                break
            if line[1] == w:
                return "This Code Already Exists"
        self.koncod = w
        self.df.loadEntry(frt, pag, p+1, data=acc[0])

    def doConDel(self, widget=None):
        self.sql.delRec("memkon", where=[("mlk_cono", "=", self.conum),
            ("mlk_code", "=", self.koncod)])

    def doCatTyp(self, frt, pag, r, c, p, i, w):
        self.cattyp = w
        cnt = 0
        for cod in self.df.c_work[4]:
            if cod[1] == "B":
                cnt += 1
        if cnt > 1:
            return "Already a Type B Category Code"

    def doCatCod(self, frt, pag, r, c, p, i, w):
        self.catcod = w
        acc = self.sql.getRec(tables="memctc", cols=["mcc_freq", "mcc_desc",
            "mcc_age_l", "mcc_and_s", "mcc_or_s"], where=[("mcc_cono", "=",
            self.conum), ("mcc_type", "=", self.cattyp), ("mcc_code", "=",
            self.catcod)], limit=1)
        if not acc:
            return "Invalid Category Code"
        cnt = 0
        for cod in self.df.c_work[4]:
            if cod[1] == self.cattyp and cod[2] == self.catcod:
                cnt += 1
        if cnt > 1:
            return "Category Code Already Exists"
        if acc[2] and self.age > acc[2]:
            if acc[3] and self.years < acc[3]:
                pass
            else:
                return "Invalid Category, Over Age Limit"
        self.df.loadEntry(frt, pag, p+1, data=acc[0])
        self.df.loadEntry(frt, pag, p+2, data=acc[1])

    def doCatNote(self, frt, pag, r, c, p, i, w):
        if self.new:
            self.df.loadEntry(frt, pag, p+1, data=self.sdate)

    def doCatStart(self, frt, pag, r, c, p, i, w):
        if w < self.period[1][0]:
            cf = PwdConfirm(self.mf, conum=self.conum, system="MEM",
                code="FinPer")
            if cf == "no":
                return "Invalid Start Date, Before Start of Financial Period"

    def doLnkNo(self, frt, pag, r, c, p, i, w):
        if w == self.memno:
            return "Invalid, Cannot Link to Self"
        acc = self.sql.getRec(tables="memmst", cols=["mlm_title",
            "mlm_surname", "mlm_names"], where=[("mlm_cono", "=",
            self.conum), ("mlm_memno", "=", w)], limit=1)
        if not acc:
            return "Invalid Membership Number"
        self.df.loadEntry(frt, pag, p+1, data=acc[0])
        self.df.loadEntry(frt, pag, p+2, data=acc[1])
        self.df.loadEntry(frt, pag, p+3, data=acc[2])

    def doChkSeq(self, pag, seq):
        if seq > 0 and not self.df.c_work[pag][seq-1][1]:
            return "Invalid Sequence"
        if not self.df.c_work[pag][seq][1]:
            self.newrow = "Y"
        else:
            self.newrow = "N"

    def doTopEnd(self):
        if self.df.pag == 0:
            self.doSetNextSeq()
            self.df.selPage(self.df.tags[self.df.pag][0])
            self.df.focusField("T", 1, 1)
        elif self.df.pag == 1:
            self.df.selPage(self.df.tags[self.df.pag][0])
            self.df.focusField("T", 2, 1)
            for num in range(1, 5):
                self.df.enableTag(num)
        elif self.df.pag == 2:
            self.df.selPage(self.df.tags[self.df.pag][0])
            self.df.focusField("T", 3, 1)
        elif self.df.pag in (3, 4, 5):
            self.doAllTags(state="disabled")
            max = self.df.colq[self.df.pag]
            seq = self.df.t_work[self.df.pag][0][0]
            self.df.loadEntry("C", self.df.pag, seq*max, data=seq)
            self.df.focusField("C", self.df.pag, ((seq*max)+1))

    def doTopExit(self):
        if self.df.pag == 0:
            self.doExit()
        else:
            if self.df.pag == 1:
                pag = 5
            else:
                pag = self.df.pag - 1
            col = self.df.last[pag][0]
            if not col:
                col = 1
            else:
                for col in range(col, 1, -1):
                    if col in self.df.skip[pag]:
                        continue
                    if self.df.topf[pag][col-1][1][0] != "O":
                        break
            self.df.selPage(self.df.tags[pag-1][0])
            self.df.focusField("T", pag, col)

    def doColEnd(self):
        self.doAllTags(state="normal", exclude=(5,6))
        self.doSetNextSeq()
        self.df.last[self.df.pag] = [0, 0]
        self.df.focusField("T", self.df.pag, 1)

    def doColExit(self):
        self.doAllTags(state="normal")
        if self.df.idx == 0:
            if self.newrow == "Y":
                self.df.clearLine(self.df.pag)
            self.df.last[self.df.pag] = [0, 0]
            self.df.focusField("T", self.df.pag, 1)

    def doAllTags(self, state="normal", exclude=()):
        for num, tag in enumerate(self.df.tags):
            if num in exclude:
                continue
            if state == "normal":
                self.df.enableTag(num)
            else:
                self.df.disableTag(num)

    def doSetNextSeq(self):
        for pag in range(3, 6):
            d = self.df.c_work[pag]
            for seq in range(0, len(d)):
                if not d[seq][1]:
                    break
            self.df.loadEntry("T", pag, 0, data=seq)

    def doUpdate(self, widget=None):
        dte=long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
        if self.new or self.edit:
            # Rollback all uncomitted transactions
            self.mf.dbm.rollbackDbase()
            if self.new:
                # Allocate New Member Number
                acc = self.sql.getRec(tables="memmst", cols=["max(mlm_memno)"],
                    where=[("mlm_cono", "=", self.conum)], limit=1)
                if not acc[0]:
                    acc = [0]
                self.memno = int(acc[0]) + 1
                self.df.loadEntry("T", 0, 0, data=self.memno)
            # Check validity
            for pag in (("T", 0, None), ("T", 1, None)):
                frt, pag, pos, mes = self.df.doCheckFields(pag)
                if mes:
                    if pag > 0 and pag != self.df.pag:
                        if frt == "T":
                            self.df.last[pag][0] = pos+1
                        else:
                            self.df.last[pag][1] = pos+1
                        self.df.selPage(self.df.tags[pag - 1][0])
                    self.df.focusField(frt, pag, (pos+1), err=mes)
                    return
            if self.idnum and self.dob % 1000000 != self.idnum / 10000000:
                self.df.selPage("Personal")
                self.df.focusField("T", 1, 7, err="Invalid Identity Number")
                return
            cat = False
            seq = 0
            for fld in self.df.c_work[4]:
                if fld[1]:
                    seq += 1
                    if fld[1] in ("A", "B"):
                        cat = True
            if not cat and self.newsta == "A":
                showError(self.mf.body, "Missing Category",
                    "There Must be a Category Type B")
                self.df.selPage(self.df.tags[3][0])
                self.df.loadEntry("T", 4, 0, data=seq)
                self.df.focusField("T", 4, 1)
                return
            # memmst table
            memmst = [self.conum]
            for fld in self.df.t_work[0][0]:
                memmst.append(fld)
            for num, fld in enumerate(self.df.t_work[1][0]):
                if num in (3, 5, 8, 10, 12, 14, 19, 20, 21):
                    continue
                memmst.append(fld)
            if self.new:
                self.sql.insRec("memmst", data=memmst)
            else:
                self.sql.updRec("memmst", data=memmst, where=[("mlm_cono",
                    "=", self.conum), ("mlm_memno", "=", self.memno)])
                if memmst != self.memmst:
                    for num, dat in enumerate(self.memmst):
                        if dat != memmst[num]:
                            self.sql.insRec("chglog", data=["memmst", "U",
                                "%03i%06i" % (self.conum, self.memno),
                                self.sql.memmst_col[num], dte, self.capnm,
                                str(dat), str(memmst[num]), 0])
            # memcto table
            if self.newocc:
                self.sql.insRec("memcto", data=[self.occup, self.newocc])
            # memsta table
            if self.new or self.oldsta != self.newsta:
                memsta = [self.conum, self.memno, self.newsta, self.sdate,
                    self.sysdt, 0]
                self.sql.insRec("memsta", data=memsta)
                if self.acs == "Y":
                    self.doAcsRecord(memmst, memsta)
            # memadd table
            self.sql.delRec("memadd", where=[("mla_cono", "=", self.conum),
                ("mla_memno", "=", self.memno)])
            for typ in ("P", "R"):
                data = [self.conum, self.memno, typ]
                if typ == "P":
                    idx = 0
                else:
                    idx = 8
                for x in range(idx, idx + 7):
                    data.append(self.df.t_work[2][0][x])
                self.sql.insRec("memadd", data=data)
                if typ == "P" and self.df.t_work[2][0][7] == "Y":
                    break
            # memkon table
            self.sql.delRec("memkon", where=[("mlk_cono", "=", self.conum),
                ("mlk_memno", "=", self.memno)])
            for dat in self.df.c_work[3]:
                if not dat[1]:
                    break
                data = [self.conum, self.memno, dat[1], dat[3]]
                self.sql.insRec("memkon", data=data)
            # memcat table
            self.sql.delRec("memcat", where=[("mlc_cono", "=", self.conum),
                ("mlc_memno", "=", self.memno)])
            for dat in self.df.c_work[4]:
                if not dat[1]:
                    break
                new = [self.conum, self.memno, dat[1], dat[2], dat[5], dat[6],
                    dat[7], 0]
                for old in self.memcat:
                    if old[:4] == new[:4]:
                        new[7] = old[7]
                        break
                self.sql.insRec("memcat", data=new)
            memcat = self.sql.getRec(tables="memcat", where=[("mlc_cono",
                "=", self.conum), ("mlc_memno", "=", self.memno)],
                order="mlc_type, mlc_code")
            if memcat != self.memcat:
                newcat = memcat[:]
                oldcat = self.memcat[:]
                for new in newcat:
                    ncono, nmemno, ntype, ncode, x, nstart, nend, x = new
                    for old in oldcat:
                        ocono, omemno, otype, ocode, x, ostart, oend, x = old
                        if old[:4] == new[:4]:
                            if ostart != nstart:
                                self.sql.insRec("chglog", data=["memcat",
                                    "U", "%03i%06i%1s%02i" % (ocono, omemno,
                                    otype, ocode), "mlc_start", dte,
                                    self.capnm, str(ostart), str(nstart), 0])
                            if oend != nend:
                                self.sql.insRec("chglog", data=["memcat",
                                    "U", "%03i%06i%1s%02i" % (ocono, omemno,
                                    otype, ocode), "mlc_end", dte,
                                    self.capnm, str(oend), str(nend), 0])
                            oldcat.remove(old)
                            memcat.remove(new)
                            break
                for cono, memno, ctype, code, note, start, end, x in oldcat:
                    self.sql.insRec("chglog", data=["memcat", "D",
                        "%03i%06i%1s%02i" % (cono, memno, ctype, code),
                        "", dte, self.capnm, str(start), str(end), 0])
                for cono, memno, ctype, code, note, start, end, x in memcat:
                    self.sql.insRec("chglog", data=["memcat", "N",
                        "%03i%06i%1s%02i" % (cono, memno, ctype, code),
                        "", dte, self.capnm, str(start), str(end), 0])
            # memlnk table
            self.sql.delRec("memlnk", where=[("mll_cono", "=", self.conum),
                ("mll_memno", "=", self.memno)])
            if self.newsta == "A":
                for dat in self.df.c_work[5]:
                    if not dat[1]:
                        break
                    data = [self.conum, self.memno, dat[1]]
                    self.sql.insRec("memlnk", data=data)
            self.mf.dbm.commitDbase()
            if self.acs == "Y":
                self.dba.commitDbase()
            # Flags
            self.new = False
            self.edit = False
        self.df.selPage(self.df.tags[0][0])
        self.df.focusField("T", 0, 1)
        self.df.loadEntry("T", 0, 0, data=self.memno)
        self.doMemNo("T", 0, 0, 1, 0, 0, self.memno)

    def doAcsRecord(self, memmst, memsta):
        if self.new:
            cnt = self.sqa.sqlRec("Select max(mst_sq) from master", limit=1)[0]
            chk = True
            while chk:
                cnt += 1
                chk = self.sqa.sqlRec(("Select * from master where mst_id=?",
                    (str(cnt),)), limit=1)
            nam = memmst[5].split()
            if len(nam) == 1:
                nam.append("")
            self.sqa.sqlRec(("Insert into master values (?, ?, ?, ?, ?, ?, ?, "\
                "?, ?, ?, ?, ?, ?, ?, ?)", (cnt, memmst[2], nam[0], nam[1],
                memmst[4], "", str(cnt), memmst[6], 0, 0, 1, "01000000", None,
                None, 1)))
            try:
                self.sqa.sqlRec(("Delete from employee where mst_sq=? "\
                    "or emp_employeeno=?", (cnt, str(memmst[1]))))
            except:
                pass
            self.sqa.sqlRec(("Insert into employee values(?, ?, ?, ?, ?, ?)",
                (cnt, str(memmst[1]), "", "", 1, "01000000")))
        else:
            tag = self.sqa.sqlRec(("Select a.tag_code from tag a, employee b "\
                "where b.emp_employeeno=? and b.mst_sq=a.mst_sq",
                (str(memmst[1]),)), limit=1)
            if tag:
                if memsta[2] == "A":
                    self.sqa.sqlRec(("Update tag set tag_suspend=?, "\
                        "tag_blacklist=?, tag_expiry=?, tag_modified=? "\
                        "where tag_code=?", (0, 0, 0, 1, tag[0])))
                else:
                    self.sqa.sqlRec(("Update tag set tag_suspend=?, "\
                        "tag_blacklist=?, tag_expiry=?, tag_modified=? "\
                        "where tag_code=?", (1, 0, memsta[3], 1, tag[0])))
                self.sqa.sqlRec(("Update tag_t_a_g set ttag_modified=? "\
                    "where tag_code=?", (1, tag[0])))

    def doTrans1(self, widget=None):
        self.df.focusField("T", 6, 1)

    def doTrans2(self, frt, pag, r, c, p, i, w):
        self.select = w
        if self.select == "A":
            self.doAccess()
            return
        elif self.select == "S":
            self.doSportsman()
            return
        elif self.select == "U":
            self.doStatusHistory()
            return
        whr = [("mlt_cono", "=", self.conum), ("mlt_memno", "=", self.memno)]
        ord = "mlt_trdt"
        col, all = getTrn(self.mf.dbm, "mem", whr=whr, ord=ord)
        if all:
            data = []
            for dat in all:
                data.append([
                    dat[col.index("mlt_trdt")],
                    dat[col.index("mlt_curdt")],
                    dat[col.index("mlt_batch")],
                    dat[col.index("mlt_type")],
                    dat[col.index("mlt_refno")],
                    dat[col.index("mlt_tramt")],
                    dat[col.index("paid")],
                    dat[col.index("balance")],
                    dat[col.index("mlt_ctyp")],
                    dat[col.index("mlt_ccod")],
                    dat[col.index("mlt_desc")]])
            tit = "Transactions for Account: %s - %s %s %s" % (self.memno,
                self.title, self.initial, self.surname)
            col = (
                ("mlt_trdt", "   Date", 10, "D1", "N"),
                ("mlt_curdt", "Curr-Dt", 7, "D2", "N"),
                ("mlt_batch", "Batch", 7, "Na", "N"),
                ("mlt_type", "Typ", 3, ("XX", mltrtp), "N"),
                ("mlt_refno", "Reference", 9, "Na", "Y"),
                ("mlt_tramt", "    Amount", 11.2, "SD", "N"),
                ("alloc", "      Paid", 11.2, "SD", "N"),
                ("balan", "   Balance", 11.2, "SD", "N"),
                ("mlt_ctyp", "T", 1, "UA", "N"),
                ("mlt_ccod", "Cd", 2, "UI", "N"),
                ("mlt_desc", "Details", 30, "NA", "N"))
            state = self.df.disableButtons()
            while True:
                rec = SelectChoice(self.df.nb.Page6, tit, col, data, sort=False)
                if rec.selection:
                    self.df.mstAlw.hide()
                    sp = SplashScreen(self.df.nb.Page6,
                        "Displaying Document.... Please Wait....")
                    whr = [
                        ("mlt_cono", "=", self.conum),
                        ("mlt_memno", "=", self.memno),
                        ("mlt_type", "=", rec.selection[4]),
                        ("mlt_refno", "=", rec.selection[5])]
                    TabPrt(self.mf, tabs="memtrn", wher=whr, pdia=False)
                    sp.closeSplash()
                    self.df.mstAlw.show_all()
                else:
                    break
            self.df.enableButtons(state=state)
        self.doTrans1()

    def doAccess(self):
        try:
            data = self.sqa.sqlRec(("Select a.tr_date, a.tr_time, "\
                "a.tr_direction, a.tr_reasoncode from transack a, employee b "\
                "where b.emp_employeeno=? and a.tr_mstsq=b.mst_sq "\
                "order by a.tr_date, a.tr_time", (str(self.memno),)))
            if data:
                tit = "Access Control for Account: %s - %s %s %s" % \
                    (self.memno, self.title, self.initial, self.surname)
                col = (
                    ("tr_date", "Date", 10, "D1", "N"),
                    ("tr_time", "Time", 5, "tM", "N"),
                    ("tr_direction", "Direction", 9, "UI", "N"),
                    ("tr_reasoncode", "Reason", 9, "UI", "N"))
                state = self.df.disableButtons()
                rec = SelectChoice(self.df.nb.Page6, tit, col, data, sort=False)
                self.df.enableButtons(state=state)
        except:
            pass
        self.doTrans1()

    def doSportsman(self):
        data = self.sql.getRec(tables="memtrs", cols=["mst_trdt", "mst_type",
            "mst_refno", "mst_tramt", "mst_ctyp", "mst_ccod", "mst_desc"],
            where=[("mst_cono", "=", self.conum), ("mst_memno", "=",
            self.memno)])
        if data:
            for n, a in enumerate(data):
                if not a[6]:
                    acc = self.sql.getRec(tables="memctc", cols=["mcc_desc"],
                        where=[("mcc_cono", "=", self.conum), ("mcc_type",
                        "=", a[4]), ("mcc_code", "=", a[5])], limit=1)
                    if acc:
                        data[n][6] = acc[0]
            tit = "Sportsman Transactions for Account: %s - %s %s %s" % \
                (self.memno, self.title, self.initial, self.surname)
            col = (
                ("mst_trdt", "   Date", 10, "D1", "N"),
                ("mst_type", "Typ", 3, ("XX", mltrtp), "N"),
                ("mst_refno", "Reference", 9, "Na", "N"),
                ("mst_tramt", "    Amount", 11.2, "SD", "N"),
                ("mst_ctyp", "T", 1, "UA", "N"),
                ("mst_ccod", "Cd", 2, "UI", "N"),
                ("mst_desc", "Details", 30, "NA", "Y"))
            state = self.df.disableButtons()
            rec = SelectChoice(self.df.nb.Page6, tit, col, data, sort=True)
            self.df.enableButtons(state=state)
        self.doTrans1()

    def doStatusHistory(self):
        all = self.sql.getRec(tables="memsta", cols=["mls_capdt", "mls_date",
            "mls_status"], where=[("mls_cono", "=", self.conum), ("mls_memno",
            "=", self.memno)], order="mls_seq")
        if not all:
            return
        data = []
        for dat in all:
            if not dat[0]:
                dat[0] = dat[1]
            if dat[2] == "A":
                stat = "Active"
            elif dat[2] == "D":
                stat = "Deceased"
            elif dat[2] == "X":
                stat = "Defaulted"
            elif dat[2] == "I":
                stat = "Inactive"
            elif dat[2] == "R":
                stat = "Resigned"
            elif dat[2] == "S":
                stat = "Suspended"
            data.append([dat[0], dat[1], stat])
        tit = "Status History for Account: %s - %s %s %s" % \
            (self.memno, self.title, self.initial, self.surname)
        col = (
            ("mls_capdt", "Cap-Date", 10, "d1", "N"),
            ("mls_date", "Eff-Date", 10, "d1", "N"),
            ("mls_status", "Status", 9, "NA", "N"))
        state = self.df.disableButtons()
        rec = SelectChoice(self.df.nb.Page6, tit, col, data, sort=False)
        self.df.enableButtons(state=state)
        self.doTrans1()

    def doNotes(self, widget=None):
        state = self.df.disableButtons()
        self.df.mstAlw.hide()
        nf = NotesCreate(self.mf, self.conum, self.conam, self.capnm, "MEM",
            self.memno)
        self.df.mstAlw.show()
        self.df.enableButtons(state=state)

    def doPhoto(self, widget=None):
        if self.image:
            return
        for ext in ("jpg", "JPG"):
            jpg = os.path.join(self.photo, "%s.%s" % (self.memno, ext))
            if os.path.isfile(jpg):
                break
            jpg = None
        if jpg:
            self.image = ShowImage(self.df.nb.Page7, jpg)

    def doPrint(self, widget=None):
        titl = "Select the required print options"
        data = (
            ("1", "Information"),
            ("2", "Transactions"),
            ("3", "Statement"))
        ss = SChoice(self.mf, titl=titl, data=data)
        self.mf.updateStatus("")
        if ss.selection:
            state = self.df.disableButtons()
            self.df.mstAlw.hide()
            if ss.selection[0] == "1":
                table = "memmst"
                whr = [("mlm_cono", "=", self.conum),
                    ("mlm_memno", "=", self.memno)]
                rp = TabPrt(self.mf, self.conum, self.conam, name=__name__,
                    tabs=table, wher=whr, keys=[self.memno])
                view = rp.view
                mail = rp.mail
                exit = rp.exit
            elif ss.selection[0] == "2":
                name = "%s %s %s" % (self.title, self.initial, self.surname)
                heads = ["Member's Transactions",
                    "Member: %s  Name: %s" % (self.memno, name)]
                whr = [("mlt_cono", "=", self.conum),
                    ("mlt_memno", "=", self.memno)]
                col, all = getTrn(self.mf.dbm, "mem", whr=whr)
                cols = []
                data = []
                dic = self.sql.memtrn_dic
                for num, rec in enumerate(all):
                    dat = []
                    for nam in [
                        "mlt_refno",
                        "mlt_trdt",
                        "mlt_type",
                        "mlt_tramt",
                        "paid",
                        "balance",
                        "mlt_desc"]:
                        if not num:
                            if nam == "paid":
                                cols.append(["paid", "SD", 11.2, "Paid"])
                            elif nam == "balance":
                                cols.append(["balance", "SD", 11.2, "Balance"])
                            else:
                                cols.append([nam, dic[nam][2], dic[nam][3],
                                    dic[nam][5]])
                        dat.append(rec[col.index(nam)])
                    data.append(dat)
                gtots=["mlt_tramt", "paid", "balance"]
                prtdia = ("Y","V")
                rp = RepPrt(self.mf, conum=self.conum, conam=self.conam,
                    name=__name__, ttype="D", tables=data, heads=heads,
                    cols=cols, trtp=["mlt_type", mltrtp], gtots=gtots,
                    prtdia=prtdia, view=None, mail=None)
            else:
                def prtEnd():
                    pf.closeProcess()
                    self.mf.closeLoop()
                data = self.sql.getRec(tables="memmst", where=[("mlm_cono",
                    "=", self.conum), ("mlm_memno", "=", self.memno)], limit=1)
                tit = ("Print Options",)
                fld = []
                pf = TartanDialog(self.mf, tops=True, title=tit, eflds=fld,
                    tend=((prtEnd,"N"),), view=("N","V"), mail=("N","Y"))
                self.mf.startLoop()
                if pf.repprt[0][1] == "V":
                    prtnam = "view"
                else:
                    prtnam = pf.repprt[0][2]
                mail = pf.repprt[1][1]
                noprt = pf.repprt[1][3]
                if self.ynd:
                    date = self.nxtdt
                else:
                    date = self.tme
                callModule(self.mf, None, "ml3040", coy=(self.conum,
                    self.conam), args=(data, date, prtnam, mail, noprt))
            self.df.mstAlw.show()
            self.mf.gtkDispatch()
            self.df.enableButtons(state=state)
            if self.new or self.edit:
                self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def raiseExtraTrans(self):
        self.refno = 0
        acc = [self.memno]
        lnk = self.sql.getRec(tables="memlnk", cols=["mll_lnkno"],
            where=[("mll_cono", "=", self.conum), ("mll_memno", "=",
            self.memno)])
        for l in lnk:
            chk = self.sql.getRec(tables="memmst", cols=["mlm_state"],
                where=[("mlm_cono", "=", self.conum), ("mlm_memno", "=",
                l[0])], limit=1)
            if chk and chk[0] == "A":
                acc.append(l[0])
        for memno in acc:
            if self.ynd:
                data = doChkCatChg(self.mf, self.conum, memno,
                    self.period[2][0], self.nxtdt)
                if data:
                    if not data[3]:
                        self.doRaiseCharge(memno, "B", data[0], data[1],
                            data[2], data[3], data[4], data[5], data[6],
                            skip=True)
                    self.sql.delRec("memcat", where=[("mlc_cono", "=",
                        self.conum), ("mlc_memno", "=", memno),
                        ("mlc_type", "=", "B"), ("mlc_code", "=", data[0])])
                    self.sql.insRec("memcat", data=[self.conum, memno,
                        "B", data[7], "", self.nxtdt, 0, 0])
            cols = [ "mlc_type", "mlc_code", "mcc_desc", "mcc_freq",
                "mlc_start", "mlc_end", "mlc_last", "mcc_glac"]
            wher = [
                ("mlc_cono", "=", self.conum),
                ("mlc_memno", "=", memno),
                ("mlc_start", ">", 0),
                ("mlc_start", "<=", self.nxtdt),
                ("mcc_cono=mlc_cono",),
                ("mcc_type=mlc_type",),
                ("mcc_code=mlc_code",),
                ("mcc_freq", "<>", "N")]
            cats = self.sql.getRec(tables=["memcat", "memctc"], cols=cols,
                where=wher, order="mlc_type, mlc_code")
            for ctyp, code, desc, freq, start, end, last, glac in cats:
                if start > self.nxtdt:
                    # Not yet Started
                    continue
                if last and end and end < self.nxtdt:
                    # Ended
                    continue
                if last and freq == "O":
                    # Once Off
                    continue
                if last and last > self.period[2][0]:
                    # Already Raised for Next Period in Advance
                    continue
                if not self.ynd and last and freq == "A" and \
                    last >= self.period[1][0] and last <= self.period[2][0]:
                    # Already Raised in Financial Period
                    continue
                self.doRaiseCharge(memno, ctyp, code, start, end, last, freq,
                    desc, glac)

    def doRaiseCharge(self, memno, ctyp, code, start, end, last, freq, desc, glac, skip=False):
        if freq == "O":
            dte = True
            nxt = False
        else:
            dte = False
            if self.ynd or freq == "M":
                nxt = True
            else:
                nxt = False
            if not last:
                if dateDiff(start, self.tme, "days") > self.ldays:
                    dte = True
                else:
                    nxt = True
        if dte:
            trdt = start
            amt = self.doGetCharge(ctyp, code, trdt)
            if amt:
                self.doUpdateTables(memno, ctyp, code, desc, glac, trdt, amt)
        if not skip and nxt:
            if end and self.nxtdt > end:
                return
            trdt = self.nxtdt
            amt = self.doGetCharge(ctyp, code, trdt)
            if amt:
                self.doUpdateTables(memno, ctyp, code, desc, glac, trdt, amt)

    def doGetCharge(self, ctyp, code, date):
        prc = self.sql.getRec(tables="memctp", where=[("mcp_cono", "=",
            self.conum), ("mcp_type", "=", ctyp), ("mcp_code", "=", code),
            ("mcp_date", "<=", date)], order="mcp_date desc", limit=1)
        if not prc:
            # No Price
            return
        if prc[5] == "N" or (self.ynd and date == self.nxtdt):
            # Not Pro Rata or End of Financial Year
            amt = CCD(prc[6], "UD", 12.2).work
        else:
            # Extract Pro Rata Rate
            mths = 17 - dateDiff(date, self.period[2][0], "months")
            if mths < 1:
                mths = 12
            amt = CCD(prc[mths], "UD", 12.2).work
        if not amt:
            # No Charge
            return
        else:
            return amt

    def doUpdateTables(self, memno, ctyp, code, desc, glac, trdt, amt):
        batch = "PROFORM"
        self.refno += 1
        refno = "PF%07i" % self.refno
        curdt = trdt / 100
        # VAT Rate and Amount
        acc = self.sql.getRec(tables=["ctlvmf"], cols=["vtm_new"],
            where=[("vtm_cono", "=", self.conum), ("vtm_code", "=",
            self.taxdf)], limit=1)
        if not acc:
            rate = 0
        else:
            rate = float(acc[0])
        vat = CCD(round(((amt * rate) / 114), 2), "UD", 12.2).work
        # Members Ledger Transaction (memtrn)
        self.sql.insRec("memtrn", data=[self.conum, memno, 1, refno,
            batch, trdt, amt, vat, curdt, ctyp, code, desc, self.taxdf,
            "", self.sysdt, 0], unique="mlt_refno")
        # Members Category Record (memcat)
        self.sql.updRec("memcat", cols=["mlc_last"], data=[trdt],
            where=[("mlc_cono", "=", self.conum), ("mlc_memno", "=",
            memno), ("mlc_type", "=", ctyp), ("mlc_code", "=", code)])

    def doSortLists(self, lst1, lst2):
        return cmp(lst1[idx], lst2[idx])

    def doClear(self, widget=None):
        if self.new or self.edit:
            ok = askQuestion(self.mf.body, "Editing",
                "Are You Sure? You are busy Editing this Record!", default="no")
            if ok == "no":
                return
        # Rollback Everything Not Committed
        self.mf.dbm.rollbackDbase()
        # Flags
        self.new = False
        self.edit = False
        if self.image:
            # Clear Photo
            self.image.image.destroy()
            self.image = None
        self.df.selPage(self.df.tags[0][0])
        self.doSetButtons((), (1, 2, 3, 4, 5))
        self.df.focusField("T", 0, 1)

    def doSetButtons(self, normal, disabled):
        for but in normal:
            exec "self.df.setWidget(self.df.B%s, state='normal')" % but
        for but in disabled:
            exec "self.df.setWidget(self.df.B%s, state='disabled')" % but

    def doExit(self, widget=None):
        if self.new or self.edit:
            ok = askQuestion(self.mf.body, "Editing",
                "Are You Sure? You are busy Editing this Record!", default="no")
            if ok == "no":
                return
        # Rollback Everything Not Committed
        try:
            self.mf.dbm.rollbackDbase()
        except:
            pass
        self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    from TartanClasses import MainFrame
    mf = MainFrame(fixed=False)
    mf.dbm.openDbase()
    rpt = ml1010(mf, 1, "Company One")
