#! /usr/bin/env python

"""
SYNOPSIS
    Members Ledger Statements.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, time
from TartanClasses import ASD, Balances, CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doPrinter, doReportlab, flipImage, getModName
from tartanFunctions import getSingleRecords, showError, textFormat
from tartanWork import mltrtp

class ml3040:
    def __init__(self, mf, coy, nam, args=None):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.args = args
        no = self.setVariables()
        if not no:
            self.buildHead()
            if self.args:
                self.doEnd()
            else:
                self.mainProcess()
                self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlmes", "ctlmst", "memmst", "memtrn",
            "memadd", "memkon", "memlnk"], prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.title = ("%03i %s" % (self.conum, self.conam),
            "Statements (%s)" % __name__)
        self.t = None
        self.trtypes = []
        for q in range(0, len(mltrtp)):
            self.trtypes.append(mltrtp[q][0])
        self.pdfnam = None
        self.maxlines = 21
        self.printed = []

    def buildHead(self):
        self.head = []
        ctl = self.sql.getRec(tables="ctlmst", where=[("ctm_cono", "=",
            self.conum)], limit=1)
        for col in ("ctm_name", "ctm_add1", "ctm_add2", "ctm_add3",
            "ctm_pcode", "ctm_regno", "ctm_taxno", "ctm_tel", "ctm_fax",
            "ctm_b_name", "ctm_b_ibt", "ctm_b_acno", "ctm_logo", "ctm_flip"):
            exec "self.%s = ctl[self.sql.ctlmst_col.index('%s')]" % (col, col)
        if not self.ctm_logo or not os.path.exists(self.ctm_logo):
            self.ctm_logo = None
        elif self.ctm_flip == "N":
            self.ctm_logo = flipImage(self.ctm_logo)
        if self.ctm_logo:
            self.head.append(" ")
            self.head.append(" ")
            self.head.append(" ")
            self.head.append(" ")
            self.head.append(" ")
        else:
            mess = "%1s%-30s" % \
                ("", self.ctm_name)
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % \
                ("", self.ctm_add1, "", ("RegNo: " + self.ctm_regno))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % \
                ("", self.ctm_add2, "", ("TaxNo: " + self.ctm_taxno))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % \
                ("", self.ctm_add3, "", ("TelNo: " + self.ctm_tel))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % \
                ("", self.ctm_pcode, "", ("FaxNo: " + self.ctm_fax))
            self.head.append(mess)

    def mainProcess(self):
        self.mlm = {
            "stype": "R",
            "tables": ("memmst",),
            "cols": (
                ("mlm_memno", "", 0, "Mem-No"),
                ("mlm_oldno", "", 0, "Old-No"),
                ("mlm_idnum", "", 0, "Identity-Numb"),
                ("mlm_gender", "", 0, "G"),
                ("mlm_state", "", 0, "S"),
                ("mlm_surname", "", 0, "Surname", "Y"),
                ("mlm_names", "", 0, "Names", "F")),
            "where": [],
            "order": "mlm_surname, mlm_names",
            "sort": False}
        mss = {
            "stype": "R",
            "tables": ("ctlmes",),
            "cols": (
                ("mss_message", "", 0, "Num"),
                ("mss_detail", "NA", 50, "Details")),
            "where": [("mss_system", "=", "MEM")],
            "order": "mss_message"}
        r1s = (
            ("All", "Z"),
            ("Active", "A"),
            ("Deceased", "D"),
            ("Inactive", "I"),
            ("Resigned", "R"),
            ("Suspended", "S"),
            ("Defaulted", "X"))
        r2s = (
            ("Yes","Y"),
            ("Range","R"),
            ("Singles", "S"),
            ("Print Only","P"),
            ("Email Only","E"))
        r3s = (
            ("Yes","Y"),
            ("No","N"),
            ("Current","C"))
        r4s = (
            ("Yes","Y"),
            ("No","N"))
        r5s = (
            ("Number", "N"),
            ("Surname", "M"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Status","",
                "A","Y",self.doStatus,None,None,None),
            (("T",0,1,0),("IRB",r2s),0,"Whole File","",
                "Y","Y",self.doWhole,None,None,None),
            (("T",0,2,0),"IUI",6,"Member Start","Starting Number",
                1,"Y",self.doMember,self.mlm,None,None),
            (("T",0,2,0),"IUI",6,"End","Ending Number",
                999999,"Y",self.doMember,self.mlm,None,None),
            (("T",0,3,0),("IRB",r3s),0,"Zero Balances",
                "Include Zero Balances","C","Y",self.doZeros,None,None,None),
            (("T",0,4,0),("IRB",r4s),0,"Paid Transactions",
                "Include Paid Transactions","Y","Y",self.doPaid,None,None,None),
            (("T",0,5,0),("IRB",r4s),0,"Minus Balances",
                "Include Minus Balances","N","Y",self.doMinus,None,None,None),
            (("T",0,6,0),"IUI",3,"Message Number","",
                "","Y",self.doMessno,mss,None,("efld",)),
            (("T",0,7,0),"ID1",10,"Statement Date","",
                self.sysdtw,"Y",self.doDat,None,None,("efld",)),
            (("T",0,8,0),("IRB",r5s),0,"Sort Order","",
                "N","Y",self.doSort,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.title, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("N","y"))

    def doStatus(self, frt, pag, r, c, p, i, w):
        self.status = w
        self.mlm["where"] = [("mlm_cono", "=", self.conum)]
        if self.status != "Z":    # Not All
            self.mlm["where"].append(("mlm_state", "=", self.status))

    def doWhole(self, frt, pag, r, c, p, i, w):
        self.whole = w
        if self.whole != "R":
            self.df.loadEntry(frt, pag, p+1, data="")
            self.df.loadEntry(frt, pag, p+2, data="")
            return "sk2"

    def doMember(self, frt, pag, r, c, p, i, w):
        if p == 2:
            self.start = w
        elif w < self.start:
            return "Invalid Member, Before From"
        else:
            self.to = w

    def doZeros(self, frt, pag, r, c, p, i, w):
        self.zeros = w
        if self.zeros == "Y":
            self.paid = "Y"
            self.df.loadEntry(frt, pag, p+1, data=self.paid)
            return "sk1"

    def doPaid(self, frt, pag, r, c, p, i, w):
        self.paid = w

    def doMinus(self, frt, pag, r, c, p, i, w):
        self.minus = w

    def doMessno(self, frt, pag, r, c, p, i, w):
        if w:
            self.mes = self.sql.getRec(tables="ctlmes", cols=["mss_detail"],
                where=[("mss_system", "=", "MEM"), ("mss_message", "=", w)],
                limit=1)
            if not self.mes:
                return "Invalid Message Number"
        else:
            self.mes = None

    def doDat(self, frt, pag, r, c, p, i, w):
        self.datew = w
        self.dated = self.df.t_disp[pag][0][p]
        self.curdt = w / 100

    def doSort(self, frt, pag, r, c, p, i, w):
        self.sort = w

    def doEnd(self):
        if not self.args:
            self.df.closeProcess()
            self.prtnam = self.df.repprt[0][2]
            self.email = self.df.repprt[1][1]
            self.noprt = self.df.repprt[1][3]
        else:
            self.whole = "S"
            self.zeros = "Y"
            self.paid = "Y"
            self.minus = "Y"
            self.mes = None
            self.mes = self.sql.getRec(tables="ctlmes", cols=["mss_detail"],
                where=[("mss_system", "=", "MEM"), ("mss_message", "=", 1)],
                limit=1)
            self.curdt = self.args[1] / 100
            self.dated = CCD(self.args[1], "D1", 10).disp
            self.prtnam = self.args[2]
            self.email = self.args[3]
            self.noprt = self.args[4]
        if self.email == "N":
            self.key = "%s_all" % self.conum
            self.printSetup()
        if self.args:
            all = [self.args[0]]
        else:
            whr = [("mlm_cono", "=", self.conum)]
            if self.status != "Z":    # Not All
                whr.append(("mlm_state", "=", self.status))
            if self.sort == "N":
                ord = "mlm_memno"
            else:
                ord = "mlm_surname"
            if self.whole == "S":
                all = getSingleRecords(self.mf, "memmst", ("mlm_memno",
                    "mlm_surname", "mlm_names"), head=["X", "Number", "Surname",
                    "Names"], where=whr, order=ord, selcol="mlm_surname")
            else:
                if self.whole == "R":
                    whr.append(("mlm_memno", "between", self.start, self.to))
                all = self.sql.getRec(tables="memmst", where=whr, order=ord)
                if not all:
                    showError(self.mf.body, "Error", "No Accounts Selected")
                    self.mf.closeLoop()
                    return
                # Remove all linked accounts
                col = self.sql.memmst_col
                nos = []
                for acc in all:
                    nos.append(acc[col.index("mlm_memno")])
                chk = all[:]
                for acc in chk:
                    whr = [
                        ("mll_cono", "=", self.conum),
                        ("mll_lnkno", "=", acc[col.index("mlm_memno")])]
                    lnk = self.sql.getRec(tables="memlnk", where=whr, limit=1)
                    if lnk and lnk[1] in nos:
                        all.remove(acc)
        if all:
            if self.email == "N":
                mess = "Printing Statements"
            else:
                mess = "Printing and Emailing Statements"
            p = ProgressBar(self.mf.body, typ=mess, max=len(all), esc=True)
            for num, rec in enumerate(all):
                can = p.displayProgress(num)
                if can == "yes":
                    if self.t:
                        self.t.textLine()
                        self.t.textLine("Print Job ABORTED")
                    break
                self.doProcess(rec)
            p.closeProgress()
            if self.email == "N":
                self.c.save()
                self.doPrint()
        if not self.args:
            self.mf.closeLoop()
         
    def doProcess(self, mlm):
        self.pgnum = 0
        dmc = self.sql.memmst_col
        dma = self.sql.memadd_col
        dmk = self.sql.memkon_col
        self.memno = CCD(mlm[dmc.index("mlm_memno")], "UI", 6)
        self.doGetBalTrn()
        if not self.drt[1]:
            return
        add = self.sql.getRec(tables="memadd", where=[("mla_cono", "=",
            self.conum), ("mla_memno", "=", self.memno.work), ("mla_type", "=",
            "P")], limit=1)
        if self.zeros == "N" and not self.tbal:
            return
        elif self.zeros == "C" and not self.tbal and not self.drt[2]:
            return
        if self.minus == "N" and self.tbal < 0:
            return
        if self.email == "Y":
            self.key = "%s_%s" % (self.conum, self.memno.work)
            self.printSetup()
        tit = mlm[dmc.index("mlm_title")]
        sur = mlm[dmc.index("mlm_surname")]
        nam = mlm[dmc.index("mlm_names")]
        ini = ""
        for n, d in enumerate(nam.split()):
            if n < 3:
                if not ini:
                    ini = d[0].upper()
                else:
                    ini = "%s %s" % (ini, d[0].upper())
        self.name = "%s %s %s" % (tit.strip(), ini.strip(), sur.strip())
        if add:
            self.add1 = CCD(add[dma.index("mla_add1")], "TX", 30)
            self.add2 = CCD(add[dma.index("mla_add2")], "TX", 30)
            self.add3 = CCD(add[dma.index("mla_add3")], "TX", 30)
            self.city = CCD(add[dma.index("mla_city")], "TX", 30)
            self.pcod = CCD(add[dma.index("mla_code")], "TX", 4)
            self.region = CCD(add[dma.index("mla_region")], "TX", 30)
            self.country = CCD(add[dma.index("mla_country")], "TX", 30)
        else:
            self.add1 = CCD("", "TX", 30)
            self.add2 = CCD("", "TX", 30)
            self.add3 = CCD("", "TX", 30)
            self.city = CCD("", "TX", 30)
            self.pcod = CCD("", "TX", 4)
            self.region = CCD("", "TX", 30)
            self.country = CCD("", "TX", 30)
        self.emadd = []
        kon = self.sql.getRec(tables=["memctk", "memkon"], cols=["mlk_detail"],
            where=[("mck_type", "=", "E"), ("mlk_cono", "=", self.conum),
            ("mlk_memno", "=", self.memno.work), ("mlk_code=mck_code",)])
        for k in kon:
            self.emadd.append(k[0])
        if self.whole == "E" and not self.emadd:
            return
        elif self.whole == "P" and self.emadd:
            return
        self.printHeader()
        self.printBody()
        self.printTrailer()
        if self.email == "Y":
            self.c.save()
            self.doPrint()

    def doGetBalTrn(self):
        if self.paid == "Y":
            trans = "A"
        else:
            trans = "Y"
        bals = Balances(self.mf, "MEM", self.conum, self.curdt,
            (self.memno.work,))
        self.obal, self.tbal, self.ages, self.drt = bals.doAllBals(trans=trans)
        lnk = self.sql.getRec(tables="memlnk", cols=["mll_lnkno"],
            where=[("mll_cono", "=", self.conum), ("mll_memno", "=",
            self.memno.work)])
        if not lnk:
            return
        for l in lnk:
            bals = Balances(self.mf, "MEM", self.conum, self.curdt, (l[0],))
            obal, tbal, ages, drt = bals.doAllBals(trans=trans)
            for n, d in enumerate(drt[1]):
                drt[1][n][drt[0].index("mlt_desc")] = "%s (%s)" % \
                    (d[drt[0].index("mlt_desc")][:30],
                    d[drt[0].index("mlt_memno")])
            self.obal = float(ASD(self.obal) + ASD(obal))
            self.tbal = float(ASD(self.tbal) + ASD(tbal))
            for x in range(5):
                self.ages[x] = float(ASD(self.ages[x]) + ASD(ages[x]))
            self.drt[1].extend(drt[1])
            self.drt[2] += drt[2]
        # Sort Transaction by Date
        self.drt[1].sort(key=lambda x:x[drt[0].index("mlt_trdt")])

    def printSetup(self):
        if self.email == "Y" or not self.pdfnam:
            self.pdfnam = getModName(self.mf.wrkdir, __name__, self.key,
                ext="pdf")
        self.c, self.cpi, self.inch, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam)
        # Width in pixels of each character
        self.cwidth = self.c.stringWidth(("*"), "Tartan", self.cpi)

    def doPrint(self):
        if self.prtnam == "view":
            view = "Y"
        else:
            view = "N"
        if self.email == "N" or not self.emadd:
            mail = "N"
            mess = None
        else:
            mail = self.emadd
            mess = "%s - Statement as at %s" % (self.conam, self.dated)
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam, head=mess,
            view=view, prtnam=self.prtnam, mail=mail, noprt=self.noprt,
            splash=False, skip=True)

    def printHeader(self):
        self.pageHeading()
        self.drawDocument()
        self.pgnum = self.pgnum + 1
        self.t.textLine()
        self.t.textLine()
        self.t.setFont("Tartan-Bold", 18)
        self.t.textLine(self.head[0])
        self.t.setFont("Tartan-Bold", 10)
        self.t.textLine(self.head[1])
        self.t.textLine(self.head[2])
        self.t.textLine(self.head[3])
        self.t.textLine(self.head[4])
        self.t.textLine()
        self.t.setFont("Tartan", 10)
        mess = "%-70s%7s" % ("", "")
        self.t.textLine(mess)
        mess = "%1s%-30s" % ("", self.name)
        self.t.textLine(mess)
        addr = []
        if self.add1.work:
            addr.append(self.add1.disp)
        if self.add2.work:
            addr.append(self.add2.disp)
        if self.add3.work:
            addr.append(self.add3.disp)
        if self.city.work:
            addr.append(self.city.disp)
        if self.country.work:
            addr.append("%-s, %-s" % (self.pcod.disp, self.country.disp))
        else:
            addr.append(self.pcod.disp)
        for num, dat in enumerate(addr):
            if num == 2:
                mess = "%1s%-69s%7s" % ("", dat[:65], self.memno.disp)
            elif num == 4:
                mess = "%1s%-66s%10s" % ("", dat[:65], self.dated)
            else:
                mess = "%1s%-s" % ("", dat)
            self.t.textLine(mess)
        for x in range(num+1, 7):
            if x == 2:
                self.t.textLine("%71s%7s" % ("", self.memno.disp))
            elif x == 4:
                self.t.textLine("%67s%10s" % ("", self.dated))
            else:
                self.t.textLine()
        self.t.setFont("Tartan", 10)
        self.pglin = 0

    def pageHeading(self):
        self.c.translate(self.inch, self.inch) # Set x and y margins
        self.t = self.c.beginText()
        self.t.setFont("Tartan-Bold", self.cpi)
        self.t.setTextOrigin(0, 0)

    def printBody(self):
        obal = 0.0
        if len(self.drt[1]) > 20:
            chk = len(self.drt[1]) - 20
        else:
            chk = 0
        mlt = self.drt[0]
        for num, drt in enumerate(self.drt[1]):
            trdt = CCD(drt[mlt.index("mlt_trdt")], "d1", 10)
            refno = CCD(drt[mlt.index("mlt_refno")], "Na", 9)
            trtp = CCD(drt[mlt.index("mlt_type")], "UI", 1)
            desc = CCD(drt[mlt.index("mlt_desc")], "NA", 39)
            tramt = CCD(drt[mlt.index("mlt_tramt")], "SD", 11.2)
            if chk:
                if num < chk:
                    obal = float(ASD(obal) + ASD(tramt.work))
                    continue
                elif num == chk:
                    obal = CCD(obal, "SD", 11.2)
                    mess = "%2s%s %s %-3s %-39s %s" % ("", trdt.disp,
                        "Bal/B-Fwd", "B/F", "BALANCE BROUGHT FORWARD",
                        obal.disp)
                    self.t.textLine(mess)
                    self.pglin = self.pglin + 1
            mess = "%2s%s %s %s %s %s" % ("", trdt.disp, refno.disp,
                self.trtypes[trtp.work - 1], desc.disp, tramt.disp)
            self.t.textLine(mess)
            self.pglin = self.pglin + 1

    def printTrailer(self):
        # Print Ageing and Total
        tots = [CCD(self.tbal, "SD", 11.2).disp]
        for x in range(0, 5):
            y = CCD(self.ages[x], "SD", 11.2)
            tots.append(y.disp)
        mess = "%2s%s %s  %s  %s  %s   %s" % ("", tots[5], tots[4], tots[3],
            tots[2], tots[1], tots[0])
        for z in range(0, (self.maxlines + 2 - self.pglin)):
            self.t.textLine()
        self.t.textLine(mess)
        # Draw Banking Details Block
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*78
        b1 = 42.50
        b2 = 43.40
        self.c.rect(x, (b1*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
        self.c.rect(x, (b1*(self.inch/6)), w, (5.25*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 10)
        self.c.drawString(self.cwidth*1, (b2*(self.inch/6)),
            "Banking Details")
        self.c.drawString((self.cwidth*1), (45*(self.inch/6)),
            "Bank:                       Branch:           Account:")
        self.c.setFont("Tartan", 10)
        self.c.drawString((self.cwidth*1), (45*(self.inch/6)),
            "       %-20s          %-8s           %-16s" % (self.ctm_b_name,
            self.ctm_b_ibt, self.ctm_b_acno))
        self.c.setFont("Tartan-Bold", 10)
        self.c.drawString((self.cwidth*1), (47*(self.inch/6)), "Note:")
        self.c.setFont("Tartan", 10)
        self.c.drawString((self.cwidth*1), (47*(self.inch/6)),
            "      Please Use Your Membership Number (%s) as the Payment "\
            "Reference." % self.memno.disp)
        # Check for Message
        if self.mes:
            mes2 = self.mes[:]
        else:
            mes2 = None
        if mes2:
            # Draw Message Rectangles
            self.c.setFillGray(.8)
            x = self.cwidth*.5
            w = self.cwidth*78
            b1 = 48.50
            b2 = 49.30
            self.c.rect(x, (b1*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
            self.c.rect(x, (b1*(self.inch/6)), w, (4.75*(self.inch/6)), fill=0)
            self.c.setFillGray(0)
            self.c.setFont("Tartan-Bold", 10)
            self.c.drawString(self.cwidth*1,(b2*(self.inch/6)),"Message")
            self.c.setFont("Tartan", 10)
            for x in range(9):
                self.t.textLine()
            mes3 = textFormat(mes2[0], width=75)
            for mess in mes3[:3]:
                self.t.textLine(" " + mess)
        self.c.drawText(self.t)
        self.c.showPage()

    def drawDocument(self):
        # If logo, draw logo
        if self.ctm_logo:
            self.c.drawImage(self.ctm_logo, 0, 3, 350, 70, None)
        # Draw Titles
        self.c.setFont("Tartan-Bold", 18)
        self.c.drawString(self.cwidth*62, 2*(self.inch/6), "STATEMENT")
        self.c.drawString(self.cwidth*62, 3*(self.inch/6), "& INVOICE")
        self.c.setFont("Tartan-Bold", self.cpi)
        # Draw Account Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*42.5
        self.c.rect(x, (7.75*(self.inch/6)), w, (1*(self.inch/6)), fill=1)
        self.c.rect(x, (7.75*(self.inch/6)), w, (7.25*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 8)
        self.c.drawString(self.cwidth*1, (8.45*(self.inch/6)), "Account")
        self.c.setFont("Tartan", 10)
        # Draw Member and Date Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*64.5
        w = self.cwidth*14
        for q in (11, 13):
            self.c.rect(x, (q*(self.inch/6)), w, (1*(self.inch/6)), fill=1)
            self.c.rect(x, (q*(self.inch/6)), w, (2*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 8)
        self.c.drawCentredString((self.cwidth*71.5),
            (11.80*(self.inch/6)), "Member")
        self.c.drawCentredString((self.cwidth*71.5),
            (13.80*(self.inch/6)), "Date")
        self.c.setFont("Tartan", 10)
        # Draw Ribbon Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*78
        b1 = 22.75
        b2 = 38.25
        self.c.rect(x, (15.50*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
        self.c.rect(x, (15.50*(self.inch/6)), w, (b1*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 10)
        for q, r in ((2, "   Date"), (13, "Reference"), (23, "Typ"),
            (27, "Details"), (63, "        Amount")):
            self.c.drawString(self.cwidth*q, (16.40*(self.inch/6)), r)
        self.c.setFont("Tartan", 10)
        for q in (12.5, 22.5, 26.5, 62.5):
            self.c.line(self.cwidth*q, (15.50*(self.inch/6)), self.cwidth*q,
                (b2*(self.inch/6)))
        # Draw Total Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*78
        b1 = 38.50
        b2 = 41.25
        b3 = 39.40
        self.c.rect(x, (b1*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
        self.c.rect(x, (b1*(self.inch/6)), w, (2.75*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 10)
        for q, r in (
            (2,  "  120 Days"),
            (14, "   90 Days"),
            (26, "    60 Days"),
            (39, "    30 Days"),
            (52, "    Current"),
            (65, "   Total-Due")):
            self.c.drawString(self.cwidth*q, (b3*(self.inch/6)), r)
        self.c.setFont("Tartan", 10)
        for q in (13.5, 25.5, 38.5, 51.5, 64.5):
            self.c.line(self.cwidth*q, (b1*(self.inch/6)), self.cwidth*q,
                (b2*(self.inch/6)))

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    from TartanClasses import MainFrame
    mf = MainFrame()
    mf.dbm.openDbase()
    dr3080(mf, 2, "Company 1")
