#! /usr/bin/env python

"""
SYNOPSIS
    Members Ledger Category Changes.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import CCD, GetCtl, ProgressBar, SplashScreen, Sql
from TartanClasses import TartanDialog
from tartanFunctions import doPrinter, doReportlab, doWriteExport, getModName
from tartanFunctions import doChkCatChg, showError

class ml3060:
    def __init__(self, mf, coy, nam, args=None):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            if not args:
                self.mainProcess()
                self.mf.startLoop()
            else:
                self.reptyp = "A"
                if args[0] > args[1]:
                    self.start = CCD(args[1], "D1", 10)
                else:
                    self.start = CCD(args[0], "D1", 10)
                self.end = CCD(args[1], "D1", 10)
                self.chgtyp = "A"
                self.cat = ""
                self.cod = 0
                self.view = "V"
                self.prtnam = "view"
                self.mail = "N"
                chg = self.getRecords()
                if chg:
                    self.printReport(chg)

    def setVariables(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Members Category Changes (%s)" % __name__)
        self.sql = Sql(self.mf.dbm, ["chglog", "memcat", "memctc", "memmst"],
            prog=__name__)
        gc = GetCtl(self.mf)
        memctl = gc.getCtl("memctl", self.conum)
        if not memctl:
            return "no"
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.head = ("%03u %-30s %s" % (self.conum, self.conam, "%s"))
        self.colsh = ["Mem-No", "Titles", "Inits", "Surname", "Actions",
            "Category", "Action-Dte", "Operator", "Start-Date", " End-Date",
            "Last-Date"]
        self.forms = [("UI", 6), ("NA", 6), ("NA", 5), ("NA", 30), ("UA", 7),
            ("NA", 30), ("D1", 10), ("NA", 20), ("d1", 10), ("d1", 10),
            ("d1", 10)]
        self.t = None
        self.ctyp = (
            ("All", "A"),
            ("New", "N"),
            ("Updated", "U"),
            ("Deleted", "D"))
        self.catg = (
            ("All", "X"),
            ("Fees", "A"),
            ("Main","B"),
            ("Sports","C"),
            ("Debentures","D"))

    def mainProcess(self):
        cod = {
            "stype": "R",
            "tables": ("memctc",),
            "cols": (
                ("mcc_code", "", 0, "Cd"),
                ("mcc_desc", "", 0, "Description", "Y")),
            "where": [("mcc_cono", "=", self.conum)],
            "whera": [["T", "mcc_type", 3]],
            "order": "mcc_code",
            "size": (400, 600)}
        r1s = (
            ("Actual", "A"),
            ("Pending", "P"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Report Type","Report Type",
                "A","Y",self.doRepTyp,None,None,None),
            (("T",0,1,0),"Id1",10,"Starting Date","Starting Date",
                self.sysdtw,"Y",self.doStartDate,None,None,None),
            (("T",0,2,0),"ID1",10,"Ending   Date","Ending Date",
                self.sysdtw,"Y",self.doEndDate,None,None,None),
            (("T",0,3,0),("IRB",self.ctyp),0,"Change Type","Change Type",
                "A","Y",self.doChgTyp,None,None,None),
            (("T",0,4,0),("IRB",self.catg),0,"Category","Category",
                "X","Y",self.doCat,None,None,None),
            (("T",0,5,0),"IUI",2,"Code","Code",
                0,"Y",self.doCod,cod,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"))

    def doRepTyp(self, frt, pag, r, c, p, i, w):
        self.reptyp = w

    def doStartDate(self, frt, pag, r, c, p, i, w):
        self.start = CCD(w, "d1", 10)
        if self.reptyp == "P":
            self.end = self.start
            self.df.loadEntry(frt, pag, p+1, data=self.end.work)
            return "sk1"

    def doEndDate(self, frt, pag, r, c, p, i, w):
        if w < self.start.work:
            return "Invalid End Date, Before Start Date"
        self.end = CCD(w, "d1", 10)
        if self.reptyp == "P":
            self.chgtyp = "A"
            self.cat = None
            self.cod = None
            self.df.loadEntry(frt, pag, p+1, data="A")
            self.df.loadEntry(frt, pag, p+2, data="X")
            self.df.loadEntry(frt, pag, p+3, data=0)
            return "sk3"

    def doChgTyp(self, frt, pag, r, c, p, i, w):
        self.chgtyp = w

    def doCat(self, frt, pag, r, c, p, i, w):
        self.cat = w
        if w == "X":
            self.cat = None
            self.cod = None
            self.df.loadEntry(frt, pag, p+1, data=0)
            return "sk1"
        self.cat = w

    def doCod(self, frt, pag, r, c, p, i, w):
        self.cod = w

    def doEnd(self):
        self.df.closeProcess()
        self.view = self.df.repprt[0][1]
        self.prtnam = self.df.repprt[0][2]
        self.mail = self.df.repprt[1][2]
        if self.reptyp == "P":
            sp = SplashScreen(self.mf.body, "Preparing Report ... Please Wait")
            all = self.sql.getRec(tables="memmst", cols=["mlm_memno"],
                where=[("mlm_cono", "=", self.conum), ("mlm_state", "=", "A")])
            for acc in all:
                data = doChkCatChg(self.mf, self.conum, acc[0],
                    self.end.work, self.start.work)
                if data:
                    self.dte = long("%08i000000" % self.start.work)
                    self.sql.insRec("chglog", data=["memcat", "D",
                        "%03i%06i%1s%02i" % (self.conum, acc[0], "B", data[0]),
                        "", self.dte, "", str(data[1]), str(data[2]), 0])
                    self.sql.insRec("chglog", data=["memcat", "N",
                        "%03i%06i%1s%02i" % (self.conum, acc[0], "B", data[7]),
                        "", self.dte, "", str(self.start.work), str(0), 0])
            sp.closeSplash()
        chg = self.getRecords()
        if self.reptyp == "P":
            self.mf.dbm.rollbackDbase()
        if chg:
            if self.prtnam == "export":
                self.exportReport(chg)
            else:
                self.printReport(chg)
        self.closeProcess()

    def getRecords(self):
        whr = [("chg_tab", "=", "memcat")]
        if self.reptyp == "P":
            whr.append(("chg_dte", "=", self.dte))
        elif self.start.work:
            whr.append(("chg_dte", "between", (self.start.work * 1000000),
                (self.end.work * 1000000) + 999999))
        else:
            whr.append(("chg_dte", "<=", (self.end.work * 1000000) + 999999))
        if self.chgtyp != "A":
            whr.append(("chg_act", "=", self.chgtyp))
        if self.cat:
            key = "%03i______%1s" % (self.conum, self.cat)
            if self.cod:
                key = "%s%02i" % (key, self.cod)
            else:
                key = "%s__" % key
            whr.append(("chg_key", "like", key))
        ord = "chg_key, chg_seq"
        chg = self.sql.getRec(tables="chglog", where=whr, order=ord)
        if not chg:
            showError(self.mf.body, "Selection Error", "No Available Records")
        else:
            return chg

    def exportReport(self, chg):
        p = ProgressBar(self.mf.body, max=len(chg), esc=True)
        expnam = getModName(self.mf.wrkdir, __name__, self.conum)
        self.expheads = [self.head % self.sysdttm]
        self.expheads.append("Members Category Changes between %s and %s" %
            (self.start.disp, self.end.disp))
        self.expheads.append(self.getOptions())
        self.expcolsh = [self.colsh]
        self.expforms = self.forms
        self.expdatas = []
        lmemno = 0
        for num, dat in enumerate(chg):
            can = p.displayProgress(num)
            if can == "yes":
                p.closeProgress()
                return
            vals = self.getValues(dat)
            if not vals:
                continue
            if vals[0].work == lmemno:
                self.expdatas.append(["BODY", ["", "", "", "", vals[4],
                    vals[5].work, vals[6].work, vals[7].work, vals[8].work,
                    vals[9].work, vals[10].work]])
            else:
                self.expdatas.append(["BODY", [vals[0].work, vals[1].work,
                    vals[2].work, vals[3].work, vals[4], vals[5].work,
                    vals[6].work, vals[7].work, vals[8].work, vals[9].work,
                    vals[10].work]])
            lmemno = vals[0].work
        p.closeProgress()
        doWriteExport(xtype=self.view, name=expnam, sheet=__name__,
            heads=self.expheads, colsh=self.expcolsh, forms=self.expforms,
            datas=self.expdatas, rcdic=self.mf.rcdic)

    def printReport(self, chg):
        pdfnam = getModName(self.mf.wrkdir, __name__, self.conum, ext="pdf")
        p = ProgressBar(self.mf.body, max=len(chg), esc=True)
        self.head = ("%03u %-30s %27s %33s %46s %10s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(chg):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            vals = self.getValues(dat)
            if not vals:
                continue
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
                lmemno = 0
            if vals[0].work == lmemno:
                self.t.textLine("%6s %6s %5s %30s %s %s %s %s %s %s %s" % \
                    ("", "", "", "", vals[4], vals[5].disp, vals[6].disp,
                    vals[7].disp, vals[8].disp, vals[9].disp, vals[10].disp))
            else:
                self.t.textLine("%s %s %s %s %s %s %s %s %s %s %s" % \
                    (vals[0].disp, vals[1].disp, vals[2].disp, vals[3].disp,
                    vals[4], vals[5].disp, vals[6].disp, vals[7].disp,
                    vals[8].disp, vals[9].disp, vals[10].disp))
            lmemno = vals[0].work
            self.pglin = self.pglin + 1
        p.closeProgress()
        if self.t:
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
            view=self.view, prtnam=self.prtnam, mail=self.mail)

    def getValues(self, data):
        key = data[self.sql.chglog_col.index("chg_key")]
        cono = CCD(key[:3], "UI", 3)
        if cono.work != self.conum:
            return
        memno = CCD(key[3:9], "UI", 6)
        ctype = CCD(key[9:10], "UA", 1)
        code = CCD(key[10:12], "UI", 2)
        last = CCD(0, "d1", 10)
        if data[self.sql.chglog_col.index("chg_act")] == "D":
            act = "Removed"
        elif data[self.sql.chglog_col.index("chg_act")] == "N":
            act = "Added  "
            last = self.sql.getRec(tables="memcat", cols=["mlc_last"],
                where=[("mlc_cono", "=", self.conum), ("mlc_memno", "=",
                memno.work), ("mlc_type", "=", ctype.work), ("mlc_code",
                "=", code.work)], limit=1)
            if not last:
                last = CCD("Removed?", "NA", 10)
            else:
                last = CCD(last[0], "d1", 10)
        elif data[self.sql.chglog_col.index("chg_act")] == "U":
            act = "Changed"
        else:
            act = "Unknown"
        dte = data[self.sql.chglog_col.index("chg_dte")]
        dte = CCD(dte / 1000000, "D1", 10)
        usr = CCD(data[self.sql.chglog_col.index("chg_usr")], "NA", 20)
        old = CCD(data[self.sql.chglog_col.index("chg_old")], "d1", 10)
        new = CCD(data[self.sql.chglog_col.index("chg_new")], "d1", 10)
        mst = self.sql.getRec(tables="memmst", where=[("mlm_cono", "=",
            self.conum), ("mlm_memno", "=", memno.work)], limit=1)
        if not mst:
            return
        tit = CCD(mst[self.sql.memmst_col.index("mlm_title")], "UA", 6)
        ini = CCD(mst[self.sql.memmst_col.index("mlm_initial")], "UA", 5)
        sur = CCD(mst[self.sql.memmst_col.index("mlm_surname")], "NA", 30)
        cat = self.sql.getRec(tables="memctc", where=[("mcc_cono", "=",
            self.conum), ("mcc_type", "=", ctype.work), ("mcc_code", "=",
            code.work)], limit=1)
        if not cat:
            return
        des = CCD(cat[self.sql.memctc_col.index("mcc_desc")], "NA", 30)
        return (memno, tit, ini, sur, act, des, dte, usr, old, new, last)
 
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-32s %10s %3s %10s %89s %5s" % ("Members Category "\
            "Changes between", self.start.disp, "and", self.end.disp, "Page",
            self.pgnum))
        self.t.textLine()
        self.t.textLine(self.getOptions())
        self.t.textLine()
        self.t.textLine("%6s %-6s %-5s %-30s %1s %-30s %10s %-20s %10s %10s "\
            "%10s" % ("Mem-No", "Titles", "Inits", "Surname", "Actions",
            "Category", "Action-Dte", "Operator", "Start-Date", " End-Date",
            "Last-Date"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def getOptions(self):
        if self.reptyp == "A":
            mess = "Options: Actual"
        else:
            mess = "Options: Pending"
        for x in self.ctyp:
            if self.chgtyp == x[1]:
                mess = mess + ", Type %s" % x[0]
                break
        for x in self.catg:
            if self.cat == x[1]:
                mess = mess + ", Category %s" % x[0]
        if self.cod:
            mess = mess + ", Code %s" % self.cod
        return mess

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

