#! /usr/bin/env python

"""
SYNOPSIS
    Members Ledger Suspension Report.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, GetCtl, ProgressBar, RepPrt, Sql, SplashScreen
from TartanClasses import TartanDialog
from tartanFunctions import askQuestion, getTrn

class ml3080:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        mc = GetCtl(self.mf)
        memctl = mc.getCtl("memctl", self.conum)
        if not memctl:
           return "no"
        self.acs = memctl["mcm_acs"]
        if self.acs == "Y":
            try:
                self.dba = Dbase(rcdic={
                    "dbase": "FBird",
                    "dbhost": "%s" % str(memctl["mcm_ahst"]),
                    "dbname": "%s" % str(memctl["mcm_adbn"]),
                    "dbuser": "%s" % str(memctl["mcm_ausr"]),
                    "dbpwd": "%s" % str(memctl["mcm_apwd"]),
                    "dbdir": "",
                    "wrkdir": self.mf.wrkdir})
                self.dba.openDbase()
                self.sqa = Sql(self.dba, prog=__name__)
            except:
                self.acs = "N"
        self.sql = Sql(self.mf.dbm, ["memmst", "memsta", "memtrn"],
            prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Member's Suspension Report")
        r1s = (("Number", "N"), ("Surname", "M"))
        r2s = (("Yes", "Y"), ("No", "N"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Sort Order","Sort Order",
                "N","Y",self.doSort,None,None,None),
            (("T",0,1,0),("IRB",r2s),0,"Include Pay Plan","Include Pay Plan",
                "N","Y",self.doPayPlan,None,None,None))
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            tend=tnd, txit=txt, view=("Y","V"), mail=("Y","N"))

    def doSort(self, frt, pag, r, c, p, i, w):
        self.sort = w

    def doPayPlan(self, frt, pag, r, c, p, i, w):
        self.plan = w

    def doEnd(self):
        self.df.closeProcess()
        col = ["mlm_memno", "mlm_surname", "mlm_names", "mlm_payplan",
            "mlc_code"]
        whr = [("mlm_cono", "=", self.conum), ("mlm_state", "=", "A")]
        if self.plan == "N":
            whr.append(("mlm_payplan", "=", "N"))
        whr.extend([("mlc_cono=mlm_cono",), ("mlc_memno=mlm_memno",),
            ("mlc_type", "=", "B")])
        grp = "mlm_memno, mlm_surname, mlm_names, mlm_payplan, mlc_code"
        if self.sort == "N":
            ord = "mlm_memno"
        else:
            ord = "mlm_surname"
        sp = SplashScreen(self.mf.body, "Preparing Report ... Please Wait")
        all = self.sql.getRec(tables=["memmst", "memcat"], cols=col, where=whr,
            group=grp, order=ord)
        sp.closeSplash()
        if not all:
            self.mf.closeLoop()
            return
        p = ProgressBar(self.mf.body, typ="Generating the Report",
            max=len(all), esc=True)
        data = []
        for num, rec in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                break
            whr = [("mlt_cono", "=", self.conum), ("mlt_memno", "=", rec[0])]
            trn = getTrn(self.mf.dbm, "mem", whr=whr, zer="N")
            bl1 = 0
            bl2 = 0
            for t in trn[1]:
                bl1 = float(ASD(bl1) + ASD(t[trn[0].index("balance")]))
                if t[trn[0].index("mlt_ctyp")] == "B":
                    bl2 = float(ASD(bl2) + ASD(t[trn[0].index("balance")]))
            if not bl1 or bl1 < bl2:
                continue
            cat = self.sql.getRec(tables=["memctc", "memctp"], cols=["mcc_desc",
                "mcp_rate_01"], where=[("mcc_cono", "=", self.conum),
                ("mcc_type", "=", "B"), ("mcc_code", "=", rec[4]),
                ("mcp_cono=mcc_cono",), ("mcp_type=mcc_type",),
                ("mcp_code=mcc_code",)], order="mcp_date desc", limit=1)
            if not cat or not cat[1] or bl2 < cat[1]:
                continue
            rec[1] = "%s, %s" % (rec[1], rec[2])
            del rec[2]
            rec[3] = cat[0]
            rec.append(cat[1])
            rec.append(bl2)
            rec.append(bl1)
            data.append(rec)
        p.closeProgress()
        if can == "yes":
            self.mf.closeLoop()
            return
        name = __name__
        head = ["Members Suspension Report"]
        cols = [
            ["a", "NA",  7,   "Mem-No",          "y"],
            ["b", "NA", 50,   "Name",            "y"],
            ["d", "UA", 1,    "P",               "y"],
            ["c", "NA", 30,   "Active-Category", "y"],
            ["e", "SD", 13.2, "Subscription",    "y"],
            ["f", "SD", 13.2, "Sub-Balance",     "y"],
            ["g", "SD", 13.2, "Tot-Balance",     "y"]]
        RepPrt(self.mf, conum=self.conum, conam=self.conam, name=name,
            tables=data, heads=head, cols=cols, gtots=("f", "g"), ttype="D",
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
        if askQuestion(self.mf.body, "Suspend Members", "Suspend All "\
                    "Members on the Report?", default="no") == "yes":
            p = ProgressBar(self.mf.body, typ="Suspending Members",
                max=len(data), esc=False)
            for num, rec in enumerate(data):
                p.displayProgress(num)
                self.sql.updRec("memmst", cols=["mlm_state", "mlm_sdate"],
                    data=["S", self.sysdtw], where=[("mlm_cono", "=",
                    self.conum), ("mlm_memno", "=", rec[0])])
                self.sql.insRec("memsta", data=[self.conum, rec[0], "S",
                    self.sysdtw, self.sysdtw, 0])
                if self.acs == "Y":
                    tag = self.sqa.sqlRec(("Select a.tag_code from tag a, "\
                        "employee b where b.emp_employeeno=? and "\
                        "b.mst_sq=a.mst_sq", (str(rec[0]),)), limit=1)
                    if tag:
                        self.sqa.sqlRec(("Update tag set tag_suspend=?, "\
                            "tag_blacklist=?, tag_expiry=?, tag_modified=? "\
                            "where tag_code=?", (1, 0, self.sysdtw, 1, tag[0])))
                        self.sqa.sqlRec(("Update tag_t_a_g set "\
                            "ttag_modified=? where tag_code=?", (1, tag[0])))
            self.mf.dbm.commitDbase()
            if self.acs == "Y":
                self.dba.commitDbase()
        p.closeProgress()
        self.mf.closeLoop()

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    from TartanClasses import MainFrame
    mf = MainFrame(fixed=False)
    mf.dbm.openDbase()
    rpt = ml3080(mf, 1, "Company One")
