#! /usr/bin/env python

"""
SYNOPSIS
    Members Ledger Print Labels. L7159

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, doPrinter, getModName, getSingleRecords
from tartanFunctions import makeArray, showError

class ml3110:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ("memctc", "memmst", "memadd"),
            prog=__name__)
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None
        self.labels = {"L7159": (3, 8, 32)}

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Print Creditors Labels (%s)" % __name__)
        cod = {
            "stype": "R",
            "tables": ("memctc",),
            "cols": (
                ("mcc_code", "", 0, "Cd"),
                ("mcc_desc", "", 0, "Description", "Y")),
            "where": [],
            "order": "mcc_code",
            "size": (400, 600)}
        r1s = (
            ("All", "Z"),
            ("Active", "A"),
            ("Deceased", "D"),
            ("Inactive", "I"),
            ("Resigned", "R"),
            ("Suspended", "S"),
            ("Defaulted", "X"))
        r2s = (
            ("All", "A"),
            ("Main","B"),
            ("Sports","C"),
            ("Debentures","D"))
        r3s = (("Number","N"),("Name","M"),("Postal Code","P"))
        lab = []
        for l in self.labels:
            lab.append(l)
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Status","",
                "A","Y",self.doStat,None,None,None),
            (("T",0,1,0),("IRB",r2s),0,"Category","",
                "A","Y",self.doCat,None,None,None),
            (("T",0,2,0),"IUI",2,"Code","",
                0,"Y",self.doCod,cod,None,None),
            (("T",0,2,0),"ONA",30,""),
            (("T",0,3,0),("IRB",r3s),0,"Sort Order","",
                "N","Y",self.doSort,None,None,None),
            (("T",0,5,0),"IUA",5,"Avery A4 Code","",
                "L7159","Y",self.doAvery,None,None,("in",lab)),
            (("T",0,6,0),"IUI",2,"First Label Position","",
                1,"Y",self.doStart,None,None,("notzero",)))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"))

    def doStat(self, frt, pag, r, c, p, i, w):
        self.state = w

    def doCat(self, frt, pag, r, c, p, i, w):
        if w == "A":
            self.cat = ""
            self.cod = ""
            self.df.loadEntry(frt, pag, p+2, data="All Categories and Sections")
            return "sk1"
        self.cat = w
        self.whr = [("mcc_cono", "=", self.conum), ("mcc_type", "=", self.cat)]

    def doCod(self, frt, pag, r, c, p, i, w):
        chk = self.sql.getRec(tables="memctc", cols=["mcc_desc"],
            where=[("mcc_cono", "=", self.conum), ("mcc_type", "=", self.cat),
            ("mcc_code", "=", w)], limit=1)
        if not chk:
            return "Invalid Category Code"
        self.cod = w
        self.df.loadEntry(frt, pag, p+1, data=chk[0])

    def doSort(self, frt, pag, r, c, p, i, w):
        self.sort = w

    def doAvery(self, frt, pag, r, c, p, i, w):
        self.across, self.down, self.width = self.labels[w]

    def doStart(self, frt, pag, r, c, p, i, w):
        max = (self.across * self.down) + 1
        if not w in range (1, max):
            return "Out of range"
        self.start = w

    def doEnd(self):
        self.df.closeProcess()
        self.view = self.df.repprt[0][1]
        self.prtnam = self.df.repprt[0][2]
        self.emadd = self.df.repprt[1][2]
        self.prnt = False
        whr = [("mlm_cono", "=", self.conum)]
        if self.state != "Z":
            if self.state == "R" and self.cat:
                whr.append(("mlm_state", "in", ("A", "R")))
            else:
                whr.append(("mlm_state", "=", self.state))
        tab = ["memmst"]
        if self.cat:
            tab.append("memcat")
            whr.append(("mlc_cono=mlm_cono",))
            whr.append(("mlc_memno=mlm_memno",))
            whr.append(("mlc_type", "=", self.cat))
            if self.cod:
                whr.append(("mlc_code", "=", self.cod))
            if self.state == "A":
                whr.append(("mlc_end", "=", 0))
            elif self.state == "R":
                whr.append(("(", "mlm_state", "=", "R", "and", "mlm_sdate",
                    "between", self.sstart, self.send, "or", "mlm_state",
                    "=", "A", "and", "mlc_end", "between", self.sstart,
                    self.send, ")"))
            grp = self.sql.memmst_col[0]
            for c in self.sql.memmst_col[1:]:
                grp = "%s, %s" % (grp, c)
        else:
            grp = None
        if self.sort == "N":
            ord = "mlm_memno"
        elif self.sort == "M":
            ord = "mlm_name"
        else:
            ord = "mlm_pcod"
        all = self.sql.getRec(tables=tab, cols=self.sql.memmst_col, where=whr,
            group=grp, order=ord)
        if not all:
            showError(self.mf.body, "Error", "No Accounts Selected")
        else:
            p = ProgressBar(self.mf.body, max=len(all), esc=True)
            for num, dat in enumerate(all):
                can = p.displayProgress(num)
                if can == "yes":
                    if self.t:
                        self.t.textLine()
                        self.t.textLine("Print Job ABORTED")
                    break
                self.doProcess(dat)
            p.closeProgress()
        if self.prnt:
            if self.vcount == 0 and self.hcount == 0:
                pass
            else:
                self.pageHeading()
                self.printPage()
                self.c.drawText(self.t)
                self.c.showPage()
            self.c.save()
            doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                head=self.tit, view=self.view, prtnam=self.prtnam,
                mail=self.emadd)
        self.closeProcess()

    def doProcess(self, mlm):
        if not self.prnt:
            self.printSetup()
            self.prnt = True
        dmm = self.sql.memmst_col
        dma = self.sql.memadd_col
        memno = CCD(mlm[dmm.index("mlm_memno")], "UI", 7)
        self.memno = CCD(mlm[dmm.index("mlm_memno")], "Na", self.width - 7)
        add = self.sql.getRec(tables="memadd", where=[("mla_cono", "=",
            self.conum), ("mla_memno", "=", memno.work), ("mla_type",
            "=", "P")], limit=1)
        if not add:
            return
        tit = mlm[dmm.index("mlm_title")].strip()
        sur = mlm[dmm.index("mlm_surname")].strip()
        nam = mlm[dmm.index("mlm_names")].strip()
        ini = ""
        for n, d in enumerate(nam.split()):
            if n < 3:
                if not ini:
                    ini = d[0].upper().strip()
                else:
                    ini = "%s %s" % (ini, d[0].upper().strip())
        self.name = CCD("%s %s %s" % (tit, ini, sur), "TX", self.width)
        self.add1 = CCD(add[dma.index("mla_add1")], "TX", self.width)
        self.add2 = CCD(add[dma.index("mla_add2")], "TX", self.width)
        self.add3 = CCD(add[dma.index("mla_add3")], "TX", self.width)
        self.city = CCD(add[dma.index("mla_city")], "TX", self.width)
        self.pcod = CCD(add[dma.index("mla_code")], "TX", self.width)
        self.country = CCD(add[dma.index("mla_country")], "TX", self.width)
        self.loadStorage()
        if self.storagefull == "Y":
            self.pageHeading()
            self.printPage()
            self.c.drawText(self.t)
            self.c.showPage()
            self.clearStorage()

    def printSetup(self):
        self.c, cpi, inch, suc, duc = doReportlab(self.mf.prgdir, self.pdfnam)
        self.makeStorage()
        self.viewcnt = 0
        self.linecnt = 0
        if self.start == 1:
            self.hcount = 0
            self.vcount = 0
        else:
            self.hcount = ((self.start % self.across) + \
                (self.across - 1)) % self.across
            self.vcount = ((self.start + (self.across - 1)) / self.across) - 1

    def pageHeading(self):
        self.t = self.c.beginText(15, 20)
        self.t.setFont("Tartan", 10, 12.7)

    def printPage(self):
        self.t.textLine()
        spc = " " * self.width
        for x in range(0, self.down):
            txmemno = ""
            txname = ""
            txadd1 = ""
            txadd2 = ""
            txadd3 = ""
            txadd4 = ""
            txadd5 = ""
            for y in range(0, self.across):
                txmemno = txmemno + self.xmemno[x][y] + \
                        spc[0:self.width - len(self.xmemno[x][y])]
                txname = txname + self.xname[x][y] + \
                        spc[0:self.width - len(self.xname[x][y])]
                txadd1 = txadd1 + self.xadd1[x][y] + \
                        spc[0:self.width - len(self.xadd1[x][y])]
                txadd2 = txadd2 + self.xadd2[x][y] + \
                        spc[0:self.width - len(self.xadd2[x][y])]
                txadd3 = txadd3 + self.xadd3[x][y] + \
                        spc[0:self.width - len(self.xadd3[x][y])]
                txadd4 = txadd4 + self.xadd4[x][y] + \
                        spc[0:self.width - len(self.xadd4[x][y])]
                txadd5 = txadd5 + self.xadd5[x][y] + \
                        spc[0:self.width - len(self.xadd5[x][y])]
            self.t.textLine(" %s" % txmemno)
            self.t.textLine(" %s" % txname)
            self.t.textLine(" %s" % txadd1)
            self.t.textLine(" %s" % txadd2)
            self.t.textLine(" %s" % txadd3)
            self.t.textLine(" %s" % txadd4)
            self.t.textLine(" %s" % txadd5)
            self.t.textLine(" ")
        self.clearStorage()
            
    def makeStorage(self):
        self.xmemno = makeArray(self.across, self.down, 1, "S")
        self.xname = makeArray(self.across, self.down, 1, "S")
        self.xadd1 = makeArray(self.across, self.down, 1, "S")
        self.xadd2 = makeArray(self.across, self.down, 1, "S")
        self.xadd3 = makeArray(self.across, self.down, 1, "S")
        self.xadd4 = makeArray(self.across, self.down, 1, "S")
        self.xadd5 = makeArray(self.across, self.down, 1, "S")
        self.clearStorage()
            
    def clearStorage(self):
        for x in range(self.down):
            for y in range(self.across):
                self.xmemno[x][y] = ""
                self.xname[x][y] = ""
                self.xadd1[x][y] = ""
                self.xadd2[x][y] = ""
                self.xadd3[x][y] = ""
                self.xadd4[x][y] = ""
                self.xadd5[x][y] = ""
        self.hcount = 0
        self.vcount = 0
        self.storagefull = "N"

    def loadStorage(self):
        self.xmemno[self.vcount][self.hcount] = self.memno.disp
        self.xname[self.vcount][self.hcount] = self.name.disp
        addr = []
        if self.add1.work:
            addr.append(self.add1.disp.strip())
        if self.add2.work:
            addr.append(self.add2.disp.strip())
        if self.add3.work:
            addr.append(self.add3.disp.strip())
        if self.city.work:
            addr.append(self.city.disp.strip())
        if self.country.work:
            addr.append("%-s, %-s" % (self.pcod.disp.strip(),
                self.country.disp.strip()))
        else:
            addr.append(self.pcod.disp.strip())
        for x in range(len(addr), 5):
            addr.append("")
        self.xadd1[self.vcount][self.hcount] = addr[0]
        self.xadd2[self.vcount][self.hcount] = addr[1]
        self.xadd3[self.vcount][self.hcount] = addr[2]
        self.xadd4[self.vcount][self.hcount] = addr[3]
        self.xadd5[self.vcount][self.hcount] = addr[4]
        if self.hcount == (self.across - 1) and self.vcount == (self.down - 1):
            self.storagefull = "Y"
        elif self.hcount != (self.across - 1):
            self.hcount = self.hcount + 1
        elif self.vcount != (self.down - 1):
            self.vcount = self.vcount + 1
            self.hcount = 0

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

