#! /usr/bin/python

"""
SYNOPSIS
    Member's Status Summary.

    This file is part of Tartan Financial Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import CreateChart, TartanDialog, Sql
from tartanFunctions import mthendDate

class ml3120:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "memsta", prog=__name__)
        self.desc = {
            "A": ["Active",0,0,0,0,0,0,0,0,0,0,0,0],
            "D": ["Deceased",0,0,0,0,0,0,0,0,0,0,0,0],
            "I": ["Inactive",0,0,0,0,0,0,0,0,0,0,0,0],
            "R": ["Resigned",0,0,0,0,0,0,0,0,0,0,0,0],
            "S": ["Suspended",0,0,0,0,0,0,0,0,0,0,0,0],
            "X": ["Defaulted",0,0,0,0,0,0,0,0,0,0,0,0]}

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Member's Status Summary")
        fld = (
            (("T",0,0,0),"ID2",7,"Starting Period","",
                "","N",self.doStartPer,None,None,None),
            (("T",0,1,0),"ID2",7,"Ending   Period","",
                "","N",self.doEndPer,None,None,None))
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            tend=tnd, txit=txt)

    def doStartPer(self, frt, pag, r, c, p, i, w):
        self.sdate = (w * 100) + 1

    def doEndPer(self, frt, pag, r, c, p, i, w):
        self.edate = mthendDate((w * 100) + 1)

    def doEnd(self):
        self.df.closeProcess()
        rec = self.sql.getRec(tables="memsta", cols=["mls_status",
            "mls_date/100 as date", "count(*)"], where=[("mls_date",
            "between", self.sdate, self.edate)], group="mls_status, date")
        for r in rec:
            month = r[1] % 100
            self.desc[r[0]][month] = r[2]
        achart = []
        for s in self.desc:
            achart.append([s] + self.desc[s])
        CreateChart(self.mf, self.conum, self.conam, [self.sdate/100,
            self.edate/100], [[self.conam, "Status Summary"], "Count"],
            achart, achart)
        self.mf.closeLoop()

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    from TartanClasses import MainFrame
    mf = MainFrame(fixed=False)
    rpt = ml3120(mf, 1, "Company One")

# vim:set ts=4 sw=4 sts=4 expandtab:
