#! /usr/bin/env python

"""
SYNOPSIS
    Members Ledger Contacts Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import Sql, TartanDialog

class mlc410:
    def __init__(self, mf):
        self.mf = mf
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["chglog", "memctk"], prog=__name__)
        if self.sql.error:
            return "no"
        chk = self.sql.getRec(tables="memctk")
        if not chk:
            self.sql.insRec("memctk", data=[1,"H","Home Telephone Number"])
            self.sql.insRec("memctk", data=[2,"W","Work Telephone Number"])
            self.sql.insRec("memctk", data=[3,"M","Mobile Telephone Number"])
            self.sql.insRec("memctk", data=[4,"F","Facsimile Number"])
            self.sql.insRec("memctk", data=[5,"E","E-Mail Address"])
            self.mf.dbm.commitDbase()

    def mainProcess(self):
        tit = ("Members Ledger Contacts Maintenance (%s)" % __name__,)
        cod = {
            "stype": "R",
            "tables": ("memctk",),
            "cols": (
                ("mck_code", "", 0, "CD"),
                ("mck_desc", "", 0, "Description", "Y"))}
        fld = (
            (("T",0,0,0),"I@mck_code",0,"","",
                "","N",self.doCode,cod,None,("notzero",)),
            (("T",0,1,0),"I@mck_type",0,"","",
                "","N",self.doType,cod,None,("in",("H","W","M","F","E"))),
            (("T",0,2,0),"I@mck_desc",0,"","",
                "","N",self.doDesc,None,self.doDelete,("notblank",)))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,2),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, butt=but,
            tend=tnd, txit=txt)

    def doCode(self, frt, pag, r, c, p, i, w):
        self.code = w
        acc = self.sql.getRec(tables="memctk", where=[("mck_code", "=",
            self.code)], limit=1)
        if acc:
            self.new = False
            self.ktyp = acc[self.sql.memctk_col.index("mck_type")]
            self.df.loadEntry(frt, pag, p+1, data=self.ktyp)
            self.desc = acc[self.sql.memctk_col.index("mck_desc")]
            self.df.loadEntry(frt, pag, p+2, data=self.desc)
        else:
            self.new = True

    def doType(self, frt, pag, r, c, p, i, w):
        self.ktyp = w

    def doDesc(self, frt, pag, r, c, p, i, w):
        self.desc = w

    def doDelete(self):
        chk = self.sql.getRec(tables="memkon", cols=["count(*)"],
            where=[("mlk_code", "=", self.code)], limit=1)
        if chk[0]:
            return "Code in Use (memkon), Not Deleted"
        self.sql.delRec("memctk", where=[("mck_code", "=", self.code)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
        data = [self.code, self.ktyp, self.desc]
        if self.new:
            self.sql.insRec("memctk", data=data)
        else:
            self.sql.updRec("memctk", data=data, where=[("mck_code", "=",
                self.code)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

