#! /usr/bin/env python

"""
SYNOPSIS
    Members Ledger Month End Routine.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, GetCtl, ProgressBar, PwdConfirm, Sql
from TartanClasses import TartanDialog
from tartanFunctions import askQuestion, callModule, dateDiff, doChkCatChg
from tartanFunctions import mthendDate, projectDate, showError

class mlm010:
    def __init__(self, mf, coy, nam, per, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = per
        self.capnm = usr
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        tabs = ["chglog", "ctlbat", "ctlmst", "ctlvmf", "ctlvtf", "memage",
            "memcat", "memctc", "memctl", "memctp", "memmst", "memtrn"]
        mc = GetCtl(self.mf)
        self.taxdf = mc.getCtl("ctlmst", self.conum)["ctm_taxdf"]
        memctl = mc.getCtl("memctl", self.conum)
        if not memctl:
            return "no"
        self.glint = memctl["mcm_glint"]
        self.ldays = memctl["mcm_ldays"]
        self.lme = memctl["mcm_lme"]
        self.lmd = memctl["mcm_lme"] / 100
        yr = self.lme / 10000
        mth = ((self.lme / 100) % 100) + 1
        if mth > 12:
            yr += 1
            mth = 1
        self.tme = mthendDate((yr * 10000) + (mth * 100) + 00)
        self.ynd = False
        if self.glint == "Y":
            tabs.append("gentrn")
            ctlctl = mc.getCtl("ctlctl", self.conum)
            if not ctlctl:
                return "no"
            ctls = ["mem_ctl", "mem_pen", "vat_ctl"]
            if mc.chkRec(self.conum, ctlctl, ctls):
                return "no"
            self.memctl = ctlctl["mem_ctl"]
            self.penctl = ctlctl["mem_pen"]
            self.vatctl = ctlctl["vat_ctl"]
        else:
            self.penctl = 0
        self.sql = Sql(self.mf.dbm, tabs, prog=__name__)
        if self.glint == "Y":
            check = self.sql.getRec(tables="memctc", cols=["count(*)"],
                where=[("mcc_cono", "=", self.conum), ("mcc_freq", "<>", "N"),
                ("mcc_glac", "=", 0)], limit=1)
            if check[0]:
                showError(self.mf.body, "Invalid Category", "There Are %s "\
                    "Category Record(s) Missing G/L Accounts" % check[0])
                return "no"
        t = time.localtime()
        self.sysdt = (t[0] * 10000) + (t[1] * 100) + t[2]

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Members Month End Routine (%s)" % __name__)
        r1s = (("No", "N"), ("Yes", "Y"))
        fld = (
            (("T",0,0,0),"OD1",10,"Last Month End Date"),
            (("T",0,1,0),"ID1",10,"This Month End Date","This Month End Date",
                self.tme,"N",self.doTme,None,None,("efld",)),
            (("T",0,2,0),("IRB",r1s),0,"Raise Penalties","Raise Penalties",
                "N","N",self.doPenalty,None,None,None),
            (("T",0,3,0),"ID1",10,"Cut-off Date","Cut-off Date",
                "","N",self.doCutOff,None,None,("efld",)))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.closeProcess,)
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, focus=False)
        self.df.loadEntry("T", 0, 0, data=self.lme)
        self.df.focusField("T", 0, 2)

    def doTme(self, frt, pag, r, c, p, i, w):
        if w <= self.lme:
            return "Invalid Month-End Date, Before Last Month End"
        if w < self.period[1][0] or w > self.period[2][0]:
            return "Invalid Month-End Date, Out of Financial Period"

        if w / 100 == self.period[2][0] / 100:
            ok = askQuestion(self.mf.body, "Year End",
                "Is this the Financial Year End?", default="yes")
            if ok == "yes":
                self.ynd = True
                w = self.period[2][0]
                self.df.loadEntry(frt, pag, p, data=w)
            else:
                return "Invalid Month-End Date, Same as Year-End"
        if dateDiff(self.lme, w, "months") <> 1:
            return "There Seems to be a Missing Month End"
        self.tme = w
        self.curdt = w / 100
        yy = self.curdt / 100
        mm = (self.curdt % 100) + 1
        while mm > 12:
            yy += 1
            mm -= 12
        self.nxtdt = (yy * 10000) + (mm * 100) + 1
        self.nxtcd = self.nxtdt / 100
        self.batch = "M%06s" % self.curdt
        chk = self.sql.getRec(tables="memtrn", cols=["count(*)"],
            where=[("mlt_cono", "=", self.conum), ("mlt_type", "=", 1),
            ("mlt_batch", "=", self.batch), ("mlt_trdt", "=", self.nxtdt),
            ("mlt_curdt", "=", self.nxtcd)], limit=1)
        if chk[0]:
            return "Month-End Already Executed"
        num = self.sql.getRec(tables="memtrn", cols=["max(mlt_refno)"],
            where=[("mlt_cono", "=", self.conum), ("mlt_type", "=", 1),
            ("mlt_refno", "like", "A________")], limit=1)
        if not num[0]:
            self.refnum = 0
        else:
            self.refnum = int(num[0][4:])

    def doPenalty(self, frt, pag, r, c, p, i, w):
        self.penalty = w
        if self.penalty == "N":
            return "sk1"

    def doCutOff(self, frt, pag, r, c, p, i, w):
        if w >= self.tme:
            return "Invalid Cut-off Date"
        self.cutoff = w
        self.pcodes = {}
        acc = self.sql.getRec(tables="memctp", cols=["mcp_type", "mcp_code",
            "mcp_penalty"], where=[("mcp_cono", "=", self.conum), ("mcp_date",
            "<=", self.cutoff), ("mcp_penalty", "<>", 0)], order="mcp_date")
        for a in acc:
            self.pcodes["%s%02i" % (a[0], a[1])] = a[2]

    def doEnd(self):
        all = self.sql.getRec(tables="ctlbat", cols=["count(*)"],
            where=[("btm_cono", "=", self.conum), ("btm_styp", "=", "MEM"),
            ("btm_ind", "=", "N"), ("btm_curdt", "=", self.curdt)], limit=1)
        if all[0]:
            ok = askQuestion(self.mf.body, "Unbalanced Batches Exist",
                "There are Unbalanced Batches for this Month. You "\
                "Should Not Continue, but Print a Batch Error Report, and "\
                "Correct the Errors. Continue, YES or NO ?", default="no")
            if ok == "yes":
                ok = PwdConfirm(self.mf, conum=self.conum, system="MST",
                    code="UnbalBatch")
                if ok.flag =="no":
                    self.closeProcess()
                else:
                    self.doEnder()
            else:
                self.closeProcess()
        else:
            self.doEnder()

    def doEnder(self):
        self.df.closeProcess()
        all = self.sql.getRec(tables="memmst", cols=["mlm_memno"],
            where=[("mlm_cono", "=", self.conum), ("mlm_state", "=", "A")])
        if self.ynd:
            p = ProgressBar(self.mf.body, typ="Checking Member Categories",
                max=len(all), esc=False)
            for num, acc in enumerate(all):
                p.displayProgress(num)
                self.memno = acc[0]
                data = doChkCatChg(self.mf, self.conum, self.memno,
                    self.period[2][0], self.nxtdt)
                if data:
                    if not data[3]:
                        self.doRaiseCharge("B", data[0], data[1], data[2],
                            data[3], data[4], data[5], data[6], skip=True)
                    dte = long("%04i%02i%02i%02i%02i%02i" % \
                        time.localtime()[:-3])
                    self.sql.delRec("memcat", where=[("mlc_cono", "=",
                        self.conum), ("mlc_memno", "=", self.memno),
                        ("mlc_type", "=", "B"), ("mlc_code", "=", data[0])])
                    self.sql.insRec("chglog", data=["memcat", "D",
                        "%03i%06i%1s%02i" % (self.conum, self.memno, "B",
                        data[0]), "", dte, self.capnm, str(data[1]),
                        str(data[2]), 0])
                    self.sql.insRec("memcat", data=[self.conum, self.memno,
                        "B", data[7], "", self.nxtdt, 0, 0])
                    self.sql.insRec("chglog", data=["memcat", "N",
                        "%03i%06i%1s%02i" % (self.conum, self.memno, "B",
                        data[7]), "", dte, self.capnm, str(self.nxtdt),
                        str(0), 0])
            p.closeProgress()
            ok = askQuestion(self.mf.body, "Category Changes",
                "Would You Like to Display Category Changes?", default="yes")
            if ok == "yes":
                callModule(self.mf, None, "ml3060", coy=(self.conum,
                    self.conam), args=(projectDate(self.lme, 1), self.nxtdt))
                ok = askQuestion(self.mf.body, "Continue",
                    "Would You Like to Continue with the Month End?",
                    default="yes")
                if ok == "no":
                    self.mf.closeLoop()
                    return
        p = ProgressBar(self.mf.body, typ="F", max=len(all), esc=False)
        for num, acc in enumerate(all):
            p.displayProgress(num)
            self.memno = acc[0]
            if self.penalty == "Y":
                amount = 0
                join = "left outer join memage on mta_cono=mlt_cono and "\
                    "mta_memno=mlt_memno and mta_type=mlt_type and "\
                    "mta_refno=mlt_refno"
                cols = ["mlt_ctyp", "mlt_ccod", "sum(mlt_tramt)"]
                wher = [("mlt_cono", "=", self.conum), ("mlt_memno",
                    "=", self.memno), ("mlt_trdt", "<=", self.cutoff)]
                grps = "mlt_ctyp, mlt_ccod"
                ordr = "mlt_trdt"
                recs = self.sql.getRec(tables="memtrn", join=join, cols=cols,
                    where=wher, group=grps, order=ordr)
                for rec in recs:
                    key = "%s%02i" % (rec[0], rec[1])
                    if key in self.pcodes:
                        amt = round(rec[2] * self.pcodes[key] / 100.0, 2)
                        if amt > 0:
                            amount = float(ASD(amount) + ASD(amt))
                if amount:
                    self.doUpdateTables("", "", "Penalty on Overdue Amounts",
                        self.penctl, self.tme, self.batch, amount)
            cols = [ "mlc_type", "mlc_code", "mcc_desc", "mcc_freq",
                "mlc_start", "mlc_end", "mlc_last", "mcc_glac"]
            wher = [
                ("mlc_cono", "=", self.conum),
                ("mlc_memno", "=", self.memno),
                ("mlc_start", ">", 0),
                ("mlc_start", "<=", self.nxtdt),
                ("mcc_cono=mlc_cono",),
                ("mcc_type=mlc_type",),
                ("mcc_code=mlc_code",),
                ("mcc_freq", "<>", "N")]
            cats = self.sql.getRec(tables=["memcat", "memctc"], cols=cols,
                where=wher, order="mlc_type, mlc_code")
            for ctyp, code, desc, freq, start, end, last, glac in cats:
                if start > self.nxtdt:
                    # Not yet Started
                    continue
                if last and end and end < self.nxtdt:
                    # Ended
                    continue
                if last and freq == "O":
                    # Once Off
                    continue
                if not self.ynd and last and freq == "A" and \
                   last >= self.period[1][0] and last <= self.period[2][0]:
                    # Out of Period
                    continue
                self.doRaiseCharge(ctyp, code, start, end, last, freq,
                    desc, glac)
        p.closeProgress()
        self.sql.updRec("memctl", cols=["mcm_lme"], data=[self.tme],
            where=[("mcm_cono", "=", self.conum)])
        ok = askQuestion(self.mf.body, "Audit Trail",
            "Would You Like to Display an Audit Trail?", default="yes")
        if ok == "yes":
            callModule(self.mf, None, "ml3020", coy=(self.conum, self.conam),
                args=("F", 0, self.nxtcd, 1, self.batch))
        self.mf.dbm.commitDbase(ask=True, mess="Do You Want To Save All "\
            "Entries?\n\nPlease Note That Once The Entries Have Been "\
            "Saved, There Is No Going Back Without Restoring From Backup!")
        if self.ynd and self.mf.dbm.commit == "yes":
            callModule(self.mf, None, "msy010", coy=(self.conum, self.conam),
                period=self.period, args=("N", True))
        self.mf.closeLoop()

    def doRaiseCharge(self, ctyp, code, start, end, last, freq, desc, glac, skip=False):
        if freq == "O":
            dte = True
            nxt = False
        else:
            dte = False
            if self.ynd or freq == "M":
                nxt = True
            else:
                nxt = False
            if not last:
                if dateDiff(start, self.tme, "days") > self.ldays:
                    dte = True
                else:
                    nxt = True
        if dte:
            trdt = start
            amt = self.doGetCharge(ctyp, code, trdt)
            if amt:
                self.doUpdateTables(ctyp, code, desc, glac, trdt, amt)
        if not skip and nxt:
            if end and self.nxtdt > end:
                return
            trdt = self.nxtdt
            amt = self.doGetCharge(ctyp, code, trdt)
            if amt:
                self.doUpdateTables(ctyp, code, desc, glac, trdt, amt)

    def doGetCharge(self, ctyp, code, date):
        prc = self.sql.getRec(tables="memctp", where=[("mcp_cono", "=",
            self.conum), ("mcp_type", "=", ctyp), ("mcp_code", "=", code),
            ("mcp_date", "<=", date)], order="mcp_date desc", limit=1)
        if not prc:
            # No Price
            return
        if prc[5] == "N" or (self.ynd and date == self.nxtdt):
            # Not Pro Rata or End of Financial Year
            amt = CCD(prc[6], "UD", 12.2).work
        else:
            # Extract Pro Rata Rate
            mths = 17 - dateDiff(date, self.period[2][0], "months")
            if mths < 1:
                mths = 12
            amt = CCD(prc[mths], "UD", 12.2).work
        if not amt:
            # No Charge
            return
        else:
            return amt

    def doUpdateTables(self, ctyp, code, desc, glac, trdt, amt):
        self.refnum += 1
        ref = CCD("A%08i" % self.refnum, "Na", 9).work
        curdt = trdt / 100
        # VAT Rate and Amount
        acc = self.sql.getRec(tables=["ctlvmf"], cols=["vtm_new"],
            where=[("vtm_cono", "=", self.conum), ("vtm_code", "=",
            self.taxdf)], limit=1)
        if not acc:
            rate = 0
        else:
            rate = float(acc[0])
        vat = CCD(round(((amt * rate) / 114), 2), "UD", 12.2).work
        # Members Ledger Transaction (memtrn)
        data = [self.conum, self.memno, 1, ref, self.batch, trdt, amt, vat,
            curdt, ctyp, code, desc, self.taxdf, "", self.sysdt, 0]
        self.sql.insRec("memtrn", data=data)
        # Members Ledger Category (memcat)
        self.sql.updRec("memcat", cols=["mlc_last"], data=[trdt],
            where=[("mlc_cono", "=", self.conum), ("mlc_memno", "=",
            self.memno), ("mlc_type", "=", ctyp), ("mlc_code", "=", code)])
        if vat:
            # VAT Transaction (ctlvtf)
            val = float(ASD(0) - ASD(amt) + ASD(vat))
            tax = float(ASD(0) - ASD(vat))
            data = [self.conum, self.taxdf, "O", curdt, "M", 1, self.batch,
                ref, trdt, self.memno, desc, val, tax, self.sysdt, 0, 0]
            self.sql.insRec("ctlvtf", data=data)
        if self.glint == "N":
            return
        ref = "ML/MthEnd"
        # General Ledger Member Control
        whr = [
            ("glt_cono", "=", self.conum),
            ("glt_acno", "=", self.memctl),
            ("glt_batch", "=", self.batch),
            ("glt_curdt", "=", curdt),
            ("glt_trdt", "=", trdt),
            ("glt_type", "=", 1)]
        glt = self.sql.getRec(tables="gentrn", where=whr)
        if glt and len(glt) == 1:
            tmp = glt[0][self.sql.gentrn_col.index("glt_tramt")]
            tmp = float(ASD(tmp) + ASD(amt))
            glt[0][self.sql.gentrn_col.index("glt_tramt")] = tmp
            self.sql.updRec("gentrn", where=whr, data=glt[0])
        else:
            data = [self.conum, self.memctl, curdt, trdt, 1, ref, self.batch,
                amt, 0, "Month End Raising Charges", "", "", 0, self.sysdt, 0]
            self.sql.insRec("gentrn", data=data)
        val = float(ASD(0) - ASD(amt) + ASD(vat))
        tax = float(ASD(0) - ASD(vat))
        # General Ledger Income Account
        whr[1] = ("glt_acno", "=", glac)
        glt = self.sql.getRec(tables="gentrn", where=whr)
        if glt and len(glt) == 1:
            tmp = glt[0][self.sql.gentrn_col.index("glt_tramt")]
            tmp = float(ASD(tmp) + ASD(val))
            glt[0][self.sql.gentrn_col.index("glt_tramt")] = tmp
            tmp = glt[0][self.sql.gentrn_col.index("glt_taxamt")]
            tmp = float(ASD(tmp) + ASD(tax))
            glt[0][self.sql.gentrn_col.index("glt_taxamt")] = tmp
            self.sql.updRec("gentrn", where=whr, data=glt[0])
        else:
            data = [self.conum, glac, curdt, trdt, 1, ref, self.batch, val,
                tax, "Month End Raising Charges", self.taxdf, "", 0,
                self.sysdt, 0]
            self.sql.insRec("gentrn", data=data)
        if not tax:
            return
        # General Ledger VAT Account
        whr = [("glt_cono", "=", self.conum), ("glt_acno", "=", self.vatctl),
            ("glt_batch", "=", self.batch), ("glt_curdt", "=", curdt),
            ("glt_trdt", "=", trdt), ("glt_type", "=", 1)]
        glt = self.sql.getRec(tables="gentrn", where=whr)
        if glt and len(glt) == 1:
            tmp = glt[0][self.sql.gentrn_col.index("glt_tramt")]
            tmp = float(ASD(tmp) + ASD(tax))
            glt[0][self.sql.gentrn_col.index("glt_tramt")] = tmp
            self.sql.updRec("gentrn", where=whr, data=glt[0])
        else:
            data = [self.conum, self.vatctl, curdt, trdt, 1, ref, self.batch,
                tax, 0.00, "Month End Raising Charges", "", "", 0, self.sysdt,
                0]
            self.sql.insRec("gentrn", data=data)

    def closeProcess(self):
        self.df.closeProcess()
        self.mf.closeLoop()

