#! /usr/bin/env python

"""
SYNOPSIS
    Module Passwords Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import RepPrt, TartanDialog
from tartanWork import finsys

class ms3020:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])

    def mainProcess(self):
        tit = ("Module Passwords Listing",)
        data = [["ALL", "All Systems"]]
        syss = finsys.keys()
        syss.sort()
        for s in syss:
            data.append((s, finsys[s][0]))
        syss.append("ALL")
        stt = {
            "stype": "C",
            "titl": "Select the Required System",
            "data": data}
        fld = (
            (("T",0,0,0),"IUA",3,"System","System",
                "ALL","Y",self.doSys,stt,None,("in", syss)),)
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"))

    def doSys(self, frt, pag, r, c, p, i, w):
        self.sys = w

    def doEnd(self):
        self.df.closeProcess()
        self.printReport()
        self.closeProcess()

    def printReport(self):
        tables = ["ctlpwr"]
        heads = ["Module Passords Master Listing"]
        cols = [
            ["pwd_cono","UI",3.0,"Coy"],
            ["pwd_sys","UA",3.0,"Sys"],
            ["pwd_code","NA",20.0,"Access Code"],
            ["pwd_desc","NA",50.0,"Description"],
            ["pwd_pass","HA",30.0,"Password"]]
        if self.sys == "ALL":
            whr = []
        else:
            whr = [("pwd_sys", "=", self.sys)]
        ord = "pwd_cono, pwd_sys, pwd_code"
        state = self.df.disableButtons()
        rp = RepPrt(self.mf, name=__name__, tables=tables, heads=heads,
            cols=cols, where=whr, order=ord, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2])
        self.df.enableButtons(state=state)

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

