#! /usr/bin/env python

"""
SYNOPSIS
    System Control Record Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import Sql, TartanDialog

class msc110:
    def __init__(self, mf, usr):
        self.mf = mf
        self.user = usr
        no = self.setVariables()
        if not no:
            self.drawDialog()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlsys", "ctlpwu", "chglog"],
            prog=__name__)
        if self.sql.error:
            return "no"
        self.acc = self.sql.getRec(tables="ctlsys", limit=1)
        if not self.acc:
            self.new = True
            self.acc = ["N", 0, 0, 0, "", "", "Y", 0]
        else:
            self.new = False

    def drawDialog(self):
        tit = "System Record Maintenance (%s)" % __name__
        r1s = (("Yes","Y"),("No","N"))
        self.fld = (
            (("T",0,0,0),("IRB",r1s),0,"Enforce Passwords","",
                self.acc[0],"N",self.doMust,None,None,None),
            (("T",0,1,0),"IUI",2,"Minimum Characters","",
                self.acc[1],"N",None,None,None,None),
            (("T",0,2,0),"IUI",2,"Maximum Life (Days)","",
                self.acc[2],"N",None,None,None,None),
            (("T",0,3,0),"IUI",3,"Backup History (Days)","",
                self.acc[3],"N",None,None,None,None),
            (("T",0,4,0),"ITX",50,"SMTP Server","",
                "","N",self.doSmtp,None,None,None),
            (("T",0,5,0),"INA",20,"SMTP Username","",
                "","N",self.doUsr,None,None,None),
            (("T",0,6,0),"IHA",20,"SMTP Password","",
                "","N",None,None,None,None),
            (("T",0,7,0),("IRB",r1s),0,"SMS Service","",
                "N","N",self.doSms,None,None,None),
            (("T",0,8,0),"INA",20,"SMS Username","",
                "","N",self.doSmsUsr,None,None,None),
            (("T",0,9,0),"IHA",20,"SMS Password","",
                "","N",self.doSmsPwd,None,None,None))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,1),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)
        if not self.new:
            for n, f in enumerate(self.acc[:-1]):
                if n < 4:
                    self.df.loadEntry("T", 0, n, data=f)
                elif n == 4:
                    u = f.split("|")
                    self.df.loadEntry("T", 0, n, data=u[0])
                    if len(u) > 1:
                        self.df.loadEntry("T", 0, n+1, data=u[1])
                    if len(u) > 2:
                        self.df.loadEntry("T", 0, n+2, data=u[2])
                elif n == 5:
                    u = f.split("|")
                    if u[0]:
                        self.df.loadEntry("T", 0, n+2, data="Y")
                        self.df.loadEntry("T", 0, n+3, data=u[0])
                        if len(u) > 1:
                            self.df.loadEntry("T", 0, n+4, data=u[1])
                else:
                    self.df.loadEntry("T", 0, n+4, data=f)

    def doMust(self, frt, pag, r, c, p, i, w):
        if w == "N":
            self.df.loadEntry(frt, pag, p+1, data=0)
            self.df.loadEntry(frt, pag, p+2, data=0)
            return "sk2"

    def doSmtp(self, frt, pag, r, c, p, i, w):
        if not w:
            self.df.loadEntry(frt, pag, p+1, data="")
            self.df.loadEntry(frt, pag, p+2, data="")
            return "sk2"

    def doUsr(self, frt, pag, r, c, p, i, w):
        if not w:
            self.df.loadEntry(frt, pag, p+1, data="")
            return "sk1"

    def doSms(self, frt, pag, r, c, p, i, w):
        if w == "N":
            self.df.loadEntry(frt, pag, p+1, data="")
            self.df.loadEntry(frt, pag, p+2, data="")
            return "sk2"

    def doSmsUsr(self, frt, pag, r, c, p, i, w):
        if not w:
            return "Invalid SMS User Name"

    def doSmsPwd(self, frt, pag, r, c, p, i, w):
        if not w:
            return "Invalid SMS Password"

    def doEnd(self):
        data = []
        for x in range(0, 4):
            data.append(self.df.t_work[0][0][x])
        smtp = self.df.t_work[0][0][4]
        if self.df.t_work[0][0][5]:
            smtp = "%s|%s" % (smtp, self.df.t_work[0][0][5])
        if self.df.t_work[0][0][6]:
            smtp = "%s|%s" % (smtp, self.df.t_work[0][0][6])
        data.append(smtp)
        if self.df.t_work[0][0][7] == "Y":
            sms = "%s|%s" % (self.df.t_work[0][0][8], self.df.t_work[0][0][9])
        else:
            sms = ""
        data.append(sms)
        data.append(self.acc[-1])
        tme = time.localtime()
        if self.new:
            self.sql.insRec("ctlsys", data=data)
        elif data != self.acc:
            self.sql.updRec("ctlsys", data=data)
            dte = long("%04i%02i%02i%02i%02i%02i" % tme[:-3])
            for num, dat in enumerate(self.acc):
                if dat != data[num]:
                    self.sql.insRec("chglog", data=["ctlsys", "U",
                        "%03i" % 0, self.sql.ctlsys_col[num], dte,
                        self.user, str(dat), str(data[num]), 0])
        # Reset all password dates to current date (Temporary Fix)
        dte = (tme[0] * 10000) + (tme[1] * 100) + tme[2]
        self.sql.updRec("ctlpwu", cols=["usr_last"], data=[dte])
        self.mf.dbm.commitDbase()
        self.doExit()

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

