#! /usr/bin/env python

"""
SYNOPSIS
    Rental Tenants Transaction Audit Trail.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError
from tartanWork import rttrtp, rcmvtp

class rc3020:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["rcatnt", "rcatnm"], prog=__name__)
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.curdt = self.sysdtw / 100
        self.totind = "N"
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Rental Tenants Audit Trail (%s)" % __name__)
        data = ["All Types"]
        for x in range(1, len(rttrtp)+1):
            data.append(rttrtp[x-1][1])
        btt = {
            "stype": "C",
            "titl": "Select the Required Type",
            "data": data,
            "retn": "I"}
        btm = {
            "stype": "R",
            "tables": ("rcatnt",),
            "cols": (
                ("rtu_batch", "", 0, "Bat-Num"),
                ("rtu_type", ("xx", rttrtp), 20, "Type"),
                ("rtu_curdt", "", 0, "Cur-Dat")),
            "where": [],
            "group": "rtu_batch, rtu_type, rtu_curdt",
            "order": "rtu_type, rtu_curdt, rtu_batch"}
        r1s = (("Financial","F"),("Capture","C"))
        r2s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Period Type","Period Type",
                "F","Y",self.doPtyp,None,None,None),
            (("T",0,1,0),"Id2",7,"Starting Period","Starting Period",
                self.curdt,"Y",self.doStartPer,None,None,("efld",)),
            (("T",0,2,0),"Id2",7,"Ending Period","Ending Period",
                self.curdt,"Y",self.doEndPer,None,None,("efld",)),
            (("T",0,3,0),"Id1",10,"Starting Date","Starting Date",
                self.sysdtw,"Y",self.doStartDat,None,None,("efld",)),
            (("T",0,4,0),"Id1",10,"Ending Date","Ending Date",
                self.sysdtw,"Y",self.doEndDat,None,None,("efld",)),
            (("T",0,5,0),"IUI",1,"Type","Transaction Type",
                "","Y",self.doBatTyp,btt,None,None),
            (("T",0,6,0),"INa",7,"Batch-Number","Batch Number",
                "","Y",self.doBatNum,btm,None,None),
            (("T",0,7,0),("IRB",r2s),0,"Totals Only (Y/N)","Totals Only (Y/N)",
                "Y","Y",self.doTots,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doPtyp(self, frt, pag, r, c, p, i, w):
        self.ptyp = w
        if self.ptyp == "C":
            return "sk2"

    def doStartPer(self, frt, pag, r, c, p, i, w):
        if w > self.curdt:
            return "Invalid Period, After To-Day"
        else:
            self.sperw = w
        self.sperd = self.df.t_disp[pag][0][i]
        self.df.loadEntry(frt, pag, p, data=self.sperw)
        self.df.topf[pag][i+1][5] = self.sperw

    def doEndPer(self, frt, pag, r, c, p, i, w):
        if w == 0:
            self.eperw = self.sperw
        elif w < self.sperw:
            return "Invalid Period, Before Start"
        else:
            self.eperw = w
        self.eperd = self.df.t_disp[pag][0][i]
        self.df.loadEntry(frt, pag, p, self.eperw)
        return "sk2"

    def doStartDat(self, frt, pag, r, c, p, i, w):
        if w > self.sysdtw:
            return "Invalid Date, After To-Day"
        else:
            self.sdatw = w
        self.sperd = self.df.t_disp[pag][0][i]
        self.df.topf[pag][i+1][5] = self.sdatw

    def doEndDat(self, frt, pag, r, c, p, i, w):
        if w == 0:
            self.edatw = self.sdatw
        elif w < self.sdatw:
            return "Invalid Date, Before Start"
        else:
            self.edatw = w
        self.eperd = self.df.t_disp[pag][0][i]
        self.df.loadEntry(frt, pag, p, self.edatw)

    def doBatTyp(self, frt, pag, r, c, p, i, w):
        if w > len(rttrtp):
            return "Invalid Batch Type"
        self.btyp = w
        self.whr = [("rtu_cono", "=", self.conum)]
        if self.ptyp == "F":
            self.whr.append(("rtu_curdt", "between", self.sperw, self.eperw))
        else:
            self.whr.append(("rtu_capdt", "between", self.sdatw, self.edatw))
        if self.btyp:
            self.whr.append(("rtu_type", "=", w))
        self.df.topf[pag][i+1][8]["where"] = self.whr

    def doBatNum(self, frt, pag, r, c, p, i, w):
        self.batch = w

    def doTots(self, frt, pag, r, c, p, i, w):
        self.totsonly = w

    def doEnd(self):
        self.df.closeProcess()
        tab = ["rcatnm", "rcatnt"]
        self.col = ["rtu_owner", "rtu_code", "rtu_acno", "rtu_type",
            "rtu_batch", "rtu_refno", "rtu_trdt", "rtu_desc", "rtu_mtyp",
            "rtu_tramt", "rtu_taxamt", "rtu_taxind", "rtn_name"]
        whr = self.whr[:]
        whr.extend([("rtn_cono=rtu_cono",), ("rtn_owner=rtu_owner",),
            ("rtn_code=rtu_code",), ("rtn_acno=rtu_acno",)])
        ord = "rtu_type, rtu_batch, rtu_refno"
        all = self.sql.getRec(tables=["rcatnm", "rcatnt"], cols=self.col,
            where=whr, order=ord)
        if not all:
            showError(self.mf.body, "Transaction Error",
            "No Transactions Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        if self.totsonly == "Y":
            self.head = ("%03u %-30s %33s %6s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        else:
            self.head = ("%03u %-32s %30s %33s %31s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.bqty = 0
        self.bamt = 0
        self.bdis = 0
        self.bvat = 0
        self.tqty = 0
        self.tamt = 0
        self.tdis = 0
        self.tvat = 0
        self.gqt = [0] * (len(rttrtp) + 1)
        self.gam = [0] * (len(rttrtp) + 1)
        self.gvt = [0] * (len(rttrtp) + 1)
        self.trtp = 0
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            owner = CCD(dat[self.col.index("rtu_owner")], "NA", 7)
            code = CCD(dat[self.col.index("rtu_code")], "NA", 7)
            acno = CCD(dat[self.col.index("rtu_acno")], "NA", 7)
            name = CCD(dat[self.col.index("rtn_name")], "NA", 30)
            trtp = CCD(dat[self.col.index("rtu_type")], "UI", 2)
            batch = CCD(dat[self.col.index("rtu_batch")], "Na", 7)
            refno = CCD(dat[self.col.index("rtu_refno")], "Na", 9)
            trdt = CCD(dat[self.col.index("rtu_trdt")], "D1", 10)
            desc = CCD(dat[self.col.index("rtu_desc")], "NA", 30)
            mtyp = CCD(dat[self.col.index("rtu_mtyp")], "UI", 1)
            tramt = CCD(dat[self.col.index("rtu_tramt")], "SD", 13.2)
            taxamt = CCD(dat[self.col.index("rtu_taxamt")], "SD", 13.2)
            taxind = CCD(dat[self.col.index("rtu_taxind")], "NA", 1)
            if not self.trtp:
                self.trtp = trtp.work
                self.batch = batch.work
            if trtp.work != self.trtp:
                self.batchTotal()
                self.typeTotal()
                self.trtp = trtp.work
                self.batch = batch.work
                self.pglin = 99
            if batch.work != self.batch:
                self.batchTotal()
                self.batch = batch.work
                if self.totsonly != "Y":
                    self.typeHeading()
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            if self.totsonly != "Y":
                self.t.textLine("%s %s %s %s %s %s %s %s %s %s %s" % \
                    (owner.disp, code.disp, acno.disp, refno.disp, trdt.disp,
                    desc.disp, rcmvtp[mtyp.work-1][0], tramt.disp, taxamt.disp,
                    taxind.disp, name.disp))
                self.pglin = self.pglin + 1
            self.bqty = self.bqty + 1
            self.bamt = float(ASD(self.bamt) + ASD(tramt.work))
            self.bvat = float(ASD(self.bvat) + ASD(taxamt.work))
            self.tqty = self.tqty + 1
            self.tamt = float(ASD(self.tamt) + ASD(tramt.work))
            self.tvat = float(ASD(self.tvat) + ASD(taxamt.work))
            self.gqt[trtp.work -1] = self.gqt[trtp.work -1] + 1
            self.gam[trtp.work -1] = float(ASD(self.gam[trtp.work -1]) + \
                ASD(tramt.work))
            self.gvt[trtp.work -1] = float(ASD(self.gvt[trtp.work -1]) + \
                ASD(taxamt.work))
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.batchTotal()
                self.typeTotal()
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        if self.totsonly == "Y":
            self.t.textLine(
            "%-37s %-10s %-2s %-10s %6s %5s" % \
            ("Rental Tenants Audit Trail for Period",
            self.sperd, "to", self.eperd, "Page", self.pgnum))
        else:
            self.t.textLine(
            "%-37s %-10s %-2s %-10s %71s %5s" % \
            ("Rental Tenants Audit Trail for Period",
            self.sperd, "to", self.eperd, "Page", self.pgnum))
        self.t.textLine()
        self.pglin = 4
        if self.totind == "N":
            self.typeHeading()
        else:
            self.t.textLine("%-14s" % "Totals Summary")
            self.t.textLine()
            if self.totsonly == "Y":
                self.t.textLine(
                "%-39s%-8s  %-13s %-13s" % ("Document Type", "Quantity",
                "      Amount", "      V.A.T."))
            else:
                self.t.textLine(
                "%-26s%-8s  %-13s %-13s" % ("Document Type", "Quantity ",
                "      Amount", "      V.A.T."))
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.t.setFont("Tartan", self.cpi, self.ldg)
            self.pglin = self.pglin + 4

    def typeHeading(self):
        if self.totsonly == "N":
            batch = self.batch
        else:
            batch = "Various"
        if self.lpp - self.pglin < 7:
            self.c.drawText(self.t)
            self.c.showPage()
            self.pageHeading()
            return
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%-7s %7s %-10s %3s" % \
            ("Batch", batch, "    Type", rttrtp[self.trtp - 1][1]))
        self.t.textLine()
        if self.totsonly == "Y":
            self.t.textLine(
                "%-48s %-13s %-13s" % \
                ("Details", "      Amount", "  Tax-Amount"))
        else:
            self.t.textLine(
                "%-7s %-7s %-7s %-9s %-10s %-30s %-3s %-13s %-13s %-1s "\
                "%-30s" % ("Own-Cod", "Prm-Cod", "Acc-Num", "Reference",
                "   Date", "Remarks", "Mtp", "      Amount", "  Tax-Amount",
                "I", "Tenant"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.pglin = self.pglin + 4
        self.t.setFont("Tartan", self.cpi, self.ldg)

    def batchTotal(self):
        j = CCD(self.bamt, "SD", 13.2)
        k = CCD(self.bvat, "SD", 13.2)
        if self.totsonly == "Y":
            self.t.textLine("%-48s %13s %13s" % \
                ("Batch " + self.batch + " Totals", j.disp, k.disp))
        else:
            self.t.textLine()
            self.pglin = self.pglin + 1
            self.t.textLine("%-44s %-34s %13s %13s" % (" ",
                "Batch " + self.batch + " Totals", j.disp, k.disp))
        self.pglin = self.pglin + 1
        if self.totsonly != "Y":
            self.t.textLine()
            self.pglin = self.pglin + 1
        self.bqty = 0
        self.bamt = 0
        self.bdis = 0
        self.bvat = 0

    def typeTotal(self):
        j = CCD(self.tamt, "SD", 13.2)
        k = CCD(self.tvat, "SD", 13.2)
        if self.totsonly == "Y":
            self.t.textLine()
            self.t.textLine("%-48s %13s %13s" % \
                ("Type Totals", j.disp, k.disp))
        else:
            self.t.textLine("%-44s %-34s %13s %13s" % \
            (" ", "Type-Totals", j.disp, k.disp))
        self.t.textLine()
        self.tqty = 0
        self.tamt = 0
        self.tdis = 0
        self.tvat = 0
    
    def grandTotal(self):
        self.c.drawText(self.t)
        self.c.showPage()
        self.totind = "Y"
        self.pageHeading()
        tot = [0,0,0,0]
        for x in range(0, len(rttrtp)):
            qt = CCD(self.gqt[x], "SI", 8)
            am = CCD(self.gam[x], "SD", 13.2)
            vt = CCD(self.gvt[x], "SD", 13.2)
            if self.totsonly == "Y":
                self.t.textLine("%-39s %s %s %s" % \
                (rttrtp[x][1], qt.disp, am.disp, vt.disp))
            else:
                self.t.textLine("%-26s %s %s %s" % \
                (rttrtp[x][1], qt.disp, am.disp, vt.disp))
            tot[0] = tot[0] + qt.work
            tot[1] = float(ASD(tot[1]) + ASD(am.work))
            tot[3] = float(ASD(tot[3]) + ASD(vt.work))
        self.t.textLine()
        qt = CCD(tot[0], "SI", 8)
        am = CCD(tot[1], "SD", 13.2)
        vt = CCD(tot[3], "SD", 13.2)
        if self.totsonly == "Y":
            self.t.textLine("%-39s %s %s %s" % \
            ("Grand Totals", qt.disp, am.disp, vt.disp))
        else:
            self.t.textLine("%-26s %s %s %s" % \
            ("Grand Totals", qt.disp, am.disp, vt.disp))
        self.t.textLine()

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

