#! /usr/bin/env python

"""
SYNOPSIS
    Rental Owners Master Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, GetCtl, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError

class rc3030:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["rcaowm", "rcaowt"], prog=__name__)
        gc = GetCtl(self.mf)
        rcactl = gc.getCtl("rcactl", self.conum)
        if not rcactl:
            return "no"
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Rental Owners Master Report (%s)" % __name__)
        r1s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),"Id2",7,"Cut Off Period","Cut Off Period",
                (self.sysdtw/100),"Y",self.doCutOff,None,None,None),
            (("T",0,1,0),("IRB",r1s),0,"Totals Only","Totals Only",
                "Y","Y",self.doTots,None,None,None),
            (("T",0,2,0),("IRB",r1s),0,"Ignore Zero Balances",
                "Ignore Zero Balances","Y","Y",self.doZero,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doCutOff(self, frt, pag, r, c, p, i, w):
        self.cutoff = w
        self.period = self.df.t_disp[pag][r][p]

    def doTots(self, frt, pag, r, c, p, i, w):
        self.totsonly = w
        if self.totsonly == "Y":
            self.zero = "Y"
            self.df.loadEntry(frt, pag, p+1, data=self.zero)
            return "sk1"

    def doZero(self, frt, pag, r, c, p, i, w):
        self.zero = w

    def doEnd(self):
        self.df.closeProcess()
        all = self.sql.getRec(tables="rcaowm", cols=["rom_acno", "rom_name"],
            where=[("rom_cono", "=", self.conum)], order="rom_acno")
        if not all:
            showError(self.mf.body, "Selection Error",
            "No Accounts Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %33s %6s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        pdfnam = getModName(self.mf.wrkdir, __name__, self.conum, ext="pdf")
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, pdfnam, self.head)
        self.tots = 0
        self.pgnum = 0
        self.pglin = 99
        for x in range(0, len(all)):
            can = p.displayProgress(x)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            acc = CCD(all[x][0], "NA", 7)
            name = CCD(all[x][1], "NA", 30)
            bals = self.sql.getRec(tables="rcaowt", cols=["sum(rot_tramt)"],
                where=[("rot_cono", "=", self.conum), ("rot_acno", "=",
                acc.work), ("rot_curdt", "<=", self.cutoff)], limit=1)
            if self.zero == "Y" and not bals[0]:
                continue
            bal = CCD(bals[0], "SD", 13.2)
            if self.totsonly != "Y":
                if self.pglin > self.lpp:
                    if self.pgnum:
                        self.c.drawText(self.t)
                        self.c.showPage()
                    self.pageHeading()
                self.t.textLine("%s %-53s %s" % (acc.disp, name.disp, bal.disp))
                self.pglin = self.pglin + 1
            self.tots = float(ASD(self.tots) + ASD(bal.work))
        p.closeProgress()
        if self.totsonly == "Y" and can != "yes":
            self.pageHeading()
            self.grandTotal()
            self.c.drawText(self.t)
        elif self.t:
            if can != "yes":
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-33s %-7s %27s %5s" % ("Rental Owners Master "\
            "Report up to", self.period, "Page", self.pgnum))
        self.t.textLine()
        self.t.textLine("%-27s%-1s%-1s" % ("(Options: "\
            "Ignore-Zero-Bal: ", self.df.t_disp[0][0][2], ")"))
        self.t.textLine()
        self.t.textLine("%-7s %-53s %13s" % ("Acc-Num", "Name", "Balance "))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def grandTotal(self):
        self.t.textLine()
        t = CCD(self.tots, "SD", 13.2)
        self.t.textLine("%-7s %-53s %13s" % (" ", "Grand Totals", t.disp))
        self.t.textLine()

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

