#! /usr/bin/python

"""
SYNOPSIS
    Rentals Ledger Deposit Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, ProgressBar, RepPrt, Sql, TartanDialog
from tartanFunctions import getDeposit

class rc3090:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["rcatnm", "rcacon"], prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Rental's Ledger Outstanding Deposits (%s)" % __name__)
        r1s = (("Number","N"), ("Name","N"))
        fld = (
            (("T",0,0,0),"ID1",10,"Report Date","",
                self.sysdtw,"N",self.doDate,None,None,("efld",)),)
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"))

    def doDate(self, frt, pag, r, c, p, i, w):
        self.date = w
        self.datep = self.df.t_disp[pag][0][p]

    def doSort(self, frt, pag, r, c, p, i, w):
        self.sort = w

    def doEnd(self):
        self.df.closeProcess()
        all = self.sql.getRec(tables="rcacon", where=[("rcc_cono", "=",
            self.conum), ("rcc_start", "<=", self.date), ("rcc_cnum", "=", 1)],
            order="rcc_code, rcc_acno, rcc_cnum")
        if not all:
            showError(self.mf.body, "Selection Error", "No Records Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        con = self.sql.rcacon_col
        data = []
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                self.t.textLine()
                self.t.textLine("Print Job ABORTED")
                break
            owner = dat[con.index("rcc_owner")]
            code = dat[con.index("rcc_code")]
            acno = dat[con.index("rcc_acno")]
            dep = getDeposit(self.mf, self.conum, self.date, owner, code,
                acno)
            if not dep:
                continue
            dept = 0
            intr = 0
            admn = 0
            for d in dep:
                dept = float(ASD(dept) + ASD(d[1]))
                intr = float(ASD(intr) + ASD(d[4]))
                admn = float(ASD(admn) + ASD(d[6]))
            balance = float(ASD(dept) + ASD(intr))
            name = self.sql.getRec(tables="rcatnm", cols=["rtn_name"],
                where=[("rtn_cono", "=", self.conum), ("rtn_owner", "=", owner),
                ("rtn_code", "=", code), ("rtn_acno", "=", acno)], limit=1)[0]
            if dept:
                data.append([code, acno, name, dept, intr, balance, admn])
        p.closeProgress()
        head = ["Rental Tenants Outstanding Deposits as at %s" % self.datep]
        cols = [
            ["aa", "NA",  7,   "Pr-Code",  "y"],
            ["bb", "NA",  7,   "Acc-Num",  "y"],
            ["cc", "NA", 40,   "Name",     "y"],
            ["dd", "SD", 13.2, "Deposit",  "y"],
            ["ee", "SD", 13.2, "Interest", "y"],
            ["ff", "SD", 13.2, "Balance",  "y"],
            ["gg", "SD", 13.2, "Admin",  "y"]]
        RepPrt(self.mf, name=__name__, tables=data, heads=head, cols=cols,
            gtots=["dd", "ee", "ff", "gg"], ttype="D",
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

