#! /usr/bin/env python

"""
SYNOPSIS
    Rentals Owners Interrogation.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, CCD, NotesCreate, SChoice, Sql
from TartanClasses import SRec, TartanDialog
from tartanFunctions import getModName, doPrinter
from tartanWork import rttrtp

class rc4010:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.capnm = usr
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["rcaowm", "rcaowt"], prog=__name__)
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) +  (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
                "Rental Ledger Interrogation (%s)" % __name__)
        own = {
            "stype": "R",
            "tables": ("rcaowm",),
            "cols": (
                ("rom_acno", "", 0, "Own-Code"),
                ("rom_name", "", 0, "Name", "Y")),
            "where": [("rom_cono", "=", self.conum)]}
        tag = (
            ("Owner",self.doTagSelect,("T",0,2),("T",0,1)),
            ("Balances",self.doTagSelect,("T",0,2),("T",0,1)),
            ("Transactions",self.doTrans,("T",0,2),("T",0,1)))
        fld = (
            (("T",0,0,0),"INA",7,"Account","Account Code",
                "","N",self.doAccount,own,None,("notblank",)),
            (("T",0,0,0),"ONA",30,"Name"),
            (("T",1,0,0),"ONA",30,"Address-1"),
            (("T",1,1,0),"ONA",30,"Address-2"),
            (("T",1,2,0),"ONA",30,"Address-3"),
            (("T",1,3,0),"ONA",4,"Postal Code"),
            (("T",1,4,0),"ONA",20,"Home Number"),
            (("T",1,5,0),"ONA",20,"Office Number"),
            (("T",1,6,0),"ONA",20,"Mobile Number"),
            (("T",1,7,0),"ONA",20,"Fax Number"),
            (("T",1,8,0),"OTX",50,"E-Mail Address"),
            (("T",1,9,0),"ONA",10,"VAT Number"),
            (("T",1,10,0),"OUA",1,"VAT Default"),
            (("T",1,11,0),"ONA",20,"Bank Name"),
            (("T",1,12,0),"ONA",8,"Branch IBT"),
            (("T",1,13,0),"ONA",16,"Bank Number"),
            (("C",2,0,0),"OSD",11.2,"Value","",
                "","N",None,None,None,None,("Details",5)))
        but = (
            ("Clear",None,self.doClear,0,("T",0,0),("T",0,1)),
            ("Notes",None,self.doNotes,0,("T",0,0),("T",0,1)),
            ("Print",None,self.doPrint,0,("T",0,0),("T",0,1)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEndTop, "N"), None, None, None)
        txt = (self.doExit, None, None, None)
        self.df = TartanDialog(self.mf, title=self.tit, tags=tag, eflds=fld,
            butt=but, tend=tnd, txit=txt)
        self.df.colLabel[2][0].set_text("Rentals ")
        self.df.colLabel[2][1].set_text("Receipts")
        self.df.colLabel[2][2].set_text("Payments")
        self.df.colLabel[2][3].set_text("Journals")
        self.df.colLabel[2][4].set_text("Balance ")

    def doAccount(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="rcaowm", where=[("rom_cono", "=",
            self.conum), ("rom_acno", "=", w)], limit=1)
        if not acc:
            return "Invalid Owner"
        self.acno = w
        self.name = acc[self.sql.rcaowm_col.index("rom_name")]
        self.df.loadEntry(frt, pag, p+1, data=self.name)
        for num, fld in enumerate(acc[3:]):
            self.df.loadEntry("T", 1, num, data=fld)
        self.loadBalances()
        self.mf.updateStatus("")

    def doEndTop(self):
        self.df.last[0] = [0, 0]
        self.df.setWidget(self.df.topEntry[0][0], state="disabled")

    def loadBalances(self):
        rtl = 0
        rec = 0
        pmt = 0
        jnl = 0
        bal = self.sql.getRec(tables="rcaowt", cols=["rot_type",
            "round(sum(rot_tramt), 2)"], where=[("rot_cono", "=", self.conum),
            ("rot_acno", "=", self.acno)], group="rot_type", order="rot_type")
        if bal:
            for a in bal:
                if a[0] == 1:
                    rtl = a[1]
                if a[0] == 2:
                    rec = a[1]
                if a[0] == 3:
                    pmt = a[1]
                if a[0] == 4:
                    jnl = a[1]
        bal = float(ASD(rtl) + ASD(rec) + ASD(pmt) + ASD(jnl))
        self.df.loadEntry("C", 2, 0, data=rtl)
        self.df.loadEntry("C", 2, 1, data=rec)
        self.df.loadEntry("C", 2, 2, data=pmt)
        self.df.loadEntry("C", 2, 3, data=jnl)
        self.df.loadEntry("C", 2, 4, data=bal)
        self.trn = self.sql.getRec(tables="rcaowt", where=[("rot_cono", "=",
            self.conum), ("rot_acno", "=", self.acno)],
            order="rot_trdt, rot_type")

    def doTagSelect(self):
        self.mf.updateStatus("")

    def doTrans(self, widget=None):
        tit = "Transactions for Account: %s - %s" % (self.acno, self.name)
        tab = ["rcaowt"]
        col = (
            ("rot_trdt", "", 0, "   Date"),
            ("rot_curdt", "", 0, "Curr-Dt"),
            ("rot_batch", "", 0, "Batch"),
            ("rot_type", ("XX", rttrtp), 3, "Typ"),
            ("rot_refno", "", 0, "Reference", "Y"),
            ("rot_tramt", "", 0, "     Amount"),
            ("rot_taxamt", "", 0, " VAT-Amount"),
            ("rot_desc", "", 0, "Details"))
        whr = [("rot_cono", "=", self.conum), ("rot_acno", "=", self.acno)]
        ord = "rot_trdt, rot_type"
        state = self.df.disableButtons()
        rec = SRec(self.mf, screen=self.df.nb.Page3, title=tit, tables=tab,
            cols=col, where=whr, order=ord, rows=12)
        self.df.enableButtons(state=state)
        self.df.selPage("Owner")

    def doNotes(self, widget=None):
        state = self.df.disableButtons()
        self.df.mstAlw.hide()
        nf = NotesCreate(self.mf, self.conum, self.conam, self.capnm,
            "RCA", self.acno)
        self.df.mstAlw.show()
        self.df.enableButtons(state=state)

    def doClear(self, widget=None):
        self.df.selPage("Owner")
        self.df.focusField("T", 0, 1)

    def doPrint(self, widget=None):
        titl = "Select the required print option!"
        data = (
            ("Info Only", "I"),
            ("Transactions Only", "T"),
            ("Info and Transactions", "A"))
        ss = SChoice(self.mf, titl=titl, data=data)
        self.mf.updateStatus("")
        if not ss.selection:
            self.pr_opts = " "
        else:
            self.pr_opts = ss.selection[1]
        if self.pr_opts != " ":
            self.printDetail()
        if self.df.topq[self.df.pag]:
            self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def printDetail(self):
        self.head = ("%03u %-30s %59s %10s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        from tartanFunctions import doReportlab
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        if self.pr_opts != "T":
            self.pageHeading()
            self.printInfo()
        if self.pr_opts == "I":
            self.c.drawText(self.t)
            self.c.showPage()
            self.c.save()
            doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                head=self.tit, view="V")
        elif self.pr_opts == "A":
            if not self.trn:
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")
            else:
                self.c.drawText(self.t)
                self.c.showPage()
                self.pageHeading()
                self.pageHeadingTrans()
                self.printTrans()
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")
        elif self.pr_opts == "T":
            if self.trn:
                self.pageHeading()
                self.pageHeadingTrans()
                self.printTrans()
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")

    def printInfo(self):
        for x in range(0, len(self.df.topf[0])):
            self.t.textLine("%-26s%-25s %s" % ("",
                self.df.topf[0][x][3], self.df.t_disp[0][0][x]))
        self.t.textLine()
        for x in range(0, len(self.df.topf[1])):
            self.t.textLine("%-26s%-25s %s" % ("",
                self.df.topf[1][x][3], self.df.t_disp[1][0][x]))
        self.t.textLine()
        self.pglin = self.pglin + 3 + len(self.df.topf[2])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%26s%-34s %13s" % ("", "Details","Amount "))
        self.t.textLine("%26s%s" % ("", self.suc * 48))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        desc = (
            "Rentals ",
            "Receipts",
            "Payments",
            "Journals",
            "Balance ")
        for n, d in enumerate(self.df.c_disp[2]):
            self.t.textLine("%26s%-34s %13s" % ("", desc[n], d[0]))

    def printTrans(self):
        col = self.sql.rcaowt_col
        bal = 0
        for ct in self.trn:
            trdt = CCD(ct[col.index("rot_trdt")], "D1", 10)
            refno = CCD(ct[col.index("rot_refno")], "Na", 9)
            trtp = CCD(ct[col.index("rot_type")], "UI", 1)
            batch = CCD(ct[col.index("rot_batch")], "Na", 7)
            desc = CCD(ct[col.index("rot_desc")], "NA", 30)
            tramt = ct[col.index("rot_tramt")]
            if tramt < 0:
                debit = CCD(0, "SD", 13.2)
                credit = CCD(tramt, "SD", 13.2)
            else:
                debit = CCD(tramt, "SD", 13.2)
                credit = CCD(0, "SD", 13.2)
            bal = float(ASD(bal) + ASD(tramt))
            tot = CCD(bal, "SD", 13.2)
            taxamt = CCD(ct[col.index("rot_taxamt")], "SD", 13.2)
            if self.pglin > self.lpp:
                if self.pgnum != 0:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
                self.pageHeadingTrans()
            self.t.textLine("%s %s %s %s %s %s %s %s %s" % (trdt.disp,
                refno.disp, rttrtp[(trtp.work - 1)][0], batch.disp,
                desc.disp, debit.disp, credit.disp, tot.disp, taxamt.disp))
            self.pglin = self.pglin + 1

    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-34s %-10s %53s %5s" % \
            ("Rentals Ledger Interrogation as at", self.sysdtd, "Page",
            self.pgnum))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 6

    def pageHeadingTrans(self):
        self.t.textLine("%-16s %s %s %s %s %s" % ("",
            self.df.topf[0][0][3], self.df.t_disp[0][0][0], "",
            self.df.topf[0][1][3], self.df.t_disp[0][0][1]))
        self.t.textLine()
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%-10s %-9s %-3s %-7s %-30s %-13s %-13s %-13s" % \
            ("   Date", "Reference", "Typ", "Batch", "Remarks",
            "       Debit", "      Credit",  "     Balance"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    from TartanClasses import MainFrame
    mf = MainFrame()
    mf.dbm.openDbase()
    rc4010(mf, 1, "Company 1", "paul")
