#! /usr/bin/python

"""
SYNOPSIS
    CSV File Imports.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import TartanDialog, Sql
from tartanFunctions import askQuestion

class rc6010:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlmst", "rcaowm", "rcaprm", "rcatnm",
            "rcacon"], prog=__name__)
        self.commit = False

    def mainProcess(self):
        tit = ("%s" % "CSV File Imports",)
        fle = {
            "stype": "F",
            "types": "fle",
            "ftype": (("CSV Files", "*.csv"),)}
        coy = {
            "stype": "R",
            "tables": ("ctlmst",),
            "cols": (
                ("ctm_cono", "", 0, "Coy"),
                ("ctm_name", "", 0, "Name", "Y"))}
        r1s = (("Owner", "O"), ("Premises", "P"), ("Tenants", "T"))
        r2s = (("Yes", "Y"), ("No", "N"))
        fld = (
            (("T",0,0,0),"INA",50,"File Name","File Name",
                "","N",self.doFile,fle,None,("fle",)),
            (("T",0,1,0),("IRB",r1s),0,"File Type","File Type",
                "O","N",self.doType,None,None,None),
            (("T",0,2,0),"IUI",3,"Company Number","Company Number",
                1,"N",self.doCono,coy,None,("notzero",)),
            (("T",0,3,0),("IRB",r2s),0,"Delete Existing","Delete Existing",
                "N","N",self.doDelete,None,None,None))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt)

    def doFile(self, frt, pag, r, c, p, i, w):
        self.file = w

    def doType(self, frt, pag, r, c, p, i, w):
        self.rtyp = w

    def doCono(self, frt, pag, r, c, p, i, w):
        self.cono = w
        acc = self.sql.getRec(tables="ctlmst", where=[("ctm_cono", "=",
            self.cono)], limit=1)
        if not acc:
            return "Invalid Company"

    def doDelete(self, frt, pag, r, c, p, i, w):
        self.delete = w

    def doEnd(self):
        if self.delete == "Y":
            if self.rtyp == "O":
                self.sql.delRec("rcaowm", where=[("rom_cono", "=", self.cono)])
            elif self.rtyp == "P":
                self.sql.delRec("rcaprm", where=[("rcp_cono", "=", self.cono)])
            if self.rtyp == "T":
                self.sql.delRec("rcatnm", where=[("rtn_cono", "=", self.cono)])
                self.sql.delRec("rcacon", where=[("rcc_cono", "=", self.cono)])
        data = open(self.file, "r")
        raw = data.readlines()
        for line in raw:
            for c in (10, 12, 13):
                line = line.replace(chr(c), "")
            line = line.split(",")
            if self.rtyp == "O":
                if len(line) >  6:
                    yes = askQuestion(self.mf.body, "Owner Error",
                        "%s\n\nContinue??????" % line)
                    if yes == "no":
                        self.mf.dbm.rollbackDbase()
                        break
                else:
                    data = [self.cono]
                    data.extend(line)
                    data.extend(["","","","","","","","","",0])
                    self.sql.insRec("rcaowm", data=data)
                    self.commit = True
            elif self.rtyp == "P":
                if len(line) >  7:
                    yes = askQuestion(self.mf.body, "Premises Error",
                        "%s\n\nContinue??????" % line)
                    if yes == "no":
                        self.mf.dbm.rollbackDbase()
                        break
                else:
                    data = [self.cono]
                    data.extend(line)
                    data.append(0)
                    self.sql.insRec("rcaprm", data=data)
                    self.commit = True
            elif self.rtyp == "T":
                if len(line) >  13:
                    yes = askQuestion(self.mf.body, "Tenant Error",
                        "%s\n\nContinue??????" % line)
                    if yes == "no":
                        self.mf.dbm.rollbackDbase()
                        break
                else:
                    data = [self.cono]
                    data.extend(line[:4])
                    data.extend(["","","",""])
                    data.extend(line[4:6])
                    data.append("")
                    self.sql.insRec("rcatnm", data=data)
                    data = [self.cono]
                    data.extend(line[:3])
                    data.append(1)
                    data.extend(line[8:12])
                    data.append(0)
                    data.append(line[12])
                    self.sql.insRec("rcacon", data=data)
                    self.commit = True
        yes = askQuestion(self.mf.body, "More",
            "Would You Like To Import Another File?")
        if yes == "no":
            self.doExit()
        else:
            self.df.focusField("T", 0, 1)

    def doExit(self):
        if self.commit:
            self.mf.dbm.commitDbase(ask=True)
        self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    from TartanClasses import MainFrame
    mf = MainFrame()
    mf.dbm.openDbase()
    rpt = rc6010(mf)
