#! /usr/bin/env python

"""
SYNOPSIS
    Rental Interest File Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import CCD, FileImport, Sql, TartanDialog
from tartanFunctions import showError

class rcc310:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.user = usr
        no = self.setVariables()
        if not no:
            self.drawDialog()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["rcaint", "chglog"], prog=__name__)
        if self.sql.error:
            return "no"

    def drawDialog(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Rental System Interest Rates Maintenance (%s)" % __name__)
        dat = {
            "stype": "R",
            "tables": ("rcaint",),
            "cols": (
                ("rci_date", "", 0, "Date-Chg"),
                ("rci_prime", "", 0, "Prime"),
                ("rci_bankr", "", 0, "BankR"),
                ("rci_admin", "", 0, "Admin")),
            "where": [("rci_cono", "=", self.conum)],
            "order": "rci_date"}
        self.fld = (
            (("T",0,0,0),"ID1",10,"Date of Change","",
                0,"N",self.doDate,dat,None,("efld",)),
            (("T",0,1,0),"IUD",5.2,"Prime Rate","",
                0,"N",None,None,self.doDelete,("notzero",)),
            (("T",0,2,0),"IUD",5.2,"Bank Rate","",
                0,"N",None,None,None,("notzero",)),
            (("T",0,3,0),"IUD",5.2,"Comm Rate","",
                0,"N",None,None,None,("efld",)))
        tnd = ((self.doEnd,"Y"),)
        txt = (self.doExit,)
        but = (("_Import File",None,self.doImport,0,("T", 0, 1),("T", 0, 2),
            """Import a File with the Correct Format i.e. Date, Prime Rate, Bank Rate, Fee Rate"""),)
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, tend=tnd,
            txit=txt, butt=but)

    def doImport(self, widget=None):
        state = self.df.disableButtons()
        self.df.mstAlw.hide()
        tit = ("Interest File Import",)
        fle = {
            "stype":  "F",
            "types":  "fle",
            "ftype":  (("CSV & XLS Files", "*.[c,x][s,l][v,s]"),)}
        r1s = (("Yes", "Y"), ("No", "N"))
        fld = (
            (("T",0,0,0),"IFF",50,"File Name","File Name",
                "","N",self.doImpFle,fle,None,("file",)),
            (("T",0,1,0),("IRB",r1s),0,"Delete Existing","",
                "N","N",self.doImpDel,None,None,None))
        tnd = ((self.doImpEnd,"n"), )
        txt = (self.doImpExit, )
        self.ip = TartanDialog(self.mf, newscn=(0, 0), tops=True, title=tit,
            eflds=fld, tend=tnd, txit=txt)
        self.mf.startLoop()
        self.df.mstAlw.show()
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doImpFle(self, frt, pag, r, c, p, i, w):
        self.impfle = w

    def doImpDel(self, frt, pag, r, c, p, i, w):
        self.impdel = w

    def doImpEnd(self):
        self.ip.closeProcess()
        if self.impdel == "Y":
            self.sql.delRec("rcaint", where=[("rci_cono", "=", self.conum)])
        impcol = [
            ["Date", 0, "D1", 10],
            ["Prime Rate", 1, "UD", 5.2],
            ["Bank Rate", 2, "UD", 5.2],
            ["Fee Rate", 3, "UD", 5.2]]
        fi = FileImport(self.mf, impfle=self.impfle, impcol=impcol)
        err = None
        for num, line in enumerate(fi.impdat):
            if len(line) != len(impcol):
                err = "Line %s: Invalid Number of Fields (S/B %s is %s)" % \
                    (num + 1, len(impcol), len(line))
                break
            date = CCD(line[0], "D1", 10)
            if date.err:
                err = "Line %s: Invalid Date %s" % ((num + 1), date.work)
                break
            prime = CCD(line[1], "UD", 5.2)
            if prime.err:
                err = "Line %s: Invalid Prime Rate %s" % ((num + 1), prime.work)
                break
            bank = CCD(line[2], "UD", 5.2)
            if bank.err:
                err = "Line %s: Invalid Bank Rate %s" % ((num + 1), bank.work)
                break
            fees = CCD(line[3], "UD", 5.2)
            if fees.err:
                err = "Line %s: Invalid Fees Rate %s" % ((num + 1), fees.work)
                break
            data = [self.conum, date.work, prime.work, bank.work, fees.work]
            chk = self.sql.getRec(tables="rcaint", data=data)
            if chk:
                err = "Line %s: A Record for Date, %s, Already Exists" % \
                    (num + 1, date.disp)
                break
            self.sql.insRec("rcaint", data=[self.conum, date.work, prime.work,
                bank.work, fees.work])
        if err:
           showError(self.mf.body, "Import Error", err)
        else:
            self.mf.dbm.commitDbase(ask=True,
                mess="Would you like to COMMIT All Imported Interest Rates?")
        self.mf.closeLoop()

    def doImpExit(self, widget=None):
        self.ip.closeProcess()
        self.mf.closeLoop()

    def doDate(self, frt, pag, r, c, p, i, w):
        self.date = w
        self.acc = self.sql.getRec(tables="rcaint", where=[("rci_cono", "=",
            self.conum), ("rci_date", "=", self.date)], limit=1)
        if self.acc:
            self.new = False
            for n, f in enumerate(self.acc[1:]):
                self.df.loadEntry("T", 0, n, data=f)
        else:
            self.new = True

    def doDelete(self):
        self.sql.delRec("rcaint", where=[("rci_cono", "=", self.conum),
            ("rci_date", "=", self.date)])
        dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
        self.sql.insRec("chglog", data=["rcaint", "D", "%03i%8s" % \
            (self.conum, self.date), "", dte, self.user, "", "", 0])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        data = [self.conum]
        for x in range(0, len(self.df.t_work[0][0])):
            data.append(self.df.t_work[0][0][x])
        if self.new:
            self.sql.insRec("rcaint", data=data)
        elif data != self.acc:
            acc = self.sql.updRec("rcaint", data=data, where=[("rci_cono",
                "=", self.conum), ("rci_date", "=", self.date)])
            dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
            for num, dat in enumerate(self.acc):
                if dat != data[num]:
                    self.sql.insRec("chglog", data=["rcaint", "U",
                        "%03i%8s" % (self.conum, self.date),
                        self.sql.rcaint_col[num], dte, self.user,
                        str(dat), str(data[num]), 0])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

