#! /usr/bin/python2

"""
SYNOPSIS
    Report Printer Front End.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import types
from TartanClasses import RepPrt, TartanDialog, SChoice, SplashScreen, Sql
from tartanFunctions import askQuestion, chkAggregate, showError, showInfo
from tartanWork import dattyp

class rp1010:
    def __init__(self, mf, user=None):
        self.mf = mf
        self.user = user
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlpwm", "ffield", "ftable", "frelat",
            "rptmst", "rpttab", "rptjon", "rptcol", "rptexc", "rptord",
            "rptvar"], prog=__name__)
        if self.sql.error:
            return self.sql.error
        #=======================================================================
        # Get list of available tables
        #=======================================================================
        whr = [("ft_seq", "=", 1), ("ft_tabl", "not", "like", "rpt%")]
        if self.user:
            syss = self.sql.getRec(tables="ctlpwm", cols=["mpw_sys"],
                where=[("mpw_usr", "=", self.user), ("mpw_prg", "=", "")])
            if syss:
                whr.append(("ft_tabl", "not", "in", ("ffield", "frelat",
                    "ftable", "verupd")))
                whr.append(("ft_tabl", "not", "like", "ctl%s"))
                whr.append(("ft_tabl", "not", "like", "%sctl"))
                all = {"ar": "ass", "ms": "ctl", "cr": "crs", "dr": "drs",
                    "gl": "gen", "rt": "rtl", "si": "sls", "sl": "wag",
                    "st": "str", "gl": "gen", "wg": "wag"}
                for sys in syss:
                    whr.append(("ft_tabl", "not", "like", all[sys[0]] + "%"))
        self.tables = self.sql.getRec(tables="ftable", cols=["ft_tabl",
            "ft_desc"], where=whr, order="ft_tabl")
        if not self.tables:
            return "No Tables to Select"
        #=======================================================================
        self.expl1 = ("Column","Integer","Decimal","String","Variable",
            "Avg()","Count(*)","Max()","Min()","Sum()")
        self.expl2 = ("+","-","*","/","Stop")
        self.expl3 = ("Column","Integer","Decimal","String")
        self.excl1 = ("=","<","<=",">",">=","<>","In","Like","SLike","ELike")
        self.excl2 = ("Column","Integer","Decimal","String","Variable")
        self.excl3 = ("and","or","Stop")

    def mainProcess(self):
        tit = ("Report Writer",)
        rpt = {
            "stype": "R",
            "tables": ("rptmst",),
            "cols": (
                ("rpm_rnam", "", 0, "Name"),
                ("rpm_desc", "", 0, "Description", "Y")),
            "order": "rpm_rnam"}
        self.tabs = {
            "stype": "C",
            "titl": "Select the Required Table",
            "data": self.tables,
            "index": 0}
        self.cols = {
            "stype": "C",
            "titl": "Select the Required Column",
            "data": [],
            "index": 0}
        dtp = {
            "stype": "C",
            "titl": "Select the Required Data Type",
            "data": dattyp,
            "index": 0}
        self.exps = {
            "stype": "C",
            "titl": "Select Option",
            "data": []}
        jon = {
            "stype": "C",
            "titl": "Select Where Type",
            "data": [
                ("I", "Inner Join"),
                ("L", "Left Outer Join"),
                ("R", "Right Outer Join"),
                ("F", "Full Outer Join")]}
        typ = []
        for t in dattyp:
            typ.append(t[0])
        self.typ = tuple(typ)
        tag = (
            ("Tables",None,("T",1,1),("T",0,1)),
            ("Joins",None,("T",1,1),("T",0,1)),
            ("Columns",None,("T",1,1),("T",0,1)),
            ("Variables",None,("T",1,1),("T",0,1)),
            ("Exceptions",None,("T",1,1),("T",0,1)),
            ("Order",None,("T",1,1),("T",0,1)))
        fld = (
            # Details
            (("T",0,0,0),"INA",10,"Name","Report Name",
                "","Y",self.doRepNam,rpt,None,("notblank",)),
            (("T",0,1,0),"INA",30,"Description","Report Description",
                "","N",None,None,self.doRepDel,("notblank",)),
            (("T",0,2,0),"INA",50,"Heading-1","Heading One",
                "","N",None,None,None,("notblank",)),
            (("T",0,3,0),"INA",50,"Heading-2","Heading Two",
                "","N",None,None,None,None),
            # Tables
            (("T",1,0,0),"IUI",2,"Table Sequence","Table Sequence",
                "","N",self.doTabSeq,None,None,("between",0,14)),
            (("C",1,0,0),"OUI",2,"Sq","","","N",self.doTable,None,None,None),
            (("C",1,0,1),"INA",6,"Tables","Table Name",
                "","N",self.doRepTab,self.tabs,self.doColDel,("notblank",)),
            (("C",1,0,2),"ONA",30,"Description"),
            # Joins
            (("T",2,0,0),"IUI",2,"Join Sequence","Join Sequence",
                "","N",self.doJoinSeq,None,None,("between",0,14)),
            (("C",2,0,0),"OUI",2,"Sq","","","N",self.doJoin,None,None,None),
            (("C",2,0,1),"IUA",1,"T","Join Type","I","N",self.doJoinTyp,
                jon,self.doColDel,("in",("N","I","L","R","F"))),
            (("C",2,0,2),"INA",6,"Tables","Join Table Name",
                "","N",self.doJoinTab,self.tabs,None,("notblank",)),
            (("C",2,0,3),"INA",(70,100),"Join Columns","Join Columns",
                "","N",None,None,None,("notblank",)),
            # Columns
            (("T",3,0,0),"IUI",2,"Column Sequence","Column Sequence",
                "","N",self.doColSeq,None,None,("between",0,14)),
            (("C",3,0,0),"OUI",2,"Sq","","","N",None,None,None,None),
            (("C",3,0,1),"IUA",1,"T","Type (C/E)",
                "C","N",self.doType,None,self.doColDel,("in",("C","E"))),
            (("C",3,0,2),"ILA",(15,20),"Label","Column or Expression Label",
                "","N",self.doLabel,self.cols,None,("notblank",)),
            (("C",3,0,3),"ITv",15,"Expression","Expression Detail",
                "","N",self.doExpDet,None,None,("notblank",)),
            (("C",3,0,4),"INA",(15,50),"Heading","Column Heading",
                "","N",None,None,None,None),
            (("C",3,0,5),"INA",2,"Tp","Display Type",
                "","N",self.doColTyp,dtp,None,("in",self.typ)),
            (("C",3,0,6),"IUD",6.1,"Size","Display Size",
                "","N",self.doColSiz,None,None,("notzero",)),
            [("C",3,0,7),"IUA",1,"G","Group",
                "N","N",None,None,None,("in",("Y","N"))],
            (("C",3,0,8),"IUA",1,"S","Sub Total",
                "N","N",self.doSubTot,None,None,("in",("Y","N"))),
            (("C",3,0,9),"INA",(15,30),"Narration","Sub-Total Narration",
                "","N",None,None,None,("notblank",)),
            (("C",3,0,10),"IUA",1,"P","Advance Page After Sub-Total",
                "","N",self.doPagAdv,None,None,("in",("Y","N"))),
            (("C",3,0,11),"IUA",1,"G","Grand Total",
                "N","N",None,None,None,("in",("Y","N"))),
            (("C",3,0,12),"IUA",1,"D","Display Column",
                "Y","N",None,None,None,("in",("Y","N"))),
            # Variables
            (("T",4,0,0),"IUI",1,"Variable Sequence","Variable Sequence",
                "","N",self.doVarSeq,None,None,("between",0,9)),
            (("C",4,0,0),"OUI",1,"V","","","N",self.doVariable,None,None,None),
            (("C",4,0,1),"INA",30,"Description","Variable Description",
                "","N",self.doVarDesc,None,self.doColDel,("notblank",)),
            (("C",4,0,2),"ONA",2,"Tp","Display Type",
                "","N",None,None,None,("notblank",)),
            (("C",4,0,3),"OUD",6.1,"Size","Display Size",
                "","N",None,None,None,("notzero",)),
            # Exceptions
            (("T",5,0,0),"IUI",2,"Exception Sequence","Exception Sequence",
                "","N",self.doExcSeq,None,None,("between",0,14)),
            (("C",5,0,0),"OUI",2,"Sq","","","N",self.doExcept,None,None,None),
            (("C",5,0,1),"ITv",70,"Exception","Exception Detail",
                "","N",None,None,self.doColDel,("notblank",)),
            # Orders
            (("T",6,0,0),"IUI",2,"Order Sequence","Order Sequence",
                "","N",self.doOrdSeq,None,None,("between",0,14)),
            (("C",6,0,0),"OUI",2,"Sq","","","N",self.doOrder,None,None,None),
            (("C",6,0,1),"INA",20,"Column","Column Name",
                "","N",self.doRepOrd,self.cols,self.doColDel,("notblank",)),
            (("C",6,0,2),"ONA",30,"Description"),
            (("C",6,0,3),"IUA",1,"D","Ascending or Descending",
                "A","N",None,None,None,("in",("A","D"))))
        row = (0,14,14,14,14,14,14)
        but = (
            ("Execute",None,self.doExecute,0,None,None),
            ("Insert",None,self.doInsert,0,None,None),
            ("Clear",None,self.doClear,0,None,None),
            ("Quit",None,self.doCloseProcess,1,None,None))
        tnd = (
            (self.doTopEnd,"y"),
            (self.doTopEnd,"n"),
            (self.doTopEnd,"n"),
            (self.doTopEnd,"n"),
            (self.doTopEnd,"n"),
            (self.doTopEnd,"n"),
            (self.doTopEnd,"n"))
        txt = (
            self.doCloseProcess,
            self.doTopExit,
            self.doTopExit,
            self.doTopExit,
            self.doTopExit,
            self.doTopExit,
            self.doTopExit)
        cnd = (
            None,
            (self.doColEnd,"n"),
            (self.doColEnd,"n"),
            (self.doColEnd,"n"),
            (self.doColEnd,"n"),
            (self.doColEnd,"n"),
            (self.doColEnd,"n"))
        cxt = (
            None,
            self.doColExit,
            self.doColExit,
            self.doColExit,
            self.doColExit,
            self.doColExit,
            self.doColExit)
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, butt=but,
            tend=tnd, txit=txt, rows=row, tags=tag, cend=cnd, cxit=cxt)

    def doRepNam(self, frt, pag, r, c, p, i, w):
        self.repnam = w
        rep = self.sql.getRec(tables="rptmst", where=[("rpm_rnam", "=",
            self.repnam)], limit=1)
        if not rep:
            self.newrep = "y"
        else:
            self.newrep = "n"
            for x in range(1, len(rep)):
                self.df.loadEntry(frt, pag, p+x, data=rep[x])
            self.doLoadReport()
            self.df.setWidget(self.df.B0, "normal")

    def doLoadReport(self):
        # Tables
        dat = self.sql.getRec(tables="rpttab", where=[("rpt_rnam", "=",
            self.repnam)])
        if dat:
            for d in range(0, len(dat)):
                p = d * self.df.colq[1]
                t = dat[d][1]
                self.df.loadEntry("C", 1, p, data=d)
                self.df.loadEntry("C", 1, p+1, data=t)
                dsc = self.doGetTableDesc(t)
                if dsc:
                    self.df.loadEntry("C", 1, p+2, data=dsc)

        # Joins
        dat = self.sql.getRec(tables="rptjon", where=[("rpj_rnam", "=",
            self.repnam)], order="rpj_seq")
        if dat:
            for d in dat:
                r = int(d[1])
                p = r * self.df.colq[2]
                for i in range(0, self.df.colq[2]):
                    self.df.loadEntry("C", 2, p+i, data=d[i+1])

        # Columns
        dat = self.sql.getRec(tables="rptcol", where=[("rpc_rnam", "=",
            self.repnam)], order="rpc_seq")
        if dat:
            for d in dat:
                r = int(d[1])
                p = r * self.df.colq[3]
                for i in range(0, self.df.colq[3]):
                    if i == 4:
                        data = d[7]
                    elif i == 5:
                        data = d[5]
                    elif i == 6:
                        data = d[6]
                    else:
                        data = d[i+1]
                    self.df.loadEntry("C", 3, p+i, data=data)

        # Variables
        dat = self.sql.getRec(tables="rptvar", where=[("rpv_rnam", "=",
            self.repnam)], order="rpv_seq")
        if dat:
            for d in dat:
                r = int(d[1])
                p = r * self.df.colq[4]
                for i in range(0, self.df.colq[4]):
                    self.df.loadEntry("C", 4, p+i, data=d[i+1])

        # Exceptions
        dat = self.sql.getRec(tables="rptexc", where=[("rpx_rnam", "=",
            self.repnam)], order="rpx_seq")
        if dat:
            for d in dat:
                r = int(d[1])
                p = r * self.df.colq[5]
                for i in range(0, self.df.colq[5]):
                    self.df.loadEntry("C", 5, p+i, data=d[i+1])

        # Order
        dat = self.sql.getRec(tables="rptord", where=[("rpo_rnam", "=",
            self.repnam)], order="rpo_seq")
        if dat:
            for d in dat:
                r = int(d[1])
                p = r * self.df.colq[6]
                self.df.loadEntry("C", 6, p+0, data=d[1])
                self.df.loadEntry("C", 6, p+1, data=d[2])
                dsc = self.doGetColDetail(d[2])
                if dsc:
                    self.df.loadEntry("C", 6, p+2, data=dsc[0])
                self.df.loadEntry("C", 6, p+3, data=d[3])

    def doTabSeq(self, frt, pag, r, c, p, i, w):
        cs = self.doChkSeq(pag, w)
        if cs:
            return cs

    def doTable(self, frt, pag, r, c, p, i, w):
        if not self.df.c_work[pag][r][i+1]:
            tab = self.df.selectItem(pag, self.tabs)
            if tab == None:
                self.df.clearLine(pag)
                return "xt"
            else:
                self.df.loadEntry(frt, pag, p+1, data=tab)
                dsc = self.doGetTableDesc(tab)
                self.df.loadEntry(frt, pag, p+2, data=dsc)

    def doRepTab(self, frt, pag, r, c, p, i, w):
        dsc = self.doGetTableDesc(w)
        if dsc == None:
            return "Invalid Table"
        self.df.loadEntry(frt, pag, p+1, data=dsc)

    def doGetTableDesc(self, table):
        dsc = self.sql.getRec(tables="ftable", cols=["ft_desc"],
            where=[("ft_tabl", "=", table), ("ft_seq", "=", 1)], limit=1)
        if dsc:
            return dsc[0]

    def doJoinSeq(self, frt, pag, r, c, p, i, w):
        cs = self.doChkSeq(pag, w)
        if cs:
            return cs
        cd = self.doLoadColData()
        if cd:
            return cd

    def doJoin(self, frt, pag, r, c, p, i, w):
        pass

    def doJoinTyp(self, frt, pag, r, c, p, i, w):
        pass

    def doJoinTab(self, frt, pag, r, c, p, i, w):
        if self.df.c_work[pag][r][i+1]:
            return
        col1 = self.sql.getRec(tables="ffield", cols=["ff_name"],
            where=[("ff_tabl", "=", w)], order="ff_name")
        col2 = self.cols["data"]
        jon = ""
        tab = "Column"
        c1 = "y"
        while tab != "Stop":
            if tab == "Column":
                if c1 == "y":
                    self.cols["data"] = col1
                else:
                    self.cols["data"] = col2
                tab = self.df.selectItem(pag, self.cols)
                if tab == None:
                    jon = ""
                    break
                if not jon:
                    jon = tab
                else:
                    jon = jon + " %s" % tab
                if c1 == "y":
                    c1 = "n"
                    self.exps["data"] = ("=",)
                else:
                    c1 = "y"
                    self.exps["data"] = ("and", "Stop")
            else:
                if tab == "=":
                    self.exps["data"] = ("Column",)
                elif tab == "Column":
                    self.exps["data"] = ("and", "Stop")
                elif tab in ("and", "Stop"):
                    self.exps["data"] = ("Column",)
            tab = self.df.selectItem(pag, self.exps)
            if tab == None:
                jon = ""
                break
            if tab != "Stop" and tab not in ("Column",):
                jon = jon + " %s" % tab
        if jon:
            self.df.loadEntry(frt, pag, p+1, data=jon)
        else:
            if not self.df.c_work[pag][r][i+1]:
                self.df.clearLine(pag)
            return "xt"

    def doJoinDel(self):
        pass

    def doVarSeq(self, frt, pag, r, c, p, i, w):
        cs = self.doChkSeq(pag, w)
        if cs:
            return cs
        cd = self.doLoadColData()
        if cd:
            return cd

    def doVariable(self, frt, pag, r, c, p, i, w):
        if not self.df.c_work[pag][r][i+1]:
            col = self.df.selectItem(pag, self.cols)
            if col == None:
                self.df.clearLine(pag)
                return "xt"
            else:
                dsc = self.doGetColDetail(col)
                self.df.loadEntry(frt, pag, p+1, data=dsc[0])
                self.df.loadEntry(frt, pag, p+2, data=dsc[1])
                self.df.loadEntry(frt, pag, p+3, data=dsc[2])

    def doVarDesc(self, frt, pag, r, c, p, i, w):
        return "nd"

    def doGetColDetail(self, col):
        dsc = self.sql.getRec(tables="ffield", cols=["ff_desc", "ff_type",
            "ff_size"], where=[("ff_name", "=", col)], limit=1)
        return dsc

    def doRepDel(self):
        self.doDeleteAllRecords()
        self.mf.dbm.commitDbase()
        self.df.selPage("Tables")
        self.df.focusField("T", 0, 1)

    def doDeleteAllRecords(self):
        self.sql.delRec("rptmst", where=[("rpm_rnam", "=", self.repnam)])
        self.sql.delRec("rpttab", where=[("rpt_rnam", "=", self.repnam)])
        self.sql.delRec("rptjon", where=[("rpj_rnam", "=", self.repnam)])
        self.sql.delRec("rptcol", where=[("rpc_rnam", "=", self.repnam)])
        self.sql.delRec("rptexc", where=[("rpx_rnam", "=", self.repnam)])
        self.sql.delRec("rptvar", where=[("rpv_rnam", "=", self.repnam)])
        self.sql.delRec("rptord", where=[("rpo_rnam", "=", self.repnam)])

    def doTopEnd(self):
        pag = self.df.pag
        if pag == 0:
            self.doAllTabs("normal")
            self.doSetNextSeq()
            self.df.focusField("T", 1, 1)
        else:
            self.doAllTabs("disabled")
            max = self.df.colq[pag]
            seq = self.df.t_work[pag][0][0]
            self.df.loadEntry("C", pag, seq*max, data=seq)
            self.df.focusField("C", pag, ((seq*max)+1))

    def doTopExit(self):
        self.doAllTabs("disabled")
        self.doRptAskSave()
        self.df.selPage("Tables")
        self.df.focusField("T", 0, 1)

    def doColSeq(self, frt, pag, r, c, p, i, w):
        cs = self.doChkSeq(pag, w)
        if cs:
            return cs
        cd = self.doLoadColData()
        if cd:
            return cd

    def doType(self, frt, pag, r, c, p, i, w):
        self.ctyp = w
        if self.ctyp == "C" and not self.df.c_work[pag][r][i+1]:
            col = self.df.selectItem(pag, self.cols)
            if col == None:
                return ""
            else:
                self.df.loadEntry(frt, pag, p+1, data=col)

    def doLabel(self, frt, pag, r, c, p, i, w):
        self.lab = w
        if self.ctyp == "C":
            col = self.sql.getRec(tables="ffield",
                cols=["ff_name", "ff_head", "ff_type", "ff_size"],
                where=[("ff_name", "=", self.lab)], limit=1)
            if not col:
                return "Invalid Column"
            for x in range(1, 5):
                self.df.loadEntry(frt, pag, p+x, data=col[x-1])
            return "sk1"
        elif self.ctyp == "E" and not self.df.c_work[pag][r][i+1]:
            self.exp = ""
            tab = ""
            br = "n"
            ex = "n"
            while tab != "Stop":
                if not tab:
                    self.exps["data"] = self.expl1[:]
                    tab = self.df.selectItem(pag, self.exps)
                    if tab == None:
                        return ""
                elif tab == "Column":
                    tab = self.df.selectItem(pag, self.cols)
                    if tab == None:
                        return ""
                    self.exp = self.exp + tab
                    if br == "y":
                        self.exp = self.exp + ")"
                        br = "n"
                        if ex == "y":
                            break
                    self.exps["data"] = self.expl2[:]
                    tab = self.df.selectItem(pag, self.exps)
                    if tab == None:
                        return ""
                elif tab != "Stop":
                    if tab.find("()") != -1:
                        br = "y"
                        self.exp = self.exp + tab.replace(")", "")
                        if tab in ("Avg()", "Max()", "Min()", "Sum()"):
                            ex = "y"
                        else:
                            ex = "n"
                        tab = "Column"
                    elif tab == "Count(*)":
                        self.exp = self.exp + tab
                        break
                    elif tab in ("Integer", "Decimal", "String", "Variable"):
                        self.doConstantInput(3, tab)
                        if not self.cons:
                            return self.cons
                        else:
                            if tab == "String":
                                self.exp = self.exp + '"%s"' % self.cons
                            elif tab == "Variable":
                                ok = self.doLoadVarData()
                                if ok != "ok":
                                    return ""
                                self.exp = self.exp + "(v)%s" % self.cons
                            else:
                                self.exp = self.exp + "%s" % self.cons
                            self.exps["data"] = self.expl2[:]
                            tab = self.df.selectItem(pag, self.exps)
                            if tab == None:
                                return ""
                    else:
                        self.exp = self.exp + tab
                        tab = ""
            self.df.loadEntry(frt, pag, p+1, data=self.exp)
        elif self.ctyp == "E":
            self.exp = self.df.c_work[pag][r][i+1]

    def doExpDet(self, frt, pag, r, c, p, i, w):
        self.df.loadEntry(frt, pag, p, data=w)

    def doColTyp(self, frt, pag, r, c, p, i, w):
        self.coltyp = w

    def doColSiz(self, frt, pag, r, c, p, i, w):
        skp = "N"
        if self.ctyp == "E":
            for exp in ("Avg(","Max(","Min(","Sum(","Count(*)"):
                if self.exp.find(exp) != -1:
                    skp = "Y"
                    break
        if skp == "Y":
            self.df.loadEntry(frt, pag, p+1, data="N")
            return "sk1"

    def doConstantInput(self, pag, tab):
        if tab == "Variable":
            v = []
            for x in range(0, len(self.df.c_work[4])):
                if not self.df.c_work[4][x][1]:
                    break
                else:
                    v.append([x, self.df.c_work[4][x][1]])
            tit = "Select Variable"
            hd = ("V", "Description")
            state = self.df.disableButtons()
            sc = SChoice(self.mf, titl=tit, head=hd, data=v)
            self.df.enableButtons(state=state)
            if sc.selection in (None, []):
                self.cons = ""
            else:
                self.cons = sc.selection[0]
        else:
            tit = ("Enter %s" % tab,)
            if tab == "Integer":
                typ = "ISL"
                siz = 10
                chk = ("efld",)
            elif tab == "Decimal":
                typ = "ISD"
                siz = 13.2
                chk = ("efld",)
            elif tab == "String":
                typ = "INA"
                siz = 30
                chk = ("efld",)
            self.cons = ""
            fld = ((("T",0,0,0), typ, siz, "%s" % tab, tab,
                "", "N", None, None, None, chk),)
            tnd = ((self.doConEnd,"n"),)
            txt = (self.doConExit,)
            self.cf = TartanDialog(self.mf, newscn=(0, 0), tops=True,
                title=tit, eflds=fld, tend=tnd, txit=txt)
            self.mf.startLoop()

    def doConEnd(self):
        self.cons = self.cf.t_work[0][0][0]
        self.doConExit()

    def doConExit(self):
        self.cf.closeProcess()
        self.mf.closeLoop()

    def doExcSeq(self, frt, pag, r, c, p, i, w):
        cs = self.doChkSeq(pag, w)
        if cs:
            return cs
        cd = self.doLoadColData()
        if cd:
            return cd

    def doExcept(self, frt, pag, r, c, p, i, w):
        if self.df.c_work[pag][r][i+1]:
            return
        exc = ""
        tab = "Column"
        c1 = "y"
        while tab != "Stop":
            if tab == "Column":
                tab = self.df.selectItem(pag, self.cols)
                if tab == None:
                    exc = ""
                    break
                # Column type for the 'In' clause
                ctp = self.sql.getRec(tables="ffield", cols=["ff_type"],
                    where=[("ff_name", "=", tab)], limit=1)[0]
                if not exc:
                    exc = tab
                else:
                    exc = exc + " %s" % tab
                if c1 == "y":
                    c1 = "n"
                    self.exps["data"] = self.excl1[:]
                else:
                    c1 = "y"
                    self.exps["data"] = self.excl3[:]
            elif tab in ("Integer", "Decimal", "String", "Variable"):
                c1 = "y"
                self.doConstantInput(4, tab)
                if not self.cons:
                    return self.cons
                else:
                    if tab == "String":
                        if exc[-2:] == "In":
                            con = tuple(self.cons.split(","))
                            if ctp[1].upper() == "A":
                                exc = exc + " %s" % str(con)
                            else:
                                exc = exc + " %s" % str(con).replace("'", "")
                        elif self.cons in ('""', "''"):
                            exc = exc + ' ""'
                        elif exc[-4:] == "Like":
                            exc = exc + ' "%s%s%s"' % ("%", self.cons, "%")
                        elif exc[-5:] == "SLike":
                            exc = exc + ' "%s%s"' % (self.cons, "%")
                        elif exc[-5:] == "ELike":
                            exc = exc + ' "%s%s"' % ("%", self.cons)
                        else:
                            exc = exc + ' "%s"' % self.cons
                    elif tab == "Variable":
                        ok = self.doLoadVarData()
                        if ok != "ok":
                            return ""
                        exc = exc + " (v)%s" % self.cons
                    else:
                        exc = exc + " %s" % self.cons
                    self.exps["data"] = self.excl3[:]
            else:
                if tab in self.excl1:
                    self.exps["data"] = list(self.excl2[:])
                    if ctp[1].upper() == "A":
                        self.exps["data"].remove("Integer")
                        self.exps["data"].remove("Decimal")
                    elif ctp[1].upper() in ("1", "2", "D", "I"):
                        self.exps["data"].remove("String")
                        if ctp[1].upper() == "D":
                            self.exps["data"].remove("Integer")
                        else:
                            self.exps["data"].remove("Decimal")
                elif tab in self.excl2:
                    self.exps["data"] = self.excl3[:]
                elif tab in self.excl3:
                    tab = "Column"
            if len(exc) > 100:
                showError(self.mf.body, "Exception Error",
                    "Exception Statement Exceeds 100 Characters")
                tab = None
            if tab == "In":
                tab = "String"
            elif tab in ("Like", "SLike", "ELike"):
                self.exps["data"].remove("Column")
                tab = self.df.selectItem(pag, self.exps)
            elif tab and tab != "Column":
                tab = self.df.selectItem(pag, self.exps)
            if tab == None:
                exc = ""
                break
            if tab != "Stop" and tab not in self.excl2:
                exc = exc + " %s" % tab
        if exc:
            self.df.loadEntry(frt, pag, p+1, data=exc)
        else:
            if not self.df.c_work[pag][r][i+1]:
                self.df.clearLine(pag)
            return "xt"

    def doOrdSeq(self, frt, pag, r, c, p, i, w):
        cs = self.doChkSeq(pag, w)
        if cs:
            return cs
        cd = self.doLoadColData()
        if cd:
            return cd

    def doOrder(self, frt, pag, r, c, p, i, w):
        if not self.df.c_work[pag][r][i+1]:
            col = self.df.selectItem(pag, self.cols)
            if col == None:
                self.df.clearLine(pag)
                return "xt"
            else:
                self.df.loadEntry(frt, pag, p+1, data=col)
                dsc = self.doGetColDetail(col)
                self.df.loadEntry(frt, pag, p+2, data=dsc[0])

    def doRepOrd(self, frt, pag, r, c, p, i, w):
        dsc = self.doGetColDetail(w)
        if not dsc:
            return "Invalid Column"
        self.df.loadEntry(frt, pag, p+1, data=dsc[0])

    def doChkSeq(self, pag, seq):
        if seq > 0 and not self.df.c_work[pag][seq-1][1]:
            return "Invalid Sequence"
        if not self.df.c_work[pag][seq][1]:
            self.newrow = "Y"
        else:
            self.newrow = "N"

    def doLoadColData(self):
        tab = []
        for r in self.df.c_work[1]:
            if not r[1]:
                break
            else:
                tab.append(r[1])
        for r in self.df.c_work[2]:
            if not r[2]:
                break
            else:
                tab.append(r[2])
        if tab == []:
            return "No Tables Selected"
        dat = tuple(tab)
        data = self.sql.getRec(tables="ffield", cols=["ff_name", "ff_desc"],
            where=[("ff_tabl", "in", dat)], order="ff_name")
        if not data:
            return "No Columns to Select"
        self.cols["data"] = data

    def doLoadVarData(self):
        data = []
        for col in self.df.c_work[4]:
            if not col[1]:
                break
            data.append((col[0], col[1]))
        if data == []:
            return "No Variables to Select"
        self.exps["data"] = data
        return "ok"

    def doSubTot(self, frt, pag, r, c, p, i, w):
        self.stot = w
        if self.stot == "N":
            self.df.loadEntry(frt, pag, p+1, data="")
            self.df.loadEntry(frt, pag, p+2, data="N")
            if self.coltyp[1] not in ("I", "D"):
                self.df.loadEntry(frt, pag, p+3, data="N")
                return "sk3"
            else:
                return "sk2"

    def doPagAdv(self, frt, pag, r, c, p, i, w):
        if self.stot == "Y":
            self.df.loadEntry(frt, pag, p+1, data="N")
            return "sk1"

    def doColDel(self):
        pag = self.df.pag
        tot = self.df.rows[pag] - 1
        max = self.df.colq[pag]
        row = int(self.df.pos / max)
        seq = self.df.t_work[pag][0][0]
        for row in range(seq, tot):
            if not self.df.c_work[pag][row+1][1]:
                self.df.clearLine(self.df.pag, row=tot)
                break
            pos = row * max
            self.df.loadEntry("C", pag, pos, data=row)
            for col in range(1, max):
                data = self.df.c_work[pag][row+1][col]
                self.df.loadEntry("C", pag, pos+col, data=data)
        self.df.clearLine(self.df.pag, row=row)
        self.doAllTabs("normal")
        self.doSetNextSeq()
        self.df.focusField("T", pag, 1)
        return "nf"

    def doColEnd(self):
        self.doAllTabs("normal")
        self.doSetNextSeq()
        self.df.last[self.df.pag] = [0, 0]
        self.df.focusField("T", self.df.pag, 1)

    def doColExit(self):
        self.doAllTabs("normal")
        if self.df.idx == 0:
            if self.newrow == "Y":
                self.df.clearLine(self.df.pag)
            self.df.last[self.df.pag] = [0, 0]
            self.df.focusField("T", self.df.pag, 1)

    def doSetNextSeq(self):
        for pag in range(1, (len(self.df.tags) + 1)):
            d = self.df.c_work[pag]
            for seq in range(0, len(d)):
                if not d[seq][1]:
                    break
            self.df.loadEntry("T", pag, 0, data=seq)

    def doRptAskSave(self):
        answer = askQuestion(self.mf.body, "Save?",
            "Would you like to save this report (%s)?" % self.repnam)
        if answer == "yes":
            self.doSave()

    def doAllTabs(self, state="normal"):
        for num, tag in enumerate(self.df.tags):
            if state == "normal":
                self.df.enableTag(num)
            else:
                self.df.disableTag(num)
        for num, but in enumerate(self.df.butt):
            if state == "normal":
                but[3] = 1
            else:
                but[3] = 0
            exec "self.df.setWidget(self.df.B%s, '%s')" % (num, state)

    def doSave(self, widget=None):
        self.doDeleteAllRecords()
        val = tuple(self.df.t_work[0][0])
        self.sql.insRec("rptmst", data=val)

        for v in self.df.c_work[1]:
            if not v[1]:
                break
            val = (self.repnam, v[1])
            self.sql.insRec("rpttab", data=val)

        for v in self.df.c_work[2]:
            if not v[1]:
                break
            val = (self.repnam, v[0], v[1], v[2], v[3])
            self.sql.insRec("rptjon", data=val)

        for v in self.df.c_work[3]:
            if not v[1]:
                break
            val = [self.repnam]
            val.extend(v[0:4])
            val.extend(v[5:7])
            val.append(v[4])
            val.extend(v[7:13])
            val = tuple(val)
            self.sql.insRec("rptcol", data=val)

        for v in self.df.c_work[4]:
            if not v[1]:
                break
            val = (self.repnam, v[0], v[1], v[2], v[3])
            self.sql.insRec("rptvar", data=val)

        for v in self.df.c_work[5]:
            if not v[1]:
                break
            val = (self.repnam, v[0], v[1])
            self.sql.insRec("rptexc", data=val)

        for v in self.df.c_work[6]:
            if not v[1]:
                break
            val = (self.repnam, v[0], v[1], v[3])
            self.sql.insRec("rptord", data=val)
        self.mf.dbm.commitDbase()
        showInfo(self.mf.body, "Save", "Report Successfully Saved")

    def doExecute(self, widget=None):
        self.mf.updateStatus("")
        self.doRptAskSave()
        self.df.mstAlw.hide()
        self.doVariables()
        if not self.exit:
            err = self.doRest()
            if err:
                showError(self.mf.body, "Report %s Error" % self.repnam, err)
        self.df.mstAlw.show()
        self.df.focusField("T", self.df.pag, 1)

    def doInsert(self, widget=None):
        if self.df.frt != "T" or self.df.pag == 0 or self.df.col != 1:
            return
        tit = ("Insert Column",)
        fld = ((("T",0,0,0), "IUI", 2, "Before Sq","Insert Before Sq",
            "", "N",self.doInsSeq,None,None,("efld",)),)
        tnd = ((self.doInsEnd,"n"),)
        txt = (self.doInsExit,)
        self.ic = TartanDialog(self.mf, newscn=(0, 0), tops=True, title=tit,
            eflds=fld, tend=tnd, txit=txt)
        self.mf.startLoop()

    def doInsSeq(self, frt, pag, r, c, p, i, w):
        self.insseq = w

    def doInsEnd(self):
        self.ic.closeProcess()
        wrk = []
        for num, col in enumerate(self.df.c_work[self.df.pag]):
            if not col[1]:
                break
            if num == self.insseq:
                if self.df.pag == 1:
                    wrk.append([num,"",""])
                elif self.df.pag == 2:
                    wrk.append([num,"","",""])
                elif self.df.pag == 3:
                    wrk.append([num,"C","","","","",0,"","","","","",""])
                elif self.df.pag == 4:
                    wrk.append([num,"","",0])
                elif self.df.pag == 5:
                    wrk.append([num,""])
                elif self.df.pag == 6:
                    wrk.append([num,"","",""])
            wrk.append(col[:])
        self.df.clearFrame("C", self.df.pag)
        for num, col in enumerate(wrk):
            col[0] = num
            for n, d in enumerate(col):
                self.df.loadEntry("C", self.df.pag,
                    ((num * self.df.colq[self.df.pag]) + n), data=d)
        self.mf.closeLoop()
        self.df.loadEntry("T", self.df.pag, 0, data=self.insseq)
        self.df.focusField("T", self.df.pag, 1)

    def doInsExit(self):
        self.ic.closeProcess()
        self.mf.closeLoop()

    def doVariables(self):
        #VARIABLES
        self.var_det = []
        for x in range(0, len(self.df.c_work[4])):
            if not self.df.c_work[4][x][1]:
                break
            self.var_det.append([self.df.c_work[4][x][1],
                self.df.c_work[4][x][2], self.df.c_work[4][x][3], ""])
        tit = ("Report Variables",)
        fld = []
        for x in range(0, len(self.var_det)):
            var = self.var_det[x]
            tex = var[0] + " " * (30 - len(var[0]))
            fld.append((("T",0,x,0),"I%s" % var[1],var[2],"%s" % tex,
                "%s" % var[0],"","N",None,None,None,("efld",)))
        tnd = ((self.doVarEnd,"n"),)
        txt = (self.doVarExit,)
        self.vf = TartanDialog(self.mf, newscn=(0, 0), tops=True, title=tit,
            eflds=fld, tend=tnd, txit=txt, view=("Y","V"), mail=("Y","N"))
        self.mf.startLoop()

    def doVarEnd(self):
        for x in range(0, len(self.var_det)):
            self.var_det[x][3] = self.vf.t_work[0][0][x]
        self.exit = None
        self.doVarClose()

    def doVarExit(self):
        self.exit = "y"
        self.doVarClose()

    def doVarClose(self):
        self.vf.closeProcess()
        self.mf.closeLoop()

    def doRest(self):
        # TABLES
        tabs = []
        for x in range(0, len(self.df.c_work[1])):
            if not self.df.c_work[1][x][1]:
                break
            else:
                tabs.append(self.df.c_work[1][x][1])
        if not tabs:
            return
        # JOINS
        jons = []
        for x in range(0, len(self.df.c_work[2])):
            if not self.df.c_work[2][x][1]:
                break
            else:
                # Type, Table Name and Columns
                jons.append([self.df.c_work[2][x][1],
                    self.df.c_work[2][x][2], self.df.c_work[2][x][3]])
        # COLUMNS, EXPRESSIONS, GROUPS, STOTS and GTOTS
        cols = []
        temp = []
        grps = []
        stot = []
        gtot = []
        sums = []
        aggr = False
        expr = False
        for col in self.df.c_work[3]:
            if not col[1]:
                break
            if col[1] == "C":
                c = col[3].strip()
                cols.append([c, col[5], col[6], col[4], col[12]])
                temp.append(c)
                if col[7].upper() == "Y":
                    grps.append(c)
                if col[8].upper() == "Y":
                    stot.append([c, col[9], col[10]])
                if col[11].upper() == "Y":
                    gtot.append(c)
            elif col[1] == "E":
                c = col[3].rstrip()
                v = c.split("(v)")
                if not chkAggregate(c):
                    self.doLoadColData()
                    for chk in self.cols["data"]:
                        if chk[0] in c:
                            temp.append(chk[0])
                else:
                    aggr = True
                expr = True
                for x in range(1, len(v)):
                    n = int(v[x][:1])
                    d = self.var_det[n][3]
                    if isinstance(d, types.StringType) == 1:
                        d = '"%s"' % d
                    c = c.replace("(v)%s" % str(n), str(d))
                if c[:4].lower() == "sum(":
                    c = c.replace(c[:4], "Round(Sum(")
                    s = str(col[6]).split(".")
                    if len(s) == 1:
                        d = 0
                    else:
                        d = int(s[1])
                    c = c.replace(")", "), %s)" % d)
                if c[:6].lower() == "round(":
                    s = c.lower()
                    s = s.split("round(sum(")
                    s = s[1].split(")")
                    sums.append(s[0])
                c = c + " as %s" % col[2].strip()
                cols.append([c, col[5], col[6], col[4], col[12]])
                if col[8].upper() == "Y":
                    stot.append([col[2].strip(), col[9], col[10]])
                if col[11].upper() == "Y":
                    gtot.append(col[2].strip())
        if stot and not gtot:
            return "You Cannot Have Sub Totals Without a Grand Total"
        if not cols:
            if len(tabs) > 1:
                return "You Have Selected More than One Table But NO COLUMNS!"
            for tab in tabs:
                t = self.sql.getRec(tables="ffield", cols=["ff_name", "ff_type",
                    "ff_size", "ff_head"], where=[("ff_tabl", "=", tab)],
                    order="ff_seq")
                for c in t:
                    c.append("Y")
                    cols.extend([c])
        if sums:
            chk = []
            for sum in sums:
                s = sum.split("_")
                if not chk:
                    chk.append(s[0])
                elif not s[0] in chk:
                    showError(self.mf.body, "Sum Error",
                        "You are Aggregating More than One Table!\n\n"\
                        "The Results Will Most Probably be Incorrect!")
        if aggr or grps:
            for col in temp:
                if col not in grps:
                    grps.append(col)
            temp = ""
            for grp in grps:
                if not temp:
                    temp = grp
                else:
                    temp = "%s, %s" % (temp, grp)
            grps = temp
        # WHERE
        excs = ""
        # Check table relationships
        rels = []
        for t1 in tabs:
            for t2 in tabs:
                if t2 == t1:
                    continue
                all = self.sql.getRec(tables="frelat", cols=["rel_col1",
                    "rel_col2"], where=[("rel_tab1", "=", t1), ("rel_tab2",
                    "=", t2)])
                if not all:
                    all = self.sql.getRec(tables="frelat", cols=["rel_col1",
                        "rel_col2"], where=[("rel_tab1", "=", t2), ("rel_tab2",
                        "=", t1)])
                for rel in all:
                    if rels.count(rel) == 0:
                        rels.append(rel)
        if len(tabs) > 1 and not rels:
            showError(self.mf.body, "Relationship Error",
                "Some Relationships Between Tables %s are Missing" % tabs)
            return "error"
        for rel in rels:
            if not excs:
                excs = "%s = %s" % (rel[0], rel[1])
            else:
                excs = "%s and %s = %s" % (excs, rel[0], rel[1])
        heds = ""
        heads = [self.df.t_work[0][0][2]]
        if self.df.t_work[0][0][3]:
            heads.append(self.df.t_work[0][0][3])
        for exc in self.df.c_work[5]:
            if not exc[1]:
                break
            c = exc[1]
            v = c.split("(v)")
            for x in range(1, len(v)):
                n = int(v[x][:1])
                d = self.var_det[n][3]
                if isinstance(d, types.StringType) == 1:
                    e = c.split()
                    if e[1] == "Like":
                        d = '"%s%s%s"' % ("%", d, "%")
                    elif e[1] == "SLike":
                        d = '"%s%s"' % ("%", d)
                    elif e[1] == "ELike":
                        d = '"%s%s"' % (d, "%")
                    else:
                        d = '"%s"' % d
                c = c.replace("(v)%s" % n, "%s" % d)
                for y in self.excl1:
                    if v[x-1].find(y) != -1:
                        break
                for z in (" and ", " or "):
                    if v[x].find(z) != -1:
                        break
                h = self.var_det[n][0] + y + "%s" % d
                if not heds:
                    heds = "(%s" % h
                else:
                    heds = heds + ", " + h
            if not excs:
                excs = c
            else:
                excs = excs + " and " + c
        if not excs:
            excs = None
        else:
            excs = excs.replace('"', "'")
        if heds:
            heds = heds + ")"
        heads.append(heds)
        # ORDER
        order = ""
        for ord in self.df.c_work[6]:
            if not ord[1]:
                break
            if ord[3] == "A":
                order = order + ord[1] + " asc,"
            else:
                order = order + ord[1] + " desc,"
        if not order:
            order = None
        else:
            order = order[:-1]
        # SELECT DATA
        sel = "Select"
        col = ""
        for c in cols:
            if not col:
                col = c[0]
            else:
                col = col + ", " + c[0]
        sel = sel + " " + col + " from"
        tab = ""
        for t in tabs:
            if not tab:
                tab = t
            else:
                tab = tab + ", " + t
        sel = sel + " " + tab
        if jons:
            for j in jons:
                if j[0] == "I":
                    sel = sel + " inner join "
                elif j[0] == "L":
                    sel = sel + " left outer join "
                elif j[0] == "R":
                    sel = sel + " right outer join "
                elif j[0] == "F":
                    sel = sel + "full outer join "
                sel = sel + j[1] + " on " + j[2]
        if excs:
            sel = sel + " where " + excs
        if grps:
            sel = sel + " group by " + grps
        if order:
            sel = sel + " order by " + order
        sp = SplashScreen(self.mf.body,
            "Generating the Report\n\n         Please Wait....")
        data = self.mf.sqlRec(sel)
        sp.closeSplash()
        if data:
            # PRINT REPORT
            state = self.df.disableButtons()
            self.rpt = RepPrt(self.mf, name=self.repnam, tables=data,
                ttype="D", heads=heads, cols=cols, stots=stot, gtots=gtot,
                view=self.vf.repprt[0][1], prtnam=self.vf.repprt[0][2],
                mail=self.vf.repprt[1][2])
            self.df.enableButtons(state=state)

    def doClear(self, widget=None):
        self.doAllTabs("disabled")
        self.doRptAskSave()
        self.df.selPage("Tables")
        self.df.focusField("T", 0, 1)

    def doCloseProcess(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

