#! /usr/bin/env python

"""
SYNOPSIS
    Rental Tenants Statements.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, time
from PIL import Image
from TartanClasses import ASD, CCD, GetCtl, ProgressBar, Sql, TartanDialog
from tartanFunctions import doPrinter, doReportlab, flipImage, getModName
from tartanFunctions import getSingleRecords, showError, textFormat
from tartanWork import rttrtp

class rt3040:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.buildHead()
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        gc = GetCtl(self.mf)
        rtlctl = gc.getCtl("rtlctl", self.conum)
        if not rtlctl:
            return "no"
        self.sql = Sql(self.mf.dbm, ["ctlmes", "ctlmst", "rtlmst", "rtltrn",
            "rtlprm", "rtlcon"], prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.title = ("%03i %s" % (self.conum, self.conam),
            "Rental Tenants Statements (%s)" % __name__)
        self.t = None
        self.trtypes = []
        for q in range(0, len(rttrtp)):
            self.trtypes.append(rttrtp[q][0])
        self.maxlines = 21

    def buildHead(self):
        self.head = []
        ctl = self.sql.getRec(tables="ctlmst", where=[("ctm_cono", "=",
            self.conum)], limit=1)
        for col in ("ctm_name", "ctm_add1", "ctm_add2", "ctm_add3",
            "ctm_pcode", "ctm_regno", "ctm_taxno", "ctm_tel", "ctm_fax",
            "ctm_b_name", "ctm_b_ibt", "ctm_b_acno", "ctm_logo", "ctm_flip"):
            exec "self.%s = ctl[self.sql.ctlmst_col.index('%s')]" % (col, col)
        if not self.ctm_logo or not os.path.exists(self.ctm_logo):
            self.ctm_logo = None
        elif self.ctm_flip == "N":
            self.ctm_logo = flipImage(self.ctm_logo)
        if self.ctm_logo:
            self.head.append("%34s%9s" % ("", "STATEMENT"))
            self.head.append(" ")
            self.head.append(" ")
            self.head.append(" ")
            self.head.append(" ")
        else:
            mess = "%1s%-30s%3s%9s" % \
                ("", self.ctm_name, "", "STATEMENT")
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % \
                ("", self.ctm_add1, "", ("RegNo: " + self.ctm_regno))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % \
                ("", self.ctm_add2, "", ("TaxNo: " + self.ctm_taxno))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % \
                ("", self.ctm_add3, "", ("TelNo: " + self.ctm_tel))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % \
                ("", self.ctm_pcode, "", ("FaxNo: " + self.ctm_fax))
            self.head.append(mess)

    def mainProcess(self):
        mss = {
            "stype": "R",
            "tables": ("ctlmes",),
            "cols": (
                ("mss_message", "", 0, "Num"),
                ("mss_detail", "NA", 50, "Details")),
            "where": [("mss_system", "=", "RTL")]}
        r1s = (("Yes","Y"),("No","N"))
        fld = [
            [["T",0,0,0],("IRB",r1s),0,"Whole File","Whole File",
                "Y","Y",self.doWhole,None,None,None],
            [["T",0,1,0],("IRB",r1s),0,"Zero Balances",
                "Include Zero Balances","N","Y",self.doZeros,None,None,None],
            [["T",0,2,0],("IRB",r1s),0,"Minus Balances",
                "Include Minus Balances","N","Y",self.doMinus,None,None,None],
            [["T",0,3,0],"IUI",3,"Message Number","Message Number",
                "","Y",self.doMessno,mss,None,("efld",)],
            [["T",0,4,0],"ID1",10,"Statement Date","Statement Date",
                self.sysdtw,"Y",self.doDat,None,None,("efld",)]]
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.title, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("N","N"), save="N")

    def doWhole(self, frt, pag, r, c, p, i, w):
        self.whole = w

    def doZeros(self, frt, pag, r, c, p, i, w):
        self.zeros = w

    def doMinus(self, frt, pag, r, c, p, i, w):
        self.minus = w

    def doMessno(self, frt, pag, r, c, p, i, w):
        if w:
            self.mes = self.sql.getRec(tables="ctlmes", cols=["mss_detail"],
                where=[("mss_system", "=", "RTL"), ("mss_message", "=", w)],
                limit=1)
            if not self.mes:
                return "Invalid Message Number"
        else:
            self.mes = None

    def doDat(self, frt, pag, r, c, p, i, w):
        self.datew = w
        self.dated = self.df.t_disp[pag][0][p]
        self.curdt = w / 100

    def doEnd(self):
        self.df.closeProcess()
        self.prtnam = self.df.repprt[0][2]
        self.email = self.df.repprt[1][1]
        self.archive = self.df.repprt[2][1]
        self.pdfnam = None
        if self.email == "N":
            self.key = "%s_all" % self.conum
            self.printSetup()
        if self.whole == "N":
            all = getSingleRecords(self.mf, "rtlmst", ("rtm_code",
                "rtm_acno", "rtm_name"), where=[("rtm_cono", "=", self.conum)])
        else:
            whr = [("rtm_cono", "=", self.conum)]
            ord = "rtm_code, rtm_acno"
            all = self.sql.getRec(tables="rtlmst", where=whr, order=ord)
            if not all:
                showError(self.mf.body, "Error", "No Accounts Selected")
        if all:
            num = len(all)
            p = ProgressBar(self.mf.body, max=num, esc=True)
            for x in range(0, num):
                can = p.displayProgress(x)
                if can == "yes":
                    if self.t:
                        self.t.textLine()
                        self.t.textLine("Print Job ABORTED")
                    break
                self.doProcess(all[x])
            p.closeProgress()
            if self.email == "N":
                self.c.save()
                self.doPrint()
        self.mf.closeLoop()
         
    def doProcess(self, rtm):
        self.pgnum = 0
        tmc = self.sql.rtlmst_col
        self.prm = CCD(rtm[tmc.index("rtm_code")], "NA", 7)
        self.acc = CCD(rtm[tmc.index("rtm_acno")], "NA", 7)
        self.tots = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        rtl = self.sql.getRec(tables="rtltrn", cols=["round(sum(rtt_tramt), 2)",
            "round(sum(rtt_taxamt), 2)"], where=[("rtt_cono", "=", self.conum),
            ("rtt_code", "=", self.prm.work), ("rtt_acno", "=", self.acc.work),
            ("rtt_trdt", "<=", self.datew), ("rtt_curdt", "=", self.curdt)],
            limit=1)
        if rtl[0]:
            self.tots[0] = rtl[0]
            self.tots[2] = rtl[0]
            self.tots[5] = rtl[0]
        if rtl[1]:
            self.tots[0] = float(ASD(self.tots[0]) - ASD(rtl[1]))
            self.tots[1] = rtl[1]
        arr = self.sql.getRec(tables="rtltrn",
            cols=["round(sum(rtt_tramt), 2)"], where=[("rtt_cono", "=",
            self.conum), ("rtt_code", "=", self.prm.work), ("rtt_acno", "=",
            self.acc.work), ("rtt_curdt", "<", self.curdt)], limit=1)
        if arr[0]:
            self.tots[3] = arr[0]
            self.tots[5] = float(ASD(self.tots[5]) + ASD(arr[0]))
        if self.zeros == "N" and not self.tots[5]:
            return
        if self.minus == "N" and self.tots[5] < 0:
            return
        self.rtt = self.sql.getRec(tables="rtltrn", cols=["rtt_trdt",
            "rtt_refno", "rtt_type", "rtt_desc", "rtt_cnum",
            "round(sum(rtt_tramt), 2)"], where=[("rtt_cono", "=", self.conum),
            ("rtt_code", "=", self.prm.work), ("rtt_acno", "=", self.acc.work),
            ("rtt_type", "<>", 5), ("rtt_trdt", "<=", self.datew)],
            group="rtt_trdt, rtt_refno, rtt_type, rtt_desc, rtt_cnum",
            order="rtt_trdt, rtt_cnum, rtt_refno")
        if not self.rtt:
            self.rtt = []
        if self.email == "Y":
            self.key = "%s_%s_%s" % (self.conum, self.prm.work, self.acc.work)
            self.printSetup()
        self.name = CCD(rtm[tmc.index("rtm_name")], "NA", 30)
        self.add1 = CCD(rtm[tmc.index("rtm_addr1")], "NA", 30)
        self.emadd = CCD(rtm[tmc.index("rtm_email")], "TX", 0)
        self.vatno = CCD(rtm[tmc.index("rtm_vatnum")], "NA", 10)
        prm = self.sql.getRec(tables="rtlprm", cols=["rtp_desc",
            "rtp_addr1", "rtp_addr2", "rtp_addr3", "rtp_pcode"],
            where=[("rtp_cono", "=", self.conum), ("rtp_code", "=",
            self.prm.work)], limit=1)
        if not prm:
            prm = ["", "", "", "", ""]
        self.desc = CCD(prm[0], "NA", 30)
        if self.add1.work:
            self.add2 = CCD(rtm[tmc.index("rtm_addr2")], "NA", 30)
            self.add3 = CCD(rtm[tmc.index("rtm_addr3")], "NA", 30)
            self.pcod = CCD(rtm[tmc.index("rtm_pcode")], "NA", 4)
        else:
            self.add1 = CCD(prm[1], "NA", 30)
            self.add2 = CCD(prm[2], "NA", 30)
            self.add3 = CCD(prm[3], "NA", 30)
            self.pcod = CCD(prm[4], "NA", 4)
        self.printHeader()
        self.printBody()
        self.printTrailer()
        if self.email == "Y":
            self.c.save()
            self.doPrint()

    def printSetup(self):
        if self.email == "Y" or not self.pdfnam:
            if self.archive == "Y":
                dir = self.mf.arcdir
            else:
                dir = self.mf.wrkdir
            self.pdfnam = getModName(dir, __name__, self.key, ext="pdf")
        self.c, self.cpi, self.inch, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam)
        # Width in pixels of each character
        self.cwidth = self.c.stringWidth(("*"), "Tartan", self.cpi)

    def doPrint(self):
        if self.prtnam == "view":
            view = "Y"
        else:
            view = "N"
        if self.email == "N" or not self.emadd.work:
            mail = "N"
        else:
            mail = self.emadd.work
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.title, view=view, prtnam = self.prtnam, mail=mail)

    def printHeader(self):
        self.pageHeading()
        self.drawDocument()
        self.pgnum = self.pgnum + 1
        self.t.textLine()
        self.t.textLine()
        self.t.setFont("Tartan-Bold", 18)
        self.t.textLine(self.head[0])
        self.t.setFont("Tartan-Bold", 10)
        self.t.textLine(self.head[1])
        self.t.textLine(self.head[2])
        self.t.textLine(self.head[3])
        self.t.textLine(self.head[4])
        self.t.textLine()
        self.t.setFont("Tartan", 10)
        mess = "%-70s%7s" % ("", "")
        self.t.textLine(mess)
        mess = "%1s%-30s" % ("", self.name.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s%39s%7s" % ("", self.add1.disp, "", self.prm.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s" % ("", self.add2.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s%39s%7s" % ("", self.add3.disp, "", self.acc.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s" % ("", self.pcod.disp)
        self.t.textLine(mess)
        mess = "%67s%-10s" % ("", self.dated)
        self.t.textLine(mess)
        self.t.textLine()
        self.t.textLine()
        self.t.setFont("Tartan", 10)
        self.pglin = 0

    def pageHeading(self):
        self.c.translate(self.inch, self.inch) # Set x and y margins
        self.t = self.c.beginText()
        self.t.setFont("Tartan-Bold", self.cpi)
        self.t.setTextOrigin(0, 0)

    def printBody(self):
        obal = 0.0
        if len(self.rtt) > 20:
            chk = len(self.rtt) - 20
        else:
            chk = 0
        for num, rtt in enumerate(self.rtt):
            trdt = CCD(rtt[0], "d1", 10)
            refno = CCD(rtt[1], "Na", 9)
            trtp = CCD(rtt[2], "UI", 1)
            desc = CCD(rtt[3].upper(), "NA", 35)
            cnum = CCD(rtt[4], "UI", 3)
            tramt = CCD(rtt[5], "SD", 11.2)
            if chk:
                if num < chk:
                    obal = float(ASD(obal) + ASD(tramt.work))
                    continue
                elif num == chk:
                    obal = CCD(obal, "SD", 11.2)
                    mess = "%2s%s %s %-3s %-35s %s %s" % ("", trdt.disp,
                        "Bal/B-Fwd", "B/F", "BALANCE BROUGHT FORWARD",
                        cnum.disp, obal.disp)
                    self.t.textLine(mess)
                    self.pglin = self.pglin + 1
            mess = "%2s%s %s %s %s %s %s" % ("", trdt.disp, refno.disp,
                self.trtypes[trtp.work - 1], desc.disp, cnum.disp, tramt.disp)
            self.t.textLine(mess)
            self.pglin = self.pglin + 1

    def printTrailer(self):
        tots = [""]
        for t in self.tots:
            y = CCD(t, "SD", 11.2)
            tots.append(y.disp)
        mess = "%2s%s %s  %s  %s  %s   %s" % tuple(tots)
        for z in range(0, (self.maxlines + 2 - self.pglin)):
            self.t.textLine()
        self.t.textLine(mess)
        # Draw Property and Banking Details Block
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*78
        b1 = 42.50
        b2 = 43.40
        self.c.rect(x, (b1*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
        self.c.rect(x, (b1*(self.inch/6)), w, (4.75*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 10)
        self.c.drawString(self.cwidth*1, (b2*(self.inch/6)),
            "Property and Banking Details")
        self.c.setFont("Tartan", 10)
        self.t.textLine()
        self.t.textLine()
        self.t.textLine()
        mess = " Property: %-30s" % self.desc.disp
        if self.vatno.work:
            mess = "%s%14sVat-Number: %10s" % (mess, " ", self.vatno.disp)
        self.t.textLine(mess)
        self.t.textLine()
        self.t.textLine(" Bank: %-20s  Branch: %-8s  Account: %-16s" % \
            (self.ctm_b_name, self.ctm_b_ibt, self.ctm_b_acno))
        self.t.textLine()
        self.t.textLine()
        # Check for Message
        if self.mes:
            mes2 = self.mes[:]
        else:
            mes2 = None
        if mes2:
            # Draw Message Rectangles
            self.c.setFillGray(.8)
            x = self.cwidth*.5
            w = self.cwidth*78
            b1 = 48.50
            b2 = 49.30
            self.c.rect(x, (b1*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
            self.c.rect(x, (b1*(self.inch/6)), w, (4.75*(self.inch/6)), fill=0)
            self.c.setFillGray(0)
            self.c.setFont("Tartan-Bold", 10)
            self.c.drawString(self.cwidth*1,(b2*(self.inch/6)),"Message")
            self.c.setFont("Tartan", 10)
            self.t.textLine()
            mes3 = textFormat(mes2[0], width=75)
            for mess in mes3[:3]:
                self.t.textLine(" " + mess)
        self.c.drawText(self.t)
        self.c.showPage()

    def drawDocument(self):
        # If logo, draw logo
        if self.ctm_logo:
            self.c.drawImage(self.ctm_logo, 0, 3, 350, 70, None)
        # Draw Account Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*32.5
        self.c.rect(x, (7.75*(self.inch/6)), w, (1*(self.inch/6)), fill=1)
        self.c.rect(x, (7.75*(self.inch/6)), w, (7.25*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 8)
        self.c.drawString(self.cwidth*1, (8.45*(self.inch/6)), "Account")
        self.c.setFont("Tartan", 10)
        # Draw Account Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*64.5
        w = self.cwidth*14
        for q in (9, 11, 13):
            self.c.rect(x, (q*(self.inch/6)), w, (1*(self.inch/6)), fill=1)
            self.c.rect(x, (q*(self.inch/6)), w, (2*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 8)
        self.c.drawCentredString((self.cwidth*71.5),
            (9.80*(self.inch/6)), "Premises")
        self.c.drawCentredString((self.cwidth*71.5),
            (11.80*(self.inch/6)), "Tenant")
        self.c.drawCentredString((self.cwidth*71.5),
            (13.80*(self.inch/6)), "Date")
        self.c.setFont("Tartan", 10)
        # Draw Ribbon Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*78
        b1 = 22.75
        b2 = 38.25
        self.c.rect(x, (15.50*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
        self.c.rect(x, (15.50*(self.inch/6)), w, (b1*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 10)
        for q, r in ((2, "   Date"), (13, "Reference"), (23, "Typ"),
            (27, "Details"), (63, "CCC"), (67, "    Amount")):
            self.c.drawString(self.cwidth*q, (16.40*(self.inch/6)), r)
        self.c.setFont("Tartan", 10)
        for q in (12.5, 22.5, 26.5, 62.5, 66.5):
            self.c.line(self.cwidth*q, (15.50*(self.inch/6)), self.cwidth*q,
                (b2*(self.inch/6)))
        # Draw Ageing Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*78
        b1 = 38.50
        b2 = 41.25
        b3 = 39.40
        self.c.rect(x, (b1*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
        self.c.rect(x, (b1*(self.inch/6)), w, (2.75*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 10)
        for q, r in ((2, "    Rental"), (14, "    V.A.T."), (26, " Tot-Rental"),
            (39, "    Arrears"), (52, "   Interest"), (65, "   Total-Due")):
            self.c.drawString(self.cwidth*q, (b3*(self.inch/6)), r)
        self.c.setFont("Tartan", 10)
        for q in (13.5, 25.5, 38.5, 51.5, 64.5):
            self.c.line(self.cwidth*q, (b1*(self.inch/6)), self.cwidth*q,
                (b2*(self.inch/6)))

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    from TartanClasses import MainFrame
    mf = MainFrame()
    mf.dbm.openDbase()
    rc3040(mf, 1, "Company 1")
