#! /usr/bin/env python

"""
SYNOPSIS
    Sales By Product Report.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, CCD, GetCtl, ProgressBar, Sql
from TartanClasses import TartanDialog
from tartanFunctions import doReportlab, doWriteExport, getModName, doPrinter
from tartanFunctions import showError

class si3020:
    def __init__(self, mf, coy, nam, period):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = period
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["strgrp", "strloc", "strmf1"],
            prog=__name__)
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "strctl"
        self.locs = strctl["cts_locs"]
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i) %6s" % (t[0],
            t[1], t[2], t[3], t[4], __name__)
        self.head = ("%03u %-30s %s" % (self.conum, self.conam, "%s"))
        self.colsh = [["", "", "", "",
            [ "*********** Month to Date **********", 4, 7],
            ["*********** Year To Date ***********", 8, 11]]]
        self.colsh.append(["Grp", "Product-Code", "Description", "U.O.I",
            "Quantity", "Value", "Profit", "Prft-%", "Quantity", "Value",
            "Profit", "Prft-%"])
        self.forms = [("UA", 3), ("NA", 20), ("NA", 30), ("NA", 10), ("SL", 9),
            ("SL", 9), ("SL", 9), ("SD", 7.2), ("SL", 9), ("SL", 9), ("SL", 9),
            ("SD", 7.2)]
        self.stot = [0] * 6
        self.gtot = [0] * 6
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Sales By Product Repoprt (%s)" % __name__)
        loc = {
            "stype": "R",
            "tables": ("strloc",),
            "cols": (("srl_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Description", "Y")),
            "where": [("srl_cono", "=", self.conum)]}
        grp = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        r1s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),"ID2",7,"Cut Off Period","Cut Off Period",
                (self.sysdtw/100),"Y",self.doCut,None,None,("efld",)),
            (("T",0,1,0),"IUA",1,"Location","Location",
                "","Y",self.doLoc,loc,None,("efld",)),
            (("T",0,2,0),"IUA",3,"Product Group","Product Group",
                "","Y",self.doGroup,grp,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"))

    def doCut(self, frt, pag, r, c, p, i, w):
        self.cutoff = w
        self.cutper = self.df.t_disp[pag][r][p]
        if self.locs == "N":
            self.loc = ""
            self.locd = "ALL"
            return "sk"

    def doLoc(self, frt, pag, r, c, p, i, w):
        if not w:
            self.locd = "ALL"
        else:
            acc = self.sql.getRec(tables="strloc", cols=["srl_desc"],
                where=[("srl_cono", "=", self.conum), ("srl_loc", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Location"
            self.locd = "%s %s" % (w, acc[0])
        self.loc = w

    def doGroup(self, frt, pag, r, c, p, i, w):
        if w:
            acc = self.getGroup(w)
            if not acc:
                return "Invalid Group"
        self.group = w

    def doEnd(self):
        self.df.closeProcess()
        if not self.group:
            sgrp = ""
            egrp = "zzz"
        else:
            sgrp = egrp = self.group
        all = self.sql.getRec(tables="strmf1", cols=["st1_group", "st1_code",
            "st1_desc", "st1_uoi"], where=[("st1_cono", "=", self.conum),
            ("st1_group", ">=", sgrp), ("st1_group", "<=", egrp)],
            order="st1_group, st1_code")
        if not all:
            showError(self.mf.body, "Processing Error",
            "No Records Selected")
        elif self.df.repprt[0][2] == "export":
            self.exportReport(all)
        else:
            self.printReport(all)
        self.closeProcess()

    def exportReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        expnam = getModName(self.mf.wrkdir, __name__, self.conum)
        self.expheads = [self.head % self.sysdttm]
        self.expheads.append("Sales Report By Product up to %s" % self.cutper)
        self.expcolsh = self.colsh
        self.expforms = self.forms
        self.expdatas = []
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                p.closeProgress()
                return
            vals = self.getValues(dat)
            if not vals:
                continue
            code, desc, uoi, mq, ms, mp, mn, yq, ys, yp, yn = vals
            self.expdatas.append(["BODY", [self.grp.work, code.work, desc.work,
                uoi.work, mq.work, ms.work, mp.work, mn.work, yq.work, ys.work,
                yp.work, yn.work]])
        p.closeProgress()
        self.grandTotal()
        doWriteExport(xtype=self.df.repprt[0][1], name=expnam, sheet=__name__,
            heads=self.expheads, colsh=self.expcolsh, forms=self.expforms,
            datas=self.expdatas, rcdic=self.mf.rcdic)

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %-32s %-33s %-33s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        pdfnam = getModName(self.mf.wrkdir, __name__, self.conum, ext="pdf")
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, pdfnam, self.head)
        lstgrp = ""
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            vals = self.getValues(dat)
            if not vals:
                continue
            if lstgrp and lstgrp != self.grp.work:
                self.groupTotal()
                self.pglin = 99
            if self.pglin > self.lpp:
                if self.pgnum != 0:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            code, desc, uoi, mq, ms, mp, mn, yq, ys, yp, yn = vals
            self.t.textLine("%s %s %s %s %s %s %s     %s %s %s %s" % \
                (code.disp, desc.disp, uoi.disp, mq.disp, ms.disp,
                mp.disp, mn.disp, yq.disp, ys.disp, yp.disp, yn.disp))
            self.stot[0] = float(ASD(self.stot[0]) + ASD(mq.work))
            self.stot[1] = float(ASD(self.stot[1]) + ASD(ms.work))
            self.stot[2] = float(ASD(self.stot[2]) + ASD(mp.work))
            self.stot[3] = float(ASD(self.stot[3]) + ASD(yq.work))
            self.stot[4] = float(ASD(self.stot[4]) + ASD(ys.work))
            self.stot[5] = float(ASD(self.stot[5]) + ASD(yp.work))
            self.pglin = self.pglin + 1
            lstgrp = self.grp.work
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.groupTotal()
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])

    def getValues(self, data):
        self.grp = CCD(data[0], "UA", 3)
        code = CCD(data[1], "NA", 20)
        desc = CCD(data[2], "NA", 30)
        uoi = CCD(data[3], "NA", 10)
        bals = Balances(self.mf, "STR", self.conum, self.cutoff, keys=\
            (self.grp.work, code.work, self.loc, ("P", self.period[0])))
        m_ob, m_mv, m_cb, y_ob, y_mv, y_cb, ac, lc = bals.doStrBals()
        m_qty = 0
        m_cst = 0
        m_sls = 0
        for t, q, c, s in m_mv:
            if t == 7:
                m_qty = q
                m_cst = c
                m_sls = s
                break
        y_qty = 0
        y_cst = 0
        y_sls = 0
        for t, q, c, s in y_mv:
            if t == 7:
                y_qty = q
                y_cst = c
                y_sls = s
                break
        mq = CCD(float(ASD(0) - ASD(m_qty)), "SL", 9)
        mc = CCD(float(ASD(0) - ASD(m_cst)), "SL", 9)
        ms = CCD(float(ASD(0) - ASD(m_sls)), "SL", 9)
        mp = float(ASD(ms.work) - ASD(mc.work))
        mp = CCD(mp, "SL", 9)
        if ms.work == 0:
            mn = 0
        else:
            mn = round((mp.work * 100 / ms.work), 2)
        mn = CCD(mn, "SD", 7.2)
        yq = CCD(float(ASD(0) - ASD(y_qty)), "SL", 9)
        yc = CCD(float(ASD(0) - ASD(y_cst)), "SL", 9)
        ys = CCD(float(ASD(0) - ASD(y_sls)), "SL", 9)
        yp = float(ASD(ys.work) - ASD(yc.work))
        yp = CCD(yp, "SL", 9)
        if not ys.work:
            yn = 0
        else:
            yn = round((yp.work * 100.0 / ys.work), 2)
        yn = CCD(yn, "SD", 7.2)
        if mq.work == 0 and mc.work == 0 and ms.work == 0 and \
            yq.work == 0 and yc.work == 0 and ys.work == 0:
            return
        self.gtot[0] = float(ASD(self.gtot[0]) + ASD(mq.work))
        self.gtot[1] = float(ASD(self.gtot[1]) + ASD(ms.work))
        self.gtot[2] = float(ASD(self.gtot[2]) + ASD(mp.work))
        self.gtot[3] = float(ASD(self.gtot[3]) + ASD(yq.work))
        self.gtot[4] = float(ASD(self.gtot[4]) + ASD(ys.work))
        self.gtot[5] = float(ASD(self.gtot[5]) + ASD(yp.work))
        return (code, desc, uoi, mq, ms, mp, mn, yq, ys, yp, yn)
        
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-29s %-7s %98s %5s" % \
            ("Sales Report By Product up to", self.cutper, "Page", self.pgnum))
        self.t.textLine()
        acc = self.getGroup(self.grp.work)
        if acc:
            grpd = acc[0]
        else:
            grpd = "Missing Group Record"
        self.t.textLine("%s %s %s     %s %s" % \
            ("Group:", self.grp.disp, grpd, "Location:", self.locd))
        self.t.textLine()
        self.t.textLine("%-62s %-36s %-5s%-36s " % ("",
            "*********** Month T0 Date **********", "",
            "*********** Year To Date ***********"))
        self.t.textLine("%-20s %-30s %-10s %9s %9s %9s %7s     "\
            "%9s %9s %9s %7s" % ("Product-Code", "Description", "U.O.I",
            "Quantity ", "Value ", "Profit ", "Prft-% ",
            "Quantity ", "Value ", "Profit ", "Prft-% "))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 9

    def getGroup(self, grp):
        acc = self.sql.getRec(tables="strgrp", cols=["gpm_desc"],
            where=[("gpm_cono", "=", self.conum), ("gpm_group", "=", grp)],
            limit=1)
        return acc

    def groupTotal(self):
        self.t.textLine()
        mq = CCD(self.stot[0], "SL", 9)
        ms = CCD(self.stot[1], "SL", 9)
        mp = CCD(self.stot[2], "SL", 9)
        if ms.work == 0:
            mn = 0
        else:
            mn = round((mp.work * 100 / ms.work), 2)
        mn = CCD(mn, "SD", 7.2)
        yq = CCD(self.stot[3], "SL", 9)
        ys = CCD(self.stot[4], "SL", 9)
        yp = CCD(self.stot[5], "SL", 9)
        if ys.work == 0:
            yn = 0
        else:
            yn = round((yp.work * 100 / ys.work), 2)
        yn = CCD(yn, "SD", 7.2)
        self.t.textLine("%-20s %-41s %s %s %s %s     %s %s %s %s" % \
            ("", "Group Totals", mq.disp, ms.disp, mp.disp, mn.disp,
            yq.disp, ys.disp, yp.disp, yn.disp))
        self.stot = [0] * 6

    def grandTotal(self):
        mq = CCD(self.gtot[0], "SL", 9)
        ms = CCD(self.gtot[1], "SL", 9)
        mp = CCD(self.gtot[2], "SL", 9)
        if ms.work == 0:
            mn = 0
        else:
            mn = round((mp.work * 100 / ms.work), 2)
        mn = CCD(mn, "SD", 7.2)
        yq = CCD(self.gtot[3], "SL", 9)
        ys = CCD(self.gtot[4], "SL", 9)
        yp = CCD(self.gtot[5], "SL", 9)
        if ys.work == 0:
            yn = 0
        else:
            yn = round((yp.work * 100 / ys.work), 2)
        yn = CCD(yn, "SD", 7.2)
        if self.df.repprt[0][2] == "export":
            self.expdatas.append(["ULINES"])
            self.expdatas.append(["TOTAL", ["", "", "Grand Totals", "", mq.work,
                ms.work, mp.work, mn.work, yq.work, ys.work, yp.work, yn.work]])
            self.expdatas.append(["ULINED"])
            return
        self.t.textLine()
        self.t.textLine("%-20s %-41s %s %s %s %s     %s %s %s %s" % \
            ("", "Grand Totals", mq.disp, ms.disp, mp.disp, mn.disp,
            yq.disp, ys.disp, yp.disp, yn.disp))

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

