#! /usr/bin/env python

"""
SYNOPSIS
    Product Sales History Report.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, CreateChart, CCD, GetCtl, ProgressBar
from TartanClasses import Sql, TartanDialog
from tartanFunctions import doReportlab, doPrinter, doWriteExport, getModName
from tartanFunctions import getSingleRecords, showError
from tartanWork import mthnam

class si3040:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["strgrp", "strloc", "strmf1"],
            prog=__name__)
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "strctl"
        self.locs = strctl["cts_locs"]
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i) %6s" % (t[0],
            t[1], t[2], t[3], t[4], __name__)
        self.head = ("%03u %-30s %s" % (self.conum, self.conam, "%s"))
        self.forms = [("UA",3),("NA",20),("NA",30),("NA",10)] + [("SL",10)] * 12
        self.stots = [0] * 12
        self.gtots = [0] * 12
        self.mchart = []
        self.tipe = None
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Product Sales History (%s)" % __name__)
        loc = {
            "stype": "R",
            "tables": ("strloc",),
            "cols": (("srl_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Description", "Y")),
            "where": [("srl_cono", "=", self.conum)]}
        grp = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        chm = {
            "stype": "R",
            "tables": ("drschn",),
            "cols": (
                ("chm_chain", "", 0, "Chn"),
                ("chm_name", "", 0, "Name", "Y")),
            "where": (("chm_cono", "=", self.conum),)}
        r1s = (("Quantity","Q"),("Value","V"),("Profit","P"))
        r2s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),"ID2",7,"Cut Off Period","Cut Off Period",
                self.sysdtw/100,"Y",self.doCut,None,None,("efld",)),
            (("T",0,1,0),("IRB",r1s),0,"Type","Type",
                "V","Y",self.doType,None,None,None),
            (("T",0,2,0),"IUA",1,"Location","Location",
                "","Y",self.doLoc,loc,None,("efld",)),
            (("T",0,3,0),"IUA",3,"Product Group","Product Group",
                "","Y",self.doGroup,grp,None,None),
            (("T",0,4,0),"IUI",3,"Chain Store","Chain Store",
                0,"Y",self.doDrsChn,chm,None,None),
            (("T",0,5,0),("IRB",r2s),0,"All Accounts","All Accounts",
                "Y","Y",self.doDrsAcc,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"))

    def doCut(self, frt, pag, r, c, p, i, w):
        self.coffw = w
        self.coffd = self.df.t_disp[pag][r][p]

    def doType(self, frt, pag, r, c, p, i, w):
        self.rtype = w
        if self.locs == "N":
            self.loc = ""
            self.locd = "ALL"
            return "sk"

    def doLoc(self, frt, pag, r, c, p, i, w):
        if not w:
            self.locd = "ALL"
        else:
            acc = self.sql.getRec(tables="strloc", cols=["srl_desc"],
                where=[("srl_cono", "=", self.conum), ("srl_loc", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Location"
            self.locd = "%s %s" % (w, acc[0])
        self.loc = w

    def doGroup(self, frt, pag, r, c, p, i, w):
        if w:
            acc = self.getGroup(w)
            if not acc:
                return "Invalid Group"
        self.grp = w

    def doDrsChn(self, frt, pag, r, c, p, i, w):
        self.chain = w

    def doDrsAcc(self, frt, pag, r, c, p, i, w):
        self.acno = []
        if w == "Y":
            return
        whr = [("drm_cono", "=", self.conum)]
        if self.chain:
            whr.append(("drm_chain", "=", self.chain))
        acc = getSingleRecords(self.mf, "drsmst", ("drm_acno",
            "drm_name"), where=whr, order="drm_chain, drm_acno")
        for a in acc:
            self.acno.append(a[2])

    def doEnd(self):
        self.df.closeProcess()
        if not self.grp:
            sgrp = ""
            egrp = "zzz"
        else:
            sgrp = egrp = self.grp
        all = self.sql.getRec(tables="strmf1", cols=["st1_group", "st1_code",
            "st1_desc", "st1_uoi"], where=[("st1_cono", "=", self.conum),
            ("st1_group", ">=", sgrp), ("st1_group", "<=", egrp)],
            order="st1_group, st1_code")
        if not all:
            showError(self.mf.body, "Processing Error", "No Records Selected")
        else:
            self.start = self.coffw - 99
            if self.start % 100 == 13:
                self.start = self.start + 88
            s = self.start % 100
            self.mthhead = ""
            self.colsh = ["Grp", "Stock-Code", "Description", "U.O.I."]
            for x in range(12):
                if self.mthhead:
                    self.mthhead = "%s %9s " % (self.mthhead, mthnam[s][1])
                else:
                    self.mthhead = "%9s " % mthnam[s][1]
                self.colsh.append(mthnam[s][1])
                s += 1
                if s == 13:
                    s = 1
            if self.df.repprt[0][2] == "export":
                self.exportReport(all)
            else:
                self.printReport(all)
        self.closeProcess()

    def exportReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        expnam = getModName(self.mf.wrkdir, __name__, self.conum)
        self.expheads = [self.head % self.sysdttm]
        self.expheads.append("Stores Sales History for the 12 Month Period "\
            "to %s" % self.coffd)
        self.expcolsh = [self.colsh]
        self.expforms = self.forms
        self.expdatas = []
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                p.closeProgress()
                return
            vals = self.getValues(dat)
            if not vals:
                continue
            cod, dsc, uoi, amt, purchd, purchw = vals
            line = ["BODY", [self.grp.work,cod.work,dsc.work,uoi.work] + purchw]
            self.expdatas.append(line)
        p.closeProgress()
        self.grandTotal()
        doWriteExport(xtype=self.df.repprt[0][1], name=expnam, sheet=__name__,
            heads=self.expheads, colsh=self.expcolsh, forms=self.expforms,
            datas=self.expdatas, rcdic=self.mf.rcdic)

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %49s %33s %60s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        pdfnam = getModName(self.mf.wrkdir, __name__, self.conum, ext="pdf")
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        self.lstgrp = ""
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            vals = self.getValues(dat)
            if not vals:
                continue
            cod, dsc, uoi, amt, purchd, purchw = vals
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            self.t.textLine("%s %s %s %s %s" % (self.grp.disp, cod.disp,
                dsc.disp, uoi.disp, purchd))
            self.pglin = self.pglin + 1
            self.lstgrp = self.grp.work
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.groupTotal()
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
        if self.df.repprt[0][1] == "X" or not self.tipe:
            return
        CreateChart(self.mf, self.conum, self.conam, [self.start, self.coffw],
            [self.tit, self.tipe], None, self.mchart, ylab=self.tipe)

    def getValues(self, data):
        self.grp = CCD(data[0], "UA", 3)
        if self.df.repprt[0][2] != "export":
            if self.lstgrp and self.lstgrp != self.grp.work:
                self.groupTotal()
        cod = CCD(data[1], "NA", 20)
        dsc = CCD(data[2], "NA", 30)
        uoi = CCD(data[3], "NA", 10)
        bals = Balances(self.mf, "STR", self.conum, self.coffw,
            keys=(self.grp.work, cod.work, self.loc, ("S", self.start)))
        more = []
        if self.chain:
            more.append(("stt_chain", "=", self.chain))
        if self.acno:
            more.append(("stt_acno", "in", tuple(self.acno)))
        this, hist = bals.doStrHist(more=more)
        bals=hist[2]
        purchd = ""
        purchw = []
        tot = 0
        for x in range(11, -1, -1):
            if self.rtype == "Q":
                amt = float(ASD(0) - ASD(bals[x][0]))
            elif self.rtype == "V":
                amt = float(ASD(0) - ASD(bals[x][2]))
            else:
                amt = float(ASD(0) - ASD(bals[x][2]) + ASD(bals[x][1]))
            amt = CCD(round(amt, 0), "SL", 10)
            tot = float(ASD(tot) + ASD(amt.work))
            self.stots[x] = float(ASD(self.stots[x]) + ASD(amt.work))
            self.gtots[x] = float(ASD(self.gtots[x]) + ASD(amt.work))
            purchd = purchd + amt.disp
            purchw.append(amt.work)
        if tot == 0:
            return
        return (cod, dsc, uoi, amt, purchd, purchw)
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-47s %-7s %124s %5s" % \
            ("Stores Sales History for the 12 Month Period to", self.coffd,
            "Page", self.pgnum))
        self.t.textLine()
        if not self.loc:
            loc = "All"
        else:
            loc = self.loc
        if not self.grp:
            grp = "All"
        else:
            grp = self.grp
        acc = self.getGroup(self.grp.work)
        if acc:
            grpd = acc[0]
        else:
            grpd = "Missing Group Record"
        if self.rtype == "Q":
            self.tipe = "Quantities"
        elif self.rtype == "V":
            self.tipe = "Values"
        else:
            self.tipe = "Profits"
        self.t.textLine("%s %s %s    %s %s    %s" % \
            ("Group:", self.grp.disp, grpd, "Location:", self.locd, self.tipe))
        self.t.textLine()
        self.t.textLine("%-3s %-20s %-30s %-10s %s" % \
            ("Grp", "Stock-Code", "Description", "U.O.I.", self.mthhead))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def getGroup(self, grp):
        acc = self.sql.getRec(tables="strgrp", cols=["gpm_desc"],
            where=[("gpm_cono", "=", self.conum), ("gpm_group", "=", grp)],
            limit=1)
        return acc

    def groupTotal(self):
        purchd = ""
        mchart = ["T", "Group %s Totals" % self.lstgrp]
        p = None
        for x in range(11,-1,-1):
            a = CCD(self.stots[x], "SL", 10)
            self.stots[x] = 0
            purchd = purchd + a.disp
            mchart.append(a.work)
            if a.work:
                p = "y"
        if p:
            self.mchart.append(mchart)
            self.t.textLine()
            self.t.textLine("%-24s %-41s %s" % (" ", mchart[1], purchd))
            self.c.drawText(self.t)
            self.c.showPage()
            self.pageHeading()

    def grandTotal(self):
        if self.df.repprt[0][2] == "export":
            self.expdatas.append(["ULINES"])
            self.expdatas.append(["TOTAL",
                ["", "", "Grand Totals", ""] + self.gtots])
            self.expdatas.append(["ULINED"])
            return
        purchd = ""
        mchart = ["T", "Grand Totals"]
        for x in range(11,-1,-1):
            a = CCD(self.gtots[x], "SL", 10)
            purchd = purchd + a.disp
            mchart.append(a.work)
        self.mchart.append(mchart)
        self.t.textLine()
        self.t.textLine("%-24s %-41s %s" % (" ", mchart[1], purchd))
        self.t.textLine()

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

