#! /usr/bin/env python

"""
SYNOPSIS
    Sales By Customer By Product Report.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, GetCtl, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError

class si3050:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["drsact", "drstyp", "drsmst", "strmf1",
            "strtrn"], prog=__name__)
        if self.sql.error:
            return self.sql.error
        gc = GetCtl(self.mf)
        drsctl = gc.getCtl("drsctl", self.conum)
        if not drsctl:
            return "no"
        self.chains = drsctl["ctd_chain"]
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Cust/Product Sales Report (%s)" % __name__)
        chm = {
            "stype": "R",
            "tables": ("drschn",),
            "cols": (
                ("chm_chain", "", 0, "Chn"),
                ("chm_name", "", 0, "Name", "Y")),
            "where": (("chm_cono", "=", self.conum),)}
        drm = {
            "stype": "R",
            "tables": ("drsmst",),
            "cols": (
                ("drm_acno", "", 0, "Acc-Num"),
                ("drm_name", "", 0, "Name", "Y")),
            "order": "drm_acno"}
        if self.chains == "Y":
            drm["where"] = [("drm_cono", "=", self.conum)]
            drm["whera"] = [["T", "drm_chain", 2]]
        else:
            drm["where"] = [("drm_cono", "=", self.conum)]
        dra = {
            "stype": "R",
            "tables": ("drsact",),
            "cols": (
                ("dac_code", "", 0, "Cod"),
                ("dac_desc", "", 0, "Description", "Y"))}
        drt = {
            "stype": "R",
            "tables": ("drstyp",),
            "cols": (
                ("dtp_code", "", 0, "Cod"),
                ("dtp_desc", "", 0, "Description", "Y"))}
        r1s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),"ID2",7,"Start Period","Start Period",
                (self.sysdtw/100),"Y",self.doSdate,None,None,("efld",)),
            (("T",0,1,0),"ID2",7,"End Period","End Period",
                (self.sysdtw/100),"Y",self.doEdate,None,None,("efld",)),
            (("T",0,2,0),"IUI",3,"Chain Store","Chain Store",
                "","Y",self.doDrsChn,chm,None,None),
            (("T",0,3,0),"INA",7,"Account Number","Account Number",
                "","Y",self.doDrsAcc,drm,None,None),
            (("T",0,4,0),"IUA",3,"Business Activity","Business Activity",
                "","Y",self.doDrsAct,dra,None,None),
            (("T",0,5,0),"IUA",3,"Business Type","Business Type",
                "","Y",self.doDrsTyp,drt,None,None),
            (("T",0,6,0),("IRB",r1s),0,"Sales Value Only","Sales Value Only",
                "Y","Y",self.doVals,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doSdate(self, frt, pag, r, c, p, i, w):
        self.sdate = w

    def doEdate(self, frt, pag, r, c, p, i, w):
        self.edate = w
        if self.chains == "N":
            self.chain = 0
            return "sk1"

    def doDrsChn(self, frt, pag, r, c, p, i, w):
        self.chain = w

    def doDrsAcc(self, frt, pag, r, c, p, i, w):
        self.acno = w
        if self.chain or self.acno:
            self.drsact = ""
            self.drstyp = ""
            return "sk2"

    def doDrsAct(self, frt, pag, r, c, p, i, w):
        if w:
            acc = self.sql.getRec(tables="drsact", where=[("dac_code", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Account Activity"
        self.drsact = w

    def doDrsTyp(self, frt, pag, r, c, p, i, w):
        if w:
            acc = self.sql.getRec(tables="drstyp", where=[("dtp_code", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Account Activity"
        self.drstyp = w

    def doVals(self, frt, pag, r, c, p, i, w):
        self.vals = w

    def doEnd(self):
        self.df.closeProcess()
        tab = ["strmf1", "strtrn"]
        col = ["stt_group", "stt_code", "stt_loc", "stt_type", "stt_ref1",
            "stt_ref2", "stt_trdt", "stt_desc", "stt_chain", "stt_acno",
            "round(sum(stt_qty),2)", "round(sum(stt_cost),2)",
            "round(sum(stt_sell),2)", "st1_desc"]
        whr = [
            ("stt_cono", "=", self.conum),
            ("stt_type", "=", 7),
            ("stt_cono=st1_cono",),
            ("stt_group=st1_group",),
            ("stt_code=st1_code",),
            ("stt_curdt", ">=", self.sdate),
            ("stt_curdt", "<=", self.edate)]
        if self.chain:
            whr.append(("stt_chain", "=", self.chain))
        if self.acno:
            whr.append(("stt_acno", "=", self.acno))
        if self.drsact or self.drstyp:
            tab.append("drsmst")
            whr.extend([
                ("stt_chain=drm_chain",),
                ("stt_acno=drm_acno",)])
            if self.drsact:
                whr.append(("drm_bus_activity", "=", self.drsact))
            if self.drstyp:
                whr.append(("drm_bus_type", "=", self.drstyp))
        grp = "stt_group, stt_code, stt_loc, stt_type, stt_ref1, stt_ref2, "\
            "stt_trdt, stt_desc, stt_chain, stt_acno, st1_desc"
        ord = "stt_chain, stt_acno, stt_type, stt_ref1, stt_group, stt_code"
        all = self.sql.getRec(tables=tab, cols=col, where=whr, group=grp,
            order=ord)
        if not all:
            showError(self.mf.body, "Processing Error",
            "No Records Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        if self.vals == "Y":
            spc1 = " " * 12; spc2 = " " * 11
        else:
            spc1 = " " * 28; spc2 = " " * 27
        self.head = ("%03u %-30s %s %35s %s %6s" % \
            (self.conum, self.conam, spc1, self.sysdttm, spc2, __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.dtot = [0] * 3
        self.gtot = [0] * 3
        old_chn = all[0][8]
        old_drs = all[0][9]
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            self.chn = CCD(dat[8], "UI", 3)
            self.drs = CCD(dat[9], "UA", 7)
            self.grp = CCD(dat[0], "NA", 3)
            self.cod = CCD(dat[1], "NA", 20)
            typ = CCD(dat[3], "UI", 1)
            rf1 = CCD(dat[4], "Na", 9)
            rf2 = CCD(dat[5], "Na", 9)
            qty = CCD(float(ASD(0) - ASD(dat[10])), "SD", 11.2)
            cst = CCD(float(ASD(0) - ASD(dat[11])), "SD", 11.2)
            sll = CCD(float(ASD(0) - ASD(dat[12])), "SD", 11.2)
            des = CCD(dat[13], "NA", 30)
            prf = float(ASD(sll.work) - ASD(cst.work))
            prf = CCD(prf, "SD", 11.2)
            if sll.work == 0:
                pcn = 0
            else:
                pcn = round((prf.work * 100.0 / sll.work), 2)
            pcn = CCD(pcn, "SD", 7.2)
            if old_chn != self.chn.work or old_drs != self.drs.work:
                self.drsTotal()
                self.pglin = 99
            if self.pglin > self.lpp:
                if self.pgnum != 0:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            if self.vals == "Y":
                self.t.textLine("%s %s %s %s %s %s %s %s" % \
                    (rf1.disp, "Sls", rf2.disp, self.grp.disp,
                    self.cod.disp, des.disp, qty.disp, sll.disp))
            else:
                self.t.textLine("%s %s %s %s %s %s %s %s %s %s %s" % \
                    (rf1.disp, "Sls", rf2.disp, self.grp.disp,
                    self.cod.disp, des.disp, qty.disp, sll.disp,
                    cst.disp, prf.disp, pcn.disp))
            self.pglin = self.pglin + 1
            self.dtot[0] = float(ASD(self.dtot[0]) + ASD(cst.work))
            self.dtot[1] = float(ASD(self.dtot[1]) + ASD(sll.work))
            self.gtot[0] = float(ASD(self.gtot[0]) + ASD(cst.work))
            self.gtot[1] = float(ASD(self.gtot[1]) + ASD(sll.work))
            old_chn = self.chn.work
            old_drs = self.drs.work
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.drsTotal()
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        if self.vals == "Y":
            spc = " " * 49
        else:
            spc = " " * 81
        self.t.textLine("%-30s %-10s %s %4s %5s" % \
            ("Sales By Customer Report as at", self.sysdtd, spc,
            "Page", self.pgnum))
        self.t.textLine()
        if not self.drsact:
            act = "All"
        else:
            act = self.drsact
        if not self.drstyp:
            typ = "All"
        else:
            typ = self.drstyp
        self.t.textLine("%24s%7s%3s%12s%7s%3s%10s%3s%3s%6s%3s%3s%13s%1s%1s" % (
            "(Options: Start Period: ", self.df.t_disp[0][0][0], "",
            "End Period: ", self.df.t_disp[0][0][1], "",
            "Activity: ", act, "",
            "Type: ", typ, "",
            "Values Only: ", self.vals, ")"))
        self.t.textLine()
        drn = self.sql.getRec(tables="drsmst", cols=["drm_name"],
            where=[("drm_cono", "=", self.conum), ("drm_chain", "=",
            self.chn.work), ("drm_acno", "=", self.drs.work)], limit=1)
        if drn:
            drsn = drn[0]
        else:
            drsn = ""
        self.t.textLine("%s %s    %s %s    %s %s" % \
            ("Chain:", self.chn.disp, "Acc-Num:", self.drs.disp, "Name:", drsn))
        self.t.textLine()
        if self.vals == "Y":
            self.t.textLine("%-9s %-3s %-9s %-3s %-20s %-30s%11s %11s" % \
                ("Reference", "Typ", "Ref-Num-2", "Grp", "Product-Code",
                "Description", "Quantity", "Sell-Value"))
        else:
            self.t.textLine("%-9s %-3s %-9s %-3s %-20s %-30s%11s %11s "\
                "%11s %11s %7s" % ("Reference", "Typ", "Ref-Num-2", "Grp",
                "Product-Code", "Description", "Quantity", "Sell-Value",
                "Cost-Value", "Profit", "Prf-%"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 10

    def drsTotal(self):
        cst = CCD(self.dtot[0], "SD", 12.2)
        sll = CCD(self.dtot[1], "SD", 12.2)
        prf = float(ASD(sll.work) - ASD(cst.work))
        prf = CCD(prf, "SD", 11.2)
        if sll.work == 0:
            pcn = 0
        else:
            pcn = round((prf.work * 100.0 / sll.work), 2)
        pcn = CCD(pcn, "SD", 7.2)
        self.t.textLine("%s" % (self.suc * len(self.head)))
        if self.vals == "Y":
            self.t.textLine("%-48s %-41s %s" % (" ",
                "Customer Totals", sll.disp))
        else:
            self.t.textLine("%-48s %-41s %s%s %s %s" % (" ",
                "Customer Totals", sll.disp, cst.disp, prf.disp, pcn.disp))
        self.t.textLine()
        self.dtot = [0] * 2

    def grandTotal(self):
        cst = CCD(self.gtot[0], "SD", 12.2)
        sll = CCD(self.gtot[1], "SD", 12.2)
        prf = float(ASD(sll.work) - ASD(cst.work))
        prf = CCD(prf, "SD", 11.2)
        if sll.work == 0:
            pcn = 0
        else:
            pcn = round((prf.work * 100.0 / sll.work), 2)
        pcn = CCD(pcn, "SD", 7.2)
        if self.vals == "Y":
            self.t.textLine("%-48s %-41s %s" % \
            (" ", "Grand Totals", sll.disp))
        else:
            self.t.textLine("%-48s %-41s %s%s %s %s" % \
            (" ", "Grand Totals", sll.disp, cst.disp, prf.disp, pcn.disp))

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

