#! /usr/bin/env python

"""
SYNOPSIS
    Sales By Salesman Report.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError

class si3060:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlrep", "strmf1", "strtrn"],
            prog=__name__)
        if self.sql.error:
            return self.sql.error
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Stock Movement Report (%s)" % __name__)
        rpm = {
            "stype": "R",
            "tables": ("ctlrep",),
            "cols": (
                ("rpm_rep", "", 0, "Rep"),
                ("rpm_name", "", 0, "Name", "Y")),
            "where": [("rpm_cono", "=", self.conum)]}
        fld = (
            (("T",0,0,0),"ID2",7,"Start Period","Start Period",
                (self.sysdtw/100),"Y",self.doSdate,None,None,("efld",)),
            (("T",0,1,0),"ID2",7,"End Period","End Period",
                (self.sysdtw/100),"Y",self.doEdate,None,None,("efld",)),
            (("T",0,2,0),"IUA",3,"Salesman","Salesman",
                "","Y",self.doRep,rpm,None,("efld",)))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doSdate(self, frt, pag, r, c, p, i, w):
        self.sdate = w

    def doEdate(self, frt, pag, r, c, p, i, w):
        self.edate = w

    def doRep(self, frt, pag, r, c, p, i, w):
        if w:
            acc = self.sql.getRec(tables="ctlrep", cols=["rpm_name"],
                where=[("rpm_cono", "=", self.conum), ("rpm_rep", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Salesman"
        self.rep = w

    def doEnd(self):
        self.df.closeProcess()
        if not self.rep:
            self.srep = ""
            self.erep = "zzz"
        else:
            self.srep = self.rep; self.erep = self.rep
        col = ["stt_rep", "stt_group", "stt_code", "stt_loc", "stt_type",
            "stt_ref1", "stt_ref2", "stt_trdt", "stt_desc", "stt_acno",
            "stt_qty", "stt_cost", "stt_sell", "st1_desc"]
        whr = [("stt_cono", "=", self.conum), ("stt_cono=st1_cono",),
            ("stt_group=st1_group",), ("stt_code=st1_code",), ("stt_curdt",
            ">=", self.sdate), ("stt_curdt", "<=", self.edate), ("stt_rep",
            ">=", self.srep), ("stt_rep", "<=", self.erep), ("stt_type",
            "=", 7)]
        ord = "stt_rep, stt_type, stt_batch, stt_ref1"
        all = self.sql.getRec(tables=["strmf1", "strtrn"], cols=col,
            where=whr, order=ord)
        if not all:
            showError(self.mf.body, "Processing Error",
            "No Records Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %32s %33s %33s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.stot = [0] * 3
        self.gtot = [0] * 3
        old_rep = ""
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            self.rep = CCD(dat[0], "Na", 3)
            grp = CCD(dat[1], "NA", 3)
            cod = CCD(dat[2], "NA", 20)
            typ = CCD(dat[4], "UI", 3)
            rf1 = CCD(dat[5], "Na", 9)
            rf2 = CCD(dat[6], "Na", 9)
            drs = CCD(dat[9], "NA", 7)
            qty = CCD(float(ASD(0) - ASD(dat[10])), "SD", 11.2)
            cst = CCD(float(ASD(0) - ASD(dat[11])), "SD", 11.2)
            sll = CCD(float(ASD(0) - ASD(dat[12])), "SD", 11.2)
            des = CCD(dat[13], "NA", 30)
            prf = float(ASD(sll.work) - ASD(cst.work))
            prf = CCD(prf, "SD", 11.2)
            if sll.work == 0:
                pcn = 0
            else:
                pcn = round((prf.work * 100.0 / sll.work), 2)
            pcn = CCD(pcn, "SD", 7.2)
            if old_rep and old_rep != self.rep.work:
                self.repTotal()
                self.pglin = 99
            if self.pglin > self.lpp:
                if self.pgnum != 0:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            self.t.textLine("%s %s %s %s %s %s %s %s %s %s %s %s" % \
                (rf1.disp, "Sls", drs.disp, rf2.disp, grp.disp, cod.disp,
                des.disp, qty.disp, sll.disp, cst.disp, prf.disp, pcn.disp))
            self.pglin = self.pglin + 1
            self.stot[0] = float(ASD(self.stot[0]) + ASD(sll.work))
            self.stot[1] = float(ASD(self.stot[1]) + ASD(cst.work))
            self.gtot[0] = float(ASD(self.gtot[0]) + ASD(sll.work))
            self.gtot[1] = float(ASD(self.gtot[1]) + ASD(cst.work))
            old_rep = self.rep.work
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.repTotal()
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-30s %-10s %94s %5s" % \
            ("Sales By Salesman Report as at",
            self.sysdtd, "Page", self.pgnum))
        self.t.textLine()
        self.t.textLine("%-24s%-7s%-3s%-11s%-7s%-1s" % \
            ("(Options: Start Period-", self.df.t_disp[0][0][0], "",
                "End Period-", self.df.t_disp[0][0][1], ")"))
        self.t.textLine()
        rep = self.sql.getRec(tables="ctlrep", cols=["rpm_name"],
            where=[("rpm_cono", "=", self.conum), ("rpm_rep", "=",
            self.rep.work)], limit=1)
        if not rep:
            repn = rep[0]
        else:
            repn = ""
        self.t.textLine("%-5s %3s %s" % ("Rep", self.rep.disp, repn))
        self.t.textLine()
        self.t.textLine("%-9s %-3s %-7s %-9s %-3s %-20s %-30s%11s %11s " \
            "%11s %11s %7s" % ("Reference", "Typ", "Drs-Num", "Ref-Num-2",
            "Grp", "Product-Code", "Description", "Quantity",
            "Sell-Value", "Cost-Value", "Profit", "Prf-%"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 10

    def repTotal(self):
        sll = CCD(self.stot[0], "SD", 12.2)
        cst = CCD(self.stot[1], "SD", 12.2)
        prf = float(ASD(sll.work) - ASD(cst.work))
        prf = CCD(prf, "SD", 11.2)
        if sll.work == 0:
            pcn = 0
        else:
            pcn = round((prf.work * 100.0 / sll.work), 2)
        pcn = CCD(pcn, "SD", 7.2)
        self.t.textLine()
        self.t.textLine("%-56s %-41s %s%s %s %s" % \
            (" ", "Rep Totals", sll.disp, cst.disp, prf.disp, pcn.disp))
        self.t.textLine()
        self.stot = [0] * 2

    def grandTotal(self):
        sll = CCD(self.gtot[0], "SD", 12.2)
        cst = CCD(self.gtot[1], "SD", 12.2)
        prf = float(ASD(sll.work) - ASD(cst.work))
        prf = CCD(prf, "SD", 11.2)
        if sll.work == 0:
            pcn = 0
        else:
            pcn = round((prf.work * 100.0 / sll.work), 2)
        pcn = CCD(pcn, "SD", 7.2)
        self.t.textLine("%-56s %-41s %s%s %s %s" % \
            (" ", "Grand Totals", sll.disp, cst.disp, prf.disp, pcn.disp))

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

