#! /usr/bin/python2

"""
SYNOPSIS
    Reprinting of Sales Documents.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import PrintInvoice, Sql, TartanDialog
from tartanFunctions import getSingleRecords

class si3080:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "slsiv1", prog=__name__)

    def mainProcess(self):
        tit = ("Reprint Sales Documents",)
        iv1 = {
            "stype": "R",
            "tables": ("slsiv1","drsmst"),
            "cols": [
                ("si1_docno", "",  0, "Doc-Num"),
                ("si1_date", "",  0, "Date"),
                ("si1_chain", "",  0, "Chn"),
                ("si1_acno",  "", 0, "Acc-Num"),
                ("drm_name",  "", 0, "Name", "Y")],
            "where": [("si1_cono", "=", self.conum), ("si1_cono=drm_cono",),
                ("si1_chain=drm_chain",), ("si1_acno=drm_acno",)],
            "whera": [["T", "si1_rtn", 1, 0]],
            "screen": self.mf.body}
        r1s = (("Copies", "C"), ("Originals", "O"))
        r2s = (
            ("Inv","I"),
            ("C-N","C"),
            ("S-O","O"),
            ("Qte","Q"))
        r3s = (("Range","R"),("Singles", "S"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Document Mode","Document Mode",
                "C","N",self.doMode,None,None,None),
            (("T",0,1,0),("IRB",r2s),0,"Document Type","Document Type",
                "I","N",self.doType,None,None,None),
            (("T",0,2,0),("IRB",r3s),0,"Documents","Documents",
                "S","Y",self.doSelect,None,None,None),
            (("T",0,3,0),"IUI",7,"From Number","From Document Number",
                "","N",self.doDocno,iv1,None,("notzero",)),
            [("T",0,4,0),"IUI",7,"To   Number","To Document Number",
                "","N",self.doDocno,iv1,None,("notzero",)])
        self.df = TartanDialog(self.mf, screen=self.mf.body, title=tit,
            eflds=fld, tend=((self.doEnd, "y"),), txit=(self.doExit,),
            view=("N","V"), mail=("B","N"))

    def doMode(self, frt, pag, r, c, p, i, w):
        if w == "C":
            self.copy = "y"
        else:
            self.copy = "n"
            self.df.topf[pag][5][5] = "V"
            if len(self.df.topf[pag]) == 8:
                self.df.topf[pag][7][5] = "N"

    def doType(self, frt, pag, r, c, p, i, w):
        self.typ = w

    def doSelect(self, frt, pag, r, c, p, i, w):
        self.select = w
        if self.select == "S":
            self.frm = 0
            self.to = 0
            return "sk2"

    def doDocno(self, frt, pag, r, c, p, i, w):
        if w:
            doc = self.sql.getRec(tables="slsiv1", where=[("si1_cono", "=",
                self.conum), ("si1_rtn", "=", self.typ), ("si1_docno", "=", w)])
            if not doc:
                return "Document Number Does Not Exist"
        if p == 3:
            self.frm = w
            self.df.topf[pag][4][5] = self.frm
        elif w < self.frm:
            return "Document Number Before From Number"
        else:
            self.to = w

    def doEnd(self):
        self.df.closeProcess()
        prt = self.df.repprt[0][2]
        eml = (self.df.repprt[1][1], self.df.repprt[1][2])
        if self.select == "S":
            iv1 = []
            all = getSingleRecords(self.mf, "slsiv1", ("si1_docno",
                "si1_date", "si1_chain", "si1_acno"), where=[("si1_cono",
                "=", self.conum), ("si1_rtn", "=", self.typ)])
            if all:
                for rec in all:
                    iv1.append([rec[2]])
        else:
            iv1 = self.sql.getRec(tables="slsiv1", cols=["si1_docno"],
            where=[("si1_cono", "=", self.conum), ("si1_rtn", "=", self.typ),
            ("si1_docno", ">=", self.frm), ("si1_docno", "<=", self.to)])
        if iv1:
            for doc in iv1:
                PrintInvoice(self.mf, self.conum, self.typ, doc[0], prtnam=prt,
                    email=eml, copy=self.copy)
        self.mf.closeLoop()

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

