#! /usr/bin/python

"""
SYNOPSIS
    File Maintenance for Staff Loans Record.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import Sql, TartanDialog

class sl1010:
    def __init__(self, mf, conum, conam, user):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        self.user = user
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["chglog", "wagedc", "wagmst", "waglmf"],
            prog=__name__)

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Salaries/Wages Loans Masterfile Maintenance (%s)" % __name__)
        wgm = {
            "stype": "R",
            "tables": ("wagmst",),
            "cols": (
                ("wgm_empno", "", 0, "EmpNo"),
                ("wgm_sname", "", 0, "Surname", "Y")),
            "where": [("wgm_cono", "=", self.conum)]}
        lnm = {
            "stype": "R",
            "tables": ("waglmf",),
            "cols": (
                ("wlm_empno", "", 0, "EmpNo"),
                ("wlm_loan", "", 0, "Ln"),
                ("wlm_desc", "", 0, "Description", "Y")),
            "where": [("wlm_cono", "=", self.conum)],
            "whera": [["T", "wlm_empno", 0]],
            "index": 1}
        ced = {
            "stype": "R",
            "tables": ("wagedc",),
            "cols": (("ced_type", "", 0, "T"),
                ("ced_code", "", 0, "Cde"),
                ("ced_desc", "", 0, "Description", "Y")),
            "where": [("ced_cono", "=", self.conum), ("ced_type", "=", "D")],
            "index": 1}
        fld = (
            (("T",0,0,0),"IUI",5,"Emp-Num","Employee Number",
                "","Y",self.doEmp,wgm,None,("notzero",)),
            (("T",0,0,0),"ONA",30,""),
            (("T",0,1,0),"IUI",3,"Loan-Num","Loan Number",
                "","N",self.doLoan,lnm,None,("notzero",)),
            (("T",0,2,0),"INA",30,"Description","Description",
                "","N",None,None,None,("notblank",)),
            (("T",0,3,0),"IUI",3,"Deduction Code","Deduction Code",
                "","N",self.doDed,ced,None,("notzero",)),
            (("T",0,4,0),"IUD",6.2,"Interest Percentage","Interest Percentage",
                "","N",None,None,None,("efld",)),
            (("T",0,5,0),"ID1",10,"Start Date","Start Date",
                "","N",None,None,None,("efld",)),
            (("T",0,6,0),"IUD",10.2,"Deduction Amount","Deduction Amount",
                "","N",None,None,None,("efld",)))
        tnd = ((self.doEnd,"n"),(self.doEnd,"y"))
        txt = (self.doExit,self.doExit)
        but = (
            ("Accept",None,self.doAccept,0,("T",0,4),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,4),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, butt=but)

    def doEmp(self, frt, pag, r, c, p, i, w):
        self.empno = w
        acc = self.sql.getRec(tables="wagmst", cols=["wgm_sname"],
            where=[("wgm_cono", "=", self.conum), ("wgm_empno", "=",
            self.empno)], limit=1)
        if not acc:
            return "Invalid Employee Number"
        self.df.loadEntry("T", pag, p+1, data=acc[0])

    def doLoan(self, frt, pag, r, c, p, i, w):
        self.loan = w
        acc = self.sql.getRec(tables=["wagmst", "waglmf"], cols=["wlm_empno",
            "wgm_sname", "wlm_loan", "wlm_desc", "wlm_ded_code", "wlm_int_per",
            "wlm_start_date", "wlm_ded_amt"], where=[("wlm_cono", "=",
            self.conum), ("wlm_empno", "=", self.empno), ("wlm_loan", "=",
            self.loan), ("wgm_cono=wlm_cono",), ("wgm_empno=wlm_empno",)],
            limit=1)
        if not acc:
            return "Invalid Loan Number"
        for num, dat in enumerate(acc):
            self.df.loadEntry("T", 0, num, data=dat)
        self.old = [self.conum, acc[0]]
        self.old.extend(acc[2:])

    def doDed(self, frt, pag, r, c, p, i, w):
        self.code = w
        desc = self.sql.getRec(tables="wagedc", cols=["ced_desc"],
            where=[("ced_cono", "=", self.conum), ("ced_type", "=", "D"),
            ("ced_code", "=", w)], limit=1)
        if not desc:
            return "Invalid Code"

    def doEnd(self):
        data = [self.conum]
        for p in range(0, len(self.df.t_work[0][0])):
            if p == 1:
                continue
            data.append(self.df.t_work[0][0][p])
        if data != self.old:
            self.sql.updRec("waglmf", data=data, where=[("wlm_cono", "=",
                self.conum), ("wlm_empno", "=", self.empno), ("wlm_loan", "=",
                self.loan)])
            dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
            for num, dat in enumerate(self.old):
                if dat != data[num]:
                    self.sql.insRec("chglog", data=["waglmf",
                    "U", "%03i%05i%02i" % (self.conum, self.empno, self.loan),
                    self.sql.waglmf_col[num], dte, self.user, str(dat),
                    str(data[num]), 0])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.last[0] = [0, 0]
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        if not self.df.pag:
            self.doCloseProcess()
        elif self.df.pag == 1:
            self.df.focusField("T", 0, 2)

    def doCloseProcess(self):
        self.df.closeProcess()
        self.mf.closeLoop()

