#! /usr/bin/env python

"""
SYNOPSIS
    Stores Selling Price Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import GetCtl, Sql, TartanDialog

class st1020:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["strgrp", "strmf1", "strmf2",
            "strloc", "strprc"], prog=__name__)
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "no"
        self.locs = strctl["cts_locs"]
        self.prices = strctl["cts_prices"]

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Stores Selling Price Maintenance (%s)" % __name__)
        gpm = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        stm = {
            "stype": "R",
            "tables": ("strmf1",),
            "cols": (
                ("st1_group", "", 0, "Grp"),
                ("st1_code", "", 0, "Product Code"),
                ("st1_desc", "", 0, "Description", "Y")),
            "where": [("st1_cono", "=", self.conum)],
            "whera": [["T", "st1_group", 0]],
            "index": 1}
        loc = {
            "stype": "R",
            "tables": ("strloc",),
            "cols": (("srl_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Description", "Y")),
            "where": [("srl_cono", "=", self.conum)]}
        stl = {
            "stype": "R",
            "tables": ("strmf2","strmf1","strloc"),
            "cols": (("st2_group", "", 0, "Grp"),
                ("st2_code", "", 0, "Product Code"),
                ("st1_desc", "", 0, "Description"),
                ("st2_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Description", "Y")),
            "where": [("st1_cono=st2_cono",), ("st1_group=st2_group",),
                    ("st1_code=st2_code",), ("st2_cono=srl_cono",),
                    ("st2_loc=srl_loc",), ("st1_cono", "=", self.conum)],
            "whera": [["T", "st1_group", 0], ["T", "st1_code", 1]],
            "index": 3}
        fld = [
            (("T",0,0,0),"IUA",3,"Product Group","Product Group",
                "","Y",self.doGroup,gpm,None,("notblank",)),
            (("T",0,1,0),"INA",20,"Product Code","Product Code",
                "","N",self.doCode,stm,None,("notblank",)),
            (("T",0,1,0),"ONA",30,""),
            (("T",0,2,0),"IUA",1,"Location","Location",
                "1","N",self.doLoc,stl,None,("notblank",)),
            (("T",0,2,0),"ONA",30,"")]
        for ind in range(self.prices):
            row = ind + 3
            lvl = ind + 1
            fld.append([["T",0,row,0],"IUD",8.2,"Price Level %s" % lvl,"",
                "","N",None,None,None,("efld",)])
        tnd = ((self.doEnd,"Y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt)

    def doGroup(self, frt, pag, r, c, p, i, w):
        self.group = w
        acc = self.sql.getRec(tables="strgrp", cols=["gpm_group"],
            where=[("gpm_cono", "=", self.conum), ("gpm_group", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Group"

    def doCode(self, frt, pag, r, c, p, i, w):
        self.code = w
        acc = self.sql.getRec(tables="strmf1", cols=["st1_desc"],
            where=[("st1_cono", "=", self.conum), ("st1_group", "=",
            self.group), ("st1_code", "=", w)], limit=1)
        if not acc:
            return "Invalid Code"
        self.df.loadEntry("T", pag, p+1, data=acc[0])
        if self.locs == "N":
            self.loc = "1"
            err = self.checkLoc()
            if err:
                return err
            self.doLoadPrices()
            return "sk2"

    def doLoc(self, frt, pag, r, c, p, i, w):
        self.loc = w
        err = self.checkLoc()
        if err:
            return err
        self.doLoadPrices()

    def checkLoc(self):
        acc = self.sql.getRec(tables="strloc", cols=["srl_desc"],
            where=[("srl_cono", "=", self.conum), ("srl_loc", "=", self.loc)],
            limit=1)
        if not acc:
            return "Invalid Location Code"
        self.df.loadEntry("T", 0, 3, data=self.loc)
        self.df.loadEntry("T", 0, 4, data=acc[0])
        acc = self.sql.getRec(tables="strmf2", cols=["st2_bin"],
            where=[("st2_cono", "=", self.conum), ("st2_group", "=",
            self.group), ("st2_code", "=", self.code), ("st2_loc", "=",
            self.loc)], limit=1)
        if not acc:
            return "Invalid Location For This Product Code"

    def doLoadPrices(self):
        acc = self.sql.getRec(tables="strprc", cols=["stp_level", "stp_price"],
            where=[("stp_cono", "=", self.conum), ("stp_group", "=",
            self.group), ("stp_code", "=", self.code), ("stp_loc", "=",
            self.loc)], order="stp_level")
        lvl = 1
        new = []
        stop = False
        for num in range(0, self.prices):
            if not acc or lvl > len(acc):
                new.append(0)
                lvl += 1
                continue
            while acc[num][0] != lvl:
                new.append(0)
                lvl += 1
                if lvl > self.prices:
                    stop = True
            if not stop:
                new.append(acc[num][1])
                lvl += 1
        for num, prc in enumerate(new):
            self.df.loadEntry("T", 0, num+5, data=prc)

    def doEnd(self):
        data = [self.conum, self.group, self.code, self.loc]
        for num, price in enumerate(self.df.t_work[0][0][5:]):
            dat = data[:]
            lvl = num + 1
            dat.extend([lvl, price])
            acc = self.sql.getRec(tables="strprc", where=[("stp_cono", "=",
                self.conum), ("stp_group", "=", self.group), ("stp_code", "=",
                self.code), ("stp_loc", "=", self.loc), ("stp_level", "=",
                lvl)], limit=1)
            if acc:
                self.sql.updRec("strprc", data=dat, where=[("stp_cono", "=",
                    self.conum), ("stp_group", "=", self.group), ("stp_code",
                    "=", self.code), ("stp_loc", "=", self.loc), ("stp_level",
                    "=", lvl)])
            elif price:
                self.sql.insRec("strprc", data=dat)
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()
