#! /usr/bin/env python

"""
SYNOPSIS
    Stores Recipe Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import CCD, GetCtl, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, getSingleRecords
from tartanFunctions import doPrinter, showError

class st3050:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["strgrp", "strmf1", "strloc", "strrcp"],
            prog=__name__)
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "strctl"
        self.locs = strctl["cts_locs"]
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Stores Recipe Listing (%s)" % __name__)
        loc = {
            "stype": "R",
            "tables": ("strloc",),
            "cols": (
                ("srl_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Description", "Y")),
            "where": [("srl_cono", "=", self.conum)]}
        grp = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (
                ("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        r1s = (("Yes","Y"), ("Singles", "S"))
        r2s = (("Yes","Y"), ("No","N"))
        if self.locs == "N":
            self.loc = "1"
            self.locd = "Location One"
            fld = []
        else:
            fld = [[("T",0,0,0),"IUA",1,"Location","Location",
                "1","Y",self.doLoc,loc,None,("notblank",)]]
        fld.extend([
            (("T",0,1,0),"IUA",3,"Product Group","Product Group",
                "","Y",self.doGroup,grp,None,None),
            (("T",0,2,0),("IRB",r1s),0,"Whole File","Whole File",
                "S","Y",self.doWhole,None,None,None),
            (("T",0,3,0),("IRB",r2s),0,"Recipe per Page","Recipe per Page",
                "N","Y",self.doPage,None,None,None)])
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doLoc(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="strloc", cols=["srl_desc"],
            where=[("srl_cono", "=", self.conum), ("srl_loc", "=", w)], limit=1)
        if not acc:
            return "Invalid Location"
        self.loc = w
        self.locd = acc[0]

    def doGroup(self, frt, pag, r, c, p, i, w):
        if not w:
            self.group = ""
        else:
            acc = self.sql.getRec(tables="strgrp", cols=["gpm_desc"],
                where=[("gpm_cono", "=", self.conum), ("gpm_group", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Group"
            self.group = w

    def doWhole(self, frt, pag, r, c, p, i, w):
        self.whole = w

    def doPage(self, frt, pag, r, c, p, i, w):
        self.page = w

    def doEnd(self):
        self.df.closeProcess()
        if not self.group:
            self.sgrp = ""
            self.egrp = "zzz"
        else:
            self.sgrp = self.group
            self.egrp = self.group
        whr = [("st1_cono", "=", self.conum), ("st1_group", ">=", self.sgrp),
            ("st1_group", "<=", self.egrp), ("st1_type", "=", "R")]
        ord = "st1_group, st1_code"
        if self.whole == "S":
            all = getSingleRecords(self.mf, "strmf1",
                ("st1_group", "st1_code", "st1_desc"), where=whr, order=ord)
        else:
            all = self.sql.getRec(tables="strmf1", where=whr, order=ord)
        if not all:
            showError(self.mf.body, "Processing Error", "No Records Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %33s %6s" % (self.conum, self.conam,
            self.sysdttm, __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        oldrec = ""
        self.pgnum = 0
        self.pglin = 99
        st1 = self.sql.strmf1_col
        srr = self.sql.strrcp_col
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            grp = CCD(dat[st1.index("st1_group")], "UA", 3)
            cod = CCD(dat[st1.index("st1_code")], "NA", 20)
            desc = CCD(dat[st1.index("st1_desc")], "UA", 30)
            whr = [("srr_cono", "=", self.conum), ("srr_group", "=", grp.work),
                ("srr_code", "=", cod.work), ("srr_loc", "=", self.loc)]
            ord = "srr_rgroup, srr_rcode"
            rec = self.sql.getRec(tables="strrcp", where=whr, order=ord)
            if not rec:
                continue
            newrec = "%s%s" % (grp.work, cod.work)
            if oldrec and newrec != oldrec:
                self.pageHeading(grp.disp, cod.disp, desc.disp, chg=True)
            for z in rec:
                if self.pglin > self.lpp:
                    if self.pgnum:
                        self.c.drawText(self.t)
                        self.c.showPage()
                    self.pageHeading(grp.disp, cod.disp, desc.disp)
                gp  = CCD(z[srr.index("srr_rgroup")], "UA", 3)
                cd  = CCD(z[srr.index("srr_rcode")],  "NA", 20)
                qt  = CCD(z[srr.index("srr_rqty")], "UD", 11.2)
                dd = self.sql.getRec(tables="strmf1", cols=["st1_desc"],
                    where=[("st1_cono", "=", self.conum), ("st1_group", "=",
                    gp.work), ("st1_code", "=", cd.work)], limit=1)
                if dd:
                    dc = dd[0]
                else:
                    dc = ""
                self.t.textLine("%s %s %-38s %s" % (gp.disp, cd.disp, dc,
                    qt.disp))
                self.pglin = self.pglin + 1
                oldrec = newrec
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
            
    def pageHeading(self, grp, cod, desc, chg=False):
        if self.pgnum and self.page == "N" and chg and \
                            self.pglin < (self.lpp - 10):
            self.t.textLine()
            self.t.textLine()
            self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
            self.pglin += 2
        else:
            if self.pgnum and chg:
                self.c.drawText(self.t)
                self.c.showPage()
            self.t = self.c.beginText(self.ogn[0], self.ogn[1])
            self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
            self.pgnum = self.pgnum + 1
            self.t.textLine(self.head)
            self.t.textLine()
            self.t.textLine("%-19s %-10s %38s %5s" % ("Stock Recipes as at",
                self.sysdtd, "Page", self.pgnum))
            self.t.textLine()
            self.pglin = 4
        self.t.textLine("%-5s %3s  %-4s %s  %-8s %s %s" % ("Group", grp,
            "Code", cod, "Location", self.loc, self.locd))
        self.t.textLine()
        self.t.textLine("%-3s %-20s %-38s %11s" % ("Grp", "Product-Code",
            "Description", "Quantity"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.pglin += 4
        self.t.setFont("Tartan", self.cpi, self.ldg)

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

