#! /usr/bin/env python

"""
SYNOPSIS
    Stores Ledger Print Labels. L7159

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, CCD, GetCtl, ProgressBar, Sql
from TartanClasses import TartanDialog
from tartanFunctions import doReportlab, makeArray, getModName, doPrinter
from tartanFunctions import showError

class st3110:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlvmf", "strgrp", "strloc", "strprc",
            "strmf1", "strmf2"], prog=__name__)
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "no"
        self.locs = strctl["cts_locs"]
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None
        self.labels = {"L7159": (3, 8, 32)}

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Print Stores Labels (%s)" % __name__)
        lab = []
        for l in self.labels:
            lab.append(l)
        loc = {
            "stype": "R",
            "tables": ("strloc",),
            "cols": (
                ("srl_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Description", "Y")),
            "where": [("srl_cono", "=", self.conum)]}
        gpm = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (
                ("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        stm = {
            "stype": "R",
            "tables": ("strmf1",),
            "cols": (
                ("st1_group", "", 0, "Grp"),
                ("st1_code", "", 0, "Product Code"),
                ("st1_desc", "", 0, "Description", "Y")),
            "where": [("st1_cono", "=", self.conum)],
            "whera": [["T", "st1_group", 2, 0]],
            "index": 1}
        r1s = (("Normal", "N"), ("Recipes", "R"), ("All", "A"))
        r2s = (("Yes", "Y"), ("No", "N"), ("All", "A"))
        r3s = (("No", "N"), ("Yes", "Y"))
        fld = [
            (("T",0,0,0),"ID1",10,"Reporting Date","",
                self.sysdtw,"Y",self.doDate,None,None,("efld",)),
            [("T",0,1,0),"IUA",1,"Location","",
                0,"Y",self.doLoc,loc,None,None],
            (("T",0,2,0),"IUA",3,"Product Group","",
                "","Y",self.doGrp,gpm,None,None),
            (("T",0,3,0),"INA",20,"Product Code","",
                "","Y",self.doCode,stm,None,None),
            (("T",0,4,0),("IRB",r1s),0,"Item Types","",
                "N","Y",self.doTypes,None,None,None),
            (("T",0,5,0),("IRB",r2s),0,"Value Indicator","",
                "Y","Y",self.doValInd,None,None,None),
            (("T",0,6,0),("IRB",r3s),0,"Include Out of Stock ","",
                "N","Y",self.doOuts,None,None,None),
            (("T",0,7,0),"IUA",10,"Cost Price Code","",
                "","Y",self.doCCode,None,None,("notblank",)),
            (("T",0,8,0),"IUA",5,"Avery A4 Code","",
                "L7159","Y",self.doAvery,None,None,("in",lab)),
            (("T",0,9,0),"IUI",2,"First Label Position","",
                1,"Y",self.doStart,None,None,("notzero",))]
        if self.locs == "N":
            fld[1][1] = "OUA"
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"))
    
    def doDate(self, frt, pag, r, c, p, i, w):
        self.date = w
        if self.locs == "N":
            self.loc = "1"
            self.df.loadEntry(frt, pag, p+1, data=self.loc)
            return "sk1"

    def doLoc(self, frt, pag, r, c, p, i, w):
        self.loc = w
        if self.loc:
            acc = self.sql.getRec(tables="strloc", cols=["srl_desc"],
                where=[("srl_cono", "=", self.conum), ("srl_loc", "=",
                self.loc)], limit=1)
            if not acc:
                return "Invalid Location Code"

    def doGrp(self, frt, pag, r, c, p, i, w):
        self.grp = w
        if self.grp:
            acc = self.sql.getRec(tables="strgrp", cols=["gpm_group"],
                where=[("gpm_cono", "=", self.conum), ("gpm_group", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Group"

    def doCode(self, frt, pag, r, c, p, i, w):
        self.code = w
        if self.code:
            acc = self.sql.getRec(tables="strmf1", cols=["st1_desc"],
                where=[("st1_cono", "=", self.conum), ("st1_group", "=",
                self.grp), ("st1_code", "=", w)], limit=1)
            if not acc:
                return "Invalid Code"

    def doTypes(self, frt, pag, r, c, p, i, w):
        self.types = w

    def doValInd(self, frt, pag, r, c, p, i, w):
        self.valind = w

    def doOuts(self, frt, pag, r, c, p, i, w):
        self.outs = w

    def doCCode(self, frt, pag, r, c, p, i, w):
        self.ccode = w
        spl = w.split()
        if len(spl) != 1 or len(spl[0]) != 10:
            return "Must Have 10 Valid Characters"

    def doAvery(self, frt, pag, r, c, p, i, w):
        self.across, self.down, self.width = self.labels[w]

    def doStart(self, frt, pag, r, c, p, i, w):
        maxl = (self.across * self.down) + 1
        if not w in range (1, maxl):
            return "Out of range"
        self.start = w

    def doEnd(self):
        self.df.closeProcess()
        self.view = self.df.repprt[0][1]
        self.prtnam = self.df.repprt[0][2]
        self.emadd = self.df.repprt[1][2]
        self.prnt = False
        whr = [("st1_cono", "=", self.conum)]
        if self.grp:
            whr.append(("st1_group", "=", self.grp))
        if self.code:
            whr.append(("st1_code", "=", self.code))
        if self.types in ("N", "R"):
            whr.append(("st1_type", "=", self.types))
        if self.valind in ("Y", "N"):
            whr.append(("st1_value_ind", "=", self.valind))
        if self.loc:
            whr.append(("st2_loc", "=", self.loc))
        whr.extend([
            ("st2_cono=st1_cono",),
            ("st2_group=st1_group",),
            ("st2_code=st1_code",)])
        all = self.sql.getRec(tables=["strmf1", "strmf2"], cols=["st2_loc",
            "st2_group", "st2_code", "st1_desc", "st1_value_ind",
            "st1_vatcode"], where=whr, order="st2_group, st2_code, st2_loc")
        if not all:
            showError(self.mf.body, "Error", "No Records Selected")
        else:
            p = ProgressBar(self.mf.body, max=len(all), esc=True)
            for num, dat in enumerate(all):
                can = p.displayProgress(num)
                if can == "yes":
                    if self.t:
                        self.t.textLine()
                        self.t.textLine("Print Job ABORTED")
                    break
                self.doProcess(dat)
            p.closeProgress()
        if self.prnt:
            if self.vcount == 0 and self.hcount == 0:
                pass
            else:
                self.pageHeading()
                self.printPage()
                self.c.drawText(self.t)
                self.c.showPage()
            self.c.save()
            doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                head=self.tit, view=self.view, prtnam=self.prtnam,
                mail=self.emadd)
        self.closeProcess()

    def doProcess(self, dat):
        if not self.prnt:
            self.printSetup()
            self.prnt = True
        self.loc = CCD(dat[0], "UA", 1)
        self.group = CCD(dat[1], "UA", 3)
        self.code = CCD(dat[2], "TX", 20)
        self.desc = CCD(dat[3], "TX", 30)
        valind = CCD(dat[4], "UA", 1)
        self.vat = CCD(dat[5], "UA", 1)
        # Calculate cost price code ########################################
        bals = Balances(self.mf, "STR", self.conum, self.date / 100,
            (self.group.work, self.code.work, self.loc.work)).doStrBals()
        if self.outs == "N" and not bals[5][0]:
            return
        if valind == "N":
            icost = 0
        else:
            if bals[5][0] == 1:
                icost = bals[5][1]
            else:
                icost = bals[6]
            if icost <= 0:
                icost = bals[7]
        cost = CCD(int(icost * 100), "UI", 9)
        self.cost = " " * (9 - len(str(cost.work)))
        for x in str(cost.work):
            self.cost = self.cost + self.ccode[int(x)]
        ####################################################################
        prc = self.sql.getRec(tables="strprc", cols=["stp_price"],
            where=[("stp_cono", "=", self.conum), ("stp_group", "=",
            self.group.work), ("stp_code", "=", self.code.work),
            ("stp_loc", "=", self.loc.work), ("stp_level", "=", 1)],
            limit=1)
        if not prc or not prc[0]:
            self.price1 = CCD(0, "UD", 9.2)
            self.price2 = CCD(0, "UD", 9.2)
        else:
            self.price1 = CCD(prc[0], "UD", 9.2)
            vat = self.sql.getRec(tables="ctlvmf", cols=["vtm_new"],
                where=[("vtm_cono", "=", self.conum), ("vtm_code", "=",
                self.vat.work)], limit=1)
            if not vat:
                self.vatrate = CCD(0, "UD", 9.2)
            else:
                self.vatrate = CCD(vat[0], "UD", 9.2)
            self.price2 = CCD(round((self.price1.work * float(ASD(100) + \
                ASD(self.vatrate.work)) / 100.0), 2), "OUD", 9.2)
        self.loadStorage()
        if self.storagefull == "Y":
            self.pageHeading()
            self.printPage()
            self.c.drawText(self.t)
            self.c.showPage()
            self.clearStorage()

    def printSetup(self):
        self.c, cpi, inch, suc, duc = doReportlab(self.mf.prgdir, self.pdfnam)
        self.makeStorage()
        self.viewcnt = 0
        self.linecnt = 0
        if self.start == 1:
            self.hcount = 0
            self.vcount = 0
        else:
            self.hcount = ((self.start % self.across) + \
                (self.across - 1)) % self.across
            self.vcount = ((self.start + (self.across - 1)) / self.across) - 1
         
    def pageHeading(self):
        self.t = self.c.beginText(15, 20)
        self.t.setFont("Tartan", 10, 12.7)

    def printPage(self):
        self.t.textLine()
        spc = " " * self.width
        for x in range(0, self.down):
            lin1 = ""
            lin2 = ""
            lin3 = ""
            lin4 = ""
            lin5 = ""
            for y in range(0, self.across):
                lin = self.xdate[x][y] + "  " + self.xcost[x][y]
                lin1 = lin1 + lin + spc[0:self.width - len(lin)]
                lin = self.xloc[x][y] + "  " + self.xgroup[x][y] + \
                    "  " +  self.xcode[x][y]
                lin2 = lin2 + lin + spc[0:self.width - len(lin)]
                lin3 = lin3 + self.xdesc[x][y] + spc[0:self.width - \
                    len(self.xdesc[x][y])]
                lin4 = " "
                lin = self.xvat[x][y] + " " + self.xprice1[x][y] + " " + \
                    self.xprice2[x][y]
                lin5 = lin5 + lin + spc[0:self.width - len(lin)]
            self.t.textLine(" %s" % lin1)
            self.t.textLine(" %s" % lin2)
            self.t.textLine(" %s" % lin3)
            self.t.textLine(" %s" % lin4)
            self.t.textLine(" %s" % lin5)
            self.t.textLine(" ")
            self.t.textLine(" ")
            self.t.textLine(" ")
        self.clearStorage()
            
    def makeStorage(self):
        self.xdate = makeArray(self.across, self.down, 1, "S")
        self.xcost = makeArray(self.across, self.down, 1, "S")
        self.xloc = makeArray(self.across, self.down, 1, "S")
        self.xgroup = makeArray(self.across, self.down, 1, "S")
        self.xcode = makeArray(self.across, self.down, 1, "S")
        self.xdesc = makeArray(self.across, self.down, 1, "S")
        self.xfill = makeArray(self.across, self.down, 1, "S")
        self.xvat = makeArray(self.across, self.down, 1, "S")
        self.xprice1 = makeArray(self.across, self.down, 1, "S")
        self.xprice2 = makeArray(self.across, self.down, 1, "S")
        self.clearStorage()
            
    def clearStorage(self):
        for x in range(self.down):
            for y in range(self.across):
                self.xdate[x][y] = ""
                self.xcost[x][y] = ""
                self.xloc[x][y] = ""
                self.xgroup[x][y] = ""
                self.xcode[x][y] = ""
                self.xdesc[x][y] = ""
                self.xfill[x][y] = ""
                self.xvat[x][y] = ""
                self.xprice1[x][y] = ""
                self.xprice2[x][y] = ""
        self.hcount = 0
        self.vcount = 0
        self.storagefull = "N"

    def loadStorage(self):
        self.fill = "                              "
        self.xdate[self.vcount][self.hcount]=self.df.t_disp[0][0][0]
        self.xcost[self.vcount][self.hcount]=self.cost
        self.xloc[self.vcount][self.hcount]=self.loc.disp
        self.xgroup[self.vcount][self.hcount]=self.group.disp
        self.xcode[self.vcount][self.hcount]=self.code.disp
        self.xdesc[self.vcount][self.hcount]=self.desc.disp
        self.xfill[self.vcount][self.hcount]=self.fill
        self.xvat[self.vcount][self.hcount]=self.vat.disp
        self.xprice1[self.vcount][self.hcount]=self.price1.disp
        self.xprice2[self.vcount][self.hcount]=self.price2.disp
        if self.hcount == (self.across - 1) and \
            self.vcount == (self.down - 1):
            self.storagefull = "Y"
        elif self.hcount != (self.across - 1):
            self.hcount = self.hcount + 1
        elif self.vcount != (self.down - 1):
            self.vcount = self.vcount + 1
            self.hcount = 0

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

