#! /usr/bin/env python

"""
SYNOPSIS
    Stores Stock to Order Report.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, CCD, GetCtl, ProgressBar, Sql
from TartanClasses import TartanDialog
from tartanFunctions import doPrinter, doReportlab, doWriteExport, getModName
from tartanFunctions import showError

class st3120:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["strloc", "strmf1", "strmf2"],
            prog=__name__)
        if self.sql.error:
            return self.sql.error
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "strctl"
        self.locs = strctl["cts_locs"]
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % (t[0],
            t[1], t[2], t[3], t[4])
        self.head = ("%03u %-30s %s" % (self.conum, self.conam, "%s"))
        self.colsh = ["Grp", "Product-Code", "Description", "U.O.I",
            "Qty-Balance", "Qty-OnOrder", "Qty-BkOrder", "Qty-ToOrder"]
        self.forms = [("UA", 3), ("NA", 20), ("NA", 30), ("NA", 10),
            ("SD", 12,2), ("SD", 12.2), ("SD", 12.2), ("SD", 12.2)]
        self.stot = 0
        self.gtot = 0
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Stock On Hand Report (%s)" % __name__)
        loc = {
            "stype": "R",
            "tables": ("strloc",),
            "cols": (
                ("srl_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Description", "Y")),
            "where": [("srl_cono", "=", self.conum)]}
        grp = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (
                ("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        r1s = (("Yes","Y"), ("No","N"))
        fld = (
            (("T",0,0,0),"ID1",10,"Reporting Date","Reporting Date",
                self.sysdtw,"Y",self.doDate,None,None,("efld",)),
            (("T",0,1,0),"IUA",1,"Location","Location",
                "","Y",self.doLoc,loc,None,("efld",)),
            (("T",0,2,0),"IUA",3,"Product Group","Product Group",
                "","Y",self.doGrp,grp,None,("efld",)),
            (("T",0,3,0),("IRB",r1s),0,"Include Zeros","Include Zeros",
                "N","Y",self.doZero,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"))

    def doDate(self, frt, pag, r, c, p, i, w):
        self.repdtw = w
        self.repdtd = self.df.t_disp[pag][0][0]

    def doLoc(self, frt, pag, r, c, p, i, w):
        if not w:
            self.loc = ""
            self.locd = "ALL"
        else:
            acc = self.sql.getRec(tables="strloc", cols=["srl_desc"],
                where=[("srl_cono", "=", self.conum), ("srl_loc", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Location"
            self.loc = w
            self.locd = acc[0]

    def doGrp(self, frt, pag, r, c, p, i, w):
        if w:
            acc = self.sql.getRec(tables="strgrp", cols=["gpm_desc"],
                where=[("gpm_cono", "=", self.conum), ("gpm_group", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Group"
        self.group = w

    def doZero(self, frt, pag, r, c, p, i, w):
        self.zero = w

    def doEnd(self):
        self.df.closeProcess()
        whr = [("st1_cono", "=", self.conum)]
        if self.group:
            whr.append(("st1_group", "=", self.group))
        whr.append(("st1_value_ind", "=", "A"))
        all = self.sql.getRec(tables="strmf1", cols=["st1_group", "st1_code",
            "st1_desc", "st1_uoi"], where=whr, order="st1_group, st1_code")
        if not all:
            showError(self.mf.body, "Processing Error",
            "No Records Selected")
        elif self.df.repprt[0][2] == "export":
            self.exportReport(all)
        else:
            self.printReport(all)
        self.mf.closeLoop()

    def exportReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        expnam = getModName(self.mf.wrkdir, __name__, self.conum)
        self.expheads = [self.head % self.sysdttm]
        self.expheads.append("Stock to Order Report as at %s" % self.sysdtd)
        self.expheads.append("(Options: Report Date %s)" % self.repdtd)
        self.expheads.append("Location %s  %s" % (self.loc, self.locd))
        self.expcolsh = [self.colsh]
        self.expforms = self.forms
        self.expdatas = []
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                p.closeProgress()
                return
            vals = self.getValues(dat)
            if not vals:
                continue
            self.expdatas.append(["BODY", [vals[0].work, vals[1].work,
                vals[2].work, vals[3].work, vals[4].work, vals[5].work,
                vals[6].work, vals[7].work]])
        p.closeProgress()
        doWriteExport(xtype=self.df.repprt[0][1], name=expnam, sheet=__name__,
            heads=self.expheads, colsh=self.expcolsh, forms=self.expforms,
            datas=self.expdatas, rcdic=self.mf.rcdic)

    def printReport(self, all):
        chrs = len(self.colsh)
        for f in self.forms:
            chrs += int(f[1])
        if f[0][0].lower() == "s":
            chrs -= 2
        else:
            chrs -= 1
        pad = chrs - 35 - len(self.sysdttm)
        self.head1 = self.head % (" " * pad + self.sysdttm)
        self.head2 = "Stock to Order Report as at %s%s" % (self.sysdtd, "%s%s")
        pad = chrs - len(self.head2) + 4 - 11 # %s%s and ' Page     1'
        self.head2 = self.head2 % (" " * pad, " Page %5s")
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        pdfnam = getModName(self.mf.wrkdir, __name__, self.conum, ext="pdf")
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, pdfnam, self.head1)
        self.pgnum = 0
        self.pglin = 99
        old_grp = ""
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            vals = self.getValues(dat)
            if not vals:
                continue
            if self.pglin > self.lpp:
                if self.pgnum != 0:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            self.t.textLine("%s %s %s %s %s %s %s %s" % (vals[0].disp,
                vals[1].disp, vals[2].disp, vals[3].disp, vals[4].disp,
                vals[5].disp, vals[6].disp, vals[7].disp))
            self.pglin = self.pglin + 1
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.c.drawText(self.t)
                if self.pglin > (self.lpp - 2):
                    self.c.showPage()
                    self.pageHeading()
                self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
        self.closeProcess()

    def getValues(self, data):
        grp = CCD(data[0], "UA", 3)
        code = CCD(data[1], "NA", 20)
        desc = CCD(data[2], "UA", 30)
        uoi = CCD(data[3], "NA", 10)
        whr = [("st2_cono", "=", self.conum), ("st2_group", "=", grp.work),
            ("st2_code", "=", code.work)]
        if self.loc:
            whr.append(("st2_loc", "=", self.loc))
        st2 = self.sql.getRec(tables="strmf2", cols=["st2_reord_ind",
            "st2_reord_level", "st2_reord_qty"], where=whr, limit=1)
        if not st2:
            return
        if self.zero == "N" and not st2[1]:
            return
        req = CCD(st2[1], "SD", 12.2)
        bals = Balances(self.mf, "STR", self.conum, self.repdtw,
            keys=(grp.work, code.work, self.loc))
        figs = [0] * 8
        cbal, obal, bbal = bals.doStrOrds()
        tbal = float(ASD(cbal.work) + ASD(obal.work) - ASD(bbal.work))
        if self.zero == "N" and req.work < tbal:
            return
        tbal = float(ASD(req.work) - ASD(tbal))
        if tbal < st2[2]:
            tbal = st2[2]
        tbal = CCD(tbal, "SD", 12.2)
        if self.zero == "N" and not tbal.work:
            return
        if self.zero == "Y" and not cbal.work and not obal.work and not bbal.work and not tbal.work:
            return
        return (grp, code, desc, uoi, cbal, obal, bbal, tbal)
            
    def pageHeading(self):
        self.pgnum = self.pgnum + 1
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine(self.head1)
        self.t.textLine()
        self.t.textLine(self.head2 % self.pgnum)
        self.t.textLine()
        self.t.textLine("%-22s%-10s%-1s" % \
            ("(Options: Report Date-", self.df.t_disp[0][0][0], ")"))
        self.t.textLine()
        self.t.textLine("%-8s %s  %s" % \
            ("Location", self.loc, self.locd))
        self.t.textLine()
        self.t.textLine("%-3s %-20s %-30s %-10s %11s  %11s  %11s  %11s" % \
            ("Grp", "Product-Code", "Description", "U.O.I", "Qty-Balance",
            "Qty-OnOrder", "Qty-BkOrder", "Qty-ToOrder"))
        self.t.textLine("%s" % (self.suc * len(self.head1)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 10

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

