#! /usr/bin/env python

"""
SYNOPSIS
    Stores Interrogation.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, CCD, GetCtl, NotesCreate, SChoice
from TartanClasses import Sql, SRec, RepPrt, TabPrt, TartanDialog
from tartanFunctions import askChoice, getModName
from tartanWork import mthnam, sttrtp

class st4010:
    def __init__(self, mf, coy, nam, period, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = period
        self.capnm = usr
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlvmf", "drschn", "drsmst", "genmst",
            "strgrp", "strloc", "strmf1", "strmf2", "strprc", "strrcp",
            "strtrn", "struoi"], prog=__name__)
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "no"
        self.glint = strctl["cts_glint"]
        self.locs = strctl["cts_locs"]
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) +  (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.per_from = 0
        self.per_to = 0

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
                "Stores Interrogation (%s)" % __name__)
        gpm = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (
                ("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        stm = {
            "stype": "R",
            "tables": ("strmf1",),
            "cols": (
                ("st1_group", "", 0, "Grp"),
                ("st1_code", "", 0, "Product Code"),
                ("st1_type", "", 0, "T"),
                ("st1_desc", "", 0, "Description", "Y")),
            "where": [("st1_cono", "=", self.conum)],
            "whera": [["T", "st1_group", 0, 0]],
            "index": 1}
        stl = {
            "stype": "R",
            "tables": ("strmf2","strmf1","strloc"),
            "cols": (
                ("st2_group", "", 0, "Grp"),
                ("st2_code", "", 10, "Product Code"),
                ("st1_desc", "", 30, "Description"),
                ("st2_loc", "", 1, "L"),
                ("srl_desc", "", 20, "Location", "Y")),
            "where": [("st1_cono=st2_cono",), ("st1_group=st2_group",),
                    ("st1_code=st2_code",), ("st2_cono=srl_cono",),
                    ("st2_loc=srl_loc",), ("st1_cono", "=", self.conum)],
            "whera": [["T", "st1_group", 0, 0], ["T", "st1_code", 1, 0]],
            "index": 3}
        tag = (
            ("Basic", self.doTagSelect, ("T",0,0), ("T",0,1)),
            ("Flags", self.doTagSelect, ("T",0,0), ("T",0,1)),
            ("Balances", self.doTagSelect, ("T",0,0), ("T",0,1)),
            ("History", self.doTagSelect, ("T",0,0), ("T",0,1)),
            ("Trans", self.doTrans1, ("T",0,0), ("T",0,1)))
        fld = (
            (("T",0,0,0),"IUA",3,"Grp","Product Group",
                "","N",self.doGroup,gpm,None,("notblank",)),
            (("T",0,0,0),"INA",20,"Code","Product Code",
                "","N",self.doCode,stm,None,("notblank",)),
            (("T",0,0,0),"ONA",30,""),
            (("T",0,0,0),"IUA",1,"Loc","Location",
                "","N",self.doLoc,stl,None,("efld",)),
            (("T",0,0,0),"ONA",20,""),
            (("T",1,0,0),"OUA",1,"Type of Item"),
            (("T",1,1,0),"ONA",10,"Units Of Issue"),
            (("T",1,1,0),"ONA",20,""),
            (("T",1,2,0),"OUI",7,"Units Per Pack"),
            (("T",1,3,0),"OUA",1,"Value Indicator"),
            (("T",1,4,0),"OUA",1,"VAT Code"),
            (("T",1,4,0),"ONA",29,""),
            (("T",1,5,0),"OUI",3,"Exclusive Chainstore"),
            (("T",1,5,0),"ONA",27,""),
            (("T",1,6,0),"ONA",7,"Exclusive Account"),
            (("T",1,6,0),"ONA",23,""),
            (("T",1,7,0),"OUI",7,"Sales Code"),
            (("T",1,7,0),"ONA",23,""),
            (("T",1,8,0),"OUI",7,"COS Code"),
            (("T",1,8,0),"ONA",23,""),
            (("T",1,9,0),"OUA",8,"Bin Number"),
            (("T",1,10,0),"OUA",1,"Re-Order Indicator"),
            (("T",1,11,0),"OUI",7,"Re-Order Level"),
            (("T",1,12,0),"OUI",7,"Re-Order Quantity"),
            (("T",1,13,0),"OUD",6.2,"Selling Price Markup"),
            (("T",2,0,0),"OUD",7.2,"Lead Time"),
            (("T",2,1,0),"OSD",10.2,"Growth Factor"),
            (("T",3,0,0),"Od1",10,"Date Last Receipt"),
            (("T",3,1,0),"Od1",10,"Date Last Issue"),
            (("T",3,2,0),"OSD",12.2,"Quantity Balance"),
            (("T",3,3,0),"OSD",12.2,"Value Balance"),
            (("T",3,4,0),"OSD",12.2,"Average Unit Cost"),
            (("T",3,5,0),"OSD",12.2,"Quantity on Order"),
            (("T",3,6,0),"OSD",12.2,"Quantity on Sales"),
            (("C",4,0,0),"OSD",13.2,"Qty-Receipts","",
                "","N",None,None,None,None,("Months",13)),
            (("C",4,0,1),"OSD",13.2,"Val-Receipts"),
            (("C",4,0,2),"OSD",13.2,"Qty-Issues"),
            (("C",4,0,3),"OSD",13.2,"Val-Issues"),
            (("T",5,0,0),"Id2",7,"Start Period", "Start Period (0=All)",
                "","N",self.doTrans2a,None,None,("efld",)),
            (("T",5,0,0),"Id2",7,"End Period", "End Period",
                "","N",self.doTrans2b,None,None,("efld",)))
        but = (
            ("Clear",None,self.doClear,1,("T",0,0),("T",0,1)),
            ("Recipe",None,self.doRecipe,None,None,None),
            ("Notes",None,self.doNotes,0,("T",0,0),("T",0,1)),
            ("Print",None,self.doPrint,1,("T",0,0),("T",0,1)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEndTop,"N"),None,None,None,None,(self.doTrans2c,"N"))
        txt = (self.doExit,None,None,None,None,self.doExit)
        self.df = TartanDialog(self.mf, title=self.tit, screen=self.mf.body,
            tags=tag, eflds=fld, butt=but, tend=tnd, txit=txt)
        yer = self.sysdtw / 10000
        mon = (self.sysdtw % 10000) / 100
        self.hist_tit = []
        for x in range(1, 14):
            if x == 13:
                txt = "Last 12 Month Total"
                self.df.colLabel[4][x-1].set_text(txt)
                self.hist_tit.append(txt)
            else:
                nam = mthnam[mon][1]
                nam = nam + (" " * (15-len(nam))) + str(yer)
                self.df.colLabel[4][x-1].set_text(nam)
                self.hist_tit.append(nam)
            if x != 13:
                mon -= 1
                if not mon:
                    mon = mon + 12
                    yer -= 1

    def doGroup(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="strgrp", cols=["gpm_group"],
            where=[("gpm_cono", "=", self.conum), ("gpm_group", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Group"
        self.group = w

    def doCode(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="strmf1", where=[("st1_cono", "=",
            self.conum), ("st1_group", "=", self.group), ("st1_code", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Code"
        self.code = w
        self.desc = acc[self.sql.strmf1_col.index("st1_desc")]
        self.df.loadEntry("T", 0, 2, data=self.desc)
        d = 3
        for pag in range(1, 2):
            for x in range(0, 15):
                if pag == 1 and x == 2:
                    self.loadUoi(x, acc[d-1])
                elif pag == 1 and x == 6:
                    self.loadVat(x, acc[d-1])
                elif pag == 1 and x == 8:
                    self.loadChnExcl(x, acc[d-1])
                elif pag == 1 and x == 10:
                    self.loadAccExcl(x, acc[d-1])
                elif pag == 1 and x in (12, 14):
                    self.loadGen(x, acc[d-1])
                else:
                    self.df.loadEntry("T", pag, x, data=acc[d])
                    if d == 3:
                        self.itype = acc[d]
                        d += 2
                    else:
                        d += 1
        if self.locs == "N":
            self.df.loadEntry("T", 0, 3, data="1")
            no = self.doLoc(frt, pag, r, c+1, p+1, i+1, "1")
            if no:
                return no
            return "sk2"

    def doLoc(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="strloc", cols=["srl_desc"],
            where=[("srl_cono", "=", self.conum), ("srl_loc", "=", w)], limit=1)
        if not acc:
            return "Invalid Location Code"
        self.df.loadEntry("T", 0, 4, data=acc[0])
        acc = self.sql.getRec(tables="strmf2", where=[("st2_cono", "=",
            self.conum), ("st2_group", "=", self.group), ("st2_code", "=",
            self.code), ("st2_loc", "=", w)], limit=1)
        if not acc:
            return "This location does not exist for this product"
        self.loc = w
        d = 4
        for pag in range(1, 2):
            for x in range(15, 20):
                self.df.loadEntry("T", pag, x, data=acc[d])
                d += 1
        for pag in range(2, 3):
            for x in range(0, 2):
                self.df.loadEntry("T", pag, x, data=acc[d])
                d += 1
        self.loadBalances()
        if self.itype == "R":
            self.df.setWidget(self.df.B1, "normal")
        else:
            self.df.setWidget(self.df.B1, "disabled")

    def doEndTop(self):
        self.df.last[0] = [0, 0]
        self.df.setWidget(self.df.topEntry[0][0], state="disabled")

    def loadUoi(self, pos, uoi):
        acc = self.sql.getRec(tables="struoi", cols=["unm_desc"],
            where=[("unm_cono", "=", self.conum), ("unm_unit", "=", uoi)],
            limit=1)
        if acc:
            self.df.loadEntry("T", 1, pos, data=acc[0])

    def loadChnExcl(self, pos, chn):
        self.chn = chn
        if self.chn:
            acc = self.sql.getRec(tables="drschn", cols=["chm_name"],
                where=[("chm_cono", "=", self.conum), ("chm_chain", "=",
                self.chn)], limit=1)
            if acc:
                self.df.loadEntry("T", 1, pos, data=acc[0])

    def loadAccExcl(self, pos, drm):
        acc = self.sql.getRec(tables="drsmst", cols=["drm_name"],
            where=[("drm_cono", "=", self.conum), ("drm_chain", "=", self.chn),
            ("drm_acno", "=", drm)], limit=1)
        if acc:
            self.df.loadEntry("T", 1, pos, data=acc[0])

    def loadVat(self, pos, vat):
        acc = self.sql.getRec(tables="ctlvmf", cols=["vtm_desc"],
            where=[("vtm_cono", "=", self.conum), ("vtm_code", "=", vat)],
            limit=1)
        if not vat:
            self.df.loadEntry("T", 1, pos, data="Invalid VAT Record")
        else:
            self.df.loadEntry("T", 1, pos, data=acc[0])

    def loadGen(self, pos, gen):
        if self.glint == "N":
            return
        acc = self.sql.getRec(tables="genmst", cols=["glm_desc"],
            where=[("glm_cono", "=", self.conum), ("glm_acno", "=", gen)],
            limit=1)
        if acc:
            self.df.loadEntry("T", 1, pos, data=acc[0])

    def loadBalances(self):
        bals = Balances(self.mf, "STR", self.conum, (self.sysdtw/100),
            keys=(self.group, self.code, self.loc, ("P", self.period[0])))
        m_ob, m_mv, m_cb, y_ob, y_mv, y_cb, ac, lc, = bals.doStrBals()
        cb, oo, bo = bals.doStrOrds()
        this, hist = bals.doStrHist()
        rec = hist[0]
        rec.append(this[0])
        iss = hist[1]
        iss.append(this[1])
        lrec = self.sql.getRec(tables="strtrn", cols=["max(stt_trdt)"],
            where=[("stt_cono", "=", self.conum), ("stt_group", "=",
            self.group), ("stt_code", "=", self.code), ("stt_loc", "=",
            self.loc), ("stt_type", "in", (1,3))])
        if not lrec[0][0]:
            lastrec = 0
        else:
            lastrec = lrec[0][0]
        liss = self.sql.getRec(tables="strtrn", cols=["max(stt_trdt)"],
            where=[("stt_cono", "=", self.conum), ("stt_group", "=",
            self.group), ("stt_code", "=", self.code), ("stt_loc", "=",
            self.loc), ("stt_type", "in", (2,4,7))])
        if not liss[0][0]:
            lastiss = 0
        else:
            lastiss = liss[0][0]
        qbal = float(y_cb[0])
        vbal = float(y_cb[1])
        if qbal:
            cost = round((vbal / qbal), 2)
        else:
            cost = 0.0
        cst = CCD(cost, "SD", 10.2)
        self.df.loadEntry("T", 3, 0, data=lastrec)
        self.df.loadEntry("T", 3, 1, data=lastiss)
        self.df.loadEntry("T", 3, 2, data=qbal)
        self.df.loadEntry("T", 3, 3, data=vbal)
        self.df.loadEntry("T", 3, 4, data=cst.disp)
        self.df.loadEntry("T", 3, 5, data=oo.disp)
        self.df.loadEntry("T", 3, 6, data=bo.disp)
        p = 0
        for x in range (0, 13):
            i = 0
            self.df.loadEntry("C", 4, p, data=rec[x][0])
            p = p + 1
            i = i + 1
            self.df.loadEntry("C", 4, p, data=rec[x][1])
            p = p + 1
            i = i + 1
            amt = float(ASD(0) - ASD(iss[x][0]))
            self.df.loadEntry("C", 4, p, data=amt)
            p = p + 1
            i = i + 1
            amt = float(ASD(0) - ASD(iss[x][1]))
            self.df.loadEntry("C", 4, p, data=amt)
            p = p + 1
            i = i + 1

    def doTagSelect(self):
        self.mf.updateStatus("")

    def doRecipe(self, widget=None):
        tit = "Recipe"
        state = self.df.disableButtons()
        rcp = self.getRecipeItems()
        rec = SRec(self.mf, screen=self.mf.body, title=tit, cols=rcp[0],
            where=rcp[1], wtype="D", butt=[("Print", self.printRecipe)])
        self.df.enableButtons(state=state)

    def printRecipe(self, widget=None):
        self.df.mstAlw.hide()
        hds = ["Stores's Recipe",
            "Location: %s  Code: %s  Description: %s" % (self.loc, self.code,
            self.desc)]
        rcp = self.getRecipeItems()
        RepPrt(self.mf, conum=self.conum, conam=self.conam, name=__name__,
            heads=hds, cols=rcp[0], tables=rcp[1], ttype="D", prtdia=("Y","V"))
        self.df.mstAlw.show()

    def getRecipeItems(self):
        tab = ["strrcp", "strmf1"]
        col = ["srr_rgroup", "srr_rcode", "st1_desc", "srr_rqty"]
        whr = [("srr_cono", "=", self.conum), ("srr_group", "=", self.group),
            ("srr_code", "=", self.code), ("srr_loc", "=", self.loc),
            ("st1_group=srr_rgroup",), ("st1_code=srr_rcode",)]
        odr = "srr_rgroup, srr_code"
        items = self.sql.getRec(tables=tab, cols=col, where=whr, order=odr)
        for seq, item in enumerate(items):
            bals = Balances(self.mf, "STR", self.conum, (self.sysdtw / 100),
                keys=(item[0], item[1], self.loc, ("P", self.period[0])))
            m_ob, m_mv, m_cb, y_ob, y_mv, y_cb, ac, lc, = bals.doStrBals()
            items[seq].extend([y_cb[0], lc])
        return ((
            ("srr_rgroup","UA",3.0,"Grp"),
            ("srr_rcode","NA",20.0,"Product-Code"),
            ("st1_desc","NA",30.0,"Description"),
            ("srr_rqty","UD",11.2,"   Quantity"),
            ("balance","SD",13.2,"    In-Stock"),
            ("cost","UD",11.2,"  Last-Cost")), items)

    def doTrans1(self, widget=None):
        self.df.focusField("T", 5, 1)

    def doTrans2a(self, frt, pag, r, c, p, i, w):
        self.per_from = w
        self.df.loadEntry("T", 5, 1, data=w)
        if not self.per_from:
            return "nc"

    def doTrans2b(self, frt, pag, r, c, p, i, w):
        self.per_to = w
        if not self.per_to or self.per_to < self.per_from:
            return "Invalid Period"

    def doTrans2c(self):
        tit = "Transactions for Item: %s %s - %s" % \
            (self.group, self.code, self.desc)
        tab = ["strtrn"]
        crt = (("stt_trdt", "", 0, "   Date"),
                ("stt_batch", "", 0, "Batch"),
                ("stt_type", ("XX", sttrtp), 3, "Typ"),
                ("stt_ref1", "", 0, "Reference","Y"),
                ("stt_ref2", "", 0, "Ref-Num-2"),
                ("stt_qty", "", 0, "  Quantity"),
                ("stt_cost", "", 0, "      Cost"),
                ("stt_sell", "", 0, "   Selling"),
                ("stt_desc", "", 0, "Details"))
        whr = [("stt_cono", "=", self.conum), ("stt_group", "=", self.group),
            ("stt_code", "=", self.code), ("stt_loc", "=", self.loc)]
        if self.per_from:
            whr.extend([("stt_curdt", ">=", self.per_from), ("stt_curdt",
                "<=", self.per_to)])
        odr = "stt_trdt, stt_ref1"
        state = self.df.disableButtons()
        while True:
            rec = SRec(self.mf, screen=self.df.nb.Page5, title=tit, tables=tab,
                cols=crt, where=whr, order=odr, rows=12)
            if rec.selection:
                typ = rec.selection[2]
                doc = rec.selection[3]
                if self.itype == "R" and typ == 7:
                    bt = (("Transaction", 1), ("Recipe", 2))
                    ask = askChoice(self.df.nb.Page5, head="Details",
                        mess="Transaction Details or the Recipe Details",
                        butt=bt, default=1)
                else:
                    ask = 1
                if ask == 1:
                    wher = [
                        ("stt_cono", "=", self.conum),
                        ("stt_group", "=", self.group),
                        ("stt_code", "=", self.code),
                        ("stt_loc", "=", self.loc),
                        ("stt_type", "=", typ),
                        ("stt_ref1", "=", doc)]
                    TabPrt(self.mf, tabs="strtrn", wher=wher, pdia=False)
                else:
                    rct = "Recipe Details"
                    rcc = (
                        ("si3_seq", "", 0, ""),
                        ("si3_rgroup", "", 0, ""),
                        ("si3_rcode", "", 0, ""),
                        ("st1_desc", "", 0, ""),
                        ("si3_rqty", "", 0, ""),
                        ("si3_cost", "", 0, ""))
                    rcw = (
                        ("si3_cono", "=", self.conum),
                        ("si3_rtn", "=", "I"),
                        ("si3_docno", "=", doc),
                        ("st1_cono=si3_cono",),
                        ("st1_group=si3_rgroup",),
                        ("st1_code=si3_rcode",))
                    rco = "si3_seq"
                    SRec(self.mf, title=rct, tables=["slsiv3", "strmf1"],
                        cols=rcc, where=rcw, order=rco)
            else:
                break
        self.df.enableButtons(state=state)
        self.df.focusField("T", 5, 1)

    def doNotes(self, wiget=None):
        state = self.df.disableButtons()
        self.df.mstAlw.hide()
        nf = NotesCreate(self.mf, self.conum, self.conam, self.capnm, "STR",
            "%3s%s" % (self.group, self.code))
        self.df.mstAlw.show()
        self.df.enableButtons(state=state)

    def doClear(self, widget=None):
        self.df.selPage("Basic")
        self.df.focusField("T", 0, 1)

    def doPrint(self, widget=None):
        titl = "Select the required print options"
        data = [
            ("1", "Information"),
            ("2", "Transactions")]
        if self.itype == "R":
            data.append(("3", "Recipe"))
        ss = SChoice(self.mf, titl=titl, data=data)
        self.mf.updateStatus("")
        if ss.selection:
            state = self.df.disableButtons()
            self.df.mstAlw.hide()
            if ss.selection[0] == "1":
                head = "Stores Masterfile Record"
                tab = ["strmf1", "strmf2"]
                col = [
                    "st1_group", "st1_code", "st2_loc", "st1_desc", "st1_type",
                    "st1_uoi", "st1_units", "st1_value_ind", "st1_vatcode",
                    "st1_chn_excl", "st1_acc_excl", "st1_sls", "st1_cos",
                    "st2_bin", "st2_reord_ind", "st2_reord_level",
                    "st2_reord_qty", "st2_markup", "st2_lead", "st2_growth"]
                whr = [("st1_cono", "=", self.conum), ("st1_group", "=",
                    self.group), ("st1_code", "=", self.code),
                    ("st2_cono=st1_cono",), ("st2_group=st1_group",),
                    ("st2_code=st1_code",), ("st2_loc", "=", self.loc)]
                TabPrt(self.mf, self.conum, self.conam, name=__name__,
                    head=head, tabs=tab, cols=col, wher=whr, keys=[self.group,
                    self.code, self.loc])
            elif ss.selection[0] == "2":
                tab = ["strtrn"]
                hds = ["Stores's Transactions",
                    "Location: %s  Code: %s  Description: %s" % (self.loc,
                    self.code, self.desc)]
                col = []
                dic = self.sql.strtrn_dic
                for c in ["stt_ref1", "stt_trdt", "stt_type", "stt_qty",
                                    "stt_cost", "stt_sell", "stt_desc"]:
                    col.append([c, dic[c][2], dic[c][3], dic[c][5]])
                whr = [("stt_cono", "=", self.conum), ("stt_group", "=",
                    self.group), ("stt_code", "=", self.code), ("stt_loc", "=",
                    self.loc)]
                if self.per_from:
                    whr.extend([("stt_curdt", ">=", self.per_from),
                        ("stt_curdt", "<=", self.per_to)])
                odr="stt_trdt, stt_ref1"
                gtots=["stt_qty", "stt_cost", "stt_sell"]
                RepPrt(self.mf, conum=self.conum, conam=self.conam,
                    name=__name__, tables=tab, heads=hds, cols=col, where=whr,
                    order=odr, gtots=gtots, prtdia=("Y","V"))
            else:
                self.printRecipe()
            self.df.mstAlw.show()
            self.mf.gtkDispatch()
            self.df.enableButtons(state=state)
            self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

