#! /usr/bin/env python

"""
SYNOPSIS
    Stores Stock Take Report.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import Balances, CCD, GetCtl, ProgressBar, Sql
from TartanClasses import TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError

class st5010:
    def __init__(self, mf, coy, nam, period):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = period
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["strloc", "strgrp", "strmf1", "strmf2"],
            prog=__name__)
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "no"
        self.locs = strctl["cts_locs"]
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Stock Take Report (%s)" % __name__)
        loc = {
            "stype": "R",
            "tables": ("strloc",),
            "cols": (
                ("srl_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Description", "Y")),
            "where": [("srl_cono", "=", self.conum)]}
        grp = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (
                ("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        r1s = (("Number","N"),("Bin Number","B"))
        r2s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Sort Order","Sort Order",
                "N","Y",self.doSort,None,None,None),
            (("T",0,1,0),"IUA",1,"Location","Location",
                "","Y",self.doLoc,loc,None,("efld",)),
            (("T",0,1,0),"ONA",30,""),
            (("T",0,2,0),"IUA",8,"First Bin Number","First Bin Number",
                "","Y",self.doFbin,None,None,("efld",)),
            (("T",0,3,0),"IUA",8,"Last Bin Number","Last Bin Number",
                "","Y",self.doLbin,None,None,("efld",)),
            (("T",0,4,0),"IUA",3,"Product Group","Product Group",
                "","Y",self.doGroup,grp,None,None),
            (("T",0,5,0),"IUI",5,"Quantity To Print","Quantity To Print",
                "","Y",self.doQty,None,None,("efld",)),
            (("T",0,6,0),("IRB",r2s),0,"Ignore Zero Balances","Ignore "\
                "Zero Balances","N","Y",self.doZero,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.closeProcess, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doSort(self, frt, pag, r, c, p, i, w):
        self.sort = w
        if self.locs == "N":
            self.loc = "1"
            self.df.loadEntry(frt, pag, i+1, data=self.loc)
            self.loadLoc()
            return "sk2"

    def doLoc(self, frt, pag, r, c, p, i, w):
        self.loc = w
        self.loadLoc()
        if not self.locd:
            return "Invalid Location"
        if self.sort == "N":
            self.fbin = ""
            self.lbin = "zzzzzzzz"
            return "sk3"

    def loadLoc(self):
        acc = self.sql.getRec(tables="strloc", cols=["srl_desc"],
            where=[("srl_cono", "=", self.conum), ("srl_loc", "=", self.loc)],
            limit=1)
        if acc:
            self.locd = acc[0]
            self.df.loadEntry("T", 0, 2, data=self.locd)
        else:
            self.locd = None

    def doFbin(self, frt, pag, r, c, p, i, w):
        self.fbin = w

    def doLbin(self, frt, pag, r, c, p, i, w):
        if w:
            if w < self.fbin:
                return "Invalid Last Bin < First Bin"
            self.lbin = w
        else:
            self.lbin = "zzzzzzzz"

    def doGroup(self, frt, pag, r, c, p, i, w):
        if not w:
            self.group = ""
        else:
            acc = self.sql.getRec(tables="strgrp", cols=["gpm_desc"],
                where=[("gpm_cono", "=", self.conum), ("gpm_group", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Group"
            self.group = w

    def doQty(self, frt, pag, r, c, p, i, w):
        self.qty = w

    def doZero(self, frt, pag, r, c, p, i, w):
        self.zero = w

    def doEnd(self):
        self.df.closeProcess()
        if not self.loc:
            self.sloc = ""
            self.eloc = "z"
        else:
            self.sloc = self.loc; self.eloc = self.loc
        if not self.group:
            self.sgrp = ""
            self.egrp = "zzz"
        else:
            self.sgrp = self.group; self.egrp = self.group
        if self.sort == "N":
            sortby = "st2_loc, st2_group, st2_code"
        else:
            sortby = "st2_loc, st2_bin, st2_group, st2_code"
        all = self.sql.getRec(tables=["strmf1", "strmf2"], cols=["st2_group",
            "st2_code", "st2_loc", "st2_bin", "st1_desc", "st1_uoi"],
            where=[("st1_cono=st2_cono",), ("st1_group=st2_group",),
            ("st1_code=st2_code",), ("st1_type", "not", "in", ("J", "R")),
            ("st2_cono", "=", self.conum), ("st2_group", ">=", self.sgrp),
            ("st2_group", "<=", self.egrp), ("st2_loc", ">=", self.sloc),
            ("st2_loc", "<=", self.eloc), ("st2_bin", ">=", self.fbin),
            ("st2_bin", "<=", self.lbin)], order=sortby)
        if not all:
            showError(self.mf.body, "Processing Error",
            "No Records Selected")
        else:
            self.printReport(all)
        self.mf.closeLoop()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %3s %45s %2s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        count = 0
        old_grp = ""
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            grp = CCD(dat[0], "UA", 3)
            self.groupd = grp.disp
            code = CCD(dat[1], "NA", 20)
            loc = CCD(dat[2], "UA", 1)
            bin = CCD(dat[3], "UA", 8)
            desc = CCD(dat[4], "UA", 30)
            uoi = CCD(dat[5], "NA", 10)
            bals = Balances(self.mf, "STR", self.conum, (self.sysdtw/100),
                keys=(grp.work, code.work, loc.work, ("P", self.period[0])))
            m_ob, m_mv, m_cb, y_ob, y_mv, y_cb, ac, lc = bals.doStrBals()
            if self.zero == "Y" and not y_cb[0]:
                continue
            if old_grp and old_grp != grp.work:
                self.pglin = 99
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            self.t.textLine("%s   %s   %s   %s   %s" % \
                (grp.disp, code.disp, desc.disp, uoi.disp, bin.disp))
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.pglin = self.pglin + 2
            old_grp = grp.work
            count = count + 1
            if self.qty and count >= self.qty:
                break
        p.closeProgress()
        if can != "yes" and self.pgnum:
            if self.t:
                self.c.drawText(self.t)
            self.c.showPage()
        elif self.pgnum:
            if self.t:
                self.c.drawText(self.t)
            self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-23s %-10s %41s %5s" % \
            ("Stock Take Report as at", self.sysdtd, "Page", self.pgnum))
        self.t.textLine()
        self.t.textLine("%-5s %3s      %-8s %s  %s" % \
            ("Group", self.groupd, "Location", self.loc, self.locd))
        self.t.textLine()
        self.t.textLine("%-3s   %-20s   %-30s   %-10s   %-8s %-10s" % (
            "Grp", "Product-Code", "Description", "U.O.I", "Bin-Loc",
            "Quantities"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def closeProcess(self):
        self.df.closeProcess()
        self.mf.closeLoop()

