#! /usr/bin/python

"""
SYNOPSIS
    Stores Stock Take Returns.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import Balances, GetCtl, Sql, TartanDialog

class st5020:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.dataHeader()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["strloc", "strgrp", "strmf1", "strmf2",
            "strprc", "strvar"], prog=__name__)
        if self.sql.error:
            return self.sql.error
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "no"
        self.locs = strctl["cts_locs"]
        self.automu = strctl["cts_automu"]
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.last = 0

    def dataHeader(self):
        self.tit = ("%03i %s" % (self.conum, self.conam), "Stores Ledger "\
            "Stock Take Returns (%s)" % __name__)
        loc = {
            "stype": "R",
            "tables": ("strloc",),
            "cols": (
                ("srl_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Description", "Y")),
            "where": [("srl_cono", "=", self.conum)]}
        self.gpm = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (
                ("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        stm1 = {
            "stype": "R",
            "tables": ("strmf1",),
            "cols": (
                ("st1_group", "", 0, "Grp"),
                ("st1_code", "", 0, "Product Code"),
                ("st1_desc", "", 0, "Description", "Y")),
            "where": [("st1_cono", "=", self.conum), ("st1_type", "not",
                "in", ("J", "R"))],
            "whera": [["T", "st1_group", 4, 0]],
            "order": "st1_group, st1_code",
            "index": 1}
        r1s = (("Number","N"),("Bin Number","B"))
        r2s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Sort Order","Sort Order",
                "N","Y",self.doSort,None,None,None),
            (("T",0,1,0),"IUA",1,"Location","Location",
                "","Y",self.doLoc,loc,None,("efld",)),
            (("T",0,2,0),("IRB",r2s),0,"Auto Sequence","Auto Sequence",
                "N","Y",self.doAuto,None,None,None),
            (("T",0,3,0),"IUA",8,"First Bin Number","First Bin Number",
                "","Y",self.doFbin,None,None,("efld",)),
            (("T",0,4,0),"IUA",3,"First Group","First Group",
                "","Y",self.doFgrp,self.gpm,None,None),
            (("T",0,5,0),"INA",20,"First Code","First Code",
                "","Y",self.doFcode,stm1,None,None),
            (("T",0,6,0),("IRB",r2s),0,"Selling Prices","Selling Prices",
                "N","Y",self.doSell,None,None,None),
            (("T",0,7,0),("IRB",r2s),0,"Automatic Markup","Automatic Markup",
                self.automu,"Y",self.doMkUp,None,None,None))
        tnd = ((self.endPage1,"y"),)
        txt = (self.exitPage1,)
        self.df1 = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt)

    def doSort(self, frt, pag, r, c, p, i, w):
        self.sort = w
        if self.locs == "N":
            self.loc = "1"
            self.df1.loadEntry(frt, pag, p+1, data=self.loc)
            return "sk1"

    def doLoc(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="strloc", cols=["srl_desc"],
            where=[("srl_cono", "=", self.conum), ("srl_loc", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Location Code"
        self.loc = w

    def doAuto(self, frt, pag, r, c, p, i, w):
        self.auto = w
        if self.auto != "Y":
            return "sk3"
        self.bin = ""
        self.fgrp = ""
        self.fcode = ""
        if self.sort == "N":
            return "sk1"

    def doFbin(self, frt, pag, r, c, p, i, w):
        self.fbin = w

    def doFgrp(self, frt, pag, r, c, p, i, w):
        if self.fgrp:
            acc = self.sql.getRec(tables="strgrp", cols=["gpm_desc"],
                where=[("gpm_cono", "=", self.conum), ("gpm_group", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Group"
        self.fgrp = w
        if not self.fgrp:
            self.fcode = ""
            self.df1.loadEntry(frt, pag, p+1, data=self.fcode)
            return "sk1"

    def doFcode(self, frt, pag, r, c, p, i, w):
        if self.fcode:
            acc = self.sql.getRec(tables="strmf1", cols=["st1_type"],
                where=[("st1_cono", "=", self.conum), ("st1_group", "=",
                self.fgrp), ("st1_code", "=", w)], limit=1)
            if not acc:
                return "Invalid Code"
            if acc[0] in ("J", "R"):
                return "Invalid Code (Group Item)"
        self.fcode = w

    def doSell(self, frt, pag, r, c, p, i, w):
        self.sprices = w
        if self.sprices == "N":
            self.mkup = "N"
            self.df1.loadEntry(frt, pag, p+1, data=self.mkup)
            return "sk1"

    def doMkUp(self, frt, pag, r, c, p, i, w):
        self.mkup = w

    def endPage1(self):
        self.df1.closeProcess()
        self.dataBody()
        if self.auto == "Y":
            whr = [("st1_cono", "=", self.conum)]
            if self.fgrp:
                whr.append(("st1_group", ">=", self.fgrp))
            if self.fcode:
                whr.append(("st1_code", ">=", self.fcode))
            whr.extend([("st1_type", "not", "in", ("J","R")),
                ("st2_cono=st1_cono",), ("st2_group=st1_group",),
                ("st2_code=st1_code",), ("st2_loc", "=", self.loc)])
            self.codes = self.sql.getRec(tables=["strmf1", "strmf2"],
                cols=["st1_group", "st1_code", "st1_desc", "st1_uoi",
                "st2_bin"], where=whr, order="st1_group, st1_code")
            if not self.codes:
                showError(self.mf.body, "No Records",
                    "No Stock records on File")
                self.exitPage2()
            else:
                self.doNextOne()

    def exitPage1(self):
        self.df1.closeProcess()
        self.mf.closeLoop()

    def dataBody(self):
        stm2 = {
            "stype": "R",
            "tables": ("strmf1",),
            "cols": (
                ("st1_group", "", 0, "Grp"),
                ("st1_code", "", 0, "Product Code"),
                ("st1_desc", "", 0, "Description", "Y")),
            "where": [("st1_cono", "=", self.conum), ("st1_type", "not",
                "in", ("J", "R"))],
            "whera": [["C", "st1_group", 0, 0]],
            "index": 1}
        if self.auto == "Y":
            fld = [
                (("C",0,0,0),"ONA",3,"Grp"),
                (("C",0,0,1),"ONA",20,"Product Code")]
        else:
            fld = [
                (("C",0,0,0),"INA",3,"Grp","Product Group",
                    "r","N",self.doGroup2,self.gpm,None,("notblank",)),
                (("C",0,0,1),"INA",20,"Product Code","Product Code",
                    "","N",self.doCode2,stm2,None,("notblank",))]
        fld.extend([
            (("C",0,0,2),"ONA",16,"Description"),
            (("C",0,0,3),"ONA",10,"U.O.I"),
            (("C",0,0,4),"OUA",8,"Bin"),
            (("C",0,0,5),"IUD",9.2,"Unit-Cost","Unit Cost Price",
                "","N",self.doUcost,None,None,("efld",))])
        if self.sprices == "Y" and self.mkup == "N":
            fld.extend([
                (("C",0,0,6),"IUD",9.2,"Unit-Sell","Unit Selling Price",
                    "","N",self.doUsell,None,None,("efld",)),
                (("C",0,0,7),"ISD",9.2,"Quantity","Quantity",
                    "","N",self.doQty,None,None,("efld",))])
        else:
            fld.append(
                (("C",0,0,6),"ISD",9.2,"Quantity","Quantity",
                    "","N",self.doQty,None,None,("efld",)))
        cnd = ((self.endPage2,"y"),) 
        cxt = (self.exitPage2,) 
        self.df2 = TartanDialog(self.mf, title=self.tit, eflds=fld,
            cend=cnd, cxit=cxt)

    def doGroup2(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="strgrp", cols=["gpm_desc"],
            where=[("gpm_cono", "=", self.conum), ("gpm_group", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Group"
        self.group2 = w

    def doCode2(self, frt, pag, r, c, p, i, w):
        ac1 = self.sql.getRec(tables="strmf1", cols=["st1_type", "st1_desc"],
            where=[("st1_cono", "=", self.conum), ("st1_group", "=",
            self.group2), ("st1_code", "=", w)], limit=1)
        if not ac1:
            return "Invalid Code"
        if ac1[0] in ("J", "R"):
            return "Invalid Code (Group Item)"
        ac2 = self.sql.getRec(tables="strmf2", where=[("st2_cono", "=",
            self.conum), ("st2_group", "=", self.group2), ("st2_code", "=",
            w), ("st2_loc", "=", self.loc)], limit=1)
        if not ac2:
            return "Invalid Location For This Product"
        self.code2 = w
        self.df2.loadEntry("C", 0, p+1, data=ac1[0])
        self.df2.loadEntry("C", 0, p+2, data=ac1[1])
        self.bin = ac2[self.sql.strmf2_col.index("st2_bin")]
        self.df2.loadEntry("C", 0, p+3, data=self.bin)
        cst, self.usell, qty = self.doGetValues()
        self.df2.loadEntry("C", 0, p+4, data=cst)
        if self.sprices == "Y" and self.mkup == "N":
            self.df2.loadEntry("C", 0, p+5, data=self.usell)
            self.df2.loadEntry("C", 0, p+6, data=qty)
        else:
            self.df2.loadEntry("C", 0, p+5, data=qty)

    def doUcost(self, frt, pag, r, c, p, i, w):
        self.ucost = w
        if self.sprices == "N" or self.usell:
            return
        acc = self.sql.getRec(tables="strgrp", cols=["gpm_markup"],
            where=[("gpm_cono", "=", self.conum), ("gpm_group", "=",
            self.group2)], limit=1)
        mup = acc[0]
        acc = self.sql.getRec(tables="strmf2", cols=["st2_markup"],
            where=[("st2_cono", "=", self.conum), ("st2_group", "=",
            self.group2), ("st2_code", "=", self.code2)], limit=1)
        if acc[0]:
            mup = acc[0]
        if not mup:
            return
        self.usell = round((self.ucost * (100 + mup) / 100.0), 2)
        if self.mkup == "N":
            self.df2.loadEntry(frt, pag, p+1, data=self.usell)

    def doUsell(self, frt, pag, r, c, p, i, w):
        self.usell = w

    def doQty(self, frt, pag, r, c, p, i, w):
        self.qty = w
        self.amt1 = round((self.qty * self.ucost), 2)

    def endPage2(self):
        self.updateTables()
        self.df2.advanceLine(0)
        if self.auto == "Y":
            self.last += 1
            self.doNextOne()

    def updateTables(self):
        # Stores Variance Transaction
        whr = [("stv_cono", "=", self.conum), ("stv_group", "=", self.group2),
            ("stv_code", "=", self.code2), ("stv_loc", "=", self.loc)]
        if self.sql.getRec(tables="strvar", where=whr, limit=1):
            self.sql.delRec("strvar", where=whr)
        self.sql.insRec("strvar", data=[self.conum, self.group2, self.code2,
            self.loc, self.bin, self.qty, self.ucost, self.usell])
        self.mf.dbm.commitDbase()

    def doNextOne(self):
        if self.last == len(self.codes):
            self.exitPage2()
        else:
            p = self.df2.pos
            r = self.df2.row
            i = self.df2.idx
            data = self.codes[self.last]
            self.group2 = data[0]
            self.code2 = data[1]
            self.bin = data[4]
            cst, self.usell, qty = self.doGetValues()
            if self.sprices == "Y" and self.mkup == "N":
                max = 8
                data.extend([cst, self.usell, qty])
            else:
                max = 7
                data.extend([cst, qty])
            for x in range(0, max):
                self.df2.loadEntry("C", 0, (p-5+x), data=data[x])
            self.df2.focusField("C", 0, p)

    def doGetValues(self):
        var = self.sql.getRec(tables="strvar", where=[("stv_cono", "=",
            self.conum), ("stv_group", "=", self.group2), ("stv_code", "=",
            self.code2), ("stv_loc", "=", self.loc)], limit=1)
        if var:
            qty = var[self.sql.strvar_col.index("stv_qty")]
            cst = var[self.sql.strvar_col.index("stv_ucost")]
            if not cst:
                bal = self.doGetCost(self.group2, self.code2)
                cst = bal[0]
        else:
            cst, qty = self.doGetCost(self.group2, self.code2)
        sel = 0
        if self.sprices == "Y" and self.mkup == "N":
            strprc = self.sql.getRec(tables="strprc", where=[("stp_cono",
                "=", self.conum), ("stp_group", "=", self.group2), ("stp_code",
                "=", self.code2), ("stp_loc", "=", self.loc), ("stp_level",
                "=", 1)], limit=1)
            if strprc:
                sel = strprc[self.sql.strprc_col.index("stp_price")]
        return (cst, sel, qty)

    def doGetCost(self, grp, code):
        bals = Balances(self.mf, "STR", self.conum, (self.sysdtw / 100),
            keys=(grp, code, self.loc))
        vals = bals.doStrBals()
        return (vals[7], vals[5][0])

    def exitPage2(self):
        self.df2.closeProcess()
        self.mf.closeLoop()
