#! /usr/bin/env python

"""
SYNOPSIS
    Stores Location File Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import GetCtl, Sql, TartanDialog

class stc410:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "strloc", prog=__name__)
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "no"
        self.locs = strctl["cts_locs"]
        if self.locs == "N":
            return "no"

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Stores Locations File Maintenance (%s)" % __name__)
        loc = {
            "stype": "R",
            "tables": ("strloc",),
            "cols": (
                ("srl_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Description", "Y")),
            "where": [("srl_cono", "=", self.conum)]}
        self.fld = (
            (("T",0,0,0),"IUA",1,"Location Code","Location Code",
                "","N",self.doLoc,loc,None,("notblank",)),
            (("T",0,1,0),"INA",30,"Description","Description",
                "","N",None,None,self.doDelete,("notblank",)),
            (("T",0,2,0),"INA",30,"Address Line 1","Address Line 1",
                "","N",None,None,None,None),
            (("T",0,3,0),"INA",30,"Address Line 2","Address Line 2",
                "","N",None,None,None,None),
            (("T",0,4,0),"INA",30,"Address Line 3","Address Line 3",
                "","N",None,None,None,None),
            (("T",0,5,0),"INA",30,"Address Line 4","Address Line 4",
                "","N",None,None,None,None))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,2),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",0,0)))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doCloseProcess, )
        abt = None
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)

    def doLoc(self, frt, pag, r, c, p, i, w):
        if w == "0":
            return "Invalid Location"
        self.loc = w
        acc = self.sql.getRec(tables=["strloc"], where=[("srl_cono", "=",
            self.conum), ("srl_loc", "=", self.loc)], limit=1)
        if not acc:
            self.new = "Y"
        else:
            self.new = "N"
            for x in range(1, self.df.topq[pag]):
                self.df.loadEntry(frt, pag, p+x, data=acc[x+1])

    def doDelete(self):
        self.sql.delRec("strloc", where=[("srl_cono", "=", self.conum),
            ("srl_loc", "=", self.loc)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        data = [self.conum]
        for x in range(0, len(self.df.t_work[0][0])):
            data.append(self.df.t_work[0][0][x])
        if self.new == "Y":
            self.sql.insRec("strloc", data=data)
        else:
            self.sql.updRec("strloc", data=data, where=[("srl_cono", "=",
            self.conum), ("srl_loc", "=", self.loc)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doCloseProcess(self):
        self.df.closeProcess()
        self.mf.closeLoop()

