#! /usr/bin/python2

"""
SYNOPSIS
    Converting and Importing Old Tartan Visual Cobol Data Files.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, sys, time
from TartanClasses import ASD, CCD, DBCreate, ProgressBar, SplashScreen, Sql
from TartanClasses import TartanDialog
from tartanFunctions import showError, showException
from tartanWork import mthnam

class ti1010:
    def __init__(self, mf, newver, fledir="", table=None):
        self.mf = mf
        self.fledir = fledir
        self.newver = newver
        if table:
            self.doFleImp(["%s.csv" % table])
            self.doUpdTab()
        elif self.mf.display:
            self.drawDialog()
            self.mf.startLoop()
        else:
            self.doEnd()

    def drawDialog(self):
        tit = (("Import Old VC-Isam Files"),)
        dir = {
            "stype":  "F",
            "types":  "dir",
            "initd":  ""}
        fld = (
            (("T",0,0,0),"IFF",50,"Files Directory","Files Directory",
                self.fledir,"N",self.doDir,dir,None,("dir",)),)
        but = (("Cancel",None,self.doExit,1,None, None),)
        tnd = ((self.doEnd, "y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, screen=self.mf.body,
            tops=True, title=tit, eflds=fld, butt=but, tend=tnd, txit=txt)

    def doDir(self, frt, pag, r, c, p, i, w):
        self.fledir = w

    def doEnd(self):
        if self.mf.display:
            self.df.closeProcess()
        err = self.doCreateDbase()
        if not err:
            self.doFleCvt()
            self.doFleImp()
            self.doCvt2V3()
            self.doUpdTab()
        else:
            print err
        if self.mf.display:
            self.mf.closeLoop()

    def doCreateDbase(self):
        os.rename(os.path.join(self.mf.prgdir, "csv", "crstrn_fld.csv"),
            os.path.join(self.mf.prgdir, "csv", "newcrt_fld.csv"))
        off = open(os.path.join(self.mf.prgdir, "csv", "crstrn_fld.csv"), "w")
        off.write('''0,"crt_cono","UI","3.0","Company Number","Coy"
1,"crt_acno","NA","7.0","Account Number","Acc-Num"
2,"crt_agedt","D1","10.0","Age Date","Age-Date"
3,"crt_type","UI","2.0","Transaction Type","TT"
4,"crt_ref1","Na","9.0","Reference Number","Reference"
5,"crt_batch","Na","7.0","Batch Number","BatchNo"
6,"crt_trdt","D1","10.0","Transaction Date","Trans-Date"
7,"crt_ref2","Na","9.0","Reference 2","Ref-Num-2"
8,"crt_tramt","SD","11.2","Transaction Amount","Trn-Amount"
9,"crt_disamt","SD","11.2","Discount Amount","Dis-Amount"
10,"crt_taxamt","SD","11.2","Tax Amount","Tax-Amount"
11,"crt_trbal","SD","11.2","Transaction Balance","Trans-Bal"
12,"crt_nextm","SD","11.2","Affects Next Month","Next-Month"
13,"crt_disper","UD","5.2","Discount %","Dis-%"
14,"crt_curdt","D2","7.0","Current Date","Curr-Dt"
15,"crt_paydt","d1","10.0","Payment Date","Pay-Date"
16,"crt_payind","UA","1.0","Payment Indicator","P"
17,"crt_payamt","SD","11.2","Payment Amount","Pay-Amount"
18,"crt_desc","NA","30.0","Description","Description"
19,"crt_taxind","UA","1.0","Tax Indicator","V"
20,"crt_batind","UA","1.0","Batch Indicator","B"
21,"crt_hist","UA","1.0","Historical","H"
22,"crt_capdt","d1","10.0","Capture Date","Capture-Dt"
23,"crt_seq","US","10.0","Sequence","Sequence"
''')
        off.close()

        os.rename(os.path.join(self.mf.prgdir, "csv", "crstrn_idx.csv"),
            os.path.join(self.mf.prgdir, "csv", "newcrt_idx.csv"))
        off = open(os.path.join(self.mf.prgdir, "csv", "crstrn_idx.csv"), "w")
        off.write('''"Creditors Transactions",1,"U","crt_seq","","","",""
"",2,"U","crt_cono","crt_acno","crt_type","crt_ref1",""
"",3,"N","crt_cono","crt_acno","crt_type","crt_ref2",""
''')
        off.close()

        os.rename(os.path.join(self.mf.prgdir, "csv", "drstrn_fld.csv"),
            os.path.join(self.mf.prgdir, "csv", "newdrt_fld.csv"))
        off = open(os.path.join(self.mf.prgdir, "csv", "drstrn_fld.csv"), "w")
        off.write('''0,"drt_cono","UI","3.0","Company Number","Coy"
1,"drt_chain","UI","3.0","Chain Number","Chn"
2,"drt_acno","NA","7.0","Account Number","Acc-Num"
3,"drt_agedt","D1","10.0","Age Date","Age-Date"
4,"drt_type","UI","2.0","Transaction Type","TT"
5,"drt_ref1","Na","9.0","Reference 1","Reference"
6,"drt_batch","Na","7.0","Batch Number","BatchNo"
7,"drt_trdt","D1","10.0","Transaction Date","Trans-Date"
8,"drt_ref2","Na","9.0","Reference 2","Ref-Num-2"
9,"drt_tramt","SD","11.2","Transaction Amount","Trn-Amount"
10,"drt_disamt","SD","11.2","Discount Amount","Dis-Amount"
11,"drt_taxamt","SD","11.2","Tax Amount","Tax-Amount"
12,"drt_trbal","SD","11.2","Transaction Balance","Trans-Bal"
13,"drt_nextm","SD","11.2","Affects Next Month","Next-Month"
14,"drt_curdt","D2","7.0","Current Date","Curr-Dt"
15,"drt_desc","NA","30.0","Description","Description"
16,"drt_taxind","UA","1.0","Tax Indicator","V"
17,"drt_batind","UA","1.0","Batch Indicator","B"
18,"drt_hist","UA","1.0","Historical","H"
19,"drt_capdt","d1","10.0","Capture Date","Capture-Dt"
20,"drt_seq","US","10.0","Sequence","Sequence"
''')
        off.close()

        os.rename(os.path.join(self.mf.prgdir, "csv", "drstrn_idx.csv"),
            os.path.join(self.mf.prgdir, "csv", "newdrt_idx.csv"))
        off = open(os.path.join(self.mf.prgdir, "csv", "drstrn_idx.csv"), "w")
        off.write('''"Debtors Transactions",1,"U","drt_seq","","","",""
"",2,"U","drt_cono","drt_chain","drt_acno","drt_type","drt_ref1"
"",3,"N","drt_cono","drt_chain","drt_acno","drt_curdt",""
''')
        off.close()

        opts = [
            ("-c", "i"),
            ("-d", os.path.join(self.mf.prgdir, "csv"))]
        if self.mf.display:
            opts.append(("-l", True))
        else:
            opts.append(("-l", False))
        opts.extend([
            ("-p", self.mf.rcdic["dbpwd"]),
            ("-u", self.mf.rcdic["dbuser"]),
            ("-w", False),
            ("-x", False),
            ("-s", "tartan")])
        create = DBCreate(mf=self.mf, opts=opts)

        os.remove(os.path.join(self.mf.prgdir, "csv", "crstrn_fld.csv"))
        os.rename(os.path.join(self.mf.prgdir, "csv", "newcrt_fld.csv"),
            os.path.join(self.mf.prgdir, "csv", "crstrn_fld.csv"))
        os.remove(os.path.join(self.mf.prgdir, "csv", "crstrn_idx.csv"))
        os.rename(os.path.join(self.mf.prgdir, "csv", "newcrt_idx.csv"),
            os.path.join(self.mf.prgdir, "csv", "crstrn_idx.csv"))
        os.remove(os.path.join(self.mf.prgdir, "csv", "drstrn_fld.csv"))
        os.rename(os.path.join(self.mf.prgdir, "csv", "newdrt_fld.csv"),
            os.path.join(self.mf.prgdir, "csv", "drstrn_fld.csv"))
        os.remove(os.path.join(self.mf.prgdir, "csv", "drstrn_idx.csv"))
        os.rename(os.path.join(self.mf.prgdir, "csv", "newdrt_idx.csv"),
            os.path.join(self.mf.prgdir, "csv", "drstrn_idx.csv"))

        if not create.put:
            return "error"

        t = time.localtime()
        sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        sql = Sql(self.mf.dbm, ["verupd"], prog=__name__)
        sql.insRec("verupd", data=[self.newver, sysdtw])
        self.mf.dbm.commitDbase()
        self.mf.dbm.closeDbase()

    def doFleCvt(self):
        uty = os.path.join(self.mf.prgdir, "uty")
        os.chdir(uty)
        dir = os.path.normpath(self.fledir) + os.sep
        wrk = os.path.normpath(self.mf.wrkdir) + os.sep
        for mod in (
            ("coexp0", "Control"),
            ("glexp0", "General Ledger"),
            ("crexp0", "Creditors Ledger"),
            ("drexp0", "Debtors Ledger"),
            ("stexp0", "Stores Ledger"),
            ("siexp0", "Sales Invoicing")):
            sp = SplashScreen(self.mf.body,
                "Converting %s Files\n\nPlease Wait..." % mod[1])
            prg = os.path.join(uty, mod[0])
            if sys.platform == "win32":
                prg = prg + ".exe"
            ok = os.spawnv(os.P_WAIT, prg, (prg, dir, wrk))
            sp.closeSplash()
            if ok != 0:
                showError(self.mf.body, "Error in Tartan File Conversion",
                    "Program: %s\nFrom   : %s\nTo     : %s\n\n"\
                    "Error Code: %s" % (prg, dir, wrk, ok))
                break

    def doFleImp(self, files=None):
        self.mf.dbm.openDbase()
        if not files:
            files = os.listdir(self.mf.rcdic["wrkdir"])
            files.sort()
        for flenam in files:
            flenam = flenam.lower()
            ext = flenam.split(".")
            if len(ext) != 2 or ext[1] != "csv":
                continue
            styp = flenam[:3]
            if styp in ("ass","ctl","gen","crs","drs","str","sls","wag"):
                try:
                    path = os.path.join(self.mf.rcdic["wrkdir"], flenam)
                    if flenam in ("assgrp.csv", "asstrn.csv"):
                        os.remove(path)
                        continue
                    ip = open(path, "r")
                    idata = ip.readlines()
                    ip.close()
                    if self.mf.display:
                        self.pb = ProgressBar(self.mf.body, max=len(idata), \
                            typ="Importing File %s" % flenam)
                    else:
                        print "Importing File %s" % flenam
                    self.importFile(styp, flenam[:6], idata)
                    self.mf.dbm.commitDbase()
                    os.remove(path)
                except:
                    if self.mf.display:
                        showException(self.mf.body, self.mf.rcdic["wrkdir"], \
                            "Error on Path %s" % path, dbm=self.mf.dbm)
                    else:
                        print "Error on Path %s" % path
                    self.mf.dbm.rollbackDbase()
                if self.mf.display:
                    try:
                        self.pb.closeProgress()
                    except:
                        pass
        self.mf.dbm.closeDbase()

    def importFile(self, styp, flenam, idata):
        t = time.localtime()
        sysdtw = ((t[0] * 10000) + (t[1] * 100) + t[2])
        if flenam == "assmst":
            sql = Sql(self.mf.dbm, [flenam, "assgrp", "asstrn"], prog=__name__)
        elif flenam == "strmf1":
            sql = Sql(self.mf.dbm, [flenam, "strgrp"], prog=__name__)
        elif flenam == "strmf2":
            sql = Sql(self.mf.dbm, [flenam, "strloc"], prog=__name__)
        else:
            sql = Sql(self.mf.dbm, flenam, prog=__name__)
        #############################################
        # Delete Existing Records - Not Necessary ???
        #############################################
        sql.delRec(flenam)
        if flenam == "assmst":
            sql.delRec("assgrp")
            sql.delRec("asstrn")
        if flenam == "strmf1":
            sql.delRec("strgrp")
        if flenam == "strmf2":
            sql.delRec("strloc")
        #############################################
        if flenam == "ctlmes":
            mod = None
            mes = None
        for cnt, dat in enumerate(idata):
            if self.mf.display:
                self.pb.displayProgress(cnt)
            if dat.find("\032") == -1 and len(dat) != 1:
                if dat == "\r\n":
                    break
                elif dat == "\n":
                    break
                elif dat == "\r":
                    break
                else:
                    for x in range(0, 32):
                        dat = dat.replace(chr(x), " ")
                    dat = dat.replace(chr(34), "/")           # "
                    dat = dat.replace(chr(39), "/")           # '
                    dat = dat.replace(chr(44), "/")           # ,
                    dat = dat.replace(chr(92), "/")           # \
                    dat = dat.replace(chr(96), "/")           # `
                    for x in range(127, 155):
                        dat = dat.replace(chr(x), " ")
                    dat = dat.split("|")
                    if flenam == "ctlctl":
                        dat.extend(["", "", ""])              # Bank Details
                    elif flenam == "ctlmes":
                        if mod == dat[0] and mes == dat[1]:
                            data[2] = "%s %s" % (data[2], dat[3])
                            continue
                        elif mod:
                            sql.insRec(flenam, data=data)
                    elif flenam == "ctlmst":
                        dat = dat[:19] + dat[20:] + [dat[19]]
                    elif flenam == "ctlynd":
                        dat.append("N")
                    elif flenam == "crsctl":
                        dat = dat[:2] + ["E", "", 0, 0]
                    elif flenam == "drsctl":
                        dat = dat[:2] + ["E", dat[2], "N", "N", "Y"]
                    elif flenam == "strctl":
                        dat = dat[:3] + [dat[5]]
                    data = []
                    for n, l in enumerate(dat):
                        if flenam == "ctlnot" and n == 3:
                            continue
                        if flenam == "ctlvmf" and n == 7:
                            continue
                        elif flenam == "ctlvtf" and n == 9:
                            data.append("")
                        elif flenam == "ctlmes" and n == 2:
                            continue
                        elif flenam == "genrpt" and n == 2:
                            l = l[1:]
                        elif flenam == "strtrn" and n == 9:
                            continue
                        elif flenam == "strtrn" and n == 23:
                            continue
                        elif flenam == "slsiv2" and n == 12:
                            l = l[2:]
                        try:
                            data.append(l.strip())
                        except:
                            data.append(l)
                    if flenam == "ctlvtf":
                        data.append(0)
                    elif flenam == "ctlmes":
                        mod = data[0]
                        mes = data[1]
                        continue
                    ###################################################
                    # Alterations to file layouts, fields added/deleted
                    ###################################################
                    elif flenam == "genbal":
                        del data[4]
                    elif flenam == "strgrp":
                        del data[3]
                        data.insert(6, "000.0")
                    elif flenam == "strmf1":
                        data.insert(3, "M")
                        del data[6]
                        del data[6]
                        del data[6]
                    elif flenam == "crsmst":
                        newc = data[:9]
                        newc.extend([data[11],data[9]])
                        newc.extend(["","","","",0])
                        newc.extend([data[10],data[14],data[15],data[13]])
                        newc.extend(data[16:])
                        newc.extend(["",0,"",0])
                        data = newc[:]
                    elif flenam == "drsmst":
                        newd = data[:10]
                        newd.extend([data[12],data[10]])
                        newd.extend(["","","",""])
                        newd.extend([data[13],data[14],data[11]])
                        newd.extend(data[15:26])
                        data = newd[29:]
                    ###################################################
                    elif flenam == "genmst":
                        cono = CCD(data[0], "UI", 3)
                        acno = CCD(data[1], "UI", 7)
                        acc = sql.getRec(tables=flenam, where=[("glm_cono", "=",
                            cono.work), ("glm_acno", "=", acno.work)])
                        if acc:
                            continue
                    elif flenam == "ctlnot":
                        data.extend(["Unknown",0])
                    elif flenam == "crstrn":
                        data.append(0)
                    elif flenam == "drstrn":
                        data.append(0)
                    elif flenam == "gentrn":
                        data = data[:12] + [0,data[13],0]
                    elif flenam == "genrpt":
                        if not data[8]:
                            data[8] = "0000000"
                        if not data[9]:
                            data[9] = "0000000"
                    elif flenam == "strgrp":
                        data[7] = "000.0"
                    elif flenam == "strtrn" and data[13] < "200401":
                        continue
                    elif flenam == "strtrn":
                        data = data[:3] + data[5]
                    elif flenam == "assmst":
                        coy = CCD(data[0], "UI", 3).work
                        grp = CCD(data[1], "UA", 3).work
                        cod = CCD(data[2], "Na", 7).work
                        dsc = CCD(data[3], "NA", 30).work
                        pdt = CCD(data[4], "D1", 10).work
                        sdt = CCD(data[5], "D1", 10).work
                        cst = CCD(data[6], "SD", 11.2).work
                        im1 = CCD(data[7], "SD", 11.2).work
                        wo1 = CCD(data[8], "SD", 11.2).work
                        sa1 = CCD(data[9], "SD", 11.2).work
                        cd1 = CCD(data[10], "SD", 11.2).work
                        rd1 = CCD(data[11], "SD", 11.2).work
                        im2 = CCD(data[12], "SD", 11.2).work
                        wo2 = CCD(data[13], "SD", 11.2).work
                        sa2 = CCD(data[14], "SD", 11.2).work
                        cd2 = CCD(data[15], "SD", 11.2).work
                        rd2 = CCD(data[16], "SD", 11.2).work
                        cin = CCD(data[17], "UA", 1).work
                        crt = CCD(data[18], "UD", 6.2).work
                        rin = CCD(data[19], "UA", 1).work
                        rrt = CCD(data[20], "UD", 6.2).work
                        if not grp:
                            continue
                        if not pdt:
                            continue
                        rec = sql.getRec(tables="assgrp", where=[("asg_cono",
                            "=", coy), ("asg_group", "=", grp)], limit=1)
                        if not rec:
                            sql.insRec("assgrp", data=[coy, grp, "",
                            cin, crt, rin, rrt, 0, 0, 0])
                        data = [coy, grp, cod, dsc, cin, crt, rin, rrt]
                        fixd = [coy, grp, cod, 3, "Take/On", "Take/On"]
                        tran = fixd[:] + [pdt, 1, cst, cst, 0.0, pdt/100,
                            "Original Cost", "", "", sysdtw, 0]
                        sql.insRec("asstrn", data=tran)
                        if sdt:
                            sal = float(ASD(sa1) + ASD(sa2))
                            tran = fixd[:] + [sdt, 5, sal, sal, 0.0, sdt/100,
                                "Selling Price", "", "", sysdtw, 0]
                            sql.insRec("asstrn", data=tran)
                        if im1:
                            tran = fixd[:] + [20060228, 2, im1, im1, 0.0,
                                200602, "Improvements", "", "", sysdtw, 0]
                            sql.insRec("asstrn", data=tran)
                        if wo1:
                            tran = fixd[:] + [20060228, 3, wo1, wo1, 0.0,
                                200602, "Written Off", "", "", sysdtw, 0]
                            sql.insRec("asstrn", data=tran)
                        if cd1 or rd1:
                            if sdt and sdt <= 20060228:
                                ddt = sdt
                            else:
                                ddt = 20060228
                            tran = fixd[:] + [ddt, 4, cd1, rd1, 0.0,
                                ddt/100, "Depreciation", "", "", sysdtw, 0]
                            sql.insRec("asstrn", data=tran)
                        if im2:
                            tran = fixd[:] + [sysdtw, 2, im2, im2, 0.0,
                                sysdtw/100, "Improvements", "", "", sysdtw, 0]
                            sql.insRec("asstrn", data=tran)
                        if wo2:
                            tran = fixd[:] + [sysdtw, 3, wo2, wo2, 0.0,
                                sysdtw/100, "Written Off", "", "", sysdtw, 0]
                            sql.insRec("asstrn", data=tran)
                        if cd2 or rd2:
                            if sdt and sdt > 20060228:
                                ddt = sdt
                            else:
                                ddt = sysdtw
                            tran = fixd[:] + [ddt, 4, cd2, rd2, 0.0,
                                ddt/100, "Depreciation", "", "", sysdtw, 0]
                            sql.insRec("asstrn", data=tran)
                if flenam == "wagtf2":
                    old = sql.getRec(tables=flenam, cols=["wt2_hrs",
                        "wt2_eamt", "wt2_ramt"], where=[("wt2_cono", "=",
                        data[0]), ("wt2_empno", "=", data[1]), ("wt2_page",
                        "=", data[2]), ("wt2_date", "=", data[3]), ("wt2_type",
                        "=", data[4]), ("wt2_code", "=", data[5])], limit=1)
                    if old:
                        hrs = float(ASD(old[0]) + ASD(data[6]))
                        emt = float(ASD(old[1]) + ASD(data[7]))
                        rmt = float(ASD(old[2]) + ASD(data[8]))
                        sql.updRec(flenam, cols=["wt2_hrs", "wt2_eamt",
                            "wt2_ramt"], data=[hrs, emt, rmt],
                            where=[("wt2_cono", "=", data[0]), ("wt2_empno",
                            "=", data[1]), ("wt2_page", "=", data[2]),
                            ("wt2_date", "=", data[3]), ("wt2_type", "=",
                            data[4]), ("wt2_code", "=", data[5])])
                        continue
                #==============================================================
                # Work around for unique keys (crstrn and drstrn)
                #==============================================================
                if flenam == "crstrn":
                    unique = "crt_ref1"
                elif flenam == "drstrn":
                    unique = "drt_ref1"
                else:
                    unique = None
                sql.insRec(flenam, data=data, unique=unique, format="y")
                #==============================================================
        if flenam == "ctlmes" and mod:
            sql.insRec(flenam, data=data)
        #======================================================================
        # With creditors and debtors transactions create balancing records
        #======================================================================
        if flenam == "crstrn":
            self.doCrsTrnBal(sql)
        if flenam == "drstrn":
            self.doDrsTrnBal(sql)
        #======================================================================
        # With stores masterfile records, auto populate groups and locations
        #======================================================================
        if flenam == "strmf1":
            rec = sql.getRec(tables=flenam, cols=["st1_cono",
                "st1_group"], group="st1_cono, st1_group")
            if rec:
                for grp in rec:
                    sql.insRec("strgrp", data=[grp[0], grp[1], "", "", "",
                        0, 0, 0])
        if flenam == "strmf2":
            rec = sql.getRec(tables=flenam, cols=["st2_cono",
                "st2_loc"], group="st2_cono, st2_loc")
            if not rec:
                for loc in rec:
                    sql.insRec("strloc", data=[loc[0], loc[1], ""])
        #======================================================================

    def doCrsTrnBal(self, sql):
        bals = sql.getRec(tables="crstrn", cols=["crt_cono", "crt_acno",
            "round(sum(crt_tramt), 2)", "round(sum(crt_disamt), 2)",
            "round(sum(crt_trbal), 2)"], group="crt_cono, crt_acno")
        if not bals:
            return
        for bal in bals:
            net = float(ASD(bal[2]) + ASD(bal[3]))
            if bal[4] == net:
                continue
            amnt = float(ASD(bal[4]) - ASD(net))
            date = sql.getRec(tables="crstrn", cols=["min(crt_trdt)"],
                where=[("crt_cono", "=", bal[0]), ("crt_acno", "=", bal[1])],
                limit=1)
            yr = date[0] / 10000
            mt = date[0] % 10000 / 100 - 1
            if mt == 0:
                yr = yr - 1
                mt = 12
            dy = mthnam[mt][2]
            trdt = (yr * 10000) + (mt * 100) + dy
            curdt = trdt / 100
            sql.insRec("crstrn", data=[bal[0], bal[1], trdt, 3, "Imp/Bal",
                "Imp/Bal", trdt, "", amnt, 0, 0, 0, 0, 0, curdt, 0, "N", 0,
                "Import Balance", " ", "", "Y", trdt, 0])

    def doDrsTrnBal(self, sql):
        bals = sql.getRec(tables="drstrn", cols=["drt_cono", "drt_chain",
            "drt_acno", "round(sum(drt_tramt),2)", "round(sum(drt_disamt),2)",
            "round(sum(drt_trbal),2)"], group="drt_cono, drt_chain, drt_acno")
        if not bals:
            return
        for bal in bals:
            net = float(ASD(bal[3]) + ASD(bal[4]))
            if bal[5] == net:
                continue
            amnt = float(ASD(bal[5]) - ASD(net))
            date = sql.getRec(tables="drstrn", cols=["min(drt_trdt)"],
                where=[("drt_cono", "=", bal[0]), ("drt_chain", "=", bal[1]),
                ("drt_acno", "=", bal[2])], limit=1)
            yr = date[0] / 10000
            mt = date[0] % 10000 / 100 - 1
            if mt == 0:
                yr = yr - 1
                mt = 12
            dy = mthnam[mt][2]
            trdt = (yr * 10000) + (mt * 100) + dy
            curdt = trdt / 100
            sql.insRec("drstrn", data=[bal[0], bal[1], bal[2], trdt, 3,
                "Imp/Bal", "Imp/Bal", trdt, "", amnt, 0, 0, 0, 0, curdt,
                "Import Balance", "", "", "Y", trdt, 0])

    def doCvt2V3(self):
        if self.mf.display:
            sp = SplashScreen(self.mf.body,
            "Converting Files to Version 3\n\nPlease Wait...")
        else:
            print "Converting Files to Version 3, Please Wait..."
        for table in ("crsage", "drsage"):
            opts = [
                ("-u", self.mf.dbuser),
                ("-c", "t"),
                ("-d", os.path.join(self.mf.prgdir, "csv")),
                ("-i", False)]
            if self.mf.display:
                opts.append(("-l", True))
            else:
                opts.append(("-l", False))
            opts.extend([
                ("-t", table),
                ("-w", False),
                ("-x", False)])
            if self.mf.dbpwd:
                opts.append(("-p", self.mf.dbpwd))
            DBCreate(mf=self.mf, opts=opts)
        sql = Sql(self.mf.dbm, ["drsctl", "drstrn", "drsage", "crsctl",
            "crstrn", "crsage"], prog=__name__)
        # Discount - Debtors
        typ = "Debtors Discount"
        all = sql.getRec(tables="drstrn", where=[("drt_disamt", "<>", 0)])
        if not self.mf.display:
            print typ
        else:
            pb = ProgressBar(self.mf.body, max=len(all), typ=typ)
        for num, rec in enumerate(all):
            if self.mf.display:
                pb.displayProgress(num)
            rec[4] = 7
            rec[9] = rec[10]
            rec[10] = 0
            rec[11] = 0
            rec[12] = 0
            rec[13] = 0
            rec[20] = 0
            sql.insRec("drstrn", data=rec)
        if self.mf.display:
            pb.closeProgress()
        # Balancing Amount - Debtors
        typ = "Debtors Balancing"
        sql.delRec("crstrn", where=[("crt_batch", "=", "Imp/Bat"),
            ("crt_ref1", "=", "Imp/Amt")])
        all = sql.getRec(tables="drstrn", cols=["drt_cono", "drt_chain",
            "drt_acno", "round(sum(drt_tramt),2)", "round(sum(drt_trbal),2)"],
            group="drt_cono, drt_chain, drt_acno")
        if not self.mf.display:
            print typ
        else:
            pb = ProgressBar(self.mf.body, max=len(all), typ=typ)
        for num, rec in enumerate(all):
            cono, chain, acno, tramt, trbal = rec
            if self.mf.display:
                pb.displayProgress(num)
            diff = float(ASD(tramt) - ASD(trbal))
            if diff:
                dte = sql.getRec(tables="drstrn", cols=["min(drt_trdt)"],
                    where=[("drt_cono", "=", cono), ("drt_chain", "=",
                    chain), ("drt_acno", "=", acno)], limit=1)[0]
                diff = float(ASD(0) - ASD(diff))
                if diff < 0:
                    trn = 3
                else:
                    trn = 6
                sql.insRec("drstrn", data=[cono, chain, acno, dte, trn,
                    "Bal/Ref", "Bal/Bat", dte, "", diff, 0, 0, 0, 0,
                    dte/100, "Balancing Amount", "N", "", "Y",
                    self.sysdtw, 0])
        if self.mf.display:
            pb.closeProgress()
        # Ageing - Debtors
        typ = "Debtors Ageing"
        all = sql.getRec(tables="drstrn",
            where=[("drt_trbal <> drt_tramt",)])
        if not self.mf.display:
            print typ
        else:
            pb = ProgressBar(self.mf.body, max=len(all), typ=typ)
        for num, rec in enumerate(all):
            if self.mf.display:
                pb.displayProgress(num)
            data = rec[:3]
            data.extend(rec[4:6])
            data.extend([rec[14], 2, "Cvt/Ref"])
            data.append(float(ASD(rec[9]) - ASD(rec[12])))
            data.append(0)
            sql.insRec("drsage", data=data)
        if self.mf.display:
            pb.closeProgress()
        # Discount - Creditors
        typ = "Creditors Discount"
        all = sql.getRec(tables="crstrn", where=[("crt_disamt", "<>", 0)])
        if not self.mf.display:
            print typ
        else:
            pb = ProgressBar(self.mf.body, max=len(all), typ=typ)
        for num, rec in enumerate(all):
            if self.mf.display:
                pb.displayProgress(num)
            rec[3] = 6
            rec[8] = rec[9]
            rec[9] = 0
            rec[10] = 0
            rec[11] = 0
            rec[12] = 0
            rec[13] = 0
            rec[15] = 0
            rec[17] = 0
            rec[23] = 0
            sql.insRec("crstrn", data=rec)
        if self.mf.display:
            pb.closeProgress()
        # Balancing Amount - Creditors
        typ = "Creditors Balancing"
        sql.delRec("crstrn", where=[("crt_batch", "=", "Imp/Bat"),
            ("crt_ref1", "=", "Imp/Amt")])
        all = sql.getRec(tables="crstrn", cols=["crt_cono", "crt_acno",
            "round(sum(crt_tramt),2)", "round(sum(crt_trbal),2)"],
            group="crt_cono, crt_acno")
        if not self.mf.display:
            print typ
        else:
            pb = ProgressBar(self.mf.body, max=len(all), typ=typ)
        for num, rec in enumerate(all):
            cono, acno, tramt, trbal  = rec
            if self.mf.display:
                pb.displayProgress(num)
            diff = float(ASD(tramt) - ASD(trbal))
            if diff:
                dte = sql.getRec(tables="crstrn", cols=["min(crt_trdt)"],
                    where=[("crt_cono", "=", cono), ("crt_acno", "=", acno)],
                    limit=1)[0]
                diff = float(ASD(0) - ASD(diff))
                sql.insRec("crstrn", data=[cono, acno, dte, 3, "Bal/Ref",
                    "Bal/Bat", dte, "", diff, 0, 0, 0, 0, 0, dte/100, 0,
                    "", 0, "Balancing Amount", "N", "", "Y", self.sysdtw, 0])
        if self.mf.display:
            pb.closeProgress()
        # Ageing - Creditors
        typ = "Creditors Ageing"
        all = sql.getRec(tables="crstrn", where=[("crt_trbal <> crt_tramt",)])
        if not self.mf.display:
            print typ
        else:
            pb = ProgressBar(self.mf.body, max=len(all), typ=typ)
        for num, rec in enumerate(all):
            if self.mf.display:
                pb.displayProgress(num)
            data = rec[:2]
            data.extend(rec[3:5])
            cd = sql.getRec(tables="crstrn", cols=["crt_curdt"],
                where=[("crt_cono", "=", rec[0]), ("crt_acno", "=", rec[1]),
                ("crt_type", "=", 5), ("crt_ref1", "=", rec[7])], limit=1)
            if cd:
                data.append(cd[0])
            else:
                data.append(rec[14])
            data.extend([5, "Cvt/Ref"])
            data.append(float(ASD(rec[8]) - ASD(rec[11])))
            data.append(0)
            sql.insRec("crsage", data=data)
        if self.mf.display:
            pb.closeProgress()
            sp.closeSplash()
            self.mf.updateStatus("")
        self.mf.dbm.commitDbase()
        self.mf.dbm.closeDbase()

    def doUpdTab(self):
        import tb1020
        mfm = ("-m", self.mf)
        dir = ("-d", os.path.join(self.mf.prgdir, "csv"))
        usr = ("-u", self.mf.dbuser)
        pwd = ("-p", self.mf.dbpwd)
        sys = ("-s", "tartan")
        tab = ("-t", "ctlvtf, crstrn, drstrn")
        ver = ("-v", self.newver)
        fmt = ("-f", "y")
        self.mf.dbm.openDbase()
        upd = tb1020.tb1020(opts=[mfm, dir, usr, pwd, sys, tab, ver, fmt])
        self.mf.dbm.closeDbase()

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    import getopt
    from TartanClasses import MainFrame
    from tartanFunctions import loadRcFile
    opts, args = getopt.getopt(sys.argv[1:], "d:i:t:v:")
    dir = ""
    gui = True
    tab = None
    ver = "3.2.5"
    for o, v in opts:
        if o == "-d":
            dir = v
        elif o == "-i":
            if v.lower() == "n":
                gui = False
        elif o == "-t":
            tab = v.lower()
        elif o == "-v":
            ver = v
    # Load the rcfile if it exists else run TartanConfig to create one
    rcfile = None
    rcdic = None
    while not rcdic:
        rcdic = loadRcFile()
        if not rcdic:
            print "No tartanrc File and display=False, Aborting"
            sys.exit()
            cfg = TartanConfig(rcfile=rcfile)
            if not cfg.rcfile:
                print "TartanConfig Error, Aborting"
                sys.exit()
            rcfile = cfg.rcfile
    mf = MainFrame(rcdic=rcdic, display=gui)
    ti1010(mf, ver, dir, tab)
