#! /usr/bin/python

"""
SYNOPSIS
    Salaries - Data Capture.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import GetCtl, Sql, SChoice, TartanDialog
from tartanFunctions import callModule

class wg2010:
    def __init__(self, mf, conum, conam, usr):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        self.capnm = usr
        no = self.setVariables()
        if not no:
            self.dataHeader()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["wagedc", "wagmst", "wagcap"],
            prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        gc = GetCtl(self.mf)
        wagctl = gc.getCtl("wagctl", self.conum)
        if not wagctl:
            return "no"
        self.m_hrs = wagctl["ctw_m_hrs"]
        self.w_hrs = wagctl["ctw_w_hrs"]
        self.d_hrs = wagctl["ctw_d_hrs"]
        self.edit = None

    def dataHeader(self):
        wgm = {
            "stype": "R",
            "tables": ("wagmst",),
            "cols": (
                ("wgm_empno", "", 0, "Emp-Num"),
                ("wgm_sname", "", 0, "Surname", "Y"),
                ("wgm_fname", "", 0, "First Names")),
            "where": [("wgm_cono", "=", self.conum), ("wgm_term", "=", 0)]}
        wed = {
            "stype": "R",
            "tables": ("wagedc",),
            "cols": (
                ("ced_code", "", 0, "Code"),
                ("ced_desc", "", 0, "Description", "Y")),
            "where": [("ced_cono", "=", self.conum)],
            "whera": [["C", "ced_type", 5, 0]]}
        tit = ("%-3d %s" % (self.conum, self.conam),
            "Wages and Salaries Data Capture (%s)" % __name__)
        fld = (
            (("C",0,0,0),"IUI",5,"EmpNo","Employee Number",
                "r","N",self.doEmpno,wgm,None,("efld",)),
            (("C",0,0,1),"ONA",20,"Surname"),
            (("C",0,0,2),"ONA",3,"Bra"),
            (("C",0,0,3),"OUI",1,"D"),
            (("C",0,0,4),"IUI",5,"JobNo","Job Number",
                "r","N",self.doJobno,None,None,("efld",)),
            (("C",0,0,5),"IUA",1,"T","Earnings/Deduction/NoPay",
                "r","N",self.doType,None,None,("in", ("E","D"))),
            (("C",0,0,6),"IUI",3,"Cde","Code Number",
                "","N",self.doCode,wed,None,("efld",)),
            (("C",0,0,7),"ONA",20,"Description"),
            (("C",0,0,8),"IUA",1,"P","Pay (Y or N)",
                "Y","N",self.doPay,None,None,("in", ("Y","N"))),
            [("C",0,0,9),"ISD",9.2,"Amount","Amount",
                "","N",self.doAmt,None,None,None])
        but = (
            ("Edit",None,self.editData,0,("C",0,1),("C",0,2)),
            ("Interrogate",None,self.queryWag,0,("C",0,1),("C",0,2)))
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, butt=but,
            cend=((self.endData,"y"),), cxit=(self.exitData,))

    def doEmpno(self, frt, pag, r, c, p, i, w):
        self.empno = w
        rec = self.readWagmst()
        if not rec:
            return "Invalid Employee Number"
        if rec[4]:
            return "Employment Terminated"
        self.df.loadEntry(frt, pag, p+1, rec[0])
        self.df.loadEntry(frt, pag, p+2, rec[1])
        self.df.loadEntry(frt, pag, p+3, rec[2])
        self.freq = rec[3]

    def doJobno(self, frt, pag, r, c, p, i, w): pass

    def doType(self, frt, pag, r, c, p, i, w):
        self.rtyp = w

    def doCode(self, frt, pag, r, c, p, i, w):
        self.code = w
        rec = self.readWagedc()
        if not rec:
            return "Invalid Code"
        self.df.loadEntry(frt, pag, p+1, rec[0])
        if self.rtyp == "E" and self.code == 1:
            if self.freq == "M":
                hrs = self.m_hrs
            elif self.freq == "W":
                hrs = self.w_hrs
            elif self.freq == "D":
                hrs = self.d_hrs
            else:
                hrs = self.m_hrs
            self.df.colf[0][9][5] = hrs
        else:
            self.df.colf[0][9][5] = ""

    def doPay(self, frt, pag, r, c, p, i, w):
        pass

    def doAmt(self, frt, pag, r, c, p, i, w):
        self.xrow = r

    def readWagmst(self):
        rec = self.sql.getRec(tables="wagmst", cols=["wgm_sname", "wgm_branch",
            "wgm_dept", "wgm_freq", "wgm_term"], where=[("wgm_cono", "=",
            self.conum), ("wgm_empno", "=", self.empno)], limit=1)
        if not rec:
            return None
        else:
            return rec

    def readWagedc(self):
        rec = self.sql.getRec(tables="wagedc", cols=["ced_desc"],
            where=[("ced_cono", "=", self.conum), ("ced_type", "=", self.rtyp),
            ("ced_code", "=", self.code)], limit=1)
        if not rec:
            return None
        else:
            return rec

    def endData(self):
        r = self.xrow
        data = [self.conum, self.empno, self.df.c_work[0][r][2],
            self.df.c_work[0][r][3], self.df.c_work[0][r][4],
            self.df.c_work[0][r][5], self.df.c_work[0][r][6],
            self.df.c_work[0][r][8], self.df.c_work[0][r][9], "N",
            self.sysdtw]
        if self.edit:
            data.append(self.edit)
            self.sql.updRec("wagcap", data=data, where=[("wcp_seq",
                "=",self.edit)])
            self.edit = None
        else:
            data.append(0)
            self.sql.insRec("wagcap", data=data)
        self.df.advanceLine(0)

    def exitData(self):
        self.mf.dbm.commitDbase()
        self.df.closeProcess()
        self.mf.closeLoop()

    def editData(self, widget=None):
        # Display captured items and allow editing
        col = ["wcp_empno", "wgm_sname", "wcp_branch", "wcp_dept", "wcp_job",
            "wcp_type", "wcp_code", "ced_desc", "wcp_ind", "wcp_amt", "wcp_seq"]
        whr = [("wcp_cono", "=", self.conum), ("wcp_paid", "=", "N"),
            ("wgm_cono=wcp_cono",), ("wgm_empno=wcp_empno",),
            ("ced_cono=wcp_cono",), ("ced_type=wcp_type",),
            ("ced_code=wcp_code",)]
        data = self.sql.getRec(tables=["wagcap", "wagmst", "wagedc"],
            cols=col, where=whr, order="wcp_empno")
        if not data:
            self.df.focusField(self.df.frt, self.df.pag, self.df.col)
            return
        titl = "Captured Items"
        head = ("EmpNo", "Surname", "Bch", "D", "JobNo", "T", "Cod",
            "Description", "I", "Amount","Sequence")
        lin = {
            "stype": "C",
            "titl": titl,
            "head": head,
            "typs": [
                ("UI", 5),
                ("NA", 30),
                ("NA", 3),
                ("UI", 1),
                ("UI", 5),
                ("UA", 1),
                ("UI", 3),
                ("NA", 20),
                ("UA", 1),
                ("SD", 9.2),
                ("UI", 10)],
            "data": data,
            "butt": [("Delete", self.doDelete, True)]}
        state = self.df.disableButtons()
        self.mf.updateStatus("Select an Item to Edit")
        chg = SChoice(self.mf, deco=True, titl=lin["titl"], head=lin["head"],
            data=lin["data"], typs=lin["typs"], mode="S", retn="D",
            butt=lin["butt"], scrn=self.df.mstFrame)
        self.df.enableButtons(state=state)
        if chg and chg.selection:
            for num, fld in enumerate(chg.selection):
                if num in (0, 4, 5, 6):
                    self.df.doKeyPressed("C", 0, self.df.pos, fld)
                elif num == (len(chg.selection) - 1):
                    self.edit = int(fld)
                else:
                    self.df.loadEntry(self.df.frt, self.df.pag, num, data=fld)
        else:
            self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doDelete(self, args):
        self.sql.delRec("wagcap", where=[("wcp_seq", "=", args[-1:][0])])

    def queryWag(self, widget=None):
        callModule(self.mf, self.df, "wg4010", coy=(self.conum, self.conam),
            period=None, user=self.capnm)
