#! /usr/bin/env python

"""
SYNOPSIS
    Salaries and Wages Data Capture Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError

class wg3020:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.doVariables()
        self.doProcess()
        self.mf.startLoop()

    def doVariables(self):
        self.sql = Sql(self.mf.dbm, ["wagcap", "wagmst", "wagedc"],
            prog=__name__)
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None

    def doProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Salaries Data Capture Listing (%s)" % __name__)
        fld = (
            (("T",0,0,0),"ID1",10,"Reporting Date","Reporting Date",
                self.sysdtw,"Y",self.doDate,None,None,("efld",)),)
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doDate(self, frt, pag, r, c, p, i, w):
        self.date = self.df.t_disp[0][0][0]

    def doEnd(self):
        self.df.closeProcess()
        all = self.sql.getRec(tables=["wagcap", "wagmst", "wagedc"],
            cols=["wcp_empno", "wcp_branch", "wcp_dept", "wcp_job",
            "wcp_type", "wcp_code", "wcp_ind", "wcp_amt", "wgm_sname",
            "ced_desc"], where=[("wcp_cono", "=", self.conum),
            ("wcp_cono=wgm_cono",), ("wcp_empno=wgm_empno",),
            ("ced_cono=wcp_cono",), ("ced_type=wcp_type",),
            ("ced_code=wcp_code",), ("wcp_paid", "=", "N")],
            order="wcp_empno, wcp_type desc, wcp_code")
        if not all:
            showError(self.mf.body, "Transaction Error",
            "No Transactions Available")
        else:
            self.doPrintReport(all)
        self.closeProcess()

    def doPrintReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %10s %33s %10s %10s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        old_empno = 0
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            emp = CCD(dat[0], "UI", 5)
            nam = CCD(dat[8], "NA", 30)
            brn = CCD(dat[1], "NA", 3)
            dep = CCD(dat[2], "UI", 1)
            job = CCD(dat[3], "UI", 5)
            typ = CCD(dat[4], "UA", 1)
            cod = CCD(dat[5], "UI", 3)
            des = CCD(dat[9], "NA", 30)
            ind = CCD(dat[6], "UA", 1)
            amt = CCD(dat[7], "SD", 14.2)
            if old_empno and old_empno != emp.work:
                self.t.textLine()
                self.pglin = self.pglin + 1
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.doPageHeading()
            self.t.textLine("%s %s %s %s %s %s %s %s %s %s" % \
                (emp.disp, nam.disp, brn.disp, dep.disp, job.disp,
                typ.disp, cod.disp, des.disp, ind.disp, amt.disp))
            old_empno = emp.work
            self.pglin = self.pglin + 1
        p.closeProgress()
        if self.t:
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
            
    def doPageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-35s %-10s %48s %5s" % \
            ("Salaries Data Capture Report as at",
            self.date, "Page", self.pgnum))
        self.t.textLine()
        self.t.textLine("%-5s %-30s %-3s %-1s %-5s %-1s %-3s %-30s %-1s " \
            "%-11s" % ("EmpNo", "Surname", "Brn", "D", "JobNo",
                "T", "Cod", "Description", "I", "       Amount"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 6

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

