#! /usr/bin/python2

"""
SYNOPSIS
    Salaries and Wages Earnings and Deduction Analysis.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import ASD, RepPrt, Sql, TartanDialog
from tartanFunctions import showError

class wg3040:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["wagmst", "wagedc", "wagtf2"],
            prog=__name__)

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Salaries and Wages Earnings and Deduction Analysis")
        dte = {
            "stype": "R",
            "tables": ("wagtf1",),
            "cols": (("wt1_date", "", 0, "Date"),),
            "where": [("wt1_cono", "=", self.conum)],
            "group": "wt1_date",
            "order": "wt1_date"}
        r1s = (("Earnings","E"),("Deductions","D"),("Both","B"))
        r2s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),"ID1",10,"Start Date","Start Date",
                "","N",self.doSDate,dte,None,("efld",)),
            (("T",0,1,0),"ID1",10,"End   Date","End   Date",
                "","N",self.doEDate,dte,None,("efld",)),
            (("T",0,2,0),("IRB",r1s),10,"Type","Type",
                "B","N",self.doType,None,None,None),
            (("T",0,3,0),("IRB",r2s),10,"Code per Page","New Page per Code",
                "N","N",self.doNewPg,None,None,None))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doSDate(self, frt, pag, r, c, p, i, w):
        self.sdatew = w
        self.sdated = self.df.t_disp[0][0][0]
        self.df.topf[0][1][5] = self.sdatew

    def doEDate(self, frt, pag, r, c, p, i, w):
        if w < self.sdatew:
            return "Invalid End Date"
        self.edatew = w
        self.edated = self.df.t_disp[0][0][1]

    def doType(self, frt, pag, r, c, p, i, w):
        self.rtyp = w

    def doNewPg(self, frt, pag, r, c, p, i, w):
        self.newpg = w

    def doEnd(self):
        self.df.closeProcess()
        col = ["wt2_type", "wt2_code", "ced_desc", "wt2_empno", "wgm_sname",
            "wgm_fname", "round(sum(wt2_eamt),2)", "round(sum(wt2_ramt),2)"]
        whr = [("wt2_cono", "=", self.conum)]
        if self.sdatew == self.edatew:
            whr.append(("wt2_date", "=", self.edatew))
        else:
            whr.append(("wt2_date", "between", self.sdatew, self.edatew))
        if self.rtyp != "B":
            whr.append(("wt2_type", "=", self.rtyp))
        whr.extend([("wgm_cono=wt2_cono",), ("ced_cono=wt2_cono",),
            ("ced_type=wt2_type",), ("ced_code=wt2_code",),
            ("wgm_empno=wt2_empno",)])
        grp = "wt2_type, wt2_code, ced_desc, wt2_empno, wgm_sname, wgm_fname"
        if self.sdatew == self.edatew:
            grp = "%s, wt2_date" % grp
        ord = "wt2_type desc, wt2_code asc, wt2_empno"
        all = self.sql.getRec(tables=["wagmst", "wagedc", "wagtf2"], cols=col,
            where=whr, group=grp, order=ord)
        if not all:
            showError(self.mf.body, "Selection Error",
            "No Transactions for the Period")
        else:
            self.doPrintReport(all)
        self.closeProcess()

    def doPrintReport(self, all):
        name = __name__
        tables = []
        for col in all:
            if col[0] == "D":
                col[6] = float(ASD(0) - ASD(col[6]))
                col[7] = float(ASD(0) - ASD(col[7]))
            tables.append(col)
        if self.rtyp == "D":
            heads = ["Salaries and Wages Deductions Report"]
        elif self.rtyp == "E":
            heads = ["Salaries and Wages Earnings Report"]
        else:
            heads = ["Salaries and Wages Earnings and Deductions Report"]
        opts = "From %s to %s" % (self.sdated, self.edated)
        cols = [
            ["wt2_type", "UA", 1, "T", "N"],
            ["wt2_code", "UI", 3, "Cod", "N"],
            ["ced_desc", "NA", 30, "Description", "N"],
            ["wt2_empno", "UI", 5, "EmpNo", "Y"],
            ["wgm_sname", "NA", 30, "Surname", "Y"],
            ["wgm_fname", "NA", 30, "First Names", "Y"]]
        cols.extend([
            ["wt2_eamt", "SD", 13.2, "Employee-Amt", "Y"],
            ["wt2_ramt", "SD", 13.2, "Employer-Amt", "Y"]])
        stots = [["ced_desc", "Total", "%s" % self.newpg]]
        gtots = ["wt2_eamt", "wt2_ramt"]
        view = self.df.repprt[0][1]
        prtnam = self.df.repprt[0][2]
        mail = self.df.repprt[1][2]
        state = self.df.disableButtons()
        rpt = RepPrt(self.mf, name=name, tables=tables, heads=heads,
            opts=opts, cols=cols, stots=stots, gtots=gtots, conum=self.conum,
            conam=self.conam, ttype="D", view=view, prtnam=prtnam, mail=mail)
        self.df.enableButtons(state=state)

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

