#! /usr/bin/env python

"""
SYNOPSIS
    Salaries and Wages IRP5 Statements Printing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time, os
from TartanClasses import ASD, CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import dateDiff, doPrinter, doReportlab
from tartanFunctions import getModName, getSingleRecords, sendMail, showError
from tartanWork import payeRates, siteRates

class wg3050:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlsys", "ctlmst", "emllog", "wagctl",
            "wagedc", "wagmst", "wagirp", "wagtf2", "wagrcv"], prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        cts = self.sql.getRec(tables="ctlsys", limit=1)
        ctc = self.sql.getRec(tables="ctlmst", where=[("ctm_cono", "=",
            self.conum)], limit=1)
        ctw = self.sql.getRec(tables="wagctl", where=[("ctw_cono", "=",
            self.conum)], limit=1)
        if not ctc or not ctw:
            return "no"
        self.smtp = cts[self.sql.ctlsys_col.index("sys_smtp")].strip()
        self.coad1 = CCD(ctc[self.sql.ctlmst_col.index("ctm_add1")], "NA", 30)
        self.coad2 = CCD(ctc[self.sql.ctlmst_col.index("ctm_add2")], "NA", 30)
        self.coad3 = CCD(ctc[self.sql.ctlmst_col.index("ctm_add3")], "NA", 30)
        self.copcd = CCD(ctc[self.sql.ctlmst_col.index("ctm_pcode")], "NA", 4)
        self.cname = CCD(ctc[self.sql.ctlmst_col.index("ctm_contact")],"NA", 30)
        self.cotel = CCD(ctc[self.sql.ctlmst_col.index("ctm_tel")], "NA", 15)
        self.efrom = ctc[self.sql.ctlmst_col.index("ctm_email")].strip()
        self.coreg = CCD(ctw[self.sql.wagctl_col.index("ctw_regno")], "NA", 10)
        self.codip = CCD(ctw[self.sql.wagctl_col.index("ctw_irp_dip")], "NA", 1)
        self.totrecs = 0
        self.totcode = 0
        self.totamnt = 0
        self.totalls = 0
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Salaries IRP5 Statements (%s)" % __name__)
        wgm = {
            "stype": "R",
            "tables": ("wagmst",),
            "cols": (
                ("wgm_empno", "", 0, "EmpNo"),
                ("wgm_sname", "", 0, "Surname"),
                ("wgm_fname", "", 0, "Names"),
                ("wgm_start", "", 0, "Start-Date"),
                ("wgm_term", "", 0, "Term-Date"))}
        r1s = (("Live", "L"), ("Test", "T"))
        r2s = (("Yes", "Y"), ("No", "N"))
        r3s = (("Yes","Y"),("Range","R"),("Singles", "S"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Submission Type","Submission Type",
                "T","Y",self.doType,None,None,None),
            (("T",0,1,0),"IUI",4,"Tax Year","Tax Year",
                0,"Y",self.doYear,None,None,("notzero",)),
            (("T",0,2,0),"ID1",10,"Cut Off Date","Cut Off Date",
                0,"N",self.doDate,None,None,("efld",)),
            (("T",0,3,0),("IRB",r2s),0,"Reprint","Reprint Statements",
                "Y","Y",self.doReprint,None,None,None),
            (("T",0,4,0),("IRB",r2s),0,"Preview","Preview Only",
                "Y","Y",self.doPreview,None,None,None),
            (("T",0,5,0),("IRB",r3s),0,"Whole File","Whole File",
                "S","Y",self.doWhole,None,None,None),
            (("T",0,6,0),("IRB",r2s),10,"Include Other Companies","Include "\
                "Other Companies","N","N",self.doCoy,None,None,None),
            (("T",0,7,0),"IUI",5,"From Employee","From Employee",
                "","Y",self.doEmp,wgm,None,None),
            (("T",0,8,0),"IUI",5,"To   Employee","To Employee",
                "","Y",self.doEmp,wgm,None,None),
            (("T",0,9,0),("IRB",r2s),0,"E-Mail R.O.R.","E-Mail Receiver of "\
                "Revenue","N","Y",self.doEml,None,None,None,None,"""Selecting this option will result in the electronic form being submitted i.e. emailed to paye.cc@sars.gov.za"""))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","P"), mail=("B","N"))

    def doType(self, frt, pag, r, c, p, i, w):
        if w == "L":
            self.rtype = "LIVE"
        else:
            self.rtype = "TEST"

    def doYear(self, frt, pag, r, c, p, i, w):
        self.taxyr = w
        self.sdate =  CCD(((self.taxyr - 1) * 10000) + 301, "D1", 10)
        if self.taxyr % 4:
            self.edate = (self.taxyr * 10000) + 200 + 28
        else:
            self.edate = (self.taxyr * 10000) + 200 + 29
        if self.sysdtw < self.edate:
            self.edate = self.sysdtw
        self.edate = CCD(self.edate, "D1", 10)
        self.df.loadEntry(frt, pag, p+1, data=self.edate.work)

    def doDate(self, frt, pag, r, c, p, i, w):
        yr = w / 10000
        if yr < (self.taxyr - 1) or yr > self.taxyr:
            return "Invalid Date"
        mt = (w / 100) % 100
        if mt < 3 and yr < self.taxyr:
            return "Invalid Date"
        if mt > 2 and yr == self.taxyr:
            return "Invalid Date"
        dy = w % 100
        self.edate = CCD(w, "D1", 10)
        whr = [("wgm_cono", "=", self.conum), ("wgm_start", "<",
            self.edate.work), ("(", "wgm_term", "=", 0, "or", "wgm_term",
            ">=", self.sdate.work, ")")]
        self.df.topf[0][7][8]["where"] = whr
        self.df.topf[0][8][8]["where"] = whr

    def doReprint(self, frt, pag, r, c, p, i, w):
        self.reprint = w
        seq = self.sql.getRec(tables="wagirp", cols=["max(wip_irpno)"],
            where=[("wip_cono", "=", self.conum)], limit=1)
        if not seq[0]:
            self.lastreg = 0
            self.lastirp = 0
        else:
            self.lastirp = seq[0]
            reg = self.sql.getRec(tables="wagirp", cols=["wip_genno"],
                where=[("wip_cono", "=", self.conum), ("wip_irpno", "=",
                self.lastirp)], limit=1)
            self.lastreg = reg[0]
        if self.reprint == "Y":
            self.preview = "Y"
            self.df.loadEntry("T", 0, p+1, data=self.preview)
            return "sk1"

    def doPreview(self, frt, pag, r, c, p, i, w):
        self.preview = w

    def doWhole(self, frt, pag, r, c, p, i, w):
        self.whole = w
        if self.whole in ("Y", "S"):
            self.semp = 0
            self.eemp = 0
            self.df.loadEntry("T", 0, p+2, data=self.semp)
            self.df.loadEntry("T", 0, p+3, data=self.eemp)
            if self.whole == "S":
                self.other = "N"
                self.eml = "N"
                self.df.loadEntry("T", 0, p+1, data=self.other)
                self.df.loadEntry("T", 0, p+4, data=self.eml)
                return "sk4"
        else:
            self.other = "N"
            self.df.loadEntry("T", 0, p+1, data=self.other)
            return "sk1"

    def doCoy(self, frt, pag, r, c, p, i, w):
        self.other = w
        return "sk2"

    def doEmp(self, frt, pag, r, c, p, i, w):
        chk = self.sql.getRec(tables="wagmst", cols=["wgm_sname"],
            where=[("wgm_cono", "=", self.conum), ("wgm_empno", "=", w)],
            limit=1)
        if not chk:
            return "Invalid Employee Number"
        if c == 8:
            self.semp = w
        else:
            self.eemp = w
            self.eml = "N"
            self.df.loadEntry("T", 0, p+1, data=self.eml)
            return "sk1"

    def doEml(self, frt, pag, r, c, p, i, w):
        self.eml = w

    def doEnd(self):
        self.df.closeProcess()
        self.prtnam = self.df.repprt[0][2]
        self.email = self.df.repprt[1][1]
        self.emldf = self.df.repprt[1][2]
        error = self.doWriteHeader()
        if error:
            self.closeProcess()
            return
        self.pdfnam = None
        if self.email == "N":
            self.printSetup()
        if not self.eemp:
            self.eemp = 99999
        if self.other == "Y":
            self.doSelCoy()
            if self.con == "X":
                return "no"
            whr = [("wgm_cono", "in", tuple(self.con))]
        else:
            whr = [("wgm_cono", "=", self.conum)]
        whr.extend([("wgm_empno", "between", self.semp, self.eemp),
            ("wgm_start", "<=", self.edate.work), ("(", "wgm_term",
            "=", 0, "or", "wgm_term", ">", self.sdate.work, ")")])
        if self.whole == "S":
            all = getSingleRecords(self.mf, "wagmst",
                ("wgm_empno", "wgm_sname"), where=whr)
        else:
            all = self.sql.getRec(tables="wagmst", where=whr,
                order="wgm_cono, wgm_empno")
            if not all:
                showError(self.mf.body, "Error",
                "No Employees Selected")
        if all:
            num = len(all)
            p = ProgressBar(self.mf.body, max=num, esc=True)
            for x in range(0, num):
                can = p.displayProgress(x)
                if can == "yes":
                    if self.t:
                        self.t.textLine()
                        self.t.textLine("Print Job ABORTED")
                    break
                self.doProcess(all[x])
            p.closeProgress()
            #Employer Total
            tot = CCD(self.totamnt, "UD", 15.2)
            self.irp5fl.write('6010,%015i,6020,%015i,6030,%015.2f,9999\n' % \
                (self.totrecs, self.totcode, tot.work))
            self.totalls += 1
            #Trailer Record
            self.irp5fl.write('7010,%015i,9999\n' % self.totalls)
            self.irp5fl.close()
            if self.email == "N":
                self.doPrint()
            if self.preview == "Y":
                self.mf.dbm.rollbackDbase()
            else:
                self.mf.dbm.commitDbase()
            if self.eml == "Y":
                toad = "paye.cc@sars.gov.za"
                subj = "IRP5 Disk Submission"
                mess = ""
                try:
                    sendMail(self.smtp, self.efrom, [toad, self.efrom], subj,
                        mess, self.irp5nm)
                    ok = "OK"
                except:
                    ok = "FAILED"
                self.sql.insRec("emllog", data=[self.efrom, toad,
                    "%s - %s" % (subj, self.irp5nm),
                    "%04i-%02i-%02i %02i:%02i" % time.localtime()[0:5], ok])
                self.mf.dbm.commitDbase()
        self.closeProcess()

    def doWriteHeader(self):
        #Header Record
        if self.reprint == "N" or not self.lastreg:
            self.lastreg += 1
        self.irp5nm = os.path.join(self.mf.wrkdir,
            "irp5fl_%03i_%04i.txt" % (self.conum, self.taxyr))
        self.irp5fl = open(self.irp5nm, "w")
        try:
            self.irp5fl.write('1010,"%-70s",1020,%010i,1030,"%-30s",'\
            '1040,"%-16s",1060,"%-35s",1070,"%-35s",1080,"%-35s",'\
            '1100,%04i,1110,%08i,1120,%04i,1130,"%4s",9999\n' % \
            (self.conam, int(self.coreg.work), self.cname.work,
            self.cotel.work, self.coad1.work, self.coad2.work,
            self.coad3.work, int(self.copcd.work), self.edate.work,
            self.lastreg, self.rtype))
        except:
            showError(None, "Header Record Error", "There is a Problem "\
                "with your Company or Control Records, Please Fix the "\
                "Problem and then Reprint")
            return "error"
        self.totalls += 1
        #Employer Details
        try:
            self.irp5fl.write('2010,"%-70s",2020,%010i,2030,%04i,2040,'\
            '"%-35s",2050,"%-35s",2060,"%-35s",2080,%04i,9999\n' % \
            (self.conam, int(self.coreg.work), self.taxyr, self.coad1.work,
            self.coad2.work, self.coad3.work, int(self.copcd.work)))
        except:
            showError(None, "Header Record Error", "There is a Problem "\
                "with your Company or Control Records, Please Fix the "\
                "Problem and then Reprint")
            return "error"
        self.totrecs += 1
        self.totcode = 24289
        self.totalls += 1

    def doSelCoy(self):
        tit = ("Company Selection",)
        self.coys = self.sql.getRec(tables="ctlmst", cols=["ctm_cono",
            "ctm_name"], order="ctm_cono")
        coy = {
            "stype": "C",
            "titl": "",
            "data": self.coys,
            "mode": "M",
            "comnd": self.doCoyCmd}
        r1s = (("Yes","Y"),("Include","I"),("Exclude","E"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"All Companies","All Companies",
                "Y","N",self.doAllCoy,None,None,None,None),
            (("T",0,1,0),"INA",30,"Companies","Companies",
                "","N",self.doCoySel,coy,None,None,None))
        self.cf = TartanDialog(self.mf, screen=self.mf.body, tops=True,
            title=tit, eflds=fld, tend=((self.doCoyEnd, "y"),),
            txit=(self.doCoyExit,))
        self.mf.startLoop()

    def doAllCoy(self, frt, pag, r, c, p, i, w):
        self.con = w
        if self.con == "Y":
            self.con = []
            for coy in self.coys:
                self.con.append(coy[0])
            return "nd"
        elif self.con == "I":
            self.cf.topf[pag][2][8]["titl"] = "Select Companies to Include"
        else:
            self.cf.topf[pag][2][8]["titl"] = "Select Companies to Exclude"

    def doCoyCmd(self, frt, pag, r, c, p, i, w):
        c = ""
        for co in w:
            if int(co[0]):
                c = c + str(int(co[0])) + ","
        if len(c) > 1:
            c = c[:-1]
        self.cf.loadEntry(frt, pag, p, data=c)

    def doCoySel(self, frt, pag, r, c, p, i, w):
        if w[-1:
           ] == ",": w = w[:-1]
        self.coy = w.split(",")

    def doCoyEnd(self):
        if self.con == "I":
            self.con = self.coy
        elif self.con == "E":
            self.con = []
            for co in self.coys:
                self.con.append(int(co[0]))
            for co in self.coy:
                del self.con[self.con.index(int(co))]
            self.con.sort()
        self.doCoyClose()

    def doCoyExit(self):
        self.con = "X"
        self.doCoyClose()

    def doCoyClose(self):
        self.cf.closeProcess()
        self.mf.closeLoop()

    def doProcess(self, wgm):
        col = self.sql.wagmst_col
        self.com = CCD(wgm[col.index("wgm_cono")], "UI", 3)
        self.emp = CCD(wgm[col.index("wgm_empno")], "UI", 5)
        paye = CCD(wgm[col.index("wgm_paye")], "UA", 1)
        if paye.work != "Y":
            return
        if self.reprint == "N":
            chk = self.sql.getRec(tables="wagirp", cols=["wip_irpno"],
                where=[("wip_cono", "=", self.com.work), ("wip_empno", "=",
                self.emp.work), ("wip_year", "=", self.taxyr)], limit=1)
            if chk:
                return
        self.sname = CCD(wgm[col.index("wgm_sname")], "NA", 30)
        fname = wgm[col.index("wgm_fname")].split()
        if len(fname) == 1:
            self.fname = CCD(fname[0], "NA", 30)
        else:
            self.fname = CCD("%s %s" % (fname[0], fname[1]), "NA", 30)
        self.inits = ""
        for i in fname:
            self.inits = self.inits + i[0]
        if self.emldf:
            self.emadd = CCD(self.emldf, "TX", 0)
        else:
            self.emadd = CCD(wgm[col.index("wgm_emadd")], "TX", 0)
        self.nature = CCD(wgm[col.index("wgm_nature")], "NA", 1)
        self.idno = CCD(wgm[col.index("wgm_idno")], "UL", 13)
        self.dob = CCD(wgm[col.index("wgm_dob")], "D1", 10)
        self.addr1 = CCD(wgm[col.index("wgm_addr1")], "NA", 30)
        self.addr2 = CCD(wgm[col.index("wgm_addr2")], "NA", 30)
        self.addr3 = CCD(wgm[col.index("wgm_addr3")], "NA", 30)
        self.pcode = CCD(wgm[col.index("wgm_pcode")], "NA", 4)
        self.taxno = CCD(wgm[col.index("wgm_taxno")], "NA", 10)
        self.start = CCD(wgm[col.index("wgm_start")], "D1", 10)
        if self.start.work < self.sdate.work:
            self.start = self.sdate
        self.term = CCD(wgm[col.index("wgm_term")], "D1", 10)
        if not self.term.work:
            self.term = self.edate
        self.freq = CCD(wgm[col.index("wgm_freq")], "UA", 1)
        # Test number with modulus 10
        if self.taxno.work and len(self.taxno.work) == 10:
            paye10 = 0
            for x in range(0, 9, 2):
                paye10b = int(self.taxno.work[x]) * 2
                paye10 = paye10 + (paye10b / 10) + (paye10b % 10)
                if  x < 8:
                    paye10 = paye10 + int(self.taxno.work[(x + 1)])
            paye10 = paye10 % 10
            paye10 = 10 - paye10
            if  paye10 != int(self.taxno.work[9]):
                self.taxno = CCD("0000000000", "NA", 10)
        else:
            self.taxno = CCD("0000000000", "NA", 10)
        #
        self.vol = CCD(wgm[col.index("wgm_vol_over")], "UA", 10)
        rate = CCD(wgm[col.index("wgm_fix_rate")], "UD", 6.2)
        if rate.work:
            self.fixed = "Y"
        else:
            self.fixed = "N"
        self.direct = CCD(wgm[col.index("wgm_direct")], "NA", 13)
        #
        # Extract all transactions
        wgt = self.sql.getRec(tables=["wagtf2", "wagedc", "wagrcv"],
            cols=["wt2_type", "ced_ror", "ced_taxcode", "rcv_desc",
            "round(sum(wt2_eamt), 2)"], where=[("wt2_cono", "=",
            self.com.work), ("wt2_empno", "=", self.emp.work),
            ("wt2_date", "between", self.sdate.work, self.edate.work),
            ("ced_cono=wt2_cono",), ("ced_type=wt2_type",),
            ("ced_code=wt2_code",), ("ced_ror", "between", 3601, 4103),
            ("rcv_code=ced_ror",)],
            group="wt2_type, ced_ror, ced_taxcode, rcv_desc",
            order="wt2_type desc, ced_ror")
        if wgt:
            if self.email == "Y":
                self.printSetup()
            self.printHeader()
            # Retirement Funding
            self.rfund = "N"
            for trn in wgt:
                if trn[1] in (4001, 4002, 4003, 4004):
                    self.rfund = "Y"
            #
            trtp = None
            for trn in wgt:
                if trn[0] == "E":
                    trtp = self.doEarningCode(trtp, trn)
                else:
                    if trtp == "E":
                        self.printGross()
                    if trn[1] < 4101:
                        trtp = self.doDeductionCode(trtp, trn)
                    elif trn[1] == 4102:
                        trtp = "T"
                        site = self.doSite()
                        self.printTax(site, trn)
            if trtp == "E":
                self.printGross()
            if trtp != "T" and self.ttyp == "IT3(A)":
                self.printTax()
            if self.preview == "N":
                self.sql.insRec("wagirp", data=[self.conum, self.taxyr,
                    self.lastreg, self.emp.work, self.lastirp])
            if self.email == "Y":
                self.doPrint()

    def doEarningCode(self, trtp, trn):
        if not trtp:
            self.t.setFont("Tartan-Bold", self.cpi)
            self.t.textLine("Income Sources")
            self.t.textLine("Code    Description              "\
                "                        R/F-Ind       Amount")
            self.t.setFont("Tartan", self.cpi)
        # Earning Codes 3601-3612, 3651-3667, 3701-3714, 3751-3768
        # 3801-3810, 3851-3860, 3813-3863, 3901-3907, 3951-3957
        trtp = "E"
        if trn[1] == 3601:
            rfund = self.rfund
        else:
            rfund = "N"
        if trn[1] == 3810:
            trn[4] = round((trn[4] / 3), 2)
        self.t.textLine("%-4s    %-50s  %1s       %9s" % \
            (trn[1], trn[3], rfund, int(trn[4])))
        self.emprec = '%s,%04i,"%1s",%015i' % \
            (self.emprec, trn[1], rfund, int(trn[4]))
        self.totcode = self.totcode + trn[1]
        if trn[1] in (3602, 3604, 3609, 3612, 3703, 3705, 3709, 3714):
            ### Non Taxable Income
            self.totntax = self.totntax + int(trn[4])
        else:
            if trn[2] == "O":
                ### Annual Payment
                self.totannp = self.totannp + int(trn[4])
            if rfund == "Y":
                ### Retirement Funding ###
                self.totyrfi = self.totyrfi + int(trn[4])
            else:
                self.totnrfi = self.totnrfi + int(trn[4])
            self.totincm = self.totincm + int(trn[4])
        self.totamnt = float(ASD(self.totamnt) + ASD(int(trn[4])))
        return trtp

    def doDeductionCode(self, trtp, trn):
        if trtp in (None, "E"):
            self.t.setFont("Tartan-Bold", self.cpi)
            self.t.textLine("Deductions")
            self.t.textLine("Code    Description              "\
                "                        Clearance     Amount")
            self.t.setFont("Tartan", self.cpi)
        # Deduction Codes 4001-4007, 4018, 4024, 4025
        trtp = "D"
        self.t.textLine("%-4s    %-50s  %1s       %9s" % \
            (trn[1], trn[3], "N", int(trn[4])))
        self.emprec = '%s,%04i,%015i' % \
            (self.emprec, trn[1], int(trn[4]))
        self.totcode = self.totcode + trn[1]
        self.totamnt = float(ASD(self.totamnt) + ASD(int(trn[4])))
        return trtp

    def doSite(self):
        max = siteRates[str(self.taxyr)] / self.base * self.pers
        inc = round((max * self.base / self.pers), 4)
        rates = payeRates[str(self.taxyr)]["rates"]
        allow = payeRates[str(self.taxyr)]["allow"]
        tax = 0
        for lvl in rates:
            if inc > lvl[0]:
                tax = round(lvl[1] + (((inc - lvl[0] + 1) * lvl[2]) / 100.0), 2)
            if tax:
                break
        tax = float(ASD(tax) - ASD(allow[0]))
        age = dateDiff(self.dob.work, self.edate.work, "years")
        if age > 64:
            tax = float(ASD(tax) - ASD(allow[1]))
        if age > 74:
            tax = float(ASD(tax) - ASD(allow[2]))
        tax = round((tax * self.pers / self.base), 2)
        if tax < 0:
            return 0
        else:
            return tax

    def printSetup(self):
        dir = self.mf.wrkdir
        if self.email == "Y":
            key = "%s_%s" % (self.com.work, self.emp.work)
            self.pdfnam = getModName(dir, __name__, key, ext="pdf")
        elif not self.pdfnam:
            self.pdfnam = getModName(dir, __name__, self.conum, ext="pdf")
        self.c, self.cpi, self.inch, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam)
        self.cwidth = self.c.stringWidth(("*"), "Tartan", 8)

    def printHeader(self):
        # Employee Totals
        self.totannp = 0
        self.totntax = 0
        self.totntax = 0
        self.totyrfi = 0
        self.totnrfi = 0
        self.totincm = 0
        # Calculate Pay Periods
        if self.freq.work == "M":
            self.base = 12
            y1 = int(self.term.work / 10000)
            y2 = int(self.start.work / 10000)
            m1 = int((self.term.work - (y1 * 10000)) / 100)
            m2 = int((self.start.work - (y2 * 10000)) / 100)
            self.pers = 1 + ((y1 - y2) * self.base) + (m1 - m2)
            while self.pers > self.base:
                self.pers = self.pers - self.base
        else:
            days = dateDiff(self.start.work, self.term.work, "days")
            if self.freq.work == "W":
                per = 7
                self.base = 52
            else:
                per = 14
                self.base = 26
            self.pers = int(days / per)
            if days % per > 0:
                self.pers = self.pers + 1
            if self.pers > self.base:
                self.pers = self.base
        #
        # Extract PAYE Paid
        tax = self.sql.getRec(tables=["wagtf2", "wagedc"],
            cols=["round(sum(wt2_eamt), 2)"], where=[("wt2_cono", "=",
            self.com.work), ("wt2_empno", "=", self.emp.work), ("wt2_date",
            "between", self.sdate.work, self.edate.work),
            ("ced_cono=wt2_cono",), ("ced_type=wt2_type",),
            ("ced_code=wt2_code",), ("ced_ror", "=", 4102)])
        if tax[0][0]:
            self.tax = CCD(tax[0][0], "UD", 12.2)
        else:
            self.tax = CCD(0, "UD", 12.2)
        if self.tax.work:
            self.ttyp = "IRP 5"
        else:
            self.ttyp = "IT3(A)"
        #
        self.c.translate(self.inch, self.inch) # Set x and y margins
        self.t = self.c.beginText()
        self.t.setTextOrigin(0, 0)

        self.drawDocument()

        if self.reprint == "Y":
            acc = self.sql.getRec(tables="wagirp", cols=["wip_irpno"],
                where=[("wip_cono", "=", self.conum), ("wip_year", "=",
                self.taxyr), ("wip_empno", "=", self.emp.work)], limit=1)
            if not acc:
                self.lastirp += 1
            else:
                self.lastirp = acc[0]
        else:
            self.lastirp += 1
        self.irp5 = CCD(self.lastirp, "UI", 8)

        self.t.textLine()
        self.t.textLine()
        self.t.setFont("Tartan-Bold", self.cpi)
        self.t.textLine("%6s                    EMPLOYEES TAX "\
            "CERTIFICATE                      %6s" % (self.ttyp, self.ttyp))
        self.t.textLine()
        self.t.textLine("Employer Information")
        self.t.setFont("Tartan", 8)
        self.t.textLine("Trading Name      %-30s" % self.conam)
        self.t.textLine("IRP 5 Number      %s                        "\
            "Business Address  %s" % (self.irp5.disp, self.coad1.disp))
        self.t.textLine("Reference No.     %s                      "\
            "                  %s" % (self.coreg.disp, self.coad2.disp))
        self.t.textLine("Tax Year          %s                            "\
            "                  %s" % (self.taxyr, self.coad3.disp))
        self.t.textLine("Dipl. Immunity    %s                               "\
            "                  %s" % (self.codip.disp, self.copcd.disp))
        self.t.textLine()
        self.t.setFont("Tartan-Bold", self.cpi)
        self.t.textLine("Employee Information")
        self.t.setFont("Tartan", 8)
        self.t.textLine("Nature of Person  %s                               "\
            "Employee Surname  %s" % (self.nature.disp, self.sname.disp))
        self.t.textLine("First Two Names   %s  "\
            "Employee Initials %s" % (self.fname.disp, self.inits))
        self.t.textLine("Identity Number   %s                   "\
            "Residential Addr  %s" % (self.idno.disp, self.addr1.disp))
        self.t.textLine("Passport Number                                   "\
            "                  %s" % self.addr2.disp)
        self.t.textLine("Date of Birth     %s                      "\
            "                  %s" % (self.dob.disp, self.addr3.disp))
        self.t.textLine("Company/CC/Trust                                  "\
            "                  %s" % self.pcode.disp)
        self.t.textLine("Income Tax Number %s" % self.taxno.disp)
        self.t.textLine("                                                  "\
            "Employee Number   %s" % self.emp.disp)
        self.t.textLine()
        self.t.setFont("Tartan-Bold", self.cpi)
        self.t.textLine("Tax Calculation Information")
        self.t.setFont("Tartan", 8)
        self.t.textLine("Employed From  %s        Employed To    %s"\
            "        Voluntary Over-Deduction  %s" % (self.start.disp,
            self.term.disp, self.vol.disp))
        self.t.textLine("Periods in Year  %08.4f        Periods Worked"\
            "   %08.4f        Fixed Rate %s Directive %s" % (self.base,
            self.pers, self.fixed, self.direct.disp))
        self.t.textLine()
        #Employee Details
        try:
            pcode = int(self.pcode.work)
        except:
            pcode = 0
        try:
            if self.dob.work % 1000000 != self.idno.work / 10000000:
                raise Exception
            else:
                self.emprec = '3010,"%08i",3020,"%1s",3030,"%-120s",'\
                    '3040,"%-90s",3050,"%-5s",3060,%013i,3070,"%-13s",'\
                    '3080,%08i,3090,"%-16s",3100,%010i,3110,"%-35s",'\
                    '3120,"%-35s",3130,"%-35s",3140,"%-35s",3150,"%-20s",'\
                    '3160,"%25s",3170,%08i,3180,%08i,3190,"%1s",3200,%08.4f,'\
                    '3210,%08.4f,3220,"%1s",3230,"%-13s"' % \
                    (int(self.irp5.work), self.nature.work, self.sname.work,
                    self.fname.work, self.inits, int(self.idno.work), "",
                    int(self.dob.work), "", int(self.taxno.work),
                    self.addr1.work, self.addr2.work, self.addr3.work, "",
                    pcode, self.emp.work, int(self.start.work),
                    int(self.term.work), self.vol.work, self.base,
                    self.pers, self.fixed, self.direct.work)
                self.totrecs += 1
        except:
            showError(None, "Header Record Error", "There is a Problem "\
                "with Employee %s's Record, Please Fix the Problem and "\
                "then Reprint" % self.emp.disp)
            return "error"
        self.totcode = self.totcode + 71760

    def printGross(self):
        self.t.textLine()
        self.t.setFont("Tartan-Bold", self.cpi)
        self.t.textLine("Gross Remuneration")
        self.t.textLine("Code    Description                            "\
            "                        Amount")
        self.t.setFont("Tartan", self.cpi)
        amount = CCD(self.totannp, "UI", 15)
        self.t.textLine("3695    Annual Payments                        "\
            "               %s" % amount.disp)
        amount = CCD(self.totntax, "UI", 15)
        self.t.textLine("3696    Non-Taxable Income                     "\
            "               %s" % amount.disp)
        amount = CCD(self.totyrfi, "UI", 15)
        self.t.textLine("3697    Retirement Funding Income              "\
            "               %s" % amount.disp)
        amount = CCD(self.totnrfi, "UI", 15)
        self.t.textLine("3698    Non-Retirement Funding Income          "\
            "               %s" % amount.disp)
        amount = CCD(self.totincm, "UI", 15)
        self.t.textLine("3699    Gross Remuneration                     "\
            "               %s" % amount.disp)
        self.t.textLine()
        #Gross Earnings
        self.emprec = '%s,3695,%015i,3696,%015i,3697,%015i,3698,%015i,'\
            '3699,%015i' % (self.emprec, self.totannp, self.totntax,
            self.totyrfi, self.totnrfi, self.totincm)
        self.totcode = self.totcode + 18485
        self.totamnt = float(ASD(self.totamnt) + ASD(self.totannp))
        self.totamnt = float(ASD(self.totamnt) + ASD(self.totntax))
        self.totamnt = float(ASD(self.totamnt) + ASD(self.totyrfi))
        self.totamnt = float(ASD(self.totamnt) + ASD(self.totnrfi))
        self.totamnt = float(ASD(self.totamnt) + ASD(self.totincm))

    def printTax(self, site=None, trn=None):
        self.t.textLine()
        self.t.setFont("Tartan-Bold", self.cpi)
        self.t.textLine("Employees Tax Deductions")
        self.t.textLine("Code    Description                            "\
            "                        Amount")
        self.t.setFont("Tartan", self.cpi)
        if not site and not trn:
            self.t.textLine("4150    Reason Code                            "\
                "                  %s" % "          02")
            #Tax Codes IT3(a) 4150
            self.emprec = '%s,4150,%02i,9999\n' % (self.emprec, 2)
            self.totcode = self.totcode + 14149
            self.totamnt = float(ASD(self.totamnt) + ASD(2))
        else:
            site = CCD(site, "UD", 15.2)
            if site.work >= trn[4]:
                site = CCD(trn[4], "UD", 15.2)
                paye = 0
            else:
                paye = float(ASD(trn[4]) - ASD(site.work))
            paye = CCD(paye, "UD", 15.2)
            total = CCD(trn[4], "UD", 15.2)
            self.t.textLine("4101    Standard Income Tax - SITE             "\
                "               %s" % site.disp)
            self.t.textLine("4102    Pay As You Earn                        "\
                "               %s" % paye.disp)
            self.t.textLine("4103    Total Employees Tax                    "\
                "               %s" % total.disp)
            #Tax Codes 4101-4103
            self.emprec = '%s,4101,%015.2f,4102,%015.2f,4103,%015.2f,9999\n' % \
                (self.emprec, site.work, paye.work, total.work)
            self.totcode = self.totcode + 22305
            self.totamnt = float(ASD(self.totamnt) + ASD(site.work))
            self.totamnt = float(ASD(self.totamnt) + ASD(paye.work))
            self.totamnt = float(ASD(self.totamnt) + ASD(total.work))
        self.c.drawText(self.t)
        self.c.showPage()
        self.irp5fl.write(self.emprec)
        self.totalls += 1

    def drawDocument(self):
        pass

    def doPrint(self):
        self.c.save()
        if self.prtnam == "view":
            view = "Y"
        else:
            view = "N"
        if self.email == "N" or not self.emadd.work:
            mail = "N"
        else:
            mail = self.emadd.work
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam, head=self.tit,
            view=view, prtnam = self.prtnam, mail=mail)

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

