#! /usr/bin/python

"""
SYNOPSIS
    Salaries and Wages Payslips Reprinting.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, time
from TartanClasses import ASD, GetCtl, PrintPayslip, Sql, TartanDialog
from tartanFunctions import getSingleRecords

class wg3070:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["wagctl", "wagmst"], prog=__name__)
        gc = GetCtl(self.mf)
        wagctl = gc.getCtl("wagctl", self.conum)
        if not wagctl:
            return "no"
        self.bestac = wagctl["ctw_bestac"]
        self.besttp = wagctl["ctw_besttp"]
        t = time.localtime()
        self.sysdtw = ((t[0] * 10000) + (t[1] * 100) + t[2])
        self.etotal = 0
        self.empnos = []

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Salaries and Wages Payslips Reprint (%s)" % __name__)
        prd = {
            "stype": "R",
            "tables": ("wagtf1",),
            "cols": [
                ("wt1_date", "",  0, "Date")],
            "where": [("wt1_cono", "=", self.conum)],
            "group": "wt1_date",
            "order": "wt1_date"}
        r1s = (("Weekly","W"),("Fortnightly","F"),("Monthly","M"))
        r2s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),"ID1",10,"Pay-Run Date","Pay-Run Date",
                "","N",self.doRunDate,prd,None,("efld",)),
            (("T",0,1,0),("IRB",r1s),0,"Frequency","Payment Frequency",
                "M","N",self.doFreq,None,None,None),
            (("T",0,2,0),("IRB",r2s),0,"Whole File","Whole File",
                "N","N",self.doWhole,None,None,None),
            (("T",0,3,0),"INA",3,"Branch","Branch",
                "","N",self.doBranch,None,None,None),
            (("T",0,4,0),"IUI",1,"Department","Department",
                "","N",self.doDept,None,None,None))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("N","Y"))

    def doRunDate(self, frt, pag, r, c, p, i, w):
        self.rundt = w
        self.curdt = w / 100
        y = w / 10000
        m = (w % 10000) / 100
        if m < 3:
            self.sdate = ((y - 1) * 10000) + 301
        else:
            self.sdate = (y * 10000) + 301

    def doFreq(self, frt, pag, r, c, p, i, w):
        self.freq = w

    def doWhole(self, frt, pag, r, c, p, i, w):
        self.whole = w
        if self.whole == "Y":
            self.branch = ""
            self.dept = ""
            return "sk2"

    def doBranch(self, frt, pag, r, c, p, i, w):
        self.branch = w

    def doDept(self, frt, pag, r, c, p, i, w):
        self.dept = w

    def doEnd(self):
        self.df.closeProcess()
        self.prtnam = self.df.repprt[0][2]
        self.email = self.df.repprt[1][1]
        self.noprt = self.df.repprt[1][3]
        # Export File for Bank
        if self.bestac:
            self.export = open(os.path.join(self.mf.wrkdir, "bank.txt"), "w")
            self.export.write("%1s%4s%-40s%8s%1s%8s%-15s%1s%2s%1s%9s%2s%4s"\
            "\r\n" % ("*", self.bestac, self.conam, self.sysdtw, "Y", "",
            "SALARIES EFT", "+", self.besttp, 0, "", "01", "LIVE"))
        else:
            self.export = None
        whr = [("wgm_cono", "=", self.conum), ("wgm_freq", "=", self.freq),
            ("wgm_term", "=", 0, "or", "wgm_term", ">=", self.rundt)]
        if self.branch:
            whr.append(("wgm_branch", "=", self.branch))
        if self.dept:
            whr.append(("wgm_dept", "=", self.dept))
        if self.whole == "N":
            all = getSingleRecords(self.mf, "wagmst",
                ("wgm_empno", "wgm_sname"), where=whr)
            for emp in all:
                self.empnos.append(emp[1])
        else:
            all = self.sql.getRec(tables="wagmst", cols=["wgm_empno"],
                where=whr, order="wgm_empno")
            for emp in all:
                self.empnos.append(emp[0])
        if self.empnos:
            self.doProcess()
            if self.export:
                value = long(round((self.etotal * 100), 0))
                self.export.write("%1s%4s%1s%30s%013u%47s\r\n" % \
                    (2, self.bestac, "T", "", value, ""))
                self.export.close()
        ######################
        self.mf.closeLoop()

    def doProcess(self):
        psl = PrintPayslip(self.mf, self.conum, self.empnos, self.rundt,
            progs="y", runtp="c", prtnam=self.prtnam, email=self.email,
            noprt=self.noprt, export=self.export, bestac=self.bestac)
        self.etotal = float(ASD(self.etotal) + ASD(psl.etotal))

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

