#! /usr/bin/env python

"""
SYNOPSIS
    Salaries Interrogation.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, NotesCreate, SChoice, Sql, SRec
from TartanClasses import TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter

class wg4010:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.capnm = usr
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["wagmst", "wagedc", "wagcod", "wagbal",
            "wagtf1", "wagtf2"], prog=__name__)
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) +  (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        if t[1] < 3:
            self.taxyr = t[0]
        else:
            self.taxyr = t[0] + 1
        self.startb = None
        self.startt = None
        self.endb = None
        self.endt = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Salaries/Wages Interrogation (%s)" % __name__)
        wgm = {
            "stype": "R",
            "tables": ("wagmst",),
            "cols": (
                ("wgm_empno", "", 0, "Emp-Num"),
                ("wgm_sname", "", 0, "Surname"),
                ("wgm_fname", "", 0, "Namse")),
            "where": [("wgm_cono", "=", self.conum)]}
        tag = (
            ("General", self.doTagSelect, ("T",0,2), ("T",0,1)),
            ("Tax", self.doTagSelect, ("T",0,2), ("T",0,1)),
            ("Bank", self.doTagSelect, ("T",0,2), ("T",0,1)),
            ("Earnings", self.doTagSelect, ("T",0,2), ("T",0,1)),
            ("Deductions", self.doTagSelect, ("T",0,2), ("T",0,1)),
            ("Balances", self.doTagSelect, ("T",0,2), ("T",0,1)),
            ("Progs", self.doTaxYear, ("T",0,2), ("T",0,1)),
            ("Trans", self.doTaxYear, ("T",0,2), ("T",0,1)))
        fld = (
            (("T",0,0,0),"IUI",5,"Emp-Num","Employee Number",
                "","Y",self.doEmpno,wgm,None,("notzero",)),
            (("T",0,0,0),"ONA",3,"Branch"),
            (("T",0,0,0),"OUI",1,"Department"),
            (("T",0,0,0),"OUI",1,"Class"),
            (("T",1,0,0),"ONA",30,"Surname"),
            (("T",1,1,0),"ONA",30,"Names"),
            (("T",1,2,0),"OD1",10,"Date of Birth"),
            (("T",1,3,0),"ONA",16,"ID Number"),
            (("T",1,4,0),"ONA",16,"Spouse Name"),
            (("T",1,5,0),"ONA",16,"Spouse ID Number"),
            (("T",1,6,0),"ONA",30,"Address Line 1"),
            (("T",1,7,0),"ONA",30,"Address Line 2"),
            (("T",1,8,0),"ONA",30,"Address Line 3"),
            (("T",1,9,0),"ONA",4,"Postal Code"),
            (("T",1,10,0),"ONA",16,"Telephone Number"),
            (("T",1,11,0),"OTX",50,"E-Mail Address"),
            (("T",1,12,0),"OD1",10,"Start Date"),
            (("T",1,13,0),"Od1",10,"Term  Date"),
            (("T",1,14,0),"ONA",3,"Status"),
            (("T",1,15,0),"OUD",10.2,"Salary"),
            (("T",1,16,0),"OUA",1,"Pay Freq"),
            (("T",1,17,0),"OUA",1,"Pay Type"),
            (("T",2,0,0),"OUA",1,"P.A.Y.E."),
            (("T",2,1,0),"ONA",16,"Tax Office"),
            (("T",2,2,0),"ONA",10,"Tax Number"),
            (("T",2,3,0),"OUA",1,"Nature of Employee"),
            (("T",2,4,0),"OUI",9,"Reg Number"),
            (("T",2,5,0),"OUA",1,"Voluntary Excess"),
            (("T",2,6,0),"OUD",6.2,"Fixed Rate"),
            (("T",2,7,0),"ONA",13,"Directive"),
            (("T",3,0,0),"OUA",1,"Account Type"),
            (("T",3,1,0),"OUI",8,"Branch Code"),
            (("T",3,2,0),"ONA",16,"Account Number"),
            (("T",3,3,0),"ONA",30,"Bank Name"),
            (("C",4,0,0),"OUI",3,"Cod"),
            (("C",4,0,1),"ONA",30,"Description"),
            (("C",4,0,2),"OSD",13.2,"Amnt/Rate"),
            (("C",5,0,0),"OUI",3,"Cod"),
            (("C",5,0,1),"ONA",30,"Description"),
            (("C",5,0,2),"OSD",13.2,"Employee"),
            (("C",5,0,3),"OSD",13.2,"Employer"),
            (("T",6,0,0),"OSD",13.2,"Balance-1"),
            (("T",6,1,0),"OSD",13.2,"Balance-2"),
            (("T",6,2,0),"OSD",13.2,"Balance-3"),
            (("T",7,0,0),"IUI",4,"Tax Year", "Tax Year",
                self.taxyr,"N",self.doBals,None,None,None),
            (("C",7,0,0),"OUA",1,"T"),
            (("C",7,0,1),"OUI",3,"Cod"),
            (("C",7,0,2),"ONA",30,"Description"),
            (("C",7,0,3),"OSD",13.2,"Hours"),
            (("C",7,0,4),"OSD",13.2,"Value"),
            (("T",8,0,0),"IUI",4,"Tax Year", "Tax Year",
                self.taxyr,"N",self.doTrans,None,None,None))
        tnd = (
            None, None, None, None, None, None, None, None,None)
        txt = (
            self.doExit, None, None, None, None, None, None, None, self.doExit)
        but = (
            ("Clear",None,self.doClear,0,("T",0,0),("T",0,1)),
            ("Notes",None,self.doNotes,0,("T",0,0),("T",0,1)),
            ("Print",None,self.doPrint,0,("T",0,0),("T",0,1)),
            ("Quit",None,self.doExit,1,None,None))
        row = [0, 0, 0, 0, 15, 15, 0, 15, 0]
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tags=tag,
            rows=row, tend=tnd, txit=txt, butt=but)

    def doEmpno(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="wagmst", where=[("wgm_cono", "=",
            self.conum), ("wgm_empno", "=", w)], limit=1)
        if not acc:
            return "Invalid Employee Number"
        self.empno = w
        self.sname = acc[5]
        self.fname = acc[6]
        acc.insert(18, acc.pop())                    # Termination Date
        acc.insert(19, acc.pop())                    # Status Code
        self.df.loadEntry("T", 0, 1, data=acc[2])
        self.df.loadEntry("T", 0, 2, data=acc[3])
        self.df.loadEntry("T", 0, 3, data=acc[4])
        d = 5
        for pag in range(1, 4):
            for x in range(0, self.df.topq[pag]):
                self.df.loadEntry("T", pag, x, data=acc[d])
                d = d + 1
        self.loadDetail()
        self.mf.updateStatus("")

    def loadDetail(self):
        ear = self.sql.getRec(tables=["wagcod", "wagedc"], cols=["wcd_type",
            "wcd_code", "ced_desc", "wcd_eamt", "wcd_ramt"],
            where=[("wcd_cono", "=", self.conum), ("wcd_empno", "=",
            self.empno), ("wcd_type", "=", "E"), ("ced_cono=wcd_cono",),
            ("ced_type=wcd_type",), ("ced_code=wcd_code",)], order="wcd_code")
        p = 0
        for e in ear:
            for y in range (1, 4):
                self.df.loadEntry("C", 4, p, data=e[y])
                p += 1
        ded = self.sql.getRec(tables=["wagcod", "wagedc"], cols=["wcd_type",
            "wcd_code", "ced_desc", "wcd_eamt", "wcd_ramt"],
            where=[("wcd_cono", "=", self.conum), ("wcd_empno", "=",
            self.empno), ("wcd_type", "=", "D"), ("ced_cono=wcd_cono",),
            ("ced_type=wcd_type",), ("ced_code=wcd_code",)], order="wcd_code")
        p = 0
        for d in ded:
            for y in range (1, 5):
                self.df.loadEntry("C", 5, p, data=d[y])
                p += 1
        bal = self.sql.getRec(tables="wagbal", where=[("wbl_cono", "=",
            self.conum), ("wbl_empno", "=", self.empno)])
        if bal:
            for b in bal:
                self.df.loadEntry("T", 6, (b[2] - 1), data=b[3])

    def doTagSelect(self):
        self.mf.updateStatus("")

    def doBals(self, frt, pag, r, c, p, i, w):
        self.startb = (((w - 1) * 10000) + 301)
        if w % 4 == 0:
            self.endb = ((w * 10000) + 229)
        else:
            self.endb = ((w * 10000) + 228)
        wt2 = self.sql.getRec(tables="wagtf2", cols=["wt2_type", "wt2_code",
            "round(sum(wt2_hrs),2)", "round(sum(wt2_eamt),2)"],
            where=[("wt2_cono", "=", self.conum), ("wt2_empno", "=",
            self.empno), ("wt2_date", "between", self.startb, self.endb)],
            group="wt2_type, wt2_code", order="wt2_type desc, wt2_code")
        if wt2:
            for num, line in enumerate(wt2):
                typ = CCD(line[0], "UA", 1.0)
                cod = CCD(line[1], "UI", 3.0)
                det = self.sql.getRec(tables="wagedc", cols=["ced_desc"],
                    where=[("ced_cono", "=", self.conum), ("ced_type", "=",
                    typ.work), ("ced_code", "=", cod.work)], limit=1)
                if not det:
                    des = "Unknown"
                else:
                    des = det[0]
                self.df.loadEntry("C", pag, (num * 5), data=typ.work)
                self.df.loadEntry("C", pag, (num * 5) + 1, data=cod.work)
                self.df.loadEntry("C", pag, (num * 5) + 2, data=des)
                if line[2]:
                    self.df.loadEntry("C", pag, (num * 5) + 3, data=line[2])
                self.df.loadEntry("C", pag, (num * 5) + 4, data=line[3])

    def doTrans(self, frt, pag, r, c, p, i, w):
        self.startt = (((w - 1) * 10000) + 301)
        if w % 4 == 0:
            self.endt = ((w * 10000) + 229)
        else:
            self.endt = ((w * 10000) + 228)
        tit = "Transactions for Employee: %s - %s %s" % \
            (self.empno, self.fname, self.sname)
        tab = ["wagtf1"]
        wcp = (("wt1_date", "", 0, "Date"),
                ("wt1_page", "", 0, "Number"),
                ("wt1_nhrs", "", 9.2, "Norm-Hrs"),
                ("wt1_npay", "", 9.2, "Norm-Pay"),
                ("wt1_taxbl", "", 11.2, "Tx-Pay"),
                ("wt1_notax", "", 11.2, "Non-Tx-Pay"),
                ("wt1_taxdd", "", 11.2, "Tx-Ded"),
                ("wt1_nondd", "", 11.2, "Non-Tx-Ded"))
        whr = [("wt1_cono", "=", self.conum), ("wt1_empno", "=", self.empno),
            ("wt1_date", "between", self.startt, self.endt)]
        ord = "wt1_date"
        state = self.df.disableButtons()
        rec = SRec(self.mf, screen=self.df.nb.Page7, title=tit, tables=tab,
            cols=wcp, where=whr, order=ord, rows=12)
        self.df.enableButtons(state=state)
        self.doTaxYear()

    def doTaxYear(self, widget=None):
        if self.df.pag == 7:
            self.df.clearFrame("C", 7)
            self.df.focusField("T", 7, 1)
        else:
            self.df.focusField("T", 8, 1)

    def doNotes(self, widget=None):
        state = self.df.disableButtons()
        self.df.mstAlw.hide()
        nf = NotesCreate(self.mf, self.conum, self.conam, self.capnm, "WAG",
            self.empno)
        self.df.mstAlw.show()
        self.df.enableButtons(state=state)

    def doClear(self, widget=None):
        self.df.selPage("General")
        self.df.focusField("T", 0, 1)

    def doPrint(self, widget=None):
        head = "Select the required print options"
        data = (
            ("Info Only", "I"),
            ("Transactions Only", "T"),
            ("Info and Transactions", "A"))
        state = self.df.disableButtons()
        ss = SChoice(self.mf, head=head, data=data)
        self.df.enableButtons(state=state)
        self.mf.updateStatus("")
        if ss.selection == [] or ss.selection == None:
            self.pr_opts = " "
        else:
            self.pr_opts = ss.selection[1]
        if self.pr_opts != " ":
            self.printDetail()
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def printDetail(self):
        from tartanFunctions import doReportlab
        self.head = ("%03u %-30s %47s %10s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        if self.pr_opts != "T":
            self.pageHeading()
            self.printInfo()
        if self.pr_opts == "I":
            self.c.drawText(self.t)
            self.c.showPage()
            self.c.save()
            doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                head=self.tit, view="V")
        elif self.pr_opts == "A":
            self.wagtrn = self.doGetTrans()
            if not self.wagtrn:
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")
            else:
                self.c.drawText(self.t)
                self.c.showPage()
                self.pageHeading()
                self.pageHeadingTrans()
                self.printTrans()
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")
        elif self.pr_opts == "T":
            self.wagtrn = self.doGetTrans()
            if self.wagtrn:
                self.pageHeading()
                self.pageHeadingTrans()
                self.printTrans()
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")

    def printInfo(self):
        self.t.textLine("%26s %-25s %s" % \
            ("", self.df.topf[0][0][3], self.df.t_disp[0][0][0]))
        self.pglin = self.pglin + 1
        self.t.textLine()
        self.pglin = self.pglin + 1
        #General Screen
        for x in range(0, len(self.df.topf[1])):
            self.t.textLine("%26s %-25s %s" % \
                ("", self.df.topf[1][x][3], self.df.t_disp[1][0][x]))
            self.pglin = self.pglin + 1
        self.t.textLine()
        self.pglin = self.pglin + 1
        #Tax Screen
        for x in range(0, len(self.df.topf[2])):
            self.t.textLine("%26s %-25s %s" % \
                ("", self.df.topf[2][x][3], self.df.t_disp[2][0][x]))
            self.pglin = self.pglin + 1
        self.t.textLine()
        self.pglin = self.pglin + 1
        #Bank Details Screen
        if self.df.t_work[1][0][15] == "E":
            for x in range(0, len(self.df.topf[3])):
                self.t.textLine("%26s %-25s %s" % \
                    ("", self.df.topf[3][x][3], self.df.t_disp[3][0][x]))
                self.pglin = self.pglin + 1
            self.t.textLine()
            self.pglin = self.pglin + 1
        #Balances Screen
        self.pglin = self.pglin + 1
        for x in range(0, len(self.df.topf[6])):
            self.t.textLine("%26s %-25s %s" % \
                ("", self.df.topf[6][x][3], self.df.t_disp[6][0][x]))
            self.pglin = self.pglin + 1
        self.t.textLine()
        self.pglin = self.pglin + 1
        #Earnings and Deductions Screen
        self.t.textLine("%26s %29s" % ("", "Earnings and Deductions Codes"))
        self.pglin = self.pglin + 1
        self.t.textLine("%26s %29s" % ("", self.suc * 29))
        self.pglin = self.pglin + 1
        self.t.textLine("%26s %-3s %-1s %-30s %12s  %12s " % \
            ("", "Cod", "T", "Description", "Employee", "Employer"))
        self.pglin = self.pglin + 1
        self.t.textLine("%26s %s" % ("", self.suc * 63))
        self.pglin = self.pglin + 1
        for x in range(0, len(self.df.colf[4])):
            if self.df.c_work[4][x][0] not in ("", 0):
                self.t.textLine("%26s %s %s %s %12s" % \
                    ("", self.df.c_disp[4][x][0], "E",
                    self.df.c_disp[4][x][1], self.df.c_disp[4][x][2]))
                self.pglin = self.pglin + 1
        for x in range(0, len(self.df.colf[5])):
            if self.df.c_work[5][x][0] not in ("", 0):
                self.t.textLine("%26s %s %s %s %12s %12s" % \
                    ("", self.df.c_disp[5][x][0], "D",
                    self.df.c_disp[5][x][1], self.df.c_disp[5][x][2],
                    self.df.c_disp[5][x][3]))
                self.pglin = self.pglin + 1
        #Progressive Balances Screen
        if self.startb == None or self.endb == None:
            pass
        else:
            self.t.textLine()
            self.pglin = self.pglin + 1
            s = CCD(self.startb, "D1", 10.0)
            e = CCD(self.endb, "D1", 10.0)
            self.t.textLine("%26s %-25s %10s %2s %10s" % \
                ("", "Progressive Balances from", s.disp, "to", e.disp))
            self.pglin = self.pglin + 1
            self.t.textLine("%26s %s" % ("", self.suc * 50))
            self.pglin = self.pglin + 1
            self.t.textLine("%26s %-3s %-1s %-30s %12s  %12s " % \
                ("", "Cod", "T", "Description", "Hours", "Value"))
            self.pglin = self.pglin + 1
            self.t.textLine("%26s %s" % ("", self.suc * 63))
            self.pglin = self.pglin + 1
            for x in range(0, len(self.df.colf[7])):
                if self.df.c_work[7][x][0]:
                    if self.df.c_work[7][x][3]:
                        hr = self.df.c_disp[7][x][3]
                    else:
                        hr = "             "
                    self.t.textLine("%26s %s %s %s %12s %12s" % ("",
                        self.df.c_disp[7][x][0], self.df.c_disp[7][x][1],
                        self.df.c_disp[7][x][2], hr, self.df.c_disp[7][x][4]))
                    self.pglin = self.pglin + 1
        self.t.textLine()
        self.pglin = self.pglin + 1

    def doGetTrans(self):
        if self.startt == None or self.endt == None:
            return
        wagtrn = self.sql.getRec(tables="wagtf1", cols=["wt1_date",
            "wt1_page", "wt1_nhrs", "wt1_npay", "wt1_taxbl", "wt1_notax",
            "wt1_taxdd, wt1_nondd"], where=[("wt1_cono", "=", self.conum),
            ("wt1_empno", "=", self.empno), ("wt1_date", "between",
            self.startt, self.endt)], order="wt1_date")
        return wagtrn

    def printTrans(self):
        for x in range(0, len(self.wagtrn)):
            date = CCD(self.wagtrn[x][0], "D1", 10)
            page = CCD(self.wagtrn[x][1], "UI", 5)
            nhrs = CCD(self.wagtrn[x][2], "SD", 10.2)
            npay = CCD(self.wagtrn[x][3], "SD", 10.2)
            taxbl = CCD(self.wagtrn[x][4], "SD", 10.2)
            notax = CCD(self.wagtrn[x][5], "SD", 10.2)
            taxdd = CCD(self.wagtrn[x][6], "SD", 10.2)
            nondd = CCD(self.wagtrn[x][7], "SD", 10.2)
            if self.pglin > self.lpp:
                if self.pgnum != 0:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
                self.pageHeadingTrans()
            net = CCD((float(ASD(taxbl.work) + ASD(notax.work) - \
                ASD(taxdd.work) - ASD(nondd.work))), "OSD", 10.2)
            self.t.textLine("%s %s %s %s %s %s %s %s %s" % (date.disp,
                page.disp, nhrs.disp, npay.disp, taxbl.disp, notax.disp,
                taxdd.disp, nondd.disp, net.disp))
            self.pglin = self.pglin + 1

    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-38s %-10s %37s %5s" % \
            ("Salaries and Wages Interrogation as at", self.sysdtd, "Page",
            self.pgnum))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 6

    def pageHeadingTrans(self):
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine()
        self.t.textLine("%8s Emp-Num: %5s Surname: %s Names: %s" % \
            ("", self.empno, self.df.t_disp[1][0][0], self.df.t_disp[1][0][1]))
        self.t.textLine()
        s = CCD(self.startt, "D1", 10.0)
        e = CCD(self.endt, "D1", 10.0)
        self.t.textLine("%8s %-17s %10s %2s %10s" % \
            ("", "Transactions from", s.disp, "to", e.disp))
        self.t.textLine()
        self.t.textLine("%-10s %5s %9s  %9s  %9s  %9s  %9s  %9s  %9s " % \
            ("   Date", "Page", "Norm-Hrs", "Norm-Pay", "Taxbl-Pay",
            "Non-Taxbl", "Tax-D-Ded", "Non-D-Ded", "Net-Pay"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 10

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

