#! /usr/bin/env python

"""
SYNOPSIS
    Branches File Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import Sql, TartanDialog 

class wgc210:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "ctlbra", prog=__name__)

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Branches File Maintenance (%s)" % __name__)
        bch = {
            "stype": "R",
            "tables": ("ctlbra",),
            "cols": (("bra_code", "", 0, "Cod"),
                ("bra_name", "", 0, "Name", "Y")),
            "where": [("bra_cono", "=", self.conum)]}
        self.fld = (
            (("T",0,0,0),"INA",3,"Code","Branch Code",
                "","N",self.doBranch,bch,None,("notblank",)),
            (("T",0,1,0),"INA",30,"Name","Name",
                "","N",None,None,self.doDelete,("notblank",)),
            (("T",0,2,0),"INA",30,"Address Line 1","Address Line 1",
                "","N",None,None,None,None),
            (("T",0,3,0),"INA",30,"Address Line 2","Address Line 2",
                "","N",None,None,None,None),
            (("T",0,4,0),"INA",30,"Address Line 3","Address Line 3",
                "","N",None,None,None,None),
            (("T",0,5,0),"IUI",4,"Postal Code","Postal Code",
                "","N",None,None,None,None),
            (("T",0,6,0),"INA",20,"Telephone Number","Telephone Number",
                "","N",None,None,None,None),
            (("T",0,7,0),"INA",20,"Fax Number","Fax Number",
                "","N",None,None,None,None),
            (("T",0,8,0),"INA",20,"Contact Name","Contact Name",
                "","N",None,None,None,None),
            (("T",0,9,0),"INA",20,"Contact Mobile","Contact Mobile",
                "","N",None,None,None,None),
            (("T",0,10,0),"ITX",50,"Contact E-Mail","Contact E-Mail",
                "","N",None,None,None,("email",)))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,2),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)

    def doBranch(self, frt, pag, r, c, p, i, w):
        self.code = w
        acc = self.sql.getRec(tables="ctlbra", where=[("bra_cono", "=",
            self.conum), ("bra_code", "=", self.code)], limit=1)
        if not acc:
            self.new = "Y"
        else:
            self.new = "N"
            for x in range(0, self.df.topq[pag]):
                self.df.loadEntry(frt, pag, p+x, data=acc[x+1])

    def doDelete(self):
        dat = (self.conum, self.code)
        self.sql.delRec("ctlbra", data=dat, where=[("bra_cono", "=",
            self.conum), ("bra_code", "=", self.code)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        dat = [self.conum]
        for x in range(0, len(self.df.t_work[0][0])):
            dat.append(self.df.t_work[0][0][x])
        if self.new == "Y":
            self.sql.insRec("ctlbra", data=dat)
        else:
            self.sql.updRec("ctlbra", data=dat, where=[("bra_cono", "=",
            self.conum), ("bra_code", "=", self.code)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

