#! /usr/bin/env python

"""
SYNOPSIS
    Salaries Receiver of Revenue Codes Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import RepPrt, TartanDialog

class wgc420:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])

    def mainProcess(self):
        tit = ("Receiver of Revenue Codes Listing (%s)" % __name__,)
        r1s = (("Number","N"),("Description","D"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Sort Order          ","Sort Order",
                "N","Y",self.doSort,None,None,None),)
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"))

    def doSort(self, frt, pag, r, c, p, i, w):
        self.sort = w

    def doEnd(self):
        self.df.closeProcess()
        tables = ["wagrcv"]
        heads = ["Receiver of Revenue Codes Listing"]
        cols = [["rcv_code","UI",4.0,"Code"],
                ["rcv_desc","NA",50.0,"Description"]]
        if self.sort == "N":
            order = "rcv_code"
        else:
            order = "rcv_desc"
        opts = "Sort Order: %s" % self.sort
        state = self.df.disableButtons()
        rp = RepPrt(self.mf, name=__name__, tables=tables, heads=heads,
            cols=cols, order=order, opts=opts, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2])
        self.df.enableButtons(state=state)
        self.mf.closeLoop()

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

